function [M0Error,M0]=ames2err();
%AMES2ERR    Check error/consistency of AMES2 calculations.
%	     [ERR,M0]=AMES2ERR() computes the error betwwen a Mach number used
%	     as input to the AMES2 functions and the Mach number calculated
%	     from AMES2 output. AMES2ERR generates a logarithmically spaced
%	     vector of 250 Mach numbers from 0.01 to 10. It then uses the AMES2
%	     function to calculate isentropic and normal shock functions
%	     corresponding to these Mach numbers. The values of these functions
%	     are then used as input to the AMES2 function in order to obtain
%	     a Mach number which corresponds to the fuction value. Theoretic-
%	     ally, the intital and final Mach numbers should be the same. In
%	     general, they are not. The difference, returned as ERR, is the
%	     error in the calculations. M0 is the initial vector of Mach
%	     numbers.
%
% 	     Note: Due to very small errors, some calculations may produce the
%	           following error.
%
%		   Warning:  Matrix is close to singular or badly scaled.
%
%	    See also, AMES2 and AMES2PLT.

% Compressible Flow Toolbox, Version 1.1, 23-FEB-96
% Written By: Kevin J. Melcher - 12/94
%             NASA Lewis Research Center, Cleveland, Ohio
%             Cleveland State Univiersity, Cleveland, Ohio


%Get table values as a function of Mach Number
 M0(:) = [logspace(-2,1,250) 1 sqrt(2)];
 M0    = sort(M0);

 Gamma=1.4;
 FM0=ames2(1,M0,Gamma);
 [n,m]=size(FM0);

%Delete Existing Figure Windows
 get(get(gcf,'parent'),'children');

%Setup New Figure Windows
 ss=get(0,'ScreenSize');
 figure(1);
 fpos=get(1,'Position');
 dfpos=round([-0.025*ss(3) +0.025*ss(3) 0 0]);
 fpos=[ss(3)-fpos(3) 0 fpos(3:4)];
 
%Get Mach Number as a function of table values
 Mn=FM0; jbeg=1;
 for j=jbeg:m
   disp(sprintf('  Computing Errors for Column %2d of %d',j,m));
   F=ames2(j,FM0(:,j),Gamma);
   [nF,mF]=size(F);
   if nF==n,
      Mn(:,j)=F(:,1);
   else
    FM0min=min(FM0(:,j)); imin=find(FM0(:,j)==FM0min);
    FM0max=max(FM0(:,j)); imax=find(FM0(:,j)==FM0max);
    if 1<imin & imin<n,
      M0break=mean(FM0(imin:imin+1,1));
    elseif 1<imax & imax<n,
      M0break=mean(FM0(imax-1:imax,1));
    end
    ilt1=find(M0<=M0break);
     Mn(ilt1,j)=F(2*ilt1-1,1);
    igt1=find(M0>M0break);
     Mn(igt1,j)=F(2*igt1,1);
   end
  %Compute Errors
   MnError(:,j)=Mn(:,j)-M0;

  %Plot the Results
   nplts=4;
   iplt = fix((j-jbeg)/(nplts/2));
   r=(j-jbeg)-(nplts/2)*iplt;
   if r==0;
     figure(iplt+1); clf;
     fpos=fpos+dfpos;
     set(iplt+1,'Position',fpos);
   else
     subplot(nplts,1,1);
     title('Test Consistency of AMES2.M');
   end
  %Columns 9-16 only have valid solutions for M0>=1
   if j<9, i=1:n;
   else,   i=find(M0>=1);    
   end

  %Plot Absolute Error
   subplot(nplts,1,2*r+1)
   semilogx(M0(i),MnError(i,j));
   xlabel('Mach No.')
   ylabel(sprintf('Error (%d)',j));
   %Plot Percent Error
   subplot(nplts,1,2*r+2)
   semilogx(M0(i),100*MnError(i,j)./M0(i));
   xlabel('Mach No.')
   ylabel(sprintf('%%Error (%d)',j));
   drawnow;
 end%for
