function [x1, y1] = approxi(x, y, r, bd)
% APPROXI  Approximation of a function y = y(x) to reduce the number of pairs.
%          The data points x and y are vectors with lengths n, the lengths of 
%          the returned vectors x1 and y1 is n1 (n1 < n). The key of algorithm 
%          is to analyse the first order derivative of the function.
%
%          use:
%          [x1, y1] = approxi(x, y, r, [bd]);
%
%          x  -> x-data of the function (vector)
%          y  -> y-data of the function (vector)
%          r  -> value for the rate of increase (scalar)
%                good results for 0.1 < r < 0.5 (please test!)
%          bd -> optional, if bd unequal 0, then comparison of y(x) and y1(x1) 
%                will be plotted
%
% example: t=0:0.001:5; y=2*sin(2*pi*.3*t)+.2*cos(2*pi*3*t);
%          [t1, y1]=approxi(t, y, 0.5, 1);
%
%          L. Merz, 1. Sep. 1995
%          DASA-Airbus Hamburg, EF 345, Kreetslag 10, 21129 Hamburg, Germany
%          Phone: +49.40.7437.3367
%          Fax:   +49.40.7437.4187
%          Email 100533.2037@compuserve.com

if nargin == 3,
   bd = 0;
end;
   
l = length(x);

x1(1) = x(1);
y1(1) = y(1);
k = 1;

for i=2:l-1,
    m1 = (y(i+1) - y1(k)) / (x(i+1) - x1(k));
    m2 = (y(i+1) - y(i)) / (x(i+1) - x(i));

    if abs(m1*r) < abs(m2-m1),
       k = k+1;
       x1(k) = x(i);
       y1(k) = y(i);
    end;
end;

k = k+1;
x1(k) = x(l);
y1(k) = y(l);

fprintf('number of data points \n');
fprintf('before: %d\n', l);
fprintf('later:  %d\n\n', length(x1));

if bd,
   plot(x, y, x1, y1); 
   grid, xlabel('x, x1'), ylabel('y, y1')
   title('Result of Approximation');
   legend('before','later');
   
   hold on; plot(x1, y1, 'o'); hold off;
end;        
     












