/******************************************/
/*                                        */
/* Copyright (c) 1996 by Adam L. Schwartz */
/*          All right reserved.           */
/*                                        */
/*********************************************************************/
/* Compile using: cmex -O3 LBFGS.c                                   */
/*********************************************************************/
/* IMPORTANT: clear LBFGS mex file before running.  This causes a    */
/*            restart.                                               */
/*********************************************************************/
/* Mex file to compute the L-BFGS search direction.                  */
/*                                                                   */
/* [drct,count] = LBFGS(x,grad,scale,I,DISP);                        */
/*                                                                   */
/* x --- current value of decision variables.                        */
/* grad --- gradient of cost w.r.t to x,                             */
/* scale --- (optional) scalar scale factor for initial Hessian,     */
/* I --- (optional) indices of unconstrained decision variables.     */
/* DISP --- (optional) No printouts when DISP == 0.                  */
/*                                                                   */
/* drct --- LBFGS quasi-Newton search direction in unconstrained     */
/*          subspace, scaled steepest descent in constrained         */
/*          subspace.                                                */
/*********************************************************************/
/* References:                                                       */
/* D.C. Liu and J. Nocedal, "On the Limited Memory BFGS Method       */
/* for Large Scale Optimization,"Math. Prog. 45 (1989), pp. 503-528. */
/* J. Nocedal, "Updating quasi-Newton matrices with limited storage,"*/
/* Mathematics of Computation 35 (1980), pp. 773-782.                */
/*                                                                   */
/* Modifications: Resets if d'g/(scale*g'g)   > -TOL1.               */
/*                Resets if d'd/(scale^2*g'g) > TOL2.                */
/*                Skip update if y's/(g'g)    < TOL3.                */
/*        (inner-products are taken on unconstrained subspace only.) */
/* scale member [G_MIN , G_MAX].                                     */
/*********************************************************************/

#include "mex.h"
#include <math.h>

#define MIN_INPUTS 2
#define MAX_INPUTS 5
#define MIN_OUTPUTS 0
#define MAX_OUTPUTS 2

#define MAX_STORAGE 12                /* Number of updates to store. */
#define TOL1        0.2
#define TOL2        1.e3
#define TOL3        1.e-3
#define G_MIN       1.e-3
#define G_MAX       1.e3
#define AUTO_SCALE 1  /* Use gamma = y's/y'y instead of user-supplied scale. */
                      /*        User's scale factor used on first iteration. */

#ifndef NULL
#define NULL 0
#endif

/* These variables are declared globally so that  */
/* free_memory() can be called without arguments. */
static int initialized=0;
static double *y[MAX_STORAGE],*s[MAX_STORAGE],*old_x,*old_grad,*q;
static double *temp_s,*temp_y;

#ifdef __STDC__
void 
#else
int
#endif
mexFunction(nlhs,lhs,nrhs,rhs)
     int nlhs,nrhs;
     Matrix *lhs[], *rhs[];
{
  static int iter,reset,count,prev_size;
  void free_memory();
  double inner_prod();
  int x_size,incr,bound,i,j,k;
  int *I,size_I,disp_p;
  double *x,*grad,*drct,scale,user_scale;
  double rho[MAX_STORAGE],alpha[MAX_STORAGE],beta;
  double old_scale,norm2_g,norm_d;

  mexAtExit(free_memory);

  /*************************************************************/
  /*                 C H E C K   I N P U T S                   */
  /*************************************************************/

  if ( nrhs < MIN_INPUTS || nrhs > MAX_INPUTS )
    mexErrMsgTxt("Wrong number of inputs.");
  if ( nlhs < MIN_OUTPUTS || nlhs > MAX_OUTPUTS )
    mexErrMsgTxt("Wrong number of ouptuts.");

  x     = mxGetPr(rhs[0]);
  grad = mxGetPr(rhs[1]);
  if ( nrhs >= 3)
    user_scale = mxGetScalar(rhs[2]);
  else
    user_scale = 1.0;

  x_size = mxGetN(rhs[0])*mxGetM(rhs[0]);
  if ( mxGetM(rhs[1])*mxGetN(rhs[1]) != x_size ) {
    free_memory();
    mexErrMsgTxt("x and grad not same size.");
  }
  if ( x_size != prev_size ) reset = 1;

  if ( nrhs >= 4 && mxGetPr(rhs[3]) != NULL ) {
    size_I = mxGetN(rhs[3])*mxGetM(rhs[3]);
    I = (int*)mxCalloc(size_I,sizeof(int));
    for ( i = 0 ; i < size_I ; i++ )
      I[i] = (int) mxGetPr(rhs[3])[i] - 1;
  }
  else {
    size_I = x_size;
    if (size_I > 0) {
      I = (int*)mxCalloc(size_I,sizeof(int));
      for ( i = 0 ; i < size_I ; i++ )
	I[i] = i;
    }
  }  
  if ( size_I > x_size ) {
    free_memory();
    mexErrMsgTxt("Index set I_i is too big.");
  }

  if ( nrhs >= 5 )
    disp_p = (int) mxGetScalar(rhs[4]);
  else
    disp_p = 1;
  
  /*************************************************************/
  /*             A L L O C A T E    M E M O R Y                */
  /*************************************************************/

  /* Here I use calloc() instead of mxCalloc() because I need */
  /* the memory to remain allocated after LBFGS returns.      */
  if ( reset ) {
    if ( x_size != prev_size ) {
      free_memory();
      for ( i = 0 ; i < MAX_STORAGE ; i++ ) {
	s[i] = (double*)calloc(x_size,sizeof(double));
	y[i] = (double*)calloc(x_size,sizeof(double));
      }
      old_x    = (double*)calloc(x_size,sizeof(double));
      old_grad = (double*)calloc(x_size,sizeof(double));
      q        = (double*)calloc(x_size,sizeof(double));
      temp_s   = (double*)calloc(x_size,sizeof(double));
      temp_y   = (double*)calloc(x_size,sizeof(double));
      prev_size = x_size;
      initialized = 1;
    }
    reset = 0;
    count = 0;
    iter = 0;
  }
  
  lhs[0] = mxCreateFull(x_size,1,0);
  drct = mxGetPr(lhs[0]);
  
  /*************************************************************/
  /*             M A I N   C O M P U T A T I O N               */
  /*************************************************************/

  norm2_g = inner_prod(grad,grad,I,size_I);
  if ( iter > 0 ) {
    i = (iter-1) % MAX_STORAGE;
    for ( k = 0 ; k < x_size ; k++ ) {
      temp_s[k] = x[k] - old_x[k];
      temp_y[k] = grad[k] - old_grad[k];
    }

    /* If |s_k'*y_k| is too small, skip update on this iteration. */
    rho[i] = inner_prod(temp_y,temp_s,I,size_I);
    if ( size_I > 0 && rho[i]/norm2_g < TOL3 ) { 
      /* Go back to previous iteration and use previously stored updates. */
      /* Also, use old_scale unless back to 0-th iteration in which case  */
      /* use user-supplied scaling.                                       */
      iter--;
      scale = user_scale;
      if (disp_p) mexPrintf("L-BFGS: Skipping update.  |s'y|/g'g = %g\n",
			    fabs(rho[i])/norm2_g);
    }
    else {
      for ( k = 0 ; k < x_size ; k++ ) {
	s[i][k] = temp_s[k];
	y[i][k] = temp_y[k];
      }
#if AUTO_SCALE
      if ( size_I > 0 ) {
	scale=(inner_prod(y[i],s[i],I,size_I)/inner_prod(y[i],y[i],I,size_I));
	scale=(scale > G_MIN*user_scale) ? scale : G_MIN*user_scale;
	scale=(scale < G_MAX*user_scale) ? scale : G_MAX*user_scale;
      }
      else
	scale = user_scale;
#endif
    }
  }
  else
    scale = user_scale;

  /* Let drct = -scale*grad on constrained subspace. */
  for ( k = 0 ; k < x_size ; k++ )
    drct[k] = -scale*grad[k];

  if ( size_I > 0 ) {
    if ( iter <= MAX_STORAGE ) {
      incr = 0;
      bound = iter;
    }
    else {
      incr = iter - MAX_STORAGE;
      bound = MAX_STORAGE;
    }
    
    for ( k = 0 ; k < size_I ; k++ )
      q[I[k]] = grad[I[k]];
    
    for ( i = bound-1 ; i >= 0 ; i-- ) {
      j = (i + incr) % MAX_STORAGE;
      rho[j] = 1.0/inner_prod(y[j],s[j],I,size_I);
      alpha[i] = rho[j]*inner_prod(s[j],q,I,size_I);
      for ( k = 0 ; k < size_I ; k++ ) 
	q[I[k]] -= alpha[i]*y[j][I[k]];
    }
    
    /* Compute -H*g on unconstrained subspace directly (not H*g). */
    for ( k = 0 ; k < size_I ; k++ )
      drct[I[k]] = -scale*q[I[k]];
    
    for ( i = 0 ; i < bound ; i++ ) {
      j = (i + incr) % MAX_STORAGE;
      beta = rho[j]*inner_prod(y[j],drct,I,size_I);
      for ( k = 0 ; k < size_I ; k++ )
	/* Compute -H*g directly instead of H*g, */
	/* thus use +beta instead of -beta.      */
	drct[I[k]] -= (alpha[i]+beta)*s[j][I[k]];
    }
    
    /* Reset if search direction is not sufficiently downhill. */
    norm_d = sqrt(inner_prod(drct,drct,I,size_I));
    if ( (inner_prod(drct,grad,I,size_I)/(scale*norm2_g)) > -TOL1 ) {
      if (disp_p) mexPrintf("L-BFGS direction not sufficiently downhill...Resetting\n");
      reset = 1;
    }
    else
      /* Reset if search direction is too big. */
      if ((inner_prod(drct,drct,I,size_I)/(norm2_g*scale*scale)) > TOL2 ) {
	if (disp_p) mexPrintf("L-BFGS direction too big...Resetting\n");
	reset = 1;
      }
    if (reset) {
      for ( k = 0 ; k < size_I ; k++ )
	drct[I[k]] = -user_scale*grad[I[k]];
      count = 0;
      reset = 0;
      iter = 0;
    }
    
    count++;
    iter++;
    
    if (nlhs == 2) {
      lhs[1] = mxCreateFull(1,1,0);
      mxGetPr(lhs[1])[0] = (double) count;
    }
    
    mxFree(I);
  }

  /*************************************************************/
  /* S T O R E   O L D   G R A D I E N T  A N D  C O N T R O L */
  /*************************************************************/
  
  for ( k = 0 ; k < x_size ; k++ ) {
    old_x[k] = x[k];
    old_grad[k] = grad[k];
  }
  old_scale = scale;
  
}


double
inner_prod(x,y,I,size_I)
     int *I,size_I;
     double *x,*y;
{
  register int i,index;
  double result;
  
  result = 0.0;
  for ( i = 0 ; i < size_I ; i++ ) {
    index = I[i];
    result += x[index]*y[index];
  }

  return result;
}

void free_memory()
{
  int i;
  
  if (initialized) {
    free(temp_y); free(temp_s);
    free(q); free(old_grad); free(old_x);
    for ( i = MAX_STORAGE-1 ; i >= 0 ; i-- ) {
      free(y[i]); free(s[i]);
    }
  }
  initialized = 0;
}
