%RIOTSdem1	Demonstrate use of simulate() and sp_plot().

%
% Copyright (c) 1996 by Adam L. Schwartz
% All right reserved.  
%

clear simulate
clear FLAGS
clear global FLAGS

clc,clg,home,echo on,hold off

% We will use the Rayleigh optimal control problem for this demonstration.
% 
% In your 'systems' directory type:
% 
%   cmex.link rayleigh.o
% 
% at the UNIX prompt to link-in the Rayleigh system.

pause                              % TOUCH ANY KEY TO CONTINUE.

% The first thing we can do is to print out some information about the
% system we just linked in to RIOTS by calling  simulate(0) . 

simulate(0);

% The first few lines of text following this call to simulate() are
% messages produced by the Rayleigh system itself.  Rayleigh was
% programmed to check if the global variable  FLAGS  is defined.  If
% FLAGS  is not defined, a warning message is printed the first time
% simulate() is called and no flags are loaded.

pause                              % TOUCH ANY KEY TO CONTINUE.

% The Rayleigh systems has two states and one input.  Before
% performing a simulation we must choose a time interval, a
% discretization level, a set of initial conditions for the state and
% a control function.

% The control function is a spline given by its spline coefficients.
% For a discretization level of  N  and a spline order of  r , the
% number of control coefficients for each input is equal to  N+r-1 . 

N  = 20;                  % Discretization level (number of intervals).
x0 = [-5;-5];             % initial conditions.
t  = [0:2.5/N:2.5];       % t is a row vector representing the integration
                          % mesh and spline breakpoints for the time
                          % interval [0,2.5].
u  = ones(1,N+2-1);       % u is a row vector of spline coefficient, all 1's,
                          % for a second order spline (piecewise linear).

pause                              % TOUCH ANY KEY TO CONTINUE.
clc,home

% We can now simulate the system:

[f,x]=simulate(1,x0,u,t,4,2);


% The first argument to simulate() always specifies which function
% simulate() is to perform; in the above call, the 1 means that
% simulate() should integrate the systems equations.  The last two
% arguments mean that we're going to use a fourth-order method, RK4,
% and produce 2 outputs: f (the value of objective function) and  x
% (the state trajectories).

pause                              % TOUCH ANY KEY TO CONTINUE.

% Let's take a look at the system trajectories produced by this simulation.

plot(t,x)
xlabel('Time'),title('State Trajectories')
legend('x1','x2')
	
pause                              % TOUCH ANY KEY TO CONTINUE.

% Now that we performed a system simulation, we can compute gradients using

[du,dx0,p] = simulate(2,1,2);

% This second form for simulate() requires that a system simulation
% using the first form has already been performed.  The outputs for
% this call are, respectively, the gradient of the objective with
% respect to the spline coefficients, the derivative of the objective
% with respect to the initial conditions x0, and the solution of
% adjoint equation associated with the objective function.

% Alternatively, we could have computed all of these outputs at the same
% time using

[f,x,du,dx0,p] = simulate(1,x0,u,t,4,4);

pause                              % TOUCH ANY KEY TO CONTINUE.

% Since du, the gradient of the objective function with respect to
% the control spline, is itself the coeffients of a spline, we use
% sp_plot() to view it:

legend off
sp_plot(t,du);
xlabel('Time'), title('Gradient with Respect to the Control')

pause                              % TOUCH ANY KEY TO CONTINUE.
clc,home
 
% Let's now redo the simulation using for the control representation
% piecewise constant, linear and quadratic splines (with u(t) = 1).
% Then we will compare the resulting gradients, du , which will be
% splines of the same order as the control.  We'll use the variable
% stepsize integration routine which is integration algorithm 5.

u = ones(1,N+1-1);                     % Piecewise constant control
[f1,x1,du1] = simulate(1,x0,u,t,5,3);

u = ones(1,N+2-1);                     % Linear spline control.
[f2,x2,du2] = simulate(1,x0,u,t,5,3);

u = ones(1,N+3-1);                     % Quadratic spline control.
[f3,x3,du3] = simulate(1,x0,u,t,5,3);

pause                              % TOUCH ANY KEY TO CONTINUE.

% We use sp_plot() with three argument to evaluate the various splines
% at the time points in some vector  tau .  This allows us to plot
% together splines of different order.

tau = [0:2.5/100:2.5];
du_val1 = sp_plot(t,du1,tau);
du_val2 = sp_plot(t,du2,tau);
du_val3 = sp_plot(t,du3,tau);

plot(tau , [du_val1 ; du_val2 ; du_val3 ])
xlabel('Time'), ylabel('Graident w.r.t to control')
title('Comparison of first, second and third order spline gradients.')
legend('Piecwise constant','Linear spline','Quadratic spline')

pause                              % TOUCH ANY KEY TO CONTINUE.
