%RIOTSdem2	Solving optimal control problems.

%
% Copyright (c) 1996 by Adam L. Schwartz
% All right reserved.  
%

clc,clg,home,echo on,hold off

% We will again use the Rayleigh optimal control problem for this
% demonstration.
% 
% If you haven't already done so, in your "systems" directory type:
% 
%   cmex.link rayleigh.o
% 
% at the UNIX prompt to link-in the Rayleigh system.

pause                              % TOUCH ANY KEY TO CONTINUE.

% The file rayleigh.c implements the Rayleigh optimal control problem.  
% There are actually two versions of the problem: an unconstrained version
% and a version with the endpoint constraint  x1(2.5) = 0 .  To determine
% which version the user wants to solve, rayleigh.c includes a call to
% get_flags() which checks to see if the global variable  FLAGS  is defined
% in the Matlab workspace.  If  FLAGS  is not defined, or if  FLAGS = 0 ,
% the unconstrained problem is solved.  If  FLAGS = 1 , the constrained
% problem is solved.

pause                              % TOUCH ANY KEY TO CONTINUE.

clear simulate

% Let's solve the unconstrained problem first using second order splines
% on a uniform integration mesh with  N = 50  intervals.

global FLAGS
FLAGS = 0;

N  = 50;
x0 = [-5;-5];
t  = [0:2.5/N:2.5];
u0 = zeros(1,N+2-1);               % Second order spline---initial guess
                                   % u(t) = 0.

pause                              % TOUCH ANY KEY TO CONTINUE.

% To solve this problem we can use either riots() or pdmin().  We will
% first use pdmin with the second-order Runge-Kutta integration
% routine, indicated by the input 2, and allow, at most, 100
% iterations.  The outputs we request will be the control solution,
% trajectory solution and the value of the objective function.
% 
% [u,x,f] = pdmin(x0,u0,t,[],[],[],100,2);
%
% The brackets ([]) in the input list indicate that those input argument
% should be set to their default.  In this case, the brackets indicate
% no lower bounds on the controls, no upper bounds, and no system 
% parameters.

pause                              % TOUCH ANY KEY TO CONTINUE.

[u,x,f] = pdmin(x0,u0,t,[],[],[],100,2);

pause                              % TOUCH ANY KEY TO CONTINUE.

% A similar call to riots() will also solve this problem.
% 
% [u,x,f] = riots(x0,u0,t,[],[],[],100,2);

pause                              % TOUCH ANY KEY TO CONTINUE.

[u,x,f] = riots(x0,u0,t,[],[],[],100,2);

pause                              % TOUCH ANY KEY TO CONTINUE.

% Let's take a look at the solution.

sp_plot(t,u);
xlabel('Time'),ylabel('Optimal Control')
title('Linear spline solution on uniform grid with 50 intervals')

pause                              % TOUCH ANY KEY TO CONTINUE.
clc,home

% Now let's resolve the problem using quadratic splines and the fourth
% order Runge-Kutta integration routine.

u0 = zeros(1,N+3-1);

% u2 = riots(x0,u0,t,[],[],[],100,4);
 
pause                              % TOUCH ANY KEY TO CONTINUE.

u2 = riots(x0,u0,t,[],[],[],100,4);

pause                              % TOUCH ANY KEY TO CONTINUE.

tau = [0:2.5/100:2.5];
u_val1 = sp_plot(t,u,tau);
u_val2 = sp_plot(t,u2,tau); 

plot(tau,u_val2,tau,u_val1,':')
xlabel('Time'), ylabel('Optimal Control')
title('Unconstrained Rayleigh Problem')
legend('third order spline','second order spline')


% Will zoom-in on the first portion of the plot to get a better look
% at the difference in these solutions.  Note that these solutions are
% computed on the same integartion mesh.

pause                              % TOUCH ANY KEY TO CONTINUE.

tau = [0:0.6/100:0.6];
u_val1 = sp_plot(t,u,tau);
u_val2 = sp_plot(t,u2,tau); 

plot(tau,u_val2,tau,u_val1,'-.')
xlabel('Time'), ylabel('Optimal Control')
title('Unconstrained Rayleigh Problem')
legend('third order spline','second order spline')

pause                              % TOUCH ANY KEY TO CONTINUE.

% Let's also look at the trajectory solutions.

plot(t,x)
xlabel('Time'), ylabel('Optimal Trajectory')
title('Unconstrained Rayleigh Problem (N=50)')
legend('x1','x2')

pause                              % TOUCH ANY KEY TO CONTINUE.
clc,home

% Now suppose we wish to solve this problem with the control bound 
% 0 <= u(t) <= 4.  We just modify the call to pdmin() or riots()
% to be:
%
% u3 = riots(x0,u0,t,0,4,[],100,4);

u0 = zeros(1,N+1);     % Go back to second-order splines because of
                       % the presence of control bounds.

pause                              % TOUCH ANY KEY TO CONTINUE.

u3 = riots(x0,u0,t,0,4,[],100,4);

pause                              % TOUCH ANY KEY TO CONTINUE.

legend off
sp_plot(t,u3)
axis([0 2.5 -0.5 4.5])
xlabel('Time'), ylabel('Optimal Control')
title('Rayleigh Problem with control bounds (N=50)')

pause                              % TOUCH ANY KEY TO CONTINUE.

% The main optimization program, riots(), becomes very slow as the size
% of the problem begins to exceed 500-1000 decision variables (depending
% on which SQP algorithm is linked with riots().  On the other hand, the 
% amount of work that pdmin() performs grows only linearly with the size
% of the problem.  Here we'll solve the same problem, but with N = 2000.

pause                              % TOUCH ANY KEY TO CONTINUE.

TOL = 1e-6;
tt = [0:2.5/2000:2.5];

u3 = pdmin(x0,ones(1,2001),tt,0,4,[],[100;TOL],4);

sp_plot(tt,u3)
axis([0 2.5 -0.5 4.5])
xlabel('Time'), ylabel('Optimal Control')
title('Rayleigh Problem with control bounds (N=2000)')

pause                              % TOUCH ANY KEY TO CONTINUE.
clc,home

% Next, let's solve the Rayleigh problem with the endpoint constraint
% x1(2.5) = 0  included.  We must set FLAGS = 1 and clear simulate()
% so that FLAGS will get read.

FLAGS = 1;
clear simulate

pause                              % TOUCH ANY KEY TO CONTINUE.

% Since this problem has a state constraint, we can no longer use pdmin()
% to solve it.  We can still use riots() or we can use aug_lagrng().  The
% call to riots() is exactly the same as before (except will remove the
% lower bound on the controls by specifying [])
%
% u4 = riots(x0,u0,t,[],4,[],100,4);

pause                              % TOUCH ANY KEY TO CONTINUE.

[u4,x4,f4] = riots(x0,u0,t,0,4,[],100,4);

pause                              % TOUCH ANY KEY TO CONTINUE.

% We can also solve this problem using aug_lagrng().  In general,
% riots() works much better than aug_lagrn().  However, aug_lagrng()
% can deal with much higher discretization levels than riots().
% The call is very similar:
%
% u5 = aug_lagrng(x0,u0,t,[],4,[],10,5,4);
%
% In this call, the last three arguments are different than for riots();
% they indicate, respectively, a maximum of 10 inner iterations per outer
% iterations, a maximum of 5 outer iterations, and use the fourth order
% Runge-Kutta integration routine.

pause                              % TOUCH ANY KEY TO CONTINUE.

u5 = aug_lagrng(x0,u0,t,[],4,[],10,5,4);

pause                              % TOUCH ANY KEY TO CONTINUE.

% Let's view the control solution.

sp_plot(t,u5)
axis([0 2.5 -2 5])
xlabel('Time'), ylabel('Optimal Control')
title('Endpoint constrained Rayleigh Problem with upper control bound')

pause                              % TOUCH ANY KEY TO CONTINUE.

% Let's take a look at the state trajectory solution.  Notice that
% the endpoint constraint  x1(2.5)=0  is statisfied.

plot(t,x4)
xlabel('Time'), ylabel('Optimal Trajectory')
legend('x1','x2')
title('Endpoint constrained Rayleigh Problem with upper control bound')

pause                              % TOUCH ANY KEY TO CONTINUE.
