function [errorA,errorB,max_error]=check_deriv(x,u,t,params,index,central,DISP)

% function [errorA,errorB,max_error] = 
%                 check_deriv(x,u,t,{params},{index},{central},{DISP})
%
%  This function checks the derivatives of the system objective and 
%  constraint functions directly (unlike check_grad)
%  by comparing them to finite-difference approximations.
%
%  INPUTS:
%  x is an n-vector, u is an m-vector, and t is a scalar.
%  If index is given then neq[3] = index.
%
%  If central = 1, then central differences are performed.
%  If central = 0, then foward differences are performed.
%  It is useful to do both because some large quantities that look 
%  like erros may in fact be OK.  If the errors become smaller
%  under central differences than under forwards differences then
%  you are probably alright.
%
%  DISP  --- Minimal information display if DISP = 0 (default = 1).
%
%  OUTPUTS:
%  errorA is difference between dh(x)/dx and finite-difference approximation.
%  errorB is difference between dh(x)/du and finite-difference approximation.
%  max_error is maximum error detected between analytic derivatives and
%            finite-difference approximations.
%
%  The error for forward differences is approximately h |f''()| / 2.
%  The error for central differences is approximately h^2 |f'''()| / 6.
%
%  WARNING---Currently there is no provision for ensuring that the finite
%  difference perturbations do not extend into undefined regions of the
%  state-space.

%
% Copyright (c) 1996 by Adam L. Schwartz
% All right reserved.  
%

if nargin < 7, DISP = 1; end
if DISP == [], DISP = 1; end
if nargin < 6, central=0; end 
if central == [], central=0; end
if nargin < 5, index = 1; end
if index == [], index = 1; end
if nargin < 4, params = []; end
if nargin < 3
   disp(['USAGE: [errorA,errorB,max_error] = ' ...
         'check_deriv(x,u,t,{params},{index},{central},{DISP})']);
   return;
end
fprintf('============================================================================\n')

pert_size= eps^(1/3);

if length(t) > 1, error('t must be a scalar.'), end
info=simulate(0);
if length(params) ~= info(3);
  error('Wrong number of system parameters.'); 
end
info=simulate(0,params);

nstates = info(1); ninputs = info(2);
if sum(size(x) ~= [nstates 1]), error('x should be n x 1.'); end
if sum(size(u) ~= [ninputs 1]), error('u should be m x 1.'); end
if sum(size(t) ~= [1 1]), error('t should be a scalar.'); end

nfunctions = info(6);
n_traj_constraints = sum(info(7:8));
n_end_constraints = sum(info(9:12));
n = info(1); m = info(2); nparams = info(3);

xd = zeros(n);
A  = zeros(n,n);
B  = zeros(n,m);
zd = zeros(1,nfunctions+n_traj_constraints);
lx = zeros(nfunctions+n_traj_constraints,n);
lu = zeros(nfunctions+n_traj_constraints,m);
E = zeros(1,nfunctions+n_end_constraints);
Ex0 = zeros(nfunctions+n_end_constraints,n);
Exf = zeros(nfunctions+n_end_constraints,n);

%---------- First store the nominal values for each function.-------------

[xd,ztemp] = simulate(3,x,u,t,1,index);
[A,B,lxtemp,lutemp]=simulate(4,x,u,t,1,index);
zd(1) = ztemp; lx(1,:) = lxtemp; lu(1,:) = lutemp;
[Etemp,Ex0temp,Exftemp] = simulate(5,x,x,t);
E(1) = Etemp; Ex0(1,:) = Ex0temp; Exf(1,:) = Exftemp;

for fnc = 2:nfunctions,
  [dum1,ztemp] = simulate(3,x,u,t,fnc,index);
  [dum1,dum2,lxtemp,lutemp]=simulate(4,x,u,t,fnc,index);
  zd(fnc) = ztemp; lx(fnc,:) = lxtemp; lu(fnc,:) = lutemp;
  [Etemp,Ex0temp,Exftemp] = simulate(5,x,x,t,fnc);
  E(fnc) = Etemp; Ex0(fnc,:) = Ex0temp; Exf(fnc,:) = Exftemp;
end
for fnc = nfunctions+1:nfunctions+n_traj_constraints,
  [dum1,ztemp] = simulate(3,x,u,t,fnc,index);
  [dum1,dum2,lxtemp,lutemp]=simulate(4,x,u,t,fnc,index);
  zd(fnc) = ztemp; lx(fnc,:) = lxtemp; lu(fnc,:) = lutemp;
end
for fnc = nfunctions+1:nfunctions+n_end_constraints,
  [Etemp,Ex0temp,Exftemp] = simulate(5,x,x,t,fnc);
  E(fnc) = Etemp; Ex0(fnc,:) = Ex0temp; Exf(fnc,:) = Exftemp;
end

errorA  = zeros(n,n);
errorB  = zeros(n,m);
max_error = 0;
%------- Compute finite differences for h() and l() functions.---------

errorlx = zeros(1,n);
errorlu = zeros(1,m);
for fnc = 1:nfunctions+n_traj_constraints,
  
  % First perturb x.
  for i = 1:n,
    x1 = x; x2 = x;
    pert = pert_size*max(1,abs(x(i)));
    if x(i) < 0, pert_x = -pert; else pert_x = pert; end,
    x1(i) = x(i)+pert_x;
    x2(i) = x(i)-pert_x;
    [xd1,zd1] = simulate(3,x1,u,t,fnc,index);
    if ~central
      diffx = (xd1 - xd)/pert_x;
      diffz = (zd1 - zd(fnc))/pert_x;
    else 
      [xd2,zd2] = simulate(3,x2,u,t,fnc,index);
      diffx = (xd1 - xd2)/(2*pert_x);
      diffz = (zd1 - zd2)/(2*pert_x);
    end,
    if fnc == 1, errorA(:,i) = diffx - A(:,i); end,
    errorlx(i) = diffz - lx(fnc,i);
  end,

  % Now perturb u.
  for i = 1:m,
    u1 = u; u2 = u;
    pert = pert_size*max(1,abs(u(i)));
    if u(i) < 0, pert_u = -pert; else pert_u = pert; end,
    u1(i) = u(i)+pert_u;
    u2(i) = u(i)-pert_u;
    [xd1,zd1] = simulate(3,x,u1,t,fnc,index);
    if ~central	
      diffx = (xd1 - xd)/pert_u;
      diffz = (zd1 - zd(fnc))/pert_u;
    else 
      [xd2,zd2] = simulate(3,x,u2,t,fnc,index);
      diffx = (xd1 - xd2)/(2*pert_u);
      diffz = (zd1 - zd2)/(2*pert_u);
    end,
    if fnc == 1, errorB(:,i) = diffx - B(:,i); end,
    errorlu(i) = diffz - lu(fnc,i);
  end,
  max_error=max([max(max(abs(errorA))) max(max(abs(errorB))) max_error]);
  max_error=max([max(max(abs(errorlx))) max(max(abs(errorlu))) max_error]);

  if DISP
    if fnc == 1
      disp('              System matrices:'),
      disp('Error in h_x = '),
      disp(errorA),
      disp('Error in h_u = '),
      disp(errorB),
    end,
    if fnc <= nfunctions
      disp(['              For function ' num2str(fnc) ':']),
    else
      disp(['         For trajecotry constraint ' num2str(fnc-1) ':']),
    end
    fprintf(1,'Error in l_x = '),
    disp(errorlx),
    fprintf('Error in l_u = '),
    disp(errorlu),
  end
end

%--------- Compute finite differences for endpoint constraints.----------

errorEx0 = zeros(1,n);
errorExf = zeros(1,n);
for fnc = 1:nfunctions+n_end_constraints,
  
  % First perturb x.
  for i = 1:n,
    x1 = x; x2 = x;
    pert = pert_size*max(1,abs(x(i)));
    if x(i) < 0, pert_x = -pert; else pert_x = pert; end,
    x1(i) = x(i)+pert_x;
    x2(i) = x(i)-pert_x;
    % First perturb initial state.
    E1 = simulate(5,x1,x,t,fnc);
    if ~central
      diffE = (E1 - E(fnc))/pert_x;
    else 
      E2 = simulate(5,x2,x,t,fnc);
      diffE = (E1 - E2)/(2*pert_x);
    end,
    errorEx0(i) = diffE - Ex0(fnc,i);
    % Now perturb final state.
    E1 = simulate(5,x,x1,t,fnc);
    if ~central
      diffE = (E1 - E(fnc))/pert_x;
    else 
      E2 = simulate(5,x,x2,t,fnc);
      diffE = (E1 - E2)/(2*pert_x);
    end,
    errorExf(i) = diffE - Exf(fnc,i);
  end,
  max_error=max([max(max(abs(errorEx0))) max(max(abs(errorExf))) max_error]);

  if DISP
    if fnc <= nfunctions,
      disp(['              For function ' num2str(fnc) ':']),
      fprintf(1,'Error in g_x0 = '),
      disp(errorEx0),
      fprintf('Error in g_xf = '),
      disp(errorExf),
    else
      disp(['         For endpoint constraint ' num2str(fnc-nfunctions) ':']),
      fprintf(1,'Error in g_x0 = '),
      disp(errorEx0),
      fprintf('Error in g_xf = '),
      disp(errorExf),
    end
  end
end
fprintf('Maximum error reported is %g.\n',max_error)
fprintf('============================================================================\n')
