%  This function is called by riots() to approximate the Hessian.  The
%  result is exact for Linear/Quadratic problems.  Second derivatives
%  for linear constraints and constraints with negligible Lagrange
%  multipliers are not estimated.  Also, perturbation columns for
%  trajectory constraints are not computed when the column occurs
%  after the index of the trajectory constraint since these columns
%  are zero.  Also, advantage is taken of the Hessian's symmetry by
%  only computing the lower triangular part.  The ordering of the
%  Lagrange multipliers is assumed to be that of NPSOL.
%
%  Assumes that there is only one objective function.
%
%  If K = 0, leave Hessian in original coordinates.
%  If factor nonzero, perform Cholesky decomposition,
%  if factor == 0, return compute Hessian without making it positive def.
%
%  rt_Minv should only be supplied if it's diagonal.
%
%  Ideally, I'd choose perturbation to stay in feasible region, but I don't.
%
%  For large N, the computation time is dominated by the SVD computation.

%
% Copyright (c) 1996 by Adam L. Schwartz
% All right reserved.  
%

function H2 = comp_hess(x0,u,t,ialg,lambda,free_x0,scale,K,points,factor,var,rt_Minv)

if nargin < 4,
 disp(['Usage: H = comp_hess(x0,u,t,ialg,lambda,free_x0,' ...
       '{scale},{K},{points},{factor},{var},{rt_Minv})'])
 return;
end
if nargin < 7, scale = []; end
if nargin < 8, K = 0; end
if nargin < 9, points = 0; end
if nargin < 10, factor = 0; end
if nargin < 11, var = 0; end
if nargin < 12, rt_Minv = []; end

disp(' ')
disp('Computing Hessian by finite-difference approximation.')

ct = cputime;
pert_size = eps^(1/3);   % Size of finite-difference perturbations.
TOL = eps^(1/3);         % Size of zero for Lagrange multipliers.

info = simulate(0);
if scale == [], scale = ones(1,sum(info(7:12))); end
n_free_x0 = length(free_x0);
ninputs = size(u,1);
length_u = size(u,2);
N1 = ninputs*length_u;
N  = N1 + n_free_x0;
if lambda ~= []
  nfunctions = info(6);
  nlt_ineq = info(7);
  lt_ineq  = info(8);
  nle_ineq = info(9);
  le_ineq  = info(10);
  nle_eq   = info(11);
  le_eq    = info(12);
  LTI = info(13);
  if LTI == 0
    nlt_ineq = nlt_ineq+lt_ineq; lt_ineq = 0;
    nle_ineq = nle_ineq+le_ineq; le_ineq = 0;
    nle_eq   = nle_eq+le_eq;     le_eq   = 0;
  end
  %  The order of constraints is as follows:
  %    1   bounds on u
  %    2   bounds on free x0's
  %    3   linear endpoint inequality           
  %    4   linear trajectory inequality         
  %    5   linear endpoint equality             
  %    6   nonlinear endpoint inequality        
  %    7   nonlinear trajectory inequality      
  %    8   nonlinear endpoint equality          
  N3 = N+le_ineq;
  N4 = N3+lt_ineq*points;
  N5 = N4+le_eq;
  N6 = N5+nle_ineq;
  N7 = N6+nlt_ineq*points;
  N8 = N7+nle_eq;
  if nle_ineq>0,
    lambda6 = lambda(N5+1:N6); 
    scale6  = scale(2:1+nle_ineq);
  end
  if nlt_ineq>0,
    lambda7 = lambda(N6+1:N7); 
    scale7  = scale(2+nle_ineq:1+nle_ineq+nlt_ineq);
  end
  if nle_eq>0,
    lambda8 = lambda(N7+1:N8); 
    scale8  = scale(2+nle_ineq+nlt_ineq+le_ineq+lt_ineq: ...
                    1+nle_ineq+nlt_ineq+le_ineq+lt_ineq+nle_eq);
  end
  M = 1 + N8-N5;  % Total number of functions + nonlinear constraints.
else
  M = 1;
end 

%
% --- First compute and store gradients evaluated at (x0,u).
%
DU = zeros(N,M);

% Compute gradient of objective function.
[j,x,du,dz]=simulate(1,x0,u,t,ialg,3); end
k = 1;
DU(:,k)=[dz(free_x0);du(:)];

% Compute gradients for nonlinear inequality constraints.
offset = 1 + (nlt_ineq+lt_ineq)*points;
for j = 1:N6-N5
  k = k+1;
  if abs(lambda6(j)) > TOL
    [du,dz] = simulate(2,j+offset,2);
    DU(:,k) = [dz(free_x0);du(:)];
  end
end

% Compute gradients for nonlinear trajectory constraints.
offset = 1;
for j = 1:N7-N6
  k = k+1;
  if abs(lambda7(j)) > TOL
    [du,dz] = simulate(2,j+offset,2);
    DU(:,k) = [dz(free_x0);du(:)];
  end
end

% Compute gradients for nonlinear equality constraints.
offset = 1 + (nlt_ineq+lt_ineq)*points + nle_ineq + le_ineq;
for j = 1:N8-N7
  k = k+1;
  if abs(lambda8(j)) > TOL
    [du,dz] = simulate(2,j+offset,2);
    DU(:,k) = [dz(free_x0);du(:)];
  end
end

%
% --- Now compute gradients evaluated at ( x0+dz , u+du )
% --- and construct Hessian one column at a time.
%

if K <= 2
  USE_SYMMETRY = 1;
else
  USE_SYMMETRY = 0;
end
H = zeros(N,N);
fprintf('There are %d columns.\n',N)
fprintf('Computing column:')
for col = 1:N
  fprintf(' %d',col)
  if col > n_free_x0
    index = col-n_free_x0;
    if USE_SYMMETRY
      start = max(1,fix((index-1)/ninputs)-max(K,2)+2);
      N0 = n_free_x0 + (start-1)*ninputs+1;
    else
      start = 1;
    end
    pert = pert_size*max(abs(u(index)),1);
    if u(index) < 0, pert = -pert; end
    u(index) = u(index)+pert;
    simulate(1,x(:,start),u(:,start:length_u),t(start:points),ialg,0);
    u(index) = u(index)-pert;
  else
    start = 1;
    l = free_x0(col);
    pert = pert_size*max(abs(x0(l)),1);
    if x0(l) < 0, pert = -pert; end
    x0(l) = x0(l)+pert;
    simulate(1,x0,u,t,ialg,0);
    x0(l) = x0(l)-pert;
  end

  k = 1;
  [du,dz] = simulate(2,1,2);
  % Note that, the length of  du  is only N-N0+1 since simulate
  % the simulation is started at time k = start.
  if (col <= n_free_x0 | ~USE_SYMMETRY)
    H(:,col) = scale(1)*([dz(free_x0);du(:)] - DU(:,k))/pert;
  else
    H(N0:N,col) = scale(1)*(du(:) - DU(N0:N,k))/pert;
  end

  % Compute gradients for nonlinear inequality constraints.
  offset = 1 + (nlt_ineq+lt_ineq)*(points-start+1);
  for j = 1:N6-N5
    k = k+1;
    if abs(lambda6(j)) > TOL,
      [du,dz] = simulate(2,j+offset,2);
      if (col <= n_free_x0 | ~USE_SYMMETRY)
        H(:,col) = H(:,col) - ...
                   scale6(j)*lambda6(j)*([dz(free_x0);du(:)] - DU(:,k))/pert;
      else
        H(N0:N,col) = H(N0:N,col) - ...
                      scale6(j)*lambda6(j)*(du(:) - DU(N0:N,k))/pert;
      end
    end
  end

  % Compute perturbed gradient for nonlinear trajectory constraints.
  offset = 1;
  for j = 1:N7-N6
    k = k+1;
    l = rem(j-1,points)+1;
    % Only compute gradient if:
    % 1. Associate mulitpliers is not negligible.
    % 2. The constraint index is during or after control perturbation.
    %    (Note: start <= col-n_free_x0)
    if abs(lambda7(j)) > TOL  &  l >= col-n_free_x0 
      % Remember that the simulation is started from  start  so, 
      % the j-th points of a trajectory constraint occurs at the
      % time index j-start+1.
      [du,dz] = simulate(2,j+offset-start+1,2);
      if (col <= n_free_x0 | ~USE_SYMMETRY)
        H(:,col) = H(:,col) - ...
                   scale7(l)*lambda7(j)*([dz(free_x0);du(:)] - DU(:,k))/pert;
      else
        H(N0:N,col) = H(N0:N,col) - ...
                      scale7(l)*lambda7(j)*(du(:) - DU(N0:N,k))/pert;
      end
    end
  end

  % Compute perturbed gradient for nonlinear equality constraints.
  offset = 1 + (nlt_ineq+lt_ineq)*(points-start+1) + nle_ineq + le_ineq;
  for j = 1:N8-N7
    k = k+1;
    if abs(lambda8(j)) > TOL,
      [du,dz] = simulate(2,j+offset,2);
      if (col <= n_free_x0 | ~USE_SYMMETRY)
        H(:,col) = H(:,col) - ...
                   scale8(j)*lambda8(j)*([dz(free_x0);du(:)] - DU(:,k))/pert;
      else
        H(N0:N,col) = H(N0:N,col) - ...
                      scale8(j)*lambda8(j)*(du(:) - DU(N0:N,k))/pert;
      end
    end
  end

end

global HH
HH = H;

if USE_SYMMETRY
  % Upper triangular part of H is junk.
  H = diag(diag(H)) + tril(H,-1) + tril(H,-1)';
else
  H = (H+H')/2;                          % Make sure H is symmetric.
end

ct1 = cputime;
fprintf('\nTime to compute discrete Hessian = %g seconds.\n',ct1-ct)

% --- Rearrange order of Hessian to match order expected by riots.c
% --- which has free_x0 variables after the control variables.
if n_free_x0 > 0
  H = [H(n_free_x0+1:N,n_free_x0+1:N)  H(n_free_x0+1:N,1:n_free_x0) ;
       H(1:n_free_x0,n_free_x0+1:N)    H(1:n_free_x0,1:n_free_x0)];
end

% Add Hessian corresponding to the penalty on the total variation of
% the derivative of the control.
% Must have NN1 > 3.
if var > 0
  NN1 = N1/ninputs;
  D = spdiags([-ones(NN1-1,1) ones(NN1-1)],[0 1],NN1-1,NN1);
  if K == 1
    DT = diff(t)';
    DELTA = spdiags(DT(1:points-2),0,points-2,points-2);
  else
    DELTA = spdiags(diff(t)',0,points-1,points-1);
  end
  Hvar = D(1:NN1-2,1:NN1-1)*DELTA*D;
  Hvar = full(Hvar'*Hvar*var);
  H(1:N1,1:N1) = H(1:N1,1:N1)+kron(Hvar,eye(ninputs));
end

%
% --- Put H into transformed coordinates.
%
if K ~= 0
  if rt_Minv ~= []
    rt_Minv=spdiags([kron(rt_Minv,ones(1,ninputs)) ones(1,n_free_x0)]',0,N,N);
  else
    global sqp_sqrt_M_inv
    if sqp_sqrt_M_inv ~= []
      rt_Minv = sqp_sqrt_M_inv;
    else
      [U,D] = schur(full(transform(t,K)));
      rt_Minv = U*spdiags(ones(N,1)./sqrt(diag(D)),0,N,N)*U';
    end
  end
  H = rt_Minv'*H*rt_Minv;
end

%
% --- Ensure H > 0 and perform Cholesky factorization.
%
if factor
  if M == 1 | 1                                    % Always try Cholesky first.
    fprintf('Computing Cholesky factorization.\n')
    [H2,p] = chol(H);	
    if p > 0, 
      disp('Warning: Hessian not positive definite.')
      M = 2;                       % Proceed to next through next if statement.
    end
  end
  if M > 1
    fprintf('Computing SVD.\n')
    [U,S,V] = svd(H);              
    S = diag(S);
    I = find(S<TOL);
    S(I) = TOL*ones(size(I));
    S = diag(S);                   % negate negative eigenvalues.
    H = U*S*U';                    % This works because H = H'
    fprintf('Computing Cholesky factorization.\n')
    H2 = chol(H);	
  end
else
  H2 = H;
end

fprintf('Time to do linear algebra on Hessian = %g seconds.\n',cputime-ct1)
