function [error,norm_zd]=control_error(x01,u1,t1,ze1,x02,u2,t2,ze2,Tf,M)

% function [error,norm_zd]=control_error(x01,u1,t1,ze1,x02,u2,t2,ze2,{Tf},{M})
%
% This function takes information produced by est_errors() to
% get a good estimate,  error,  for the L2-norm error between ze2 and
% the optimal control (an extremal control).  If ue1 - ue2 is small
% this estimate is likely to be too convservative.  The output  
% norm_zd is the H2 norm of the difference z1 - z2.  The L2-norm is
% computed for the time interval specified by the endpoints of t2.
% For free final time problems,
% t should be scaled by the final time scale factor.
%
% z1 = [u1 ; x0_1] is a solution corresponding to the time vector t1.
% ze1 is the control error estimate from est_errors().
%
% z2 = [u2 ; x0_2] is a solution corresponding to the time vector t2.
% ze2 is the control error estimate from est_errors().
%
% Tf is the final time scale factor for free final time problems.  
% Default = 1.
%
% M is the spline transform matrix for u2's breakpoints.  If M is given,
% it is assumed that the spline given by (t1,u1) is a spline of the same 
% order as the spline given by (t2,u2) and that t1 is contained in t2.
% In other words, it is assumed that (t1,u1) also belongs to the spline
% space containing (t2,u2).
% In this case, the inner product of the two splines is computed in the
% u2 spline space by interpolating u1 up to t2 and using M.
%
% If M is not specified, the inner product of (t1,u1) and (t2,u2) is
% computed by integrating the spline that interpolates their product.

%
% Copyright (c) 1996 by Adam L. Schwartz
% All right reserved.  
%

if nargin < 4
  disp(['Usage: [error,norm_zd]='...
        'control_error(x01,u1,t1,ze1,x02,u2,t2,ze2,{Tf},{M})'])
  return
end
if nargin < 9, Tf = []; end
if nargin < 10, M = []; end
if Tf == [], Tf=1; end

order1 = length(u1) - length(t1) + 2;
order2 = length(u2) - length(t2) + 2;
x01 = x01(:,1); x02 = x02(:,1);

if M == []

  % If necessary, scale time by the final time scale factor.
  if Tf ~= 1
    t1 = t1(1) + cumsum([0 diff(t1)]*Tf);
    t2 = t2(1) + cumsum([0 diff(t2)]*Tf);
  end

  sp1 = spmak(augknt(t1,order1),u1);
  sp2 = spmak(augknt(t2,order2),u2);

  % Now I'm going to create a spline which interpolates the square of
  % the difference between sp1 and sp2.  I first create a knot sequence
  % that has enough multiplicity to handle the reduced smoothness and
  % higher-orderness of the product.  Then, I interpolate a spline to
  % the square of the values of sp1-sp2 taken at knot averages.
  % The knot sequence will not have enough multiplicity if there are
  % any repeated breakpoints in t1 or t2.
  % It would probably be faster to do this using Guassian quadrature and
  % I wouldn't have to worry about repeated knots that way either.

  order = order1 + order2 - 1;
  knots = sort([t1 t2]);
  I = find(abs(diff(knots)) < 1e-8);
  knots(I) = [];
  knots = kron(knots,[1 1]);
  knots = augknt(knots,order);         % This elminates extra knot endpoints.

  tau = aveknt(knots,order);
  u_diff = fnval(sp1,tau) - fnval(sp2,tau);
  sp_prod = spapi(knots,tau,u_diff.*u_diff);

  sp_int = fnint(sp_prod);
  u_inner_prod = fnval(sp_int,t1(length(t1))) - fnval(sp_int,t1(1));

  diff_x0 = x01 - x02;

  norm_zd = sqrt(u_inner_prod + diff_x0'*diff_x0);

else

  order = order1;
  m = size(u2,1);

  % --- INTERPOLATE U1 TO SAME DIMENSION AS U2.
  points = length(t2);
  if order == 1
     u12 = sp_plot(t1,u1,t2(1:points-1));
  elseif order == 2
     u12 = sp_plot(t1,u1,t2);
  else
     sp1 = spmak(augknt(t1,order),u1);

     new_knots = augknt(t2,order);
     tau = aveknt(new_knots,order);
     sp12 = spapi(new_knots,tau,fnval(sp1,tau));
     [breaks,u12] = spbrk(sp12);
  end

  diff_u  = u12-u2;
  diff_uM = diff_u*M;
  diff_x0 = x01 - x02;
  diff_zM = [diff_uM(:) ; diff_x0];
  diff_z  = [diff_u(:)  ; diff_x0];

  norm_zd = sqrt(diff_zM'*diff_z);

end

error = norm_zd*(ze2/abs(ze1-ze2));
