/******************************************/
/*                                        */
/* Copyright (c) 1996 by Adam L. Schwartz */
/*          All right reserved.           */
/*                                        */
/******************************************/
/* 4th order Runge-Kutta. */
/*                        */
/*   0  |                 */
/*  1/2 | 1/2             */
/*  1/2 |  0  1/2         */
/*   1  |  0   0   1      */
/* ---------------------- */
/*      | 1/6 1/3 1/3 1/6 */
/********************************************************************/
/* Hager's formulas are used to compute the adjoint and gradient.   */
/* The gradients are modified to reflect the fact                   */
/* that the only one control sample is used for the middle samples  */
/* since c2 = c3.                                                   */
/********************************************************************/
/* Here's an easy way to plot out the spline of order k             */
/* corresponding to its coefficients (say u) in Matlab:             */
/* >> fnplt(spmak(augknt(t,K),u))                                   */
/*                                OR                                */
/* >> u=fnval(spmak(augknt(t,K),u)) ; plot(t,u)                     */
/********************************************************************/
/* This version uses Carl deBoor's BSPLVB() routine to compute      */
/* values for the B-spline basis functions so that a non-uniform    */
/* mesh can be handled.  Unlike the previous, uniform mesh version, */
/* the knots sequence of the Bspline has, implicitly, k repeated    */
/* knots at the beginning and end of the time vector.               */
/********************************************************************/
/* This implementation assumes that the inputs are K-th order       */
/* spline.  The dimension of controls is m x (N-1)+K.               */
/*                                       K can be 2, 3, or 4.       */
/* The gradients are computed with respect to the coefficients      */
/* alpha_k, associated with the spline basis elements.              */
/********************************************************************/
#define LINEAR    2
#define QUADRATIC 3
#define CUBIC     4
#define MAX_ORDER 4	               /* Maximum order of splines. */

#include "mex.h"
#include "utility.h"
#define STATIC
#include "system.h"

extern long unsigned int H_calls,DH_calls;

void RK4(nlhs, lhs, nrhs, rhs)
     int nlhs, nrhs;
     Matrix *lhs[], *rhs[];
{
  static unsigned int points = 0;
  static int initialized = 0,linear_p,Order;
  static int neq[] = {0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  static int neqt,ninputs,nparams,nfunctions,ntraj_constr,nend_pt_constr;
  static double *state,*adjoint,*dJdu,*dJdalpha,*bcoefs,*spline,*Y,*x0;
  static double *x,*p,z,zdot,*k1,*k2,*k3,*k4,*Y2store,*Y3store,*Y4store;
  static double *l_x1,*l_x2,*l_x3,*l_x4,*h_x1,*h_x2,*h_x3,*h_x4,*temp;
  static double *l_u1,*l_u2,*l_u3,*l_u4,*h_u1,*h_u2,*h_u3,*h_u4,*mid_control;
  static double *q3,*q2,*q1,*J_x0;
  
  int i, j, k, index, row, col, action, form, f_num, start, traj_constr;
  int return_x=0,return_j=0,return_p=0;
  int comp_x=0,comp_j=0,comp_du=0,comp_lte=0;
  double t,tout,deltaT,*q0,*uvalue,*du,duvalue,Bival[MAX_ORDER];
  Matrix *u,*interval,*ans1,*ans2,*ans3,*ans4,*ans5;
  
  if ( nrhs == 0 ) {
    if ( initialized ) {
      free(dJdu-ninputs); free(x0); 
      free(bcoefs); free(spline); free(Y-3);
      free(Y2store); free(Y3store); free(Y4store); free(state);
      free(h_u1); free(h_u2); free(h_u3); free(h_u4);
      free(l_u1); free(l_u2); free(l_u3); free(l_u4);
      free(h_x1); free(h_x2); free(h_x3); free(h_x4); 
      free(l_x1); free(l_x2); free(l_x3); free(l_x4);
      free(q3); free(q2); free(q1); free(J_x0);
      free(x); free(k1); free(k2); free(k3); free(k4); 
      initialized = 0;
    }
    return;
  }

  form = (int) mxGetScalar(rhs[0]);
  if ( form == 1 ) {
    /* Form 1 implies a system integration is required. */
    u = rhs[2];
    interval = rhs[3];
    traj_constr = 0;

    f_num = neq[4] = 1;				 /* Evaluate first function. */
    comp_x = 1;
    action = (int) mxGetScalar(rhs[5]);
    if (action > 0 ) return_j = 1;
    if (action > 1 ) return_x = 1;
    if (action > 2 ) comp_du  = 1;
    if (action > 3 ) return_p = 1;

    if ( points != mxGetN(interval) ) {
      /* If the number of mesh points has changed since the last simulation */
      /* we have to resize the arrays that depend on the mesh size.         */
      points = mxGetN(interval);
      if (initialized) {
	free(dJdu-ninputs); free(Y-3);
        free(bcoefs); free(spline); free(state);
	free(Y4store); free(Y3store); free(Y2store); 
	Y2store = (double*)callocate(points-1,sizeof(double)*neqt,35);
	Y3store = (double*)callocate(points-1,sizeof(double)*neqt,36);
	Y4store = (double*)callocate(points-1,sizeof(double)*neqt,37);
	state   = (double*)callocate(points,sizeof(double)*neqt,38);
	spline  = (double*)callocate(2*(points-1)+1,sizeof(double)*ninputs,39);
	bcoefs  = (double*)callocate((points-2)+MAX_ORDER,
				     sizeof(double)*ninputs,40);
	Y       = (double*)mallocate(sizeof(double)*(points+6),41);
	Y      += 3;
	/* Note, the length of dJdu should only be 3*(points-1) but adding */
	/* ninputs zeros at the beginning and ninputs zeros at the         */
	/* end makes computation of gradient easier.                       */
	dJdu    = (double*)callocate(3*(points-1)+2,sizeof(double)*ninputs,42);
	dJdu += ninputs;
      }
    }
    
    /* This is the main memory allocation section.  The functions */
    /* mallocate() and callocate() call malloc() and calloc().    */
    if (!initialized) {
      /* Get system information. */
      init(neq,NULL);
      neqt=neq[0];
      ninputs=neq[1];
      nparams=neq[2];
      nfunctions=neq[5];
      ntraj_constr = neq[6]+neq[7];
      nend_pt_constr = neq[8]+neq[9]+neq[10]+neq[11];
      linear_p=neq[12];

      x = (double*)mallocate(sizeof(double)*neqt,1);
      p = x;
      k1 = (double*)callocate(neqt,sizeof(double),2);
      k2 = (double*)callocate(neqt,sizeof(double),3);
      k3 = (double*)callocate(neqt,sizeof(double),4);
      k4 = (double*)callocate(neqt,sizeof(double),5);
      q3 = (double*)mallocate(sizeof(double)*neqt,6);
      q2 = (double*)mallocate(sizeof(double)*neqt,7);
      q1 = (double*)mallocate(sizeof(double)*neqt,8);
      J_x0 = (double*)callocate(neqt,sizeof(double),9);
      temp = l_x1 = (double*)callocate(neqt,sizeof(double),9);
      l_x2 = (double*)callocate(neqt,sizeof(double),10);
      l_x3 = (double*)callocate(neqt,sizeof(double),11);
      l_x4 = (double*)callocate(neqt,sizeof(double),12);
      h_x1 = (double*)callocate(neqt,sizeof(double)*neqt,13);
      h_x2 = (double*)callocate(neqt,sizeof(double)*neqt,14);
      h_x3 = (double*)callocate(neqt,sizeof(double)*neqt,15);
      h_x4 = (double*)callocate(neqt,sizeof(double)*neqt,16);
      mid_control = l_u1 = (double*)callocate(ninputs,sizeof(double),17);
      l_u2 = (double*)callocate(ninputs,sizeof(double),18);
      l_u3 = (double*)callocate(ninputs,sizeof(double),19);
      l_u4 = (double*)callocate(ninputs,sizeof(double),20);
      h_u1 = (double*)callocate(neqt,sizeof(double)*ninputs,21);
      h_u2 = (double*)callocate(neqt,sizeof(double)*ninputs,22);
      h_u3 = (double*)callocate(neqt,sizeof(double)*ninputs,23);
      h_u4 = (double*)callocate(neqt,sizeof(double)*ninputs,24);
      Y2store = (double*)callocate(points-1,sizeof(double)*neqt,25);
      Y3store = (double*)callocate(points-1,sizeof(double)*neqt,26);
      Y4store = (double*)callocate(points-1,sizeof(double)*neqt,27);
      state   = (double*)callocate(points,sizeof(double)*neqt,28);
      x0      = (double*)mallocate(sizeof(double)*(neqt),29);
      /* spline containts the values of the control spline evaluated */
      /* at mesh points and at midpoints.  bcoefs contains the       */
      /* spline coefficients.                                        */
      spline = (double*)callocate(2*(points-1)+1,sizeof(double)*ninputs,30);
      bcoefs = (double*)callocate((points-2)+MAX_ORDER,
				  sizeof(double)*ninputs,31);
      /* We add an extra 3 points to the beginning and end of Y    */
      /* to simplify calculation of u[t] from spline coefficients. */
      Y  = (double*)mallocate(sizeof(double)*(points+6),32);
      Y += 3;
      /* Note, the length of dJdu should only be 3*(points-1) but adding */
      /* ninputs zeros at the beginning and ninputs zeros at the         */
      /* end makes computation of gradient easier.                       */
      dJdu  = (double*)callocate(3*(points-1)+2,sizeof(double)*ninputs,33);
      dJdu += ninputs;
      initialized = 1;
    }
    if ( mxGetM(rhs[1]) != neqt )
      mexErrMsgTxt("x0 wrong length.");
    if ( mxGetM(u) != ninputs) 
      mexErrMsgTxt("Not enough control inputs.");

    Order = mxGetN(u) - (points-2);
    if ( Order < 2 || Order > MAX_ORDER )
      mexErrMsgTxt("u wrong length...spline order K = 2, 3, or 4.");

    /****************************************************/
    /* Save x0, Y for calls later by simulate() form 2. */
    /****************************************************/
    for ( i = 0 ; i < neqt ; i++ )
      x0[i] = mxGetPr(rhs[1])[i];
    for ( i = 0 ; i < points ; i++ )
      Y[i] = mxGetPr(interval)[i];
    /* Pad beginning and end of Y with t_i and t_f respectively. */
    Y[-3] = Y[-2] = Y[-1] = Y[0];
    Y[points+2] = Y[points+1] = Y[points] = Y[points-1];

    /*************************************************************/
    /* The following segment computes, from the spline           */
    /* coefficients, the values of the u(t) evaluated at the     */
    /* points t = { t_0, t_1/2, t_1, t_3/2, ... , t_N-1/2, t_N } */
    /*************************************************************/
    for ( i = 0 ; i < (points-2)+Order ; i++)
      for ( j = 0 ; j < ninputs ; j++ )
	bcoefs[i*ninputs+j] = mxGetPr(u)[i*ninputs+j];
    for ( i = 0 ; i < points-1 ; i++ ) {
      bsplvb(Order,Y+i-(Order-1),Y[i],Bival);
      /* First compute value of u(t) at t = Y[i]. */
      /* Generally, Bival[3] = 0 unless there is a repeated knot. */
      for ( j = 0 ; j < ninputs ; j++ ) {
	spline[2*i*ninputs+j] = Bival[0]*bcoefs[i*ninputs+j];
	for ( k = 1 ; k < Order ; k++ )
	  spline[2*i*ninputs+j] += Bival[k]*bcoefs[(i+k)*ninputs+j];
      }
      /* Now compute value of u(t) a t = (Y[i]+Y[i+1])/2. */
      bsplvb(Order,Y+i-(Order-1),(Y[i]+Y[i+1])/2,Bival);
      for ( j = 0 ; j < ninputs ; j++ ) {
	spline[(2*i+1)*ninputs+j] = Bival[0]*bcoefs[i*ninputs+j];
	for ( k = 1 ; k < Order ; k++ )
	  spline[(2*i+1)*ninputs+j] += Bival[k]*bcoefs[(i+k)*ninputs+j];
      }
    }
    /* Just finish off the endpoint. */
    i = points-2;
    bsplvb(Order,Y+i-(Order-1),Y[i+1],Bival);
    for ( j = 0 ; j < ninputs ; j++ ) {
      spline[2*(i+1)*ninputs+j] = Bival[0]*bcoefs[i*ninputs+j];
      for ( k = 1 ; k < Order ; k++ )
	spline[2*(i+1)*ninputs+j] = Bival[k]*bcoefs[(i+k)*ninputs+j];
    }
  }
  else {
    if ( form == 2 ) {
      /* System has already been simulated with the current control and */
      /* initial conditions.  Now compute gradients, constraint values, */
      /* or other objective function values.                            */
      f_num  = (int) mxGetScalar(rhs[1]);           /* Function number. */
      action = (int) mxGetScalar(rhs[2]);           /* Action number.   */
      if (f_num > nfunctions && f_num <= nfunctions + ntraj_constr*points)
	traj_constr = 1;
      else
	traj_constr = 0;
      if (f_num > nfunctions + ntraj_constr*points + nend_pt_constr)
	mexErrMsgTxt("Function number too big.");
      if (f_num < 1) mexErrMsgTxt("Functions number must be > 0.");
      
      switch (action) {
      case 3:
	return_p = 1;
      case 2: 
	comp_du = 1;
	break;
      case 1:
	comp_j = 1;
      }
    }
    else
      comp_lte = 1;
  }
  
  if (comp_x) {
    /* Integrate system dynamics and compute the system trajectories. */
  
    /* Get initial condition on state. */
    for (j = 0 ; j < neqt ; j++)
      state[j] = x[j] = x0[j];
    
    t = Y[0]; z = 0.0;
    for ( i = 0 ; i < points-1 ; i++) {
      tout = Y[i+1];
      deltaT = tout - t;
      neq[3] = i;
      index = i*neqt;
      
      uvalue = spline+2*i*ninputs;
      H_calls += 4;
      h(neq,&t,x,uvalue,k1);
      if (return_j) {
	zdot = l(neq,&t,x,uvalue); neq[3] = i;
	z += deltaT*zdot/6;
      }
      for ( j = 0 ; j < neqt ; j++) 
	Y2store[index+j] = x[j] + deltaT/2*k1[j];
      
      t+=deltaT/2;
      uvalue = spline+(2*i+1)*ninputs;
      h(neq,&t,Y2store+index,uvalue,k2);
      if (return_j) {
	zdot = l(neq,&t,Y2store+index,uvalue); neq[3] = i;
	z += deltaT*zdot/3;
      }
      for ( j = 0 ; j < neqt ; j++) 
	Y3store[index+j] = x[j] + deltaT/2*k2[j];
      
      h(neq,&t,Y3store+index,uvalue,k3);
      if (return_j) {
	zdot = l(neq,&t,Y3store+index,uvalue); neq[3] = i;
	z += deltaT*zdot/3;
      }
      for ( j = 0 ; j < neqt ; j++) 
	Y4store[index+j] = x[j] + deltaT*k3[j];
      
      t = tout;
      uvalue = spline+2*(i+1)*ninputs;
      h(neq,&t,Y4store+index,uvalue,k4);
      if (return_j) {
	zdot = l(neq,&t,Y4store+index,uvalue);
	z += deltaT*zdot/6;
      }
      
      for ( j = 0 ; j < neqt ; j++) 
	state[(i+1)*neqt+j] = x[j] += 
	  deltaT*(k1[j] + 2*(k2[j] + k3[j]) + k4[j])/6;
    }
    if (return_j) {
      ans1 = mxCreateFull(1,1,REAL);
      lhs[0] = ans1;
      *mxGetPr(ans1) = g(neq,&t,x0,x) + z;
    }
    if (return_x) {
      ans2 = mxCreateFull(neqt,points,REAL);
      lhs[1] = ans2;
      index = 0;
      for ( i = 0 ; i < points ; i++ ) 
	for ( j = 0 ; j < neqt ; j++ ) {
	  mxGetPr(ans2)[index] = state[index];
	  index++;
	}
    }
  }

  /* Compute estimates of local truncation error by starting from each */
  /* compute x_k, taking two half sized steps and comparing result     */
  /* with x_k+1.                                                       */
  if (comp_lte) {
    ans1 = mxCreateFull(neqt+1,points-1,REAL);
    lhs[0] = ans1;
   
    zdot = 0.0;
    for ( i = 0 ; i < points-1 ; i++) {
      neq[3] = i;
      index = i*neqt;
      t = Y[i]; tout = Y[i+1];
      deltaT = (tout - t)/2;

      for (j = 0 ; j < neqt ; j++)
	x[j] = state[index+j];
      
      /* Compute first half step. */
      H_calls += 8;
      uvalue = spline+2*i*ninputs;
      h(neq,&t,x,uvalue,k1);
      zdot = l(neq,&t,x,uvalue); neq[3] = i;
      mxGetPr(ans1)[i*(neqt+1)+neqt] = deltaT*zdot/3;
      for ( j = 0 ; j < neqt ; j++) 
	temp[j] = x[j] + deltaT/2*k1[j];
      z = deltaT*zdot/6;
      t += deltaT/2;
      bsplvb(Order,Y+i-(Order-1),t,Bival);
      for ( j = 0 ; j < ninputs ; j++ ) {
	mid_control[j] = Bival[0]*bcoefs[i*ninputs+j];
	for ( k = 1 ; k < Order ; k++ )
	  mid_control[j] += Bival[k]*bcoefs[(i+k)*ninputs+j];
      }
      h(neq,&t,temp,mid_control,k2);
      zdot = l(neq,&t,temp,mid_control); neq[3] = i;
      z += deltaT*zdot/3;
      for ( j = 0 ; j < neqt ; j++) 
	temp[j] = x[j] + deltaT/2*k2[j];
      h(neq,&t,temp,mid_control,k3);
      zdot = l(neq,&t,temp,mid_control); neq[3] = i;
      z += deltaT*zdot/3;
      for ( j = 0 ; j < neqt ; j++) 
	temp[j] = x[j] + deltaT*k3[j];
      t += deltaT/2;				  /* t = (Y[i+1] + Y[i]) / 2 */
      uvalue = spline+(2*i+1)*ninputs;
      h(neq,&t,temp,uvalue,k4);
      zdot = l(neq,&t,temp,uvalue); neq[3] = i;
      z += deltaT*zdot/6;
      for ( j = 0 ; j < neqt ; j++)
	x[j] += deltaT*(k1[j] + 2*(k2[j] + k3[j]) + k4[j])/6;
      zdot = l(neq,&t,Y2store+index,uvalue); neq[3] = i;
      mxGetPr(ans1)[i*(neqt+1)+neqt] += deltaT*2*zdot/3;
      
      /* Now compute second half step. */
      h(neq,&t,x,uvalue,k1);
      zdot = l(neq,&t,x,uvalue); neq[3] = i;
      z += deltaT*zdot/6;
      for ( j = 0 ; j < neqt ; j++) 
	temp[j] = x[j] + deltaT/2*k1[j];
      zdot = l(neq,&t,Y3store+index,uvalue); neq[3] = i;
      mxGetPr(ans1)[i*(neqt+1)+neqt] += deltaT*2*zdot/3;
      t += deltaT/2;
      bsplvb(Order,Y+i-(Order-1),t,Bival);
      for ( j = 0 ; j < ninputs ; j++ ) {
	mid_control[j] = Bival[0]*bcoefs[i*ninputs+j];
	for ( k = 1 ; k < Order ; k++ )
	  mid_control[j] += Bival[k]*bcoefs[(i+k)*ninputs+j];
      }
      h(neq,&t,temp,mid_control,k2);
      zdot = l(neq,&t,temp,mid_control); neq[3] = i;
      z += deltaT*zdot/3;
      for ( j = 0 ; j < neqt ; j++) 
	temp[j] = x[j] + deltaT/2*k2[j];
      h(neq,&t,temp,mid_control,k3);
      zdot = l(neq,&t,temp,mid_control); neq[3] = i;
      z += deltaT*zdot/3;
      for ( j = 0 ; j < neqt ; j++) 
	temp[j] = x[j] + deltaT*k3[j];
      t = tout;
      uvalue = spline+2*(i+1)*ninputs;
      h(neq,&t,temp,uvalue,k4);
      zdot = l(neq,&t,temp,uvalue); neq[3] = i;
      z += deltaT*zdot/6;

      for ( j = 0 ; j < neqt ; j++) {
	x[j] += deltaT*(k1[j] + 2*(k2[j] + k3[j]) + k4[j])/6;
	mxGetPr(ans1)[i*(neqt+1)+j] = x[j] - state[index+neqt+j];
      }
      zdot = l(neq,&t,Y4store+index,uvalue);
      mxGetPr(ans1)[i*(neqt+1)+neqt] += deltaT*zdot/3 - z;
    }
  }
  
  if ( comp_j ) {
    /* Compute an objective function from the computed state trajectory. */
    ans1 = mxCreateFull(1,1,REAL);
    lhs[0] = ans1;
    if ( f_num <= nfunctions ) {
      neq[4] = f_num;
      t = Y[0]; z = 0.0;
      for ( i = 0 ; i < points-1 ; i++) {
	tout = Y[i+1];
	deltaT = tout-t;
	neq[3] = i;
	index = i*neqt;
	uvalue = spline+2*i*ninputs;
	zdot = l(neq,&t,state+index,uvalue);
	if (neq[3] == -1) break;				   /* l == 0 */
	z += deltaT/6*zdot;
	t += deltaT/2;
	uvalue = spline+(2*i+1)*ninputs;
	zdot = l(neq,&t,Y2store+index,uvalue);
	z += deltaT*zdot/3;
	zdot = l(neq,&t,Y3store+index,uvalue);
	z += deltaT*zdot/3;
	t = tout;
	uvalue = spline+2*(i+1)*ninputs;
	zdot = l(neq,&t,Y4store+index,uvalue);
	z += deltaT*zdot/6;
      }
      *mxGetPr(ans1) = g(neq,&t,x0,state+(points-1)*neqt) + z;
    }
    else {
      if (!traj_constr) {
	/* This must be an endpoint constraint. */
	neq[4] = f_num - ntraj_constr*points;
	*mxGetPr(ans1) = g(neq,Y+points-1,x0,state+(points-1)*neqt);
      }
      else {
	/* This is a trajectory constraint. */
	/* neq[4] = the trajectory constraint number. */
	neq[4] = (f_num - nfunctions - 1)/points + nfunctions + 1; 
	neq[3] = i = f_num - nfunctions - (neq[4]-1-nfunctions)*points - 1;
	uvalue = spline+2*i*ninputs;
	mxGetPr(ans1)[0] = l(neq,Y+i,state+i*neqt,uvalue);
      }
    }
  }
  
  /*******************************************************************/
  /* Compute adjoint and gradient.  The strategy is to first compute */
  /* The derivatives with respect to u(t_i+c_j*Delta) and then use   */
  /* the chainrule to compute dJ/d_alpha_i = sum_k dJ/du_k*B(i,t_k). */
  /*******************************************************************/
  if (comp_du) {
    ans3 = mxCreateFull(ninputs,(points-2)+Order,REAL);
    dJdalpha = mxGetPr(ans3); 
    ans5 = mxCreateFull(neqt,points,REAL);
    adjoint = mxGetPr(ans5);
    
    zero_out(J_x0,neqt);
    if (!traj_constr) {
      if (f_num <= nfunctions)
	neq[4] = f_num;
      else
	neq[4] = f_num - ntraj_constr*points;
      neq[3] = start = points - 1;
      t = Y[start];
      zero_out(p,neqt);
      Dg(neq,&t,x0,state+start*neqt,J_x0,p);		       /* Get p(Tf). */
    }
    else { 
      neq[4] = (f_num - nfunctions - 1)/points + nfunctions + 1;
      start = f_num - nfunctions - (neq[4]-1-nfunctions)*points - 1; 
      /* Adjoint and gradient are zero for i > start. */
      for ( i = points-1 ; i > start ; i-- )
	for ( j = 0 ; j < neqt ; j++ )
	  adjoint[i*neqt+j] = 0.0;
      /* I have to zero out dJdu here since it was callocated elsewhere. */
      for ( i = 3*(points-1)-1 ; i > 3*start-1 ; i-- ) 
	for ( j = 0 ; j < ninputs ; j++ )
	  dJdu[i*ninputs+j] = 0.0;
      /* Get p(Tf), we'll add in (d/du_i)(phi_i) at the end. */
      t = Y[start]; 
      neq[3] = i = start;
      zero_out(p,neqt);
      uvalue = spline+2*start*ninputs;
      Dl(neq,&t,state+start*neqt,uvalue,p,l_u1);	/* I don't use l_u1. */
    }

    for ( j = 0 ; j < neqt ; j++)
      adjoint[start*neqt+j] = p[j];

    zero_out(l_x1,neqt); zero_out(l_u1,ninputs);
    zero_out(l_x2,neqt); zero_out(l_u2,ninputs);
    zero_out(l_x3,neqt); zero_out(l_u3,ninputs);
    zero_out(l_x4,neqt); zero_out(l_u4,ninputs);

    if (linear_p) {
      DH_calls += 4;
      Dh(neq,&t,Y4store,spline,h_x4,h_u4);
      Dh(neq,&t,Y3store,spline,h_x3,h_u3);
      Dh(neq,&t,Y2store,spline,h_x2,h_u2);
      Dh(neq,&t,state,spline,h_x1,h_u1);
    }

    for ( i = start-1 ; i >= 0 ; i-- ) {
      neq[3] = i;
      t=Y[i+1];
      tout = Y[i];
      deltaT = t-tout;
      
      index = i*neqt;
      
      if (!linear_p) {
	DH_calls += 4;
	uvalue = spline + 2*(i+1)*ninputs;
	Dh(neq,&t,Y4store+index,uvalue,h_x4,h_u4);
	t -= deltaT/2;
	uvalue = spline + (2*i+1)*ninputs;
	Dh(neq,&t,Y3store+index,uvalue,h_x3,h_u3);
	Dh(neq,&t,Y2store+index,uvalue,h_x2,h_u2);
	t = tout;
	uvalue = spline + 2*i*ninputs;
	Dh(neq,&t,state+index,uvalue,h_x1,h_u1);
      }

      if ( f_num <= nfunctions ) {
	t = Y[i+1];
	uvalue = spline + 2*(i+1)*ninputs;
	Dl(neq,&t,Y4store+index,uvalue,l_x4,l_u4);
	t -= deltaT/2;
	uvalue = spline + (2*i+1)*ninputs;
	Dl(neq,&t,Y3store+index,uvalue,l_x3,l_u3);
	Dl(neq,&t,Y2store+index,uvalue,l_x2,l_u2);
	t = tout;
	uvalue = spline + 2*i*ninputs;
	Dl(neq,&t,state+index,uvalue,l_x1,l_u1);
      }

      /* Compute intermediate adjoint variables. */
      for ( row = 0 ; row < neqt ; row++ ) {
	q3[row] = p[row] + l_x4[row]*deltaT/2;
	for ( col = 0 ; col < neqt ; col++ )
	  q3[row] += h_x4[col*neqt+row]*p[col]*deltaT/2;
      }
      for ( row = 0 ; row < neqt ; row++ ) {
	q2[row] = p[row] + l_x3[row]*deltaT/2;
	for ( col = 0 ; col < neqt ; col++ )
	  q2[row] += h_x3[col*neqt+row]*q3[col]*deltaT/2;
      }
      for ( row = 0 ; row < neqt ; row++ ) {
	q1[row] = p[row] + l_x2[row]*deltaT;
	for ( col = 0 ; col < neqt ; col++ )
	  q1[row] += h_x2[col*neqt+row]*q2[col]*deltaT;
      }

      /* Compute gradients. */
      du = dJdu+(3*i+2)*ninputs;
      for ( row = 0 ; row < ninputs ; row++ ) {	 
	du[row] = l_u4[row]*deltaT/6;
	for ( col = 0 ; col < neqt ; col++ )
	  du[row] += h_u4[col*ninputs+row]*p[col]*deltaT/6;
      }
      du -= ninputs; 
      for ( row = 0 ; row < ninputs ; row++ ) {	 
	du[row] = (l_u3[row] + l_u2[row])*deltaT/3;
	for ( col = 0 ; col < neqt ; col++ )
	  du[row] += (h_u3[col*ninputs+row]*q3[col]
		      + h_u2[col*ninputs+row]*q2[col])*deltaT/3;
      }
      du -= ninputs;
      for ( row = 0 ; row < ninputs ; row++ ) {	 
	du[row] = l_u1[row]*deltaT/6;
	for ( col = 0 ; col < neqt ; col++ )
	  du[row] += h_u1[col*ninputs+row]*q1[col]*deltaT/6;
      }

      /* Compute next adjoint variable. */
      q0 = adjoint + index;
      for ( row = 0 ; row < neqt ; row++ ) {
        q0[row] = p[row] + 
	  (l_x1[row] + 2*(l_x2[row]+l_x3[row]) + l_x4[row])*deltaT/6;
        for ( col = 0 ; col < neqt ; col ++ )
          q0[row]+=(h_x1[col*neqt+row]*q1[col]
		     +2*(h_x2[col*neqt+row]*q2[col]+h_x3[col*neqt+row]*q3[col])
		     +h_x4[col*neqt+row]*p[col])*deltaT/6;
      }
      for ( row = 0 ; row < neqt ; row++ )
        p[row] = q0[row];
    }

    if (traj_constr) {
      /* Add in (d/du_i)(phi(x_i,u_i)). */
      zero_out(l_u1,ninputs);
      uvalue = spline + 2*start*ninputs;
      Dl(neq,&t,state+start*neqt,uvalue,l_x1,l_u1);
      if ( start < points - 1) {
	for ( row = 0 ; row < ninputs ; row++ )
	  dJdu[3*start*ninputs+row] = l_u1[row];
      }
      else {
	for ( row = 0 ; row < ninputs ; row++ )
	  dJdu[(3*start-1)*ninputs+row] += l_u1[row];
      }
    }

    /************************************************************************/
    /* At this point, we have computed dJ/du.  Now we will compute dJ/dalpha*/
    /* using the chain rule: dJ/dalpha = dJ/du*du/dalpha.                   */
    /* This computation relies on the fact that u(t_k+DELTA) = u(t_{k+1}).  */
    /* I renumbered the alpha's so that their indices start at 0 not 1.     */
    /************************************************************************/
#define Dalpha(A,B) dJdalpha[(A)*ninputs + B]

    /* Note, dJdu has ninputs extra zeros at the begining and end.         */
    /* Thus, Du(k,0,j) = 0 for k = -3,-2,-1 and k=points,points+1,points+2 */
    /* I assume that Dalpha[] starts out filled with zeros and the alphas  */
    /* are indexed starting from 0, not 1.                                 */
    for ( k = 0 ; k < points-1 ; k++ ) {
      du = dJdu + 3*k*ninputs;
      /* Add in contribution from interval enpoint. */
      bsplvb(Order,Y+k-(Order-1),Y[k],Bival);
      for ( j = 0 ; j < ninputs ; j++ ) {
	duvalue = (du[j-ninputs] + du[j]);
	for ( i = 0 ; i < Order ; i++ )
	  Dalpha(k+i,j) += Bival[i]*duvalue;
      }
      /* Add in contribution from interval midpoint. */
      bsplvb(Order,Y+k-(Order-1),(Y[k]+Y[k+1])/2,Bival);
      for ( j = 0 ; j < ninputs ; j++ ) {
	duvalue = du[j+ninputs];
	for ( i = 0 ; i < Order ; i++ )
	  Dalpha(k+i,j) += Bival[i]*duvalue;
      }
    }
    /* Finish off the last index. */
    k = points-2;
    du = dJdu + 3*(k+1)*ninputs;         /* Recall, there is an extra       */
                                         /* ninputs 0's at the end of dJdu. */
    bsplvb(Order,Y+k-(Order-1),Y[k+1],Bival);  
    for ( j = 0 ; j < ninputs ; j++ ) {
      duvalue = (du[j-ninputs] + du[j]);
      for ( i = 0 ; i < Order ; i++ )
	Dalpha(k+i,j) += Bival[i]*duvalue;
    }

    if ( form == 1 ) i = 2; else i = 0;
    lhs[i] = ans3;

    if ( nlhs >= 2+i ) {
      /* Compute gradient w.r.t. initial conditions. */
      ans4 = mxCreateFull(neqt,1,REAL);
      for ( j = 0 ; j < neqt ; j++ )
	mxGetPr(ans4)[j] = J_x0[j]+p[j];		     /* Add in p(0). */
      lhs[1+i] = ans4;
    }
      
    if (return_p) 
      lhs[2+i] = ans5;
    else
      mxFreeMatrix(ans5);
  }
 
  return;
}
