/******************************************/
/*                                        */
/* Copyright (c) 1996 by Adam L. Schwartz */
/*          All right reserved.           */
/*                                        */
/*****************************************************************/
/* Modified: Nov. 5, 1996.                                       */
/*****************************************************************/
/* This program calls LSODA to integrate the system.  Gradients  */
/* are computed approximately.  The inputs can be splines of     */
/* order = 1, 2, 3, or 4.                                        */
/*****************************************************************/
/* control dimension = m x N+K-1 */
/* Relative and absolute tolerances can be specified as follows:
     [ialg]       (in this case, atol = rtol = 1e-8 and n_intrnl_knots = 1)
     [ialg;n_intrnl_knot]  (in this case, atol = rtol = 1e-8).
     [ialg;n_intrnl_knot;rtol]  (in this case, atol = 1e-8).
     [ialg;n_intrnl_knot;rtol;atol]
*/

#define RTOL 1e-8
#define ATOL 1e-8
#define MAX_ORDER 4                     /* Maximum spline order. */

#include "mex.h"
#include "utility.h"
#include <math.h>
#define STATIC
#include "system.h"

extern int lsoda_();
extern int intdy_();

/* global variable used to point to current */
static int neqt,ninputs,nparams,nfunctions,ntraj_constr,nend_pt_constr,f_num;
static int n_intrnl_knots,linear_p,hermite_initialized = 0,Order;
static double *ctrlptr,*ctrl_val,*Y;
static double *h_x,*l_x,*h_u,*l_u;
static double *state,*x_knot_values,*xdot_knot_values,*x_interp;
static double **coefficients,*T,Bival[MAX_ORDER];

extern long unsigned int H_calls,DH_calls;

void adams(nlhs, lhs, nrhs, rhs)
     int nlhs, nrhs;
     Matrix *lhs[], *rhs[];
{
  static void fjac(),hjac(),fnc(),hfnc();
  static int initialized = 0, old_n_knots = -1, deriv = 1;
  static int neq[] = {0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  int index, n_knots_changed_p = 0,Stiff_p;
  int istate;       	        /* Variables that lsoda() modifies shouldn't */
  static int iopt = 0;		/* be static.				     */
  int jt = 1;                   /* Jacobians calculated analytically.        */
  static int itask = 4; /* Don't interpolate beyond control discontinuities. */
  static int liw;
  static int lrw;
  static int *iwork;
  static int    itol = 1;
  static double atol;
  static double rtol;
  static double *rwork;
  static double *x,*p,*z,*dJ,*xdot,*zdot,*J_x0;
  
  static unsigned int points = 0;
  static double *adjoint,*dJdalpha,*control,*x0;
  int i,j,k,row,col,action,form,N,start,traj_constr,length,knot_index;
  int return_x=0,return_j=0,return_p=0;
  int comp_x=0,comp_j=0,comp_du=0,iflag;
  double t,tout,deltaT;
  Matrix *u,*interval,*ans1,*ans2,*ans3,*ans4,*ans5,*ialg;
  
  if ( nrhs == 0 ) {
    if (hermite_initialized) {
      free(coefficients[0]);	 /* It was all allocated in one, big chunck. */
      free(coefficients); free(T);
      hermite_initialized = 0;
    }
    if ( initialized ) {
      free(iwork); free(rwork); free(xdot); free(J_x0);
      free(xdot_knot_values); free(x_knot_values);
      free(x); free(h_x); free(l_x); free(h_u); free(l_u); 
      free(state); free(x0); free(x_interp); free(control); 
      free(Y-MAX_ORDER+1);
      initialized = 0;
    }
    return;
  }
  
  form = (int) mxGetScalar(rhs[0]);
  if ( form == 1 ) {
    u = rhs[2];
    interval = rhs[3];
    traj_constr = 0;
  
    ialg = rhs[4];
    length = ( mxGetM(ialg) > mxGetN(ialg) ) ? mxGetM(ialg) : mxGetN(ialg);
    if ( length > 4 )
      mexErrMsgTxt("ialg = [ialg;knots;rtol;atol].");
    if ( length < 2 ) 
      n_intrnl_knots = 1;
    else {
      n_intrnl_knots = (int) mxGetPr(ialg)[1];
      if ( (double) n_intrnl_knots != mxGetPr(ialg)[1] )
	mexErrMsgTxt("You must specify an integer number of internal knots.");
      if ( n_intrnl_knots < 1 ) n_intrnl_knots = 1;
    }
    n_intrnl_knots += 2;     /* Also store endpoint values in each interval. */
    switch (length) {
    case 4:
      atol = mxGetPr(ialg)[3];
      rtol = mxGetPr(ialg)[2];
      break;
    case 3:
      atol = ATOL;
      rtol = mxGetPr(ialg)[2];
      break;
    default:
      rtol = RTOL;
      atol = ATOL;
    }
    if ( n_intrnl_knots != old_n_knots ) {
      n_knots_changed_p = 1;
      old_n_knots = n_intrnl_knots;
    }

    neq[4] = f_num = 1;				 /* Evaluate first function. */
    comp_x = 1;
    action = (int) mxGetScalar(rhs[5]);
    if (action > 0 ) return_j = 1;
    if (action > 1 ) return_x = 1;
    if (action > 2 ) comp_du  = 1;
    if (action > 3 ) return_p = 1;

    if ( points != mxGetN(interval) ) {
      points = mxGetN(interval);
      if ( points < 2 ) 
	mexErrMsgTxt("There must be at least two time points!");

      if (initialized) {
	free(xdot_knot_values); free(x_knot_values); 
	free(ctrl_val); free(control); free(Y-MAX_ORDER+1); free(state);
	state   = (double*)callocate(points,sizeof(double)*neqt,19);
	control = (double*)callocate(points-2+MAX_ORDER,
				     sizeof(double)*ninputs,21);
        Y       = (double*)mallocate(sizeof(double)*(points+2*(MAX_ORDER-1)),
				     22);
	Y      += MAX_ORDER-1;
	x_knot_values = (double*)callocate(points*n_intrnl_knots,
					    sizeof(double)*neqt,23);
	xdot_knot_values = (double*)callocate(points*n_intrnl_knots,
					       sizeof(double)*neqt,24);
      }
    }
    else 
      if (n_knots_changed_p) {
	free(xdot_knot_values); free(x_knot_values); 
	x_knot_values = (double*)callocate(points*n_intrnl_knots,
					    sizeof(double)*neqt,25);
	xdot_knot_values = (double*)callocate(points*n_intrnl_knots,
					       sizeof(double)*neqt,26);
      }
    
    Order = mxGetN(u) - (points-2);
    if ( Order < 1 || Order > 4 )
      mexErrMsgTxt("u wrong length...spline order can be 1, 2, 3, or 4.");

    /* Allocate main workspace memory. */
    if (!initialized) {
      init(neq,NULL);
      neqt=neq[0];
      ninputs = neq[1];
      nparams = neq[2];
      nfunctions = neq[5];
      ntraj_constr = neq[6]+neq[7];
      nend_pt_constr = neq[8]+neq[9]+neq[10]+neq[11];
      linear_p = neq[12];

      /* N is the size of the adjoint + gradient vectors that will */
      /* will need to be stored.                                   */
      N = neqt+MAX_ORDER*ninputs;    
      liw = 20 + N;
      lrw = 22 + N*(N+16);
      iwork = (int*)mallocate(sizeof(int)*liw,1);
      rwork = (double*)mallocate(sizeof(double)*lrw,2);
      x = (double*)mallocate(sizeof(double)*N,3);
      z = x + neqt;
      p = x;
      dJ = p + neqt;
      xdot = (double*)callocate(neqt+1,sizeof(double),4);
      zdot = xdot + neqt;
      J_x0 = (double*)mallocate(sizeof(double)*neqt,5);
      h_x = (double*)callocate(neqt,sizeof(double)*neqt,6);
      l_x = (double*)callocate(neqt,sizeof(double)*neqt,7);
      h_u = (double*)callocate(neqt,sizeof(double)*ninputs,8);
      l_u = (double*)callocate(ninputs,sizeof(double),9);
      state = (double*)callocate(points,sizeof(double)*neqt,10);
      x_knot_values = (double*)callocate(points*n_intrnl_knots,
					  sizeof(double)*neqt,11);
      xdot_knot_values = (double*)callocate(points*n_intrnl_knots,
					     sizeof(double)*neqt,12);
      x_interp = (double*)mallocate(sizeof(double)*neqt,13);
      x0    = (double*)mallocate(sizeof(double)*neqt,14);
      control = (double*)callocate(points-2+MAX_ORDER,
				   sizeof(double)*ninputs,16);
      Y     = (double*)mallocate(sizeof(double)*(points+2*(MAX_ORDER-1)),17);
      Y    += MAX_ORDER-1;
      ctrl_val = (double*)mallocate(sizeof(double)*ninputs,18);
      initialized=1;
    }

    if ( mxGetM(rhs[1]) != neqt )
      mexErrMsgTxt("x0 wrong length.");
    if ( mxGetM(u) != ninputs) 
      mexErrMsgTxt("Not enough control inputs.");

    /* Save x0, u, Y for calls later by simulate() form 2. */
    for ( i = 0 ; i < neqt ; i++ )
      x0[i] = mxGetPr(rhs[1])[i];
    /* Store spline coefficients in control[]. */
    for ( i = 0 ; i < (points-2)+Order ; i++)
      for ( j = 0 ; j < ninputs ; j++ )
	control[i*ninputs+j] = mxGetPr(u)[i*ninputs+j];
    for ( i = 0 ; i < points ; i++ )
      Y[i] = mxGetPr(interval)[i];
    /* Pad beginning and end of Y with t_i and t_f respectively. */
    for ( i = 1 ; i < MAX_ORDER ; i++ ) {
      Y[0-i] = Y[0];
      Y[points-1+i] = Y[points-1];
    }
  }
  else {
    f_num = (int) mxGetScalar(rhs[1]);          /* Function number. */
    action = (int) mxGetScalar(rhs[2]);         /* Action number.   */
    if (f_num > nfunctions && f_num <= nfunctions + ntraj_constr*points)
      traj_constr = 1;
    else
      traj_constr = 0;
    if (f_num > nfunctions + ntraj_constr*points + nend_pt_constr)
      mexErrMsgTxt("Function number too big.");
    if (f_num < 1) mexErrMsgTxt("Functions number must be > 0.");

    switch (action) {
    case 3:
      return_p = 1;
    case 2: 
      comp_du = 1;
      break;
    case 1:
      comp_j = 1;
    }
  }

  if ( comp_j ) {
    if ( f_num <= nfunctions ) {
      /* Compute cost below in next big if statement. */
      /* These is very inefficient for multiple objective functions and */
      /* possibly inefficient even for a single objective function.     */
      neq[4] = f_num;
      return_j = 1;
    }
    else {
      ans1 = mxCreateFull(1,1,REAL);
      lhs[0] = ans1;
      if (!traj_constr) {
	/* This must be an endpoint constraint. */
	neq[4] = f_num - ntraj_constr*points;
	*mxGetPr(ans1) = g(neq,Y+points-1,x0,state+(points-1)*neqt);
      }
      else {
	/* This is a trajectory constraint. */
	/* neq[4] = the trajectory constraint number. */
	neq[4] = (f_num - nfunctions - 1)/points + nfunctions + 1; 
	neq[3] = i = f_num - nfunctions - (neq[4]-1-nfunctions)*points - 1;
	/**************************************************************/
	/* Evaluate B-Spline at time Y[i].                            */
	/* The conditional for (i==points-1) is needed because the    */
	/* last time breakpoint must be considered as occuring at     */
	/* the right end of the relevant time interval, not the left. */
	/**************************************************************/
	if ( i == points - 1 ) {
	  ctrlptr=control+(i-1)*ninputs;
	  bsplvb(Order,Y+i-Order,t,Bival);
	}
	else {
	  ctrlptr=control+i*ninputs;
	  bsplvb(Order,Y+i-(Order-1),t,Bival);
	}
	for ( j = 0 ; j < ninputs ; j++ ) {
	  ctrl_val[j] = Bival[0]*ctrlptr[j];
	  for ( k = 1 ; k < Order ; k++ )
	    ctrl_val[j] += Bival[k]*ctrlptr[k*ninputs+j];
	}
	mxGetPr(ans1)[0] = l(neq,Y+i,state+i*neqt,ctrl_val);
      }
    }
  }

  if ( comp_x || return_j ) {

    /* Get initial conditions. */
    for (j = 0 ; j < neqt ; j++)
      x[j] = state[j] = x0[j];
    x[neqt] = 0.0;                                      /* Initial cost = 0. */
    
    /* Initialize lsoda() optional arguments. */
    
    t = Y[0]; 
    neq[0] = neqt+1;					 /* also integrate z */
    *zdot = 0.0;
    Stiff_p = 0;
    for ( i = 0 ; i < points-1 ; i++) {
      deltaT = Y[i+1] - Y[i];
      rwork[0] = Y[i+1];		     /* Set tcrit = next breakpoint. */
      neq[3] = i;
      ctrlptr = control + i*ninputs;
      istate = 1;		       /* Restart lsoda at EACH breakpoint.  */

      knot_index = i*n_intrnl_knots;
      for ( j = 0 ; j < neqt ; j++) 
	x_knot_values[knot_index*neqt+j] = x[j];
      fnc(neq,&t,x,xdot_knot_values+knot_index*neqt);

      /* Within each control interval, compute values of x and xdot */
      /* at n_intrnl_knots knot points.                             */
      for ( k = 1 ; k < n_intrnl_knots ; k++ ) {

	/* Inexact arithmetic may cause tout > tcrit. */
	if ( k < n_intrnl_knots-1 )
	  tout = Y[i] + k*deltaT/(n_intrnl_knots-1);
	else
	  tout = Y[i+1];

	do {
	  lsoda_(fnc,neq,x,&t,&tout,&itol,&rtol,&atol,&itask,&istate,&iopt,
		 rwork,&lrw,iwork,&liw,fjac,&jt);
	  switch (istate) {
	  case -6:
	    mexErrMsgTxt("LSODA crashed because absolute tol = 0 and a soln component went to zero.");
	    break;
	  case -5:
	  case -4:
	  case -3: 
	    mexErrMsgTxt("LSODA crashed.");
	    break;
	  case -2:
	    mexPrintf("LSODA: Too much accuracy requested, increasing tolerance.\n");
	    atol *= rwork[13];
	    rtol *= rwork[13];
	    istate = 3;
	    break;
	  case -1:
	    mexPrintf("LSODA: Excessive amount of work done on this step.\n");
	    break;
	  }
	} while ( t != tout );
	if ( iwork[18] == 2 ) Stiff_p = 1;

	knot_index++;
	/* Store values of x at internal knot point. */
	for ( j = 0 ; j < neqt ; j++) 
	  x_knot_values[knot_index*neqt+j] = x[j];
	/* Compute and store values of xdot at internal knot point. */
	intdy_(&rwork[12],&deriv,&rwork[20],neq,xdot_knot_values+knot_index*neqt,
	       &iflag);
	if ( iflag == -2 )
	  mexErrMsgTxt("Error calling intdy_: t was illegal");
      }
      for ( j = 0 ; j < neqt ; j++) 
        state[(i+1)*neqt+j] = x[j];
    }

    if (Stiff_p) mexPrintf("LSODA detected stiffness.\n");

    if (return_j) {
      ans1 = mxCreateFull(1,1,REAL);
      lhs[0] = ans1;
      *mxGetPr(ans1) = g(neq,&t,x0,x) + *z;
    }
    if (return_x) {
      ans2 = mxCreateFull(neqt,points,REAL);
      lhs[1] = ans2;
      index = 0;
      for ( i = 0 ; i < points ; i++ ) 
	for ( j = 0 ; j < neqt ; j++ ) {
	  mxGetPr(ans2)[index] = state[index];
	  index++;
	}
    }
  }    

  if (comp_du) {
    ans3 = mxCreateFull(ninputs,(points-2)+Order,REAL);
    dJdalpha = mxGetPr(ans3);
    ans5 = mxCreateFull(neqt,points,REAL);
    adjoint = mxGetPr(ans5);

    zero_out(J_x0,neqt);
    if (!traj_constr) {
      if (f_num <= nfunctions)
	neq[4] = f_num;
      else
	neq[4] = f_num - ntraj_constr*points;
      start = points - 1;
      t = Y[start];
      zero_out(p,neqt);
      Dg(neq,&t,x0,state+start*neqt,J_x0,p);		       /* Get p(Tf). */
/*
      if ( Order > 1 )
	zero_out(dJdalpha+start*ninputs,ninputs);
*/
    }
    else { 
      neq[4] = (f_num - nfunctions - 1)/points + nfunctions + 1;
      start = f_num - nfunctions - (neq[4]-1-nfunctions)*points - 1; 
      neq[3] = i = start;
      /* Adjoint and gradient are zero for i > start. */
      for ( i = points-1 ; i > start ; i-- )
	for ( j = 0 ; j < neqt ; j++ )
	  adjoint[i*neqt+j] = 0.0;
/*
      for ( i = points-2 ; i > start ; i-- ) 
	for ( j = 0 ; j < ninputs ; j++ )
	  dJdalpha[i*ninputs+j] = 0.0;
*/
      t = Y[start];

      /* Get p(Tf) and add in (d/dalpha_i)(phi_i(x_i,u_i). */
      zero_out(p,neqt); zero_out(l_u,ninputs);
      if ( start == points - 1 ) {
	ctrlptr=control+(i-1)*ninputs;
	bsplvb(Order,Y+i-Order,t,Bival);
      }
      else {
	ctrlptr=control+i*ninputs;
	bsplvb(Order,Y+i-(Order-1),t,Bival);
      }
      for ( j = 0 ; j < ninputs ; j++ ) {
	ctrl_val[j] = Bival[0]*ctrlptr[j];
	for ( k = 1 ; k < Order ; k++ )
	  ctrl_val[j] += Bival[k]*ctrlptr[k*ninputs+j];
      }
      Dl(neq,&t,state+start*neqt,ctrl_val,p,l_u);
      for ( k = 0 ; k < Order ; k++ )
	for ( j = 0 ; j < ninputs ; j++ )
	    dJdalpha[(i+k)*ninputs+j] += Bival[k]*l_u[j];
    }

    for ( j = 0 ; j < neqt ; j++) 
      adjoint[start*neqt+j] = p[j];

    zero_out(l_x,neqt); zero_out(l_u,ninputs); 
    neq[0] = neqt+Order*ninputs;
    zero_out(dJ,Order*ninputs);
    t = Y[start];

    if (linear_p) {
      /* If system is linear, value of control and state don't matter. */
      Dh(neq,&t,state,control,h_x,h_u); DH_calls++;
    }

    for ( i = start-1 ; i >= 0 ; i-- ) {
      istate = 1;					 /* Restart lsoda(). */
      rwork[0] = tout = Y[i];
      deltaT = Y[i+1] - tout;
      neq[3] = i;
      ctrlptr = control + i*ninputs;
      
      do {
	lsoda_(hfnc,neq,x,&t,&tout,&itol,&rtol,&atol,&itask,&istate,&iopt,
	       rwork,&lrw,iwork,&liw,hjac,&jt);
	switch (istate) {
	case -6:
	  mexErrMsgTxt("LSODA crashed because absolute tol = 0 and a soln component went to zero.");
	  break;
	case -5:
	case -4:
	case -3: 
	  mexErrMsgTxt("LSODA crashed.");
	  break;
	case -2:
	  mexPrintf("LSODA: Too much accuracy requested, increasing tolerance.\n");
	  atol *= rwork[13];
	  rtol *= rwork[13];
	  istate = 3;
	  break;
	case -1:
	mexPrintf("LSODA: Excessive amount of work done on this step.\n");
	  break;
	}
      } while ( t != tout );
      
      /* Successfully integrated to tout.  Store adjoint and gradients. */
      /* The following relies on the fact that dJdalpha   */
      /* is initially all zeros.   Recall, dJ = p + neqt. */
      for ( k = 0 ; k < Order ; k++ )
	for ( j = 0 ; j < ninputs ; j++ ) {
	  dJdalpha[(i+k)*ninputs+j] += dJ[k*ninputs+j];
	  dJ[k*ninputs+j] = 0.0;
	}
      for ( j = 0 ; j < neqt ; j++) 
	adjoint[i*neqt+j] = p[j];
    }
    
    if ( form == 1 ) i = 2; else i = 0;
    lhs[i] = ans3;
    
    if ( nlhs >= 2+i ) {
      /* Compute gradient w.r.t. initial conditions. */
      ans4 = mxCreateFull(neqt,1,REAL);
      for ( j = 0 ; j < neqt ; j++ )
	mxGetPr(ans4)[j] = J_x0[j]+p[j];		     /* Add in p(0). */
      lhs[1+i] = ans4;
    }
      
    if (return_p) 
      lhs[2+i] = ans5;
    else
      mxFreeMatrix(ans5);
  }
}

static void 
fjac(neq,t,y,md,mu,pd,nrowpd)
     int *neq,*md,*mu,*nrowpd;
     double *t,*y,*pd;
{
  int i,j,k,row,col;
  double *x;

  i = neq[3];                    /* Current time index. */
  x = state+i*neqt;
  /* Compute Control value from B-spline coefficients. */
  bsplvb(Order,Y+i-(Order-1),*t,Bival);
  for ( j = 0 ; j < ninputs ; j++ ) {
    ctrl_val[j] = Bival[0]*ctrlptr[j];
    for ( k = 1 ; k < Order ; k++ )
      ctrl_val[j] += Bival[k]*ctrlptr[k*ninputs+j];
  }
  
  Dh(neq,t,x,ctrl_val,h_x,h_u); DH_calls++;
  Dl(neq,t,x,ctrl_val,l_x,l_u); DH_calls++;
  
  /* Fortran reads C matrices transposed, so save it transposed. */
  for ( row = 0 ; row < neqt ; row++ ) {
    for ( col = 0 ; col < neqt; col++ )
      pd[row*(neqt+1)+col] = h_x[col*neqt+row];
    pd[row*(neqt+1)+neqt] = l_x[row];
  }
  for ( col = 0 ; col < neqt ; col++ )
    pd[neqt*(neqt+1)+col] = 0.0;
}

static void 
hjac(neq,t,y,md,mu,pd,nrowpd)
     int *neq,*md,*mu,*nrowpd;
     double *t,*y,*pd;
{
  static void interpolate();
  int i,j,k,row,col,size_pd;
  double *x;
  
  i = neq[3];                    /* Current time index. */

  /* Compute Control value from B-spline coefficients. */
  bsplvb(Order,Y+i-(Order-1),*t,Bival);
  for ( j = 0 ; j < ninputs ; j++ ) {
    ctrl_val[j] = Bival[0]*ctrlptr[j];
    for ( k = 1 ; k < Order ; k++ )
      ctrl_val[j] += Bival[k]*ctrlptr[k*ninputs+j];
  }

  if (!linear_p) {
    interpolate(i,*t,x_interp);
    Dh(neq,t,x_interp,ctrl_val,h_x,h_u); DH_calls++;
  }
  size_pd = neqt+Order*ninputs;

  /* Fortran reads C matrices transposed which is what we want in this case. */
  for ( row = 0 ; row < neqt; row++ )
    for ( col = 0 ; col < neqt ; col++ )
      pd[row*size_pd+col] = -h_x[row*neqt+col];

  for ( row = 0 ; row < neqt; row++ )
    for ( col = 0 ; col < ninputs ; col++ )
      for ( k = 0 ; k < Order ; k++ )
	pd[row*size_pd+neqt+k*ninputs] = -h_u[row*ninputs+col]*Bival[k];

  for ( row = neqt ; row < size_pd ; row++ )
    for ( col = 0 ; col < size_pd ; col++ )
      pd[row*size_pd+col] = 0.0;
}

static void
fnc(neq,t,x,xdot)
     int *neq;
     double *t,*x,*xdot;
{
  int i,j,k;

  i = neq[3];                    /* Current time index. */
  
  /* Compute Control value from B-spline coefficients. */
  bsplvb(Order,Y+i-(Order-1),*t,Bival);
  for ( j = 0 ; j < ninputs ; j++ ) {
    ctrl_val[j] = Bival[0]*ctrlptr[j];
    for ( k = 1 ; k < Order ; k++ )
      ctrl_val[j] += Bival[k]*ctrlptr[k*ninputs+j];
  }
    
  h(neq,t,x,ctrl_val,xdot); H_calls++;
  xdot[neqt] = l(neq,t,x,ctrl_val);
  neq[3] = i;				    /* neq[3] can be changed by l(). */
}

/* This is fnc computes the RHS for the adjoint equation. */
static void
hfnc(neq,t,p,pdot)
     int *neq;
     double *t,*p,*pdot;
{
  static void interpolate();
  int row,col,i,j,k;
  double dt,*dJ;
  
  i = neq[3];
  dJ = pdot+neqt;

  /* Compute Control value from B-spline coefficients. */
  bsplvb(Order,Y+i-(Order-1),*t,Bival);
  for ( j = 0 ; j < ninputs ; j++ ) {
    ctrl_val[j] = Bival[0]*ctrlptr[j];
    for ( k = 1 ; k < Order ; k++ )
      ctrl_val[j] += Bival[k]*ctrlptr[k*ninputs+j];
  }

  interpolate(i,*t,x_interp);
  
  if (!linear_p) {Dh(neq,t,x_interp,ctrl_val,h_x,h_u); DH_calls++;}

  if ( f_num <= nfunctions )
    Dl(neq,t,x_interp,ctrl_val,l_x,l_u);
  
  /* Integrate adjoint equation. */
  for ( row = 0 ; row < neqt ; row++ ) {
    pdot[row] = -l_x[row];
    for ( col = 0 ; col < neqt ; col++ )
      pdot[row] -= h_x[col*neqt+row]*p[col];
  }
  
  /* Integrate H_u(t) to get gradient. */
  for ( row = 0 ; row < ninputs ; row++ ) {
    dJ[row] = -l_u[row];
    for ( col = 0 ; col < neqt ; col++ )
      dJ[row] -= h_u[col*ninputs+row]*p[col];
    for ( k = Order-1 ; k >= 0 ; k-- )
      dJ[k*ninputs+row] = dJ[row]*Bival[k];
  }
}

/**********************************************************************/
/* This routine computes x_interp from a hermite interpolation of x() */
/* at the HERMITE_ORDER internal knot points closest to t.            */
/* HERMITE_ORDER >= 2.                                                */
/* HERMITE_ORDER = 2: cubic ; HERMITE_ORDER = 3: quintic ; etc...     */
/* The Hermite interpolant is computed using divided differences, see */
/* Atkinson, pp. 137-141.                                             */
/**********************************************************************/
/* 
   The coefficients for the Hermite polynomial correspond to the expansion

   p(x) = coef[0] + (t-t1)*coef[1] + (t-t1)^2*coef[2] + (t-t1)^2*(t-t2)*coef[3]
         + (t-t1)^2*(t-t2)^2*coef[4] + (t-t1)^2*(t-t2)^2*(t-t3)*coef[5],

   for a quintic.
*/

#define HERMITE_ORDER 3

static void 
interpolate(index,t,x_interp)
     int index;
     double t,*x_interp;
{
  int i,j,k,order,first_knot;
  double temp,t1,deltaT;
  static void compute_hermite();
  static int old_first_knot = -1;

  if (!hermite_initialized) {
    /* Create array of dimension [neqt][2*HERMITE_ORDER] for coefficients */
    /* of the hermite interpolant.                                */
    coefficients    = (double**)calloc(neqt,sizeof(double*));
    coefficients[0] = (double*)calloc(neqt,2*HERMITE_ORDER*sizeof(double));
    for ( i = 1 ; i < neqt ; i++ )
      coefficients[i] = coefficients[i-1] + 2*HERMITE_ORDER;
    T = (double*)malloc(2*HERMITE_ORDER*sizeof(double));
    hermite_initialized = 1;
  }

  order = HERMITE_ORDER;
  if ( order > n_intrnl_knots ) order = n_intrnl_knots;

  /* Compute the knot index, k, corresponding to first */
  /* internal knot at time greater or equal to time t. */
  /* Note that numerical round-off may make k greater  */
  /* than n_intrnl_knots-1.  But this wont matter.     */
  deltaT = Y[index+1] - Y[index];
  k = (int) ceil( (n_intrnl_knots-1) * (t-Y[index])/deltaT );

  /* Figure out which internal knot is first for the hermite   */
  /* polynomial; k indicates how many knots _past_  index  the */
  /* the first knot should start.                              */
#if 0
  if ( k <= (order+1)/2 )
    k = 0;
  else 
    if ( k >= n_intrnl_knots - order/2 )
      k = n_intrnl_knots - order;
    else
      k = k-(order+1)/2;
#endif
  k -= order/2;
  if ( k < 0 ) 
    k = 0;
  else
    if ( k > n_intrnl_knots - order ) k = n_intrnl_knots - order;

  first_knot = index*n_intrnl_knots + k;
  
  if ( first_knot != old_first_knot ) {
    /* Make T = (t[0] t[0] t[1] t[1] ... t[order-1] t[order-1]). */
    for ( i = 0 ; i < order ; i++ )
      T[2*i+1] = T[2*i] = Y[index] + (k+i)*deltaT/(n_intrnl_knots-1);

    compute_hermite(first_knot,order,T,coefficients[0]);
    old_first_knot = first_knot;
  }

  /* Compute x(t) from hermite interpolate. */
  temp = 1.0;
  for ( i = 0 ; i < neqt ; i++ ) x_interp[i] = 0.0;

  for ( i = 0 ; i < 2*order ; i++ ) {
    for ( j = 0 ; j < neqt ; j++ ) 
      x_interp[j] += temp*coefficients[j][i];
    temp *= t - T[i];
  }
}


/* Compute hermite coefficients using divided differences. */
/* See hermite.m for a clearer picture of what's going on. */
static void
compute_hermite(first_knot,order,T,D)
     int first_knot,order;
     double T[],D[][2*HERMITE_ORDER];
{
  int level,j,k,N;
  double *x,*xdot,dT;

  N = 2*order;
  x = x_knot_values+first_knot*neqt;
  xdot = xdot_knot_values+first_knot*neqt;
  
  for ( j = 0 ; j < neqt ; j++ )
    D[j][0] = x[j];

  /* Load D[][] with first divided differences. */
  for ( level=1,k=0 ; level<N ; level+=2,k++ )
    for ( j = 0 ; j < neqt ; j++ )
      D[j][level] = xdot[k*neqt+j];

  for ( level=2,k=0 ; level<N ; level+=2,k++ ) {
    dT = T[level] - T[level-1];
    for ( j = 0 ; j < neqt ; j++ )
      D[j][level] = (x[(k+1)*neqt+j] - x[k*neqt+j])/dT;
  }
  
  /* Now compute the rest of the coefficients.            */
  /* Work backwards so I don't overwrite values needed in */
  /* subsequent caluculations.                            */
  for ( level = 2 ; level < N ; level++ )
    for ( k = N-level-1 ; k >= 0 ; k-- ) {
      dT = T[k+level] - T[k];
      for ( j = 0 ; j < neqt ; j++ )
	D[j][k+level] = (D[j][k+level] - D[j][k+level-1])/dT;
    }
}
