/******************************************/
/*                                        */
/* Copyright (c) 1996 by Adam L. Schwartz */
/*          All right reserved.           */
/*                                        */
/****************************************************/
/* This is for simulating systems given in the form */
/* x[i+1] = h_i(x[i],u[i])                          */
/* z[i+1] = l_i(x[i],u[i])                          */
/* The controls are of dimension m x N.             */
/****************************************************/

#include "mex.h"
#include "utility.h"
#define STATIC
#include "system.h"

extern long unsigned int H_calls,DH_calls;

void discrete(nlhs, lhs, nrhs, rhs)
     int nlhs, nrhs;
     Matrix *lhs[], *rhs[];  /* lhs is a pointer to the return arguments */
                             /* rhs is a pointer to the input arguments  */
{
  /* neq[0] -- order n ; neq[1] -- ninputs ; neq[2] -- # parameters */
  static unsigned int points = 0;
  static double *state,*adjoint,*dJdu,*control,*Y,*x0;
  static int neq[] = {0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0};  
  static int initialized = 0,linear_p = 0;
  static int neqt,ninputs,nparams,nfunctions,ntraj_constr,nend_pt_constr;
  static double *x,*xdot,z,*p,*temp,*tempB;
  static double *h_x,*l_x,*h_u,*l_u,*J_x0;
  static double *parameters;

  int i, j, index, row, col, action, form, f_num, start, traj_constr;
  int return_x=0,return_j=0,return_p=0;
  int comp_x=0,comp_j=0,comp_du=0;
  double t,tout;
  Matrix *u,*interval,*ans1,*ans2,*ans3,*ans4,*ans5;
  
  if ( nrhs == 0 ) {
    if ( initialized ) {
      free(x); free(xdot); free(temp); 
      free(J_x0); free(l_x); free(h_x);
      free(tempB); free(l_u); free(h_u); free(state);
      free(x0); free(control); free(Y);
      initialized = 0;
    }
    return;
  }
  
  form = (int) mxGetScalar(rhs[0]);
  if ( form == 1 ) {
    u = rhs[2];
    interval = rhs[3];
    traj_constr = 0;
  
    neq[4] = f_num = 1;					   /* Evaluate first */
    comp_x = 1;						   /* function.	     */
    action = (int) mxGetScalar(rhs[5]);
    if (action > 0 ) return_j = 1;
    if (action > 1 ) return_x = 1;
    if (action > 2 ) comp_du  = 1;
    if (action > 3 ) return_p = 1;

    if ( points != mxGetN(interval) ) {
      points = mxGetN(interval);
      if (initialized) {
	free(Y); free(control); free(state); 
	state   = (double*)callocate(points,sizeof(double)*neqt,13);
        control = (double*)callocate(points-1,sizeof(double)*ninputs,14);
        Y       = (double*)mallocate(sizeof(double)*points,15);
      }
    }
    
    /* On first call to driver only. */
    if (!initialized) {
      init(neq,NULL);
      neqt=neq[0];
      ninputs=neq[1];
      nparams=neq[2];
      nfunctions=neq[5];
      ntraj_constr = neq[6]+neq[7];
      nend_pt_constr = neq[8]+neq[9]+neq[10]+neq[11];
      linear_p = neq[12];

      x = (double*)mallocate(sizeof(double)*neqt,1);
      p = x;
      xdot  = (double*)callocate(neqt,sizeof(double),2);
      temp  = (double*)mallocate(sizeof(double)*neqt,3);
      J_x0   = (double*)callocate(neqt,sizeof(double),4);
      l_x   = (double*)callocate(neqt,sizeof(double),4);
      h_x   = (double*)callocate(neqt,sizeof(double)*neqt,5);
      tempB = (double*)mallocate(sizeof(double)*ninputs,6);
      l_u   = (double*)callocate(ninputs,sizeof(double),7);
      h_u   = (double*)callocate(neqt,sizeof(double)*ninputs,8);
      state = (double*)callocate(points,sizeof(double)*neqt,9);
      x0    = (double*)mallocate(sizeof(double)*(neqt),10);
      control = (double*)callocate(points-1,sizeof(double)*ninputs,11);
      Y     = (double*)mallocate(sizeof(double)*points,12);
      initialized = 1;
    }
    if ( mxGetM(rhs[1]) != neqt )
      mexErrMsgTxt("x0 wrong length.");
    if ( mxGetN(u) != points-1 || mxGetM(u) != ninputs)
      mexErrMsgTxt("u wrong length; u should be ninputs x (points-1).");

    /* Save x0, u, Y for calls later by simulate() form 2. */
    for ( i = 0 ; i < neqt ; i++ )
      x0[i] = mxGetPr(rhs[1])[i];
    for ( i = 0 ; i < points-1 ; i++)
      for ( j = 0 ; j < ninputs ; j++ )
	control[i*ninputs+j] = mxGetPr(u)[i*ninputs+j];  
    for ( i = 0 ; i < points ; i++ )
      Y[i] = mxGetPr(interval)[i];
    
  }
  else {
    f_num   = (int) mxGetScalar(rhs[1]);	 /* Function number. */
    action = (int) mxGetScalar(rhs[2]);	         /* Action number.   */
    if (f_num > nfunctions && f_num <= nfunctions + ntraj_constr*points)
      traj_constr = 1;
    else
      traj_constr = 0;
    if (f_num > nfunctions + ntraj_constr*points + nend_pt_constr)
      mexErrMsgTxt("Function number too big.");
    if (f_num < 1) mexErrMsgTxt("Functions number must be > 0.");

    switch (action) {
    case 3:
      return_p = 1;
    case 2: 
      comp_du = 1;
      break;
    case 1:
      comp_j = 1;
    }
  }

  if ( comp_x ) {
  
    /* Get initial conditions. */
    for (j = 0 ; j < neqt ; j++)
      state[j] = x[j] = x0[j];
    
    t = Y[0]; z = 0.0;
    for ( i = 0 ; i < points-1 ; i++) {
      tout = Y[i+1];
      neq[3] = i;
      h(neq,&t,x,control+i*ninputs,xdot); H_calls++;
      if (return_j) {
	/* If neq[3] = -1, l==0, but it's just is fast to not check. */
	z += l(neq,&t,x,control+i*ninputs);
      }
      for ( j = 0 ; j < neqt ; j++)
	state[(i+1)*neqt+j] = x[j] = xdot[j];
      t = tout;
    }
    if (return_j) {
      ans1 = mxCreateFull(1,1,REAL);
      lhs[0] = ans1;
      *mxGetPr(ans1) = g(neq,&t,x0,x) + z;
    }
    if (return_x) {
      ans2 = mxCreateFull(neqt,points,REAL);
      lhs[1] = ans2;
      index = 0;
      for ( i = 0 ; i < points ; i++ ) 
	for ( j = 0 ; j < neqt ; j++ ) {
	  mxGetPr(ans2)[index] = state[index];
	  index++;
	}
    }
  }
  
  if ( comp_j ) {
    ans1 = mxCreateFull(1,1,REAL);
    lhs[0] = ans1;
    if ( f_num <= nfunctions ) {
      neq[4] = f_num;
      t = Y[0]; z = 0.0;
      for ( i = 0 ; i < points-1 ; i++) {
	neq[3] = i;
	z += l(neq,&t,state+i*neqt,control+i*ninputs);
	if (neq[3] == -1) break;				   /* l == 0 */
	tout = Y[i+1];						     
	t = tout;
      }
      *mxGetPr(ans1) = g(neq,&t,x0,state+(points-1)*neqt) + z;
    }
    else {
      if (!traj_constr) {
	/* This must be an endpoint constraint. */
	neq[4] = f_num - ntraj_constr*points;
	*mxGetPr(ans1) = g(neq,Y+points-1,x0,state+(points-1)*neqt);
      }
      else {
	/* This is a trajectory constraint. */
	/* neq[4] = the trajectory constraint number. */
	neq[4] = (f_num - nfunctions - 1)/points + nfunctions + 1; 
	neq[3] = i = f_num - nfunctions - (neq[4]-1-nfunctions)*points - 1;
	if ( i < points - 1 )
	  l(neq,Y+i,state+i*neqt,control+i*ninputs,mxGetPr(ans1));
	else
	  /* Reuse the Nth control value to evaluate the (N+1)th constriaint.*/
	  mxGetPr(ans1)[0] = l(neq,Y+i,state+i*neqt,control+(i-1)*ninputs);
      }
    }
  }
  
  if ( comp_du ) {
    ans3 = mxCreateFull(ninputs,points-1,REAL);
    dJdu = mxGetPr(ans3);
    ans5 = mxCreateFull(neqt,points,REAL);
    adjoint = mxGetPr(ans5);

    zero_out(J_x0,neqt); /* Have to zero_out J_x0 even if traj. constr. */
    if (!traj_constr) {
      if (f_num <= nfunctions)
	neq[4] = f_num;
      else
	neq[4] = f_num - ntraj_constr*points;
      neq[3] = start = points - 1;
      t = Y[start];
      zero_out(p,neqt);
     Dg(neq,&t,x0,state+start*neqt,J_x0,p);		       /* Get p(Tf). */
    }
    else { 
      neq[4] = (f_num - nfunctions - 1)/points + nfunctions + 1;
      start = f_num - nfunctions - (neq[4]-1-nfunctions)*points - 1; 
      /* Adjoint and gradient are zero for i > start. */
      for ( i = points-1 ; i > start ; i-- )
	for ( j = 0 ; j < neqt ; j++ )
	  adjoint[i*neqt+j] = 0.0;
/*
      for ( i = points-2 ; i > start ; i-- ) 
	for ( j = 0 ; j < ninputs ; j++ )
	  dJdu[i*ninputs+j] = 0.0;
*/
      t = Y[start];
      neq[3] = start;
      /* Get p(Tf). */
      zero_out(p,neqt); 
      if ( start < points - 1 ) {
	/* Add in (d/du_i)(phi(x_i,u_i)). */
/*	zero_out(dJdu+i*ninputs,ninputs); */
	Dl(neq,&t,state+start*neqt,control+start*ninputs,p,dJdu+i*ninputs);
      }
      else 
	/* We'll add in (d/du_i)(phi_i) at the end. */
	Dl(neq,&t,state+start*neqt,control+(i-1)*ninputs,p,l_u);
    }

    for ( j = 0 ; j < neqt ; j++)
      adjoint[start*neqt+j] = p[j];

    /* Not really necessary if nfunctions=1 and no constraints. */
    zero_out(l_x,neqt); zero_out(l_u,ninputs);

    if (linear_p) {
      Dh(neq,&t,state,control,h_x,h_u); DH_calls++;
    }
    
    for ( i = start-1 ; i >= 0 ; i-- ) {
      tout = Y[i];
      neq[3] = i;
      if (!linear_p)
	Dh(neq,&t,state+i*neqt,control+i*ninputs,h_x,h_u); DH_calls++;
      /* if f_num > nfunctions, we must be	     */
      /* computing the gradient of a constraint.     */
      if ( f_num <= nfunctions )
	Dl(neq,&t,state+i*neqt,control+i*ninputs,l_x,l_u);
      t = tout;
      
      for ( row = 0 ; row < ninputs ; row++ ) {
	tempB[row] = l_u[row];
	for ( col = 0 ; col < neqt ; col++ )
	  tempB[row] += h_u[col*ninputs+row]*p[col];
	dJdu[i*ninputs+row] = tempB[row]; 
      }
      
      for ( row = 0 ; row < neqt ; row++ ) {
	temp[row] = l_x[row];
	for ( col = 0 ; col < neqt ; col++ )  
	  temp[row] += h_x[col*neqt+row]*p[col];	      /* A' times p. */
      }
      for ( row = 0 ; row < neqt ; row++ ) 
	adjoint[i*neqt+row] = p[row] = temp[row];
    }

    if ( traj_constr && start == points-1 ) {
      /* Add in (d/du_i)(phi(x_i,u_i)). */
      zero_out(l_u,ninputs);
      Dl(neq,&t,state+start*neqt,control+(start-1)*ninputs,l_x,l_u);
      for ( row = 0 ; row < ninputs ; row++ )
	dJdu[(start-1)*ninputs+row] += l_u[row];
    }
 
    if ( form == 1 ) i = 2; else i = 0;
    lhs[i] = ans3;				     /* Return dJdu. */
    
    if ( nlhs >= 2+i ) {
      /* Compute gradient w.r.t. initial conditions. */
      ans4 = mxCreateFull(neqt,1,REAL);
      for ( j = 0 ; j < neqt ; j++ )
	mxGetPr(ans4)[j] = J_x0[j]+p[j];
      lhs[1+i] = ans4;				    /* Return dJdx0. */
    }
      
    if (return_p) 
      lhs[2+i] = ans5;				  /* Return adjoint. */
    else
      mxFreeMatrix(ans5);
  }
  
  return;
}
