/******************************************/
/*                                        */
/* Copyright (c) 1996 by Adam L. Schwartz */
/*          All right reserved.           */
/*                                        */
/****************************************************************/
/* Explicit trapezoidal rule. Controls are piecewise linear and */
/* continuous of dimension m x (N+1).                           */
/* Hager's formulas for the adjoint and gradient are used.      */
/* The gradients are modified to reflect the use of             */
/* continuous controls.  So DJ_k = DJ_{k-1,2}+DJ_{k,1}.         */
/****************************************************************/
#include "mex.h"
#include "utility.h"
#define STATIC
#include "system.h"

extern long unsigned int H_calls,DH_calls;

void trapezoid(nlhs, lhs, nrhs, rhs)
     int nlhs, nrhs;
     Matrix *lhs[], *rhs[];  /* lhs is a pointer to the return arguments */
                             /* rhs is a pointer to the input arguments  */
{
  static unsigned int points = 0;
  static int initialized = 0, linear_p;
  static int neq[] = {0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  static int neqt,ninputs,nparams,nfunctions,ntraj_constr,nend_pt_constr;
  static double *state,*adjoint,*dJdu,*control,*mid_control,*Y,*x0;
  static double *x,*xdot,*k2,*Y2store,*p;
  static double *temp,*l_x1,*l_x2,*h_x1,*h_x2,*J_x0;
  static double *l_u1,*l_u2,*h_u1,*h_u2,*q1;
  
  int i, j, index, row, col, action, form, f_num, start, traj_constr;
  int return_x=0,return_j=0,return_p=0;
  int comp_x=0,comp_j=0,comp_du=0,comp_lte=0;
  double t,tout,deltaT,z,zdot1,zdot2,*q0;
  Matrix *u,*interval,*ans1,*ans2,*ans3,*ans4,*ans5;

  if ( nrhs == 0 ) {
    if ( initialized ) {
      free(x); free(xdot); free(k2); free(J_x0);
      free(l_x1); free(l_x2); free(h_x1); free(h_x2);
      free(l_u1); free(l_u2); free(h_u1); free(h_u2); free(q1);
      free(Y2store); free(state); free(x0); free(control); free(Y);
      initialized = 0;
    }
    return;
  }
  
  form = (int) mxGetScalar(rhs[0]);
  if ( form == 1 ) {
    u = rhs[2];
    interval = rhs[3];
    traj_constr = 0;
  
    f_num = neq[4] = 1;				 /* Evaluate first function. */
    comp_x = 1;
    action = (int) mxGetScalar(rhs[5]);
    if (action > 0 ) return_j = 1;
    if (action > 1 ) return_x = 1;
    if (action > 2 ) comp_du  = 1;
    if (action > 3 ) return_p = 1;

    if ( points != mxGetN(interval) ) {
      points = mxGetN(interval);
      if (initialized) {
	free(Y); free(control); free(state); free(Y2store);
	Y2store = (double*)callocate(points-1,sizeof(double)*neqt,18);
	state = (double*)callocate(points,sizeof(double)*neqt,19);
	control = (double*)callocate(points,sizeof(double)*ninputs,20);
	Y     = (double*)mallocate(sizeof(double)*points,21);
      }
    }
    
    if (!initialized) {
      init(neq,NULL);
      neqt=neq[0];
      ninputs=neq[1];
      nparams=neq[2];
      nfunctions=neq[5];
      ntraj_constr = neq[6]+neq[7];
      nend_pt_constr = neq[8]+neq[9]+neq[10]+neq[11];
      linear_p = neq[12];

      x = (double*)mallocate(sizeof(double)*neqt,1);
      p = x;
      xdot = (double*)callocate(neqt,sizeof(double),2);
      k2 = (double*)callocate(neqt,sizeof(double),3);
      J_x0 = (double*)callocate(neqt,sizeof(double),4);
      temp = l_x1 = (double*)callocate(neqt,sizeof(double),4);
      l_x2 = (double*)callocate(neqt,sizeof(double),5);
      h_x1 = (double*)callocate(neqt,sizeof(double)*neqt,6);
      h_x2 = (double*)callocate(neqt,sizeof(double)*neqt,7);
      mid_control = l_u1 = (double*)callocate(ninputs,sizeof(double),8);
      l_u2 = (double*)callocate(ninputs,sizeof(double),9);
      h_u1 = (double*)callocate(neqt,sizeof(double)*ninputs,10);
      h_u2 = (double*)callocate(neqt,sizeof(double)*ninputs,11);
      q1   = (double*)mallocate(sizeof(double)*neqt,12);
      Y2store = (double*)callocate(points-1,sizeof(double)*neqt,13);
      state   = (double*)callocate(points,sizeof(double)*neqt,14);
      x0      = (double*)mallocate(sizeof(double)*(neqt),15);
      control = (double*)callocate(points,sizeof(double)*ninputs,16);
      Y       = (double*)mallocate(sizeof(double)*points,17);
      initialized = 1;
    }
    if ( mxGetM(rhs[1]) != neqt )
      mexErrMsgTxt("x0 wrong length.");
    if ( mxGetN(u) != points || mxGetM(u) != ninputs )
      mexErrMsgTxt("u wrong length.");

    /* Save x0, u, Y for calls later by simulate() form 2. */
    for ( i = 0 ; i < neqt ; i++ )
      x0[i] = mxGetPr(rhs[1])[i];
    for ( i = 0 ; i < points ; i++)
      for ( j = 0 ; j < ninputs ; j++ )
	control[i*ninputs+j] = mxGetPr(u)[i*ninputs+j];
    for ( i = 0 ; i < points ; i++ )
      Y[i] = mxGetPr(interval)[i];
    
  }
  else {
    if ( form == 2 ) {
      /* Form = 2. */
      f_num  = (int) mxGetScalar(rhs[1]);		 /* Function number. */
      action = (int) mxGetScalar(rhs[2]);		 /* Action number.   */
      if (f_num > nfunctions && f_num <= nfunctions + ntraj_constr*points)
	traj_constr = 1;
      else
	traj_constr = 0;
      if (f_num > nfunctions + ntraj_constr*points + nend_pt_constr)
	mexErrMsgTxt("Function number too big.");
      if (f_num < 1) mexErrMsgTxt("Functions number must be > 0.");
      
      switch (action) {
      case 3:
	return_p = 1;
      case 2: 
	comp_du = 1;
	break;
      case 1:
	comp_j = 1;
      }
    }
    else
      comp_lte = 1;
  }
  
  if (comp_x) {
    
    /* Get initial condition on state. */
    for (j = 0 ; j < neqt ; j++)
      x[j] = state[j] = x0[j];
    
    t = Y[0]; z = zdot1 = zdot2 = 0.0;
    for ( i = 0 ; i < points-1 ; i++) {
      tout = Y[i+1];
      deltaT = tout - t;
      neq[3] = i;
      index = i*neqt;
      H_calls += 2;
      h(neq,&t,x,control+i*ninputs,xdot);
      if (return_j) 
	zdot1 = l(neq,&t,x,control+i*ninputs); neq[3] = i;
      for ( j = 0 ; j < neqt ; j++) 
	Y2store[index+j] = x[j] + deltaT*xdot[j];
      t=tout;
      h(neq,&t,Y2store+index,control+(i+1)*ninputs,k2);
      if (return_j) {
	zdot2 = l(neq,&t,Y2store+index,control+(i+1)*ninputs);
	z += deltaT*(zdot1 + zdot2)/2;
      }
      for ( j = 0 ; j < neqt ; j++)
	state[(i+1)*neqt+j] = x[j] += deltaT*(xdot[j]+k2[j])/2;
    }
    if (return_j) {
      ans1 = mxCreateFull(1,1,REAL);
      lhs[0] = ans1;
      *mxGetPr(ans1) = g(neq,&t,x0,x) + z;
    }
    if (return_x) {
      ans2 = mxCreateFull(neqt,points,REAL);
      lhs[1] = ans2;
      index = 0;
      for ( i = 0 ; i < points ; i++ ) 
	for ( j = 0 ; j < neqt ; j++ ) {
	  mxGetPr(ans2)[index] = state[index];
	  index++;
	}
    }
  }

  /* Compute estimates of local truncation error by starting from each */
  /* compute x_k, taking two half sized steps and comparing result     */
  /* with x_k+1.                                                       */
  if ( comp_lte ) {
    ans1 = mxCreateFull(neqt+1,points-1,REAL);
    lhs[0] = ans1;
    
    zdot1 = zdot2 = 0.0;
    for ( i = 0 ; i < points-1 ; i++) {
      neq[3] = i;
      index = i*neqt;
      t = Y[i]; tout = Y[i+1];
      deltaT = (tout - t)/2;                  /* Half of the interval. */

      for ( j = 0 ; j < neqt ; j++ )
	x[j] = state[index+j];

      /* Take first half step. */
      H_calls += 4;
      h(neq,&t,x,control+i*ninputs,xdot);
      zdot1 = l(neq,&t,x,control+i*ninputs); neq[3] = i;
      mxGetPr(ans1)[i*(neqt+1)+neqt] = deltaT*zdot1;
      for ( j = 0 ; j < neqt ; j++ ) 
	temp[j] = x[j] + deltaT*xdot[j];
      t+=deltaT;
      for ( j = 0 ; j < ninputs ; j++ )
	mid_control[j] = (control[i*ninputs+j] + control[(i+1)*ninputs+j])/2;
      h(neq,&t,temp,mid_control,k2);
      zdot2 = l(neq,&t,temp,mid_control); neq[3] = i;
      for ( j = 0 ; j < neqt ; j++)
	x[j] += deltaT*(xdot[j]+k2[j])/2;
      z = deltaT*(zdot1+zdot2)/2;

      /* Take second half step. */
      h(neq,&t,x,mid_control,xdot);
      zdot1 = l(neq,&t,x,mid_control); neq[3] = i;
      for ( j = 0 ; j < neqt ; j++ ) 
	temp[j] = x[j] + deltaT*xdot[j];
      t=tout;
      h(neq,&t,temp,control+(i+1)*ninputs,k2);
      zdot2 = l(neq,&t,temp,control+(i+1)*ninputs); neq[3] = i;
      z += deltaT*(zdot1+zdot2)/2;
      for ( j = 0 ; j < neqt ; j++ ) {
	x[j] += deltaT*(xdot[j]+k2[j])/2;
	mxGetPr(ans1)[i*(neqt+1)+j] = state[index+neqt+j] - x[j];
      }
      zdot2 = l(neq,&t,Y2store+index,control+(i+1)*ninputs);
      mxGetPr(ans1)[i*(neqt+1)+neqt] += deltaT*zdot2 - z;
    }
  }
  
  if ( comp_j ) {
    ans1 = mxCreateFull(1,1,REAL);
    lhs[0] = ans1;
    if ( f_num <= nfunctions ) {
      neq[4] = f_num;
      t = Y[0]; z = zdot1 = zdot2 = 0.0;
      for ( i = 0 ; i < points-1 ; i++) {
	tout = Y[i+1];
	neq[3] = i;
	deltaT = tout-t;
	zdot1 = l(neq,&t,state+i*neqt,control+i*ninputs);
	if (neq[3] == -1) break;				   /* l == 0 */
	t = tout;
	zdot2 = l(neq,&t,Y2store+i*neqt,control+(i+1)*ninputs);
	z += deltaT*(zdot1 + zdot2)/2;
      }
      *mxGetPr(ans1) = g(neq,&t,x0,state+(points-1)*neqt) + z;
    }
    else {
      if (!traj_constr) {
	/* This must be an endpoint constraint. */
	neq[4] = f_num - ntraj_constr*points;
	*mxGetPr(ans1) = g(neq,Y+points-1,x0,state+(points-1)*neqt);
      }
      else {
	/* This is a trajectory constraint. */
	/* neq[4] = the trajectory constraint number. */
	neq[4] = (f_num - nfunctions - 1)/points + nfunctions + 1; 
	neq[3] = i = f_num - nfunctions - (neq[4]-1-nfunctions)*points - 1;
	mxGetPr(ans1)[0] = l(neq,Y+i,state+i*neqt,control+i*ninputs);
      }
    }
  }
 
  if (comp_du) {
    ans3 = mxCreateFull(ninputs,points,REAL);
    ans5 = mxCreateFull(neqt,points,REAL);
    dJdu = mxGetPr(ans3);
    adjoint = mxGetPr(ans5);

    zero_out(J_x0,neqt);
    if (!traj_constr) {
      if (f_num <= nfunctions)
	neq[4] = f_num;
      else
	neq[4] = f_num - ntraj_constr*points;
      neq[3] = start = points - 1;
      t = Y[start];
      zero_out(p,neqt);
      Dg(neq,&t,x0,state+start*neqt,J_x0,p);		       /* Get p(Tf). */
/*      zero_out(dJdu+start*ninputs,ninputs); */
    }
    else { 
      neq[4] = (f_num - nfunctions - 1)/points + nfunctions + 1;
      start = f_num - nfunctions - (neq[4]-1-nfunctions)*points - 1; 
      /* Adjoint and gradient are zero for i > start. */
      for ( i = points-1 ; i > start ; i-- ) {
	for ( j = 0 ; j < neqt ; j++ )
	  adjoint[i*neqt+j] = 0.0;
/*
	for ( j = 0 ; j < ninputs ; j++ )
	  dJdu[i*ninputs+j] = 0.0;
*/
      }
      /* Get p(Tf).  */
      t = Y[start]; 
      neq[3] = i = start;
      zero_out(p,neqt); 
/*    zero_out(dJdu+i*ninputs,ninputs); */
      /* Add in (d/du_i)(phi(x_i,u_i)). */
      Dl(neq,&t,state+start*neqt,control+start*ninputs,p,dJdu+i*ninputs);
    }
    
    for ( j = 0 ; j < neqt ; j++)
      adjoint[start*neqt+j] = p[j];

    zero_out(l_x1,neqt); zero_out(l_u1,ninputs);
    zero_out(l_x2,neqt); zero_out(l_u2,ninputs);

    if ( linear_p ) {
      DH_calls += 2;
      Dh(neq,&t,state,control,h_x1,h_u1);
      Dh(neq,&t,Y2store,control,h_x2,h_u2);
    }

    for ( i = start-1 ; i >= 0 ; i-- ) {
      neq[3] = i;
      t=Y[i+1];
      tout = Y[i];
      deltaT = t-tout;
       
      if (!linear_p) {
	DH_calls += 2;
	Dh(neq,&t,Y2store+i*neqt,control+(i+1)*ninputs,h_x2,h_u2);
	t = tout;
	Dh(neq,&t,state+i*neqt,control+i*ninputs,h_x1,h_u1);
      }
      if ( f_num <= nfunctions ) {
	t = Y[i+1];
	Dl(neq,&t,Y2store+i*neqt,control+(i+1)*ninputs,l_x2,l_u2);
	t = tout;
	Dl(neq,&t,state+i*neqt,control+i*ninputs,l_x1,l_u1);
      }

      for ( row = 0 ; row < neqt ; row++ ) {
	q1[row] = p[row] + deltaT*l_x2[row];
	for ( col = 0 ; col < neqt ; col++ )
	  q1[row] += deltaT*h_x2[col*neqt+row]*p[col];
      }

      for ( row = 0 ; row < ninputs ; row++ ) {
	dJdu[(i+1)*ninputs+row] += l_u2[row]*deltaT/2;
	dJdu[i*ninputs+row]      = l_u1[row]*deltaT/2;
	for ( col = 0 ; col < neqt ; col++ ) {
	  dJdu[(i+1)*ninputs+row] += h_u2[col*ninputs+row]*p[col]*deltaT/2;
	  dJdu[i*ninputs+row]     += h_u1[col*ninputs+row]*q1[col]*deltaT/2;
	}
      }

      /* Compute next adjoint variable. */
      q0 = adjoint + i*neqt;
      for ( row = 0 ; row < neqt ; row++ ) {
	q0[row] = p[row] + (l_x1[row] + l_x2[row])*deltaT/2;
	for ( col = 0 ; col < neqt ; col ++ )
	  q0[row] += (h_x1[col*neqt+row]*q1[col]
		       + h_x2[col*neqt+row]*p[col])*deltaT/2;
      }
      for ( row = 0 ; row < neqt ; row++ )
	p[row] = q0[row];
    }

    if ( form == 1 ) i = 2; else i = 0;
    lhs[i] = ans3;

    if ( nlhs >= 2+i ) {
      /* Compute gradient w.r.t. initial conditions. */
      ans4 = mxCreateFull(neqt,1,REAL);
      for ( j = 0 ; j < neqt ; j++ )
	mxGetPr(ans4)[j] = J_x0[j] + p[j];		     /* Add in p(0). */
      lhs[1+i] = ans4;
    }
      
    if (return_p) 
      lhs[2+i] = ans5;
    else
      mxFreeMatrix(ans5);
  }

  return;
}
