function [new_t,u,x,J,G,E]=...
         outer(x0,u0,t,Umin,Umax,params,N_inner,N_outer,ialg,tol,strategy,Disp)

% [new_t,u,x,J,G,E]=
% outer(x0,u0,t,Umin,Umax,params,N_inner,N_outer,ialg,{tol},{strategy},{Disp})
%
% INPUTS:
% =======
% x0,u0,t,Umin,Umax,params --- The usual,
%                         (Umin and Umax must be ninputs x 1 column vectors).
% N_inner  --- Maximum number of iterations allowed on each call to riots().
% N_outer  --- Maximum number of outer loops.  If N_outer has two components,
%              the 2nd component is used as the maximum discretization level.
% ialg     --- Integration algorithm.
% tol      --- tol = [tol1 tol2]
%              Optimality tolerangce: || grad L || sub H < tol1*(1+abs(J)),
%                                     sum of constraint violations < tol2,
%				      control error < tol3*(1+max(abs(u)))*T.
%              Default: tol1 = eps^(1/3), tol2 = eps^(1/4), tol3 = eps^(1/6).
% strategy --- Mesh redistribution strategy (see distribute.m).  Default=3.
% Disp     --- 0: No output from riots() [default]
%              1: Normal output from riots().
%
% OUTPUTS:
% ========
% new_t --- Final vector of time breakpoints.
% u     --- Final control solution.
% x     --- Final trajectory solution.
% J(i)  --- Value of the objective function after outer iteration i.
% G(i)  --- Sum of constraint violations after outer iteration i.
% E(i)  --- Estimate of the control error after outer iteration i+1.
%
% x, J and G are computed using a variable stepsize integration routine to
% minimize effect of discretization errors.
%
% At the end of every information, the norm of the gradient of the
% Lagrangian in the unconstrained subspace is display along with 
% G(i), J(i) and E(i).
%
% After each outer iteration, the control solution is plotted.  However,
% for free final time problems, the time axis will not be scaled properly.

% It would be easy to modify this probram to accept m-functions Umin(t) 
% and Umax(t) in order to specify non-constant control bounds.
%
% If you want to add a penalty on the control derivative variation, set
% VAR below to a small positive value (see riots documentation).

%
% Copyright (c) 1996 by Adam L. Schwartz
% All right reserved.  
%

if nargin < 9
  disp(['Usage: [new_t,u,x,J,G,E]=outer([x0,{fixed,{x0min,x0max}}],u0,t,'...
        'Umin,Umax,params,'])
  disp(['                               '...
        'N_inner,[N_outer,{max_N}],ialg,{[tol1,tol2,tol3]},{strategy},{Disp})'])
  return
end
if nargin < 12, Disp = 0; end
if nargin < 11, strategy = 3; end 
if nargin < 10, tol = []; end
if length(Umin) > 1 | length(Umax) > 1,
  error('Umin and Umix should be scalars.')
end
if strategy == [], strategy = 3; end
if tol == [], tol = [eps^(1/3) eps^(1/4) eps^(1/6)]; end
if length(tol) == 1, tol = [tol tol tol]; end

FAC = 10;   % Amount to increase integration accuracy on each iteration.
VAR = 0;    % Penalty on the total variation of the control derivative.
Tf  = 1;    % For free final time problems, Tf should be made equal to
            % the free final time scaling after each call to riots().

ct = cputime;
N = length(t)-1;
K = 2 + length(u0) - length(t);
miter=[N_inner,VAR];
if size(x0,2) > 1
  fixed_x0 = x0(:,2);
  n_free_x0 = sum(~fixed_x0);
else
  fixed_x0 = [];
  n_free_x0 = 0;
end
if length(N_outer) > 1
 max_N = N_outer(2); 
 N_outer = N_outer(1); 
else
 max_N = inf;
end

J = zeros(1,N_outer);
G = zeros(1,N_outer);
E = zeros(1,N_outer-1);
riots_TOL = [sqrt(eps),eps^(1/3),0,1e6]; 

if VAR ~= 0
  fprintf(['Adding a penalty of %g times the variation ' ...
           'of the control derivative.\n'],VAR)
end

[u,x,f,g,lambda]=riots(x0,u0,t,Umin,Umax,params,miter,ialg,riots_TOL,0,Disp);
x0(:,1) = x(:,1);

sp_plot(t,u)
title('Control solution after first outer iteration.')
drawnow

[ie,ze,ff,gg,xx] = est_errors(x0,u,t,Tf,ialg,lambda);
global_ie = sum(ie);
J(1) = ff; G(1) = gg;
global_ie_prev = global_ie; ze_prev = ze; t_prev = t;
x0_prev = x0; u_prev  = u;

[new_t,u0]=distribute(t,u,x,ialg,[],n_free_x0,3);

disp('========Completed  1  OUTER  iteration=========')
fprintf('Norm of Lagrangian gradient = %g\n',ze)
fprintf('Sum of constraint errors    = %g\n',gg)
fprintf('Objective function value    = %g\n',ff)
fprintf('Integration error           = %g\n',global_ie)
fprintf('Integration algorithm       = %d\n',ialg)
disp('===============================================')

for iter=2:N_outer
  miter(2) = VAR*abs(f);
  riots_TOL(1) = ze/100;
  riots_TOL(2) = global_ie;

  if VAR ~= 0
    fprintf(['Adding a penalty of %g times the variation ' ...
             'of the control derivative.\n'],VAR)
  end

  [u,x,f,g,lambda]=...
        riots(x0,u0,new_t,Umin,Umax,params,miter,ialg,riots_TOL,0,Disp);
  x0(:,1) = x(:,1);

  sp_plot(new_t,u)
  S = sprintf('Control solution after outer iteration %d.',iter);
  title(S)
  drawnow

  [ie,ze,ff,gg,xx,M]=est_errors(x0,u,new_t,Tf,ialg,lambda);
  global_ie = sum(ie);
  J(iter) = f;
  G(iter) = gg;

  [E(iter-1),diff_z] = ...
                control_error(x0_prev,u_prev,t_prev,ze_prev,x0,u,new_t,ze,Tf);

  fprintf('\n========Completed %3d OUTER  iterations========\n',iter)
  fprintf('Norm of Lagrangian gradient = %g\n',ze)
  fprintf('Sum of constraint errors    = %g\n',gg)
  fprintf('Objective function value    = %g\n',ff)
  fprintf('Integration error           = %g\n',global_ie)
%  fprintf('Integration algorithm       = %d\n',ialg)
  fprintf('Change in solutions         = %g\n',diff_z)
  fprintf('Control error estimate      = %g\n',E(iter-1))
  disp('===============================================')

  terminate = ( ze <= tol(1)*(1+abs(ff))  &  gg <= tol(2) & ...
                 E(iter-1) < tol(3)*(1+max(abs(u)))*t(length(t)) );
  if terminate
    disp('Normal Termination.')
    break
  end

  if iter < N_outer

  % I commented out the following integration order adjustment scheme
  % because RK3 seems to freak out for problems with singular arcs.
  % Otherwise, this seemed to work well.

%    if global_ie < E(iter-1)/1000 & K == 2 & ( ialg > 2 & ialg < 5 )
%      ialg = ialg-1;
%      fprintf('Reducing integration order to ialg = %d.\n',ialg)
%    elseif global_ie > E(iter-1)/10 & ialg < 4
%      ialg = ialg+1;
%      fprintf('Increasing integration order to ialg = %d.\n',ialg)
%    end

    global_ie_prev = global_ie; 
    ze_prev = ze; t_prev = new_t; x0_prev = x0; u_prev = u;
    [new_t,u0]=distribute(new_t,u,x,ialg,[],n_free_x0,strategy,FAC);

  else
    disp('Reached maximum number of outer iterations.')
  end

  if length(new_t)-1 > max_N,
    new_t = t_prev;
    disp('Surpased the maximum discretization level.')
    break;
  end

end

J = J(1:iter); G = G(1:iter); E = E(1:iter-1);
x = xx;

fprintf('CPU time = %g seconds.\n',cputime-ct)
