function [u,x,J,inform,I_a,I_i,MM] = ...
	 pdmin(x0,u0,t,Umin,Umax,params,miter,ialg,method,k,Disp)

% [u,x,f,inform,I_a,I_i,M] = 
%                pdmin([x0,{fixed,{x0min,x0max}}],u0,t,{Umin},{Umax},{params},
%                      [miter;TOL],ialg,{method},{params},{[k;{scale}]},{Disp})
%
% 'pdmin' stands for Projected Descent Minimization.
%
% INPUTS:
%  x0 --- initial state column vector
%  fixed --- if fixed is specified, those initials states with a corresponding
%	     zero in the vector fixed, will be free to change.
%  x0min --- Lower bound on any x0 variables that are not fixed.
%  x0max --- Upper bound on any x0 variables that are not fixed.
%  u0 --- initial control guess 
%  t  --- time discretization column vector
%  Umin --- matrix of same size as u0, specifying lower bounds.
%  Umax --- matrix of same size as u0, specifying upper bounds.
%           Alternatively, Umin and Umax can be column vectors of length
%           ninputs specifying bounds that are constants with respect to time.
%           If u0 is outside of the bounds specified by Umin and Umax,
%           it is projected into those bounds.
%           Use [] to specify no bounds.
%  params --- are fixed system parameters that are passed to simulate, 
%             use [] if no parameters.
%  miter --- is number of iterations.  
%  TOL --- (optional, default = sqrt(eps)) 
%          If specified, use TOL as stopping criterion.  The default value
%          of sqrt(eps) is about as small as TOL can be set an still achieve
%          a normal termination.
%  ialg --- is the integration algorithm used in calls to simulate().
%  k --- If specified, use beta^k times a factor as initial perturbation
%        for scale calculation.
%        Use [] if you don't want to specify k.
%  scale --- (default is 1).  The perturbation is scale*beta^k.
%  Disp --- If 0 no print output; if 1 minimal printout; if 2 full print out.
%           (default is 2).  For full display, need about 117 columns.
%
%  method is an optional string equal to one of the following:
%         {'steepest','vm','conjgr'}
%          'steepest' --- steepest descent
%          'vm'       --- L-BFGS (limited-memory variable metric)
%          'conjgr'   --- Polak-Ribiere conjugate gradient
%          The default method is 'vm'.
%
% OUTPUTS:
%  u      --- solution for optimal control.
%  J      --- Row vector of costs at each iteration.
%  x      --- optimal trajectory.
%  inform --- [terminate_reason,norm_grad,k,scaling]  where 
%	       terminate_reason is a number corresponding to the reason
%	       for termination:
%                 -1 -- Integration produced NaN or Inf.
%		   0 -- Normal termination test satisfied.
%	           1 -- Completed maximum number of iterations.
%		   2 -- Direction vector too small.
%		   3 -- All variables at bounds and going to stay that way.
%		   4 -- Gradient too small.
%		   5 -- Step size too small.
%		   6 -- User test satisfied (user test returned 2).
%  Generally, terminate_reason < 0 means bad termination.
%             norm_grad is the norm of the unscaled, unconstrained gradient
%                       in function space.
%             k is the final step-size (beta^k) from the armijo test.
%             scaling is the current scaling.
%             iter is the iteration in which termination occured.
%  Note: if terminate_reason == 5, then the value of k should be ignored.
%
%  I_a     --- index of active bound constraints
%  I_i     --- index of inactive bound constraints.
%  M       --- Coordinate transformation.
%
%  If the global variable "USER_FUNCTION_NAME" is defined,
%  pdmin() will call the m-file function specified by that variable
%  to evaluate the system functions and gradients.  This can be used to
%  construct a composite function from several functions.  For instance,
%  an exact penalty function can be formed thus making a constrained
%  problem look like an unconstrained problem.
%  The syntax for the user function should be
%
%    [f0,x,grad_u,grad_x0] = USER_FUNCTION_NAME(x0,u,t,ialg,action)
%
%  If the global variable "USER_TEST_NAME" is defined,
%  pdmin() will call the m-file function specified by that variable
%  at the end of each iteration. 
%  The user test should have the syntax:
%
%    user_terminate = USER_TEST_NAME(f0,x,u,grad_u,grad_x0,I_i,free_x0)
%
%    I_i is a column vector indexing all elements of [u(:) ; x0(free_x0)]
%    that are not actively constrained.
%
%  If user_terminate = 1, and the other termination conditions are
%  satisfied, then pdmin.m will terminate.  If user_terminate = 2,
%  then pdmin.m will terminate without regard to the other termination
%  tests.
%  Note that this function can be used to just do some operations
%  at the end of each iteration by always returning 1.
%
%  It's a good idea to 'clear global' any user-defined global variables
%  after the program calling pdmin.m finishes.
%
%  Don't use diagonal inverse Hessian scaling, it's a waste of time.

%
% Copyright (c) 1996 by Adam L. Schwartz
% All right reserved.  
%

global COUNT USER_TEST_NAME USER_FUNCTION_NAME Minv DISP
% The globals below are use by armijo.
global ALPHA BETA GOLDSTEIN_TEST User_function_exist User_armijo_test_exist

TRANSFORM = 1;   % Set TRANSFORM to 0 to disable coordinate transformation.

if nargin < 11, DISP = 2; else DISP = Disp; end
if nargin < 10, k = []; end
if nargin < 9,  method = 'vm'; end
if nargin < 8
   disp('USAGE: ')
   disp(['[u,x,f,inform,I_a,I_i,M] = pdmin([x0,{fixed,{x0min,x0max}}]'...
         ',u0,t,Umin,Umax,params,'])
   disp(['                                 '...
         '[miter,{tol}],ialg,{method},{[k;{scale}]},{Disp})']),
   return,
end
if DISP == []; DISP = 2; end
if method == []; method = 'vm'; end

steepest = 0; vm = 0; conjgr = 0;
if strcmp(method,''), method = 'vm'; end
if strcmp(method,'steepest'),  steepest  = 1; end
if strcmp(method,'vm'),        vm        = 1; end
if strcmp(method,'conjgr'),    conjgr    = 1; end
if ~(steepest | vm | conjgr),
  error('No such method.')
end

ALPHA = 1/2;
BETA = 3/5;
KL = -40;                 % Maximum stepsize = BETA^KL
KU = 40;                  % Minimum stepsize = BETA^KU

GOLDSTEIN_TEST = 0;       % Used by armijo.m
FIT = 0; % Best not to FIT with steepest descent, otherwise k is likely
         % to change from one iteration to the next.
if conjgr
  FIT = 1;
end
if vm
  clear LBFGS             % Reset L-BFGS routine.
  ALPHA = 1/3;
  GOLDSTEIN_TEST = 0;     % This allows k=0 to be acepted with out extra 
end                       % function evaluations.  Armijo wont perform a 
                          % quadratic fit if GOLDSTEIN_TEST = 1.
if length(miter)==1
  TOL = sqrt(eps); 
else
  TOL = miter(2);
  miter = miter(1);
end
dT = t(2)-t(1);
N = length(t)-1;         % discretization level.
n = size(x0,1);          % number of states.
m = size(u0,1);          % number of inputs.

length_u = size(u0,2);
ncontrols = m*length_u;

%-------------- Do some input argument checking. ----------------

info = simulate(0,params);
nstates = info(1); ninputs = info(2); nparams = info(3);
if n ~= info(1)
  error(['x0 should be length ' num2str(info(1)) '.'])
end
if m ~= info(2)
  error(['Control input must have ' num2str(info(2)) ' rows.'])
end
% Do a rough check on the length of the control input.
if length(u0) < length(t)-1 | length(u0) > length(t)+8
  error(['Control input is wrong length.'])
end

if size(params,1)*size(params,2) ~= info(3)
  error(['Need ' num2str(info(3)) ' system parameters.'])
end
if size(t,1) > 1, error('t must be a row vector'), end
if DISP > 0, simulate(0); end

%-------------- Figure what bounds are being used. ---------------

if size(x0,2) > 1,
  x_bounds = 1;
  fixed_x0 = find(x0(:,2) ~= 0);
  free_x0 = find(x0(:,2) == 0); 
  n_free_x0 = length(free_x0);
  if size(x0,2) > 2,
    if size(x0,3)==3
       error(['If you specify bounds for x0, you must specify upper '...
              'and lower bounds.'])
    end
    x0min = x0(free_x0,3); 
    x0max = x0(free_x0,4); 
    x0 = x0(:,1);
    if x0min ~= [] & x0max ~= []
      x0 = max(min(x0,x0max),x0min);
      margin_x0 = max((x0max-x0min)/100,.2);
    elseif x0min ~= []
      x0 = max(x0,x0min);
      margin_x0 = max((x0-x0min)/100,.2);
    elseif x0max ~= []
      x0 = min(x0,x0max);
      margin_x0 = max((x0max-x0)/100,.2);
    end
  else
    x0 = x0(:,1);
    x0min = -inf*ones(size(free_x0));
    x0max = inf*ones(size(free_x0));
    margin_x0 = ones(n_free_x0,1);
  end
else
  x_bounds = 0;
  fixed_x0  = 1:n; free_x0 = [];
  n_free_x0 = 0;
end
n_fixed_x0 = n-n_free_x0;

% Expand out Umin and Umax if necessary.
if size(Umin,2) == 1
  Umin = Umin(:,ones(1,length_u));
end
if size(Umax,2) == 1
  Umax = Umax(:,ones(1,length_u));
end

if Umin == [] & Umax == []
  u_bounds = 0;
  margin_u = zeros(size(u));
else
  u_bounds = 1;
  % Project u0 into bounds.
  if Umin ~= [] & Umax ~= []
    u0 = max(min(u0,Umax),Umin);
    margin_u = max((Umax-Umin)/100,.2);
  elseif Umin ~= []
    u0 = max(u0,Umin);
    margin_u = max((u0-Umin)/100,.2);
  elseif Umax ~= []
    u0 = min(u0,Umax);
    margin_u = max((Umax-u0)/100,.2);
  end
end
bounds = u_bounds | x_bounds;

%----------------C O O R D I N A T E   T R A N S F O R M----------------

if TRANSFORM
  representation = 1;
  order = length_u - length(t) + 2;
  if order == 1 
    representation = 2;
    MM=diff(t)';
  elseif order == 2
    if u_bounds | ialg == 2
      representation = 2; 
    else 
      representation = 1;
    end
    if representation == 1
      MM = transform(t,order);
    else
      DT = diff(t);
      MM = (([DT 0] + [0 DT])/2)';
    end
  else
    representation = 1;
    MM = transform(t,order);
  end
else
   representation = 2;
   if order == 1
     MM = diff(t);
   elseif order == 2
     DT = diff(t);
     MM = (([DT 0] + [0 DT])/2)';
   else
     MM = ones(length_u,1);
   end
end

if representation == 1,
  [U,D] = schur(full(MM));          % This takes a while if N is big.
  rt_D = sqrt(diag(D));
  M = U*diag(rt_D)*U';
  Minv = U*diag(ones(size(rt_D))./rt_D)*U';
end
if representation == 2,
  % Create sparse transformation matrix.
  M    = spdiags(sqrt(MM),0,length_u,length_u);
  Minv = spdiags(ones(length_u,1)./sqrt(MM),0,length_u,length_u);
  MM   = spdiags(MM,0,length_u,length_u);
end

% Transform coordinates.
u = u0*M;
margin_u = margin_u*M;

%----------------PERFORM INITIAL FUNCTION EVALUATIONS-------------------

% Do the following checks only once, they're horrendously slow.
if exist_in_workspace('USER_FUNCTION_NAME') == 1, User_function_exist = 1; end
if exist_in_workspace('USER_TEST_NAME') == 1, User_test_exist = 1; end
if exist_in_workspace('USER_ARMIJO_TEST_NAME') == 1
  User_armijo_test_exist = 1; 
end

if bounds,
  if Umin == [], Umin = -inf*ones(size(u0)); else Umin = Umin*M; end
  if Umax == [], Umax =  inf*ones(size(u0)); else Umax = Umax*M; end
  if x0min == [], x0min = -inf*ones(size(free_x0)); end
  if x0max == [], x0max = inf*ones(size(free_x0)); end
  BL = [Umin(:) ; x0min];
  BU = [Umax(:) ; x0max];
  margin = [margin_u(:) ; margin_x0];
end

if User_function_exist
  [f0,x,grad_u,grad_x0] = feval(USER_FUNCTION_NAME,x0,u*Minv,t,ialg,4);
else
  [f0,x,grad_u,grad_x0] = simulate(1,x0,u*Minv,t,ialg,3);
end
if isnan(f0) | isinf(f0),
  error('Given x0,u0 cause failure in integration algorithm');
end
J = ones(1,miter+1); J(1) = f0;
grad_u = grad_u*Minv;
grad_x0(fixed_x0) = zeros(n_fixed_x0,1);
grad = [grad_u(:) ; grad_x0(free_x0)];
z = [u(:);x0(free_x0)];

%-------------C O M P U T E   I N I T I A L   S C A L I N G-------------

if k == [], k=0; end
if length(k) == 1, 
  lambda = BETA^k;
  Scale = lambda*(1+max(abs(u)))/(0.01+max(abs(grad_u)))/100;
  k = 0;   % The Scale factor will incorporate the stepsize for k <> 0.

  x0pert = -Scale*grad_x0;
  upert  = -Scale*grad_u;
  pert   = [upert(:) ; x0pert(free_x0)];
  if bounds
    [dz,I_a,I_i] = project(z,pert,BL,BU,margin,Scale*grad);
  else
    I_i = [1:ncontrols+n_free_x0]';
    I_a = [];
    dz = [upert(:) ; x0pert(free_x0)];
  end
  gamma=gestimate(f0,x0,u,dz,grad,t,ialg,I_i,free_x0);
  if isnan(gamma), gamma = 10; end
  Scale = 1/max(gamma,eps^(1/4));
  if DISP, fprintf('Initial Scale factor = %6.4g\n',Scale), end
elseif length(k) == 2,
  Scale = k(2); k = k(1);
  if bounds
    [dummy,I_a,I_i] = project(z,[],BL,BU,margin,Scale*grad);
  else
    I_i = [1:ncontrols+n_free_x0]';
    I_a = [];
    dz = [upert(:) ; x0pert(free_x0)];
  end
else
  error('k must be a vector of length 1 or 2.')  
end

%---------------------------- M A I N   L O O P -------------------------

if vm, method = 'L-BFGS'; end
if DISP==2, fprintf('Method = %s.\n',method), end
if GOLDSTEIN_TEST & DISP==2, fprintf('Goldstein test turned on.\n'), end

I_changed = 0;
COUNT = 0;                  % Used by vm and conjgr.
if DISP==2
  if FIT, 
    disp('Quadratic fitting on.'), 
  else 
    disp('Quadratic fitting off.'), 
  end
  %  disp(['representation = ' num2str(representation)])
end

terminate_reason = 1;
for iter = 1:miter,

  temp = grad_u*Minv;
  if max(abs([temp(:);grad_x0(free_x0)])) < eps^(3/4)
    terminate_reason = 4;
    break
  end

  %-----------C O M P U T E   S E A R C H   D I R E C T I O N----------

  grad = [grad_u(:) ; grad_x0(free_x0)];
  z = [u(:) ; x0(free_x0)];
  if bounds
    old_I_i = I_i;
    [dummy,I_a,I_i] = project(z,[],BL,BU,margin,Scale*grad);
    cond_4 = max(min([z(I_a)-BL(I_a);BU(I_a)-z(I_a)])) == 0;
    if length(I_i) == 0 & cond_4;
      terminate_reason = 3;
      break
    end
    if length(old_I_i) ~= length(I_i)
      I_changed = 1;
    elseif max(abs(I_i - old_I_i)) ~= 0
      I_changed = 1;
    else
      I_changed = 0;
    end   
  end

  drct_u = -Scale*grad_u;
  drct_x0 = -Scale*grad_x0;

  if steepest, COUNT = COUNT + 1; 
  % The following search direction subroutine calls are made with a 
  % generic column vectors for the decision variables and gradients.
  elseif conjgr
    cj_drct = conj_grad([grad_u(:);grad_x0(free_x0)],Scale,I_i);
    drct_u = reshape(cj_drct(1:ncontrols),m,length_u);
    drct_x0(free_x0) = cj_drct(ncontrols+1:ncontrols+n_free_x0);
  elseif vm, 
    [vm_drct,COUNT] = LBFGS(z,[grad_u(:);grad_x0(free_x0)],Scale,I_i,DISP); 
    drct_u = reshape(vm_drct(1:ncontrols),m,length_u);
    drct_x0(free_x0) = vm_drct(ncontrols+1:ncontrols+n_free_x0);
  end

  drct = [drct_u(:) ; drct_x0(free_x0)];
  drct_size = norm(drct(I_i));
  z_size = norm(z(I_i));
  if drct_size < eps^(1/2)*(1 + z_size)/100
    terminate_reason = 2;
    break
  end

  %----------------P E R F O R M   L I N E   S E A R C H----------------

  f_old = f0; x_old = x;
  % Armijo will set x = [], if its last simulation was not at the 
  % chosen stepsize.  If x ~= [], then no need to resimulate.
  [lambda,k,f0,znew,x] = armijo(x0,u,t,drct_u,drct_x0,grad,[k,KL,KU],ialg,...
                                I_a,I_i,BL,BU,FIT,free_x0,f0);
  if ( k >= KU )
    terminate_reason = 5;
    break
  end
  uold = u;x0_old=x0;
  u = reshape(znew(1:ncontrols),m,length_u);
  x0(free_x0) = znew(ncontrols+1:ncontrols+n_free_x0);

  %-----------------R E S I M U L A T E   S Y S T E M------------------

  if User_function_exist
    [f0,x,grad_u,grad_x0] = feval(USER_FUNCTION_NAME,x0,u*Minv,t,ialg,4);
  else
    if x == []
      [f0,x,grad_u,grad_x0] = simulate(1,x0,u*Minv,t,ialg,3);
    else
      [grad_u,grad_x0] = simulate(2,1,2);
    end
  end
  if isnan(f0) | isinf(f0),
    terminate_reason = -1;	
    u = uold; x0 = x0_old; x = x_old;
    break;
  end
  J(iter+1) = f0;

  grad_u = grad_u*Minv;
  grad_x0(fixed_x0) = zeros(n_fixed_x0,1);

  %------------------T E R M I N A T I O N   T E S T-------------------
  
  if User_test_exist
    user_terminate = feval(USER_TEST_NAME,f0,x,u,grad_u,grad_x0,I_i,free_x0);
    if user_terminate == 2
      terminate_reason = 6;
      break
    end
  else	
    user_terminate = 1;
  end

  % NOTE: The expirements in "A Family of Projected Descent Methods for
  %       "Optimization Problems with Simple Bounds" were run with the 
  %       antecedent of the if statement below set to 1.
  %       the computation following the else statement is the correct
  %       way to compute the function space norm of the gradient.
  if 0,
    temp = grad_u*Minv;  
    temp = [temp(:) ; grad_x0(free_x0)];
    norm_grad = Scale*norm(temp(I_i))/length(I_i);
  else
    temp = [grad_u(:) ; grad_x0(free_x0) ];
    norm_grad = Scale*sqrt(temp(I_i)'*temp(I_i));
  end

  if bounds, term_str = 'FFFF'; else term_str = 'FFF'; end
  cond_1 = norm_grad < TOL^(4/3)*(1+abs(f0)) | length(I_i) == 0;
  if cond_1, term_str(1)='T'; end
  cond_2 = f_old - f0 < TOL^2*(1+abs(f0))*100;
  if cond_2, term_str(2)='T'; end
  cond_3 = norm(uold-u,'inf') < TOL*(1+norm(u,'inf'));
  if cond_3, term_str(3)='T'; end
  if I_a ~= []
    cond_4 = max(min([z(I_a)-BL(I_a);BU(I_a)-z(I_a)])) == 0;
    if cond_4, term_str(4) = 'T'; end
  else
    cond_4 = 1;
  end
  terminate = cond_1 & cond_2 & cond_3 & cond_4 & user_terminate;

  if DISP
    if iter == 1
      fprintf('Completed 1    pdmin iter ;')
    else
      fprintf('Completed %-4d pdmin iters;',iter)
    end
  end
  if DISP==2 
    fprintf(' step = %+.2e (k=%+3d),',lambda,k)
    fprintf(' ||free_grad|| = %8.2e, %s, cost = %-17.16g',...
	    norm_grad,term_str,f0)
    if I_changed, fprintf(' *\n'), else fprintf('\n'), end
  elseif DISP==1
    fprintf(' (k=%+3d),',k)
    fprintf(' ||free_grad|| = %8.2e, %s',norm_grad,term_str)
    if I_changed, fprintf(' *\n'), else fprintf('\n'), end
  end

  if terminate, 
    terminate_reason = 0; 
    break
  end

end

%------- T R A N S F O R M   B A C K  T O  O R I G I N A L ------------
%---- C O O R D I N A T E S   A N D   D I S P L A Y   I N F O ---------

u = u*Minv;
if any(terminate_reason == [0 1 6])
  J = J(1:iter+1);
else
  J = J(1:iter);
end

clear global ALPHA BETA GOLDSTEIN_TEST COUNT Minv 
clear global User_function_exist User_armijo_test_exist

pos = min(iter,4)-1;
endings = ['st','nd','rd','th'];
suffix = endings(pos*2+1:pos*2+2);
termination = ['Integration produced NaN or Inf.                  ';...
	       'Normal termination test satisfied.                ';...
	       'Completed maximum number of iterations.           ';...
	       'Direction vector too small.                       ';...
	       'All controls at bounds and going to stay that way.';...
	       'Gradient too small.                               ';...
	       'Step size too small.                              ';...
	       'User test satisfied.                              '];

fprintf('\nFinished pdmin loop on the %d-%s iteration.\n',iter,suffix)
fprintf('%s\n',termination(terminate_reason+2,:))
if terminate_reason >= 0, 
  if bounds
    fprintf('Number of active bounds = %d.\n',length(I_a));
  end
end

% In case there is a premature termination on the first iteration:
if norm_grad == []
    temp = [grad_u(:) ; grad_x0(free_x0) ];
    norm_grad = Scale*sqrt(temp(I_i)'*temp(I_i));
end

inform=[terminate_reason,norm_grad/Scale,k,Scale,iter];
