/******************************************/
/*                                        */
/* Copyright (c) 1996 by Adam L. Schwartz */
/*          All right reserved.           */
/*                                        */
/******************************************/

/* The calling syntax is describe below.  There are many ways
   to call simulate().  This is to allow (i) flexibility and (ii) efficiency
   (e.g. you don't have to keep re-integrating the dynamics to get the values
     of different objective functions or to compute gradients, etc.)

                      DESCIPTION Of INPUTS And OUTPUTS
                      ================================
   The following table describes all of the various inputs and outputs
   used by simulate().  
                                   rows      columns     description
                                  ------    ---------   -------------
   Inputs:  x0                      n           1       initial state
            xf                      n           1       final state
            u                       m       (see note)  control vector
	    t                       1          N+1      time vector
            tf                      1           1       final time
	    ialg                  1 or 3        1       integration algorithm
	    action                  1           1       (see below)
            f_num                   1           1       function number

   Outputs: f                       1           1       cost
            x                       n          N+1      state trajectory
	    p                       n          N+1      adjoint trajectory
            lte                     n          N+1      error estimates
	    xdot                    n        N+1 or 1   h(t,x,u)
	    zdot                    1           1       l(t,x,u)
	    du                      m       (see Note)  gradient w.r.t. u
            dx0                     1           1       gradient w.r.t. x0
            g_x0                    1           n       df/dx0
            g_xf                    1           n       df/dxf
	    h_x                     n           n       dh/dx
	    h_u                     n           m       dh/du
	    l_x                     1           n       dl/dx
	    l_u                     1           m       dl/du

   ( n = # of states, m = # inputs, N = discretization leval, r depends
     on integration routine: 1 for Euler, 2 for Trapezoidal, 
     1, 3 or 4 for RK4, and 1 or 2 for Stride and Adams. )

   NOTE: The number of columns in u and du depends on the integration 
   algorithm (IALG) being used.  The following table gives these lengths:
   
        IALG      | Number of columns in u, du. (control representation)
   ---------------------------------------------------------------------
   0  (Discrete)  | N      --- piecewise constant.  
   1  (Euler)     | N      --- piecewise constant.  
   2  (Trapezoid) | N+1    --- continuous piecewise linear.
   3  (RK3)       | N+1    --- conntinous piecewise linear.
   4  (RK4)       | N-1+K  --- B-splines of order K (= 2,3,4).
   5  (Lsoda)     | N-1+K  --- B-splines of order K (= 1,2,3,4).
      with user supplied analytical Jacobians (Dh,Dl).
   6  (Lsoda)     | N-1+K  --- B-splines of order K (= 1,2,3,4).
      without user supplied analytical Jacobians (Dh,Dl).
   ----------------------------------------------------------------------
   Note, the following call sequences perform the same function and take
   about the same amount of execution time:

   [f,x,du] = simulate(1,x0,u,t,ialg,3)

   f=simulate(1,x0,u,t,ialg,1)
   du=simulate(2,1,2)

   simulate(1,x0,u,t,ialg,0)
   f=simulate(2,1,1)
   du=simulate(2,1,2)

   (Calling simulate(1,...) always re-integrates the system dynamics).

   Finally, in forms 1 and 2 below, simulate will return null vectors for
   any extra outputs specifies in the call to simulate().
*/
/****************************************************************************/
/***********************      CALLING SYNTAX       **************************/
/****************************************************************************/
/* The first argument to simulate() determines the form of the simulation.  */
/*                                                                          */
/*                                                                          */
/* simulate(-1) unlinks current driver (used for programming purposes only).*/
/*                                                                          */
/*                             ---- OR ----                                 */
/*                                                                          */
/* [info,simed] = simulate(0,{params})                                      */
/*                                                                          */
/* This form returns information about the system.                          */
/* If params is given, the message from activate() will be printed.         */
/*                                                                          */
/* params = system parameters (it's OK to specify params=[]).               */
/*                                                                          */
/* info(1)  = # states                                                      */
/* info(2)  = # inputs                                                      */
/* info(3)  = # system parameters                                           */
/* info(4)  = reserved                                                      */
/* info(5)  = reserved                                                      */
/* info(6)  = # objective functions                                         */
/* info(7)  = # nonlinear inequality trajectory constraints,                */
/* info(8)  = # linear inequality trajectory constraints,                   */
/* info(9)  = # nonlinear endpoint inequality constraints,                  */
/* info(10) = # linear endpoint inequality constraints,                     */
/* info(11) = # nonlinear endpoint equality constraints,                    */
/* info(12) = # linear endpoint equality constraints,                       */
/* info(13) = 0: nonlinear, 1: linear, 2: LTI, 3: LQ, 4: LQ and LTI         */
/* info(14) = # of points in the current integration mesh.                  */
/*                                                                          */
/* simed = 1 if simulate(1,...) has been called.                            */
/*                                                                          */
/*                                                                          */
/*                             ---- OR ----                                 */
/*                                                                          */
/*                                                                          */
/* [f,x,du,dx0,p]=simulate(1,x0,u,t,ialg,action)                            */
/*                                                                          */
/* This form integrates the dynamics and evaluates the first objective      */
/* functions.                                                               */
/*                                                                          */
/* Inputs: x0,u,t=[t0:tf:dT]',ialg,action                                   */
/* (only the first column of x0 is used in the simulation).                 */
/* Outputs: f  --- value of the first objective function.                   */
/*          x  --- state                                                    */
/*          du --- dJ/du                                                    */
/*          dx0 --- dJ/dx0 (z is parameters)                                */
/*          p  --- adjoint                                                  */
/*                                                                          */
/* For the variable stepsize alogrithms (5,6), the number of internal       */
/* knot points (to store values of x(t) and xdot(t) for gradient            */
/* computations) and error tolerance can be passed in the ialg parameter as */
/*   ialg = [ialg;n_intrnl_knots;rtol;atol]                                 */
/*   Default values: n_intrnl_knots = 1;                                    */
/*                   rtol = atol = 1e-8.                                    */
/*                                                                          */
/*  The available algorithms are:                                           */
/*  ialg = 0 --- discrete-time                                              */
/*         1 --- euler                                                      */
/*         2 --- etrapezoid                                                 */
/*         3 --- 3rd order RK                                               */
/*         4 --- 4th order RK                                               */
/*         5 --- LSODA                                                      */
/*         6 --- LSODA without user supplied Jacobians (Dh,Dl).             */
/*                                                                          */
/* Action = 0: Return nothing.                                              */
/* Action = 1: Return f (for first function).                               */
/* Action = 2: Return f,x.                                                  */
/* Action = 3: Return f,x,du,dx0.                                           */
/* Action = 4: Return f,x,du,dx0,p.                                         */
/*                                                                          */
/*                                                                          */
/*                             ---- OR ----                                 */
/*                                                                          */
/*                                                                          */
/* j=simulate(2,f_number,1)                                                 */
/* [du,dx0,p]=simulate(2,f_number,action)                                   */
/*                                                                          */
/* This form can only be called if simulate(1,...) has already been called. */
/* It can return function values and gradients for objective functions      */
/* and constraints.                                                         */
/*                                                                          */
/* Inputs: f_number,action                                                  */
/* action = 2: Return du,dx0                                                */
/* action = 3: Return du,dx0,p                                              */
/*                                                                          */
/* The order of function and gradient evaluations is as follows:            */
/* Let n1 = # of objective functions                                        */
/*     n2 = n1 + (# of trajectory constraints)*(N+1)                        */
/*     n3 = n2 + (# of endpoint constraints)                                */
/*                                                                          */
/* Then, for                                                                */
/*                                                                          */
/*   1 <= f_number <= n1, evaluate the f_number objective function ;        */
/*                                                                          */
/*   n1 < f_number <= n2, evaluate the j-th trajectory constraint at time   */
/*                        t = i*DELTA where, with k = f_number - n1 - 1,    */
/*                             j = k mod (N+1) + 1                          */
/*                        and  i = f_number - n1 - j*(N+1) - 1              */
/*                                                                          */
/*   n2 < f_number <= n3, evaluate the j-th endpoint constraint where       */
/*                        j = f_number - n2.                                */
/*                                                                          */
/* In words, first the objective functions are evaluated, then the          */
/* trajectory constraints are evaluated at each discrete time point, and    */
/* finally, the endpoint constraints are evaluated.                         */
/*                                                                          */
/*                                                                          */
/*                             ---- OR ----                                 */
/*                                                                          */
/*                                                                          */
/* [xdot,zdot] = simulate(3,x,u,t,f_num)                                    */
/* [xdot,zdot] = simulate(3,x,u,t,f_num,i)                                  */
/* [xdot,zdot,pdot] = simulate(3,x,u,t,p)                                   */
/* [xdot,zdot,pdot] = simulate(3,x,u,t,p,i)                                 */
/*                                                                          */
/* Returns xdot = h(t,x,u); zdot = l(t,x,u); pdot = -h_x(t,x,u)'*p - l_x.   */
/* The outputs are the same length as t (as must be x and u).               *
/* f_num defaults to 1 and only affects zdot.                               */
/* If 't' is a vector, then 'i' cannot be specified.                        */
/* If 'i' is specified, then neq[3] is set to i-1, otherwise neq[3] = 0.    */
/*                                                                          */
/*                                                                          */
/*                             ---- OR ----                                 */
/*                                                                          */
/*                                                                          */
/* [h_x,h_u,l_x,l_u] = simulate(4,x,u,t,f_num)                              */
/* [h_x,h_u,l_x,l_u] = simulate(4,x,u,t,f_num,i)                            */
/*                                                                          */
/* Returns the linearized system and integral cost matrices.                */
/* l_x and l_u are column vectors.                                          */
/* f_num defaults to 1 and only affect l_x, l_u.                            */
/* If 'i' is specified, then neq[3] is set to i-1, otherwise neq[3] = 0.    */
/*                                                                          */
/*                                                                          */
/*                             ---- OR ----                                 */
/*                                                                          */
/*                                                                          */
/* [g,g_x0,g_xf] = simulate(5,x0,xf,tf,f_num)                               */
/*                                                                          */
/* Returns the initial and end cost function and its linearizations.        */
/* g is a scalar.                                                           */
/* g_x0 and g_xf are column vectors.                                        */
/* f_num defaults to 1.                                                     */
/*                                                                          */
/*                                                                          */
/*                             ---- OR ----                                 */
/*                                                                          */
/*                                                                          */
/* stats = simulate(6)                                                      */
/*                                                                          */
/* Provides statistics on how many times f,x,du,dx0,p have been computed    */
/* since last call to simulate(6) or simulate(-1).                          */
/*                                                                          */
/* stats = [n_H,n_DH,n_x,n_J,n_grad]                                        */
/*                                                                          */
/*                                                                          */
/*                             ---- OR ----                                 */
/*                                                                          */
/*                                                                          */
/* lte = simulate(7)                                                        */
/*                                                                          */
/* This call will return estimates of the local truncation error            */
/* from the previous integration of the system dynamics.                    */
/*                                                                          */
/* lte(k), k=0,...,N-1, is a vector of the local truncation errors at time  */
/*                      t_k.  lte(k) = 0 if ialg < 1 or ialg > 4.           */
/*                                                                          */
/*                                                                          */
/****************************************************************************/
/* If DYNAMIC is defined at compile time, dld() will be used for dynamic    */
/* linking.  This doesn't work with the MEX interface so don't use it!      */
/****************************************************************************/
/* If there is a IEEE floating point error, simulate will return NaN.       */
/****************************************************************************/
#ifndef DYNAMIC
#define STATIC
#endif

#include "mex.h"
#include "utility.h"
#include "system.h"
#ifndef STATIC
#include <string.h>
#include "dld.h"
#endif

/****************************************************************************/
/* If IEEE is true, then certain floating point exceptions will be trapped  */
/* and simulate() will return NaN.                                          */
/****************************************************************************/
#define IEEE 1
#if IEEE
#include <floatingpoint.h>
#include <signal.h>
#include <setjmp.h>
#endif

#define MIN_INPUTS  1
#define MAX_INPUTS  6
#define MIN_OUTPUTS 0
#define MAX_OUTPUTS 5
#define SLEN 256

/*********************I M P O R T A N T ***********************/
/* This is an improtant define.  If number of items of system */
/* information changes, you must also change this.            */
/**************************************************************/
#define N_INFO 14

#ifndef NULL
#define NULL 0
#endif

long unsigned int H_calls,DH_calls;	    /* These are set by the drivers. */

#ifndef STATIC
#define HARD 1
extern void load_system();
static void (*integrator)();
#else
extern void discrete();
extern void euler();
extern void trapezoid();
extern void RK3();
extern void RK4();
extern void adams();
#endif 

static char *file_name,*driver_name;
static int current_ialg = -1,sim_flag=0,N_mesh_pts;

static void error(error_msg)
     char *error_msg;
{
#if IEEE
  ieee_handler("set","division",SIGFPE_DEFAULT);
  ieee_handler("set","invalid",SIGFPE_DEFAULT);
#endif
  mexErrMsgTxt(error_msg);
}

#if IEEE
sigfpe_handler_type old_handler1,old_handler2;
jmp_buf env;

static void ieee_error(sig,code,scp,addr)
     char *addr;
     int sig,code;
     struct sigcontext *scp; 
     /* sigcontext defined in /usr/include/sys/signal.h */
{
  sim_flag = 0;
  ieee_handler("set","division",SIGFPE_DEFAULT);
  ieee_handler("set","invalid",SIGFPE_DEFAULT);
  mexPrintf("Caught a floating point exception (code %x).\n",code);
  longjmp(env,1);
}
#endif

#if DIRECT

void simulate(nlhs, lhs, nrhs, rhs)

#else

#ifdef __STDC__
void 
#else
int
#endif
mexFunction(nlhs, lhs, nrhs, rhs)

#endif
     int nlhs, nrhs;
     Matrix *lhs[], *rhs[];	 /* lhs is a pointer to the return arguments */
				 /* rhs is a pointer to the input arguments  */
{
  static void unlink_integrator(),call_integrator();
  static void finish();
  char error_msg[SLEN],**undef_strings;
  static char *message;
  static int system_loaded_p=0,params_loaded;
  static int nstates,ninputs,nparams,nfunctions,points,sys_params[N_INFO];
  static int n_function_evaluations[3];
  int i,j,row,col,action,ialg,form,dld_error,index;
  double *Atemp,*Btemp,*lx_temp,*lu_temp;
  Matrix *info,*temp,*x,*p,*x0,*u,*xdot,*zdot,*pdot;
  Matrix *A,*B,*l_x,*l_u,*t,*E_xf,*E_x0,*E;

  mexAtExit(finish);

  if (nrhs < MIN_INPUTS || nrhs > MAX_INPUTS ||
      nlhs < MIN_OUTPUTS || nlhs > MAX_OUTPUTS) {
    mexPrintf("Usage:\n");
    mexPrintf("------\n");
    mexPrintf("[info,simed] = simulate(0,{params})\n\n");
    mexPrintf("[f,x,du,dx0,p]=simulate(1,x0,u,t,ialg,action)\n\n");
    mexPrintf("f=simulate(2,f_number,1)\n");
    mexPrintf("[du,dx0,p]=simulate(2,f_number,action)\n\n");
    mexPrintf("[xdot,zdot] = simulate(3,x,u,t,{f_num,{i}})\n");
    mexPrintf("[xdot,zdot,pdot] = simulate(3,x,u,t,p,{i})\n\n");
    mexPrintf("[h_x,h_u,l_x,l_u] = simulate(4,x,u,t,{f_num,{i}})\n\n");
    mexPrintf("[g,g_x0,g_xf] = simulate(5,x0,xf,tf,{f_num})\n\n");
    mexPrintf("[n_H,n_DH,n_J,n_x,n_grad] = simulate(6)\n\n");
    mexPrintf("lte = simulate(7)\n\n");
    for ( i = 0 ; i < nlhs ; i++ )
      lhs[i] = mxCreateFull(0,0,0);
    return;
  }

#if IEEE
  /* Save old exception handlers and set up handlers to intercept */
  /* IEEE floating point errors.                                  */
  old_handler1 = 
    (sigfpe_handler_type) ieee_handler("get","division",old_handler1);
  old_handler2 = 
    (sigfpe_handler_type) ieee_handler("get","invalid",old_handler2);
  ieee_handler("set","division",(sigfpe_handler_type)ieee_error);
  ieee_handler("set","invalid",(sigfpe_handler_type)ieee_error);

  /* ieee_error() will longjmp back to here.  The outputs are set */
  /* to NaN, and control is returned to the caller of simulate(). */
  if (setjmp(env)) {
    if ( nlhs == 0 ) {
      lhs[0] = mxCreateFull(1,1,REAL);
      mxGetPr(lhs[0])[0] = mexGetNaN();
    }
    else {
      for ( i = 0 ; i < nlhs ; i++ ) {
	if (lhs[i] == NULL)
	  lhs[i] = mxCreateFull(1,1,REAL);
	mxGetPr(lhs[i])[0] = mexGetNaN();
      }
    }
    return;
  }
#endif

  /*************************************************************/
  /*                 Load system information                   */
  /*************************************************************/
  if (!system_loaded_p) {
    sim_flag = 0;
    H_calls = DH_calls = 0;
    for ( i = 0 ; i < 3 ; i++ ) n_function_evaluations[i] = 0;
    for ( i = 0 ; i < N_INFO ; i++ )
      sys_params[i] = 0;
    sys_params[5] = 1;			/* Default # functions is 1. */
#ifndef STATIC    
    load_system("simulate.mex4","system.o",sys_params,&message);
#else
    activate(&message);
    init(sys_params,NULL);
#endif
    nstates = sys_params[0]; 
    ninputs = sys_params[1]; 
    nparams = sys_params[2];
    nfunctions = sys_params[5];
    system_loaded_p = 1;
    if (nparams == 0) 
      params_loaded = 1;
    else
      params_loaded = 0;
  }

  form = (int) mxGetScalar(rhs[0]);

  if ( !params_loaded && form >= 1 && form <= 5 )
    error("Can't simulate system, haven't load system parameters yet.");

  /* Switch statement determines which form of simulate() is being done. */
  switch (form) {

  /*************************************************************/
  /*                      C A S E  - O N E                     */
  /*************************************************************/

  case -1:
    system_loaded_p = 0;
    sim_flag = 0;
    finish();
    break;

  /*************************************************************/
  /*                     C A S E   Z E R O                     */
  /*************************************************************/

  case 0:
    if ( nrhs > 2 )
      error("Too many input arguments.");
    if ( nlhs > 2 )
      error("Maximum of two output argument.");
    if ( nlhs == 0 ) {
      if ( message != NULL ) 
	mexPrintf("%s\n",message);
      if ( sys_params[12] )
	mexPrintf("This is a linear system with ");
      else
	mexPrintf("This is a nonlinear system with ");
      mexPrintf("%d states, %d inputs and %d parameters,\n",
		sys_params[0],sys_params[1],sys_params[2]);
      mexPrintf("     %d objective function,\n",sys_params[5]);
      mexPrintf("     %d nonlinear and %d linear trajectory constraints,\n",
		sys_params[6],sys_params[7]);
      mexPrintf("     %d nonlinear and %d linear endpoint inequality constraints,\n",sys_params[8],sys_params[9]);
      mexPrintf("     %d nonlinear and %d linear endpoint equality constraints.\n",sys_params[10],sys_params[11]);
    }
    
    /* Pass arguments, if present, to the system. */
    if ( nrhs == 2 && mxGetPr(rhs[1]) != NULL ) {
      if ( mxGetM(rhs[1])*mxGetN(rhs[1]) != nparams )
	error("Wrong number of parameters.");
#ifndef STATIC
      (*initor)(sys_params,mxGetPr(rhs[1]));
#else
      init(sys_params,mxGetPr(rhs[1]));
#endif
      params_loaded = 1;
    }

    /* sys_params[] already loaded when system linked. */
    info = mxCreateFull(1,N_INFO,REAL);
    lhs[0] = info;
    for ( i = 0 ; i < N_INFO ; i++ )
      mxGetPr(info)[i] = (double) sys_params[i];
    mxGetPr(info)[3] = 0.0;
    if ( sim_flag )
      mxGetPr(info)[13] = N_mesh_pts;
    else
      mxGetPr(info)[13] = 0;
    if ( nlhs == 2 ) {
      temp = mxCreateFull(1,1,REAL);
      *mxGetPr(temp) = (double) sim_flag;
      lhs[1] = temp;
    }
    break;
    
  /*************************************************************/
  /*                      C A S E   T W O                      */
  /*************************************************************/

  case 2:
    if ( sim_flag == 0 )
      error("Must simulate system first.");

    /* Check for correct number of input/output arguments. */
    if ( nrhs != 3 )
      error("Wrong number of input arguments.");

    action = (int) mxGetScalar(rhs[2]);
    if ( (action < 1) || (action > 3) ) 
      error("Simulate: action argument is either 1,2, or 3.");

    call_integrator(nlhs,lhs,nrhs,rhs);

    /* Add new functions evaluations to statistics. */
    switch (action) {
    case 3:
    case 2:
      ++n_function_evaluations[2];  /* grad computed. */
      break;
    case 1:
      ++n_function_evaluations[1];  /* J  computed. */
    }

    /* Supply null matrices for any extra output arguments. */
    if ( nlhs > action )
      for ( i = action ; i < nlhs ; i++ )
	lhs[i] = mxCreateFull(0,0,REAL);
    break;

  /*************************************************************/
  /*                      C A S E   O N E                      */
  /*************************************************************/

  case 1:
    /* Check for correct number of input/output arguments. */
    if ( nrhs != 6 )
      error("Wrong number of input arguments.");

    ialg = (int) mxGetScalar(rhs[4]);
    action = (int) mxGetScalar(rhs[5]);
    if ( (action < 0) || (action > 4) )
      error("Simulate: action argument is either 0, 1, 2, 3, or 4.");

    if ( mxGetM(rhs[3]) != 1 )
      error("Simulate: t must be a row vector.");

#ifndef STATIC
    
    if ( ialg < 0 )
      error("Integration algorithm number must be positive.");

    /* If integrator already linked in, skip this part. */
    if (ialg != current_ialg) {

      if ( current_ialg >= 0 )
	unlink_integrator();

      /* REMEBER: if you change this, you also must change transformation. */
      switch (ialg) {
      case 0: 
        file_name = "drivers/discrete.o";
	driver_name = "discrete";
        break;
      case 1:
        file_name = "drivers/euler.o";
	driver_name = "euler";
	break;	     
      case 2:
        file_name = "drivers/trapezoid.o";
	driver_name = "trapezoid";
	break;
      case 3:
        file_name = "drivers/RK3.o";
	driver_name = "RK3";
	break;
      case 4:
        file_name = "drivers/RK4.o";
	driver_name = "RK4";
	break;
      case 5:
        file_name = "drivers/adams.o";
	driver_name = "adams";
	break;
      case 6:
        file_name = "drivers/adams.o";
	driver_name = "adams";
	break;
      default:
	error("Invalid integration routine number.");
      }
 
      if ((dld_error = dld_link(file_name)) != 0) {
	sprintf(error_msg,"Can't link %s, error = %d",file_name,dld_error);
	error(error_msg);
      }
      
      if ( ialg == 5 || ialg == 6) {
	if ((dld_error = dld_link("drivers/lsoda/lsoda.a")) != 0) 
	  error("Can't link lsoda.");
	if ((dld_error = dld_link("/usr/tools/lib/liblinpackd.a")) != 0) 
	  error("Can't link linpackd.");
	if ((dld_error = dld_link("/usr/lang/SC1.0/libF77.a")) != 0) 
	  error("Can't link libF77.");
	if ((dld_error = dld_link("/usr/lang/SC1.0/cg89/libm.a")) != 0) 
	  error("Can't link libm.");
	if ((dld_error = dld_link("/usr/lib/libc.a")) != 0) 
	  error("Can't link libc.");
      }
      
      if ((integrator = (void (*)()) dld_get_func(driver_name)) == 0)
	error("Integration function not found.");
      else {
	if (!(dld_function_executable_p(driver_name))) {
	  mexPrintf("Integration routine not executable.  " 
		    "%d undefined symbols.\n",dld_undefined_sym_count);
	  undef_strings = dld_list_undefined_sym();
	  for ( i = 0 ; i < dld_undefined_sym_count ; i++ ) 
	    mexPrintf("%s\n",undef_strings[i]);
	  return;
	}
      }
    }
    (*integrator)(nlhs,lhs,nrhs,rhs);
    current_ialg = ialg;
#else
    if ( ialg < 0 || ialg > 6 )
      error("Invalid Integration routine number.");

    if (ialg != current_ialg && current_ialg >= 0 )
      unlink_integrator();	   /* This will just unallocate some memory. */
    current_ialg = ialg;
    call_integrator(nlhs,lhs,nrhs,rhs);
#endif

    N_mesh_pts = mxGetN(rhs[3]);
    sim_flag = 1;

    /* Add new functions evaluations to statistics. */
    switch (action) {
    case 4:
    case 3:
      ++n_function_evaluations[2];  /* grad computed. */
    case 2:
    case 1:
      ++n_function_evaluations[1];  /* J  computed. */
    case 0:
      ++n_function_evaluations[0];  /* x  computed. */
    }

    /* Supply null matrices for any extra output arguments. */
    if ( action <= 2 && nlhs > action )
      for ( i = action ; i < nlhs ; i++ )
	lhs[i] = mxCreateFull(0,0,REAL);
    else 
      if ( nlhs > action+1 )
	for ( i = action+1 ; i < nlhs ; i++ )
	  lhs[i] = mxCreateFull(0,0,REAL);
    break;
    

  /*************************************************************/
  /*                     C A S E S   3 - 4                     */
  /*************************************************************/

  case 3:
  case 4:
    if ( nrhs < 4 || nrhs > 6 )
      error("Need 4, 5 or 6 input arguments.");
    if ( (form == 4 && nlhs > 4) || (form == 3 && nlhs > 3) )
      error("Too many outputs.");

    x = rhs[1];
    u = rhs[2];
    t = rhs[3]; points = mxGetM(t)*mxGetN(t);

    if ( mxGetN(x) != points || mxGetM(x) != nstates )
      error("State is wrong dimension.");
    if ( mxGetN(u) != points || mxGetM(u) != ninputs ) 
      error("Control is wrong dimension.");

    index = 0;
    if ( form == 3 ) {
      if ( nrhs == 6 )
	if ( points > 1 )
	  error("Too many inputs for vector t case.");
	else
	  if ( mxGetPr(rhs[5]) != NULL ) {
	    index = ((int) mxGetScalar(rhs[5]))-1;	 /* Index number.    */
	    if ( index < 0 )
	      error("Time index must be at least 1.");
	  }
      if ( nlhs == 3 ) {
	if ( nrhs < 5 )
	  error("Not enough inputs.");
	p = rhs[4];
	if ( mxGetN(p) != points || mxGetM(p) != nstates )
	  error("Adjoint is wrong dimension.");
      }
      else
	if ( nrhs > 4 )
	  if ( mxGetPr(rhs[4]) == NULL )
	    sys_params[4] = 1;
	  else
	    sys_params[4] = (int) mxGetScalar(rhs[4]);	 /* Function number. */
	else
	  sys_params[4] = 1;
    }
    else {
      if ( nrhs > 4 )
	if ( mxGetPr(rhs[4]) == NULL )
	  sys_params[4] = 1;
	else
	  sys_params[4] = (int) mxGetScalar(rhs[4]);	 /* Function number. */
      else
	sys_params[4] = 1;
      if ( nrhs > 5 )
	if ( mxGetPr(rhs[5]) != NULL ) {
	  index = ((int) mxGetScalar(rhs[4]))-1;	 /* Index number.    */
	  if ( index < 0 )
	    error("Time index must be at least 1.");
	}
    }
    
    if ( sys_params [4] < 1 )
      error("Function number must be greater than 0.");

#ifndef STATIC
    switch (form) {
    case 3:
      xdot = mxCreateFull(nstates,points,REAL);
      if (nlhs >= 2) {
	zdot = mxCreateFull(1,points,REAL);
	*mxGetPr(zdot) = 0.0;
      }
      if (nlhs == 3) {
	pdot = mxCreateFull(nstate,points,REAL);
	Atemp = (double*)mxCalloc(nstates,sizeof(double)*nstates);
	Btemp = (double*)mxCalloc(nstates,sizeof(double)*ninputs);
	lx_temp = (double*)mxCalloc(nstates,sizeof(double));
	lu_temp = (double*)mxCalloc(ninputs,sizeof(double));
      }
      sys_params[3] = index;
      for ( i = 1 ; i <= points ; i++ ) {
	j = i - 1;
	if ( points > 1 ) sys_params[3] = j;
	(*dld_h)(sys_params,mxGetPr(t)+j,mxGetPr(x)+j*nstates,
		 mxGetPr(u)+j*ninputs,mxGetPr(xdot)+j*nstates); H_calls++;
	if (nlhs >= 2 ) {
	  mxGetPr(zdot)[j] =(*dld_l)(sys_params,mxGetPr(t)+j,
				     mxGetPr(x)+j*nstates,
				     mxGetPr(u)+j*ninputs);
	  if ( sys_params[3] == -1 ) {		 /* System is of Mayer type. */
	    mxGetPr(zdot)[j] = 0.0;
	    sys_params[3] = index;
	  }
	}
	if (nlhs == 3) {
	  (*dld_Dh)(sys_params,mxGetPr(t)+j,mxGetPr(x)+j*nstates,
		    mxGetPr(u)+j*ninputs,Atemp,Btemp); DH_calls++;
	  (*dld_Dl)(sys_params,mxGetPr(t)+j,mxGetPr(x)+j*nstates,
		    mxGetPr(u)+j*ninputs,lx_temp,lu_temp);
	  for ( k = 0 ; k < nstates ; k++ ) {
	    mxGetPr(pdot)[j*nstates+k] = -lx_temp[k];
	    for ( l = 0 ; l < nstates ; l++ )
	      mxGetPr(pdot)[j*nstates+k] -= 
		Atemp[l*nstates+k]*mxGetPr(p)[j*nstates+k];
	  }
	}
      }
      lhs[0] = xdot;
      if (nlhs >= 2)
	lhs[1] = zdot;
      if (nlhs == 3) { 
	mxFree(lu_temp); mxFree(lx_temp); 
	mxFree(Btemp); mxFree(Atemp); 
	lhs[2] = pdot;
      }
      break; 
    case 4:
      Atemp = (double*)mxCalloc(nstates,sizeof(double)*nstates);
      Btemp = (double*)mxCalloc(nstates,sizeof(double)*ninputs);
      sys_params[3] = index;
      (*dld_Dh)(sys_params,mxGetPr(t),mxGetPr(x),mxGetPr(u),Atemp,Btemp);
      DH_calls++;

      A = mxCreateFull(nstates,nstates,REAL);
      transpose(Atemp,mxGetPr(A),nstates,nstates);
      lhs[0] = A;
      if ( nlhs > 1 ) {
	B = mxCreateFull(nstates,ninputs,REAL);
	transpose(Btemp,mxGetPr(B),nstates,ninputs);
	lhs[1] = B;
      }
      mxFree(Btemp); mxFree(Atemp);
      if ( nlhs > 2 ) {
	l_x = mxCreateFull(1,nstates,REAL);
	l_u = mxCreateFull(1,ninputs,REAL);
	sys_params[3] = index;
	(*dld_Dl)(sys_params,mxGetPr(t),mxGetPr(x),mxGetPr(u),
		  mxGetPr(l_x),mxGetPr(l_u));
	lhs[2] = l_x;
	if ( nlhs > 3 )
	  lhs[3] = l_u;
	else
	  mxFreeMatrix(l_u);
      }
      break;
    }
#else
    switch (form) {
    case 3:
      xdot = mxCreateFull(nstates,points,REAL);
      if (nlhs >= 2)
	zdot = mxCreateFull(1,points,REAL);
      if (nlhs == 3) {
	pdot = mxCreateFull(nstates,points,REAL);
	Atemp = (double*)mxCalloc(nstates,sizeof(double)*nstates);
	Btemp = (double*)mxCalloc(nstates,sizeof(double)*ninputs);
	lx_temp = (double*)mxCalloc(nstates,sizeof(double));
	lu_temp = (double*)mxCalloc(ninputs,sizeof(double));
      }
      sys_params[3] = index;
      for ( i = 1 ; i <= points ; i++ ) {
	j = i-1;
	if ( points > 1 ) sys_params[3] = j;
	h(sys_params,mxGetPr(t)+j,mxGetPr(x)+j*nstates,
	  mxGetPr(u)+j*ninputs,mxGetPr(xdot)+j*nstates); H_calls++;

	if (nlhs >= 2 ) {
	  mxGetPr(zdot)[j] = l(sys_params,mxGetPr(t)+j,
			       mxGetPr(x)+j*nstates,mxGetPr(u)+j*ninputs);
	  if ( sys_params[3] == -1 ) {		 /* System is of Mayer type. */
	    mxGetPr(zdot)[j] = 0.0;
	    sys_params[3] = index;
	  }
	}
	if (nlhs == 3) {
	  Dh(sys_params,mxGetPr(t)+j,mxGetPr(x)+j*nstates,mxGetPr(u)+j*ninputs,
	     Atemp,Btemp); DH_calls++;
	  Dl(sys_params,mxGetPr(t)+j,mxGetPr(x)+j*nstates,mxGetPr(u)+j*ninputs,
	     lx_temp,lu_temp);
	  for ( row = 0 ; row < nstates ; row++ ) {
	    mxGetPr(pdot)[j*nstates+row] = -lx_temp[row];
	    for ( col = 0 ; col < nstates ; col++ )
	      mxGetPr(pdot)[j*nstates+row] -= 
		Atemp[col*nstates+row]*mxGetPr(p)[j*nstates+col];
	  }
	}
      }
      lhs[0] = xdot;
      if (nlhs >= 2)
	lhs[1] = zdot;
      if (nlhs == 3) { 
	mxFree(lu_temp); mxFree(lx_temp); 
	mxFree(Btemp); mxFree(Atemp); 
	lhs[2] = pdot;
      }
      break;
    case 4:
      Atemp = (double*)mxCalloc(nstates,sizeof(double)*nstates);
      Btemp = (double*)mxCalloc(nstates,sizeof(double)*ninputs);
      sys_params[3] = index;
      Dh(sys_params,mxGetPr(t),mxGetPr(x),mxGetPr(u),Atemp,Btemp); DH_calls++;

      A = mxCreateFull(nstates,nstates,REAL);
      transpose(Atemp,mxGetPr(A),nstates,nstates);
      lhs[0] = A;
      if ( nlhs > 1 ) {
	B = mxCreateFull(nstates,ninputs,REAL);
	transpose(Btemp,mxGetPr(B),nstates,ninputs);
	lhs[1] = B;
      }
      mxFree(Btemp); mxFree(Atemp);
      if ( nlhs > 2 ) {
	l_x = mxCreateFull(1,nstates,REAL);
	l_u = mxCreateFull(1,ninputs,REAL);
	sys_params[3] = index;
	Dl(sys_params,mxGetPr(t),mxGetPr(x),mxGetPr(u),
	   mxGetPr(l_x),mxGetPr(l_u));
	lhs[2] = l_x;
	if ( nlhs > 3 )
	  lhs[3] = l_u;
	else
	  mxFreeMatrix(l_u);
      }
      break;
    }
#endif
    
    break;
    
  /*************************************************************/
  /*                      C A S E   F I V E                    */
  /*************************************************************/

  case 5:
    if ( nrhs < 4 || nrhs > 5 )
      error("Need 4 or 5 input arguments.");
    if ( nlhs > 3 )
      error("Too many outputs.");
    x0 = rhs[1];
    x  = rhs[2];
    t  = rhs[3]; points = mxGetM(t)*mxGetN(t);
    if ( nrhs == 5 )
      if ( mxGetPr(rhs[4]) == NULL )
	sys_params[4] = 1;				 /* Function number. */
      else
	sys_params[4] = (int) mxGetScalar(rhs[4]);	 /* Function number. */
    else
      sys_params[4] = 1;
    if ( points > 1 )
      error("t must be a scalar.");      
    if ( !( (mxGetM(x) == nstates && mxGetN(x) == 1)
 	  || (mxGetM(x) == 1 && mxGetN(x) == nstates) ) )
      error("State is wrong dimension.");
    if ( !( (mxGetM(x0) == nstates && mxGetN(x0) == 1)
 	  || (mxGetM(x0) == 1 && mxGetN(x0) == nstates) ) )
      error("Initial state is wrong dimension.");
    
    if ( sys_params [4] < 1 )
      error("Function number must be greater than 0.");

#ifndef STATIC
    E = mxCreateFull(1,1,REAL);
    *mxGetPr(E) = 
      (*dld_g)(sys_params,mxGetPr(t),mxGetPr(x0),mxGetPr(x));
    lhs[0] = E;
    if (nlhs > 1) {
      E_x0 = mxCreateFull(1,nstates,REAL);
      E_xf = mxCreateFull(1,nstates,REAL);
      (*dld_Dg)(sys_params,mxGetPr(t),mxGetPr(x0),
		   mxGetPr(x),mxGetPr(E_x0),mxGetPr(E_xf));
      lhs[1] = E_x0;
      if ( nlhs > 2 )
        lhs[2] = E_xf;
      else
        mxFreeMatrix(E_xf);
    }
#else
    E = mxCreateFull(1,1,REAL);
    *mxGetPr(E) = g(sys_params,mxGetPr(t),mxGetPr(x0),mxGetPr(x));
    lhs[0] = E;
    if (nlhs > 1) {
      E_x0 = mxCreateFull(1,nstates,REAL);
      E_xf = mxCreateFull(1,nstates,REAL);
      Dg(sys_params,mxGetPr(t),mxGetPr(x0),mxGetPr(x),
	    mxGetPr(E_x0),mxGetPr(E_xf));
      lhs[1] = E_x0;
      if ( nlhs > 2 )
        lhs[2] = E_xf;
      else
        mxFreeMatrix(E_xf);
    }
#endif
    
    break;

  /*************************************************************/
  /*                      C A S E   S I X                      */
  /*************************************************************/

  case 6:
    if ( nlhs > 1 )
      error("Too many outputs.");
    if ( nrhs > 1 )
      error("Too many inputs.");
    lhs[0] = mxCreateFull(1,5,REAL);
    mxGetPr(lhs[0])[0] = (double) H_calls;
    mxGetPr(lhs[0])[1] = (double) DH_calls;
    H_calls = DH_calls = 0;
    for ( i = 0 ; i < 3 ; i++ ) {
      mxGetPr(lhs[0])[i+2] = (double) n_function_evaluations[i];
      n_function_evaluations[i] = 0;
    }
    break;

  /*************************************************************/
  /*                    C A S E   S E V E N                    */
  /*************************************************************/

  case 7:
    if ( sim_flag == 0 )
      error("Must simulate system first.");

    if ( nlhs > 1 )
      error("Too many outputs.");
    if ( nrhs > 1 )
      error("Too many inputs.");
    if ( current_ialg < 1 || current_ialg > 4 ) {
      lhs[0] = mxCreateFull(1,1,REAL); mxGetPr(lhs[0])[0] = 0.0;
      break;
    }

    call_integrator(nlhs,lhs,nrhs,rhs);
    n_function_evaluations[0] += 2;      /* x computed on doubled grid. */
    break;

  default:
    error("Form must be between -1 and 7.");
  }
  
#if IEEE
  ieee_handler("set","division",SIGFPE_DEFAULT);
  ieee_handler("set","invalid",SIGFPE_DEFAULT);
#endif

  return;
}


/*************************************************************/
/*      O T H E R   S U N D R Y   F U N C T I O N S          */
/*************************************************************/

static void
call_integrator(nlhs,lhs,nrhs,rhs)
     int nlhs, nrhs;
     Matrix *lhs[], *rhs[];
{
  int i;

  /* I clear lhs[i] in case a floating point error gets trapped.  That */
  /* way I know for sure that lhs[i] == NULL implies that lhs[i] has   */
  /* not had memory allocated to it.                                   */
  for ( i = 0 ; i < nlhs ; i++ )
    lhs[i] = NULL;

#ifndef STATIC
  (*integrator)(nlhs,lhs,nrhs,rhs);
#else
  switch (current_ialg) {
  case 0:
    discrete(nlhs,lhs,nrhs,rhs);
    break;
  case 1:
    euler(nlhs,lhs,nrhs,rhs);
    break;
  case 2:
    trapezoid(nlhs,lhs,nrhs,rhs);
    break;
  case 3:
    RK3(nlhs,lhs,nrhs,rhs);
    break;
  case 4:
    RK4(nlhs,lhs,nrhs,rhs);
    break;
  case 5:
    adams(nlhs,lhs,nrhs,rhs);
    break;
  case 6:
    adams(nlhs,lhs,nrhs,rhs);
    break;
  }
#endif
}

/* Cause dynamically allocated memory to be freed. */
/* Unlink driver if it was dynamically linked.     */
static void 
unlink_integrator()
{
#ifndef STATIC
  char error_msg[20];
  int dld_error;

  (*integrator)(0,0,0,0); 
  if ( 0 != (dld_error = dld_unlink_by_file(file_name,HARD)) ) {
    sprintf(error_msg,"Couldn't unlink previous driver, %d.",dld_error);
    error(error_msg);
  }
#else
  call_integrator(0,0,0,0);
#endif
}
  
static void
finish()
{
  sim_flag = 0;
  if ( current_ialg >= 0 ) {
    unlink_integrator();
#ifndef STATIC
    load_system("simulate.mex4",NULL,NULL,NULL);	   /* Unlink system. */
#endif
    current_ialg = -1;
  }
#if IEEE
  ieee_handler("set","division",SIGFPE_DEFAULT);
  ieee_handler("set","invalid",SIGFPE_DEFAULT);
#endif
}
