% This function takes the vector of Lagrange multipliers produced from
% riots() or aug_lagrng() and breaks it up into groups according to
% which type of constraint the multliplier corresponds to:
% 
% le_ineq      linear, endpoint inequality
% le_eq        linear, endpoint equality
% nle_ineq     nonlinear, endpoint inequality
% nle_eq       nonlinear, endpoint equality
% lt           linear trajectory inequality
% nlt          nonlinear trajectory inequality
% obj          objective functions (=1 if only one objective)
%
% Each of these outputs is a column vector except lt and nlt have 
% one column per trajectory constraint.
% 
% This is the same order as the constraint order used by simulate(). 
% obj = 1 unless riots (linked with CFSQP) is used to solve a problem
%         with more than one objective function.
%
% NOTE: linear constraints are treated as nonlinear constraints if the
%       system dynamics are nonlinear.
%
%
% Inputs are,
% lambda --- column of Lagrange multipliers returned from one of the
%	     optimization programs.
% N1     --- Number of mesh points.
% N2     --- Number of decision variables (ncontrols + length(n_free_x0)).

%
% Copyright (c) 1996 by Adam L. Schwartz
% All right reserved.  
%

function [le_ineq,le_eq,nle_ineq,nle_eq,lt,nlt,obj] = sort_lambda(lambda,N1,N2)

info = simulate(0);
N_functions = info(6);
N_nlt_ineq = info(7);
N_lt_ineq  = info(8);
N_nle_ineq = info(9);
N_le_ineq  = info(10);
N_nle_eq   = info(11);
N_le_eq    = info(12);
LTI = info(13);
if LTI == 0
  N_nlt_ineq = N_nlt_ineq+N_lt_ineq; N_lt_ineq = 0;
  N_nle_ineq = N_nle_ineq+N_le_ineq; N_le_ineq = 0;
  N_nle_eq   = N_nle_eq+N_le_eq;     N_le_eq   = 0;
end
obj = 1;
N_constraints = sum(info(9:12)) + sum(info(7:8))*N1;
if N_constraints > 0 
  if lambda == []
    error('There are constraints...need lagrange multipliers.')
  else
    global opt_program
    le_ineq  = [];
    le_eq    = [];
    nle_ineq = [];
    nle_eq   = [];
    lt  = [];
    nlt = [];
    if strcmp(opt_program,'NPSOL')
      N3 = N2 + N_le_ineq;
      N4 = N3 + N_lt_ineq*N1;
      N5 = N4 + N_le_eq;
      N6 = N5 + N_nle_ineq;
      N7 = N6 + N_nlt_ineq*N1;
      N8 = N7 + N_nle_eq;
      if N_le_ineq,  le_ineq  = lambda(N2+1:N3); end
      if N_le_eq,    le_eq    = lambda(N4+1:N5); end
      if N_nle_ineq, nle_ineq = lambda(N5+1:N6); end
      if N_nle_eq,   nle_eq   = lambda(N7+1:N8); end
      lt  = reshape(lambda(N3+1:N4),N1,N_lt_ineq);
      nlt = reshape(lambda(N6+1:N7),N1,N_nlt_ineq);
    elseif strcmp(opt_program,'CFSQP')
      N3 = N2 + N_nle_ineq + N_functions;
      N4 = N3 + N_nlt_ineq*N1;
      N5 = N4 + N_le_ineq;
      N6 = N5 + N_lt_ineq*N1;
      N7 = N6 + N_nle_eq;
      N8 = N7 + N_le_eq;
      obj = lambda(N2+1:N2+N_functions);
      if N_nle_ineq, nle_ineq = -lambda(N2+1+N_functions:N3); end
      if N_le_ineq,  le_ineq  = -lambda(N4+1:N5); end
      if N_nle_eq,   nle_eq   = -lambda(N6+1:N7); end
      if N_le_eq,    le_eq    = -lambda(N5+1:N8); end
      nlt = -reshape(lambda(N3+1:N4),N1,N_nlt_ineq);
      lt  = -reshape(lambda(N5+1:N6),N1,N_lt_ineq);
    elseif strcmp(opt_program,'aug_lagrng')
      nle_ineq = []; le_ineq = []; 
      nlt = []; lt = [];
      nle_eq = lambda(1:N_nle_eq);
      le_eq  = lambda(N_nle_eq+1:N_nle_eq+N_le_eq);
    else
      error('sort_lambda: opt_program unknown.');
    end
  end
end

