/***********************************************************/
/* CALLING EXAMPLE:                                        */
/* >> N = 100;                                             */
/* >> T = 5;                                               */
/* >> t = [0:T/N:T];                                       */
/* >> u0 = zeros(1,N+2);                                   */
/* >> x0 = [1;0];                                          */
/* >> [u,x,f]=riots(x0,u0,t,[],[],[],100,4);               */
/***********************************************************/
/* PROBLEM DESCRIPTION:

   This defines the Van der Pol, nonlinear oscillator.
   See, "Constraint-space conjugate gradient method for function
   minimization and optimal control problems," Willoughby and Pierson, 
   Int. J. Control, vol. 14, no. 6, 1971, 1121-1135.

   x1dot = x2
   x2dot = -x1+(1-x1^2)*x2 + u

   x0=[1;0]

   Minimize:   J = \int_0^5 (x1^2 + x2^2 + u^2)/2 dt

   subject to: -x1(5) + x2(5) - 1 = 0.
*/

#define F_NUM neq[4]

#define NSTATES 2
#define NINPUTS 1
#define LEEC 1       /* Number of linear endpoint equality constraints. */

extern int points;
extern double *Y;

void activate(message)
     char **message;
{
  *message = "Van der Pol systems with enpoint constraint.";
} 

void init(neq,params)
     int neq[];
     double *params;
{
  neq[0]  = NSTATES;
  neq[1]  = NINPUTS;
  neq[11] = LEEC;
}

void h(neq,t,x,u,xdot)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS],xdot[NSTATES];
{
  xdot[0] = x[1];
  xdot[1] = -x[0] + (1.0-x[0]*x[0])*x[1] + u[0];
}

double Dh(neq,t,x,u,h_x,h_u)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS];
     double h_x[NSTATES][NSTATES],h_u[NSTATES][NINPUTS];
{
  h_x[0][1] = 1.0;
  h_x[1][0] = -1.0 - 2*x[0]*x[1];
  h_x[1][1] = 1.0 - x[0]*x[0];

  h_u[1][0] = 1.0;
  return 0.0;
}

double l(neq,t,x,u)
int neq[];
double *t,x[NSTATES],u[NINPUTS];
{
  double control;
   
  control = u[0];
  return (x[0]*x[0] + x[1]*x[1] + control*control)/2;
}

void Dl(neq,t,x,u,l_x,l_u)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS],l_x[NSTATES],l_u[NINPUTS];
{
  l_x[0] = x[0];
  l_x[1] = x[1];
  l_u[0] = u[0];
}

double g(neq,t,x0,xf)
     int neq[];
     double *t,x0[NSTATES],xf[NSTATES];
{

  if (F_NUM == 1)
    return 0.0;
  else
    return -xf[0] + xf[1] - 1.0;
}

double Dg(neq,t,x0,xf,J_x0,J_xf)
     int neq[];
     double *t,x0[NSTATES],xf[NSTATES],J_x0[NSTATES],J_xf[NSTATES];
{
  if (F_NUM == 2) {
    J_xf[0] = -1.0;
    J_xf[1] = 1.0;
  }
  return 0.0;
}
