/***********************************************************/
/* CALLING EXAMPLE:                                        */
/* >> N = 100;                                             */
/* >> t = [0:1/N:1];                                       */
/* >> u0 = ones(1,N+1);                                    */
/* >> x0 = [0 1 1 0.1]';                                   */
/* >> fixed = [1 1 1 0]';                                  */
/* >> [u,x,f]=riots([x0 fixed],u0,t,0,3.5,[],100,4);       */
/***********************************************************/
/* PROBLEM DESCRIPTION:                                    */
/* Goddard Rocket Problem --- has a singular arc.          */
/* Modified to allow a trajectory constraint.              */
/*
   v_dot = (1/m)[u(t)-D(v,r)] - 1/r^2 ; v(0) = 0
   r_dot = v                          ; r(0) = 1
   m_dot = -(1/c)u(t)                 ; m(0) = 1

   subject to:   m(tf)=0.6, 
                 0 <= u(t) <= u_max = 3.5.
                 A*q <= QMAX   (where q = rho*v^2/2).

   Objective: Determine u(t) to maximize height above ground
              (i.e. minimize -r(tf)).  

   v = veritcal velocity
   r = radial altitude above earth (r=1 is earth's surface)
   m = mass of vehicle
   u = thrust
   c = 0.5 = specific impulse (impulse per unit mass of fuel burned).
   D = drag = (1/2)rho_0*v^2*C_D*A
   rho = rho_0*exp(beta(1-r))
   A*rho_0 = 12,400.
   QMAX = 10.

   Initial guess: tf = 1; u(t) = u0(t) = 1;
   NOTE: u0(t) must be big enough so that rocket doesn't fall below ground! 

   The trajectory constraint is disabled unless there is a non-empty
   global variable FLAGS defined in Matlab's workspace.  If FLAGS is 
   defined as such, then QMAX must be specified using a system parameter.
*/

#include <math.h>

#define NSTATES 4
#define NINPUTS 1
#define LEEC  1            /* Linear endpoint equality constraints. */

#define FNUM neq[4]

#define V x[0]
#define R x[1]
#define M x[2]
#define M0 1.0
#define MF 0.6
#define BETA 500.0
#define C_D 0.05
#define Arho0 12400.0
#define C 0.5

#define tau x[3]

extern int get_flags();
static int Constraints;
static int QMAX;

void activate(message)
     char **message;
{
  int n,flags[1];

  *message = "Goddard";
  n = 1;
  if ( get_flags(flags,&n) > 0 ) {
    Constraints = 1;
    printf("Constrained problem.\n");
  }
  else {
    Constraints = 0;
    printf("Unconstrained problem.\n");
  }
} 

void init(neq,params)
     int neq[];
     double *params;
{
  int i;

  if ( params == 0 ) {
    neq[0] = NSTATES;
    neq[1] = NINPUTS;
    neq[11] = LEEC;
    if ( Constraints == 1 ) {
      neq[2] = 1;		    /* One system parameter to specify QMAX. */
      neq[7] = 1;
    }
    else 
      neq[7] = 0;
  } 
  else
    QMAX = params[0];
}

void h(neq,t,x,u,xdot)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS],xdot[NSTATES];
{
  double rho,q,D;

  rho = exp(BETA*(1.0-R));
  q = rho*V*V/2;
  D = q*C_D*Arho0;

  xdot[0] = tau*( (u[0]-D)/M - 1.0/(R*R) );     /* v_dot */
  xdot[1] = tau*V;                              /* r_dot */
  xdot[2] = -tau*u[0]/C;                        /* m_dot */
  xdot[3] = 0.0;
}

void Dh(neq,t,x,u,h_x,h_u)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS];
     double h_x[NSTATES][NSTATES],h_u[NSTATES][NINPUTS];
{
  double rho,D,dDdv,dDdr;
  
  rho  = exp(BETA*(1.0-R));
  dDdv = rho*V*C_D*Arho0;
  D    = V*dDdv/2;
  dDdr = -BETA*D;

  h_u[0][0] = tau/M;
  h_u[2][0] = -tau/C;

  h_x[0][0] = -tau*dDdv/M;
  h_x[0][1] = tau*(-dDdr/M + 2.0/(R*R*R));
  h_x[0][2] = -tau*(u[0]-D)/(M*M);
  h_x[0][3] = (u[0]-D)/M - 1.0/(R*R);
  h_x[1][0] = tau;
  h_x[1][3] = V;
  h_x[2][3] = -u[0]/C;
}

double l(neq,t,x,u)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS];
{
  double rho,q;

  if (FNUM == 1) {
    neq[3] = -1;	
    return 0.0;
  }
  else {
    rho  = Arho0*exp(BETA*(1.0-R));
    return rho*V*V/2 - QMAX;
  }
}

double Dl(neq,t,x,u,l_x,l_u)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS],l_x[NSTATES];
{
  double rho;
  if (FNUM == 2) {
    rho  = Arho0*exp(BETA*(1.0-R));
    l_x[0] = rho*V;
    l_x[1] = -BETA*rho*V*V/2;
  }
  return 0.0;
}

double g(neq,t,x0,x)
     int neq[];
     double *t,x0[NSTATES],x[NSTATES];
{
  if (FNUM == 1)
    return -R;				      /* Want to maximize R. */
  else
    return M - MF;
}

double Dg(neq,t,x0,xf,J_x0,J_xf)
     int neq[]; 
     double *t,x0[NSTATES],xf[NSTATES],J_x0[NSTATES],J_xf[NSTATES];
{
  if (FNUM == 1)
    J_xf[1] = -1.0;
  else
    J_xf[2] = 1.0;
  return 0.0;
}
