/******************************************/
/*                                        */
/* Copyright (c) 1996 by Adam L. Schwartz */
/*          All right reserved.           */
/*                                        */
/*************************************************************************/
/* If this code is linked with the simulate.c or any other program which */
/* calls the user's system functions, then the user can supply his       */
/* functions as Matlab m-files.                                          */
/* Using m-files instead of ojbect code can result in execution running  */
/* ten times or more slower.                                             */
/*************************************************************************/

#include "/usr/sww/matlab/extern/include/mex.h"

#ifndef NULL
#define NULL 0
#endif

#define N_INFO 13

static int nstates,ninputs,nparams;
static Matrix *M_neq,*M_t,*M_x,*M_u,*M_x0,*M_xf;
static Matrix *rhs[4],*lhs[1];

/****************************************/
/* Matlab call: message = sys_activate; */
/****************************************/
void activate(message)
char **message;
{
  int i,cols;
  
  mexCallMATLAB(1,lhs,0,rhs,"sys_activate");
  cols = mxGetN(lhs[0]);

  /* Note: Matlab will automatically free this memory. */
  *message = mxCalloc(sizeof(char),cols+1);

  for ( i = 0 ; i < cols ; i++ )
    (*message)[i] = (char)mxGetPr(lhs[0])[i];
  (*message)[i] = '\0';
  mxFree(lhs[0]);
  return;
} 

/*********************************/
/* Matlab call: neq = sys_init() */
/*               ----  OR  ----  */
/*              sys_init(params) */
/*********************************/
void init(neq,params)
     int neq[];
     double *params;
{
  int i,j,rows;
  Matrix *M_params;
  
  if (params == NULL) {
    /* sys_init.m should returns a two row matrix.  The top row is */
    /* the index j of neq[] to be set and the bottom rows is the   */
    /* corresponding value for neq[j].                             */
    M_params = mxCreateFull(0,0,0);
    rhs[0] = M_params;
    mexCallMATLAB(1,lhs,1,rhs,"sys_init");
    rows = mxGetM(lhs[0]);
    if ( rows > 0 && mxGetN(lhs[0]) != 2 )
      mexErrMsgTxt("Information matrix neq must have two columns.");

    /* The values of neq[i] default to 0 except neq[5] = 1. */
    for ( i = 0 ; i < rows ; i++ ) {
      j = (int)mxGetPr(lhs[0])[i]-1;
      neq[j] = (int)mxGetPr(lhs[0])[rows+i];
    }
    mxFreeMatrix(lhs[0]);
    nstates = neq[0];
    ninputs = neq[1];
    nparams = neq[2];
  }
  else {
    M_params = mxCreateFull(nparams,1,0);
    for ( i = 0 ; i < nparams ; i++ )
      mxGetPr(M_params)[i] = params[i];
    rhs[0] = M_params;
    mexCallMATLAB(0,lhs,1,rhs,"sys_init");
    mxFreeMatrix(M_params);
  }
}

/***********************************************************************/
/* This function creates temporary memory for input argument Matrices, */
/* calls MATLAB and then frees the temporary storage.                  */
/* Note that there is no way around using mxCreateFull() because one   */
/* cannot have a statement like: Matrix M_x;  since Matrix is not a    */
/* completely specified type---the compiler is not able to allocate    */
/* storage for M_x.  Also, I cannot use mxSetPr() to cause because     */
/* Matlab will try to free the memory pointed to by mxSetPr() after    */
/* call is finished.                                                   */
/*                                                                     */
/* call_type = 1: arg3 = x,  arg4 = u ;                                */
/* call_type = 2: arg3 = x0, arg4 = xf.                                */
/* deriv = 0: return 1 argument.                                       */
/* deriv = 1: return call_type+1 arguments.                            */
/***********************************************************************/
void call_Matlab(neq,t,arg3,arg4,f_name,call_type,deriv,ans)
     char *f_name;
     int neq[],call_type,deriv;
     double *t,arg3[],arg4[];
     Matrix *ans[];
{
  int i;

  M_neq  = mxCreateFull(N_INFO,1,0);
  M_t    = mxCreateFull(1,1,0);
  for ( i = 0 ; i < N_INFO ; i++ )
    mxGetPr(M_neq)[i] = (double)neq[i];
  mxGetPr(M_neq)[3] = (double)neq[3]+1.0;  /* discrete-time index +1 to   */
  *mxGetPr(M_t) = *t;                      /* conform to Matlab indexing. */
  rhs[0] = M_neq; rhs[1] = M_t;

  if (call_type == 1) {
    M_x = mxCreateFull(nstates,1,0);
    M_u = mxCreateFull(ninputs,1,0);
    for ( i = 0 ; i < nstates ; i++ )
      mxGetPr(M_x)[i] = arg3[i];
    for ( i = 0 ; i < ninputs ; i++ )
      mxGetPr(M_u)[i] = arg4[i];
    rhs[2] = M_x; rhs[3] = M_u;
  }
  else {
    M_x0 = mxCreateFull(nstates,1,0);
    M_xf = mxCreateFull(nstates,1,0);
    for ( i = 0 ; i < nstates ; i++ ) {
      mxGetPr(M_x0)[i] = arg3[i];
      mxGetPr(M_xf)[i] = arg4[i];
    }
    rhs[2] = M_x0; rhs[3] = M_xf;
  }

  if (!deriv)
    mexCallMATLAB(1,ans,4,rhs,f_name);
  else
    mexCallMATLAB(call_type+1,ans,4,rhs,f_name);

  if (call_type == 1) {
    mxFreeMatrix(M_u); mxFreeMatrix(M_x);
  }
  else {
    mxFreeMatrix(M_xf); mxFreeMatrix(M_x0);
  }
  mxFreeMatrix(M_t); mxFreeMatrix(M_neq);
}

/*****************************************/
/* Matlab call: xdot = sys_h(neq,t,x,u); */
/*****************************************/
void h(neq,t,x,u,xdot)
     int neq[];
     double *t,x[],u[],xdot[];
{
  int i;
  Matrix *ans[1];

  call_Matlab(neq,t,x,u,"sys_h",1,0,ans);
  for ( i = 0 ; i < nstates ; i++ )
    xdot[i] = mxGetPr(ans[0])[i];
  mxFreeMatrix(ans[0]);
}

/***********************************************/
/* Matlab call: [h_x,h_u] = sys_Dh(neq,t,x,u); */
/***********************************************/
void Dh(neq,t,x,u,h_x,h_u)
     int neq[];
     double *t,x[],u[],h_x[],h_u[];
{
  int i,j;
  Matrix *ans[2];

  call_Matlab(neq,t,x,u,"sys_Dh",1,1,ans);
  /* Load the transpose of the matrices returned by Matlab */
  /* into h_x and h_u.                                     */
  for ( i = 0 ; i < nstates ; i++ )
    for ( j = 0 ; j < nstates ; j++ )
      h_x[i*nstates+j] = mxGetPr(ans[0])[j*nstates+i];
  for ( i = 0 ; i < nstates ; i++ )
    for ( j = 0 ; j < ninputs ; j++ )
      h_u[i*ninputs+j] = mxGetPr(ans[1])[j*nstates+i];
  mxFreeMatrix(ans[1]); mxFreeMatrix(ans[0]);
}

/*****************************************/
/* Matlab call: zdot = sys_l(neq,t,x,u); */
/*****************************************/
double l(neq,t,x,u)
     int neq[];
     double *t,x[],u[];
{
  double zdot;
  Matrix *ans[1];

  call_Matlab(neq,t,x,u,"sys_l",1,0,ans);
  zdot = mxGetScalar(ans[0]);
  mxFreeMatrix(ans[0]);
  return zdot;
}

/***************************************************/
/* Matlab call: [l_x,l_u,l_t] = sys_Dl(neq,t,x,u); */
/***************************************************/
double Dl(neq,t,x,u,l_x,l_u)
     int neq[];
     double *t,x[],u[],l_x[],l_u[];
{
  int i;
  Matrix *ans[2];

  call_Matlab(neq,t,x,u,"sys_Dl",1,1,ans);
  for ( i = 0 ; i < nstates ; i++ )
    l_x[i] = mxGetPr(ans[0])[i];
  for ( i = 0 ; i < ninputs ; i++ )
    l_u[i] = mxGetPr(ans[1])[i];
  mxFreeMatrix(ans[1]); mxFreeMatrix(ans[0]);
  return 0.0;
}

/****************************************/
/* Matlab call: J = sys_g(neq,t,x0,xf); */
/****************************************/
double g(neq,t,x0,xf)
     int neq[];
     double *t,x0[],xf[];
{
  double temp;
  Matrix *ans[1];

  call_Matlab(neq,t,x0,xf,"sys_g",2,0,ans);

  temp = mxGetScalar(ans[0]);
  mxFreeMatrix(ans[0]);
  return temp;
}

/*******************************************************/
/* Matlab call: [J_x0,J_xf,J_t] = sys_Dg(neq,t,x0,xf); */
/*******************************************************/
double Dg(neq,t,x0,xf,J_x0,J_xf)
     int neq[]; 
     double *t,x0[],xf[],J_x0[],J_xf[];
{
  int i;
  double temp;
  Matrix *ans[3];

  call_Matlab(neq,t,x0,xf,"sys_Dg",2,1,ans);
  for ( i = 0 ; i < nstates ; i++ ) {
    J_x0[i] = mxGetPr(ans[0])[i];
    J_xf[i] = mxGetPr(ans[1])[i];
  }
  temp = mxGetScalar(ans[2]);
  mxFreeMatrix(ans[2]); mxFreeMatrix(ans[1]); mxFreeMatrix(ans[0]);
  return temp;
}
