/************************************************************/
/* CALLING EXAMPLE:                                         */
/* >> N = 100;                                              */
/* >> T = 2.9;                                              */
/* >> t = [0:T/N:T];                                        */
/* >> u0 = zeros(1,N+1);                                    */
/* >> x0 = [1 1]';                                          */
/* >> [u,x,f]=riots(x0,u0,t,-1,1,[],[100;1e-3],3);          */
/* or,                                                      */
/* >> FLAGS = 1;                                            */
/* >> clear simulate;                                       */
/* >> C1 = 1; C2 = 1;                                       */
/* >> [u,x,f]=riots([x0;0],u0,t,-1,1,[C1;C2],[200;1e-3],3); */
/* or,                                                      */
/* >> FLAGS = 2;                                            */
/* >> clear simulate;                                       */
/* >> C1 = 100; C2 = 100;                                   */
/* >> [u,x,f]=riots([x0;0],u0,t,-1,1,[C1;C2],[100;1e-3],3); */
/*                                                          */
/* To plot the phase space trajectories and constraints:    */
/* >> tt=[0:2*pi/100:2*pi];                                 */
/* >> plot(x(1,:),x(2,:),sin(tt)/3+1,cos(tt)*0.3+0.4,':');  */
/* >> axis('equal')                                         */
/************************************************************/
/* PROBLEM DESCRIPTION:                                     */
/* Nonlinear problem with two trajectory constraints.

   Taken from "Clipping-off gradient algorithms to compute optimal
   controls with constrained magnitude," Quintana and Davison, Int. 
   J. Control, 1974, v. 20, no. 2, pp. 243-255 (example 3).

   x1_dot = x2                        ; x1(0) = 1
   x2_dot = u - 0.1*(1 + 2 x1^2 ) x2  ; x2(0) = 1

   t = [0:2.9/100:2.9] ; u0(t) = 0.

   Minimize 5*x1(2.9)^2 + x2(2.9)^2
   
   subject to: | u(t) | <= 1
   
               phi1(t) = 1 - 9*(x1(t) - 1)^2 - [(x2(t) - 0.4)/0.3]^2 <= 0
	       phi2(t) = -0.8 - x2(t) <= 0 

   The optimal control has three switching times: 0.418, 1.21, and 2.25.

   If FLAGS = 1, then the trajectory constraints are converted into 
   endpoint constraints using penalty functions, with penalty weights
   c1 and c2 (given as system parameters), by adding a extra state
   variable:

       x3_dot(t) = C1*max(0,phi1(t))^2 + C2*max(0,phi2(t))^2  ; x3(0) = 0;
   
   and replacing the trajectory constraints with the endpoint constraint
   
       x3(2.9) = 0;

   If FLAGS = 2, then the trajectory constraints are dealt with by
   by adding x3(2.9) to the objective function as a penalty function.
   In this case, you'll need C1 and C2 to be at least 100.

   I believe this problem is a singular control prblem.  When solving
   with FLAGS = 0, it is a good idea to set 'var = 1e-3'.

*/

#define NSTATES 2
#define MAX_NSTATES 3
#define NINPUTS 1
#define NLTC 1
#define LTC 1
#define FNUM neq[4]

/*
   Method = 0: Use trajectory constraints.
   Method = 1: Convert trajectory constraints into endpoint constraints.
   Method = 2: Use penalty function method.
*/
extern int get_flags();
static int Method;
static double C1,C2;

void activate(message)
     char **message;
{
  int n,flags[1];

  *message = "System 5";
  n = 1;
  if ( get_flags(flags,&n) == 0 || flags[0] == 0 ) {
    Method = 0;
    printf("Trajectory constraints handled directly.\n");
  }
  else {
    switch (flags[0]) {
    case 1:
      Method = 1;
      printf("Trajectory constraints via integral penalty functions.\n");
      break;
    case 2:
      Method = 2;
      printf("Trajectory constraints via integral penalty functions.\n");
      break;
    default:
      Method = 0;
      printf("Must have FLAGS = 0 , 1 , or 2.  Using direct method.\n");
      break;
    }
  }
} 

void init(neq,params)
     int neq[];
     double *params;
{
  if ( params == 0 ) {
    neq[1] = NINPUTS;
    switch (Method) {
    case 0:
      neq[0] = NSTATES;
      neq[6] = NLTC;		       /* One nonlinear trajectory constr. */
      neq[7] = LTC;		       /* One linear trajectory constr. */
      break;
    case 1:
      neq[0] = NSTATES+1;              /* There's an extra state variable. */
      neq[2] = 2;		       /* Two parameters, C1 and C2. */
      neq[10] = 1;                     /* One endpoint constraint. */
      break;
    case 2:
      neq[0] = NSTATES+1;              /* There's an extra state variable. */
      neq[2] = 2;		       /* Two parameters, C1 and C2. */
      break;
    }
  }
  else {
    if (Method) {
      C1 = params[0];
      C2 = params[1];
    }
  }
}

void h(neq,t,x,u,xdot)
     int neq[];
     double *t,x[],u[],xdot[];
{
  double temp,temp1,temp2;

  xdot[0] = x[1];
  xdot[1] = u[0] - 0.1*(1.0+2*x[0]*x[0])*x[0];

  if (Method) {
    temp1 = x[0] - 1.0; temp2 = (x[1] - 0.4)/0.3;
    temp = 1.0 - 9*temp1*temp1 - temp2*temp2;
    temp1 = (temp <= 0.0 ) ? 0.0 : C1*temp*temp;

    temp = -0.8 - x[1];
    temp2 = ( temp <= 0.0 ) ? 0.0 : C2*temp*temp;

    xdot[2] = temp1+temp2;
  }
}

void Dh(neq,t,x,u,h_x,h_u)
     int neq[];
     double *t,x[],u[];
     double h_x[],h_u[][NINPUTS];  /* NOTE: I can't define h_x[] as a double */
                                   /* array since I don't know in advance    */
                                   /* how many columns it has (2 or 3).      */
{
  double temp,temp1,temp2;

  h_u[1][0] = 1.0;

#define H1(a,b) h_x[2*(a)+(b)]
#define H2(a,b) h_x[3*(a)+(b)]

  if (!Method) {
    /* h_x is a 2x2 array. */
    H1(0,1) = 1.0;
    H1(1,0) = -0.1*(1.0+6*x[0]*x[0]);
  }
  else {
    /* h_x is a 3x3 array. */
    H2(0,1) = 1.0;
    H2(1,0) = -0.1*(1.0+6*x[0]*x[0]);
    
    temp1 = x[0] - 1.0; temp2 = (x[1] - 0.4)/0.3;
    temp = 1.0 - 9*temp1*temp1 - temp2*temp2;
    if ( temp > 0.0 ) {
      H2(2,0) = 2*C1*temp*( -18*(x[0]-1) );
      H2(2,1) = 2*C1*temp*( -2/0.09*(x[1]-0.4) );
    }
    else {
      H2(2,0) = 0.0;
      H2(2,1) = 0.0;
    }

    temp = -0.8 - x[1];
    if ( temp > 0.0 )
      H2(2,1) -= 2*C2*temp;
  }
}

double l(neq,t,x,u)
     int neq[];
     double *t,x[],u[];
{
  double temp1,temp2;

  switch (FNUM) {
  case 1:
    neq[3] = -1;
    return 0.0;
    break;
  case 2:
    temp1 = x[0] - 1.0;  temp2 = (x[1] - 0.4)/0.3;
    return 1.0 - 9*temp1*temp1 - temp2*temp2;
    break;
  case 3:
    return -0.8 - x[1];
    break;
  }
}

double Dl(neq,t,x,u,l_x,l_u)
     int neq[];
     double *t,x[],u[],l_x[],l_u[];
{
  switch (FNUM) {
  case 2:
    l_x[0] = -18*(x[0]-1);
    l_x[1] = -(2/0.09)*(x[1]-0.4);
    break;
  case 3:
    l_x[1] = -1.0;
    break;
  }
  return 0.0;
}

double g(neq,t,x0,xf)
     int neq[];
     double *t,x0[],xf[];
{
  switch (FNUM) { 
  case 1:
    if (Method < 2)
      return 5*xf[0]*xf[0] + xf[1]*xf[1];
    else
      return 5*xf[0]*xf[0] + xf[1]*xf[1]  +  xf[2];
    break;
  case 2:
    return xf[2];
    break;
  }
}

double Dg(neq,t,x0,xf,J_x0,J_xf)
     int neq[]; 
     double *t,x0[],xf[];
     double J_x0[],J_xf[];
{
  switch (FNUM) {
  case 1:
    J_xf[0] = 10*xf[0];
    J_xf[1] = 2*xf[1];
    if (Method == 2) J_xf[2] = 1.0;
    break;
  case 2:
    J_xf[2] = 1.0;
    break;
  }
  return 0.0;
}
