/***********************************************************/
/* CALLING EXAMPLE:                                        */
/* >> N = 40;                                              */
/* >> t = [0:1/N:1];                                       */
/* >> u0 = zeros(1,N+1);                                   */
/* >> x0 = [0 1]';                                         */
/* >> L = 1/9;                                             */
/* >> [u,x,f]=riots(x0,u0,t,[],[],L,100,4);                */
/***********************************************************/
/* PROBLEM DESCRIPTION:                                    */
/* Stryk, "Numerical Solution of Optimal Control Problems by Direct 
   Collocation", in Int. Series of Numerical Mathematics, Vol. 111, 
   pp. 129-143.  Taken from Bryson & Ho, "Applied Optimal Control," 
   pp 120-123 (Example 2).  This problem has a non-smooth optimal
   control even though there are no control bounds.

   This problem has two switching times at which the control is non-smooth
   and the adjoint variable is discontinuous.

   x_dot = v        x(0)=0, x(1)=0;
   v_dot = u        v(0)=1, v(1)=-1;

   Minimize int_0^1 u^2/2 dt
   subject to g(x) = x(t) - L <= 0.  (L = 1/9)

   This is a second-order state constraint.

   For kicks, you can set an upper bound on u(t).  The tightest upper
   bound that will admit a solution is about u(t) <= -0.85055.
*/

#define NSTATES 2
#define NINPUTS 1
#define NPARAMS 1
#define NFUNCTIONS 1
#define NLTC  0            /* Nonlinear trajectory inequality constraints. */

#define LTC   1            /* Linear trajectory inequality constraints.    */
#define LEEC  2            /* Linear endpoint equality constraints.        */

#define NLEEC 0            /* Nonlinear endpoint equality constraints.     */
#define NLEIC 0            /* Nonlinear endpoint inequality constraints.   */
#define LEIC  0            /* Linear endpoint inequality constraints.      */
#define LTI   4            /* 1 => Linear, 2 => LTI, 3 => LQ, 4 => LQTI    */

#define FNUM neq[4]

static double L;

void activate(message)
     char **message;
{
  int i;

  *message = "Switch";
  return;
} 

void init(neq,params)
     int neq[];
     double *params;
{
  int i;

  if ( params == 0 ) {
    neq[0] = NSTATES;
    neq[1] = NINPUTS;
    neq[2] = NPARAMS;
    neq[5] = NFUNCTIONS;
    neq[6] = NLTC;
    neq[7] = LTC;
    neq[8] = NLEIC;
    neq[9] = LEIC;
    neq[10] = NLEEC;
    neq[11] = LEEC;
    neq[12] = LTI;
  }
  else
    L = params[0];
  return;
}

void h(neq,t,x,u,xdot)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS],xdot[NSTATES];
{

  xdot[0] = x[1];
  xdot[1] = u[0];
}

void Dh(neq,t,x,u,h_x,h_u)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS];
     double h_x[NSTATES][NSTATES],h_u[NSTATES][NINPUTS];
{
  h_x[0][1] = 1.0;
  h_u[1][0] = 1.0;
}

double l(neq,t,x,u)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS];
{
  double zdot; 

  if (FNUM == 1)
    zdot = u[0]*u[0]/2;
  else
    zdot = x[0] - L;
  return zdot;
}

double Dl(neq,t,x,u,l_x,l_u)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS],l_x[NSTATES],l_u[NINPUTS];
{
  if (FNUM == 1)
    l_u[0] = u[0];
  if (FNUM == 2)
    l_x[0] = 1.0;
  return 0.0;
}

double g(neq,t,x0,xf)
     int neq[];
     double *t,x0[NSTATES],xf[NSTATES];
{
  switch (FNUM) {
  case 1:
    return 0.0;
    break;
  case 2: 
    return xf[0];
    break;
  case 3:
    return xf[1] + 1.0;
    break;
  }
}

double Dg(neq,t,x0,xf,J_x0,J_xf)
     int neq[]; 
     double *t,x0[NSTATES],xf[NSTATES],J_x0[NSTATES],J_xf[NSTATES];
{
  switch (FNUM) {
  case 1:
    break;
  case 2: 
    J_xf[0] = 1.0;
    break;
  case 3:
    J_xf[1] = 1.0;
    break;
  }
  return 0.0;
}
