function Malpha = transform(t,order)

% Malpha = transform(t,order)
% 
% This function computes the transform matrix Malpha for spline
% controls assuming repeated endpoint knots based on formula (A2.11b).
% Given any two splines with B-spline coefficients u and v, defined on
% the knot sequence augknt(t,order) , their inner-product as functions
% in L2 is equal to  trace(u*Malpha*v') . Uniform breakpoints sequences
% are handled much faster for order = 1,2,3,4.
% 
% This does work for repeated interior knots!
% 
% INPUTS:
% =======
% t      --- row vector of breakpoints.
% order  --- the order of the splines (e.g. 4 for cubic splines).
%
% The number of spline coefficients in each row of control inputs
% must equal to N-1+order where N is the number of elements in deltaT.
%
% OUTPUT:
% =======
% Malpha --- The transform matrix in sparse format.

%
% Copyright (c) 1996 by Adam L. Schwartz
% All right reserved.  
%

global old_DT old_order old_M 

if nargin < 2
  disp('Usage: M = transform(t,order)')
  return
end

deltaT = diff(t);
N = length(deltaT);
n = N+order-1;                   % dimension of the spline space.
d = order-1;                     % d is number of off-diagonals above diagonal.

% Check to see if Malpha has been previously computed.
if old_M ~= []
  if order == old_order & length(deltaT) == length(old_DT)
    if sum(abs(deltaT - old_DT)) < sqrt(eps)
      Malpha = old_M;
      return
    end
  end
end

if order == 1
  Malpha = spdiags(deltaT',0,N,N);
  old_M = Malpha; old_DT = deltaT; old_order = order;
  return
end

if (max(deltaT)+eps)/(min(deltaT)+eps) > 1+sqrt(eps) | order > 4
  UNIFORM = 0;
else
  UNIFORM = 1;
end

if UNIFORM
  DT = max(deltaT);
  if order == 2
    A = spdiags(ones(n,1)*[4 1 1],[0 1 -1],n,n);
    A(1,1)=2;
    A(n,n)=2;
    Malpha=A*DT/6;
  elseif order == 3
    A = spdiags(ones(n,1)*[66 26 26 1 1],[0 1 -1 2 -2],n,n);
    A(1,1) = 24; A(n,n) = 24;
    A(2,2) = 40; A(n-1,n-1) = 40;
    A(1,2) = 14; A(n-1,n) = 14;
    A(2,1) = 14; A(n,n-1) = 14;
    A(1,3) = 2; A(n,n-2) = 2;
    A(3,1) = 2; A(n-2,n) = 2;
    A(2,3) = 25; A(n-2,n-1) = 25;
    A(3,2) = 25; A(n-1,n-2) = 25;
    Malpha = A*DT/120;
  elseif order == 4
    d1 = [720 1116 1647 ones(1,n-6)*2416 1647 1116 720]';
    d2 = [441 787.5 1132 ones(1,n-7)*1191 1132 787.5 441]';
    d3 = [93 174 119.5 ones(1,n-8)*120 119.5 174 93]';
    d4 = [6 1.5 ones(1,n-7) 1.5 6]';
    A = spdiags([d1 [d2;0] [0;d2] [d3;0;0] [0;0;d3] [d4;0;0;0] [0;0;0;d4]], ...
                [0 -1 1 -2 2 -3 3],n,n);
    Malpha = A*DT/5040;
  end

else
  
  % --- Compute transform matrix S for Representation R1 on nonuniform mesh.
  % --- Use this to compute the spline transform matrix.
  % --- This is based on formula (A2.11b).

  C = 0:1/(order-1):1;
  Q = ones(order);	
  for i = 1:order
    Q(i,:) = C.^(i-1);
  end
  Q = inv(Q);
  M = Q*hilb(order)*Q';

  S = kron(spdiags(deltaT',0,N,N),M);

  %
  % --- Create PHI matrix.  (This should be written in C).
  %
  %  PHI = sparse([],[],[],N*order,n,2*order*n);
  %  Allocate PHI this way instead in case there are repeated interior knots.
  PHI = zeros(N*order,n);
  knots = augknt(t,order);
  tau = kron(t(1:N),ones(1,order)) + kron(deltaT,C);

  i = -(order-1)*order;
  j = order+1;
  dorder = order*(order-1);
  for col = 1:n
    % phi is a Bspline which is non-zero from indices start to stop.
%     phi = bspline(knots(col:col+order));
% --- Create phi using code from bspline.m. (bspline does plotting).
% ------------------------------------------------------------------
    t = knots(col:col+order);
    k = length(t)-1;
    adds=ones(1,k-1);
    tt=[adds*t(1) t(:)' adds*t(k+1)];
    a=[adds*0 1 adds*0]; 

    %  From this, generate the pp description
    inter=find( diff(tt)>0 );l=length(inter);
    tx=ones(l,1)*[2-k:k-1]+inter'*ones(1,2*(k-1));tx(:)=tt(tx);
    tx=tx-tt(inter)'*ones(1,2*(k-1));
    b=ones(l,1)*[1-k:0]+inter'*ones(1,k);b(:)=a(b);
    c=sprpp(tx,b);x=[tt(inter) tt(2*k)];
    phi=ppmak(x,c,1);
% ------------------------------------------------------------------
    %
    %  Evaluate phi(t) at points in tau where it is non-zero.
    %  Store these values in pval.
    %
    start = max(i,1);
    stop  = min(j,N*order);
    pval = fnval(phi,tau(start:stop))';
    % 
    % The next two sequences of  if  statment are needed to zero out
    % values of pval that occur on the wrong side of a discontinuity
    % when there is a repeated interior knot of multplicity order.
    %
    if start+dorder < N*order
      if tau(start)==tau(start+dorder), pval(1) = 0; end
    end
    if stop-dorder > 1
      if tau(stop) == tau(stop-dorder), pval(stop-start+1) = 0; end
    end
    PHI(start:stop,col) = pval;
    i = i+order; j = j+order;
  end
  PHI = sparse(PHI);

  Malpha = PHI'*S*PHI;
  
end

old_M = Malpha; old_DT = deltaT; old_order = order;
