/******************************************/
/*                                        */
/* Copyright (c) 1996 by Adam L. Schwartz */
/*          All right reserved.           */
/*                                        */
/***********************************************************************/
/* utility.c is a collection of the following minor subroutines:       */
/*                                                                     */
/* void multiply(n,m,p,X,Y,Z)                                          */
/* void zero_out(M,n)                                                  */
/* void *mallocate(size,number)                                        */
/* void *callocate(rows,size,number)                                   */
/* void transpose(A,B,n,m)                                             */
/* void bsplvb(Order,t,x,biatx)                                        */
/* void time_fnc(t,index,flag,result)                                  */
/* void get_flags(flags,n)                                             */ 
/***********************************************************************/
#include <stdio.h>
#include "mex.h"

#ifndef NULL
#define NULL 0
#endif

/* Z = X*Y where X is nxm and Y is mxp */
void multiply(n,m,p,X,Y,Z)
int n,m,p;
double *X,*Y,*Z;
{
  register int i,j,k;
  double temp,*x,*y;

  for ( i = 0 ; i < n ; i++ ) {
    for ( j = 0 ; j < p ; j++ ) {
      x = X+i*m; y = Y+j;
      temp = 0.0;
      for ( k = 0 ; k < m ; k++,x++,y+=p ) 
	temp += (*x)*(*y);
      *(Z++) = temp;
    }
  }
}

/* Zeros out an array.  For an mxn matrix call zero_out(M,m*n). */
void zero_out(M,n)
int n;
double M[];
{
  int i;
  for ( i = 0 ; i < n ; i++ )
    *(M++) = 0.0;
  return;
}

void *mallocate(size,number)
int number;
unsigned int size;
{
  static char str[20];
  void *temp;

  temp = (void *)malloc(size);
  if ( temp == NULL ) {
    sprintf(str,"Couldn't allocate memory for %d.",number);
    mexErrMsgTxt(str);
  }

  return temp;
}


/***WARNING*** Even though calloc initially stores zeros in the
 *  memory which it allocates, variables that are static will not
 *  be reinitialized to zero when simulate is re-invoked. 
 */
void *callocate(rows,size,number)
int number;
unsigned int rows,size;
{
  static char str[20];
  void *temp;

  temp = (void *)calloc(rows,size);
  if ( temp == NULL ) {
    sprintf(str,"Couldn't allocate memory for %d.",number);
    mexErrMsgTxt(str);
  }

  return temp;
}

/* B = A' where A is n by m and B is m by n. */
void transpose(A,B,n,m)
     int n,m;
     double *A,*B;
{
  int row,column;
  double *b;

  for ( row=0 ; row<n ; row++ )
    for ( b=B+row,column=0 ; column<m ; column++,b+=n ) 
      *b = *(A++);
}


void bsplvb(Order,t,x,biatx)
     int Order;
     double *t,x,*biatx;
/***************************************************************************/
/*  from  * a practical guide to splines *  by c. de boor p. 134           */
/*  calculates the value of all possibly nonzero b-splines at x of order   */
/*  Order with knot sequence  t .   The input t is actually a pointer to   */
/*  the left-most knot for any B-splines that are non-zero at x.  So in    */
/*  the following description the knot sequence only refers to the knots   */
/*  t[0] through t[5].                                                     */
/***************************************************************************/
/* Converted from  bsplvb.f  by  f2c  and then cleaned up by A. Schwartz.  */
/* I've also modified things a bit.  Since I will only be dealing with     */
/* at most four basis functions at a time, I:                              */
/*          Got rid of the input argument 'left' which is now always Order */
/*          Got rid of 'index' which is now always = 1;                    */
/*          Got rid of 'jhigh' which is now always = Order.                */
/*                                                                         */
/* Here's a description of how calls to this subrouting work (in the       */
/* following, all indices start from 0, not 1 and Order = CUBIC = 4):      */
/* Let Y[] be the array of mesh points.  If you want to calculate the      */
/*   value of all of the non-zero B-splines at a point x such that         */
/* Y[i] <= x <= Y[i+1],  then set t = {Y[i-3] ,...., Y[i+4]} --- note that */
/* between the first three and last three mesh points, the vector t has to */
/* be padded with Y[0] on the left or Y[N] on the right, respectively.     */
/* The output vector biatx[] will contain the value, at x, of the B-spline */
/* basis functions with index {i,i+1,i+2,i+3}.                             */
/*                                                                         */
/* EXAMPLE: Y[] = {0,1,2,3,4,5} (N = 5, dimension of spline = 8)           */
/*   To compute           knot sequence         index of active B-splines  */
/* --------------     ---------------------     -------------------------  */
/*  x  in  [0,1]      t = [0 0 0 0 1 2 3 4]             0 , 1 , 2 , 3      */
/*  x  in  [1,2]      t = [0 0 0 1 2 3 4 5]             1 , 2 , 3 , 4      */
/*  x  in  [2,3]      t = [0 0 1 2 3 4 5 5]             2 , 3 , 4 , 5      */
/*  x  in  [3,4]      t = [0 1 2 3 4 5 5 5]             3 , 4 , 5 , 6      */
/*  x  in  [4,5]      t = [1 2 3 4 5 5 5 5]             4 , 5 , 6 , 7      */
/***************************************************************************/
/* ******  i n p u t  ******                                               */
/*  Order.....order of splines.                                            */
/*  t.....knot sequence, of length  left + jout  , assumed to be nonde-    */
/*        creasing.  a s s u m p t i o n . . . .                           */
/*                       t(left)  .lt.  t(left + 1)   .                    */
/*   d i v i s i o n  b y  z e r o  will result if  t(left) = t(left+1)    */
/*  x.....the point at which the b-splines are to be evaluated.            */
/* ******  o u t p u t  ******                                             */
/*  biatx.....array of length  jout , with  biatx(i)  containing the val-  */
/*        ue at  x  of the polynomial of order  jout  which agrees with    */
/*        the b-spline  b(left-jout+i,jout,t)  on the interval (t(left),   */
/*        t(left+1)) .                                                     */
/*                                                                         */
/* ******  m e t h o d  ******                                             */
/*  the recurrence relation                                                */
/*                                                                         */
/*                       x - t(i)              t(i+j+1) - x                */
/*     b(i,j+1)(x)  =  -----------b(i,j)(x) + ---------------b(i+1,j)(x)   */
/*                     t(i+j)-t(i)            t(i+j+1)-t(i+1)              */
/*                                                                         */
/*  is used (repeatedly) to generate the (j+1)-vector  b(left-j,j+1)(x),   */
/*  ...,b(left,j+1)(x)  from the j-vector  b(left-j+1,j)(x),...,           */
/*  b(left,j)(x), storing the new values in  biatx  over the old. the      */
/*  facts that                                                             */
/*            b(i,1) = 1  if  t(i) .le. x .lt. t(i+1)                      */
/*  and that                                                               */
/*            b(i,j)(x) = 0  unless  t(i) .le. x .lt. t(i+j)               */
/*  are used. the particular organization of the calculations follows al-  */
/*  gorithm  (8)  in chapter x of the text.                                */
/*                                                                         */
/*     double biatx(jhigh),t(1),x,   deltal(20),deltar(20),saved,term      */
/*     dimension biatx(jout), t(left+jout)                                 */
/***************************************************************************/
{
  static int i,j;
  static double term, saved, deltal[20], deltar[20];
  
  --biatx;
  --t;
  
  biatx[1] = 1.0;
  for ( j = 1 ; j < Order ; j++ ) {
    deltar[j - 1] = t[Order + j] - x;
    deltal[j - 1] = x - t[Order + 1 - j];
    saved = 0.0;
    for (i = 1 ; i <= j ; ++i) {
      term = biatx[i] / (deltar[i - 1] + deltal[j - i]);
      biatx[i] = saved + deltar[i - 1] * term;
      saved = deltal[j - i] * term;
    }
    biatx[j+1] = saved;
  }
}

/***************************************************************/
/* This function allows a user's C program to call a MATLAB    */
/* m-file named "sys_time_fnc" to compute a function of time.  */
/* The m-file should look like:                                */
/*                                                             */
/* function f = sys_time_fnc(t)  % t = [time;index;flag]       */
/* f = [compute something here]                                */
/*                                                             */
/* Note: there is no error checking.  In particular, result    */
/* must point to an array of doubles with enough memory to     */
/* store everything returned by sys_time_fnc.m.                */
/***************************************************************/
void time_fnc(t,index,flag,result)
     int *index,*flag;
     double *t,*result;
{
  int i;
  Matrix *lhs,*rhs;

  rhs = mxCreateFull(3,1,NULL);
  mxGetPr(rhs)[0] = *t;
  mxGetPr(rhs)[1] = (double) (*index + 1);
  mxGetPr(rhs)[2] = (double) *flag;

  mexCallMATLAB(1,&lhs,1,&rhs,"sys_time_fnc");

  for ( i = 0 ; i < mxGetN(lhs)*mxGetM(lhs) ; i++ )
    result[i] = mxGetPr(lhs)[i];

  mxFreeMatrix(rhs); mxFreeMatrix(lhs);
}

/***************************************************************/
/* This function allows a user's C program to call get a       */
/* vector of flags called FLAGS from Matlab's workspace.       */
/* FLAGS should be defined as a global variable.               */
/* The values in FLAGS should be integers.                     */
/* get_flags returns the number of FLAGS read in.              */
/* IMPORTANT: (1) It is up to the user to allocate at least *n */
/*                integer locations in the array flags[].      */
/*            (2) Any modifications to FLAGS will only be      */
/*                noticed if simulate is clear'd in Matlab.    */
/***************************************************************/
int get_flags(flags,n)
     int flags[],*n;
{
  char message[30];
  int i,nflags;
  Matrix *FLAGS;

  mexEvalString("global FLAGS");
  FLAGS = mexGetMatrix("FLAGS");

  if ( FLAGS != NULL ) {
    nflags = mxGetM(FLAGS)*mxGetN(FLAGS);
    if ( *n > nflags ) *n = nflags;
    for ( i = 0 ; i < *n ; i++ )
      flags[i] = (int)mxGetPr(FLAGS)[i];
    if ( mxGetPr(FLAGS) != NULL ) mxFreeMatrix(FLAGS);
  }
  else {
    mexPrintf("Warning: FLAGS not defined.  Define FLAGS as a global.\n");
    *n = 0;
  }
  if (*n == 1)
    sprintf(message,"Loaded 1 flag.\n");
  else
    sprintf(message,"Loaded %d flags.\n",*n);
  mexPrintf(message);
  return *n;
}
