%  function BUILDVAR
%  Builds a mu-tools VARYING matrix given a MIMO transfer function
%
%   Written March 10, 1992
%
%   This function allows one to enter a given MIMO transfer function
%   matrix directly as a mu-tools VARYING matrix.  This bypasses the
%   necessity of finding a state space realization of a system first
%   and then calculating the frequency response.  The coefficients
%   of the numerator and denominator polynomials are entered in
%   matrix form along with a vector containing the dimensions and
%   a frequency vector.  The output is a VARYING matrix for use with
%   the Mu-tools Toolbox.  Requires the m-file 'fresponse'.
%
%   Written by Jesse Leitner
%   PhD Student/Graduate Research Assistant/Teaching Assistant
%   School of Aerospace Engineering
%   Georgia Tech
%   Atlanta, Georgia  30332-0150
%   Phone (404) 894-2995
%   Email:  leitner@iscl1.gatech.edu
%
function sys = buildvar(num,den,dimens,omega)
%function sys = buildvar(num,den,dimens,omega)
%
%Puts a known MIMO transfer function matrix (tfm) into the form
%of a frequency VARYING MATRIX.
%Requires the m-file 'fresponse.'  To be used with mu-tools.
%
%num = matrix of numerator polynomials
%den = matrix of denominator polynomials
%dimens= [rows of tfm  cols of tfm]
%omega=frequency vector
%
% The num and den elements come from transfer function
% matrices of the following form:
%
%          |    num1/den1  num2/den2  num3/den3  num4/den4  |
%          |                                                |
%    |    num5/den5
 num6/den6  ...             |
%          |                                                |
%          |    num9/den9  ...                              |
%          |                                                |
%          |    ...                                         |
%
% The num and den matrices are formatted in the following
% way:
%
% num=[1 3 3 2          | coefficients of num1
%      0 1 2 1          | "             " num2
%      0 0 0 0          | "             " num3
%      1 4 2 1];        | "             " num4
%
% So each numerator or denominator polynomial must be
% formatted to have the same number of elements.  This
% can be done by adding zeros in front or multiplying by
% a special form of one, e.g. [0 1], [0 0 1], etc.
% For zero elements, enter the appropriate form of
% 'one' as described above for the denominator polynomial
% and a row of zeros for the numerator polynomial.
%
jay=sqrt(-1);
npts=length(omega);
nrows=dimens(1,1);
ncols=dimens(1,2);
for n=1:nrows*ncols
        if num(n,:)==0,
                comp(:,n)=zeros(npts,1);
                n=n+1;
        else comp(:,n)=fresponse(num(n,:),den(n,:),omega);
        end
end
for k=1:npts
        for ii=1:nrows
                for jj=1:ncols
                        i2=(k-1)*nrows+ii;
                        j1=(ii-1)*ncols+jj;
                        sys(i2,jj)=comp(k,j1);
                end
        end
end
sys(:,ncols+1)=[omega zeros((nrows-1)*npts,1)']';
sys(nrows*npts+1,:)=[zeros(1,ncols-1) npts inf];
