function [a,b,c,d] = c2deld(a,b,c,d,delay,delta);
%C2DELD     Conversion from a continuous time model with a time delay to a
%           discrete time model.
%           [DA,DB,DC,DD] = C2DELD(A,B,C,D,delay,delta)  converts from the
%           continuous system:
%                  .
%                  x = Ax + Bu
%                  y = Cx + Du
%
%           with a time delay of magnitude delay seconds on each of the
%           inputs, to the discrete time system:
%
%                 px = DAx + DBu
%                  y = DCx + DDu
%
%           using delta as the sampling period.
%
%           [DNUM,DDEN] = C2DELD(NUM,DEN,delay,delta)  does the same
%           as above for systems expressed in transfer function form.
%
%           Note that delta must be non-zero for this routine to make sense.

%           I.Kaspura
%           11/89


    nargs = nargin;
    if nargs == 4
        delay = c;
        delta = d;
        [a,b,c,d] = tf2ss(a,b);
        nargs = 6;
    end;

    error(nargchk(6,6,nargs));
    error(abcdchk(a,b,c,d));


    eta = rem(delay,delta);
    N = round((delay/delta)-eta);
    [ma,na] = size(a);
    [mb,nb]=size(b);
    if nb~=1
        error('This routine only works for single input systems');
    end

    f = s2del(a,delta);
    s=expm([[a,b]*(delta-eta);zeros(nb,na+nb)]);
    t=expm([[a,b]*delta;zeros(nb,na+nb)]);
    G0 = s(1:na,na+1:na+nb)/delta;
    G1 = t(1:na,na+1:na+nb)/delta - G0;


   if (eta>0&N~=0)
         C0=0*ones(1,N+1);
         C1=C0;
         C0(N)=1;
         C1(N+1)=1;
         Kdel=-eye(N+1)/delta+diag(ones(1,N)/delta,-1);
         Bdel=0*ones(N+1,1);Bdel(1)=1/delta;
         a=[f G0*C0+G1*C1;ones(N+1,na)*0 Kdel];
         b=[ones(na,1)*0 ; Bdel];
         c=[c d*C1];
         d=0*d;
    end

    if (eta==0&N~=0)
         C0=0*ones(1,N+1);
         C1=C0;
         C0(N)=1;
         C1(N+1)=1;
         Kdel=-eye(N+1)/delta+diag(ones(1,N)/delta,-1);
         Bdel=0*ones(N+1,1);Bdel(1)=1/delta;
         a=[f G0*C0+G1*C1;ones(N+1,na)*0 Kdel];
         b=[ones(na,1)*0 ; Bdel];
         c=[c d*C0];
         d=0*d;
    end

    if (eta>0&N==0)
         C0=0*ones(1,N+1);
         C1=C0;
         C1(N+1)=1;
         Kdel=-eye(N+1)/delta;
         Bdel=0*ones(N+1,1);Bdel(1)=1/delta;
         a=[f G1*C1;ones(N+1,na)*0 Kdel];
         b=[G0 ; Bdel];
         c=[c d*C1];
         d=0*d;
    end

   if (eta==0&N==0)
         C0=0*ones(1,N+1);
         C1=C0;
         C1(N+1)=1;
         Kdel=-eye(N+1)/delta;
         Bdel=0*ones(N+1,1);Bdel(1)=1/delta;
         a=[f G1*C1;ones(N+1,na)*0 Kdel];
         b=[G0 ; Bdel];
         c=[c d*C0];
    end


    if nargout == 2
        [a,b] = ss2tf(a,b,c,d,1);
    end





