function [mag,pha,w] = delbode(a,b,c,d,iu,delta)
%DELBODE   Calcuates the frequecy response of the discrete time system in
%          delta domain and plots the bode diagram.
%          [MAG,PHA,W] = DELBODE(A,B,C,D,iu,delta)
%
%          The function calculates the frequency response of the system
%                     px = Ax + Bu
%                      y = Cx + Du
%
%          from the iu'th input using delta as the sampling period.
%          DELBODE returns the matrices MAG and PHA (in degrees)
%          with as many rows as there are outputs and LENGTH(W) columns.
%          The frequency range required for the plot is automatically
%          calculated based upon the position of the systems poles
%          and the sampling frequency. Upon exit W conatins the points used
%          in the plot.
%
%          [MAG,PHA,W] = DELBODE(NUM,DEN,delta) calculates the frequency
%          response from the transfer function description
%          y(d)/u(d) = NUM(d)/DEN(d)  and plots it as a bode diagram.
%

%          I.Kaspura
%          11/89



nargs = nargin;
if nargs == 6
    [num,den] = ss2tf(a,b,c,d,iu);
    nargs = 3;
else
    num=a;
    den=b;
    delta=c;
end
error(nargchk(3,3,nargs));


%   Determine the frequency range required.
%   This is done by ensuring that the frequency range extends from
%   0.2 times the frequency of the lowest pole in the system, up to
%   6 times the frequency of the largest pole in the system.

%   Find the poles.
[z,p,k] = tf2zp(num,den);
num_poles = length(p);

wmax = 0;
wmin = 1e10;
for i = 1:num_poles
    wn = sqrt( (real(p(i)))^2 + (imag(p(i)))^2 );
    if wn > wmax
        wmax = wn;
    end
    if wn < wmin
        wmin = wn;
    end
end

wmax = log10(6*wmax);
wmin = log10(0.2*wmin);

%   If the frequency range selected is greater than the sampling frequency
%   reduce the range.
if delta~=0
    if wmax > log10(pi/delta*0.9)
        wmax = log10(pi/delta*0.9);
    end
end

%   Find the discrete time frequency response.
w = logspace(wmin,wmax);
h = delfreq(num,den,w,delta);
mag = abs(h);
db = 20*log10(mag);
phase = 180/pi*unwrap(imag(log(h)));

%   For each output of the system plot the magnitude and phase.
[no,ns] = size(num);
for i = 1:no
    clg;
    subplot(211);
    semilogx(w,db(:,i)); ylabel('Gain(db)'); grid;
    subplot(212);
    semilogx(w,phase(:,i)); ylabel('Phase(deg)'); xlabel('Frequency(rad)');
    grid;

    if no > 1
        fprintf(['\n Bode plot of input %1.0f .',..
                ' Press any key to continue..\n'],i);
        pause
    else
            pause
    end
end

