function h = delfreq(a,b,c,d,iu,w,delta)
%DELFREQ   Calcuates the frequency response of a discrete-time linear
%          system in the delta domain.
%              H = DELFREQ(A,B,C,D,iu,W,delta)  calculates the frequency
%          response of the system:
%                         px = Ax + Bu
%                          y = Cx + Du
%          using               -1
%                  H(d) = C(dI-A) B + D
%
%          That is, returns
%               H( (exp(jwdelta)-1)/delta )
%
%              The response is calulated from the iu'th input using delta as
%          the sampling period.  The vector W must contain the frequencies,
%          in radians, at which the response is to be evaluated. The elements
%          of W should not exceed the sampling frequency i.e.  w < 2pi/delta.
%          Upon exit the matrix H has as many columns as there are outputs
%              and LENGTH(W) rows.
%
%              H = DELFREQ(NUM,DEN,W,delta) calculates the response from the
%              transfer function description  H(d) = NUM(d)/DEN(d)  
%         where NUM and
%              DEN contain the polynomial coefficients in descending powers.


%          11/89
%              I.Kaspura


nargs = nargin;
if (nargs == 4)                          % If given the transfer function
        [m,n] = size(a);                 % convert to state space form.
        a = [a zeros(m,length(b)-n)];
        w = c;
        delta = d;
        [a,b,c,d] = tf2ss(a,b);
        nargs = 7;
        iu = 1;
end
error(nargchk(7,7,nargs));
error(abcdchk(a,b,c,d));

[no,ns] = size(c);           % Number of ouputs and number of states
nw = max(size(w));           % Number of frequency points

w = w*sqrt(-1);
w = s2del(w,delta);

[t,a] = balance(a);
b = t \ b;
c = c * t;

[p,a] = hess(a);                % Reduce A to Hessenberg form:

b = p' * b(:,iu);      % Apply similarity transformations from Hessenberg
c = c * p;             % reduction to B and C:
d = d(:,iu);
h = ltifr(a,b,w);
h = (c * h + diag(d) * ones(no,nw)).';

