function [p,l,q,alpha]=delhinf(a,b1,b2,c,delta)
%DELHINF   Solves the discrete time H-infinty game problem in delta domain.
%          [P,L,Q,alpha] = DELHINF(A,B1,B2,C,delta)
%          Given the system
%                  px = Ax + B1w + B2u
%                   z = Cx + Du
%          and the sampling period, delta, this routine determines the optimum
%          gain matrices L and Q so that
%                   u = -Lx
%                   w = Qx
%          minimises the cost funtion
%                   J(u,w) = ( norm(z)^2 - alpha^2*norm(w)^2 )/2
%
%          L and Q are given by
%                                                   -1
%             L = B2'(I-deltaP(B1B1'/alpha^2-B2B2'))  P(I+deltaA)
%                                                             -1
%             Q = (B1'/alpha^2)(I-deltaP(B1B1'/alpha^2-B2B2'))  P(I+deltaA)
%
%           where P is given by the Ricatti equation
%              0 = C'C + A'P + PA + deltaA'PA                     -1
%                     + (I+deltaA)'P(I-delta(B1B1'/alpha^2-B2B2')P)
%                         * (B1B1'/alpha^2-B2B2')P(I+deltaA)


%          I.Kaspura
%          11/89


error(nargchk(5,5,nargin));

% Find all unstable eigenvalues of A
ev=eig(a);
j=1;
for i=1:length(ev)
    if abs(ev(i))^2*delta/2 + real(ev(i)) >= 0
        unstev(j)=ev(i);
        j=j+1;
    end
end

% Check that all unstable modes are controllable and observable
for i=1:length(unstev)
    aa=a-unstev(i)*eye(a);
    for j=1:rank(a)
        stab(:,j)=aa^(j-1)*b2;
    end
    if rank(stab)~=rank(a)
        error('unstabilizable system')
    end
    for j=1:rank(a)
        detec(j,:)=c*aa^(j-1);
    end
    if rank(detec)~=rank(a)
        error('undetectable system')
    end
end


alpha=10*ceil(norm(b1)/norm(b2));
numits=1;
stop=0;

while (numits<31)&(stop==0)

    % Form associated hamiltonian matrix
    [ra,ca] = size(a);
    I = eye(ra,ca);

    m = [I,-delta*(b1*b1'/(alpha^2) - b2*b2');zeros(ra,ca),I+delta*a'];
    mcont = [a,b1*b1'/(alpha^2)-b2*b2';-c'*c,-a'];

    [v,d] = eig(mcont,m);
    d = diag(d);
    d = (delta/2)*abs(d).^2 + real(d);



    [d,index] = sort(real(d));

    chi = v(1:ca,index(1:ca));
    lambda = v((ca+1):(2*ca),index(1:ca));
    p = real(lambda/chi);

    % Check that P satisfies the Riccati equation
    t=(I+a*delta)'*p/(I-delta*(b1*b1'/(alpha^2)-b2*b2')*p);
    t=t*(b1*b1'/(alpha^2)-b2*b2')*p*(I+delta*a);
    t=t+c'*c+a'*p+p*a+delta*a'*p*a;
    if norm(t)<1e-10
        oldalpha=alpha;
        oldp=p;
        alpha=0.8*alpha;
        numits=numits+1;
    else
        stop=1;
    end

    % Check P is positive semi-definite
    if any(eig(p)<-eps)
        stop=2;
    end
end

p=oldp;
alpha=oldalpha;



% Get feedback gains
t = (I-delta*p*(b1*b1'/(alpha^2)-b2*b2'))\p*(I+delta*a);
l = b2'*t;
q = b1'*t/(alpha^2);





%if (~( (d(ca)<0) & (d(ca+1)>0) ))
%    error('Can''t order eigenvalues, (A,B) may be uncontrollable.')
%end
%check a+idelta invertible

