function [re,im,w] = delnyq(a,b,c,d,iu,delta)
%DELNYQ   Calculates the frequency response of a discrete time system
%         in delta domain and then plots the nyquist diagram.
%         [RE,IM,W] = DELNYQ(A,B,C,D,iu,delta)
%         This function finds the frequency response of the system
%
%                    px = Ax + Bu
%                     y = Cx + Du
%
%         from the iu'th input using delta as the sampling period.
%         Upon exit the vectors RE and IM contain the real and imaginary
%         components of the frequency response. The frequency range required
%         for the plot is automatically calculated based upon the position
%         of the systems poles and the sampling frequency. The response
%         is calculated and plotted at the points returned in W.
%
%         [RE,IM,W] = DELNYQ(NUM,DEN,delta)  calculates the frequency
%         response and plots the nyquist diagram using the discrete time
%         transfer function  y(d)/u(d) = NUM(d)/DEN(d).

%         I.Kaspura
%         11/89

nargs = nargin;
if nargs == 3                           % If in transfer function form
    delta = c;                          % convert it to state space.
    iu = 1;
    [a,b,c,d] = tf2ss(a,b);
    nargs = 6;
end
error(nargchk(6,6,nargs));


%   Determine the frequency range required.
%   This is done by ensuring that the frequency range extends from
%   0.2 times the frequency of the lowest pole in the system, up to
%   6 times the frequency of the largest pole in the system.

%   Find the poles.
[z,p,k] = ss2zp(a,b,c,d,1);
num_poles = length(p);

wmax = 0;
wmin = 1e10;
for i = 1:num_poles
    wn = sqrt( (real(p(i)))^2 + (imag(p(i)))^2 );
    if wn > wmax
        wmax = wn;
    end
    if wn < wmin
        wmin = wn;
    end
end

wmax = log10(6*wmax);
wmin = log10(0.2*wmin);

%   If the frequency range selected is greater than the sampling frequency
%   reduce the range.
if delta~=0
    if wmax > log10(pi/delta*0.9)
        wmax = log10(pi/delta*0.9);
    end
end
w = logspace(wmin,wmax,300);

h=delfreq(a,b,c,d,iu,w,delta);
re=real(h);
im=imag(h);

clg
axis('square');
plot(re,im);
title('Nyquist Plot');
xlabel('Real(H)');
ylabel('Imag(H)');
grid;
pause
axis('normal');
