function  [y,x] = delsim(a,b,c,d,u,delta,x0)
%DELSIM   Simulation of discrete-time linear systems in delta domain.
%         [Y,X] = DELSIM(A,B,C,D,U,delta,X0)
%
%         This function solves the linear discrete time delta state space model:
%
%                    px =  Ax  +  Bu
%                     y =  Cx  +  Du
%
%         Matrix U must have as many rows as there are inputs and each column
%         of U must correspond to a new time point. X0 may contain the initial
%         conditions if they are non-zero but is optional. Delta is the
%         sampling period. The points in U must be spaced delta apart.
%         DELSIM returns a matrix Y with as many rows as there are outputs
%         and with LENGTH(U) columns. Upon exit X contains the state time
%         history. X has as many rows as there are states and LENGTH(U) columns
%
%             [Y,X] = DELSIM(NUM,DEN,U,delta,X0) 
%         calculates the time response from
%         the transfer function description  G(d) = NUM(d)/DEN(d)  where NUM
%         and DEN contain the polynomial coefficients in descending powers.

%         I.Kaspura
%         11/89.


    nargs = nargin;
    if nargs == 5
        x0 = u;
        nargs = 4;
    end
    if nargs == 4
        u = c;
        delta = d;
        [a,b,c,d] = tf2ss(a,b);
        nargs = 6;
    end
    error(abcdchk(a,b,c,d));
    error(nargchk(6,7,nargs));


    [ninp,npts] = size(u);
    [nsta,msta] = size(a);
    if nargs == 6
        x0 = zeros(nsta,1);
    end

    for i=1:npts
        x(:,i) = x0;
        x0 = x0 + a*delta*x0 + b*delta*u(:,i);
    end

    y = c*x + d*u;


