function  F=demevas(E,A,B,l,form,write,nn);
%
%  function F=demevas(E,A,B,l,form,write,nn);
%
%   This program first transforms a system  (E,A,B)  to its controllability
% form  (if it is not already in this form), using the  QR  decomposition
% with column pivoting when possible. Then it allocates as many eigenvalues
% as possible from  l, to the pencil  (A-B*F,E)  by computing  F.
%   The algorithm allocates one eigenvalue at a time, hence complex
% eigenvalues will be allocated by using complex arithmetic. This may lead
% to a complex  F. Investigation, however, is under way to allocate two
% eigenvalues at a time, thus complex conjugate pairs may be allocated in
% one double step using only real arithmetic. The program is based on an
% algorithm by George Miminis submitted to the  IEEE TRANSACTIONS ON
% AUTOMATIC CONTROL.
%
% INPUT:  (E,A,B)  nxn, nxn, nxm, matrices respectively.
%         l        the set of eigenvalues to be allocated.
%         form     a logical variable which is  1  if  (E,A,B)  is in
%                  its controllability form, and  0  if not.
%         write    a logical variable. If it is  1  then along with possible
%                  messages, the current forms of  B,  E,  and  A,  will be
%                  displayed if relevant.
%         nn       (k+1)-dimensional vector, contains the  k  controllability
%                  indeses, with nn(k+1)=0. k  is determined by the data,
%                  and in the case of a completely controllable system it is
%                  the controllability index of the system. nn  should not
%                  be specifyed if form=0.
%
% OUTPUT: F        the feedback matrix that accomplishes the allocation.
%
%  For its proper execution  DEMEVAS.m  needs the  m.functions  ROTAPL,
%  ROT,  BACKSB,  QRTRANS.
%
%
%      Written by  George S. Miminis
%                  Department of Computer Science
%                  Memorial University of Newfoundland
%                  St. John's, Newfoundland, CANADA,
%                  A1C 5S7.
%             tel: (709)737-8635
%          e-mail: george@garfield.cs.mun.ca
%                  na.miminis@na-net.ornl.gov
%
%
[n m]=size(B);
nc=n;         % The size of the system considered controllable to this point.
etol=norm([E,B,A],1);               %  Compute a reasonable ``relative
alpha=5/7;                            %  magnitude''  (etol), and a reasonable
tol=(m+n)*(etol*eps)^alpha;          %  tolerance  (tol).
if ~form,           % Compute the controllability form.
%                   % We use no indenting in this  IF.
%
%           Downward Part.
%
[U,B,W]=qr(B);
[V,E]=qrtrans(U'*E,0);
V=V';
A=U'*A*V;
nn(1)=0;                %
s=svd(B);               %
len=length(s);          %
rnkflag=1;              %
i=1;                    %
while rnkflag & i<=len, %
      if s(i)>tol,      %    nn(1)=rank(B).
         nn(1)=nn(1)+1; %
         i=i+1;         %
      else              %
         rnkflag=0;     %
      end;              %
end;                    %
count=0;   %  The number of infinite eigenvalues that will be deflated.
mm=nc;      %  The nonzero part of  B  does not go beyond the  mm  row.
message=1;   % Display message, due to possible singularity of  E.
singular=0;   % "singular" refers to the singularity of  E.
i=nc;
flag=1;                        % "flag" turns  0  when a zero is found
while flag & (i > nn(1)),      % on any diagonal position of  E  after
      if abs(E(i,i)) <= tol,   % the  nn(1)  position.
         flag=0;
         singular=1;
      end;
      i=i-1;
end;
while singular,      % There is at least a zero on the diagonal of  E
      count=count+1; % below the  nn(1)  position.
      if message,
         disp(' ')
         disp('The system is uncontrollable within '), tol,
         disp('with respect to the "1-norm". This is due to at least')
         disp('one UNCONTROLLABLE INFINITE EIGENVALUE.')
         disp('Press any key to deflate all the uncontrollable infinite')
         disp('eigenvalues, or  Ctrl C  to terminate the process.')
         disp(' ')
         pause
         if write,
            E,
            disp('Push any key to see  B.')
            pause
            B,
            disp('Push any key to continue.')
            pause
         end;
      end;
      message=0;
      for j=i+2:nc,
          y=rot(E([j-1,j],j));      %
          E=rotapl(E,y,[j-1,j]);    % Deflate the infinite eigenvalue,
          A=rotapl(A,y,[j-1,j]);    % and apply the transformation on  A.
      end;
      for j=1:nc-1,
          y=rot(A(nc,[j+1,j]));     %
          A=rotapl(A',y,[j+1,j])';  % Reduce the last row of  A, apply the
          E=rotapl(E',y,[j+1,j])';  % transformation on  E,  and update  V.
          V=rotapl(V',y,[j+1,j])';  %
      end;
      nc=nc-1;
      ll=1:nc;
      E=E(ll,ll);   %
      A=A(ll,ll);   %  Deflate.
      B=B(ll,1:m);  %
      for j=i:-1:2,
          y=rot(E(j,[j,j-1]));      %
          E=rotapl(E',y,[j,j-1])';  % Fix the form of  E, apply the
          A=rotapl(A',y,[j,j-1])';  % transformation on  A,  and update  V.
          V=rotapl(V',y,[j,j-1])';  %
      end;
      i=nc;
      flag=1;                        % "flag" turns  0  when a zero is found
      while flag & (i > nn(1)),      % on any diagonal position of  E  after
            if abs(E(i,i)) <= tol,   % the  nn(1)  position.
               flag=0;
            end;
            i=i-1;
      end;
      if flag == 1,  % There are no more zeros on the diagonal of  E
         singular=0; % beyond the  nn(1)  position.
      end;
end;
if count > 0,
   disp(' ')
   count,
   disp('infinite eigenvalue(s) was(were) deflated.')
   disp(' ')
   disp('Push any key to continue')
   pause
end;
i=1;
flag=1;                       % "flag" turns  0  when a zero is found
while flag & (i <= nn(1)),    % on the first  nn(1)  diagonal positions
      if abs(E(i,i)) <= tol,  % of  E.
         flag=0;
      end;
      i=i+1;
end;
if flag == 0,              % There is at least a zero on the first  nn(1)
   disp(' ')               % diagonal positions of  E.
   disp('The system has a CONTROLLABLE INFINITE EIGENVALUE found within'), tol,
   disp('with respect to the "1-norm". Press any key to continue.')
   disp(' ')
   pause
   if write,
      E,
      disp('Push any key to see  B.')
      pause
      B,
      disp('Push any key to continue.')
      pause
   end;
end;
nsize=nc; % Temporary name for the size of the controllable "so far' system.
nc=nn(1); % nc  will soon take over.
k=1;
while nn(k)~=0 & nc<nsize,    %  Proceed with the reduction of  A.
      ll=nc-nn(k)+1;
      [ns,ms]=size(A(nc+1:nsize,ll:nc));
      for j=ll:ll-1+min(ms,ns),
          nor=norm(A(j:nsize,j),1);         %  Use column pivoting.
          j1=j;
          for jj=j+1:ll+ms-1,
              nor1=norm(A(j:nsize,jj),1);
              if nor1 > nor,
                 nor=nor1;
                 j1=jj;
              end;
          end;
          A(:,[j,j1])=A(:,[j1,j]);
          E(:,[j,j1])=E(:,[j1,j]);
          if j~=j1,                           %  Restore the triangular
             for jj=j:nc,                     %  form of  E,  which is
                 for i=nc:-1:jj+1,            %  destroyed when pivoting
                     y=rot(E([i-1,i],jj));    %  takes place.
                     E=rotapl(E,y,[i-1,i]);
                     A=rotapl(A,y,[i-1,i]);
                     if nc==nn(1),
                        B=rotapl(B,y,[i-1,i]);
                     end;
                 end;
             end;
          end;
          V(:,[j,j1])=V(:,[j1,j]);
          for i=nsize:-1:j+nn(k)+1,
              y=rot(A([i-1,i],j));      %
              A=rotapl(A,y,[i-1,i]);    % Reduce  A  and apply the
              E=rotapl(E,y,[i-1,i]);    % transformation on the rest of   A,
              y=rot(E(i,[i,i-1]));       %
              E=rotapl(E',y,[i,i-1])';   % Fix the form of  E,  and apply the
              A=rotapl(A',y,[i,i-1])';   % transformation on  A  and  V.
              V=rotapl(V',y,[i,i-1])';   %
          end;
      end;
      nn(k+1)=0;                  %
      s=svd(A(nc+1:nsize,ll:nc)); %
      len=length(s);              %
      rnkflag=1;                  %
      i=1;                        %
      while rnkflag & i<=len,     %
            if s(i)>tol,          %    nn(k+1)=rank(A(nc+1:nsize,ll:nc)).
               nn(k+1)=nn(k+1)+1; %
               i=i+1;             %
            else                  %
               rnkflag=0;         %
            end;                  %
      end;                        %
      if nn(k+1)>0,                 % Check for uncontrollability.
         nc=nc+nn(k+1);                   %  Update variables.
         k=k+1;                           %
      else
         disp('The system is within '), tol,
         disp('from an uncontrollable system, with respect to the "1-norm".')
         disp('This is due to at least one UNCONTROLLABLE FINITE EIGENVALUE.')
         disp('Press any key to deflate all the UNCONTROLLABLE FINITE')
         disp('EIGENVALUES and continue; or Ctrl C to terminate the process.')
         pause
         if write,
            A,
            disp('Push any key to see  E.')
            pause
            E,
            disp('Push any key to see  B.')
            pause
            B,
            disp('Push any key to continue.')
            pause
         end;
         nn(k+1)=0;
         k=k+1;
      end;
end;
if nc~=nsize,
   ll=1:nc;         %
   E=E(ll,ll);      %  Deflate all the finite uncontrollable eigenvalues,
   A=A(ll,ll);      %  and possibly some infinite uncontrollable eigenvalues.
   B=B(ll,1:m);     %
   k=k-1;           %
else
   nn(k+1)=0;
end;
%
%  Upward part
%
%                 nc  is the size of the controllable system,
%                 and it will be so until the end of the process.
%
if k~=1,
   row=nc;
   col=nc-nn(k)-nn(k-1)+1;
   for blr=k:-1:2;
       for i=row:-1:row-nn(blr)+1,
           for j=col:i-nn(blr)-1,
               y=rot(A(i,[j+1,j]));      %
               A=rotapl(A',y,[j+1,j])';  % Reduce  A  and apply the
               E=rotapl(E',y,[j+1,j])';  % transformation on the rest of   A,
               V=rotapl(V',y,[j+1,j])';  % and  E,  and update  V.
               y=rot(E([j,j+1],j));     %
               E=rotapl(E,y,[j,j+1]);   % Fix the form of  E,  and apply the
               A=rotapl(A,y,[j,j+1]);   % transformation on  A  and update  U.
               if blr==2,
                  B=rotapl(B,y,[j,j+1]);   %  Apply the transformation on  B.
               end;
           end;
       end;
       if blr>2,
          row=row-nn(blr);
          col=col-nn(blr-2);
       end;
   end;
end;
for i=min(m,nc):-1:1,
    for j=1:max(i-1,m-nc+i-1),
        y=rot(B(i,[j+1,j]));         %
        B=rotapl(B',y,[j+1,j])';     %  Reduce  B  and update  W.
        W=rotapl(W',y,[j+1,j])';     %
    end;
end;
%
%             Clear the form of  (E,A,B)  from elements that are
%             less than the tolerance used.
%
for j=1:nc-1,
    Z=zeros(nc-j,1);
    if ~isempty(Z),
       E(j+1:nc,j)=Z;
    end;
end;
if m-nn(1) > 0,
   Z=zeros(nc,m-nn(1));
   if ~isempty(Z),
      B(:,1:m-nn(1))=Z;
   end;
end;
for j=m-nn(1)+1:m,
    Z=zeros(nc-j+m-nn(1),1);
    if ~isempty(Z),
       B(j+1-(m-nn(1)):nc,j)=Z;
    end;
end;
row=0;
for r=2:k,
    row=row+nn(r-1);
    if nn(r-1)-nn(r) > 0,
       Z=zeros(nc-row,nn(r-1)-nn(r));
       if ~isempty(Z),
          A(row+1:nc,row-nn(r-1)+1:row-nn(r))=Z;
       end;
    end;
    for j=row-nn(r)+1:row,
        Z=zeros(nc-j-nn(r),1);
        if ~isempty(Z),
           A(j+nn(r)+1:nc,j)=Z;
        end;
    end;
end;
end;          % End of the computation of the controllability form.
if write,
   A, nn,
   disp('Push any key to see  E.')
   pause
   E,
   disp('Push any key to see  B.')
   pause
   B,
   disp('Push any key to continue.')
   pause
end;
%
%                          EIGENVALUE ALLOCATION
%
mc=nn(1);     % Columns of  B  in the controllable part of the system.
if m-mc > 0,
   B=B(:,m-mc+1:m);             % Delete the zero columns of  B.
end;
F=zeros(mc,nc);
nst=nc;
mst=mc;
Q=eye(nc);
for i=1:nc,
    r=1;
    while nn(r+1)==nn(r),
          r=r+1;
    end;
    if r==1,                           %  Immediate step.
       if abs(E(1,1)) <= tol,          %  Avoid forming a singular closed
          F(mc-mst+1:mc,i)= ...        %  loop pencil.
                  backsb(B(1:mst,1:mst),A(1:mst,1)-[etol zeros(1,mst-1)]');
       else
          F(mc-mst+1:mc,i)=backsb(B(1:mst,1:mst),A(1:mst,1)-l(i)*E(1:mst,1));
       end;
       B=B(2:nst,2:mst);
       mst=mst-1;
    else      %  Regular step.
       N=A-l(i)*E;
       BB=B(1:mst,1:mst);
       jj=sum(nn(1:r-1));
       for j=jj:-1:1,
           ii=[j+1,j];           %
           y=rot(N(j+nn(r),ii)); %
           N=rotapl(N.',y,ii)';  %  Eliminate elements for allocation.
           E=rotapl(E.',y,ii)';  %
           A=rotapl(A.',y,ii)';  %
           ii=ii+nc-nst;            %  Adjust indeses to the size of  Q.
           Q=rotapl(Q',y,ii)';      %
           ii=[j,j+1];          %
           y=rot(E(ii,j));      %  Fix the form of  E,  and update  A.
           E=rotapl(E,y,ii);    %
           A=rotapl(A,y,ii);    %
           if j<=mst,              %  Apply only those rotations
              B=rotapl(B,y,ii);    %  that will affect  B.
           end;                    %
       end;
       F(mc-mst+1:mc,i)=backsb(BB,N(1:mst,1));
       B=B(2:nst,:);
    end;
    A=A(2:nst,2:nst);
    E=E(2:nst,2:nst);
    nst=nst-1;
    nn(r)=nn(r)-1;
end;
if m > mc,                     %
   F=[(zeros(m-mc,nc))',F']';  %  Restore the zero rows of  F  if needed.
end;                           %
F=F*Q';
if ~form,
   if n > nc,                  %
      F=[F,zeros(m,n-nc)];     %  Restore the zero columns of  F  if needed.
   end;                        %
   F=W*F*V';
end;
