function F=eigenstr(E,A,B,L,X);
%
%     function F=eigenstr(E,A,B,L,X);
%
%     This function solves the Eigenstucture Assignment problem for Multi
%  Input Descriptor systems using State Feedback.
%     Given a COMPLETELY CONTROLLABLE system  (E,A,B),  a set  L  of desired
%  eigenvalues (in the form of the diagonal of a quasi-diagonal matrix), and
%  a matrix  X  with the corresponding desired and feasible eigenvectors or
%  eigenspaces of size two, the program allocates two eigenvalues at a time
%  along with the corresponding eigenvectors in a double step. The algorithm
%  here uses deflation. The function is called by the  DEMESAS.m  function to
%  solve the eigenstructure assignment problem of a system that may be
%  uncontrollable.
%
%  INPUT:  E,A,B    nxn, nxn, nxm  matrices respectively.
%          L        nx2  matrix, contains, in a special form, the  n
%                   eigenvalues to be allocated,
%          X        nxn  matrix with the corresponding eigenvectors or
%                   eigenspaces if the corresponding pair of eigenvalues
%                   forms a complex conjugate pair.
%
%  OUTPUT: F        The desired feedback matrix.
%
%
%  Note that for its proper execution it needs the  m.functions  ODD,  ROT,
%  ROTAPL,  QRTRANS,  and indirectly the  m.function  TRANS.
%
%
%
%      Written by  George S. Miminis
%                  Department of Computer Science
%                  Memorial University of Newfoundland
%                  St. John's, Newfoundland, CANADA,
%                  A1C 5S7.
%             tel: (709)737-8635
%          e-mail: george@garfield.cs.mun.ca
%                  na.miminis@na-net.ornl.gov
%
%
[n,m]=size(B);
F=zeros(m,n);
[VV,X]=qr(X);
R=triu(qr([E*VV,A*VV,B]));
E=R(1:n,1:n);
A=R(1:n,n+1:2*n);
B=R(1:n,2*n+1:2*n+m);
clear R
etol=norm([E,B,A,X],1);             %  Compute a reasonable ``relative
alpha=6/7;                          %  magnitude''  (etol), and a reasonable
tol=(m+n)*(etol*eps)^alpha;         %  tolerance  (tol).
if odd(n-m),
   cc=ceil((n-m)/2)-1;
else
   cc=ceil((n-m)/2);
end;
j=-1;       % In case the ``overdetermined'' part is empty,  jj  will be 1.
for i=1:cc,
    j=2*i-1;                % First column of  F  in the current double step.
    n1=n-j+1;                     % Size of the current system.
    H=zeros(2);      %  Compute  H.
    H(1,1)=E(1,1)*X(1,1)*L(j,1)-L(j,2)*(E(1,1)*X(1,2)+E(1,2)*X(2,2));
    H(1,2)=E(1,1)*X(1,1)*L(j,2)+L(j+1,2)*(E(1,1)*X(1,2)+E(1,2)*X(2,2));
    H(2,1)=-L(j,2)*E(2,2)*X(2,2);
    H(2,2)=L(j+1,2)*E(2,2)*X(2,2);
    if ((abs(E(1,1)) < tol) | (abs(E(2,2)) < tol)) & (abs(L(j,2)) > tol),
       H=etol*eye(2);          % No eigenpair is allocated.
    end;
    if abs(L(j,2)) < tol,
       if abs(E(1,1)) < tol,    % One eigenpair is allocated.
          H(1,1)=etol;
          H(2,1)=0;
          H(1,2)=L(j+1,2)*E(1,2)*X(2,2);
       end;
       if abs(E(2,2)) < tol,    % One eigenpair is allocated.
          H(2,2)=etol;
          H(2,1)=0;
          H(1,2)=L(j+1,2)*(E(1,1)*X(1,2)+E(1,2)*X(2,2));
          H(1,1)=E(1,1)*X(1,1)*L(j,1);
       end;
       if (abs(E(1,1)) > tol) & (abs(E(2,2)) > tol),      % Two real
          H(1,1)=E(1,1)*X(1,1)*L(j,1);                    % eigenvalues are
          H(2,1)=0;                                       % allocated.
          H(1,2)=L(j+1,2)*(E(1,1)*X(1,2)+E(1,2)*X(2,2));
       end;
    end;
    Y=H/X(1:2,1:2);                          % Solve Y*X(1:2,1:2)=H.
    A(1:2,1:2)=A(1:2,1:2)-Y;                 % Compute the  QR  decomposition
    R=triu(qr([B,A(:,1:2)]));                % of  B, and compute the two
    F(:,[j,j+1])=R(1:m,1:m)\R(1:m,m+1:m+2);  % current columns of  F.
    clear R
    E=E(3:n1,3:n1);      %
    A=A(3:n1,3:n1);      % Deflation.
    X=X(3:n1,3:n1);      %
    B=B(3:n1,:);         %
end;
%
% Solve the  mxm  or  mx(m+1)  ``underdetermined''  problem.
%
jj=j+2;              % First column of  F  in the underdetermined part.
L=L(jj:n,:);
[WW,B]=qrtrans(B);       % The  QR-like decomposition of  B.
[nB,m]=size(B);
rowB=1;              % The first row of  B  in the underdetermined part.
colB=1;              % The first column of  B  in the underdetermined part.
rowF=1;              % The first row of  F  in the underdetermined part.
flag=0;              % Whether or not the first step is overdetermined.
for j=jj:2:n-1,
    n1=n-j+1;                     % Size of the current system.
    H=zeros(2);      %  Compute  H.
    H(1,1)=E(1,1)*X(1,1)*L(1,1)-L(1,2)*(E(1,1)*X(1,2)+E(1,2)*X(2,2));
    H(1,2)=E(1,1)*X(1,1)*L(1,2)+L(2,2)*(E(1,1)*X(1,2)+E(1,2)*X(2,2));
    H(2,1)=-L(1,2)*E(2,2)*X(2,2);
    H(2,2)=L(2,2)*E(2,2)*X(2,2);
    if ((abs(E(1,1)) < tol) | (abs(E(2,2)) < tol)) & (abs(L(1,2)) > tol),
       H=etol*eye(2);          % No eigenpair is allocated.
    end;
    if abs(L(1,2)) < tol,
       if abs(E(1,1)) < tol,    % One eigenpair is allocated.
          H(1,1)=etol;
          H(2,1)=0;
          H(1,2)=L(2,2)*E(1,2)*X(2,2);
       end;
       if abs(E(2,2)) < tol,    % One eigenpair is allocated.
          H(2,2)=etol;
          H(2,1)=0;
          H(1,2)=L(2,2)*(E(1,1)*X(1,2)+E(1,2)*X(2,2));
          H(1,1)=E(1,1)*X(1,1)*L(1,1);
       end;
       if (abs(E(1,1)) > tol) & (abs(E(2,2)) > tol),      % Two real
          H(1,1)=E(1,1)*X(1,1)*L(1,1);                    % eigenvalues are
          H(2,1)=0;                                       % allocated.
          H(1,2)=L(2,2)*(E(1,1)*X(1,2)+E(1,2)*X(2,2));
       end;
    end;
    %
    % Compute the two current columns of  F.
    %
    Y=H/X(1:2,1:2);                        % Solve Y*X(1:2,1:2)=H.
    if odd(n-m) & j==jj,                   % The first step of the
       A(1:2,1:2)=A(1:2,1:2)-Y;            % ``underdetermined'' problem has
       R=triu(qr([B(1:nB,1:m),A(:,1:2)])); % an overdetermined system.
       F(1:m,[j,j+1])=R(1:m,1:m)\R(1:m,m+1:m+2);
    else                                   % Regular step.
       A(1:2,1:2)=A(1:2,1:2)-Y;
       F(rowF:m,[j,j+1])=B(rowB:nB,colB:m)\A(1:n1,1:2);
    end;
    %
    % Prepare the corresponding rows
    % of  F  for the backward part.
    %
    XL=zeros(n1,n1-2);
    for i=3:2:n1-1,                           %             (X12)
        XL(:,i-2:i-1)=X(:,i:i+1)*L(i:i+1,:);  % Compute  XL=(---)*L3.
    end;                                      %             (X3 )
    if odd(n),
       XL(:,n1-2)=X(:,n1)*L(n1,1);
    end;
    FF=(Y*X(1:2,3:n1)-E(1:2,1:n1)*XL)/X(3:n1,3:n1)+A(1:2,3:n1);
    if odd(n-m) & j==jj & ~isempty(FF),% First step of the underdetermined part
       flag=1;                         % with un overdetermined  2x1  system.
       y=rot(B(1:2,1));
       B(1:2,1:m)=rotapl(B(1:2,1:m),y,[1 2]);  % Get the dynamic part of
       FF=rotapl(FF,y,[1 2]);                  % the overdetermined system.
       F(1,j+2:n)=FF(1,:);             % Store the row onto the stack.
       rowF=rowF+1;
       colB=colB+1;
    elseif j<n-1,                 % The remaining steps. They are all double.
       F(rowF:rowF+1,j+2:n)=FF;      % Store the two rows onto the stack.
       rowF=rowF+2;
       colB=colB+2;
    end;
    E=E(3:n1,3:n1);      %
    A=A(3:n1,3:n1);      % Deflation.
    X=X(3:n1,3:n1);      %
    L=L(3:n1,:);         %
    rowB=rowB+2;    % ``Deflation'' of  B.
end;
if odd(n),               % If  n  is odd the last step is single.
   if abs(E)<=tol,
      F(m,n)=B(rowB,m)\(A-etol);
   else
      F(m,n)=B(rowB,m)\(A-L(1,1)*E);
   end;
end;
%
%  Backward part.
%
if odd(n),
   nn=m-2;
   colF=n;
   rowB=nB-2;
else
   nn=m-3;
   colF=n-1;
   rowB=nB-3;
end;
for rowF=nn:-2:1,
    F(rowF:rowF+1,colF:n)=B(rowB:rowB+1,rowF:rowF+1)\(F(rowF:rowF+1,colF:n) ...
                                 -B(rowB:rowB+1,rowF+2:m)*F(rowF+2:m,colF:n));
    colF=colF-2;
    rowB=rowB-2;              % Note that  colB=rowF.
end;
if flag,  % We have an overdetermined step at the very beginning.
   F(1,colF:n)=B(1,1)\(F(1,colF:n)-B(1,2:m)*F(2:m,colF:n));
end;
F(:,jj:n)=WW'*F(:,jj:n);
F=F*VV';
