function K=mevao(A,B,C,l);
%
%    function K=mevao(A,B,C,l);
%
%    This function solves the Multi input EigenValue Allocation problem
% using Output feedback (MEVAO). Given matrices  A, B, C  compute  K
% such that the spectrum of  A-BKC  contains the eigenvalues given in set  l.
% It does this by employing a method based on deflation by unitary similarity
% transformations. The algorithm allocates  min{n, m+p-1}  eigenvalues.
% The extra freedom which is available for the selection of a feasible
% eigenvector, is used to enhance the numerical stability of our algorithm.
% It can be proven that, using the extra freedom in this way we manage to
% allocate poles that are also transmission zeros (not strong) of the system.
% It is easy to show that this choice of eigenvectors helps in computing
% a matrix  K  with a reasonable Frobenious norm.
%    This program is an implementation of an algorithm presented in the
% "Third Annual Conference on Aerospace in Computational Control",
% Oxnard, California, August 89, and it is in the corresponding proceedings.
% The algorithm allocates one eigenvalue at a time, hence complex eigenvalues
% will be allocated by using complex arithmetic. This may lead to complex  K.
% Investigation is under way to allocate two eigenvalues at a time, thus
% complex conjugate pairs may be allocated in one double step using only real
% arithmetic. This program calls functions  ALLOCREM, and  MEVAOSTP.
%
% INPUT  : A   is an  nxn  matrix.
%          B   is an  nxm  matrix.
%          C   is a   pxn  matrix.
%          l   is a vector of dimension at least  min{n,m+p-1}, containing
%              the desired eigenvalues.
%
% OUTPUT : K   is the desired  mxp  gain matrix.
%
%
%      Written by  George S. Miminis
%                  Department of Computer Science
%                  Memorial University of Newfoundland
%                  St. John's, Newfoundland, CANADA,
%                  A1C 5S7.
%             tel: (709)737-8635
%          e-mail: george@garfield.cs.mun.ca
%                  na.miminis@na-net.ornl.gov
%
%
[n,m]=size(B);
[p,n]=size(C);
r=min(n,m+p-1);
if r<length(l),
   l=l(1:r);
end;
transp=0;
LEFT=eye(m);
RIGHT=eye(p);
K=zeros(m,p);
step=0;
if m>p,
   A=A.';
   AUX=C.';
   C=B.';
   B=AUX;
   transp=1;
end;
%
%  if abs(m-p)>1 then
%                    x x * * * *
%                    x x * * * *
%                    x x * * * *
%                else
%                   if abs(m-p)==1 then call  allocrem  immediately
%                      else  abs(m-p)=0  and
%                                            x * *
%                                            x * *
%                                            x * *
%                   end
%  end
%
%
if abs(m-p)>1,
   if ~transp,
      for step=1:abs(m-p)-1,
          [A,B,C,k,V]=mevaostp(A,B,C,l(step));
          K(:,step)=k;
          Z=zeros(p-step+1,step-1);
          RIGHT=RIGHT*[eye(step-1)  Z'
                           Z        V ];
      end;
   else
      for step=1:abs(m-p)-1,
          [A,B,C,k,V]=mevaostp(A,B,C,l(step));
          K(step,:)=k.';
          Z=zeros(m-step+1,step-1);
          LEFT=LEFT*[eye(step-1)  Z'
                         Z        V];
      end;
   end;
end;
if m==p,
   step=1;
   [A,B,C,k,V]=mevaostp(A,B,C,l(step));
   K(:,1)=k;
   RIGHT=RIGHT*V;
   A=A.';
   AUX=C.';
   C=B.';
   B=AUX;
end;
[A,B,C,KK]=allocrem(A,B,C,l(step+1:min(n,m+p-1)));
if abs(m-p)>=1,
   if ~transp,
       K(:,step+1:p)=KK;
   else
       K(step+1:m,:)=KK.';
   end;
end;
if m==p,
   K(:,step+1:p)=KK.';
end;
K=LEFT*K*RIGHT';
