function  F=mevas(A,B,l,info,nn);
%
%     function  F=mevas(A,B,l,info,nn);
%
%     This program solves the Pole Placement problem for Multi Input systems
%  using State Feedback. This MATLAB program is the implementation of an
%  algorithm by  MIMINIS, G. S. & PAIGE, C. C.  (to be submitted).
%     Given a system  (B,A)  and a set  l  of desired eigenvalues, the
%  program first computes the controllability form of  (B,A)
%  (if  (B,A)  is not already in this form), then it allocates two
%  eigenvalues at a time in a double step using implicit shifts. If the
%  system is uncontrollable, only a certain number of eigenvalues will be
%  allocated, and an appropriate message will be displayed. The program uses
%  directly the following  m.functions  for its proper execution:  ROTAPL,
%  ROT,  HOHVECT,  HOHAPL,  QRTRANS;  and indirectly the  m.function  TRANS.
%
%  INPUT:  A,B      nxn, nxm matrices respectively.
%          l        n-dimensional vector, contains the n eigenvalues to be
%                   allocated. Complex eigenvalues should appear in conjugate
%                   pairs.  In this program we will assume that complex
%                   eigenvalues appear first, and then real eigenvalues
%                   follow.
%          info     is a three dimensional vector with:
%
%            info(1)  1 if the controllability form of  (B,A)  needs to be
%                     computed.
%            info(2)  1 if you want the controllability form of (B,A)
%                     displayed.
%            info(3)  1 if the  SVD  is to be used for the determination of
%                     numerical ranks during the computation of the
%                     controllability form of the system; otherwise the  QR
%                     decomposition with column pivoting is used.
%            NOTE     that if  info(1)~=1  then  info(2), info(3)  are
%                     redundant, hence they need not be specified. Thus in
%                     this case  info  can be a scalar.
%
%  INPUT if info(1)~=1:
%
%          nn         (k+1)-dimensional vector, contains the row ranks of  B
%                     and of the  k-1  subdiagonal block elements of  A,
%                     with  nn(k+1)=0. k  is the controllability index of
%                     the system. nn  needs not be specified if  info(1)=1.
%
%  OUTPUT: F          The desired feedback matrix.
%
%
%      Written by  George S. Miminis
%                  Department of Computer Science
%                  Memorial University of Newfoundland
%                  St. John's, Newfoundland, CANADA,
%                  A1C 5S7.
%             tel: (709)737-8635
%          e-mail: george@garfield.cs.mun.ca
%                  na.miminis@na-net.ornl.gov
%
%
[n m]=size(B);
alpha=5/6;                               %
tol=(n+m)*(norm([B,A],1)*eps)^alpha;  % Compute a reasonable tolerance.
contr=info(1);
if contr==1,
   write=info(2);
   decomp=info(3);
end;
if contr == 1,  %  Compute the Controllability form of  (B,A).
   T=eye(n);              % Downward part.
   if decomp == 1,
      [P,B,V]=svd(B);
   else
      [P,B,V]=qr(B);
   end;
              % Computation of the controllability index  nn(1).
   nn(1)=0;
   for i=1:min(n,m),
       if abs(B(i,i)) > tol, nn(1)=nn(1)+1; end;
   end;
   if nn(1)==0,
      disp(' The input matrix is zero to machine precision.');
      disp(' Allocation is aborted.');
      return;
   end;
              % Transform and set variables.
   A=P'*A*P;
   T=T*P;
   k=1;
   rnk=nn(1);
   while nn(k) ~= 0 & rnk < n,
         AUX=A(rnk+1:n,rnk-nn(k)+1:rnk);
         if decomp == 1,
            [UAUX,AUX,VAUX]=svd(AUX);
         else
            [UAUX,AUX,VAUX]=qr(AUX);
         end;
         P=eye(n);
         P(rnk+1:n,rnk+1:n)=UAUX;
         P(rnk-nn(k)+1:rnk,rnk-nn(k)+1:rnk)=VAUX;
         A=P'*A*P;
         if k==1, B(1:nn(1),1:m)=VAUX'*B(1:nn(1),1:m); end;
         k=k+1;
         nn(k)=0;                      % Computation of controllability
         min=nn(k-1);                  % index  nn(k).
         if n-rnk < nn(k-1), min=n-rnk; end;
         for i=1:min,
             if abs(AUX(i,i)) > tol, nn(k)=nn(k)+1; end;
         end;
         rnk=rnk+nn(k);    % Set variables.
         T=T*P;
   end;
   if nn(k)==0 & rnk < n,             % Uncontrollable system.
      disp(' The system is within'); tol,
      disp(' from the nearest uncontrollable system; the procedure continues');
      disp(' with the allocation of only'); rnk,
      disp(' eigenvalues');
      if write==1,
             k, nn
                A, B
      end;
      diff=n-rnk;         % Number of states dropped.
      A=A(1:rnk,1:rnk);
      B=B(1:rnk,:);
      if rem(rnk,2)==1,
         temp=l(n);
         if imag(temp)~=0,
            disp(' The resulting system has odd number of states. A real');
            disp(' eigenvalue is thus necessary but there is not any');
            disp(' available. The program in this case will allocate');
            disp(' the real part of the last given eigenvalue');
            temp=real(temp);
         end;
         l(rnk)=temp;
      end;
      l=l(1:rnk);
      F=mevas(A,B,l,info);
      F=[F zeros(m,diff)];
      F=V*F*T';
      return;
   end;
   nn(k+1)=0;
   if nn(1)~=n,
      if nn(k) > 0,     % Upward part.
         for j=k:-1:2,
             rnk=rnk-nn(j);
             AUX=A(rnk+1:rnk+nn(j),rnk-nn(j-1)+1:rnk);
             [TAUX,AUX]=qrtrans(AUX,0);
             P=eye(n);
             P(rnk-nn(j-1)+1:rnk,rnk-nn(j-1)+1:rnk)=TAUX';
             A=P'*A*P;
             T=T*P;
         end;
         B=P'*B;
         AUX=B(1:nn(1),1:m);
         [W,AUX]=qrtrans(AUX,0);
         W=W';
         B=B*W;
         V=V*W;
      end;
   else           %  nn(1)=n
      if n < m,
         [W,B]=qrtrans(B,0);
         W=W';
         V=V*W;
      end;
   end;
%
%             Clear the form of  (B,A)  from elements that are
%             zero to machine precision.
%
   if m-nn(1) > 0,
      for i=1:n,
          for j=1:m-nn(1),
              B(i,j)=0.0;
          end;
      end;
   end;
   for j=m-nn(1)+1:m,
       for i=j+1:n,
           B(i,j)=0.0;
       end;
   end;
   row=0;
   for r=2:k,
       row=row+nn(r-1);
       if nn(r-1)-nn(r) > 0,
          for i=row+1:n,
              for j=row-nn(r-1)+1:row-nn(r),
                  A(i,j)=0.0;
              end;
          end;
       end;
       for j=row-nn(r)+1:row,
           for i=j+nn(r)+1:n,
               A(i,j)=0.0;
           end;
       end;
   end;
   if write==1,
      k, nn
          A, B
      disp(' Push any key to continue')
      pause
   end;
end;
                %
                % Eigenvalue Allocation
                %
                % Initialization
                %
F=zeros(m,n);
oddn=1;
if rem(n,2)==0,   %  If oddn=1, n  is odd, thus when  nn(1)=nn(2)+1  there
   oddn=0;        %  will be an immediate allocation with a real eigenvalue.
end;
del=0;
if m-nn(1) > 0,         % Delete the zero columns of  B  and the
   del=m-nn(1);         % corresponding rows of  F.
   B=B(:,del+1:m);
   F=F(del+1:m,:);
   m=nn(1);
end;
if cond(B) > 1/tol,
   disp(' The system is within'); tol,
   disp(' from the nearest uncontrollable system; allocation is aborted.');
   return;
end;
free=0;
s=nn(1)-nn(2);
ss=2*floor(s/2);
if ss > 0,                           %  ss/2  immediate double allocations.
   D=diag(l(1:ss));
   for i=2:2:ss,
       D(i,i-1)=-imag(D(i,i));
       D(i-1,i)=imag(D(i,i));        % Make a quasi-diagonal matrix with the
       D(i-1,i-1)=real(D(i-1,i-1));  % appropriate eigenvalues, and solve the
       D(i,i)=real(D(i,i));          % corresponding equations.
   end;
   if ss==s-1 & oddn==1,        % l(n) is real and we have the right
      ss=s;                     % conditions to allocate it immediately.
      oddn=0;
      aux=l(n);
      l(ss+1:n)=l(ss:n-1);
      l(ss)=aux;
      D(ss,ss)=l(ss);
   end;
   F(:,1:ss)=B(1:nn(1),1:nn(1)) \ (A(1:nn(1),1:ss)-eye(nn(1),ss)*D);
   if nn(1)==n,      % This takes care of the cases  n <= m.
      if del > 0,
         F=([zeros(del,n)' F'])';
      end;
      if contr==1,
         F=V*F*T';
      end;
      return;
   end;
   free=free+ss;             % Update.
   nn(1)=nn(1)-ss;
end;
Q=eye(n);                % Proceed with the remaining allocations.
index=0;                 % Used in the  r=2, nn(1)=nn(2)+1  case.
flag=0;
step=ss+1;
while step<=n-2,
    if nn(1)==nn(2)+1 & oddn==1,      % Immediate allocation of  l(n).
       if cond(B(step:(step-1)+nn(1),free+1:m)) > 1/tol,
          disp(' The system is within'); tol,
          disp(' from the nearest uncontrollable system; only'); step-1,
          disp(' eigenvalues have been allocated');
          if flag==1,
             mm=length(Pstack(1,:));
             for j=mm:-1:1,
                 row=Pstack(3,j);
                 y=[Pstack(1,j) Pstack(2,j)];
                 column=Pstack(4,j);
                 F=[F(:,1:column-1), ...
                             rotapl(F(:,column:n),y,[row+1 row+2])];
             end;
          end;
          F=F*Q';
          if contr==1
             F=V*F*T';
          end;
          return;
       end;
       F(free+1:m,step)=B(step:(step-1)+nn(1),free+1:m) \ ...
                            (A(step:(step-1)+nn(1),step)-l(n)*eye(nn(1),1));
       aux=l(n);
       l(step+1:n)=l(step:n-1);  % Move l(n) to l(step) and shift
       l(step)=aux;              % l(step:n-1) one position downwards.
       nn(1)=nn(1)-1;    % Update.
       free=free+1;
       step=step+1;
       oddn=0;
    else
       if nn(1)==nn(2)+2,  % Immediate double allocation of l(step),l(step+1).
          if cond(B(step:(step-1)+nn(1),free+1:m)) > 1/tol,
             disp(' The system is within'); tol,
             disp(' from the nearest uncontrollable system; only'); step-1,
             disp(' eigenvalues have been allocated');
             if flag==1,
                mm=length(Pstack(1,:));
                for j=mm:-1:1,
                    row=Pstack(3,j);
                    y=[Pstack(1,j) Pstack(2,j)];
                    column=Pstack(4,j);
                    F=[F(:,1:column-1), ...
                                rotapl(F(:,column:n),y,[row+1 row+2])];
                end;
             end;
             F=F*Q';
             if contr==1,
                F=V*F*T';
             end;
             return;
          end;
          D=[real(l(step)) imag(l(step)); imag(l(step+1)) real(l(step+1))];
          F(free+1:m,step:step+1)=B(step:(step-1)+nn(1),free+1:m) \ ...
                        (A(step:(step-1)+nn(1),step:step+1)-eye(nn(1),2)*D);
          free=free+2; % Update.
          step=step+2;
          nn(1)=nn(1)-2;
       else
          r=2;                   % Find the next  r.
          while nn(r)==nn(r+1),
                r=r+1;
          end;
          if r==2,               % The  r=2  case.
             q=nn(2);
             if nn(1)==nn(2),    % The  r=2 & nn(1)=nn(2)  case.
                for column=(step-1)+q:-1:step+1,
                    row=column+q;
                    y=rot(A(row,[column+1, column]));    % Defines  R.
                    A=rotapl(A',y,[column+1, column])';  %   A=A*R
                    A=rotapl(A,y,[column+1, column]);    %   A=R'*A
                    B=rotapl(B,y,[column+1, column]);    %   B=R'*B
                    Q=rotapl(Q',y,[column+1, column])';  %   Q=Q*R
                end;
                if cond(B(step+2:step+q,free+2:m)) > 1/tol,
                  disp(' The system is within'); tol,
                  disp('from the nearest uncontrollable system; only'); step-1,
                  disp(' eigenvalues have been allocated');
                   if flag==1,
                      mm=length(Pstack(1,:));
                      for j=mm:-1:1,
                          row=Pstack(3,j);
                          y=[Pstack(1,j) Pstack(2,j)];
                          column=Pstack(4,j);
                          F=[F(:,1:column-1), ...
                                      rotapl(F(:,column:n),y,[row+1 row+2])];
                      end;
                   end;
                   F=F*Q';
                   if contr==1,
                      F=V*F*T';
                   end;
                   return;
                end;
                F(free+2:m,step:step+1)=B(step+2:step+q,free+2:m)\ ...
                                          A(step+2:step+q,step:step+1);
                   %
                   %  Create the  2x2  Pole Placement problem.
                   %
                AA=A(step:step+1,step:step+1)-B(step:step+1,free+2:m)* ...
                                                F(free+2:m,step:step+1);
                beta=B(step,free+1);
                if abs(beta) < tol | abs(AA(2,1)) < tol,
                  disp(' The system is within'); tol,
                  disp('from the nearest uncontrollable system; only'); step-1,
                  disp(' eigenvalues have been allocated');
                   if flag==1,
                      mm=length(Pstack(1,:));
                      for j=mm:-1:1,
                          row=Pstack(3,j);
                          y=[Pstack(1,j) Pstack(2,j)];
                          column=Pstack(4,j);
                          F=[F(:,1:column-1), ...
                                      rotapl(F(:,column:n),y,[row+1 row+2])];
                      end;
                   end;
                   F=F*Q';
                   if contr==1,
                      F=V*F*T';
                   end;
                   return;
                end;
                   %
                   %  Solve the above Pole Placement problem.
                   %
                F(free+1,step:step+1)=[AA(1,1)+AA(2,2)-(l(step)+l(step+1)) ...
                   AA(1,2)+(AA(2,2)-l(step))*(AA(2,2)-l(step+1))/AA(2,1)]/beta;
                                 %
             else                % The case r=2 & nn(1)=nn(2)+1.
                index=index+1;
                for column=step+q:-1:step+2,
                    row=column+q;
                    y=rot(A(row,[column+1, column]));    % Defines  R.
                    A=rotapl(A',y,[column+1, column])';  %   A=A*R
                    A=rotapl(A,y,[column+1, column]);    %   A=R'*A
                    B=rotapl(B,y,[column+1, column]);    %   B=R'*B
                    Q=rotapl(Q',y,[column+1, column])';  %   Q=Q*R
                end;
                %
                % Interchange 1st and 3rd rows of the current
                % A  and  B, and the 1st and 3rd columns of the
                % current  A.
                %
                A(:,[step step+2])=A(:,[step+2 step]);
                A([step step+2],:)=A([step+2 step],:);
                B([step step+2],:)=B([step+2 step],:);
                Q(:,[step step+2])=Q(:,[step+2 step]);
                y=rot(B(step+2,[free+2, free+1]));    % Defines rotation  P.
                B=rotapl(B',y,[free+2, free+1])';     %   B=B*P
                flag=1;                           % if flag==1, Pstack exists.
                Pstack(1,index)=-y(1);            % Update the  Pstack.
                Pstack(2,index)=y(2);
                Pstack(3,index)=free;             %  Row of  P.
                Pstack(4,index)=step;             %  Column of  F.
                if cond(B(step+2:step+nn(1),free+2:m)) > 1/tol,
                  disp(' The system is within'); tol,
                  disp('from the nearest uncontrollable system; only'); step-1,
                  disp(' eigenvalues have been allocated');
                   if flag==1,
                      mm=length(Pstack(1,:));
                      for j=mm:-1:1,
                          row=Pstack(3,j);
                          y=[Pstack(1,j) Pstack(2,j)];
                          column=Pstack(4,j);
                          F=[F(:,1:column-1), ...
                                      rotapl(F(:,column:n),y,[row+1 row+2])];
                      end;
                   end;
                   F=F*Q';
                   if contr==1,
                      F=V*F*T';
                   end;
                   return;
                end;
                F(free+2:m,step:step+1)=B(step+2:step+nn(1),free+2:m)\ ...
                                          A(step+2:step+1+q,step:step+1);
                      %
                      %  Create the  2x2  Pole Placement problem.
                      %
                AA=A(step:step+1,step:step+1)-B(step:step+1,free+2:m)* ...
                                                F(free+2:m,step:step+1);
                beta=B(step+1,free+1);
                if abs(beta) < tol | abs(AA(1,2)) < tol,
                  disp(' The system is within'); tol,
                  disp('from the nearest uncontrollable system; only'); step-1,
                  disp(' eigenvalues have been allocated');
                   if flag==1,
                      mm=length(Pstack(1,:));
                      for j=mm:-1:1,
                          row=Pstack(3,j);
                          y=[Pstack(1,j) Pstack(2,j)];
                          column=Pstack(4,j);
                          F=[F(:,1:column-1), ...
                                      rotapl(F(:,column:n),y,[row+1 row+2])];
                      end;
                   end;
                   F=F*Q';
                   if contr==1,
                      F=V*F*T';
                   end;
                   return;
                end;
                      %
                      %  Solve the above Pole Placement problem.
                      %
                F(free+1,step:step+1)= ...
                  [AA(2,1)+(AA(1,1)-l(step))*(AA(1,1)-l(step+1))/AA(1,2), ...
                                    AA(1,1)+AA(2,2)-(l(step)+l(step+1))]/beta;
             end;       % ( nn(1)==nn(2) )
             free=free+1;       % Update.
          else                  % Regular double allocation  (r>2).
             q=nn(r);
             ssize=nn(1)+(r-1)*q;              % Size of the current matrix.
             A1=A(step:step-1+ssize,step:step-1+ssize);  % The current matrix.
             e=[zeros(1,nn(1)+(r-2)*q) 1 zeros(1,q-1)]';
             e(nn(1)+(r-2)*q+1)=real(l(step))*real(l(step+1))+imag(l(step))^2;
             w=A1(nn(1)+(r-2)*q+1,:);
             w=w*A1-(real(l(step))+real(l(step+1)))*w+e';  % Row of  N.
             w=[zeros(1,step-1) w zeros(1,n-ssize-step+1)];  % Enhance  w.
             %
             %  The following 'for' loop is for the
             %  first  q-1  rotations.
             %
             for column=(step-1)+nn(1)+(r-2)*q:-1:(step+1)+nn(1)+(r-3)*q,
                 row=column+q;
                 y=rot(A(row,[column+1, column]));    % Defines  R.
                 A=rotapl(A',y,[column+1, column])';  %   A=A*R
                 A=rotapl(A,y,[column+1, column]);    %   A=R'*A
                 Q=rotapl(Q',y,[column+1, column])';  %   Q=Q*R
                 w=rotapl(w',y,[column+1, column])';  %   w=w*R
             end;
             %
             %  The following 'for' loop is for the
             %  next  q-1  Householder transformations.
             %
             for column=(step-1)+nn(1)+(r-3)*q:-1:(step+1)+nn(1)+(r-4)*q,
                 row=column+q+1;
                 [u,pi]=hohvect(A(row,column:column+2),3);  % u  defines  H.
                 A(1:row,column:column+2)= ...
                         hohapl(A(1:row,column:column+2)',u,pi)'; % A=A*H.
                 col=max(1,column-q);
                 A(column:column+2,col:n)=...
                          hohapl(A(column:column+2,col:n),u,pi);  % A=H*A.
                 rlb=column-step+1;      % rlb=relative position of the first
                 if rlb <= m-free,       % column of the current  A  matrix.
                    B(column:column+2,rlb+free:m)= ...
                        hohapl(B(column:column+2,rlb+free:m),u,pi); % B=H*B.
                 end;
                 w(column:column+2)= ...
                          hohapl(w(column:column+2)',u,pi)'; % w=w*H.
                 Q(1:n,column:column+2)= ...
                       hohapl(Q(1:n,column:column+2)',u,pi)';  % Q=Q*H.
             end;
                        %
                        % The implicit Householder transformation.
                        %
             w=w(step:step-1+ssize);         % Reduce w to its normal size.
             ignor=nn(1)+(r-4)*q;
             column=step+nn(1)+(r-4)*q;
             row=column+q+1;
             [u,pi]=hohvect(w(ignor+1:ignor+3),3);  % u defines H.
             A(1:row,column:column+2)= ...
                     hohapl(A(1:row,column:column+2)',u,pi)'; % A=A*H.
             col=max(1,column-q);
             A(column:column+2,col:n)=...
                      hohapl(A(column:column+2,col:n),u,pi);  % A=H*A.
             rlb=column-step+1;          % rlb=relative position of the first
             if rlb <= m-free,           % column of the current  A  matrix.
                B(column:column+2,rlb+free:m)= ...
                    hohapl(B(column:column+2,rlb+free:m),u,pi); % B=H*B.
             end;
             Q(1:n,column:column+2)= ...
                   hohapl(Q(1:n,column:column+2)',u,pi)';  % Q=Q*H.
             clear e A1 w
             %
             %  This  for  loop is for the remaining
             %  Householder transformations.
             %
             for column=(step-1)+nn(1)+(r-4)*q:-1:step+1,
                 row=column+q+2;
                 [u,pi]=hohvect(A(row,column:column+2),3);  % u defines H.
                 A(1:row,column:column+2)= ...
                         hohapl(A(1:row,column:column+2)',u,pi)'; % A=A*H.
                 col=max(1,column-q);
                 A(column:column+2,col:n)= ...
                          hohapl(A(column:column+2,col:n),u,pi);  % A=H*A.
                 rlb=column-step+1;      % rlb=relative position of the first
                 if rlb <= m-free,       % column of the current  A  matrix.
                    B(column:column+2,rlb+free:m)= ...
                        hohapl(B(column:column+2,rlb+free:m),u,pi); % B=H*B.
                 end;
                 Q(1:n,column:column+2)= ...
                       hohapl(Q(1:n,column:column+2)',u,pi)';  % Q=Q*H.
             end;
                        %
                        %  If  nn(1)+(r-4)*q==0  then the implicit
                        %  transformation should be the last one.
                        %
             if nn(1)+(r-4)*q~=0,        %  The last transformation if needed.
                if nn(1)==nn(2),
                   column=step;
                   row=column+q+2;
                   [u,pi]=hohvect(A(row,column:column+2),3); % u defines H.
                   A(1:row,column:column+2)= ...
                           hohapl(A(1:row,column:column+2)',u,pi)'; % A=A*H.
                   col=max(1,column-q);
                   A(column:column+2,col:n)= ...
                            hohapl(A(column:column+2,col:n),u,pi);  % A=H*A.
                   rlb=column-step+1;     % rlb=relative position of the first
                   if rlb <= m-free,      % column of the current  A  matrix.
                      B(column:column+2,rlb+free:m)= ...
                          hohapl(B(column:column+2,rlb+free:m),u,pi); % B=H*B.
                   end;
                   Q(1:n,column:column+2)= ...
                         hohapl(Q(1:n,column:column+2)',u,pi)';  % Q=Q*H.
                else                        %  The  nn(1)=nn(2)+1  case.
                   H=eye(n);
                   H(:,[step step+2])=H(:,[step+2 step]);
                   A(:,[step step+2])=A(:,[step+2 step]);
                   A([step step+2],:)=A([step+2 step],:);
                   B([step step+2],:)=B([step+2 step],:);
                   Q(:,[step step+2])=Q(:,[step+2 step]);
                end;
             end;
             if cond(B(step+2:step+1+nn(1),free+1:m)) > 1/tol,
                disp(' The system is within'); tol,
                disp(' from the nearest uncontrollable system; only'); step-1,
                disp(' eigenvalues have been allocated');
                if flag==1,
                   mm=length(Pstack(1,:));
                   for j=mm:-1:1,
                       row=Pstack(3,j);
                       y=[Pstack(1,j) Pstack(2,j)];
                       column=Pstack(4,j);
                       F=[F(:,1:column-1), ...
                                   rotapl(F(:,column:n),y,[row+1 row+2])];
                   end;
                end;
                F=F*Q';
                if contr==1,
                   F=V*F*T';
                end;
                return;
             end;
             F(free+1:m,step:step+1)=B(step+2:step+1+nn(1),free+1:m)\ ...
                                       A(step+2:step+1+nn(1),step:step+1);
          end;       % ( r=2 )
          nn(r-1)=nn(r-1)-1;             %  Update.
          nn(r)=nn(r)-1;
          step=step+2;
       end;          % ( nn(1)=nn(2)+2 )
    end;             % ( nn(1)=nn(2)+1 & oddn=1 )
end;                 % ( while )
                %
                %  The allocation of the last two eigenvalues.
                %
if nn(2)==0,    %  The resulting k is 1 thus   of inputs =  of states.
   if nn(1)==2,
      D=[real(l(step)) imag(l(step)); imag(l(step+1)) real(l(step+1))];
   else           %  nn(1)=1.
      D=l(step);
   end;
   if cond(B(step:n,free+1:m)) > 1/tol,
      disp(' The system is within'); tol,
      disp(' from the nearest uncontrollable system; only'); step-1,
      disp(' eigenvalues have been allocated');
      if flag==1,
         mm=length(Pstack(1,:));
         for j=mm:-1:1,
             row=Pstack(3,j);
             y=[Pstack(1,j) Pstack(2,j)];
             column=Pstack(4,j);
             F=[F(:,1:column-1), ...
                         rotapl(F(:,column:n),y,[row+1 row+2])];
         end;
      end;
      F=F*Q';
      if contr==1,
         F=V*F*T';
      end;
      return;
   end;
   F(free+1:m,step:n)=B(step:n,free+1:m)\(A(step:n,step:n)-D);
else     %  The resulting k is 2 thus there is one input and two states left.
   if abs(B(n-1,m)) < tol | abs(A(n,n-1)) < tol,
      disp(' The system is within'); tol,
      disp(' from the nearest uncontrollable system; only'); step-1,
      disp(' eigenvalues have been allocated');
      if flag==1,
         mm=length(Pstack(1,:));
         for j=mm:-1:1,
             row=Pstack(3,j);
             y=[Pstack(1,j) Pstack(2,j)];
             column=Pstack(4,j);
             F=[F(:,1:column-1), ...
                         rotapl(F(:,column:n),y,[row+1 row+2])];
         end;
      end;
      F=F*Q';
      if contr==1,
         F=V*F*T';
      end;
      return;
   end;
   F(m,n-1:n)=[A(n-1,n-1)+A(n,n)-(l(n-1)+l(n)) ...
               A(n-1,n)+(A(n,n)-l(n-1))*(A(n,n)-l(n))/A(n,n-1)]/B(n-1,m);
end;
if del > 0,                  %
   F=[(zeros(del,n))' F']';  %  Restore the zero rows of  F  if needed.
end;                         %
if flag==1,
   mm=length(Pstack(1,:));
   for j=mm:-1:1,
       row=Pstack(3,j);
       y=[Pstack(1,j) Pstack(2,j)];
       column=Pstack(4,j);
       F=[F(:,1:column-1), ...
                   rotapl(F(:,column:n),y,[row+1 row+2])];
   end;
end;
F=F*Q';
if contr==1,
   F=V*F*T';
end;
