function  f=sevas(A,b,l,contr,write);
%
% function  f=sevas(A,b,l,contr,write);
%
%   This program solves the Pole Placement problem for single input
% systems using state feedback. The eigenvalues are allocated two at
% a time in a double step using implicit shifts. If the system is
% uncontrollable the program allocates only a certain number of eigenvalues
% and it displays an appropriate message. The program is an implementation of
% an algorithm by  MIMINIS, S.G. & PAIGE, C.C. submitted to the  SIAM
% journal on matrix analysis and applications. The program needs the
% m.functions  HOHVECT,  and  HOHAPL  for its proper execution.
%
% INPUT:  A  The  nxn  state matrix.
%         b  The n-dimensional input vector.
%         l  an n-dimensional complex vector with the desired
%            eigenvalues. Complex eigenvalues should appear in conjugate
%            pairs. one way of organizing the eigenvalues is: First
%            complex conjugate pairs and then real eigenvalues. However
%            if real eigenvalues are mixed with complex eigenvalues then
%            real eigenvalues should appear in groups of even numbers.
%     contr  1 if the controllability form of the system is to be
%            computed.
%     write  1 if the controllability form is to be displayed. Needs not
%            be specified if contr~=1.
%
% OUTPUT: f  The n-dimensional gain vector.
%
%
%    Written by  George S. Miminis
%                Department of Computer Science
%                Memorial University of Newfoundland
%                St. John's, Newfoundland, CANADA
%                A1C 5S7 .
%          tel:  (709)737-8635 .
%       e-mail:  george@garfield.cs.mun.ca
%                na.miminis@na-net.ornl.ca
%
%
if contr==1,
   [Q,b]=qr(b);
   A=Q'*A*Q;
   U=Q;
   [Q,A]=hess(A);
   U=U*Q;
   if write==1,
      A, b,
   end;
end;
n=max(size(A));
alpha=5/6;                              %
tol=(n+1)*(norm([b,A],1)*eps)^alpha; % Compute a reasonable tolerance.
oddn=1;
if rem(n,2)==0,
   oddn=0;
end;
Q=eye(n);
step=1;
while step<=n-2,
    u=[A(n,n-1)*A(n-1,n-2) ...
     A(n,n-1)*A(n-1,n-1)+A(n,n)*A(n,n-1)-(l(step)+l(step+1))*A(n,n-1) ...
     A(n,n-1)*A(n-1,n)+A(n,n)^2-(l(step)+l(step+1))*A(n,n)+l(step)*l(step+1)];
    [u,pi]=hohvect(u,3);
    A(n-2:n,:)=hohapl(A(n-2:n,:),u,pi);
    A(:,n-2:n)=hohapl(A(:,n-2:n)',u,pi)';
    Q(:,n-2:n)=hohapl(Q(:,n-2:n)',u,pi)';
    for column=n-3:-1:step,
        row=column+3;
        u=[A(row,column) A(row,column+1) A(row,column+2)];
        [u,pi]=hohvect(u,3);
        A(column:column+2,:)=hohapl(A(column:column+2,:),u,pi);
        A(:,column:column+2)=hohapl(A(:,column:column+2)',u,pi)';
        Q(:,column:column+2)=hohapl(Q(:,column:column+2)',u,pi)';
    end;
    b(step:step+2)=hohapl(b(step:step+2),u,pi);
    if abs(b(step+2))<=tol,
       disp(' The system is within'); tol, ...
       disp('  from an uncontrollable system; only  '); step-1,
       disp('  eigenvalues have been allocated');
       f(step:n)=zeros(1,n-step+1);
       f=f*Q';
       if contr==1,
          f=f*U';
       end;
       f=f';
       return;
    end;
    f(step:step+1)=A(step+2,step:step+1)/b(step+2);
    step=step+2;
end;
if oddn==1,
   if abs(b(n))<=tol,
      disp(' The last eigenvalue cannot be allocated');
      f(n)=0;
   else
      f(n)=(A(n,n)-l(n))/b(n);
   end;
else
   if abs(b(n-1))<=tol | abs(A(n,n-1))<=tol,
      disp(' The last two eigenvalues cannot be allocated');
      f(n-1:n)=[0,0];
   else
          f(n-1)=(A(n-1,n-1)+A(n,n)-l(n-1)-l(n))/b(n-1);
      f(n)=(A(n-1,n)+(A(n,n)-l(n-1))*(A(n,n)-l(n))/A(n,n-1))/b(n-1);
   end;
end;
f=f*Q';
if contr==1,
   f=f*U';
end;
f=f';
