function [reduced_num,reduced_den]=reduce(num,den,percent_error);
%FUNCTION:     Reduce
%
%SYNTAX:       [ReducedPoly1,ReducedPoly2] = reduce(Poly1,Poly2,Percent_Error)
%
%DESCRIPTION:  Given two polynomials, this MATLAB function uses ROOTS to find
%              the roots of the two polynomials and then uses these roots to
%              eliminate common factors.  The reduced polynomials are returned
%              as the results.  This function is useful for canceling poles
%              and zeros in a transfer function.
%
%PARAMETERS:   Reduced_Poly1:  original Poly1 less any factors common to
%                              Poly1 and Poly2.
%              Reduced_Poly2:  original Poly2 less any factors common to
%                              Poly1 and Poly2.
%              Poly1:          first  polynomial (num. of transfer function)
%              Poly2:          second polynomial (den. of transfer function)
%              Percent_Error:  (optional) used to determine how close roots
%                              must be before they are considered the same
%                              root.  Defaults to 1.
%
%NOTE:        This method of reconstructing the polynomials from the unique
%             roots is faster and appears to be more accurate than using
%             DECONV to divide out the common factors.


%Get the gain of each polynomial (1st non-zero value)
i = 1;  while num(i)==0 ;  i = i+1;  end;
gain_of_num =  num(i);
i = 1;  while den(i)==0 ;  i = i+1;  end;
gain_of_den =  den(i);

%Get the roots of each polynomial
roots_of_num =  roots(num);
roots_of_den =  roots(den);

%How many roots does each polynomial have?
n_roots_num  =  length(roots_of_num);
n_roots_den  =  length(roots_of_den);

%Initially assume that all roots are unique,
%      set all locations in the keep vectors
%      to 1 (true).
keep_root_num =  ones(n_roots_num,1);
keep_root_den =  ones(n_roots_den,1);

%Change percent error to actual allowable error;
if ~exist('percent_error')
       allowable_error =  0.0100000;
else
       allowable_error =  0.0100000 * percent_error;
end;


%Loop through the roots of the denominator
for j=1:n_roots_den;
       den_root =  roots_of_den(j);

%      Loop through the roots of the numerator
       for i=1:n_roots_num;
%           If neither the current root of the numerator nor the current
%           root of the denominator has been eliminated, check them for
%           commonality.
            if keep_root_num(i) & keep_root_den(j)
                 num_root =  roots_of_num(i);

%                This takes care of two zero roots
                 if num_root==0 & den_root==0
                      keep_root_num(i) =  0;   %eliminate root of num
                      keep_root_den(j) =  0;   %eliminate root of den

%                This takes care of anything else
                 else
%                     Calculate the relative error of the two roots
                      error =  abs(den_root-num_root);
                      error =  error/max(abs(den_root), ..
                                     abs(num_root));

%                     If the calculated error is less than that
%                     allowed by the user, the roots are the same,
%                     eliminate them.
                      if error <= allowable_error
                           keep_root_num(i) =  0;
                           keep_root_den(j) =  0;
                      end;
                 end;
            end;
       end;
end;


%Construct reduced numerator from the roots that have not been eliminated.
initialize  = 1;
reduced_num = 1;

for i=1:n_roots_num;
       if initialize & keep_root_num(i)
            reduced_num =  [1,-roots_of_num(i)];
            initialize  = 0;
       elseif keep_root_num(i)
            reduced_num =  conv(reduced_num,[1,-roots_of_num(i)]);
       end;
end;

%Multiply the reduced numerator by the gain to get the final form.
reduced_num =  gain_of_num * reduced_num;


%Construct reduced denominator from the roots that have not been eliminated.
initialize  = 1;
reduced_den = 1;

for j=1:n_roots_den;
       if initialize & keep_root_den(j)
            reduced_den =  [1,-roots_of_den(j)];
            initialize  = 0;
       elseif keep_root_den(j)
            reduced_den =  conv(reduced_den,[1,-roots_of_den(j)]);
       end;
end;

%Multiply the reduced denominator by the gain to get the final form.
reduced_den =  gain_of_den * reduced_den;
