function [rnum,rden]=routhred(num,den,n)
% This function computes the N'th order Routh approximate of the function
%
%                               num(s)
%                       H(s) = -------
%                               den(s)
%
% Called as      [rnum,rnum] = routhred(num,den,n)
%
% The quanities num and den are polynomial row vectors of the numerator
% and denominator respectively of the original transfer function.
% The original transfer function must be strictly proper.
%
% The quanity n is the order of the approximation desired.
%
% The quanities rnum and rden are the new numerator and denominator
% respectively, in row vector format, of the reduced transfer function.
%
% Type 'help poly' for more information on polynomial row vectors.

% Written by Steve Chicatelli           %
%            579 Brookpark Dr.          %
%            Cuyahoga Falls, Ohio 44223 %

if nargin < 3,
   errmsg('Not enough arguments given ')
   return
end
if n >= (length(den)-1)
   errmsg('Can''t compute a convergent of same or greater order as original')
   return
end
if length(num) >= length(den)
   errmsg('Transfer function must be strictly proper')
   return
end
% REVERSE ORDER OF COEFFICIENTS TO PRESERVE LOW FREQUENCY BEHAVIOR %
for c=1:length(den),
    tempden(c)=den(length(den)+1-c);
end
for c=1:length(num),
    tempnum(c)=num(length(num)+1-c);
end
den=tempden;
num=tempnum;
% PUT ORIGINAL DEN COEFFICIENTS INTO ALPHA MATRIX %
a1=0;
for c=1:2:length(den),
    a1=a1+1;
    alpha(1,a1)=den(c);
end
a2=0;
for c=2:2:length(den),
    a2=a2+1;
    alpha(2,a2)=den(c);
end
% PUT ORIGINAL NUM COEFFICIENTS INTO BETA MATRIX %
b1=0;
for c=1:2:length(num),
    b1=b1+1;
    beta(1,b1)=num(c);
end
b2=0;
for c=2:2:length(num),
    b2=b2+1;
    beta(2,b2)=num(c);
end
% COMPUTE THE REST OF THE ALPHA MATRIX %
for a=3:length(den),
    for b=1:a1-1,
        alpha(a,b) = alpha(a-2,b+1) - alpha(a-2,1)/alpha(a-1,1)*alpha(a-1,b+1);
    end
end
for a=1:length(den)-1,
    A(a)=alpha(a,1)/alpha(a+1,1);
end
% COMPUTE THE REST OF THE BETA MATRIX %
for a=3:length(num),
    for b=1:b1-1,
        beta(a,b) = beta(a-2,b+1) - beta(a-2,1)/alpha(a-1,1)*alpha(a-1,b+1);
    end
end
for a=1:length(num),
    B(a)=beta(a,1)/alpha(a+1,1);
end
% SHOW EXPANSION MATRICIES  AND COEFFICIENTS %
% alpha,beta
% A,B
% COMPUTE NEW NUMERATOR AND DENOMINATOR %
AM1 = [0 1];
AM2 = [1];
BM1 = [0 0];
BM2 = [0];
for k=1:n,
  rden = [A(k).*AM1 0] + [0 0 AM2];
  AM2 = AM1;
  AM1 = rden;
end
for k=1:n,
  rnum = [A(k).*BM1 B(k)] + [0 0 BM2];
  BM2 = BM1;
  BM1 = rnum;
end
% PUT COEFFICIENTS BACK IN ORDER %
for c=0:n,
    temp1den(c+1)=rden(length(rden)-c);
end
for c=0:n-1,
    temp1num(c+1)=rnum(length(rnum)-c);
end
rden=temp1den;
rnum=temp1num;
