function []=pbn_game(cmd,cmddata,G,g_f)

% PBN_GAME Play the Paint by Numbers game
%        PBN_GAME() initializes the game without any puzzle
%
%        The File menu allows the current puzzle to be Saved or an old
%        puzzle to be Loaded. The game can be Quit from this menu.
%
%        The object of the game is to create a picture with the
%        dark and light colored squares using the labels on the
%        rows and columns of the grid.  The labels denote the
%        number of consecutive dark squares in that row or column. 
%        Multiple groups of consecutive dark squares must be separated
%        by at least one light square, and the groups appear in the
%        order shown by the labels.
%
%        To color the squares, click on a square using the 
%        mouse buttons:
%           Paint mode:
%             Right  = empty square                (white)
%             Middle = light square                (rose)
%             Right  = dark square                 (dark gray)
%           Experimental mode:
%             Right  = empty square                (white)
%             Middle = experimental light square   (yellow)
%             Left   = experimental dark square    (blue)
%             Covering a blue square with yellow or vice versa
%               will cause it to turn green
%        Switch between Paint and Experimental mode using the Draw
%        menu.
%        Color more than one square at a time by holding the mouse
%        button down while dragging the mouse to the desired end location
%        in a row or column.
%
%        PBN_GAME(ROW_LABELS,COL_LABELS) displays the game
%        using the integer arrays labelling the rows and columns.
%
%        ROW_LABELS is an RxN array where R is the number of rows
%        in the grid and N the max. number of labels for the rows.
%
%        COL_LABELS is an MxC array where C is the number of 
%        columns in the grid and M the max. number of labels for 
%        the columns.
%
%        Use 0's for blanks in the labels.
%
%        PBN_GAME(ROW_LABELS,COL_LABELS,G) displays the game
%        with labels and with grid G
%
%        PBN_GAME(ROW_LABELS,COL_LABELS,G,FIG_H) displays the game
%        with labels and grid G in figure with handle FIG_H
%
%        G is an RxC array using the following values:
%           1 = empty square                (white)
%           2 = light square                (rose)
%           3 = dark square                 (dark gray)
%           4 = empty square                (white)
%           5 = experimental light square   (yellow)
%           6 = experimental dark square    (blue)
%           7 = experimental overlap square (green)
% 
%        [No Guarantees. M. Lubinski]

% pbn_game requires non-standard functions: helpshow.m
% pbn_game requires help files: pbn_gen.txt  pbn_mous.txt pbn_menu.txt
%                               pbn_hint.txt
% Written      07/10/93 M. Lubinski (lubinski@eecs.umich.edu)
% Last Updated 07/03/95 M. Lubinski

% Check command call
if (nargin<2)
  cmd='init';
  fh='';
end
if ~isstr(cmd)
  rl=cmd;
  cl=cmddata;
  cmd='init';
  if exist('g_f')==1
    fh=g_f;
    if (~(any(get(0,'Children')==fh)))
      error('Invalid figure.');
    end
  else
    fh = '';
  end
end

% INITIIALIZE
if strcmp(cmd,'init')

  % Setup figure
  if (isempty(fh))
    fh=figure;
  else
    set(0,'CurrentFigure',fh)
    clf
  end
  cm = [1 1 1;     1 0.9 0.9; 0.5 0.5 0.5; ...
        1 1 1;     1 1   0.4; 0.5 0.5 1;   ...
	0.4 1 0.4; 0.4 1 0.4]; 		% Colormap
  set(fh,'Name','Paint By Numbers','Colormap',cm,'Pointer','cross',...
      'WindowButtonDown','pbn_game(''m_dn'',gcf);',...
      'WindowButtonUp','pbn_game(''m_up'',[gcf 1]);');

  % Add menus
  f_menu = uimenu('Label','File');
  d_menu = uimenu('Label','DrawMode');
  u_menu = uimenu('Label','Utilities');
  h_menu = uimenu('Label','Help');
  
  uimenu(f_menu,'Label','Load...','Accelerator','l',...
      'Callback','pbn_game(''load'',gcf);');
  uimenu(f_menu,'Label','Save...','Accelerator','s','Enable','off',...
      'Callback','pbn_game(''save'',gcf);','Tag','smenu');
  uimenu(f_menu,'Label','Quit','Separator','on',...
      'Callback','close(gcf);');

  uimenu(d_menu,'Label','Paint','Accelerator','p','Tag','paint',...
      'Callback','pbn_game(''e/p'',[gcf 1]);','Checked','on');
  uimenu(d_menu,'Label','Experimental','Accelerator','e','Tag','exp',...
      'Callback','pbn_game(''e/p'',[gcf 2]);');

  uimenu(u_menu,'Label','Refresh','Accelerator','r',...
      'Callback','refresh(gcf);');
  uimenu(u_menu,'Label','Clear','Accelerator','c',...
      'Callback','pbn_game(''clr'',gcf);');

  uimenu(h_menu,'Label','General','Callback',...
      'helpshow(''pbn_gen.txt'',''PBN General Help'');');
  uimenu(h_menu,'Label','Mouse Buttons','Callback',...
      'helpshow(''pbn_mous.txt'',''PBN Mouse Help'');');
  uimenu(h_menu,'Label','Menu','Callback',...
      'helpshow(''pbn_menu.txt'',''PBN Menu Help'');');
  uimenu(h_menu,'Label','Hints','Callback',...
      'helpshow(''pbn_hint.txt'',''PBN Hints Help'');');

  % Return if no labels to put in
  if (exist('cl')~=1)
    return
  end
  
  % Get grid entries
  [R maxR]=size(rl); 			% Size of row labels
  [maxC C]=size(cl); 			% Size of column labels
  if (exist('G') ~=1) 			% Create grid if necessary
    Grid=ones(R,C);
  else 					% Check grid size
    [r c] = size(G);
    if ((r~=R) | C~=C)
      error('Dimensions of row labels, column labels and grid must agree');
    end
    Grid=G;
  end
  
  % Create grid
  tk_len = 1 / max([R C]); 		% Length of axis tickmarks
  xt=find(rem((1:C),5))+1; 		% Put tick marks on axis except
  yt=find(rem((1:R),5))+1; 		%    when divisible by 5
  g_ax=axes('AspectRatio',[C/R 1],'Tickdir','out','Ticklen',[tk_len tk_len],...
      'Box','on','Ydir','reverse','Xlim',[1 C+1],'Ylim',[1 R+1],...
      'Xtick',xt,'Ytick',yt,'Clim',[1 length(cm)],...
      'Xticklabels',[],'Yticklabels',[],'Position',[0.25 tk_len 0.7 0.7],...
      'Tag','game_axis');
  h=zeros(R,C);
  y=[1 2 2 1];
  for r=1:R
    x=[1 1 2 2];
    for c=1:C
      h(r,c)=patch(x,y,Grid(r,c),'EraseMode','none');
      x=x+1;
    end
    y=y+1;
  end  
  set(g_ax,'Userdata',h); 		% Store handles for modify colors

  % Label rows
  h=zeros(R,maxR);
  for r=1:R 				% Repeat for every row
    n=rl(r,maxR:-1:1);			% Do rightmost label 1st
    for i=find(n>0) 			% Do non-zeros (non-blanks)
      h(r,i)=text('Position',[1.5-i r+0.5],'String',int2str(n(i)),...
	  'HorizontalAlign','center','VerticalAlign','middle');
    end
  end
  set(get(g_ax,'Ylabel'),'UserData',rl); % Store row labels for saving puzzle
  set(get(g_ax,'Zlabel'),'UserData',h); % Store row handles
    
  % Label cols
  h=zeros(maxC,C);
  for c=1:C 				% Repeat for every column
    n=cl(maxC:-1:1,c); 			% Do bottom label 1st
    for i=find(n>0)' 			% Do non-zeros (non-blanks)
      h(i,c)=text('Position',[c+0.5 1.75-i],'String',int2str(n(i)),...
	  'HorizontalAlign','center','VerticalAlign','bottom');
    end
  end
  set(get(g_ax,'Xlabel'),'UserData',cl); % Store col labels for saving puzzle
  set(get(g_ax,'Title'),'UserData',h); 	% Store column handles
  
  % Add lines
  lc = [1 0.1 0.1]; 			% Line color
  for r=1:5:R+1 			% Put lines where divisible by 5
    line('Xdata',[0 C+2],'Ydata',[r-0.01 r-0.01],'Color',lc,'Clipping','off')
    line('Xdata',[0 C+2],'Ydata',[r r],          'Color',lc,'Clipping','off')
    line('Xdata',[0 C+2],'Ydata',[r+0.01 r+0.01],'Color',lc,'Clipping','off')
  end
  for c=1:5:C+1
    line('Xdata',[c-0.01 c-0.01],'Ydata',[0 R+2],'Color',lc,'Clipping','off')
    line('Xdata',[c c],          'Ydata',[0 R+2],'Color',lc,'Clipping','off')
    line('Xdata',[c+0.01 c+0.01],'Ydata',[0 R+2],'Color',lc,'Clipping','off')
  end
  
  % Enable save
  set(findobj(fh,'Tag','smenu'),'Enable','on');
  
  return
end


fh=cmddata(1);
if strcmp(cmd,'m_up')  
  %Button UP or MOTION

  % if button up, remove button motion
  if cmddata(2)==1,set(fh,'WindowButtonMot','');end
  
  % Get needed parameters
  ax=findobj(fh,'tag','game_axis');	% Axis
  if (isempty(ax)) return; end
  l1=get(fh,'Userdata');		% First position
  l2=get(ax,'CurrentPoint');		% Last position
  h=get(ax,'Userdata');			% Grid handles
  [RH CH]=size(h);

  % Return if not a valid selection 
  l1=fliplr(floor(l1(1,1:2)));
  l2=fliplr(floor(l2(1,1:2)));
  if all(l1<1) | any(l1>[RH CH]) 
    set(fh,'WindowButtonMot','');
    return; 
  end

  % Do labels
  if any(l1<1)

    % Determine if row or column label
    if l1(2)<1
      % Row label
      h=get(get(ax,'Zlabel'),'UserData'); % Get handles
      [RH CH]=size(h);
      l1(2)=1-l1(2);
      l2(2)=1-l2(2);
    else
      % Col label
      h=get(get(ax,'Title'),'UserData'); % Get handles
      [RH CH]=size(h);
      l1(1)=1-l1(1);
      l2(1)=1-l2(1);
    end
    
    % Compute row and column indecies  
    if abs(l1(1)-l2(1)) > abs(l1(2)-l2(2)) % row diff > col diff
      c=l1(2); 				% It's a column
      r=max([min([l1(1) l2(1)]) 1]):min([max([l1(1) l2(1)]) RH]); 
    else
      r=l1(1); 				% It's a row
      c=max([min([l1(2) l2(2)]) 1]):min([max([l1(2) l2(2)]) CH]);
    end

    % Color it
    cm=get(fh,'Colormap');
    sel_type = get(fh,'SelectionType');
    if sel_type(1)=='a' 		% Right button  (alt)
      c_idx=cm(1,:);
    else 				% Double click (open)
      c_idx=cm(3,:);
    end
    h=h(r,c);
    set(h(find(h~=0)),'Color',c_idx) % Set colors
    
    return
  end
    
  % Compute row and column indecies  
  if abs(l1(1)-l2(1)) > abs(l1(2)-l2(2)) % row diff > col diff
    c=l1(2); 				% It's a column
    r=max([min([l1(1) l2(1)]) 1]):min([max([l1(1) l2(1)]) RH]); 
  else
    r=l1(1);				% It's a row
    c=max([min([l1(2) l2(2)]) 1]):min([max([l1(2) l2(2)]) CH]);
  end
   
  % Color it
  sel_type = get(fh,'SelectionType');
  if sel_type(1)=='a' 			% Right button  (alt)
    c_idx=1;
  elseif sel_type(1)=='e' 		% Middle button (extend)
    c_idx=2;
  elseif sel_type(1)=='n' 		% Left button (normal)
    c_idx=3;
  else					% Double click (open)
    return
  end
  if strcmp(get(findobj(fh,'tag','paint'),'Checked'),'on') % Paint mode
    set(h(r,c),'Cdata',c_idx) 		% Set colors
  else
    c_idx = c_idx+3; 			% Experimental mode
    if (c_idx==5) 			% Check for overlapped colors
      hc2=findobj(h(r,c),'Cdata',6);
    elseif (c_idx==6)
      hc2=findobj(h(r,c),'Cdata',5);
    else
      hc2=[];
    end	
    hc2=[hc2;findobj(h(r,c),'Cdata',8)];
    set(h(r,c),'Cdata',c_idx) 		% Set colors
    if (cmddata(2)==2)			% if buttonmotion
      set(hc2,'Cdata',8)		% Temp value
    else
      set(hc2,'Cdata',7)
    end
  end
  
elseif strcmp(cmd,'m_dn')  

  % Button DOWN
  ax=findobj(fh,'tag','game_axis');
  if (isempty(ax)) return; end
  set(fh,'Userdata',get(ax,'CurrentPoint'));
  set(fh,'WindowButtonMot','pbn_game(''m_up'',[gcf 2]);');
  
elseif strcmp(cmd,'e/p')

  % Switch between experimental and paint mode
  if (cmddata(2)==1)
    set(findobj(fh,'Tag','paint'),'Checked','on');
    set(findobj(fh,'Tag','exp'),'Checked','off');
  elseif (cmddata(2)==2)
    set(findobj(fh,'Tag','paint'),'Checked','off');
    set(findobj(fh,'Tag','exp'),'Checked','on');
  end

elseif strcmp(cmd,'clr')

  % Clear grid
  ax=findobj(fh,'tag','game_axis');
  if (isempty(ax)) return; end
  set(get(ax,'UserData'),'Cdata',1);	% Clear grid
  cm=get(fh,'Colormap');
  c_idx=cm(1,:);
  h=get(get(ax,'ZLabel'),'UserData');	% Clear row labels
  set(h(find(h~=0)),'Color',c_idx); 
  h=get(get(ax,'Title'),'UserData');	% Clear column labels
  set(h(find(h~=0)),'Color',c_idx); 
  refresh(fh);
  
elseif strcmp(cmd,'load')
  % Load file
  
  % Get filename
  set(fh,'Pointer','watch');
  [sfile,spath] = uigetfile('*.mat','Load file');
  if (sfile == 0)
    set(fh,'Pointer','cross');
    return
  end
  
  % Load file
  disp(sprintf('Loading file %s%s ...',spath,sfile));
  stat=1;
  eval(sprintf('load %s%s',spath,sfile),'stat=0;');
  if ~stat
    disp(['Could not load file ',spath,sfile]);
    set(fh,'Pointer','cross');
    return
  end 
  if (exist('rl')==1) & (exist('cl')==1) & (exist('grid_save')==1)
    pbn_game(rl,cl,grid_save,fh);
    set(findobj(fh,'Tag','smenu'),'Enable','on'); % Enable save
  else
    disp(sprintf('File %s%s is not a valid pbn file',spath,sfile));
  end
  set(fh,'Pointer','cross');
  
  
elseif strcmp(cmd,'save')
  % Save file
  
  % Get stuff to save
  disp('Getting puzzle characteristics...');
  ax=findobj(fh,'tag','game_axis');
  if (isempty(ax)) return; end
  h=get(ax,'UserData');
  [rh ch] = size(h);
  grid_save=zeros(rh,ch);
  for r=1:rh				% Get grid colors
    for c=1:ch
      grid_save(r,c)=get(h(r,c),'Cdata');
    end
  end
  rl=get(get(ax,'Ylabel'),'UserData');	% Get row labels
  cl=get(get(ax,'Xlabel'),'UserData');	% Get column labels
  
  % Save file
  [sfile,spath] = uiputfile('*.mat','Save As');
  if (sfile == 0) return; end
  stat=1;
  eval(sprintf('save %s%s rl cl grid_save',spath,sfile),'stat=0;');
  if stat
    disp(['Wrote file ',spath,sfile]);
  else
    disp(['Could not write file ',spath,sfile]);
  end

else
  error(sprintf('Unknown command %s',cmd));
end

