% Filename: alignme.m
% Rev.Date: 7/10/95
% Class: 	function
%
% USAGE: alignme( Style, Parameters )
%        Style:  0 ==> alignment of selected axes
%                1 ==> spacing of selected axes
%        Parameters:
%         Style 0: align 'left' sides
%                  align 'right' sides
%                  align 'tops' 
%                  align 'bottoms'
%
%         Style 1: space with even gaps in 'L'eft/Right direction
%                  space with even gaps in 'T'op/Bottom direction
%                  space axes centers evenly in 'R'ight/Left direction
%                  space axes centers evenly in 'B'ottom/Top direction
%
%  Alignment operation applied to axes in current figure
%  window which have the object property 'Selected' turned
%  on.

% Written By: Kevin G. Kohrt, kkohrt@engr.iupui.edu
% Written On: 7/6/95 - 7/10/95
%
% Permission granted to include ALIGNME or some variation of it
% with any other script(s), as long no part of this comment section 
% is deleted or changed (excluding the Rev.Date, Filename, & Class). -KGK
%
% Adapted By: 
% Adapted On:  
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function alignme( Style, Method )  

	if nargin == 0; Style = 0; end
  %-------------------------------------------------------------------------%
  % Aquire current position of all selected axes in the current figure
  %-------------------------------------------------------------------------%
	olduni = get( gcf, 'units' );
	set( gcf, 'units', 'points' );
	Axs = findobj( gcf, 'type', 'axes' );
	Axs = findobj( Axs, 'selected', 'on' );
	for idx = 1 : length( Axs )
		Apos( idx, : ) = get( Axs(idx), 'Position' );
	end

	if Style == 0
	  %-----------------------------------------------------------------------%
	  % Ask for method of alignment if not given as input
	  %-----------------------------------------------------------------------%
		if nargin < 2 
			Method = input( 'Align L)eft, R)ight, T)op, or B)ottom? ', 's' );
		end
	  %-----------------------------------------------------------------------%
	  % Verify given method is a string
	  %-----------------------------------------------------------------------%
		if isstr( Method )
			method = lower( Method(1) );
		else
			disp( 'ERROR: Method of alignment must be a string.' )
			set( gcf, 'units', olduni ); return
		end
	  %-----------------------------------------------------------------------%
	  % Verify given method is valid, and convert to number index
	  %-----------------------------------------------------------------------%
		if 		strcmp( method, 'l' ),	pnum = 1;
		 elseif strcmp( method, 'r' ),	pnum = 3;
		 elseif strcmp( method, 't' ),	pnum = 4;
		 elseif strcmp( method, 'b' ),	pnum = 2;
		 else,	disp( [ 'ERROR: Unknown alignment command: ' Method ] )
				set( gcf, 'units', olduni ); return
		end
	  %-----------------------------------------------------------------------%
	  % Aquire current position of all selected axes in the current figure
	  %-----------------------------------------------------------------------%
		Axs = findobj( gcf, 'type', 'axes' );
		Axs = findobj( Axs, 'selected', 'on' );
		for idx = 1 : length( Axs )
			Apos( idx, : ) = get( Axs(idx), 'Position' );
		end
	  %-----------------------------------------------------------------------%
	  % Calculate and implement aligned position
	  %-----------------------------------------------------------------------%
		if pnum < 3
			Align = min( Apos( :, pnum ) );
			for idx = 1 : length( Axs )
				Apos( idx, pnum ) = Align;
				set( Axs( idx ), 'Position', Apos( idx, : ) )
			end
		else
			Align = max( Apos( :, pnum ) + Apos( :, pnum - 2 ) );
			for idx = 1 : length( Axs )
				Apos( idx, pnum-2 ) = Align - Apos( idx, pnum );
				set( Axs( idx ), 'Position', Apos( idx, : ) )
			end
		end
 	%-----------------------------------------------------------------------%
	elseif Style == 1
	  %-----------------------------------------------------------------------%
	  % Ask for method of alignment if not given as input
	  %-----------------------------------------------------------------------%
		if nargin < 2 
			Method = input( [ 'Space L)R gaps, T)B gaps,' ...
							 ' R)L centers, or B)T centers? ' ], 's' );
		end
	  %-----------------------------------------------------------------------%
	  % Verify given method is a string
	  %-----------------------------------------------------------------------%
		if isstr( Method )
			method = lower( Method(1) );
		else
			disp( 'ERROR: Method of alignment must be a string.' )
			set( gcf, 'units', olduni ); return
		end
	  %-----------------------------------------------------------------------%
	  % Verify given method is valid, and convert to number index
	  %-----------------------------------------------------------------------%
		if 		strcmp( method, 'l' ),	pnum = 1; Ga = 1;
		 elseif strcmp( method, 'r' ),	pnum = 1; Ga = 0;
		 elseif strcmp( method, 'b' ),	pnum = 2; Ga = 0;
		 elseif strcmp( method, 't' ),	pnum = 2; Ga = 1;
		 else,	disp( [ 'ERROR: Unknown alignment command: ' Method ] )
				set( gcf, 'units', olduni ); return
		end
		breath = Apos( :, pnum + 2 );
		Minedge = min( Apos( :, pnum ) );
		Maxedge = max( Apos( :, pnum ) + breath );
		Space = ( Maxedge - Minedge - sum( breath ) ) / ( size( Axs, 1 ) - 1 );
			
		Middles = Apos( :, pnum ) + breath/2 ;
		[ Middles, I ] = sort( Middles );
		for idx = 1 : length( I )
			if Ga
				Apos( I(idx), pnum ) = sum( breath(1:idx) ) - breath(idx) ...
									 + Space * (idx - 1) + Minedge;
			else
				Apos( I(idx), pnum ) = Middles( idx ) - breath/2;				
			end
			set( Axs( I(idx) ), 'Position', Apos( I(idx), : ) )
		end 	
	end		

	set( gcf, 'units', olduni );



% 7/10/95  Alignment code written
