function [x1, y1, z1] = approxi3(x, y, z, r, bd)
% APPROXI3 Approximation of a surface z = z(x, y) to reduce the number of pairs.
%          The data points x, y and z are vectors with lengths n, the lengths of 
%          the returned vectors x1, y1 and z1 is n1 (n1 < n). The key of algorithm 
%          is to analyse the first order derivative of the function.
%
%          use:
%          [x1, y1, z1] = approxi3(x, y, z, r, [bd]);
%
%          x  -> x-data of the function (vector)
%          y  -> y-data of the function (vector)
%          z  -> z-data of the function (vector)
%          r  -> value for the rate of increase (scalar)
%                good results for 0.1 < r < 0.5 (please test!)
%          bd -> optional, if bd unequal 0, then comparison of y(x) and y1(x1) 
%                will be plotted
%
% example: t = 0:0.001:5; 
%          y = 2*sin(2*pi*.3*t)+.2*cos(2*pi*3*t);
%          z = real(sqrt(y));
%          [t1, y1, z1] = approxi3(t, y, z, 0.5, 1);
%
%          L. Merz, 1. Sep. 1995
%          DASA-Airbus Hamburg, EF 345, Kreetslag 10, 21129 Hamburg, Germany
%          Phone: +49.40.7437.3367
%          Fax:   +49.40.7437.4187
%          Email 100533.2037@compuserve.com

if nargin == 4,
   bd = 0;
end;
 
fprintf('y = y(x): ');   
[xy1, y1] = approxi(x, y, r);
fprintf('z = z(x): ');   
[xz1, z1] = approxi(x, z, r);

anz_y = length(y1);
anz_z = length(z1);

l = 1;
m = 1;
n = 1;

for i=1:length(x),
    if x(i) == xy1(m) | x(i) == xz1(n),
       x1(l) = x(i);
       y1(l) = y(i);
       z1(l) = z(i);
       l = l+1;
       if x(i) == xy1(m),
          m = m+1;
       end;   
       if x(i) == xz1(n),
          n = n+1;
       end;   
    end;
end;

fprintf('total number of data points \n');
fprintf('before: (x, y)-data points %d, (x, z)-data points %d\n', anz_y, anz_z);
fprintf('later: common data pairs %d\n\n', length(x1));

if bd,
   subplot(311), plot(x, y, x1, y1); 
   grid, xlabel('x, x1'), ylabel('y, y1')
   legend('before','later');
   
   hold on; plot(x1, y1, 'o'); hold off;

   subplot(312), plot(x, z, x1, z1); 
   grid, xlabel('x, x1'), ylabel('z, z1')
   legend('before','later');
   
   hold on; plot(x1, z1, 'o'); hold off;
   
   subplot(313), plot3(x, y, z, x1, y1, z1); 
   hold on, grid, xlabel('x, x1'), ylabel('y, y1'), zlabel('z, z1')
   legend('before','later');
   
   plot3(x1, y1, z1, 'o'); hold off;
end;        

       
