function  fireworks(flag)

% FIREWORKS  Demo for "vectorized" COMETS program.
%    Displays the "fireworks" animation.
%    Interactive setting of main parameters:
%  Nmb. flares - number of "flares" in each "salvo"
%  Nmb. fires  - number of "salvos" to be performed (can be Inf)  
%  Nmb. steps  - number of timesteps in following trajectories of
%                flares.
%  Colors      - colors of flares to be chosen from.
%
%    Parameters can be adjusted for performance on diffrent machines
%    (more flares and timesteps - slower motion).

%  Kirill K. Pankratov, kirill@plume.mit.edu
%  July 14, 1994

 % Defaults ................................................
nflaresdflt = 30;          % Number of flares in each shot
nfiresdflt = inf;          % Number of shots
nstepsdflt = 25;           % Number of timesteps
coloroptdflt = 'rgymcw';   % Possible colors

maxbody = .5;              % Max. body length
vdownmin = .5;             % Min. downward velocity
shift = .3;                % Parameter in time warp
linewidmin = [8 1 1];      % Min. linewidth

 % Parameters of "Fires" ..................................
% Each row of the matrix P contains the parameters of the fire
% configuration and velocity:
%  P(:,1) - relative frequency of this type,
%  P(:,2:6) - amplitudes of azimuthal "modes",
%  P(:,7) - rotation angle for each mode,
%  P(:,8) - randomization parameter for angles of flares
%  P(:,9) - randomization parameter for velocities of flares.
%
%   freq   a0  a1  a2  a3  a4    turn  arand  vrand
P = [25     1   0   0   0   0     0      1    .4;   % Random circle
     8      0   0   0   0   0     0      1   1.5;   % Random bush
     8      1   0   0   0   0     0    .02    .1;   % Circle
     4      1   0  .2   0   0     0    .02     0;   % Ellipse
     4      1   0  .8   0   0     0    .01    .1;   % Two lobes
     3     .8   0   0  .8   0  pi/2    .02   .05;   % Three lobes
     3      0   0   1   0   0     0    .02   .05];  % Four lobes


 % Linestyles ...............................
lsh = '*';    % Head 
lsb = '-';    % Body
lst = '-';    % Tail

 % Controls setting .........................
posfire = [.0 .91 .1 .08];   % Position of the "Fire" button
posdone = [.12 .91 .1 .08];  % Position of the "Done" button
posfrpar = [.0 .64 .22 .26]; % Position of the parameters frame


if nargin==0  % Create the window and controls **********************

  fig = figure('name','Fireworks','numbertitle','off');
  hfire = uicontrol('style','check',...    % Fire button
             'units','norm','pos',posfire,...
             'string','Fire',...
             'call','fireworks(''fire'')');

  hdone = uicontrol('style','push',...     % Done button
             'units','norm','pos',posdone,...
             'string','Done',...
             'call','close');

  hfrpar = uicontrol('style','frame',...   % Parameters frame
             'units','norm','pos',posfrpar);

  possc = [posfrpar 0];
  nparam = 4;

  % Auxillary 
  v1255 = [1 2 5 5];
  v3434 = [3 4 3 4];

  cc = 1/(nparam+1);
  cpos = [.1 1-cc .8 cc*.9];
  cpos = possc(v1255)+cpos.*possc(v3434);
  hpar = uicontrol('style','text',...
             'units','norm','pos',cpos,...
             'string','Parameters');

  cpos = [.0 1-2*cc .55 cc*.9];
  cpos = possc(v1255)+cpos.*possc(v3434);
  hcoltx = uicontrol('style','text',...   % Colors text
             'units','norm','pos',cpos,...
             'string','Colors');

  cpos = [.5 1-2*cc .47 cc*.9];
  cpos = possc(v1255)+cpos.*possc(v3434);
  hcoled = uicontrol('style','edit',...   % Colors edit
             'units','norm','pos',cpos,...
             'string',coloroptdflt);

  cpos = [.0 1-3*cc .64 cc*.9];
  cpos = possc(v1255)+cpos.*possc(v3434);
  hfltx = uicontrol('style','text',...    % Flares text
             'units','norm','pos',cpos,...
             'string','Nmb. flares');

  cpos = [.65 1-3*cc .32 cc*.9];
  cpos = possc(v1255)+cpos.*possc(v3434);
  hfled = uicontrol('style','edit',...    % Flares edit
             'units','norm','pos',cpos,...
             'string',num2str(nflaresdflt));

  cpos = [.0 1-4*cc .64 cc*.9];
  cpos = possc(v1255)+cpos.*possc(v3434);
  hfrtx = uicontrol('style','text',...    % Fires text
             'units','norm','pos',cpos,...
             'string','Nmb. fires');

  cpos = [.65 1-4*cc .32 cc*.9];
  cpos = possc(v1255)+cpos.*possc(v3434);
  hfred = uicontrol('style','edit',...     % Fires edit
             'units','norm','pos',cpos,...
             'string',num2str(nfiresdflt));

  cpos = [.0 1-5*cc .64 cc*.9];
  cpos = possc(v1255)+cpos.*possc(v3434);
  hsttx = uicontrol('style','text',...     % Steps texts
             'units','norm','pos',cpos,...
             'string','Nmb. steps');

  cpos = [.65 1-5*cc .32 cc*.9];
  cpos = possc(v1255)+cpos.*possc(v3434);
  hsted = uicontrol('style','edit',...      % Steps edit
             'units','norm','pos',cpos,...
             'string',num2str(nstepsdflt));

  set(gca,'pos',[.3 0 .7 1])
  axis off
  set(gcf,'user',[hfire hcoled hfled hfred hsted]')  
  return
end

if strcmp(flag,'fire')  % If the button "Fire" is pressed ***********

  hparam = get(gcf,'user');
  hfire = hparam(1);
  coloropt = get(hparam(2),'string');
  nrock = str2num(get(hparam(3),'string'));
  maxcount = str2num(get(hparam(4),'string'));
  nstep = str2num(get(hparam(5),'string'));

  if nrock==[], nrock=nflaresdflt; end
  if maxcount==[], maxcount=nfiresdflt; end
  if nstep==[], nstep=nstepdflt; end

  fire = get(hfire,'value');
  count = 0;
  t = (0:nstep)'/nstep;
  t2 = t.^2;
  r = sqrt(t+shift)-sqrt(shift);
  ot = ones(size(t));
  freq = cumsum(P(:,1));
  freqsum = freq(length(freq));

  % Create lines .....................................
  cla
  lh = ones(3,1);
  lh(1) = line; lh(2) = line; lh(3) = line;
  axis([-.3 1.5 -1 1.5])
  axis off

  while fire    % Begin the main cycle ("fires") ```````````````````

    % Calculate trajectories of flares ................
    % Chose randomly and extract parameters
    num = rand(1)*freqsum;
    num = sum(num>freq)+1;  % Type of the fire
    a = P(num,2:6);         % Parameters of this type
    turn = P(num,7);
    arand = P(num,8);
    vrand = P(num,9);

    ang = ((1:nrock)/nrock+arand*rand(1,nrock))*2*pi-turn;
    v0 = a(1)+a(2)*cos(ang)+a(3)*cos(2*ang);
    v0 = v0+a(4)*cos(3*ang)+a(5)*cos(4*ang);
    vel = v0+vrand*rand(1,nrock);
    R = r*vel;
    vdown = vdownmin+.5*rand(1);
    x0 = rand(1); y0 = rand(1);
    ang = ang+turn;
    x = R.*cos(ang(ot,:))+x0;
    y = R.*sin(ang(ot,:))+y0;
    y = y-vdown*t2(:,ones(1,nrock));       % Gravity works

    lb = floor(maxbody*rand(1)*nstep);     % Body length
    linewid = linewidmin+floor(4*rand(1)); % Linewidth
    col = ceil(rand(1)*length(coloropt));  % Color
    col = coloropt(col);

    % Call COMETS0 ......................................
    comets0(lh,x,y,lb,col,col,'k',lsh,lsb,lst,linewid)

    fire = get(hfire,'value')&count<=maxcount; % Check if stop
    count = count+1;                           % One more fired
    set(gcf,'color',[0 0 0])                   % Erase the garbage
  end    % End fires '''''''''''''''''''''''''''''''''''''''''''''''

  set(hfire,'value',0)     % Release the button

end
