function h=datestamp(pos,stampformat)
% DATESTAMP   Add a datestamp to the current axes.
%
%             DATESTAMP(pos,format) puts the date in format 'format' at
%             location 'pos' on the current axes.  DATESTAMP returns the
%             handle of the text.
%
%             Valid positions are:
%                   'br'   bottom right corner
%                   'bl'   bottom left corner
%                   'tl'   top left corner
%                   'tr'   top right corner
%
%             Date stamp format is an integer 1-4, specifying one of the
%             following forms:
%                   1   yymmdd (e.g., '961004')
%                   2   yymmdd hh:mm:ss (e.g., '961004 10:30:30')
%                   3   dd-Mo.-yy (e.g., '04-Oct-96')
%                   4   Month dd, yyyy (e.g., 'October 4, 1996')
%
%             DATESTAMP with no input arguments puts the date in format 1 in
%             the bottom right corner.
%
%             The position must be a quoted 2-letter string.  Invalid
%             positions and formats disable the DATESTAMP function.
%
%             See also CLOCK, DATE.

% Peter Newbury, 08-Oct-1996
% newbury@math.ubc.ca, www.iam.ubc.ca/spider/newbury/home.html

if nargin==0,
  pos='br';
  stampformat=1;
elseif nargin~=2,
  disp('DATESTAMP requires ''pos'' and format.  Try help datestamp.')
  return
elseif isstr(pos)==0,
  disp('DATESTAMP argument ''pos'' must be quoted.  Try help datestamp.')
  return
end


% Die if no axes
if isempty(findobj('type','axes'))==1,
  disp('No axes exist.  Datestamp disabled.')
  return
end

% parse the datestamp position
pos=lower(pos);
limits=axis;
if strcmp(pos,'br')==1,
  % bottom right
  x0=limits(2);dx=-1;
  y0=limits(3);dy=1;
  horizontalalignment='right';
  verticalalignment='bottom';
elseif strcmp(pos,'bl')==1,
  % bottom left
  x0=limits(1);dx=1;
  y0=limits(3);dy=1;
  horizontalalignment='left';
  verticalalignment='bottom';
elseif strcmp(pos,'tl')==1,
  % top left
  x0=limits(1);dx=1;
  y0=limits(4);dy=-1;
  horizontalalignment='left';
  verticalalignment='top';
elseif strcmp(pos,'tr')==1,
  % top right
  x0=limits(2);dx=-1;
  y0=limits(4);dy=-1;
  horizontalalignment='right';
  verticalalignment='top';
else
  disp(['Invalid datestamp position ''' pos '''.  Try help datestamp.'])
  return
end

% Add a buffer between the datestamp and the plot box lines which would
% otherwise obscure the text.  There are only two types of scales, linear
% and log (which is actually log_10.)
xscale=get(gca,'Xscale');
if strcmp(xscale,'linear'),
  xbuffer=0.02*(limits(2)-limits(1));
  x=num2str(x0+dx*xbuffer);
else
  xbuffer=0.02*log10(limits(2)/limits(1));
  x=num2str( x0*10^(dx*xbuffer) );
end

yscale=get(gca,'Yscale');
if strcmp(yscale,'linear'),
  ybuffer=0.02*(limits(4)-limits(3));
  y=num2str(y0+dy*ybuffer);
else
  ybuffer=0.02*log10(limits(4)/limits(3));
  y=num2str( y0*10^(dy*ybuffer ) );
end

% Datestamp formats, constructed from [yyyy mm dd hh mm ss]=fix(clock)
now=fix(clock);
% HA! No 'year=00' problems here!
century=1900;

% abbreviated month names
month=str2mat('January','February','March','April','May','June',...
    str2mat('July','August','September','October','November','December'));

if stampformat==1,
  % yymmdd
  stamp=sprintf('%2d%02d%02d',now(1)-century,now(2),now(3));

elseif stampformat==2,
  % yymmdd hh:mm:ss
  stamp=sprintf('%2d%02d%02d %02d:%02d:%02d',...
      now(1)-1900,now(2),now(3),now(4),now(5),now(6));

elseif stampformat==3,
  % dd-Mo.-yy
  stamp=sprintf('%02d-%3s-%02d',now(3),month(now(2),1:3),now(1)-century);

elseif stampformat==4,
  % Month dd, yyyy
  stamp=sprintf('%s %d, %d',deblank(month(now(2),:)),now(3),now(1));

% Add more 'stampformat's here
% elseif stampformat==...

else
  disp(sprintf(...
      'Datestamp format %d not recognised.  Try help datestamp.',...
      stampformat))
  return
end

% Put the datestamp onto the current axes:
foo=['text(' x ',' y ',''' stamp ''')'];
h=eval(foo);

foo=['set(h'...
      ',''horizontalalignment'',''' horizontalalignment ...
      ''',''verticalalignment'',''' verticalalignment ...
      ''',''fontsize'',8,''color'',[1 1 1])' ];
eval(foo);

