function hkey=key(x,y,textstr,backgroundfill,fontname,fontsize)
% KEY   Creates/modifies a key object
%
% key(x,y,textstr,bfill,fontname,fontsize)
%       - will create/overwrite a key describing data in the current axis
%
% x,y       - Normalized position relative to the bottom left
%             corner of the current axis.
% textstr   - Matrix of textstrings to use for each line object.
% bfill     - Background fill and box. Will fill in the background
%             and put a box around the the key if 1. No background fill
%             or box if 0.
%             (optional param, default 1)
% fontname  - Fontname of text.
%            (optional param)
% fontsize  - Fontsize of text.
%            (optional param)
%
% key(x,y) - will re-position an existing key
%
% e.g.  
%      plot(1:10,1:10,'y',1:10,(1:10)*2,'b-');
%      hkey=key(0.05,0.95,['Graph 1';'Graph 2']);
%
% See also DLKEY, DELKEY
%
[hdata,htext,hkey]=figga(gca); % get list of associated axis to the current one

if hkey~=0
   axes(hkey);
end


% get axis position etc in inches
cunits=get(hdata,'units');
set(hdata,'units','inches');
dpos=get(hdata,'position');
plotwidth=dpos(3);
plotheight=dpos(4);
set(hdata,'units',cunits);


% simply repositioning
if hkey~=0 & nargin==2
% set the position
   cunits=get(hdata,'units');
   set(hdata,'units','normalized');
   ndpos=get(hdata,'position');
   set(hdata,'units',cunits);
   kpos=get(hkey,'position');
   set(hkey,'position',[ndpos(1)+x*ndpos(3) ndpos(2)+y*ndpos(4)-kpos(4) kpos(3) kpos(4)]);
   return;
end

% not enough input parameters
if nargin<3
   error('Need some more parameters!')
end

% use original positions
if hkey~=0 & length(x)==0 & length(y)==0
% set the position
   cunits=get(hdata,'units');
   set(hdata,'units','normalized');
   ndpos=get(hdata,'position');
   set(hdata,'units',cunits);
   kpos=get(hkey,'position');
   x=(kpos(1)-ndpos(1))/ndpos(3);
   y=(kpos(2)+kpos(4)-ndpos(2))/ndpos(4);
end


% obtain line objects from data axis
hltest=findobj(hdata,'type','line');
[m,n]=size(hltest);
hline=zeros(m,1);
hkeytext=zeros(m,1);
hkeyline=zeros(m,1);
hkeyextr=zeros(m,1);
no_graphs=0;
for i=1:length(hltest)
   ud=get(hltest(i),'userdata');
   [m,n]=size(ud);
   if m>0
      if strcmp(ud(1,:),'dataline')
         no_graphs=no_graphs+1;
         hline(no_graphs)=hltest(i);
	 if hkey~=0
            hkeyline(no_graphs)=abs(ud(2,1));
            hkeytext(no_graphs)=abs(ud(2,2));
         end
         if hkeyline(no_graphs)~=0
            udk=get(hkeyline(no_graphs),'userdata');
            hkeyextr(no_graphs)=abs(udk(2,3));
         end
      end
   else
      no_graphs=no_graphs+1;
      hline(no_graphs)=hltest(i);
   end
end

if no_graphs==0
   disp('Error creating key: no data lines in current axis');
   return;
end

% 
textwidths=zeros(no_graphs,1);

% lets set optional parameters
if nargin<6
   fontsize=get(0,'defaulttextfontsize');
   hkt=hkeytext(hkeytext~=0);
   if length(hkt)>0
% the text object of the first data line
      fontsize=get(hkt(1),'fontsize');
   end
end
if nargin<5
   fontname=get(0,'defaulttextfontname');
   hkt=hkeytext(hkeytext~=0);
   if length(hkt)>0
% the text object of the first data line
      fontname=get(hkt(1),'fontname');
   end
end
if nargin<4
   if hkey==0
      backgroundfill=1;
   else
      keyvisible=get(hkey,'visible');
      if strcmp(keyvisible,'on') 
         backgroundfill=1;
      else                    
         backgroundfill=0;
      end
   end
end

% create axis if there's not already one there
newkey=(hkey==0);
if hkey==0
   if backgroundfill
      visible='on';
   else
      visible='off';
   end
   hkey=axes('units','inches','xlimmode','manual','ylimmode','manual',...
             'nextplot','add','color',[0 0 0],...
		'position',[1 1 1 1],...
		'xlim',[0 1],'ylim',[0 1],...
		'xtickmode','manual',...
		'ytickmode','manual',...
                'yticklabelmode','manual',...
                'xticklabelmode','manual',...
		'box','on',...
		'visible',visible);
   set(hkey,'units','normalized');
% set userdata areas in all axes as applicable
   figsa(hdata,htext,hkey);
% this is used by figedit to move the object
   hpatch=patch(0,0,[0 0 0],...
		'clipping','off','edgecolor',[1 1 1],...
                'visible','off','userdata','hpatch');
% this is used to calculate a typical fontheight
   hchar=text(0,0,'H','units','inches','visible','off',...
			'userdata','hchar');
else
   hpatch=findobj(hkey,'userdata','hpatch');
   hchar=findobj(hkey,'userdata','hchar');
end

% need to create text and line objects for lines not possessing them!
for i=1:no_graphs
   if hkeytext(i)==0
      hkeytext(i)=text(0,0,'','units','data','fontname',fontname,...
		                'verticalalignment','middle',...
		                'horizontalalignment','left',...
				'fontsize',fontsize);
      hkeyline(i)=line(0,0);
% always create an extraline
      ude='extraline';
      ude(2,1)=hkeyline(i);
      hkeyextr(i)=line(0,0,'userdata',setstr(ude),'visible','off');
      ude='dataline';
      ude(2,3)=hkeyextr(i);
      set(hkeyline(i),'userdata',setstr(ude));
% and also set the handles of the key line and text objects 
% in the dataline's userdata area
      ud=get(hline(i),'userdata');
      if length(ud)==0
         ud='dataline'; 
      end
      ud(2,1)=hkeyline(i);
      ud(2,2)=hkeytext(i);
      set(hline(i),'userdata',setstr(ud));
   end
end

if backgroundfill
  set(hkey,'visible','on')
else
  set(hkey,'visible','off')
end
set(hchar,'fontname',fontname,'fontsize',fontsize);
ext=get(hchar,'extent');
fontheight=ext(4);
border=fontheight/3;
linewidth=fontheight*1.5;

xtext=2*border+linewidth;
yoff=fontheight/2;

no_dispgraphs=0;
[no_texts,n]=size(textstr);
if no_texts<no_graphs
% not enough text strings so pad with zeros
   if n==0 n=1; end
   textstr=[textstr;setstr(zeros(no_graphs-no_texts,n))];
end
markers='*o+x';
for i=1:no_graphs
   itext=no_graphs-i+1;
   visible=get(hline(i),'visible');
% empty string is one with all spaces/zeros 
   emptystr=all((textstr(itext,:)==' ')|(textstr(itext,:)==0));
   if (strcmp(visible,'on')) & (~emptystr)
      set(hkeytext(i),'units','inches');
      set(hkeytext(i),'position',[xtext yoff],...
  		   'string',textstr(itext,:),...
                   'visible','on',...
                   'fontname',fontname,...
                   'fontsize',fontsize);
      ext=get(hkeytext(i),'extent');
      set(hkeytext(i),'units','data');
      textwidths(i)=ext(3);
% extract colour and linestyle of main line
      ls=get(hline(i),'linestyle');
      col=get(hline(i),'color');
% just draw in the middle for markers
      if any(ls(1)==markers)
          xlin=border+linewidth/2;
          ylin=yoff;
      else
          xlin=[border border+linewidth];
          ylin=[yoff yoff];
      end
      set(hkeyline(i),'xdata',xlin,'ydata',ylin,'linestyle',ls,'color',col,...
                      'visible','on');
% extract line style and colour of any extra lines
      ud=get(hline(i),'userdata');
      if ud(2,3)~=0
         ls=get(abs(ud(2,3)),'linestyle');
         col=get(abs(ud(2,3)),'color');
% just draw in the middle for markers
         if any(ls(1)==markers)
            xlin=border+linewidth/2;
            ylin=yoff;
         else
            xlin=[border border+linewidth];
            ylin=[yoff yoff];
         end
         set(hkeyextr(i),'xdata',xlin,'ydata',ylin,'linestyle',ls,'color',col,...
                      'visible','on');
      else
         set(hkeyextr(i),'visible','off');
      end

      yoff=yoff+fontheight;
      no_dispgraphs=no_dispgraphs+1;
   else
      set(hkeytext(i),'visible','off',...
  		   'string',textstr(itext,:));
      set(hkeyline(i),'visible','off');
      if hkeyextr(i)~=0
         set(hkeyextr(i),'visible','off');
      end
   end
end

keywidth=max(textwidths)+3*border+linewidth;
keyheight=fontheight*no_dispgraphs+0.5*border;

% set size and position of key axis
pos=[dpos(1)+x*dpos(3) dpos(2)+y*dpos(4)-keyheight keywidth keyheight];
set(hkey,'units','inches');
set(hkey,'position',pos);
set(hkey,'units','normalized');

% set its co-ordinate frame to be its size in inches
set(hkey,'xlim',[0 keywidth],'ylim',[0 keyheight] );
% set patch to be the size of the axis
set(hpatch,'xdata',[0 keywidth keywidth 0],...
           'ydata',[0 0 keyheight keyheight]);
% set the handle of the key axis in the original
axes(hdata);
return
end
