function cplot(x,y,par)
%CPLOT	An enhanced plot command for PostScript printers. CPLOT
%	plots vectors or matrices with selectable line thickness
%	(not visible on screen) and in colour if the printer has
%	Colour-PostScript capabilities. Instead of GPP the commands
%	MET2PS or MET2EPS are required for postprocessing of the
%	graphics meta-file. 
%	CPLOT(X,Y) plots X versus Y in black and at standard line
%	thickness.  CPLOT(X,Y,'-r4') plots the curve in red and at
%	a linewidth of approximately 1/2 point (the unit of measure
%	for the linewidth is 1/8 point in portrait format and about
%	1/5 point in landscape format; the effect of the linewidth
%	parameter is only visible on the printout and not on the
%	screen). The linewidth parameter must be an integer number.
%	Default linewidth is set to 3/8 points (portrait format).
%	CPLOT(X,Y,'--r4') uses a dashed line instead (see PLOT).
%	CPLOT(X,Y,'c+') uses a cyan point symbol + for drawing.
%	Valid printer colours are:
%
%	 black  k	red    r	green   g	blue    b
%	 white  w	yellow y	magenta m	cyan	c
%
%	On noncolour printers different shades of gray will be
%	reproduced instead.
%	CPLOT(Y) plots the columns of Y versus their index. PLOT(Y)
%	is equivalent to PLOT(real(Y),imag(Y)) if Y is complex.
%	In all other uses of PLOT, the imaginary part is ignored.
%	See also: PLOT, MET2PS, MET2EPS, CTEXT
%	CLOGLOG, CSLOGX and CSLOGY are used for log and semi-log 
%	scales.
%	This is for MATLAB version 3.5 !

%	@LEM  U. Kaestli 7-7-93
%
%	calls getplarg.m for breaking up the plot parameter string
%
if nargin < 1, error('Not enough input arguments.'); end
if nargin > 3, error('Too many input arguments.'); end

%set defaults

lwuscale  = '0.218';	% scale 1 linewdith_unit to approx. 1/8 point
linewidth = '3';	% default linewidth 3/8 points
linecol   = 'c1';	% black color for all lines
linetype  = '';		% auto selection of linetype

%scan input arguments

if nargin == 1,
  if any(any(imag(x)) ~= 0),	% handle complex x
    y=imag(x);
    x=real(x);
  else
    [r,c]=size(x);
    if min([r,c]) == 1	% x is a vector
      y=x;
      x=1:max([r,c]);
    else		% x is a matrix
      y=x;
      x=1:r;
    end
  end

elseif nargin >= 2,
  if isstr(y),		% only x is a vector or matrix
    par=y;		% y is string for ploting parameters
    if any(any(imag(x)) ~= 0),	% handle complex x
      y=imag(x);
      x=real(x);
    else
      [r,c]=size(x);
      if min([r,c]) == 1,	% x is a vector
	y=x;
        x=1:max([r,c]);
      else			% x is a matrix
	y=x;
        x=1:r;
      end
    end
  end
% break the plot parameter string into line type, color and width

  if exist('par') == 1,
    [col,ltype,width] = getplarg(par);
    if length(col) ~= 0,
      linecol=col;
      if length(ltype) ~= 0,
	linetype=ltype;
      else			% override autotranslation of colour
	if min(size(y)) == 1,	% into a line pattern performed by gpp
	  linetype='-';		% for vectors, when no line pattern is
	end			% is specified
      end
    else
      if length(ltype) ~= 0,
	linetype=ltype;
      end
    end
    if length(width) ~= 0,
      linewidth=width;
    end
  end
end    


% plot axis if necessary (axis needs to be ploted
% separately from curve in order to insert line and
% color information for the curve in between)
[axis_state,ax] = inquire('axis');
hold_state = inquire('hold');
if hold_state == 0,
  if axis_state == 0,
    subpl=inquire('subplot');    
    plot([min(min(x)) max(max(x))],[min(min(y)) max(max(y))],'i')
    ax=axis;
    subplot(subpl);
  end
  plot(ax(1:2),ax(3:4),'.')
  hold on
end

% set color and plot curve:
% do insert a string with "escape" sequences at an "invisible"
% position, which can be interpreted by MET2PS (actually by
% the PostScript printer)
setcolor=['#P',linecol,' ',linewidth,' ',lwuscale,' mul Pc#'];
text(1,.5,setcolor,'sc')	% set color/linewidth
  plot(x,y,[linecol linetype])
text(1,.5,'#PEc#','sc')		% reset color/linewidth
if hold_state == 0,
  hold off
  if axis_state == 1,
    axis(ax);
  end
end
end
