function guimaker(command,param1)
%
%  guimaker.m
%
%	In order to reduce the time it takes to create a GUI front to any
%	custom application, I created a generic GUI development package called
%	"guimaker".  It allows the user to place uicontrol and text (static 
%	and dynamic) objects in any desired location of a figure window using
%	the mouse.  After the GUI has the desired look to it, the user can have
%	guimaker automatically create a M-file function Template.
%
%	This M-file function Template is fully functional and can be executed  
%	at the matlab command.  The user must then only add in the "guts" to
%	the file so that the buttons perform the desired actions (ie. run
%	other functions, perform calculations, etc..).  I have made this as
%	easy as I thought possible to do (Comments and critisms are 
%	appreciated).
%
%
%	To run the application just type:
%
%		guimaker
%
%	at the matlab prompt.
%
%
%	To edit an old one template (eg. saved as templatename.m) type:
%
%               templatename
%		guimaker('edit',gcf)
%
%       at the matlab prompt (see Note*1).
%
%       Note*1  Templates created with version 1 will lose the handle name
%	           that was saved previously.
%	        Templates created with version 1.1 will have handle names
%	           reappear if templatename has not been tampered with (ie
%	           edited or modified (especially 'userdata')).
%
%
% 	This utility helps create a Matlab GUI and saves a runnable
%	m-file function (default name is "template.m").
%
%	HELPFUL HINTS:
%
%		Size the window the way you want it to look at the start.
%		  The size of the window will be saved in the template
%		  in addition to its location.
%
%		Create frame objects at start.  You can always delete them
%		  if you have too many.  Place them off to the side, and after
%		  the other uicontrols have been placed and sized then resize
%		  the frame objects appropriately.
%
%		Selection of Objects (ie text or uicontrol) can be done with
%		  mouse.  For uicontrol objects you also have the option of
%		  selecting them with the appropriate choice under the "Select"
%		  menu.
%
%		The Short M-file Template option should suit most users.  The 
%		  file that is created is ready to go and only needs to have
%		  the "guts" filled in.  It can be run before the "guts"
%		  are filled in.  The Default filename is "template" (if
%		  left blank in Create M-file dialog box.
%
%		With each uicontrol there is a text string associated with it.
%		  Its the text that moves around with the uicontrol object 
%		  when you move it around in the figure with the mouse see
%		  fig 1.  This text can be edited also, so that when you
%		  create the template you will have functionally descriptive 
%		  handle in your file, otherwise it will have a default name 
%		  ('h(index)').  Intelligent name choice can make the use of
%		  radio buttons easy - see help guimaker.
%
%		  If you select an item with the mouse or from the menu it
%		  will be highlighted with so that you know which object your
%		  working with. 
%
% 	                                +-----------+
%	                                |handle name|
%	              +---------------+ +-----------+
%	              |               |
%	              |   uicontrol   |
%	              |               |
%	              +---------------+
%
%	                          fig. 1.
%
%		The text that accompanies the uicontrol objects will not
%		  show up in the m-file template. These are used to give
%		  a functional name to each uicontrol object.  MAKE SURE
%		  THAT EACH UICONTROL OBJECT HAS A UNIQUE FUNCTIONAL NAME,
%		  or else you may have problems running the created template.
%		  Exceptions : radio(1) radio(2) ... would be considered
%			       unique.
%
%		  ALSO MAKE SURE THAT THE FUNCTIONAL NAME IS NOT THE NAME
%		  OF A M-FILE FUNCTION. If this rule is not followed the
%		  callback of a uicontrol will call another function and
%		  create problems (ie. execution errors).
%
%		  In addition, I find it useful to name all my uicontrol
%		  handles with a "h" or "h_" preceding the name. I also
%		  know that none of my custom functions will start with
%		  a "h_" and thereby eliminate the problem of the previous
%		  rule.
%
%		When creating a group of radio buttons use names like
%		  radio(1), radio(2), radio(3) ... for grouping them together
%		  so that when you fill in the code you can access all of them
%		  at the same time.  For instance to reset all of these radio
%		  buttons one could use set(radio,'value',1) and all radio
%		  buttons would be reset to the same value. Use a parameter
%		  with each one to identify which was radio button was pressed
%		  and then set(radio(parameter),'value',0) . Get the idea?!
%		  If not send me mail (pmarchan@motown.ge.com).
%
%		Dynamic text is very usefull with "sliders".  You can have the
%		  text change with the value of the slider see fig 2.
%
%	                    +-----+
%	                    | 1.3 | <------slider numerical value string
%	              +---------------+
%	              |<|    |      |>| <-------slider uicontrol object.
%	              +---------------+
%
%
%
%	DESIRED UPDATES IN UPCOMING VERSIONS:
%
%		1. Menu bar creation added in.
%		2. More text properties (ie font, size , rotation ...) editable.
%		3. Requests from users.
%
%
%	BUGS:
%
%
%
%	REQUIRED CUSTOM M-FILES:
%
%		mk_ui.m
%		mk_ui_t.m
%		mk_tmplt.m
%		mk_txt_t.m
%		edit_ui.m
%		edit_txt.m
%		find_ui.m
%		find_txt.m
%		isobj.m
%		mat2str2.m
%		moveobj.m
%		movetext.m
%		sco.m
%		valid_fn.m
%
%
%	MENUS and a brief description of what they do:
%
%	  Create->
%		Uicontrol->
%			pushbutton
%			popup
%			edit
%			slider
%			checkbox
%			radiobutton
%			frame
%		Text->
%			Static
%			Dynamic
%		M-file->
%			Short Template
%			  : Create an M-file of the uicontrols in short form.
%			    List some editable uicontrol properties.
%			Medium Template
%			  : Create an M-file of the uicontrols in medium form.
%			    List most editable uicontrol properties.
%			Long Template
%			  : Create an M-file of the uicontrols in long form.
%			    List all editable uicontrol properties.
%	  Edit->
%		Move Object
%			: Move the currently selected uicontrol or text object
%			  using the mouse.
%		Resize Object
%			: Resize the currently selected uicontrol object's size
%			  using the mouse.
%		Object Attributes->
%			string
%				: Edits the currently selected object's
%				  string property.
%			Color Object->
%			     BackGround->
%				color list
%				: Edits the currently selected object's
%				  BackGround color property. For a text object
%				  this will modify the text color.
%			     ForeGround->
%				color list
%				: Edits the currently selected object's
%				  ForeGround color property. For a text object
%				  this will modify the text color.
%		List Object Properties
%			: Lists the currently selected object's properties.
%		Delete Object
%			: Deletes the currently selected object.
%	  Select->
%		: List of available uicontrols that have been created.
%		  Choosing from this list makes that uicontrol active
%		  so that it can be edited.
%		: All objects can also be chosen directly with the mouse.
%
%  usage:
%		guimaker(command,param1);
%  where
%
%		command : command number defining some operation or event
%			  that should take place.
%		param1  : parameter passed down to the defined command.
%
%  Author :  Patrick Marchand
%  Mods.  :  mat2str2.m used (custom made mat2str.m function)
%            Sort uicontrol before creating template to make sure
%               that frames are written first.
%            Allow for better reediting of previosly created templates
%               using guimaker('edit',gcf) after the current figure
%               has the previously created template run.
%  Version:  1.1

%  Copyright (c) 1994 by Patrick Marchand 
%       Permission is granted to modify and re-distribute this 
%	code in any manner as long as this notice is preserved. 
%	All standard disclaimers apply. 


if nargin == 0
   command = 'new';
end

if isstr(command)
  if strcmp(lower(command),'initialize')
	command = 0;
  elseif strcmp(lower(command),'new')
	command = 0;
  elseif strcmp(lower(command),'edit')
	command = 0;
  end
else
   hand_list = get(gcf,'userdata');
   if length(hand_list) > 0
	m1 = hand_list(1);
	m2 = hand_list(2);
	m3 = hand_list(3);
	hlist = hand_list(4:length(hand_list));
   end
end

  if command == 0
      if nargin ~= 2
	figure
      else
        figure(param1);
      end
	set(gcf,'pointer','watch');
	txt2 = ['if isobj(gco),sco(gco);else,' ...
		'if isobj(get(gca,''userdata'')),'...
		'set(get(gca,''userdata''),''selected'',''off'');end;end'];
	set(gcf,'WindowButtonDownFcn',txt2);
	set(gca,'visible','off');
	m1 = uimenu('label','Create');drawnow
	  m1a = uimenu(m1,'label','Uicontrol');drawnow
		uimenu(m1a,'label','Pushbutton',...
			  'callback',['mk_ui(''pushbutton'')' ...
			  	      ';guimaker(1)']);
		uimenu(m1a,'label','Popup',...
			  'callback',['mk_ui(''popup'')' ...
			  	      ';guimaker(1)']);
		uimenu(m1a,'label','Edit',...
			  'callback',['mk_ui(''edit'')' ...
			  	      ';guimaker(1)']);
		uimenu(m1a,'label','Slider',...
			  'callback',['mk_ui(''slider'')' ...
			  	      ';guimaker(1)']);
		uimenu(m1a,'label','Check box',...
			  'callback',['mk_ui(''checkbox'')' ...
			  	      ';guimaker(1)']);
		uimenu(m1a,'label','Radio button',...
			  'callback',['mk_ui(''radiobutton'')' ...
			  	      ';guimaker(1)']);
		uimenu(m1a,'label','Text',...
			  'callback',['mk_ui(''text'')' ...
			  	      ';guimaker(1)']);
		uimenu(m1a,'label','Frame',...
			  'callback',['mk_ui(''frame'')' ...
			  	      ';guimaker(1)']);
	  m1b = uimenu(m1,'label','Text');
		uimenu(m1b,'label','Static',...
			  'callback','guimaker(1.1,1);');
		uimenu(m1b,'label','Dynamic',...
			  'callback','guimaker(1.1,2);');
		uimenu(m1b,'label','Associated Dynamic',...
			  'callback','guimaker(1.1,3);');

	  m1c = uimenu(m1,'label','M-file');

		uimenu(m1c,'label','Short Template',...
			'callback',['set(gcf,''pointer'',''watch'');' ...
				    'mk_tmplt(''short'');'...
				    'set(gcf,''pointer'',''arrow'');']);
		uimenu(m1c,'label','Medium Template',...
			'callback',['set(gcf,''pointer'',''watch'');' ...
				    'mk_tmplt(''medium'');'...
				    'set(gcf,''pointer'',''arrow'');' ...
				    'guimaker(1);']);
		uimenu(m1c,'label','Long Template',...
			'callback',['set(gcf,''pointer'',''watch'');' ...
				    'mk_tmplt(''long'');'...
				    'set(gcf,''pointer'',''arrow'');' ...
				    'guimaker(1);']);

	m2 = uimenu('label','Edit');drawnow
		uimenu(m2,'label','Move Object',...
			  'callback','guimaker(2.0,1)');
		uimenu(m2,'label','Resize Object',...
			  'callback','guimaker(2.0,2)');

	  m2a = uimenu(m2,'label','Object Properties');
		uimenu(m2a,'label','String',...
			  'callback',['guimaker(2.3,''string'')']);
	    m2a1 = uimenu(m2a,'label','Color Object');
	      m2a1a = uimenu(m2a1,'label','Background');
	      m2a1b = uimenu(m2a1,'label','Foreground');
		uimenu(m2a1a,'label','standard',...
			   'callback',['guimaker(2.1,''standard'')']);
		uimenu(m2a1a,'label','red',...
			   'callback',['guimaker(2.1,''red'')']);
		uimenu(m2a1a,'label','yellow',...
			   'callback',['guimaker(2.1,''yellow'')']);
		uimenu(m2a1a,'label','green',...
			   'callback',['guimaker(2.1,''green'')']);
		uimenu(m2a1a,'label','blue',...
			   'callback',['guimaker(2.1,''blue'')']);
		uimenu(m2a1a,'label','cyan',...
			   'callback',['guimaker(2.1,''cyan'')']);
		uimenu(m2a1a,'label','white',...
			   'callback',['guimaker(2.1,''white'')']);
		uimenu(m2a1a,'label','black',...
			   'callback',['guimaker(2.1,''black'')']);

		uimenu(m2a1b,'label','standard',...
			   'callback',['guimaker(2.2,''standard'')']);
		uimenu(m2a1b,'label','red',...
			   'callback',['guimaker(2.2,''red'')']);
		uimenu(m2a1b,'label','yellow',...
			   'callback',['guimaker(2.2,''yellow'')']);
		uimenu(m2a1b,'label','green',...
			   'callback',['guimaker(2.2,''green'')']);
		uimenu(m2a1b,'label','blue',...
			   'callback',['guimaker(2.2,''blue'')']);
		uimenu(m2a1b,'label','cyan',...
			   'callback',['guimaker(2.2,''cyan'')']);
		uimenu(m2a1b,'label','white',...
			   'callback',['guimaker(2.2,''white'')']);
		uimenu(m2a1b,'label','black',...
			   'callback',['guimaker(2.2,''black'')']);


		uimenu(m2,'label','List Object Props',...
			  'callback',['guimaker(2.4)']);

		uimenu(m2,'label','Delete Object',...
			  'callback',['guimaker(2.5)'], ...
			  'Separator','on');

	m3 = uimenu('label','Select');drawnow
	  hlist = find_ui;

	set(gcf,'userdata',[m1 m2 m3 hlist]);
	pause(.1);
	guimaker(1);
	set(gcf,'pointer','arrow');

  elseif command == 1
	if isobj(m3)
	  delete(m3)
	end
	m3 = uimenu('label','Select');
	  hlist = find_ui;
	  for loop1 = 1:length(hlist)
	    txt = get(get(hlist(loop1),'userdata'),'string');
	    uimenu(m3,'label',txt,...
		      'callback',['guimaker(3,' ...
			'''hlist(' num2str(loop1) ')'')']);
	  end
	set(gcf,'userdata',[m1 m2 m3 hlist]);
	if length(hlist) > 0
	 if isobj(hlist(1))
	  etxt = ['sco(hlist(1))'];eval(etxt);
	 end
	end

  elseif command == 1.1
	if param1 == 1
		txt = input('Enter Static string: ','s');
		if length(txt) > 0
		   ht_tmp = text(-.1,-.1,txt);
		   sco(ht_tmp);
		end
	elseif param1 == 2
		txt = input('Enter Dynamic string: ','s');
		if length(txt) > 0
		   ht_tmp = text(-.1,-.1,txt,'userdata','dynamic');
		   sco(ht_tmp);
		end
	elseif param1 == 3
		txt = input('Enter Associated Dynamic string: ','s');
		disp('Now select the Uicontrol button it is associated with.')
		button = waitforbuttonpress;
		if strcmp(get(gco,'type'),'uicontrol')
		  if length(txt) > 0
		   ht_tmp = text(-.1,-.1,txt,'userdata',gco);
		  end
		else
		   disp('You did not select a uicontrol button.');
		   disp('   Dynamic text creation cancelled.');
		end
	end

  elseif command == 1.2
	if isobj(m3)
	  delete(m3)
	end
	m3 = uimenu('label','Select');
	  hlist = find_ui;
	  for loop1 = 1:length(hlist)
	    txt = get(get(hlist(loop1),'userdata'),'string');
	    uimenu(m3,'label',txt,...
		      'callback',['guimaker(3,' ...
			'''hlist(' num2str(loop1) ')'')']);
	  end
	set(gcf,'userdata',[m1 m2 m3 hlist]);

  elseif command == 2.0
	if isobj(gco)
	  if param1 == 1
	    % Move Object
	     if strcmp(get(gco,'type'),'uicontrol')
		moveobj(1,1);
	     elseif strcmp(get(gco,'type'),'text')
		movetext(1);
	     else
		disp('A uicontrol or text object must first be selected.');
	     end
	  elseif param1 == 2
	    % Resize Object
	     if strcmp(get(gco,'type'),'uicontrol')
		moveobj(2,1);
	     else
		disp('A uicontrol object must first be selected.');
	     end
	  end
	end

  elseif command == 2.1 | command == 2.2
	if isobj(gco)
	  if strcmp(get(gco,'type'),'uicontrol')
	    if strcmp(param1,'standard')
	      if command == 2.1
		set(gco,'BackgroundColor',[0.701961 0.701961 0.701961]);
	      else
		set(gco,'ForegroundColor',[0.701961 0.701961 0.701961]);
	      end
	    else
	      if command == 2.1
		set(gco,'BackgroundColor',param1);
	      else
		set(gco,'ForegroundColor',param1);
	      end
	    end
	    set(gco,'selected','off');
	  elseif strcmp(get(gco,'type'),'text')
	    set(gco,'color',param1);
	    set(gco,'selected','off');
	  else
	    disp('You must first select a uicontrol button or text object');
	  end
	end

  elseif command == 2.3
	if isobj(gco)
	  if strcmp(get(gco,'type'),'uicontrol')
	     button_type = get(gco,'style');
	     edit_ui(gco,button_type,param1);
	     set(gco,'selected','off');
	  elseif strcmp(get(gco,'type'),'text')
	     edit_txt(gco,param1)
	     set(gco,'selected','off');
	  else
	    disp('You must first select a uicontrol button or text object');
	  end
	end

  elseif command == 2.4
	if isobj(gco)
	  if strcmp(get(gco,'type'),'uicontrol')
	    get(gco);
	  end
	end

  elseif command == 2.5
	if isobj(gco)
	  if strcmp(get(gco,'type'),'uicontrol')
	     if isobj(get(gco,'userdata'))
		delete(get(gco,'userdata'));
	     end
	     delete(gco);
	     guimaker(1);
	  elseif strcmp(get(gco,'type'),'text')
	     match = 0;
	     ch = get(gcf,'children');
	     handle = gco;
		for loop = 1:length(hlist)
		  if strcmp(get(ch(loop),'type'),'uicontrol')
		     if handle == get(ch(loop),'userdata')
 			  match = 1;	
		     end
		  end
		end
	     if match == 1
		disp('This text is associated with a uicontrol.');
		disp('     It can only be deleted by uicontrol deletion.');
	     else
		delete(gco);
	     end	     
	  end
	end

  elseif command == 3
	etxt = ['sco(' param1 ')'];eval(etxt);
  end
