function  Link = link(obj,flag)

% LINK  "Linking" a group of text objects in a composed text.
%     Allows to create complex texts with different fonts, colors,
%     and other properties, sub- and superscripts and consisting of
%     several lines of various arrangement.
%
%   LINK(OBJ)  "Translates" the texts written in the strings
%      of the text objects OBJ  in a LaTeX-type manner and creates
%      different text objects positioned and styled accordingly.
%      If OBJ is not specified then all text objects of the current
%      figure are considered.
%   LINK(OBJ,FLAG) performs various operations with complex texts
%      linked to the "root" objects OBJ. The type of operation is
%      specified by FLAG.
%      Among the legal flags there are following:
%   UPDATE  - "updates" the linked texts, sets the relative 
%      position of complex text objects according to changed axes
%      limits or the size of a figure; also sets the orientation
%      of the comlex text according to the "rotation" property of
%      the root object.
%   ON, ENABLE  - "activates" the links (connections between the parts
%      of a complex text), but does not "update" them by itself.
%   OFF, DISABLE - "disables" or "detachs" the links
%      without actually closing the text objects or deleting the
%      information about their relations.
%   CLOSE  - closes all the text objects "linked" to the root
%      text objects OBJ.
%   INFO   - displays some additional information about the LINK
%      command.
%   DEMO   - runs the script LINKDEMO - the demonstration of the LINK
%      capabilities.
%   SYNT   - displays the "dictionary" file LINKSYNT, containing the
%      list of text commands and their interpretations.
%   GREEK  - shows part of the file LINKSYNT, containing the Greek
%      alphabet.
%   STYLES - shows the other part of this file, containing the commands
%      for environment and text properties.
%   MACRO  - shows the content of the file LINKMACR with the list of
%      "macros" - user-specified combinations and abbreviations of the
%       standard commands.
%
%     To "prepare" the figure for printing on paper use PREPRINT
%     command.

% Connections:  MAKELINK.M, LINKUPD.M, FINDLINK.M, SETLINK.M,
%               LINKINIT.M, LINKDEMO.M, LINKSYNT, LINKMACR,
%               FILE2STRMAT.M, FINDECHO.M, FINDTXTO.M

%  Kirill K. Pankratov  kirill@plume.mit.edu
%  April 3 1994.

lid = .234886;  % Generic link identification number

flags = str2mat('on','enable','off','disable','update','close');
flags = str2mat(flags,'info','demo','macro','synt');
flags = str2mat(flags,'greek','styles');

 % Handle input arguments ........................................
Link = []; Links = [];
isflag = 0; cmmd = 0;
if nargin==0, obj = inf; cmmd = 0; end
if nargin==1
  if isstr(obj)
    flag = obj; obj = inf; isflag = 1;
  end
end
if nargin==2, isflag = 1; end


 % Find the command type ........................................
if isflag
  lmin = min(size(flag,2),size(flags,2));
  A = flag(ones(size(flags,1),1),1:lmin)==flags(:,1:lmin);
  cmmd = find(all([A A]'));
end
if cmmd == []
  disp([10 '  Error: the "flag"  ' flag '  is not understood.' 10])
  return
end
cmmd = cmmd(1);

 % Find all text objects and links ............................
objls = flipud(findtxto(gcf));  % All text objects
Links = findlink(objls);        % All links
if Links~=[], ln = floor(max(Links(:,2))); else, ln = 0; end
obj = obj(:);
if obj==inf, obj = objls; end  % Operate with all texts
lo = length(obj);

 % Make new links .............................................

if cmmd==0   % Make
  for jo = 1:length(obj)
    string = linkinit(obj(jo));
    if string~=[]  % There is something to link
      ln = ln+1;
      Link = makelink(obj(jo),ln,string);
      setlink(Link)
      linkupd(Link);
    end
  end
  return
end

 % Execute the commands for existing links ........................
if cmmd<7    % On, off, update,close  ````````````````````````````
  if Links~=[]
    A = Links(:,[4:6 8:10]);
    numr = find(all(A'==0));    % ## of root objects in a Link matrix
    objr = Links(numr,1)'; lor = length(objr);
    if objr~=[]
      A = floor(obj(:,ones(1,lor)))==floor(objr(ones(lo,1),:));
      numr = numr(find(any([A; A])));  lnumr = length(numr);
      objr = obj(find(any([A'; A']))); lo = length(objr);
    else, return, end
  else, return
  end
  for jr = 1:lnumr   % For all root objects
    cl = floor(Links(numr(jr),2));
    fnd = find(floor(Links(:,2))==cl); % Current link
    if cmmd<=4               % On, off
      ison = cmmd<3;
      Links(fnd,[3 7]) = ones(size(Links(fnd,[3 7])))*ison;
      setlink(Links(fnd,:))
    elseif cmmd==5           % Update
      linkupd(Links(fnd,:));
    elseif cmmd==6           % Close
      for jf = 1:length(fnd)
        close(Links(fnd(jf),1))
      end
    end
  end

   % The rest is just information and help procedures .......
elseif cmmd==7               % Info
  info = file2strmat('linkinfo');
  disp(info)
elseif cmmd==8               % Demo
  linkdemo
elseif cmmd==9               % Macro
  linkmacro = file2strmat('linkmacr','all');
  disp(linkmacro)
elseif any(cmmd==10:12)      % Syntacs
  linksynt = file2strmat('linksynt');
  fnd = find(linksynt(:,1)==' ');
  A = linksynt(fnd,2:6)==ones(size(fnd))*'\symb';
  ns = find(all(A'));
  if cmmd==10, disp(linksynt) % All synt
  elseif cmmd==11             % Greek
    disp(linksynt(fnd(ns):fnd(ns)+26,:))
  elseif cmmd==12             % Styles
    disp(linksynt(fnd(ns+1):fnd(length(fnd)-1),:))
  end
end   % End commands 1-12 '''''''''''''''''''''''''''''''''''''

Link = Links;   % Output
