function [Pos]=lscan(ha,wdt,hgt,tol,stickytol,hl)
% LSCAN Used by LEGEND to determine a "good" place for
%	the legend to appear. LSCAN returns either the
%	position (in figure normalized units) the legend should
%	appear, or a -1 if no "good" place is found.
%
%       LSCAN searches for the best place on the graph according
%       to the following rules.
%           1. Legend must obscure as few data points as possible.
%              Number of data points the legend may cover before plot
%              is "squeezed" can be set with TOL. The default is a 
%              large number so to enable squeezing, TOL must 
%              be set. A negative TOL will force squeezing.
%           2. Regions with neighboring empty space are better.
%           3. Top and right are better than bottom and left.
%
%	LSCAN(HA,WDT,HGT,TOL,STICKYTOL,HL) returns a 2 element
% 	position vector. WDT and HGT are the Width and Height of
%	the legend object in figure normalized units. TOL
% 	and STICKYTOL are tolerances for covering up data points.
%	HL is the handle of the current legend or -1 if none exist. 
%
%	LSCAN(HA,WDT,HGT,TOL) allows up to TOL data
%	points to be covered when selecting the best
%	legend location.

%	Drea Thomas	5/7/93
%       Copyright (c) 1984-93 by The MathWorks, Inc.
% 
%	Changes made by Louis Verhagen
%	3 December 1993
%
%	1.	The original lscan was very good for linear plots but not for
%		log plots. This has been changed. 
%	2. 	The stiky bit section is also completly changed. If a legend 
%		has been put somewhere, the program checks if the amount of
%		data points under the legend is smaller than TOL. If this
%		is not the case the program will display a message and
%		put the "new" legend at a place where it thinks it is
%		appropriate
%	3.	The step size has decreased from 1/4 to 1/8 of the legend 
%		height. Some people with small legend might not like this,
%		and others with larger legend might even want a smaller 
%		step size.
%	4.	lscan now tries to put the legend in the right top,
%		right bottom, left top, left bottom first (in that order)
%		It will put in the corner which quadrant has the least 
%		amount of points.
%
%	On my parts of the code there is no copyright :-)
%	However I would consider it plagiarism to copy without naming
%	the source (That's me thus)

% Defaults

% See if legend has been recently moved by a moveaxis


sticky=0;
if nargin==6,
	if hl>0,
		udata=get(hl,'userdata');
		if length(udata)==7,
			if udata(7)==1,
				sticky=1;
				set(hl,'units','normalized');
				clp=get(hl,'position');
			end
		end
	end
end

% Calculate tile size

set(ha,'units','normalized')
cap=get(ha,'Position');

% Get data, points and text

Kids=get(ha,'children');
xlim=get(ha,'Xlim');
ylim=get(ha,'Ylim');


Xdata=[];Ydata=[];
for i=1:size(Kids),
	if strcmp(get(Kids(i),'type'),'line'),
		Xdata=[Xdata,get(Kids(i),'Xdata')];
		Ydata=[Ydata,get(Kids(i),'Ydata')];
	elseif strcmp(get(Kids(i),'type'),'text'),
		set(Kids(i),'units','data')
		tmp=get(Kids(i),'Position');
		ext=get(Kids(i),'Extent');
		Xdata=[Xdata,[tmp(1) tmp(1)+ext(3)]];
		Ydata=[Ydata,[tmp(2) tmp(2)+ext(4)]];
	end
end

if strcmp(get(ha,'xscale'),'log');
  Xdata=log(Xdata);
  xlim=log(xlim);
end
if strcmp(get(ha,'yscale'),'log');
  Ydata=log(Ydata);
  ylim=log(ylim);
end

Xdata=(Xdata-xlim(1))./(xlim(2)-xlim(1));
Ydata=(Ydata-ylim(1))./(ylim(2)-ylim(1));

%	See if sticky bit is set 
%	if so see if the amount of points coverd is smaller
%	than TOL

if sticky,
	pop=sum(sum(Xdata >= clp(1)).*(Xdata <= (clp(1)+clp(3)))...
	.*(Ydata >= clp(2)).*(Ydata <= (clp(2)+clp(4))));
	if pop <= tol,
		Pos=[clp(1),clp(2)];
		return;
	end
	disp ('Sorry, Your old Legend position covers more points than you allowed')
end  			 

%	Determine # of data points under each "tile"

i=1;j=1;
for yp=cap(2)+0.03:hgt/8:cap(2)+cap(4)-0.03-hgt,
	i=1;
	for xp=cap(1)+0.03:wdt/8:cap(1)+cap(3)-0.03-wdt,
	   pop(j,i) = ...
           sum(sum((Xdata >= xp).*(Xdata<=xp+wdt).*(Ydata>=yp).*(Ydata<=yp+hgt)));
   	   i=i+1;	
   	end
	j=j+1;
end

% Cover up fewest points.

if min(min(pop)) > tol,
	Pos=-1;
	return
end


popmin = pop == min(min(pop));
if sum(sum(popmin))>1,     % Multiple minima in # of points

	[a,b]=size(pop);
	if min(a,b)>1,

%	Try the 4 corners first  popquat & popqua are the amount of 
%	points in each quadrant. This is done so that the quadrant
%	With the least points will have the legend in the corner.

% 	right top
	corn=[0,0];
	popquat=sum(sum(pop(0.5*a:a,0.5*b:b)));
	if pop(a,b) <= popmin
		corn = [b,a];
	end;
% 	right bottom
	popqu=sum(sum(pop(1:0.5*a,0.5*b:b)));
	if pop(1,b) <= popmin & popqu < popquat
		corn = [b,1];
		popquat=popqu;
	end;
% 	left top
	popqu=sum(sum(pop(0.5*a:a,1:0.5*b)));
	if pop(a,1) <= popmin & popqu < popquat
		corn = [1,a];
		popquat=popqu;
	end;
% 	left bottom
	popqu=sum(sum(pop(1:0.5*a,1:0.5*b)));
	if pop(1,1) <= popmin & popqu < popquat
		corn = [1,1];
	end;
	if corn
	  corn=corn-1;
	  Pos=[((corn(1)*wdt/8)+cap(1)+0.03),((corn(2)*hgt/8)+cap(2)+0.03)];
	  return
	end

	% Look at adjacent tiles and see if they are empty

	pop=[pop(2,:)',pop(1:(a-1),:)']'+[pop(2:(a),:)',pop((a-1),:)']'+...
       	 [pop(:,2),pop(:,1:(b-1))]+[pop(:,2:b),pop(:,(b-1))];
	pop=[pop(2,:)',pop(1:(a-1),:)']'+[pop(2:(a),:)',pop((a-1),:)']'+...
       	 [pop(:,2),pop(:,1:(b-1))]+[pop(:,2:b),pop(:,(b-1))];
	popx=popmin.*(pop==min(pop(popmin)));

		if sum(sum(popx))>1,
			flag=1;i=a;j=b;
				while flag,
					if flag == 2,
						if popx(i,j) == 1,
							popx=popx*0;popx(i,j)=1;
							flag = 0;
						else,
							popx=popx*0;popx(i,j+1)=1;
							flag = 0;
						end
					
					else
						if popx(i,j)==1,
							flag = 2;
							popx=popx*0;popx(i,j)=1;
						else,
							i=i-1;
							if i==0,
								i=a;j=j-1;
							end
						end
					end
				end
		end
	else,
		popx=popmin*0;popx(1,1)=1;
	end
else,	% Only 1 minima in # covered points
	popx=popmin;
end

   i=find(max(popx));i=i(1);
   j=find(max(popx'));j=j(1);

Pos=[(((i-1)*wdt/8)+cap(1)+0.03),(((j-1)*hgt/8)+cap(2)+0.03)];





