function [command,handles] = saveobj(object, fig)
%SAVEOBJ stores the information necessary to recreate an object.
% The object can be a line, patch, surface, text, or image.
% Neither axes nor figures can be stored using SAVEOBJ.
% [COMMAND,HANDLES] = SAVEOBJ(OBJECT, FIG)
% returns a COMMAND string and HANDLES to invisible text
% objects which keep information that cannot be stored in
% strings. The object may then be recreated using the
% LOADOBJ command.  The input parameter FIG specifies a
% figure in which the storage uicontrols are to be created.

% Copyright (c) 1995 by Keith Rogers

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% If using MATLAB 4.2a, make sure you have a     %
% version of isobj.m that does not call MATLAB's %
% findobj function. See the MatDraw Readme file  %
% for details.                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if(~isobj(object,gcf))
	error('Not an object!');
end
if(nargin < 2)
	fig = gcf;
end
UserData = get(object,'UserData');
Type = get(object,'Type');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The following are common to all objects and %
% can be fully specified by text strings.     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

CommonBlock = ['''ButtonDownFcn'',''' get(object,'ButtonDownFcn') ''',' ...
			   '''Clipping'',''' get(object,'Clipping') ''',' ...
			   '''Interruptible'',''' get(object,'Interruptible') ''',' ...
			   '''Tag'',''' get(object,'Tag') ''',' ...
			   '''Selected'',''' get(object,'Selected') ''',' ...
			   '''Visible'',''' get(object,'Visible') ''','];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Stuff unique to LINE objects %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if(strcmp(Type,'line'))
	Color = get(object,'Color');
	command = ['line(' ...
			   '''Color'',[' num2str(Color(1)) ' ' num2str(Color(2)) ' ' num2str(Color(3)) '],' ...
			   '''EraseMode'',''' get(object,'EraseMode') ''',' ...
			   '''LineStyle'',''' get(object,'LineStyle') ''',' ...			   
			   '''Linewidth'',' num2str(get(object,'LineWidth')) ',' ...			   			   
			   '''MarkerSize'',' num2str(get(object,'MarkerSize')) ',' ...			   			   
			   '''XData'',XData' ',' ...			   			   
			   '''YData'',YData' ',' ...			   			   
			   '''ZData'',ZData' ',' ...
			   CommonBlock ...
			   '''UserData'',UserData);'];
	handles = [store(fig,4) store(fig,4)];
	set(handles(1,1),'UserData','XData');
	set(handles(1,2),'UserData',get(object,'XData'));
	set(handles(2,1),'UserData','YData');
	set(handles(2,2),'UserData',get(object,'YData'));
	set(handles(3,1),'UserData','ZData');
	set(handles(3,2),'UserData',get(object,'ZData'));
	set(handles(4,1),'UserData','UserData');
	set(handles(4,2),'UserData',get(object,'UserData'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Stuff unique to SURFACE objects %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

elseif(strcmp(Type,'surface'))
	EdgeColor = get(object,'EdgeColor');
	if(isstr(EdgeColor))
		EdgeColor = ['''' EdgeColor ''''];
	else
		EdgeColor = [' num2str(EdgeColor(1)) ' ' num2str(EdgeColor(2)) ' ' num2str(EdgeColor(3)) '];
	end
	if(isstr(FaceColor))
		FaceColor = ['''' FaceColor ''''];
	else
		FaceColor = [' num2str(FaceColor(1)) ' ' num2str(FaceColor(2)) ' ' num2str(FaceColor(3)) '];
	end
	
	command = ['surface(' ...
			   '''EdgeColor'',' EdgeColor ',' ...
			   '''FaceColor'',' FaceColor ',' ...
			   '''EraseMode'',''' get(object,'EraseMode') ''',' ...
			   '''LineStyle'',''' get(object,'LineStyle') ''',' ...			   
			   '''MeshStyle'',''' get(object,'MeshStyle') ''',' ...			   
			   '''Linewidth'',' num2str(get(object,'LineWidth')) ',' ...			   			   
			   '''MarkerSize'',' num2str(get(object,'MarkerSize')) ',' ...			   			   
			   '''XData'',XData' ',' ...			   			   
			   '''YData'',YData' ',' ...			   			   
			   '''ZData'',ZData' ',' ...
			   '''CData'',CData' ',' ...
			   CommonBlock ...
			   '''UserData'',UserData);'];
	handles = [store(fig,5) store(fig,5)];
	set(handles(1,1),'UserData','XData');
	set(handles(1,2),'UserData',get(object,'XData'));
	set(handles(2,1),'UserData','YData');
	set(handles(2,2),'UserData',get(object,'YData'));
	set(handles(3,1),'UserData','ZData');
	set(handles(3,2),'UserData',get(object,'ZData'));
	set(handles(4,1),'UserData','CData');
	set(handles(4,2),'UserData',get(object,'CData'));
	set(handles(5,1),'UserData','UserData');
	set(handles(5,2),'UserData',get(object,'UserData'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Stuff unique to TEXT objects %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

elseif(strcmp(Type,'text'))
	Color = get(object,'Color');
	command = ['text(' ...
			   '''Color'',[' num2str(Color(1)) ' ' num2str(Color(2)) ' ' num2str(Color(3)) '],' ...
			   '''EraseMode'',''' get(object,'EraseMode') ''',' ...
			   '''FontAngle'',''' get(object,'FontAngle') ''',' ...
			   '''FontName'',''' get(object,'FontName') ''',' ...
			   '''FontSize'',' num2str(get(object,'FontSize')) ',' ...			   			   
			   '''FontStrikeThrough'',''' get(object,'FontUnderline') ''',' ...
			   '''FontUnderline'',''' get(object,'FontUnderline') ''',' ...
			   '''FontWeight'',''' get(object,'FontWeight') ''',' ...
			   '''HorizontalAlignment'',''' get(object,'HorizontalAlignment') ''',' ...
			   '''Position'',Position' ',' ...			   			   
			   '''Rotation'',' num2str(get(object,'Rotation')) ',' ...			   			   
			   '''String'',''' get(object,'String') ''',' ...
			   '''Units'',''' get(object,'Units') ''',' ...
			   '''VerticalAlignment'',''' get(object,'VerticalAlignment') ''',' ...
			   CommonBlock ...
			   '''UserData'',UserData);'];
	handles = [store(fig,2) store(fig,2)];
	set(handles(1,1),'UserData','Position');
	set(handles(1,2),'UserData',get(object,'Position'));
	set(handles(2,1),'UserData','UserData');
	set(handles(2,2),'UserData',get(object,'UserData'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Stuff unique to PATCH objects %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

elseif(strcmp(Type,'patch'))
	EdgeColor = get(object,'EdgeColor');
	if(isstr(EdgeColor))
		EdgeColor = ['''' EdgeColor ''''];
	else
		EdgeColor = ['[' num2str(EdgeColor(1)) ' ' num2str(EdgeColor(2)) ' ' num2str(EdgeColor(3)) ']'];
	end
	FaceColor = get(object,'FaceColor');
	if(isstr(FaceColor))
		FaceColor = ['''' FaceColor ''''];
	else
		FaceColor = ['[' num2str(FaceColor(1)) ' ' num2str(FaceColor(2)) ' ' num2str(FaceColor(3)) ']'];
	end
	
	command = ['patch(' ...
			   '''EdgeColor'',' EdgeColor ',' ...
			   '''FaceColor'',' FaceColor ',' ...
			   '''EraseMode'',''' get(object,'EraseMode') ''',' ...
			   '''Linewidth'',' num2str(get(object,'LineWidth')) ',' ...			   			   
			   '''XData'',XData' ',' ...			   			   
			   '''YData'',YData' ',' ...			   			   
			   '''ZData'',ZData' ',' ...
			   '''CData'',CData' ',' ...
			   CommonBlock ...
			   '''UserData'',UserData);'];
	handles = [store(fig,5) store(fig,5)];
	set(handles(1,1),'UserData','XData');
	set(handles(1,2),'UserData',get(object,'XData'));
	set(handles(2,1),'UserData','YData');
	set(handles(2,2),'UserData',get(object,'YData'));
	set(handles(3,1),'UserData','ZData');
	set(handles(3,2),'UserData',get(object,'ZData'));
	set(handles(4,1),'UserData','CData');
	set(handles(4,2),'UserData',get(object,'CData'));
	set(handles(5,1),'UserData','UserData');
	set(handles(5,2),'UserData',get(object,'UserData'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Stuff unique to IMAGE objects %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

elseif(strcmp(Type,'image'))
	command = ['image(' ...
			   '''XData'',XData' ',' ...			   			   
			   '''YData'',YData' ',' ...			   			   
			   '''CData'',CData' ',' ...
			   CommonBlock ...
			   '''UserData'',UserData);'];
	handles = [store(fig,4) store(fig,4)];
	set(handles(1,1),'UserData','XData');
	set(handles(1,2),'UserData',get(object,'XData'));
	set(handles(2,1),'UserData','YData');
	set(handles(2,2),'UserData',get(object,'YData'));
	set(handles(3,1),'UserData','CData');
	set(handles(3,2),'UserData',get(object,'CData'));
	set(handles(4,1),'UserData','UserData');
	set(handles(4,2),'UserData',get(object,'UserData'));
end			   			   
			   
			   
			   
			   
