% PL	Interactive 2-D plotting for MATLAB
% usage:
%    pl('R')
%    pl( plot_parameters )
%    pl( plot_parameters ... ,('-A'|'A')) 
%    pl( plot_parameters ... ,filename)
%    pl( pl_param , pl_value )
%    pl( filename )
%
% 1.0 OVERVIEW
% PL uses the same parameters as PLOT but allows interactive manipulation
% of axes, titles, text and keys etc. When the plot is on the screen
% "ginput" is automatically called and pressing the SPACEBAR key or the 
% left mouse button will display the current cursor position.
%
% Access to PL's many interactive commands can be accessed through
% either direct keystrokes or by use of the menu. The menu is displayed
% by pressing the ESC key or clicking the "M" on the top left corner of 
% display (DOS ver) or accessing the extra menu options displayed at the 
% top of the figure window (WINDOWS ver).
% 
% Important extensions that PL gives to MATLAB include,
% i)   Ablility to display a key
% ii)  Add text with arrows
% iii) Allow multiple linestyle/marker combinations e.g. 'y-r*'
% iv)  Easily zoom in and out of the data and display the value
%      of individual data points
% v)   Save plot complete with data, keys and text to a file
%      (with extension .plt ) for later display and editing.
%
% To exit PL use the F4 or Q key or alterativeley the "Exit" pushbutton 
% ( WINDOWS ver )
%
% 2.0 USING PL WITH MULTIPLE PLOTS
% To handle multiple plots the following extra command parameters
% need be used with PL.
%	'R'  Reset PL's internal plot parameters (equivalent to SUBPLOT(111)).
% 	'-A' Adds the current plot to those already in its internal buffers.
%	     A new SUBPLOT number will be allocated automatically and the new
%	     plot stored but not displayed.
%	'A'  As above except it will display all plots in its internal 
%	     buffers.
%
% e.g to plot x1 against y1 and x2 against y2 using separate plots
%     simply enter.
%
%	pl('R') 	% reset PL
%	pl(x1,y1,'-A')	% add x1 and y1 to PL's internal buffers but returns
%			% without plotting
%	pl(x2,y2,'A')	% add x2 and y2 to PL's internal buffers and displays
%			  all plots
%
% When multiple plots are displayed on the screen the "subplot" menu options
% become operational and allow the user to zoom in on one plot and remove 
% particular plots. An 'S' is displayed against the plot that is currently 
% selected meaning that all commands will work with this plot. To select other
% plots simply click the mouse button over the plot, use the TAB button
% or the pushbuttons (WINDOWS ver) at the bottom left corner. The x axes can 
% also be linked (command from the "subplot") menu and this will mean that 
% any changes to the x axis limits (by zooming or scrolling) will be reflected 
% in all the other plots displayed.
%
% NOTE that when using multiple plots all the plotted data must exist as
%      as matrices in the calling workspace,
%	e.g x=1:100; pl('R'); pl(x,x,'-A') pl(x,x,'A')	   % OK
%
%           pl('R'); pl(1:100,1:100); pl(1:100,1:100,'-A') % WRONG
% 
% The message "Can't find previous plot data" will be displayed if temporary
% matrices are used with multiple plots (its no problem with one plot).
%
%
% 3.0 MARKING AND DISPLAYING AN INDIVIUAL DATAPOINT
% Pressing the right-hand mouse button (or INSERT key) will mark with a 
% cross the closest data point to the current mouse position and in addition
% display the x value,y value and index number for this point at the top 
% of the display. If the '>' or '<' key  ( or the '<' and '>' pushbuttons
% in the WINDOWS version) is pressed the index of this point will be 
% incremented or decremented by one. If more than one graph is displayed 
% then a vertical line will be drawn at the x position of the marked point
% and pressing the '.' key will mark and display the same 
% index using the next graph displayed. If multiple plots are displayed 
% reselecing a different plot will shift the marked point to the new plot 
% using the same index.
%
% 4.0 CONTROLLING PL FROM THE COMMAND LINE
% Keys and labels etc can be set up using the command line calls prior
% to using PL. To do this use is made of the 'R' and 'A' command options 
% (used with multiple plots),
% e.g. to set up a plot with key and label's we would,
%	pl('R')
%	pl('xlabel','x axis');
%	pl('ylabel','y axis');
%	pl('key','Graph 1');
%	pl('keyxy',[0.05 0.95]);
%	pl(x,y,'A')
%
% Here we are using PL with text strings commands. These text string commands
% allow various aspects of the display to be set up prior to display,
% A list of the current commands and their required parameters are given 
% below,
%
%	'title'		Sets up the title for a plot, requires a text
%			text string as the next parameter.
%	'xlabel'	As above but sets the x label for a plot.
%	'ylabel'	As above but sets the y label for a plot.
%	'grid'		Sets the grid on or off. Second parameter must be
%			'on' or 'off'.
%	'key'		Sets up the key. Next parameter must be the text
%			for the key.
%	'keyxy'		Set the position of a key. Next parameter must be 
%			a numeric matrix holding the x and y position of the
%			key (using normalised coordinates)
%	'linkx'		Links the x axis when using multiple plots. Next 
%			parameter must be 'on' or 'off'.
%	'loglog'	Uses log x and y axes. No other parameter required.
%	'semilogx'	Uses log x and linear y axis.
%	'semilogy'	Uses linear x and log y axis.
%	'ginput'	Prompts the user to enter points. Next parameter must
%			be a text matrix with each row of the matrix being a
%			a separate prompt. Number of of points to enter will 
%			be the number of columns of this matrix. It will return 
%			x,y,index and graph numbers as left-hand side arguments.
%	'menu'		Controls display of interactive menu (DOS only ). Next 
%			parameter must be 'on' or 'off'.
%	'fontname'	Sets name of font for axes, text etc. Next parameter 
%			must be the name of the font (WINDOWS only).
%	'axesfontsize'	Sets the size of the axes font. Next parameter must be 
%			a numeric matrix holding the size in points 
%			(WINDOWS only).
%	'textfontsixe'	As above but sets size of font used for user added 
%			text (WINDOWS only).
%	'keyfontsize'	As above but sets size of font used for key text
%			(WINDOWS only).
%
% 6.0 SAVING AND RESTORING PLOTS
% PL has the ability to save the plot including all data, text and 
% key information etc. to a MATLAB data file so that a plot can be 
% later edited. To save a plot use the "Save" option from the "Axes" menu. 
% The filename will have an extension ".plt" and to re-display a saved 
% plot simply use PL with the filename as a single parameter. e.g,
%
%	pl('test.plt')
%
% will display the contents of the file "test.plt" 
%
% 7.0 INSTALLATION AND COMPATABILITY NOTES
% PL can only be used with v3.x AT-MATLAB (DOS) and v4.x MATLAB for WINDOWS.
% The DOS version needs ANSI.SYS installed (see DOS manual for details).
% DOS files are: plprint.m         WINDOWS files are plpow10.m
%                plpow10.m                           plginput.m
%                pl.m                                plsel.m
%                pl.mex                              pl.m
%                                                    pl.dll
%
% For more information and details contact 
% Rob Oldaker at   20 Fairfield Park Road,
%                     Bath, Avon, UK.
%                  Tel (01225) 312933
%		   Email "eerrso@hull.ac.uk"
%
