function mergtex(pvec,rot,clr,ang,wgt,nam,siz)
%MERGTEX -- a function to allow the use of greek characters,
%       subscripts and superscripts in a text label placed with
%       mouse click horizontally aligned to the right of the
%       chosen point and vertically aligned centered.
%
%       syntax is a little tricky.
%       mergtex([any vector of calls to the five more primitive 
%       functions]) where the primitive functions are:
%               ntex(string)    --- for normal text
%               gtex(string)    --- for greek text (string contains 
%                                   ascii equivalents of greek chars
%                                   see gtex help)
%               stex(string)    --- for subscript text
%               ptex(string)    --- for superscript text
%               sptex(s-string,p-string) -- for both sub and super text
%       NEW     gstex(string)   --- for greek subscripts text
%	  NEW	    gptex(string)   --- for greek superscript text
%	  NEW	    gsptex(s-string,p-string) 
%					  --- for both sub and super greek text
%
%       put together calls to primitives with spaces in appropriate
%       places to produce a "complex" text string as if it were one 
%       string.
%       EXAMPLE:  mergtex([ntex('H'),stex('2'),ntex('O')]) % for water sym
%         mergtex([gtex('a'),ntex(' is an alpha and '),gtex('b'),...
%                                       ntex(' is a beta.')])
%
%       Optional arguments, rot,clr,ang,wgt,nam,siz can be specified 
%       using strings with keywords as follows:
%               'rot=90' or 'rot=vl'   for rotation
%               'clr=y' or 'clr=[1 1 0]'  for color
%               'ang=oblique' or 'ang=italic'  for fontangle
%               'wgt=bold'  or 'wgt=light'   for fontweight
%               'nam=times-roman' or 'nam=bembo'  for fontname
%               'siz=20'                for fontsize
%
%       The optional args can be ordered in any way after arg #1
%       and have different possible syntaxes. 
%
%       The optional arg rot determines the rotation of all
%       text in the clockwise direction in multiples of 90 degrees.
%       +/-90, +/-180, or +/-270
%       also rotation can be 'vl' -- vertical with the text reading up
%                            'vr' -- vertical text reading down
%                            'hd' -- upsidedown
%       Optional arg clr determines the color of all text and
%       uses either an rgb vector or the one letter color codes
%       used by matlab (see plot).
%
%	NEW:  
%       Greek characters can now also be put in subscript and/or
%	  superscript 
%
%       AUTHOR: Dan(k) Braithwaite Dept. of Hydrology UofA
%               last updated 7/20/93
%
%       OTHER nonmatlab functions called by this function:
%               mget,mstrcmp,findthx,findnxnum,d_gin
%
%	  Modifed by Louis Verhagen, Dep. Of Civil Engineering Canberra
%	  Australia		24/3/94
 
angv = [];
wgtv = [];
namv = [];
clrv = [];
sizv = [];
rotv = [];
arglis = ['rot';'clr';'ang';'wgt';'nam';'siz'];
if nargin == 0
        error('not enough input arguments');
elseif nargin > 7
        error('too enough input arguments');
else
        for i = 2:nargin  % check all args 2 up to 7
                argnam = arglis(i-1,:);
                key = eval([argnam,'(1:3)']);
                valu = eval([argnam,'(5:',int2str(length(eval(argnam))),')']);
                if strcmp(key,'rot')
                        rotv = valu;
                        % if the valu is given by keyword
                        if (rotv(1) == 'v') | (rotv(1) == 'h')
                           if(strcmp(rotv,'vl')), rotv = 90;
                           elseif(strcmp(rotv,'vr')), rotv = 270;
                           elseif(strcmp(rotv,'hd')), rotv = 180;
                           else
                                rotv = [];
                                disp('rot value is bad');
                           end
                        % if valu given as a number
                        elseif length(str2num(rotv))
                                % can't handle non-multiple of 90
                                rotv = rem(round(str2num(rotv)/90)*90,360);
                        else
                                rotv = [];
                                disp('rot value is bad')
                        end
                elseif strcmp(key,'clr')
                        clrv = valu;
                elseif strcmp(key,'ang')   %no error checking on these
                        angv = valu;
                elseif strcmp(key,'wgt')
                        wgtv = valu;
                elseif strcmp(key,'nam')
                        namv = valu;
                elseif strcmp(key,'siz')
                        sizv = eval(valu);
                else
                        eval(['disp(''unknown key ',key,''')']);
                end
        end
end
 
%set those keys that were specified
if length(rotv)
        set(pvec,'rotation',rotv);
end
if length(clrv)
        set(pvec,'color',clrv);
end
if length(angv)
        set(pvec,'fontangle',angv);
end
if length(wgtv)
        set(pvec,'fontweight',wgtv);
end
% the keys nam and siz cannot be set on all text objects
if length(namv) | length(sizv)
        udat = mget(pvec,'userdata');
        nor = pvec(find(mstrcmp(udat,'nor')));
        grk = pvec(find(mstrcmp(udat,'grk')));
        sub = pvec(find(mstrcmp(udat,'sub')));
        sup = pvec(find(mstrcmp(udat,'sup')));
        upp = pvec(find(mstrcmp(udat,'upp')));
        low = pvec(find(mstrcmp(udat,'low')));
        gpp = pvec(find(mstrcmp(udat,'gpp')));
        glw = pvec(find(mstrcmp(udat,'glw')));
        gup = pvec(find(mstrcmp(udat,'gup')));
        gub = pvec(find(mstrcmp(udat,'gub')));
end
if length(namv)
        set([nor';sub';sup';upp';low';gpp';glw';gup';gub'],'fontname',namv);
	  %all but grk
end
if length(sizv)
        set([nor';grk'],'fontsize',sizv); %full sized text
        set([sub';sup';upp';low';gpp';glw';gup';gub'],'fontsize',floor(sizv*0.7));  
	  %reduced sized
end
set(pvec,'horiz','left');
 
[x,y] = d_gin   % wait for user to select a placement in figure %
ptr = get(gcf,'pointer');
set(gcf,'pointer','watch')      % let the user know were slow
 
unis = get(gcf,'units');
set(gcf,'units','pixels');
frect = get(gcf,'posi');        % get figure position in pixels
set(gcf,'units',unis);
 
nv = length(pvec);
nxnum = findnxnum(nv);          % need to flag all text objects with the
                                % same flag
 
% get the extent of all objects based on my axes as big as the paperposition
set(findthx,'units',get(gcf,'paperunits'),'posit',get(gcf,'paperposit'));
paperextn = mget(pvec,'extent');
set(findthx,'units','norm','posit',[0 0 1 1]);  %reset
 
% set the verticalalignment for the first object based on sub,super,etc.
imp = get(pvec(1),'userdata');
if (strcmp(imp,'sub') |strcmp(imp,'low') |strcmp(imp,'gub') |strcmp(imp,'glw'))
   vt = 'top';
elseif (strcmp(imp,'sup') |strcmp(imp,'upp') |strcmp(imp,'gup') |strcmp(imp,'gpp'))
   vt = 'bottom';
else
   vt = 'middle';
end
 
%lightup first object in correct place and flag it 
set(pvec(1),'position',[x,y],'vertical',vt,'vis','on','userdata',nxnum);
 
for i=2:nv      %for each of the other objects
   imp = get(pvec(i),'userdata');
   %if created by (g)sptex do not adjust x or y
   if(~strcmp(imp,'upp') & ~strcmp(imp,'gpp'))       

     if(rotv == 90)     %x or y is adjusted depending on rotation
 %place the next object immediately after the prev and push it 6 pixels
 %closer as a fudge factor.  The expression (6/frect(4)) gives the 
 %normalized value of 6 pixels in the y direction
        y = y+paperextn(i-1,4)-(6/frect(4));
     elseif(rotv == 180)
        x = x-paperextn(i-1,3)+(6/frect(3));
     elseif(rotv == 270);
        y = y-paperextn(i-1,4)+(6/frect(4));
     else
        x = x+paperextn(i-1,3)-(6/frect(3));
     end
   end
 
   if (strcmp(imp,'sub') |strcmp(imp,'low') |strcmp(imp,'gub') |strcmp(imp,'glw'))
      vt = 'top';
   elseif (strcmp(imp,'sup') |strcmp(imp,'upp') |strcmp(imp,'gup') |strcmp(imp,'gpp'))
      vt = 'bottom';
   else
      vt = 'middle';
   end
 
   set(pvec(i),'position',[x,y],'vertical',vt,'vis','on','userdata',nxnum);
end
set(gcf,'pointer',ptr)
