function h = surftri(x,y,z,c)
%
%
%
%TRISURF	3-D shaded surface graph of flipud(triu()) with triangular patches.
%	TRISURF(X,Y,Z,C) plots the colored parametric surface defined by
%	four matrix arguments.  The view point is specified by VIEW.
%	The axis labels are determined by the range of X, Y and Z,
%	or by the current setting of AXIS.  The color scaling is determined
%	by the range of C, or by the current setting of CAXIS.  The scaled
%	color values are used as indices into the current COLORMAP.
%	The shading model is set by SHADING.
%
%	SURFTRI(X,Y,Z) uses C = Z, so color is proportional to surface height.
%
%	SURFTRI(x,y,Z) and SURFTRI(x,y,Z,C), with two vector arguments replacing
%	the first two matrix arguments, must have length(x) = n and
%	length(y) = m where [m,n] = size(Z) and n = m.  In this case, the vertices
%	of the surface patches are the triples (x(j), y(i), Z(i,j)).
%	Note that x corresponds to the columns of Z and y corresponds to
%	the rows.
%
%	SURFTRI(Z) and SURFTRI(Z,C) use x = 1:n and y = 1:m.  In this case,
%	the height, Z, is a single-valued function, defined over a
%	geometrically rectangular grid. (m = n)
%
%	SURFTRI returns a handle to a SURFACE object.
%
% 	AXIS, CAXIS, COLORMAP, HOLD, SHADING and VIEW set figure, axes, and 
% 	surface properties which affect the display of the surface.
%
%
%-------------------------------
%	Additional details:
%
%	If the NextPlot axis property is REPLACE (HOLD is off), SURFTRI resets 
%	all axis properties, except Position, to their default values
%	and deletes all axis children (line, patch, surf, image, and 
%	text objects).

%	Copyright (c) 1993 by ZIB,
%	Copyright (c) in parts 1984-1992 by The MathWorks, Inc.


%	G.W. Zumbusch 16-2-93


error(nargchk(1,4,nargin));

if nargin==1,  % Generate x,y matrices for surface z.
  if min(size(x)) == 1 | isstr(x)
      error('Invalid input argument.')
  end
  z = x;
  [m,n] = size(z);
  [x,y] = meshgrid(0:n-1,0:m-1);
  c = z;

elseif nargin==2,
  if isstr(x) | isstr(y)
	error('Invalid input argument (text).')
  end
  if min(size(x)) == 1 | min(size(y)) == 1
      error('Invalid input argument size.')
  end
  z = x; c = y;
  [m,n] = size(z);
  [x,y] = meshgrid(0:n-1,0:m-1);
  if any(size(c) ~= size(z))
      error('Invalid input argument size.')
  end

elseif nargin>=3,
  if isstr(x) | isstr(y) | isstr(z)
      error('Invalid input argument (text).')
  end
  [m,n] = size(z);
  [mx,nx] = size(x);
  [my,ny] = size(y);
  if m == 1 | n == 1
      error('Invalid input argument size.')
  end
  if nargin == 3
	c = z;
  end
  if any(size(c) ~= size(z))
      error('Invalid input argument size.')
  end
  if any(size(z) ~= size(x)) | any(size(z) ~= size(y))
      error('Invalid input argument size.')
  end
end
if isstr(c)
      error('Invalid input argument (text).')
end


if m ~= n
  error('only square matrices');
end

	m = 2*m-1;
	xneu = NaN * ones(m,m);
	yneu = xneu;
	zneu = yneu;
	cneu = zneu;

	for i2 = 1:m
	   i = i2*.5;
	   for j2 = 1:min(m,m+2-i2)
		j = (rem(i2+1,2) + j2) * .5;
		xneu(i2,j2)	= x(i,j);
		yneu(i2,j2)	= y(i,j);
		zneu(i2,j2)	= z(i,j);
		cneu(i2,j2)	= c(i,j);
	   end
	end


cax = newplot;

hh = surface(xneu,yneu,zneu,cneu);

next = lower(get(cax,'NextPlot'));
if ~ishold
	view(3);
end
if nargout == 1
	h = hh;
end
