function fout = waitbar(xfrac,name,xmax)
%WAITBAR Display wait bar.
%	H = WAITBAR(XFRAC,'title') creates and displays a wait bar of 
%	fractional length XFRAC out of 100%.  
%
%       H = WAITBAR(XFRAC,'title',XMAX) creates and displays a wait bar
%       of length XFRAC*XMAX out of XMAX.
%
%       The handle to the waitbar figure is returned in H.
%       XFRAC should be between 0 and 1.  Each subsequent
%	call to waitbar, WAITBAR(XFRAC), extends the length 
%	of the bar to the new position XFRAC.
%
%	WAITBAR is typically used inside a FOR loop that performs a 
%	lengthy	computation.  A sample usage is shown below:
%
%	    h = waitbar(0,'Please wait...');
%	    for i=1:100,
%	        % computation here %
%	        waitbar(i/100)
%	    end
%	    close(h)

%	Clay M. Thompson 11-9-92
%	Copyright (c) 1992-93 by The MathWorks, Inc.
%	$Revision: 1.9 $  $Date: 1993/09/24 18:57:13 $

%       XMAX revision by Brian K. Servis (servis@ecn.purdue.edu)

if nargin == 2
  xmax = 100;
end

if nargin >= 2,
  pos = get(0,'ScreenSize');
  f = figure('position',[pos(3)/2-180 pos(4)/2-37 360 75], ...
             'NextPlot','new','resize','off');
  colormap([])

  h = axes('Xlim',[0 xmax],'Ylim',[0 1]);
  title(name)
  set(h,'box','on')
  set(h,'position',[.05 .30 .9 .30])
  set(h,'YTickLabelMode','manual')
  set(h,'YTickLabels',[])
  set(h,'XTick',[0:xmax/4:xmax])
  if nargin == 2
    set(h,'XTickLabels',[' 0% ' ; ' 20%' ; ' 50%' ; ' 75%' ; '100%'])
  end
  
  xpatch = [0 xfrac xfrac 0];
  ypatch = [0 0 1 1];
  xline = [xmax 0 0 xmax xmax];
  yline = [0 0 1 1 0];
  p = patch(xpatch,ypatch,'r','Edgecolor','r','EraseMode','none');
  l = line(xline,yline,'eraseMode','none');
  set(l,'Color',get(gca,'Xcolor'))

else
  f = gcf;
  h = gca;
  ch = get(gca,'children');
  p = []; l = [];
  for i=1:length(ch),
    t = get(ch(i),'type');
    if strcmp(lower(t),'patch'), p = ch(i); end
    if strcmp(lower(t),'line'), l = ch(i); end
  end
  if isempty(p) | isempty(l), error('Couldn''t find waitbar handles.'); end
  xpatch = get(p,'xdata');
  xlim=get(h,'XLim');
  xmax=xlim(2);
  x = max(0,min(xmax*xfrac,xmax));
  xpatch = [xpatch(2) x x xpatch(2)];
  set(p,'Xdata',xpatch)
  xline = get(l,'xdata');
  set(l,'xdata',xline)
end

drawnow

if nargout==1,
  fout = f;
end


