function [ier,nn]=tsplot(Y,t,titl,ylbl,xlbl,nn)
% function tsplot(Y,t,titl,ylbl,xlbl,nn)
% PURPOSE:
% Plotting time series contained columnwise in matrix Y with
% (optional) independent variable stored in vector t
% (default: index).
% Other optional variables: title of the plot and
% axes labels  and vector nn containing starting and ending
% indices of Y to be plotted. nn is also returned to enable
% interactive choice of subset of time series for further
% processing.
% USAGE:
% This is an interactive, mouse driven routine, general
% rules are as follows:
% - click any of the menu items using left mouse button;
% - "value" clicking puts you in "value finding" mode
%   where each "left click" displays a pair of (t,y) values
%   on Matlab's standard output; to leave this mode click right
%   button;
% - "zoom" expects two further clicks on left or middle button,
%    which define range of independent variable to be displayed
%   (and returned to calling routine if chosen before quitting);
%   a double click causes the whole range of the series to be
%   shown;
% - "help", "grid" and "quit" should be clicked with left button;
% - "hide" is designed to hide the menu line for screen saving or
%    printing routines; click on the right button to return to menu;
% In case of detected wrong parameters, no plot will be drawn and
% ier~=0 will be returned.


% HARDWARE and Matlab version:
%   This version of the routine was written for Matlab ver. 3.27a
%  running on a monochrome Sun 3/60 workstation. The colour was
%  not used and neither were nice facilities like 'isstr' function
%  which save a lot of hassle when variable headers are used.
%
%     Adaptation to other hardware with interactive graphics
%  commands in Matlab (ginput) should not be difficult. The only
%  difference will probably be in the number of mouse buttons
%  used.
% USED m-FILES:  plt_menu, show_plt_menu
% KNOWN BUGS and other remarks:
%  * As it is difficult to clear "clicks" and keyboard buffer from
% within Matlab, some "old" clicks will sometimes be interpreted
% by the program which will result in erratic but not disastrous
% effects. To avoid this, click buttons briefly and keep the
% mouse stationary when clicking. I hope that in future versions
% of Matlab a command clearing mouse and keyboard buffers will be
% introduced.
%  * Placing the menu and scaling is based on default size of the
% Matlab's graphic window. When the window is much smaller, funny
% effects may result, when the window is larger, the menu
% is left aligned.
%  * No subplotting is provided.
%  * Extending the routine to zoom into plot both horizontally
%    and vertically is simple (click on the diagonal of the
%    desired sub-rectangle, leaving a marker, instead of clicking
%    on the desired x-range. I just did not need that facility.
% Please, report found bugs, modifications etc. to:
%       W. Tych,
%       Dept. of Environmental Sciences
%       University of Lancaster, U.K.
%       e-mail:            tych@uk.ac.lancs.cent1
%             from BITNET: tych@cent1.lancs.ac.uk
%
% Wlodek Tych, 14.Sept.1990.
%

[n,m]=size(Y);
if n<m                          % column data required
   Y=Y';
end
% defaults
if nargin<2, t=1:n; end
if nargin<3, titl='Series'; end
if nargin<4, ylbl='y'; end
if nargin<5, xlbl='index'; end
if nargin<6, nn=[1 n]; end
% initial values
items=['zoom  ';'grid  ';'value ';'quit  ';'help  ';'hide  '];
igrd=0;
n1=nn(1);n2=nn(2);
xlbl=xlbl(1:min(length(xlbl),12));
ylbl=ylbl(1:min(length(ylbl),12));
hold off;subplot;clg;  % reset graph
%
but=[1 1];   % buttons reset
while but(2)~=3
        x=[1 n];y=[1 n];
        clg;
        plot(t(n1:n2),Y(n1:n2,:));
        text(0.8,0.05,xlbl,'sc');
        ylabel(ylbl); title(titl);
        if igrd==1, grid, end
        it=plt_menu(items);
        if it==1                        % zoom
                [xx,yy,but]=ginput(2);
                x(1)=xx(1);y(1)=yy(1);
                x(2)=xx(2);y(2)=yy(2);
                clear xx yy;
                nm1=n1+(n2-n1+1)*(x(1)-t(n1))/(t(n2)-t(n1));
                nm2=n1+(n2-n1+1)*(x(2)-t(n1))/(t(n2)-t(n1));
                n1=fix(nm1); n2=fix(nm2);
                if n1>n2, tt=n1; n1=n2; n2=tt; end
                if abs(n1-n2)<5, n1=1; n2=n; end
                n1=max(n1,1);n2=min(n2+1,n);
                nn(1)=n1;nn(2)=n2;
        end
%
        if it==2                        % grid on/off
                if igrd==1, igrd=0; else, igrd=1; end
        end
%
        if it==3                        % value
                but(1)=1;
                while but(1)~=3
                        [xx,yy,but(1)]=ginput(1);
                        disp(['x=' num2str(xx) '   y=' num2str(yy)]);
                end
        end
%
        if it==4                        % quit
                but=[3 3];
        end
%
        if it==5                        % help
                help tsplot;
        end
%
        if it==6                        % hide menu (and print)
                clg;
                plot(t(n1:n2),Y(n1:n2,:));
                text(0.8,0.05,xlbl,'sc');
                ylabel(ylbl); title(titl);
                if igrd==1, grid, end
                but(1)=1;
                while but(1)~=3
                        [xx,yy,but(1)]=ginput(1);
                        % place plot saving call here
                end
        end
end
ierr=0;
%

