function [out, H] = canny1D(in,N,switch)
%
%  [OUT, H] = CANNY1D(IN, N,SWITCH)
%
%	Computes 1D canny's edge detector
%
%       IN is the input linear signal 
%       N is the filter size (neighborhood) N = 7,9,11, 13, etc.
%       Possible values for SWITCH are : 
%                'filtering' : performs a simple filtering of the signal
%                'edge'      : performs a simple edge detection of the signal
%                              without filtering
%                'both'      : performs a filtering of the input before edge detection
%                 
%       
%       OUT is the edge output
%       H is the filter coefficients
%
%       Requires FILTER2 of Image Toolbox
%
%	Example 1 :  (Smoothing with Canny's operator)
%
%               IN = zeros(50,1);IN(25:45,1)=ones(21,1);%a step edge
%               IN = IN + rand(50,1)/5;%adding some noise
%               disp('Demo 1 : Lest''s perform a smoothing of the signal')
%               [OUT, H] = canny1D(IN,11,'filtering');%filter of 11 cofficients
%               figure, plot(H/max(H)),title('Canny''s filter coefficients');
%               figure, plot(OUT/max(OUT),'g'),hold on, plot(IN/max(IN),'y'),
%               title('Input and Output signals, (Input : yellow - Output : green)');
%               disp('We notice the attenuation of noise');
%
%        Example 2 : (Edge Detection with Canny's operator)
%
%               IN = zeros(50,1);IN(25:45,1)=ones(21,1);%a step edge
%               IN = IN + rand(50,1)/5;%adding some noise
%               disp('Demo 2 : Lest''s perform an edge detection of the signal')
%               [OUT, H] = canny1D(IN,9,'both');%filter of 9 cofficients
%               figure, plot(H/max(H)),title('Canny''s filter coefficients');
%               figure, plot(OUT/max(OUT),'g'),hold on, plot(IN/max(IN),'y'),
%               title('Input and Output signals, (Input : yellow - Output : green)');
%               disp('We notice the attenuation of noise and the edge detection');
%
%
%  	See also FILTER2, CONV, EDGE (Image Toolbox). 

% S. Guetari, V.1.3, P&M Curie University - Parc Laboratory 1996 - alex@robo.jussieu.fr
% Reference : O. Faugeras, "Three dimensional computer vision - 
% a geometric viewpoint", MIT Press 1993

if (nargin==1) N=9; switch = 'filtering';
elseif (nargin==2) switch = 'filtering';
elseif (nargin==3) ;
else error('Wrong number of parameters! see Help');
end%if nargin

N = 1/floor(N/2);%Now N is the step
x  =  [-1:N:0]';%Values for x-axis
%
% Coefficients from Faugeras Book
%
a1 =  0.1486768717; 
a2 = -0.2087553476; 
a3 = -1.244653939; 
a4 = -0.7912446531; 

alpha = 2.0522; 
w = 1.56939; 
lambda1=-2;

sx=sin(w*x); cx=cos(w*x);

h =     exp(-alpha*x) .* (a1*sx + a2*cx);
h = h + exp( alpha*x) .* (a3*sx + a4*cx);

h = h - lambda1/2;

l=length(h); 

H = [h ;-flipud(h(2:l-1));h(1)];%Creating the other part by symetry

H = H / sum(H);%normalized filter
F = cumsum([H]); 
F = F / sum(F);%normalized filter

in = in(:);
if strcmp(lower(switch(1:1)),'f'),
	   disp('Filtering...')
	   out  = filter2(F,in);
	   H = F;%output smoothing filter

elseif strcmp(lower(switch(1:1)),'e');
           disp('Edge Detection...')
	   out  = filter2(H,in);

elseif strcmp(lower(switch(1:1)),'b'),
           disp('Both Filtering and Edge Detection...')
	   out  = filter2(F,in);
	   out  = filter2(H,out);

else error('Wrong value for switch! see Help')

end%if switch


