
/* matlab mexfile interface to lsode
 *
 *
 * [y istate rworko iworko] = lsode(f, jac, t, y0, rtol, atol,
 *         itsom[, rworki[, iworki]])
 * with itsom = [itask, istate, iopt, mf]
 *
 *
 * Copyright Ray Muzic 1992, 1993, 1994.
 *
 * V 1.0 Ray Muzic,  6-Apr-1992
 * V 1.1a Ray Muzic, 11-Jun-1992.  Remmoved bug in the loop that copies
 *                  initial conditions to output vextor.  Fixed up other
 *                  stuff as per lint warnings.
 * V 1.1b Ray Muzic, 20-Aug-1992.  Fixed mixed-up type conversion.
 *
 ****************************************************************************
 */

#include <ctype.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <cmex.h>

#define MAXNAMLEN 256
#define DEBUG 0

static int ntp, lrw, liw, liwn, liwm, lrwn, lrwm, mu, ml;
static char f[MAXNAMLEN];	/* prhs[0] */
static char jac[MAXNAMLEN];	/* prhs[1] */
static int *iwork;		/* plhs[7] */
static double *rwork;		/* plhs[8] */
static Matrix *fplhs[1] = { NULL };
static Matrix *fprhs[4] = { NULL, NULL, NULL, NULL };
static Matrix *jplhs[1] = { NULL };
static Matrix *jprhs[6] = { NULL, NULL, NULL, NULL, NULL, NULL };

user_fcn(int nlhs, Matrix *plhs[], int nrhs, Matrix *prhs[])
{
  double	*t;				/* prhs[2] */
  double	*y;				/* prhs[3] */
  double	*r_tol;				/* prhs[4] */
  double	*a_tol;				/* prhs[5] */
  int		itask, istate, iopt, mf;	/* prhs[6] */

  int neq, i_tol;
  char msg[MAXNAMLEN];
#ifdef mips
  void lsode_(void (*f)(), int *neq, double *y, double *t, double *tout,
        int *i_tol, double *r_tol, double *a_tol, int *itask, int *istate,
 	int *iopt, double *rwork, int *lrw, int *iwork, int *liw,
        void (*jac)(), int *mf);
#else
  void lsode(void (*f)(), int *neq, double *y, double *t, double *tout,
        int *i_tol, double *r_tol, double *a_tol, int *itask, int *istate,
 	int *iopt, double *rwork, int *lrw, int *iwork, int *liw,
        void (*jac)(), int *mf);
#endif
  void fex(), jex();
  int k, n;

#if DEBUG
  printf("nlhs: %d, nrhs: %d\n", nlhs, nrhs);
#endif


/* check for nrhs, then process each rhs argument */

  if ((nrhs > 9) || (nrhs < 7))
    mex_error("[y istate rworko iworko]=lsode(f,jac,time,y0,r_tol,a_tol,itsom[,rworki[,iworki]])\n");

  /* arg [0]:  f */
  if ((prhs[0]->m != 1) && (prhs[0]->type != 1))
    mex_error("f must be a character string\n");
  for (k = 0; k < MAXNAMLEN && k < prhs[0]->n; k++)
    f[k] = (char)prhs[0]->pr[k];
  f[k] = 0;
  if (f[0]==0) mex_error("f must be specified\n");

  /* arg [1]:  jac */
  if ((prhs[1]->m != 1) && (prhs[1]->type != 1))
    mex_error("jac must be a character string\n");
  for (k = 0; k < MAXNAMLEN && k < prhs[1]->n; k++)
    jac[k] = (char)prhs[1]->pr[k];
  jac[k] = 0;

  /* arg [2]:  time */
  if ((prhs[2]->m != 1) || (prhs[2]->n == 1))
    mex_error("t must be a row vector that includes initial time point\n");
  if (prhs[2]->pi != NULL) mex_error("only real ODEs currently supported\n");
  ntp = prhs[2]->n;
  t = (double *)calloc(ntp, sizeof(double));
  if (t == 0) mex_error("Cannot allocate space for t\n");
  for (k = 0; k < prhs[2]->n; ++k) t[k] = (prhs[2]->pr)[k];

  /* arg [3]:  y0 */
  if (prhs[3]->n != 1) mex_error("y0 must be a column vector\n");
  if (prhs[3]->pi != NULL) mex_error("only real ODEs currently supported\n");
  neq = prhs[3]->m;
  y = (double *)calloc(neq, sizeof(double));
  if (y == 0) mex_error("Cannot allocate space for y0\n");
  for (k = 0; k < prhs[3]->m; ++k) y[k] = (prhs[3]->pr)[k];

  /* arg [4]:  rtol */
  if ((prhs[4]->n != 1) || !((prhs[4]->m == neq) || (prhs[4]->m == 1)))
    mex_error("rtol must be a scalar or column vector same size as y0\n");
  if (prhs[4]->pi != NULL) mex_error("only real ODEs currently supported\n");
  r_tol = (double *)calloc((int)(prhs[4]->m), sizeof(double));
  if (r_tol == 0) mex_error("Cannot allocate space for rtol\n");
  for (k = 0; k < prhs[4]->m; ++k) r_tol[k] = (prhs[4]->pr)[k];

  /* arg [5]:  a_tol */
  if ((prhs[5]->n != 1) || !((prhs[5]->m == neq) || (prhs[5]->m == 1)))
    mex_error("atol must be a scalar or column vector same size as y0\n");
  if (prhs[5]->pi != NULL) mex_error("only real ODEs currently supported\n");
  a_tol = (double *)calloc((int)(prhs[5]->m), sizeof(double));
  if (a_tol == 0) mex_error("Cannot allocate space for atol\n");
  for (k = 0; k < prhs[5]->m; ++k) a_tol[k] = (prhs[5]->pr)[k];

  if (prhs[4]->m == 1) {
    if (prhs[5]->m == 1) {
      i_tol = 1;
    } else {
      i_tol = 2;
    }
  } else {
    if (prhs[5]->m == 1) {
      i_tol = 3;
    } else {
      i_tol = 4;
    }
  }

  /* arg [6]:  itsom [itask, istate, iopt, mf] flags) */
  if (((prhs[6]->m) * (prhs[6]->n)) != 4)
    mex_error("itsom must be a vector of length 4\n");
  itask = (prhs[6]->pr)[0];
  istate = (prhs[6]->pr)[1];
  iopt = (prhs[6]->pr)[2];
  mf = (prhs[6]->pr)[3];
  if ( ((mf==21) || (mf==24)) && (jac[0]==0))
    mex_error("jac must be specified\n");

  /* arg [8]:  iworki, need to process arg [8] before [7] since contents  */
  /* of iwork may influence size of rwork.                                */
  switch (mf) {
    case 10 : liw = 20;
      break;
    case 21 :
    case 22 :
    case 24 :
    case 25 : liw = 20 + neq;
      break;
    default :
      mex_error("unrecognized mf (=itsom(4)) parameter\n");
  }
  liwm = liw;
  liwn = 1;
  if (nrhs > 8) {
    if (prhs[8]->pi != NULL) mex_error("only real ODEs currently supported\n");
    if (((prhs[8]->m) * (prhs[8]->n)) < liw) {
      sprintf(msg, "iworki array too small. is %d, should be >= %d\n",
        ((prhs[8]->m) * (prhs[8]->n)), liw);
      mex_error(msg);
    }
    liwm = prhs[8]->m;
    liwn = prhs[8]->n;
    liw = liwm * liwn;
  }
  iwork = (int *)calloc(liw, sizeof(int));
  if (nrhs > 8) for (k = 0; k < liw; ++k) iwork[k] = (prhs[8]->pr)[k];
  if ((mf == 24) || (mf == 25)) {
    ml = iwork[0];
    mu = iwork[1];
  }

  /* arg [7]:  rworki */
  switch (mf) {
    case 10 : lrw = 20 + 16 * neq;
      break;
    case 21 :
    case 22 : lrw = 22 + 9 * neq + neq * neq;
      break;
    case 24 :
    case 25 : lrw = 22 + 10 * neq + (2 * ml + mu) * neq;
      break;
    default :
      mex_error("unrecognized mf (=itsom(4)) parameter\n");
  }
  lrwm = lrw;
  lrwn = 1;
  if (nrhs > 7) {
    if (prhs[7]->pi != NULL) mex_error("only real ODEs currently supported\n");
    if (((prhs[7]->m) * (prhs[7]->n)) < lrw) {
      sprintf(msg, "rworki array too small. is %d, should be >= %d\n",
        ((prhs[7]->m) * (prhs[7]->n)), lrw);
    mex_error(msg);
    }
    lrwm = prhs[7]->m;
    lrwn = prhs[7]->n;
    lrw = lrwm * lrwn;
  }
  rwork = (double *)calloc(lrw, sizeof(double));
  if (nrhs > 7) for (k = 0; k < lrw; ++k) rwork[k] = (prhs[7]->pr)[k];

  
/* print out explicit and defaulted values of array if DEBUG */

#if DEBUG
  printf("f: %s, jac: %s\n", f, jac);
  printf("y: ["); for(k = 0; k < neq - 1; ++k) printf("%1.5e ", y[k]); printf("%1.5e]\n", y[k]);
  printf("t: ["); for(k = 0; k < ntp - 1; ++k) printf("%1.5e ", t[k]); printf("%1.5e]\n", t[k]);
  printf("rtol: ["); for(k = 0; k < prhs[4]->m - 1; ++k) printf("%1.5e ", r_tol[k]); printf("%1.5e]\n", r_tol[k]);
  printf("atol: ["); for(k = 0; k < prhs[5]->m - 1; ++k) printf("%1.5e ", a_tol[k]); printf("%1.5e]\n", a_tol[k]);
  printf("itsom = [itask istate iopt mf]: [%d %d %d %d]\n",itask, istate, iopt, mf);
  if (nrhs > 8) {printf("iworki: ["); for(k = 0; k < (prhs[8]->m)*(prhs[8]->n) - 1; ++k) printf("%d ", iwork[k]); printf("%d]\n", iwork[k]);}
  if (nrhs > 7) {printf("rworki: ["); for(k = 0; k < (prhs[7]->m)*(prhs[7]->n) - 1; ++k) printf("%1.5e ", rwork[k]); printf("%1.5e\n", rwork[k]);}
  printf("liw: %d, lrw: %d\n", liw, lrw);
#endif

/* setup output arrays */

  switch (nlhs) {
    default :
    case 4 : /* iworko */
      if (nrhs > 8) plhs[3] = create_matrix(prhs[8]->m, prhs[8]->n, REAL);
      else plhs[3] = create_matrix(liw, 1, REAL);
    case 3 : /* rworko */
      if (nrhs > 7) plhs[2] = create_matrix(prhs[7]->m, prhs[7]->n, REAL);
      else plhs[2] = create_matrix(lrw, 1, REAL);
    case 2 : /* istate */
      plhs[1] = create_matrix(ntp-1, 1, REAL);
    case 1 :   /* y */
      plhs[0] = create_matrix(neq, ntp, REAL);
      break;
  }

/* setup f and jac argument matrices for matlab_fcn */

  fprhs[0] = create_matrix(1, 1, REAL);		/* t */
  fprhs[1] = create_matrix(neq, 1, REAL);	/* y */
  fprhs[2] = create_matrix(lrwm, lrwn, REAL);	/* rwork */
  fprhs[3] = create_matrix(liwm, liwn, REAL);	/* iwork */
  fplhs[0] = create_matrix(neq, 1, REAL);	/* ydot */
  jprhs[0] = fprhs[0];				/* t */
  jprhs[1] = fprhs[1];				/* y */
  jprhs[2] = create_matrix(1, 1, REAL);		/* ml */
  jprhs[3] = create_matrix(1, 1, REAL);		/* mu */
  jprhs[4] = fprhs[2];				/* rwork */
  jprhs[5] = fprhs[3];				/* iwork */
  jplhs[0] = create_matrix(neq, neq, REAL);	/* pd */

/* do the lsode calls */

  if (nlhs > 0) for (k = 0; k < neq; k++) (plhs[0]->pr)[k] = y[k];
           /* rfm2: V 1.1a fix:  nlhs changed from lrhs, neq from ntp */
#if DEBUG
    n = 0;
    printf("y[%1.5e]: [", t[n]); for(k = 0; k < neq - 1; ++k) printf("%1.5e ", y[k]); printf("%1.5e]\n", y[k]);
#endif
  for (n = 1; n < ntp; n++) { 
#ifdef mips
    lsode_(fex, &neq, y, &t[n-1], &t[n], &i_tol, r_tol, a_tol, &itask,
         &istate, &iopt, rwork, &lrw, iwork, &liw, jex, &mf);
#else
    lsode(fex, &neq, y, &t[n-1], &t[n], &i_tol, r_tol, a_tol, &itask,
         &istate, &iopt, rwork, &lrw, iwork, &liw, jex, &mf);
#endif
    if (nlhs > 0) for (k = 0; k < neq; k++) (plhs[0]->pr)[k + n * neq] = y[k];
    if (nlhs > 1) (plhs[1]->pr)[n-1] = istate;    
#if DEBUG
    printf("y[%1.5e]: [", t[n]); for(k = 0; k < neq - 1; ++k) printf("%1.5e ", y[k]); printf("%1.5e]\n", y[k]);
#endif
    if (istate < 0) {
      sprintf(msg, "Error. At t = %1.5e, istate = %d.  Stopping LSODE.\n", 
        t[n], istate);
      mex_error(msg);
    }
  }

  switch (nlhs) {
    default :
    case 4 : /* iworko */
      for (k = 0; k < liw; k++) (plhs[3]->pr)[k] = iwork[k];
    case 3 : /* rworko */
      for (k = 0; k < lrw; k++) (plhs[2]->pr)[k] = rwork[k];
    case 2 : /* istate */
    case 1 : /* y */
      break;
  }


/* be nice and clean up after ourselves since our mothers cannot */

  free_matrix(jplhs[0]);
  free_matrix(jprhs[3]);
  free_matrix(jprhs[2]);

  free_matrix(fplhs[0]);
  free_matrix(fprhs[3]);
  free_matrix(fprhs[2]);
  free_matrix(fprhs[1]);
  free_matrix(fprhs[0]);

  free((void *)rwork);
  free((void *)iwork);
  free((void *)a_tol);
  free((void *)r_tol);
  free((void *)y);
  free((void *)t);

#if DEBUG
  printf("done freeing\n");
#endif

}

void fex(int *neq, double *t, double *y, double *ydot)
{
  int k;

/* copy data into matrices for matlab_fcn call */

  (fprhs[0]->pr)[0] = t[0];
  for (k = 0; k < *neq; k++) (fprhs[1]->pr)[k] = y[k];
  for (k = 0; k < lrw; k++) (fprhs[2]->pr)[k] = rwork[k];
  for (k = 0; k < liw; k++) (fprhs[3]->pr)[k] = iwork[k];

/* do fcn call */

  matlab_fcn(1, fplhs, 4, fprhs, f);

/* copy output */

  for (k = 0; k < *neq; k++) ydot[k] = (fplhs[0]->pr)[k];

}

void jex(int *neq, double *t, double *y, int *jml, int *jmu, double *pd, int *nrpd)
{
   int k;


/* copy data into matrices for matlab_fcn call */

  (jprhs[0]->pr)[0] = t[0];
  for (k = 0; k < *neq; k++) (jprhs[1]->pr)[k] = y[k];
  (jprhs[2]->pr)[0] = (double)(*jml);
  (jprhs[3]->pr)[0] = (double)(*jmu);
           /* rfm2: V 1.1b fix: added (* ) in above 2 lines
  for (k = 0; k < lrw; k++) (jprhs[4]->pr)[k] = rwork[k];
  for (k = 0; k < liw; k++) (jprhs[5]->pr)[k] = iwork[k];


/* do fcn call */

  matlab_fcn(1, jplhs, 6, jprhs, jac);

/* copy output */

  for (k = 0; k < (*neq)*(*neq); k++) pd[k] = (jplhs[0]->pr)[k];

}
