function x = block_hsolve(p,h,b,nref,factor);
%BLOCK_HSOLVE  
%        A fast block Hankel solver. BLOCK_HSOLVE solves linear 
%        systems Hx=b with H a complex regular n*p x n*p block Hankel 
%        matrix (having square blocks of size p) in O(p^3 n^2) flops.
%
%        x = block_hsolve(p,h,b,nref,factor);
%
%        INPUT PARAMETERS
%
%        p       Positive integer. The size of the (square) blocks.
%
%        h       Block row vector defining the Hankel matrix. 
%                The length of h should be an odd multiple of p. 
%                Let n = (size(h,2)/p + 1)/2 and define 
%                  H_s = h(1:p,(s-1)*p+1:(s-1)*p+p) 
%                for s=1:2*n-1. Let H be the block Hankel matrix
%                  H = hankel( [H_1 ... H_n],[H_n ... H_(2*n-1)] ).
%                Then the vector h should be such that the matrix H is
%                regular.
%
%        b       Column vector of length n*p. The right hand side.
%
%        nref    Integer. The number of times that the computed solution 
%                is to be improved iteratively. Each step costs 5*n*p^2
%                + p^2*O(n log n) flops.
%
%        factor  Positive real. The larger part of the algorithm is 
%                concerned with the solution of two rational 
%                interpolation problems on the unit circle. 
%                The algorithm stops if all the residuals are smaller 
%                than factor times EPS. This means that H is singular 
%                or ill-conditioned. In the latter case you may want to 
%                lower the value of factor. 
%
%        OUTPUT
%
%        x       Matrix of size n*p x (nref + 1). The columns of this 
%                matrix contain the successive approximations to the 
%                solution of the system Hx=b. More precisely, the first 
%                column contains the solution as calculated by the 
%                algorithm without iterative improvement. The second 
%                column contains the result of one additional step of 
%                iterative improvement, and so on.
%
%        EXAMPLE
%                 p = 2; n = 3;
%                 h = -1 + 2*RAND(p,p*(2*n-1));
%                 FOR r = 1:n,
%                   FOR s = 1:n,
%                     H((r-1)*p+1:(r-1)*p+p,(s-1)*p+1:(s-1)*p+p) ...
%                     = h(1:p,(r+s-2)*p+1:(r+s-2)*p+p);
%                   END;
%                 END;
%                 b = H*ONES(n*p,1);
%                 nref = 1; factor = 1E+9;
%                 x = BLOCK_HSOLVE(p,h,b,nref,factor)
%
%        Written by Peter Kravanja and Marc Van Barel
%                   Dept. Computer Science
%                   Katholieke Universiteit Leuven
%                   Celestijnenlaan 200 A, B-3001 Heverlee, Belgium
%
%                   E-mail: na.pkravanja@na-net.ornl.gov
%                           marc@cs.kuleuven.ac.be
%        (October 10, 1996)
%
%        See also BLOCK_RATINT. 
%        Use HSOLVE if p==1.

%-----------------------------------------------------------------------
%
  if p == 1
    disp(' ');
    disp(' Warning: use HSOLVE if p==1.');
    disp(' ');
  end;
  I = sqrt(-1);
  n = (size(h,2)/p + 1)/2;
  if ~( (2*n-1)*p - size(h,2) == 0 )
    disp(' ');
    disp(' The length of h should be an odd multiple of p.');
    disp(' ');
    error;
  end;
%-----------------------------------------------------------------------
  if h == zeros(p,(2*n-1)*p)
    disp(' ');
    disp(' Hankel matrix is equal to the zero matrix.');
    disp(' ');
    error;
  end;
  if n == 1
    if rank(h) < p
      disp(' ');
      disp(' Hankel matrix is singular.');
      disp(' ');
      error;
    else
      x(:,1) = h\b;
      for r = 2:nref+1,
        x(:,r) = x(:,1);
      end; 
      return;
    end;
  end;
%-----------------------------------------------------------------------
  alpha = zeros(n,1);
  alpha(2) = 1;
  y = conj(fft(alpha));
  zeta = exp(pi*I/n);
  z = zeta*y;
  alpha = zeros(2*n,1);
  alpha(2) = 1;
  zeta_2n = conj(fft(alpha));
  zeta_n = zeta_2n(1:n);
%-----------------------------------------------------------------------
  invpivot = 1 + [0:2:2*n-2 1:2:2*n-1];
  for r = 1:p,
    for s = 1:p,
      for t = 1:2*n-1,
        h1(t) = h(r,(t-1)*p+s);
      end;
      param = conj(zeta_2n).*fft([h1.';0]);
      param = param(invpivot);
      for t = 1:n,
        c((t-1)*p+r,s) = param(t);
        d((t-1)*p+r,s) = param(t+n);
      end;
    end;
  end;
%-----------------------------------------------------------------------
  for r = 1:p,
    for t = 1:n,
      b1(t,1) = b((t-1)*p+r);
    end;
    bpr1 = conj(y).*fft(b1);
    for t = 1:n,
      bpr((t-1)*p+r,1) = bpr1(t);
    end;
  end;
%-----------------------------------------------------------------------
  F = [c; d];
  [ePtilde,eUtilde,alpha,hn] = block_ratint(y,z,F,p,factor,0);
  if hn < 2*n*p
    disp(' ');
    disp(' Matrix is (nearly) singular');
    disp(' You may want to lower the value of factor.');
    disp(' ');
    error;
  end;
  for r = 1:n,
    Utilde((r-1)*p+1:(r-1)*p+p,1:p) = (-conj(zeta^(n-1))*y(r)/n)* ...
      eUtilde((r-1)*p+1:(r-1)*p+p,:);
    Ptilde((r-1)*p+1:(r-1)*p+p,1:p) = (conj(zeta^(n-1))*y(r)/n)* ...
      ePtilde((r-1)*p+1:(r-1)*p+p,:);
  end;

  for r = 1:n,
    F((r-1)*p+1:(r-1)*p+p,1:p) = c((r-1)*p+1:(r-1)*p+p,:).';
    F((n+r-1)*p+1:(n+r-1)*p+p,1:p) = d((r-1)*p+1:(r-1)*p+p,:).';
  end;
  [eP,eU,alpha,hn] = block_ratint(y,z,F,p,factor,1);
  if hn < 2*n*p
    disp(' ');
    disp(' Matrix is (nearly) singular');
    disp(' You may want to lower the value of factor.');
    disp(' ');
    error;
  end;
  for r = 1:n,
    U((r-1)*p+1:(r-1)*p+p,1:p)=(-y(r)/n)*eU((r-1)*p+1:(r-1)*p+p,:);
    P((r-1)*p+1:(r-1)*p+p,1:p)=(y(r)/n)*eP((r-1)*p+1:(r-1)*p+p,:);
  end;
%-----------------------------------------------------------------------
  for r = 1:n,
    bpr_a((r-1)*p+1:(r-1)*p+p,1) = P((r-1)*p+1:(r-1)*p+p,:) * ...
      bpr((r-1)*p+1:(r-1)*p+p,1); 
    bpr_b((r-1)*p+1:(r-1)*p+p,1) = U((r-1)*p+1:(r-1)*p+p,:) * ...
      bpr((r-1)*p+1:(r-1)*p+p,1); 
  end;
  for r = 1:p,
    for t = 1:n,
      bpr_a1(t,1) = bpr_a((t-1)*p+r);
      bpr_b1(t,1) = bpr_b((t-1)*p+r);
    end;
    bpr_a2 = -0.5*n*conj(fft(conj(zeta_n).*ifft(y.*conj(bpr_a1))));
    bpr_b2 = -0.5*n*conj(fft(conj(zeta_n).*ifft(y.*conj(bpr_b1))));
    for t = 1:n,
      bpr_aa((t-1)*p+r,1) = bpr_a2(t);
      bpr_bb((t-1)*p+r,1) = bpr_b2(t);
    end;
  end;
  for r = 1:n,
    bpr_aaa((r-1)*p+1:(r-1)*p+p,1)=Utilde((r-1)*p+1:(r-1)*p+p,:)*...
      bpr_aa((r-1)*p+1:(r-1)*p+p,1); 
    bpr_bbb((r-1)*p+1:(r-1)*p+p,1)=Ptilde((r-1)*p+1:(r-1)*p+p,:)*...
      bpr_bb((r-1)*p+1:(r-1)*p+p,1);
  end;
  xpr = bpr_bbb - bpr_aaa;
  
  for r = 1:p,
    for t = 1:n,
      xpr1(t,1) = xpr((t-1)*p+r);
    end;
    x1 = n*zeta_n(n:-1:1).*conj(ifft(y.*conj(xpr1)))/alpha;
    for t = 1:n,
      x((t-1)*p+r,1) = x1(t);
    end;
  end;
%-----------------------------------------------------------------------
  for k = 1:nref,
    res = zeros(n*p,1);
    for r = 1:p,
      for t = 1:n,
        vec(t,1) = x((t-1)*p+r,k);
      end;
      for s = 1:p,
        for t = 1:2*n-1,
          h1(t) = h(s,(t-1)*p+r);
        end;
        cv = conv(h1.',flipud(vec));
        tmp = cv(n:2*n-1);
        for t = 1:n,
          delta((t-1)*p+s,1) = tmp(t);
        end;
      end; 
      res = res + delta;
    end;
    residue = b - res; 

    for r = 1:p,
      for t = 1:n,
        residue1(t,1) = residue((t-1)*p+r);
      end;
      residuepr1 = conj(y).*fft(residue1);
      for t = 1:n,
        residuepr((t-1)*p+r,1) = residuepr1(t);
      end;
    end;

    for r = 1:n,
      residuepr_a((r-1)*p+1:(r-1)*p+p,1) = ... 
        P((r-1)*p+1:(r-1)*p+p,:) * residuepr((r-1)*p+1:(r-1)*p+p,1);
      residuepr_b((r-1)*p+1:(r-1)*p+p,1) = ...
        U((r-1)*p+1:(r-1)*p+p,:) * residuepr((r-1)*p+1:(r-1)*p+p,1);
    end;
    for r = 1:p,
      for t = 1:n,
        residuepr_a1(t,1) = residuepr_a((t-1)*p+r);
        residuepr_b1(t,1) = residuepr_b((t-1)*p+r);
      end;
      residuepr_a2 = ...
        -0.5*n*conj(fft(conj(zeta_n).*ifft(y.*conj(residuepr_a1))));
      residuepr_b2 = ...
        -0.5*n*conj(fft(conj(zeta_n).*ifft(y.*conj(residuepr_b1))));
      for t = 1:n,
        residuepr_aa((t-1)*p+r,1) = residuepr_a2(t);
        residuepr_bb((t-1)*p+r,1) = residuepr_b2(t);
      end;
    end;
    for r = 1:n,
      residuepr_aaa((r-1)*p+1:(r-1)*p+p,1) = ...
        Utilde((r-1)*p+1:(r-1)*p+p,:) * ...
        residuepr_aa((r-1)*p+1:(r-1)*p+p,1);
      residuepr_bbb((r-1)*p+1:(r-1)*p+p,1) = ...
        Ptilde((r-1)*p+1:(r-1)*p+p,:) * ...
        residuepr_bb((r-1)*p+1:(r-1)*p+p,1);
    end;
    xnewpr = residuepr_bbb - residuepr_aaa;

    for r = 1:p,
      for t = 1:n,
        xnewpr1(t,1) = xnewpr((t-1)*p+r);
      end;
      xnew1 = n*zeta_n(n:-1:1).*conj(ifft(y.*conj(xnewpr1)))/alpha;
      for t = 1:n,
        xnew((t-1)*p+r,1) = xnew1(t);
      end;
    end;
    x(:,k+1) = x(:,k) + xnew;
  end;
  return;
%-----------------------------------------------------------------------
