function x = hsolve(h,b,nref,factor);
%HSOLVE  A fast Hankel solver.
%        HSOLVE solves linear systems Hx=b with H a complex regular 
%        n x n Hankel matrix in O(n^2) flops.
%
%        x = hsolve(h,b,nref,factor);
%        
%        INPUT PARAMETERS
%
%        h       Row vector defining the Hankel matrix. The length of h 
%                should be odd. Let n = (size(h,2) + 1)/2 and 
%                H = hankel(h(1:n),h(n:2*n-1)). The vector h should be 
%                such that the matrix H is regular.
%
%        b       Column vector of length n. The right hand side.
%
%        nref    Integer. The number of times that the computed solution 
%                is to be improved iteratively. Each step costs O(n log n) 
%                flops.
%
%        factor  Positive real. The larger part of the algorithm is 
%                concerned with the solution of a rational interpolation 
%                problem on the unit circle. The algorithm stops if all 
%                the residues are smaller than factor times EPS. This 
%                means that H is singular or ill-conditioned. In the 
%                latter case you may want to lower the value of factor. 
%
%        OUTPUT
%
%        x       Column vector of length n. The computed solution to Hx=b.
%
%        EXAMPLE 
%                 n = 5;
%                 h = -1 + 2*RAND(1,2*n-1);
%                 H = HANKEL(h(1:n),h(n:2*n-1));
%                 b = H*ONES(n,1);
%                 nref = 1; factor = 10000;
%                 x = HSOLVE(h,b,nref,factor)
%
%        Written by Peter Kravanja and Marc Van Barel
%                   Dept. Computer Science
%                   Katholieke Universiteit Leuven
%                   Celestijnenlaan 200 A, B-3001 Heverlee, Belgium
%
%                   E-mail: na.pkravanja@na-net.ornl.gov
%                           marc@cs.kuleuven.ac.be
%        (July 10, 1996)
%
%        See also RATINT.

%-----------------------------------------------------------------------
%
  I = sqrt(-1);
  n = (size(h,2) + 1)/2;
%-----------------------------------------------------------------------
  if h == zeros(1,size(h))
    disp(' ');
    disp(' Hankel matrix is equal to the zero matrix.');
    disp(' ');
    error;
  end;
  if n == 1
    x = b/h;
    return;
  end;
%-----------------------------------------------------------------------
  alpha = zeros(n,1);
  alpha(2) = 1;
  y = conj(fft(alpha));
  zeta = exp(pi*I/n);
  z = zeta*y;
  alpha = zeros(2*n,1);
  alpha(2) = 1;
  zeta_2n = conj(fft(alpha));
  zeta_n = zeta_2n(1:n);
%-----------------------------------------------------------------------
  for k = 1:n,
    pivot(2*k-1) = k;
    pivot(2*k) = n + k;
  end;
  invpivot = 1 + [0:2:2*n-2 1:2:2*n-1];
  param = conj(zeta_2n).*fft([h.'; 0]);
  param = param(invpivot);
  c = param(1:n);
  d = param(n+1:2*n);
%-----------------------------------------------------------------------
  bpr = conj(y).*fft(b); 
%-----------------------------------------------------------------------
  Z = [y; z];
  F = [c; d];
  ZZ = Z(pivot);
  FF = F(pivot);
  [DL,DR,alpha,hn]=ratint(ZZ,FF,2*n,factor);
  if hn < 2*n
    disp(' ');
    disp(' Matrix is (nearly) singular');
    disp(' You may want to lower the value of factor.');
    disp(' ');
    error;
  end;
  DL = DL(invpivot); DR = DR(invpivot);
  utilde = -conj(zeta^(n-1))*y.*DR(n+1:2*n)/n;
  u = -y.*DR(1:n)/n;
  ptilde = conj(zeta^(n-1))*y.*DL(n+1:2*n)/n;
  p = y.*DL(1:n)/n;
%-----------------------------------------------------------------------
  xpr = -n*(ptilde.*conj(fft(conj(zeta_n).*ifft(y.*conj(u.*bpr))))- ...
    utilde.*conj(fft(conj(zeta_n).*ifft(y.*conj(p.*bpr)))))/2;
  x = n*zeta_n(n:-1:1).*conj(ifft(y.*conj(xpr)))/alpha;
%-----------------------------------------------------------------------
  for k = 1:nref,
    cv = conv(h.',flipud(x));
    residue = b - cv(n:2*n-1);
    residuepr = conj(y).*fft(residue);
    xnewpr = -n*(ptilde.*conj(fft(conj(zeta_n).* ...
     ifft(y.*conj(u.*residuepr)))) - utilde.*conj(fft(conj(zeta_n).* ...
     ifft(y.*conj(p.*residuepr)))))/2;
    xnew = n*zeta_n(n:-1:1).*conj(ifft(y.*conj(xnewpr)))/alpha;
    x = x + xnew;
  end;
  return;
%-----------------------------------------------------------------------
