function [ DOPT ] = osb( MAT , TOL )

%                   [ DOPT ] = osb( MAT , TOL )
%
%     Given a square, complex matrix MAT, this function computes
%     a real, positive, diagonal matrix DOPT which minimizes
%
%                    norm( D * MAT / D , 'fro' )
%
%     over all complex, nonsingular, diagonal D. The algorithm, which
%     is described in [1], is iterative in nature and terminates
%     according to the tolerance parameter TOL. A tolerance of .01
%     is usually adequate for most purposes, and TOL defaults to this
%     value when the function is called with a single argument.
%
%     If MAT is reducible by permutation matrices [1], overflow may
%     be encountered. However, generically MAT will not be reducible
%     by permutation matrices, as this requires that at least one off-
%     diagonal entry of MAT be identically zero. If overflow should
%     occur, the null off-diagonal entries of MAT should be replaced
%     with `small' values, and the computation repeated.
%
%     [1] E.E. Osborne, `On Pre-Conditioning of Matrices,' Journal
%     of the Association for Computing Machinery,' Vol. 7, pp. 338-
%     345, 1960.

%     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*
%           Dr. C. N. Nett, GE CRD/CTB, (518) 387-5539, 5 July 87
%     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*

                                                 % INIT
CIT = 0 ;
I = 0 ;
if ( nargin == 1 ) , TOL = .01 ; end
                                                % K = 1
N = max( size( MAT ) ) ;
D = eye( N , N ) ;
                                                 % LOOP
while CIT < N
                                               % CYC
   if I == N , I = 0 ; end
   I = I + 1 ;
                                               % LDB
   TMP = MAT ;
   TMP( I , I ) = 0 ;
   R = norm( TMP( I , : ) ) ;
   S = norm( TMP( : , I ) ) ;
   LDB = sqrt( S / R ) ;
                                                % DB
   if I < N

      DB = diag( ones( 1 , N ) ) ;
      DB( I , I ) = LDB ;

   else

      DB = diag( ones( 1 , N ) / LDB ) ;
      DB( I , I ) = 1 ;

   end
                                                % K INC
   D = DB * D ;
   MAT = DB * MAT / DB ;
                                               % TOL
   if abs( LDB - 1 ) < TOL , CIT = CIT + 1 ; else CIT = 0 ; end

end

DOPT = D ;
