function [smin,vmin] = ccvl(R)
% [smin,vmin] = ccvl(R)
%
% Computes an estimate smin for the smallest singular value
% of the upper triangular matrix R, as well as an estimate
% vmin of the corresponding right singular vector.

% Ref.: A. K. Cline, A. R. Conn & C. F. Van Loan, "Generalizing the
% LINPACK condition estimator", Lecture Notes in Mathematics Vol. 909,
% pp. 73-83, Springer Verlag, 1982.

% Christian H. Bischof, Argonne National Laboratory, 06/06/89.
% Slightly modified by Per Christian Hansen, UNI-C, 01/08/91.

[m,n] = size(R);
if (m~=n), error('R is not square'), end

if (any(diag(R)==0))  % matrix is singular
  smin = 0; vmin = zeros(n,1); vmin(min(find(diag(R)==0))) = 1;
  return
end

v = zeros(n,1); fc = 0;

% First element is treated special.
v(1) = 1/R(1,1); vnorm = abs(v(1)); p = v(1)*R(1,:);

% Process rows of matrix one by one.
for i = 2:n-1

  u = R(i,i+1:n); utp = u*p(i+1:n)'; gamma = R(i,i);
  xi = p(i); phi = 1 + norm(u)^2; pnorm = norm(p(i+1:n));
  alpha = xi*phi - gamma*utp;

  if (alpha==0)
    beta = gamma^2*(vnorm^2 + pnorm^2) - xi^2*phi;
    if (beta > phi), s = 1; c = 0; else, s = 0; c = 1; end
  else
    beta = gamma^2*(vnorm^2+pnorm^2) + (xi^2-1)*phi - 2*xi*gamma*utp;
    eta = beta/(2*alpha); mu = eta + sign(alpha)*pythag(eta,1);
    root = pythag(mu,1); s = mu/root; c = -1/root;
  end

  v(1:i) = [s*v(1:i-1);(c - s*xi)/gamma];
  vnorm = pythag(s*vnorm,(c - s*xi)/gamma);
  p(i+1:n) = s*p(i+1:n) + v(i)*u;

end

% Last step is the same as in incremental condition estimation.
alpha = p(n); gamma = R(n,n);
if (alpha==0)
  beta = gamma^2*vnorm^2;
  if (beta > 1)
     s = 1; c = 0; lambda_max = beta;
  else
     s = 0; c = 1; lambda_max = 1;
  end
else
  beta = gamma^2*vnorm^2 + alpha^2 - 1; eta = beta/(2*alpha);
  mu = eta + sign(alpha)*pythag(eta,1); lambda_max = alpha*mu + 1;
  root = pythag(mu,1); s = mu/root; c = -1/root;
end
v = [s*v(1:n-1);(c-s*alpha)/gamma];
vnorm = sqrt(lambda_max)/abs(gamma);

% Compute and normalize right nullvector.
vmin = R\(v/vnorm); smin = 1/norm(vmin); vmin = smin*vmin;
