function [Q,R,Pi,rank,W,delta] = rrqr(A,tol)
% R = rrqr(A,tol)
% [Q,R,Pi,rank,W,delta] = rrqr(A,tol)
%
% Compute a rank revealing QR factorization of A:
%    A*Pi = Q*R = Q [ R_11 R_12 ] ,
%                   [  0   R_22 ]
% such that Q is m-by-n, R is triangular n-by-n, and
%  1)  R_11 is rank-by-rank and well-conditioned,
%  2)  rank = numerical rank of A with respect to tol,
%      defined as the number of singular values greater than tol,
%  3)  norm(R_22) is of the same order of magnitude as the
%      (rank+1)'th singular value.
%
% Also, return a matrix W whose columns are orthonormal and span
% an aproximation to the null space of A, and delta(rank:n1:2)
% containing lower and upper bounds for the last n-rank+1 singular
% values of A (the first rank-1 rows of delta are zero).
%
% If no tol is specified, sqrt(n)*norm(A,1)*eps is default.

% This program is an implementation of the algorithm described in the
% two papers: L. V. Foster, "Rank and null space calculations using
% matrix decomposition without column interchanges", Lin. Alg. Appl.
% 74 (1986), 47-71, and T. F. Chan, "Rank revealing QR factorizations",
% Lin. Alg. Appl. 88/89 (1987), 67-82.
% The use of the factor c_max_ratio was suggested in: C. H. Bischof
% & P. C. Hansen, "Structure preserving and rank revealing QR-factori-
% zations", SISSC, to appear

% Christian H. Bischof, Argonne National Laboratory &
% Per Christian Hansen, UNI-C, 01/08/91.

[m,n] = size(A); delta = zeros(n,2); c_max_ratio = 10;
if (nargin==1), tol = sqrt(n)*norm(A,1)*eps; end
if (nargout>4), W = []; end

% Compute an initial QR factorization A*Pi = Q*R.
[Q,R,Pi] = qr(A); Q = Q(:,1:n); R = R(1:n,:);

% Prepare for the iterations.  Estimate smallest singular value of R.
nu = 0; i = n;
[sest,v] = ccvl(R); if (nargout>5), delta(n,1) = sest; end

% Loop until a singular value estimate larger than tol is found.
while sest < tol, nu = nu+1;

  % Update the matrix W.
  if (nargout>4), W = [[v;zeros(nu-1,1)],W]; end

  % Find the element in v with greatest index, numerically
  % within a factor c_max_ratio of the numerically largest element.
  vmax = norm(v,inf);
  for k=i:-1:1
    if (vmax <= abs(v(k))*c_max_ratio), break, end
  end

  % If necessary, generate the permutation that brings the pivot
  % element of v to the last position, apply the permutation to W,
  % Pi, and R, compute a new QR factorization of R(1:i,1:i), and
  % update Q and R.
  if (k<i)
    p = [k+1:i,k]; Pi(:,k:i) = Pi(:,p); R(:,k:i) = R(:,p);
    if (nargout>4), W(k:i,:) = W(p,:); end
    for j=k:i-1
      [c,s] = gen_g(R(j,j:n),R(j+1,j:n));
      [R(j,j:n),R(j+1,j:n)] = app_g_left(c,s,R(j,j:n),R(j+1,j:n));
      R(j+1,j) = 0;
      [Q(:,j),Q(:,j+1)] = app_g_right(c,s,Q(:,j),Q(:,j+1));
    end
  end

  % Provide an upper bound for the i'th singular value.
  if (nargout>5), delta(i,2) = norm(R(i:n,i:n)); end

  % Estimate the smallest singular value of R(1:i-1,1:i-1),
  % which is a lower bound for the (i-1)'th singular value.
  i = i-1; [sest,v] = ccvl(R(1:i,1:i));
  if (nargout>5), delta(i,1) = sest; end

end

% Finish the computation.  If nargout < 2, return R.
if (nargout>5), delta(i,2) = norm(R(rank:n,rank:n)); end
rank = n - nu; if (nargout>4), W = Pi*W; end
if (nargout < 2), Q = R; end
