% Script: rrqr_demo
% This script demonstrates the Rank Revealing QR Factorization
% and its use in numerical linear algebra.

% Per Christian Hansen, UNI-C, 01/05/91.

echo on
% Demonstration of the Rank Revealing QR Factorization and some of its
% applications in numerical linear algebra for treating:
%   - rank deficient least squares problems
%   - subset selection
%   - matrix approximation
%   - subset selection.
%
% For matrices with full rank, the RRQR factorization is identical to
% an ordinary QR factorization.
% For rank deficient matrices, the RRQR factorization is GUARANTEED to
% reveal the numerical rank r of the matrix by displaying a lower
% (n-r)-by-(n-r) triangular submatrix of the triangular factor having
% elements of the same size as the smallest singular values.
% The RRQR factorization can therefore be used to as a reliable and
% computationally efficient alternative to the SVD.
pause     % Hit any key to continue ...
%
% We generate a rank deficient matrix A with numerical rank 3:
m = 10; n = 5; A = rand(m,n); [U,S,V] = svd(A);
S(4,4) = 1e-12; S(5,5) = 1e-14; A = U*S*V'; r = 3;
format short e, svd(A)
% Now we compute the RRQR factorization of A and display its
% triangular factor R:
tol = 1e-10; [Q,R,Pi,rank,W] = rrqr(A,tol); R
pause     % Hit any key to continue ...
%
% The RRQR factorization can be used to compute Truncated-SVD (TSVD)
% solutions to rank deficient least squares problems.  The TSVD solution
% to the problem  min || A x - b ||  is defined as
%    x_tsvd = V(:,1:r)*inv(S(1:r,1:r))*U(:,1:r)'*b .
% This solution can also be computed, more efficiently, by means of the
% RRQR factorization of A.  Below we compute the TSVD solution by means
% of both SVD and by RRQR, and we compute their difference:
b = rand(m,1); x_tsvd = V(:,1:r)*inv(S(1:r,1:r))*U(:,1:r)'*b;
x_tsvdrrqr = tsvd_rrqr(Q,R,Pi,W,b);
[x_tsvd,x_tsvdrrqr,x_tsvd-x_tsvdrrqr]
pause     % Hit any key to continue ...
%
% RRQR factorizations can also be used for computing the Basic solution
% and Truncated QR solution to rank deficient least squatres problems
% Let the RRQR factorizatin of A be given by
%    A*Pi = Q*[ R_11 R_12 ] .
%             [  0   R_22 ]
% Then the Basic solution is defined by
%    x_b = Pi*[ inv(R_11) 0 ]*Q'*b ,
%             [     0     0 ]
% while the Truncated QR solution is defined by
%    x_tqr = Pi*P*[ inv(T_11) 0 ]*Q'*b ,
%                 [     0     0 ]
% where P is an orthogonal matrix that annihilates R_12:
%    [ R_11 R_12 ]*P = [ T_11 0 ] .
% We compare x_tsvd, x_tqr, x_b and, below, the norms of the
% corresponding residual vectors:
x_tqr = tqr(Q,R,Pi,b,r); x_b = basic(Q,R,Pi,b,r);
[x_tsvd,x_tqr,x_b]
[norm(b-A*x_tsvd),norm(b-A*x_tqr),norm(b-A*x_b)]
% Notice that the residual are practically the same!
pause     % Hit any key to continue ...
%
% RRQR factorizations can also be used for doing subset selection, as
% an alternative to the computationally expensive SVD approach.
% Below we apply subset selection based on SVD and RRQR to the same
% matrix A as above.
% We also give the subspace angle between the two sets of columns
% produced by the two methods:
[QV,RV,PV] = qr(V(:,r+1:n)'); A_svd = A*PV(:,1:r);
A_rrqr = A*Pi(:,1:r); [A_svd,A_rrqr]
spangle(orth(A_svd),orth(A_rrqr))
% Notice that although the subsets may be different, the subspaces
% that they span are practically the same.
pause     % Hit any key to continue ...
%
% Another application of RRQR is matrix approximations.
% It is well-known that the closest rank-k approximation to A is the
% truncated SVD of A, given by
%    A_k = U(:,1:k)*S(1:k,1:k)*V(:,1:k)' .
% The RRQR yields a good approximation to A_k, given by
%    B_k = Q*[ R_11 R_12 ]*Pi .
%            [  0    0   ]
% We show norm(A_k-B_k) for k = 1,...,n (remember, rank(A) = 3):
for k=1:n
   A_k = U(:,1:k)*S(1:k,1:k)*V(:,1:k)';
   B_k = mat_approx(Q,R,Pi,k);
   norms(1,k) = norm(A_k - B_k);
end
norms
pause     % Hit any key to continue ...
%
% Yet another application of RRQR is Total Least Squares (TLS). The TLS
% solution is defined by means of the SVD of A.  An approximation to
% the TLS solution can be computed from the null-space vectors in the
% matrix W from RRQR.
% Here, we use SVD and RRQR to solve two TLS problem with the same
% matrix A as above, and right-hand sides b chosen so that the two
% problems have a small and a large residual.
% We also show the difference between the two solutions to each problem
% and, below that, the corresponding residual norms.
% First, the small-residual problem:
b = A*rand(n,1) + 1e-7*U(:,r+1:m)*rand(m-r,1);
x_svd = tls(A,b,r); x_rrqr= tls_rrqr(A,b,r);
[x_svd,x_rrqr,x_svd-x_rrqr]
[norm(b-A*x_svd),norm(b-A*x_rrqr)]
pause     % Hit any key to continue ...
%
% And now the large-residual problem:
b = b + 0.1*U(:,r+1:m)*rand(m-r,1);
x_svd = tls(A,b,r); x_rrqr= tls_rrqr(A,b,r);
[x_svd,x_rrqr,x_svd-x_rrqr]
[norm(b-A*x_svd),norm(b-A*x_rrqr)]
%
% This is the end of the RRQR demo.  Thank you for your attention.
echo off
