function [U0,V0,no_iterations] = spit(R,V)
% [U0,V0,no_iterations] = spit(R,Pi,V)
%
% Given an aproximate basis V for the numerical null-space of R,
% improve this basis by means of inverse subspace iterations.
%
% The results, U0 and V0, are bases for the left and right numerical
% null-spaces of R.  In addition, no_iterations is the number of
% iterations used.

% This algorithm is described in: T. F. Chan & P. C. Hansen,
% "Computing truncated SVD least squares solutions by rank revealing
% QR factorizations", SISSC 11 (1990), 519-530.

% Per Christian Hansen, UNI-C, 04/29/1990.

% Initialize.
[n,n] = size(R); [n,nu] = size(V); no_iterations = 0;
if (nu == 0), return, end
V1 = mgs(V); U0 = zeros(n,nu); V0 = zeros(n,nu); max_iterations = 10;

% Fix very small diagonal entries of R, and set the corresponding
% columns of U0 to [ 0; eye].
tol = sqrt(n)*norm(R,1)*eps;
p = find(abs(diag(R)) < tol ); lp = length(p);
if lp > 0
  R(p,p) = tol*eye(lp);
  U0(n-lp+1:n,nu-lp+1:nu) = eye(lp);
end

% Perform inverse iterations with R.  The quantity sint is an estimate
% of the subspace angle between the previous and the present null-space.
sint = 1; tol = tol*1.0e5;
while ( sint > tol & no_iterations < max_iterations )
  no_iterations = no_iterations + 1;
  U1 = R'\V1; U1 = U1(1:n-lp,1:nu-lp);
  U0(1:n-lp,1:nu-lp) = U1/norm(U1,'fro');
  V0 = R\U0; V0 = mgs(V0);
  sint = sqrt(nu)*norm((eye(n) - V1*V1')*V0,1); V1 = V0;
end

% Return if too many iterations.
if (no_iterations == max_iterations & sint > tol)
  error('Too many subspace iterations in spit')
end

% Orthonormalize the columns of U0.
U0(1:n-lp,1:nu-lp) = mgs(U0(1:n-lp,1:nu-lp));
