function X = tls_rrqr(A,B,rank)
% X = tls_rrqr(A,B)
% X = tls_rrqr(A,B,rank)
%
% Given A, B and rank = rank(A), compute the Total Least Squares
% solution X based on the null-space matrix W from a RRQR factorization
% of the compound matrix [A,B].  Orthonormalize the columns of Pi*W,
%    Pi*W = [ V_1 ]*RR ,  size(V_2) = [p,n-rank+p] ,
%           [ V_2 ]
% and set the TLS-RRQR solution X to
%    X = - V_1*pinv(V_2) .

% This algorithm is described in: T. F. Chan & P. C. Hansen, "Some
% applications of the rank revealing QR factorization", CAM Report
% 90-09, Dept. of Mathematics, UCLA; submitted to SISSC.

% Per Christian Hansen, UNI-C, 01/05/91.

% Initialization.
[m,n] = size(A); [m1,p] = size(B);
if (nargin==2), rank = n; end
if (m1~=m), error('A and B must have the same number of rows'), end
if (m<n-rank+p), error('The system is underdetermined'), end
W = []; c_max_ratio = 10; np = n+p;

% Compute a RRQR factorization of [A,B].  Start out with an
% initial QR factorization.
[Q,R,Pi] = qr([A,B]); R = R(1:np,:); clear Q

% Loop to treat the n-rank+p smallest singular values of [A,B].
nu = 0;
for i=np:-1:rank+1, nu = nu+1;

  % Estimate right singular vector.
  [sest,v] = ccvl(R(1:i,1:i));

  % Update the matrix W.
  W = [[v;zeros(nu-1,1)],W];

  % Find the element in v with greatest index, numerically
  % within a factor c_max_ratio of the numerically largest element.
  vmax = norm(v,inf);
  for k=i:-1:1
    if (vmax/abs(v(k)) <= c_max_ratio), break, end
  end

  % If necessary, generate the permutation that brings the pivot
  % element of v to the last position, apply the permutation to W,
  % Pi and R, and compute a new QR factorization of R(1:i,1:i).
  if (k<i)
    p = [k+1:i,k];
    W(k:i,:) = W(p,:); Pi(:,k:i) = Pi(:,p); R(:,k:i) = R(:,p);
    for j=k:i-1
      [c,s] = gen_g(R(j,j:np),R(j+1,j:np));
      [R(j,j:np),R(j+1,j:np)] = app_g_left(c,s,R(j,j:np),R(j+1,j:np));
      R(j+1,j) = 0;
    end
  end
end

% Orthonormalize the columns of Pi*W.
V = mgs(Pi*W);

% Compute the TLS-RRQR solution.
X = -((V(n+1:np,:)')\(V(1:n,:)'))';
