function [E,mlam,outlier,acc]=ellibnd(R,E0,method,a,acc);
%   Performs recursive ellipsoidal bounding.
%   Usage:
%        [E,mlam,outlier,acc]=ellibnd(R,E0,method,a,acc); 
%   R       - Data record, format [A,b1,b2] for inequality b1<=Ax<=b2
%             according to GBT specifications (see manual)
%   E0      - Initial ellipsoid, format according to GBT specifications
%   method  - Choice of algorithm, one of 
%             'imv' : Improved minimal volume algorithm (Fogel-Huang-Belforte-Bona))
%             'imtr': Improved minimal trace algorithm  (Fogel-Huang)
%             'mv1' : Minimal variance algorithm, type 1 (Dasgupta-Huang)
%             'mv2' : Minimal variance algorithm, type 2 (Canudas de Wit-Arruda-Favier)
%             'dz'  : Least squares with dead-zone  (Lozano-Ortega)
%             'wrls': Weighted recursive least squares  
%   a       - forgetting factor (inflation factor with the first five methods 
%             and weight with 'wrls') 
%   E       - resulting ellipsoid
%   mlam    - informativity of new data (a number between 0 and 1)
%   outlier - indices of outlier occurencies (where no intersection occurs)
%   acc     - accumulated weights for the weighted recrusive least squares
%             (see GBT manual for definitions, and example TRACK.M for usage)
%
%  Check at "http://sun1.bham.ac.uk/veressm/gbt.html " for new routines.
%  
%  GBT, S M Veres & S Hermsmeyer, 1993

% Extracting and checking data 
 if (nargin<2),disp(' "ellibnd.m" needs initial ellipsoid ');break;end;
 [er,ec]=size(E0);
 if (E0(1,1)~=1 | (er~=ec)), 
 disp(' Wrong format of initial ellipsoid in "ellibnd".');break; end;
 if (nargin<3), method='imv'; ,end;
 if (nargin<4), a=1;end;
 if strcmp(method,'wrls')&(nargin<5),
 disp('Weight accumulator "acc" must be used in WRLS.'); end;
 if ((a<=0)|(a>1)),
   disp('Forgetting factor must be  (0,1].');break;
 end;

% Specifying initial qauntities
 P=E0(2:er,1:ec-1);P0=P;
 sig2=norm(P);
 P=P/sig2;
 theta=E0(2:er,ec);
 dim=er-1;
 [rr,rc]=size(R);
 ol=0;
 if ~(rc-2==dim),
 disp('Incompatible input dimensions in "ellibnd".');break;end;
 if strcmp(method,'wrls')&(rr>1), 
 disp('Use WRLS and give "a" for each data, and initialise "acc" . '); 
 end;
method1=strcmp(method,'imv'); 
method2=strcmp(method,'imtr');
method3=strcmp(method,'mv1');
method4=strcmp(method,'mv2');
method5=strcmp(method,'dz');
method6=strcmp(method,'wrls');
method7=strcmp(method,'rls');
meth12=method1 | method2;

% Main loop over all rows of  record R
 for k=1:rr,

% Variables belonging to measurement k
     
     if (dim==1),
     if (R(k,1)>0),mi=R(k,2)/R(k,1);ma=R(k,3)/R(k,1);end   
     if (R(k,1)<0),ma=R(k,2)/R(k,1);mi=R(k,3)/R(k,1);end;
     if (R(k,1)==0),ma=1.0E+24;mi=-1.0E+24;end;
     sq0=sqrt(P0(1,1));
     th1=max(theta-sq0,mi);
     th2=min(theta+sq0,ma);
     P(1,1)=0.25*(th2-th1)*(th2-th1);theta=0.5*(th1+th2);
     E=[1,0;P,theta];
     return;
     end;
     
     empty=0;update=0;
     phi=R(k,1:dim)';
     g0=phi'*P*phi;
     g=sig2*g0;
     sqrt_g=sqrt(g);
     dl=0.5*(R(k,dim+2)-R(k,dim+1));
     dl2=dl*dl;
     ek=0.5*(R(k,dim+1)+R(k,dim+2))-phi'*theta;
     
     if (meth12), 
% Check if hyperplanes cut the current ellipsoid (improvement)
if (abs(ek)+dl>sqrt_g),dl=0.5*(dl-abs(ek)+sqrt_g);ek=sign(ek)*(sqrt_g-dl);end;
dl2=dl*dl;
     end;
     
    if (method1),

% Initialisating  "alpha, beta" for the "imv" algorithm

	hlp1=(dim-1)*g*g;
	hlp2=((2*dim-1)*dl2-g+ek*ek)*g;
	hlp3=(dim*(dl2-ek*ek)-g)*dl2;
	if (hlp3<0),
	     lam=max(roots([hlp1,hlp2,hlp3]));update=1;
	else lam=0;
	end;
	
	alpha=1/sig2;
	beta=lam/dl2;

     elseif (method2),

% Initialising "alpha, beta" for the "imtr" algorithm  
	
	mut=sum(diag(sig2*P));
	gam=phi'*P*P*phi;
	hlp1=3*dl2/g;
	hlp2=dl2*g0*(mut*(dl2-ek*ek)-sig2*sig2*gam)+2*dl2*(dl2*g0*mut-sig2*gam*(dl2-ek*ek));
	hlp5=g*g*(g0*mut-sig2*gam);
	hlp2=hlp2/hlp5;
	hlp3=dl2*dl2*((dl2-ek*ek)*mut-sig2*sig2*gam)/hlp5/sig2;

	if (hlp3>=0), lam=0;
	else 
	    hlp4=roots([1,hlp1,hlp2,hlp3]);
	    lam=max(hlp4(imag(hlp4)==zeros(3,1))); 
	    update=1;
	    if ((lam==[]) | (lam<0)), disp('MTR error.'); ,break; end;
	end;

	alpha=1/sig2;
	beta=lam/dl2;
	
% Initialising "alpha,beta" for the "mv1" algorithm    

     elseif (method3),

if (a>1.0-1.0e-6), 
disp(' Design parameter must be  a  (0,1)  .');
break;
end;
	if (ek*ek<dl2-sig2), lam=0;
	else
	    zeta=(dl2-sig2)/(ek*ek);update=1;
	    if (ek*ek==0), xi=a; ,end;
	    if (g0==1), xi=0.5*(1-zeta); ,end;
	    if (zeta*(g0-1)+1<=0), xi=a; ,end;
	    if (zeta*(g0-1)+1>0), xi=(1-sqrt(g0/(zeta*(g0-1)+1)))/(1-g0); ,end;
	    lam=min(xi,a);
	end;
	
	alpha=1-lam;
	beta=lam;

% Initialising "alpha,beta" for the "mv2" algorithm       

     elseif (method4),
	alpha=a;update=1;
	if ((abs(ek)>dl) & (g~=0)), beta=alpha/g0*(abs(ek)/dl-1);
	else beta=0;
	end;

     elseif(method5), 

% Initialising "alpha,beta" for the "dz" algorithm         
	alpha=1;
	if (ek*ek>dl2*(1+a)),  
	    beta=a*(1-sqrt(1+a)*dl/abs(ek))/(1+g0);update=1;
	else beta=0;
	end;
     
     elseif(method6),

% Initialising "alpha,beta" for the  "wrls" algorithms        
     
     alpha=a;beta=1;update=1;
     else
	disp('Incorrect method specified.');
     end;
     
% Standard recursive ellipsoidal bounding algorithm

     if ~(empty), if (update),
	
	P = (P - beta*P*phi*phi'*P/(alpha + beta*g0))/alpha;
	
	theta = theta + beta*P*phi*ek;

	sig2 = alpha*sig2 + beta*dl2 - alpha*beta*ek*ek/(alpha + beta*g0);
			
		  end;
     else
	ol=ol+1;
	outlier(ol)=k-1;
     end;
 
 mlam=beta/(alpha+beta);  
 if (meth12), P=P/a;end;

% end of record cycle 
 end;

% Bringing the results to GBT ellipsoidal format
  
  E=[1,zeros(1,dim);P*sig2,theta];
 
 return
 end
