function H = hadp (N)

%   HADP(N) is a Hadamard matrix of order N
%
%   HADP generates Hadamard matrices for some orders not supported
%   by the function HADAMARD.
%
%   If N is divisible by 4 and N-1 is prime, the Hadamard matrix is
%   constructed using the method from Paley's 2nd lemma.  If N is divisible
%   by 4 and N/2-1 is prime, the Hadamard matrix is constructed using the
%   method from Paley's 3rd lemma.  Many more orders can be constructed
%   using the Kronecker product property, which HADAMARD does to a limited
%   extent.
%   
%   HADP calls ISPRIME, which is not part of the main Matlab distribution.
%   I found it at ftp://ftp.mathworks.com/pub/contrib/math/isprime.m
%   This function may come with MATLAB as of version 5.0 (I think I saw
%   it in the beta version).
%
%   Frederick Wheeler (wheeler@ipl.rpi.edu)
%   July 30, 1996
%   

%   References:
%
%     R.E.A.C Paley, On Orthogonal Matrices, Journal of Mathematics and
%     Physics, Vol. 12, 1933.
%
%     Thomas Beth, Dieter Jungnickel and Hanfried Lenz, Design Theory, 
%     Cambridge University Press, Cambridge, 1986, Pp. 70-72.

if N>0 & rem(N,4)==0 & isprime(N-1)

  % one less than N must be prime
  Q = N - 1;

  % these are the squares of the elements of the Galois Field GF(Q)
  % the numbers are squared, as usual, but then modulo divided by Q
  I = rem ((1:(Q-1)).^2, Q);

  % the so called "quadratic character" is all -1's ...
  X = - ones (1, Q-1);

  % ... except where the index is a perfect square in GF(Q)
  for i = I
    X(i) = 1;
  end;

  % Paley's construction from his Lemma 2.
  H = [ones(1,N); ones(N-1,1) toeplitz([-1 X], [-1 X((Q-1):-1:1)])];

elseif N>0 & rem(N,4)==0 & isprime(N/2-1) & rem(N/2-1,4)==1

  % one less than N/2 must be prime
  Q = N/2 - 1;

  % these are the squares of the elements of the Galois Field GF(Q)
  % the numbers are squared, as usual, but then modulo divided by Q
  I = rem ((1:(Q-1)).^2, Q);

  % the so called "quadratic character" is all -1's ...
  X = - ones (1, Q-1);

  % ... except where the index is a perfect square in GF(Q)
  for i = I
    X(i) = 1;
  end;

  % Paley's intermediate construction from his Lemma 3.
  B = [0 ones(1,Q); ones(Q,1) toeplitz([0 X], [0 X((Q-1):-1:1)])];

  H = kron(B == +1, [+1 +1 ; +1 -1]) ...
    + kron(B == -1, [-1 -1 ; -1 +1]) ...
    + kron(B ==  0, [+1 -1 ; -1 -1]);

else
  error(sprintf('matrix size %d is not supported', N));
end;

return;
