function [x,f1,noi] = invs(f,xmin,xmax,xlo,xhi,ft,nsig,lim)
% INVERS performs an iteration to determine the value of the 
%        argument X of a function F(X) which will, within a 
%        specified accuracy, cause F(X) to equal FT, a specified
%        value of the function.
%
%        It is required that F be a monotonic function of X within
%        the permissible range of variation of X, which is from XMIN
%        to XMAX, and that the value FT exists within this range.
%
%        NOTE  If the slope dF/dX becomes nearly zero in some part 
%              of the range XMIN to XMAX, and if this region is
%              contained between XLO and XHI, the iteration will
%              converge very slowly, or conceivably not at all.    
% 
% 	 INPUT:
%        f           - String containing the name of the function F(X).    
%        xmin, xmax  - Permissible range of variation of X.
%	 xlo, xhi    - Region in which it is guessed that the desired 
%                      value of X will be found. If no knowledge exists 
%                      by which to estimate this region, XLO and XHI can 
%                      be set equal to XMIN and XMAX; however, the more
%                      narrowly the region is defined, the fewer will be
%                      the iterations required.
%	 ft          - Specified value of the function F(X).
%	 nsig	     - Number of significant figures to which agreement
%		       is desired between F(X) and FT. (default is 16)
%	 lim	     - Limit on the number of iterations permitted.
%		       (default is 25)	
%	 
%        OUTPUT:
%	 x           - Value that result in F(X) = FT.
%	 f1          - Actual final value of F.
%        noi         - Number of iterations actually performed.
%
%	 
%        Dominic Girard  
%        girard@megantic.net
%        3-26-96 
%	 Original fortran subroutine by L.V. Blake, NRL., 1972.
%


if nargin < 8, lim = 25; end
if nargin < 7, nsig = 16; end

test = 10 ^ (-nsig);
fd = ft;
if ft == 0, fd = 1; end
noi = 1;
delta = xhi - xlo;
x1 = xlo;
x2 = xhi;
f1 = feval(f,x1);
f2 = feval(f,x2);
slope = (f2 - f1) / (x2 - x1);
if slope == 0
  fmax = feval(f,xmax);
  fmin = feval(f,xmin);
  slope = (fmax - fmin) / (xmax - xmin);
end

while inf    
  if ((f2 - ft) * slope) < 0
    x1 = x2;
    f1 = f2;
    x2 = x2 + delta;
    if x2 > xmax, x2 = xmax; end
    f2 = feval(f,x2);
  else
    break;
  end
end 
 
while inf
  if ((ft - f1) * slope) < 0
    x2 = x1;
    x1 = x1 - delta;
    if x1 < xmin, x1 = xmin; end
    f2 = f1;
    f1 = feval(f,x1);
  else
    break;
  end
end

xa = x1;
xb = x2;
fa = f1;
fb = f2;
if abs(f2 - ft) < abs(f1 - ft)
  f22 = f2;
  f2 = f1;
  f1 = f22;
  x22 = x2;
  x2 = x1;
  x1 = x22;
end

while inf
  if noi >= lim
    disp('MESSAGE FROM FUNCTION INVERS');
    s = sprintf('FUNCTION INVERSION NOT ACCOMPLISHED WITHIN SPECIFIED %i ITERATIONS.',lim); 
    disp(s);
    return;
  elseif f1 == f2
    if f1 == fb
      xb = x1;
      x2 = xb;
    else
      xa = x1;
      x2 = xa;
    end
    x = (xa + xb) * .5;
  else
    x = (x1 - x2) * (ft - f2) / (f1 - f2) + x2;
    if x < xa, x = xa; end
    if x > xb, x = xb; end
    noi = noi + 1;
    f2 = f1;
    x2 = x1;
  end    
  f1 = feval(f,x);
  x1 = x;
  test1 = abs((f1 - ft) / fd);
  if test1 <= test, return; end
end 

