%IKINE	Inverse manipulator kinematics
%
%	Q = IKINE(DH, T)
%	Q = IKINE(DH, T, Q)
%
%	Returns the joint coordinates corresponding to the
%	end-effector transform T.  Note that the inverse kinematic solution is
%	generally not unique, and depends on the initial guess Q (which 
%	defaults to 0).
%
%	Q = IKINE(DH, TG)
%	Q = IKINE(DH, TG, Q)
%
%	Returns the joint coordinates corresponding to
%	each of the transforms in flattened form which are the rows of TG.
%	Returns one row of Q for each input transform.  The initial estimate 
%	of Q for each time step is taken as the solution from the previous 
%	time step.
%
%	Solution is computed iteratively using the pseudo-inverse of the
%	manipulator Jacobian.
%
%	Such a solution is completely general, though much less efficient 
%	than specific inverse kinematic solutions derived symbolically.
%	
%	This approach allows a solution to obtained at a singularity, but 
%	the joint angles within the null space are arbitrarily assigned.
%
%	See also FKINE, JACOBIAN.
	
%	Copyright (C) 1993 Peter Corke

function qt = ikine(dh, tr, q)
	n = numrows(dh);

	if nargin == 2,
		q = zeros(n, 1);
	else
		q = q(:);
	end

	if ishomog(tr),
		nm = 1;
		while nm > 1e-12,
			e = tr2diff(tr - fkine(dh, q'));
			dq = pinv( jacob0(dh, q) ) * e;
			q = q + dq;
			nm = norm(dq);
		end
		qt = q';
	else
		for i=1:numrows(tr),
			nm = 1;
			T = reshape(tr(i,:), 4,4);
			while nm > 1e-12,
				e = tr2diff( T - fkine(dh, q'));
				dq = pinv( jacob0(dh, q) ) * e;
				q = q + dq;
				nm = norm(dq);
			end
			qt = [qt; q'];

		end
	end
