    function isp = rabin(n,a)
% test primality of odd integers from 3 to sqrt(maxint)
% isp = rabin(n,a) : per Reference, n is prime if:
%                     n < sqrt(maxint) = ca. 94 million.
%                     maxint < 2^60    
%                     rabin(n,2) = rabin(n,3) = rabin(n,5) = 0
%                     and n ~= 25,326,001
% isp : scalar 0/1 variable - 0 if n is proven composite,
%                             1 is n is probably prime.
%   n : scalar odd integer, 3 < n < sqrt(maxint), to test
%        (note: on the Mac, maxint = 2^53, sqrt(
%   a : positive integer < n, relatively prime to n
%       (suggest using prime a)
% (maxint = maximum representable integer.)
% 
% Reference: Pomerance, C., "The Search for Prime Numbers",
% Scientific AMerican, Dec. 1982.
% R.A. Fowell, 6/29/93, fowell@netcom.com

% Theory: "an odd composite number n (with n-1 = d*2^s, d odd)
%          either is a strong pseudoprime to base a (spsp(a)) if
%          (i)  a^d       ==  1 (mod n), or
%          (ii) a^(d*2^r) == -1 (mod n), for some r in 0 <= r < s.
%          Note that if n is prime, then either (i) or (ii)
%          must hold ..." - Ref., p. 1004.

% Step 1 - factor n-1 = d*2^s
 s = 0;
 d = n - 1;
 while (rem(d,2) == 0),
   s = s+1;
   d = d/2;
 end

% Step 2 - compute a^d mod n

 r  = 1 ;  % result
 ad = a ;
 rd = d ;  % remaining exponent to compute

% examine the exponent (d) as a binary number. For each bit:
%    if bit is high, multiply current power of "a" into result.
%    in any case, shift exponent one bit, square "a"

while rd >= 1,

   if rem(rd,2) > 0, r = rem(r*ad,n); end

   rd = floor(rd/2);  
   ad = rem(ad^2,n); 

end

pass = n-1;
isp  = 0  ; % n is composite unless one of following is true:

if (r == 1 | r == pass),
      isp = 1;
else
   while s > 1,
      s = s - 1;
      r = rem(r^2, n);
      if r == pass,
           isp = 1;
           break;
      end
   end
end

