function isp = isprime(in);
%function isp = isprime(in);
%
% determine if elements of matrix 'in' are prime.
%
% A prime number is any integer n, n>1, that can not be expressed in the form
% n = a*b, where a & b are both positive integers such that a,b<n. 
% Therefore any input that is not an integer > 1 is automatically not prime
% (by definition).
%
% Any other input (i.e. integers >= 2) can be tested using a method based on
% the sieve of erastosthenes. 
% Any composite integer can be factorised into two smaller integers: 
% One >= the square root, the other <= the square root.
% Furthermore, any composite integer can be decomposed into factors that are
% all prime.
% Therefore, each prime number less than the square root of a positive integer
% must be checked to see if it is a factor of the integer. If none of these
% can factorise the integer, then it is prime.
% The sieve of erastosthenes is used to generate the prime numbers needed for
% the test. All primes less than the square root of the maximum input value
% are generated. As each prime is generated, any multiples are removed from
% the list of input values. When the process is over, only input values which
% are prime will remain.
%
% Author S.J. Searle, 29/6/93
% sjs@mod.dsto.gov.au

% construct return matrix
isp = zeros(size(in));

% create vector of indices to input values,
% weeding out input values that are not integers > 1
idx = find(~((in~=round(in))|(in<2)));

% Compute maximum possible factor and hence all possible factors
maxin = max(in(idx));
maxsqrt = sqrt(maxin);
factors = 2:floor(maxsqrt);

% For each prime factor, remove indices corresponding to values that are
% multiples of the factor, and remove any multiples of the factor from the
% list of possible prime factors.
while ~(isempty(factors)|isempty(idx))
  idx = idx(find( rem(in(idx),factors(1))|(in(idx)==factors(1)) ));
  factors = factors(find(rem(factors,factors(1))));
end

% fill return matrix
isp(idx) = ones(size(idx));

return

