function B = lexsort(A)
% lexsort(A) -- sorts rows of A in lexical order ignoring leading spaces
%
% (Submission to Drea's Desk M-file contest V2.0   2/94)
%
% Problem: Lexical sorting. You have a string matrix possibly padded with
% spaces. The challenge is to sort the rows of the string matrix in
% dictionary order (ignoring leading spaces). Ex:
%
%    A = ['hello'            lexsort(A) =>  [' bar '
%         'foo  '                            'foo  '
%         ' bar '];                          'hello'];
%
% Daniel Scharstein 2/8/94  (schar@cs.cornell.edu)
%
%
% Idea: interpret each row of A as a (normalized) number radix R, where R is
% the range of possible symbols:
%
%           R = 'z' - ' '
%
% We can compute a vector V containing these numbers by matrix-multiplying
% (A - ' ') with column vector RR = [1/R ; 1/R^2 ; 1/R^3 ; ....].  We can
% use cumprod to generate this vector: 
%
%           RR = cumprod(ones(size(A, 2), 1)/R)
%           V = (A - ' ') * RR
%
% If we just want to sort the rows of A, the following does the job:
%
%           [r, i] = sort(V); B = A(i, :)
%
% If we want to ignore leading spaces, we have to (element-wise) multiply V
% with R.^K, where K is a column vector containing the number of leading
% spaces in each row.  Here is a trick to compute K:
%
%           K = sum(cumprod(A' == ' '))'
%
% So our final step is
%
%           [r, i] = sort(V .* R.^K); B = A(i, :)
%
% The same without intermediate variables:
 
[r, i] = sort((A - ' ') * cumprod(ones(size(A, 2), 1)/('z' - ' ')) ...
              .* (('z' - ' ') .^ sum(cumprod(A' == ' '))' ));
B = A(i, :);
