function strout = lexsort(str)
%LEXSORT Lexical sorting of string arrays.
% LEXSORT(STR) sorts the rows of STR in lexical order ignoring leading
% spaces.
 
% written by
%   Douglas M. Schwarz
%   Eastman Kodak Company
%   schwarz@kodak.com
%   8 February 1994
 
% Algorithm approach is to compute a new array with fewer columns
% that will have the sort order we desire and then apply that same
% sort order to the input.
 
% Get size of input and initialize output.
[m,n] = size(str);
strout = str;
 
% Remove leading spaces by rotating rows to the left one character
% for each leading space.
if n > 1
  numLeadingSpaces = sum(cumprod(str' == ' '));
  rotationMat = hankel(2:n, [n,1:(n-1)]);
  for i = find(numLeadingSpaces)
    strout(i,:) = str(i,rotationMat(numLeadingSpaces(i),:));
  end
end
 
% Change lower case letters to upper case.
lookup1 = 1:255;
lookup1('a':'z') = 'A':'Z';
strout(:) = lookup1(strout);
 
% Re-map strout to get smallest numbers (strout = remap(strout)).
lookup2(strout) = ones(m,n);
maxInput = length(lookup2(lookup2));
lookup2(lookup2) = 1:maxInput;
strout(:) = lookup2(strout);
 
% Pack "charsPerDouble" characters into each 8-byte double.  The
% resulting matrix will have many fewer columns but still have the
% sort order we desire.
bitsPerChar = ceil(log2(maxInput));
charsPerDouble = floor(-log2(eps)/bitsPerChar); % -log2(eps) = bits/double
numDoubles = ceil(n/charsPerDouble);
charsInLastCol = rem(n-1, charsPerDouble) + 1;
r = pow2(-bitsPerChar*(1:charsPerDouble))';
x = zeros(m,numDoubles);
z = zeros(m,1);
for i = 1:(numDoubles-1)
  j = charsPerDouble*i;
  x(:,i) = pow2(strout(:,(j-charsPerDouble+1):j)*r, z);
end
if charsInLastCol == 1
  x(:,numDoubles) = strout(:,n);
else
  x(:,numDoubles) = pow2(strout(:,(n-charsInLastCol+1):n) * ...
      r(1:charsInLastCol), z);
end
 
% Sort matrix x one column at a time starting with the last column.
% The result will have the sort order we desire.
index = 1:m;
for j = numDoubles:-1:1
  [xs,i] = sort(x(:,j));
  x = x(i,:);
  index = index(i);
end
 
% Apply this sort order to str.
strout = str(index,:);
