function r = isrect(x, y)
%ISRECT True if polygon encloses a rectangular shape.
%       ISRECT(X, Y) returns 1 if X and Y are the x- and y-coordinates
%       of the vertices of a polygon that encloses a rectangular
%       shape.  Otherwise it returns 0.  X and Y must be vectors that
%       have the same length.  The input polygon must be closed; in
%       other words, X(1)==X(length(X)) and Y(1)==Y(length(Y)).  
%
%    Jim Bowlin                   Norman Vine
%    POB 532           - and -    POB 319
%    Woods Hole MA                Woods Hole MA
%            02543                        02453
%

%---------- set default return value to FALSE ----------%

   r = 0;

%---------- check for equal length to avoid error ----------%

%LENGTH%   sizex = size(x);
%LENGTH%   if length(find( size(y) ~=  sizex)) > 0
%LENGTH%     %fprintf('NO x and y must have same length.\n');
%LENGTH%     return;
%LENGTH%   end;

%LENGTH%   if min(sizex) ~= 1
%LENGTH%      %fprintf('NO x and y must be vectors.\n');
%LENGTH%      return;
%LENGTH%   end;

%---------- put vertices in Complex plane ----------%

   z =  x + i * y;

%---------- test for vertex closure ----------%

%CLOSED%   if z(length(z)) ~= z(1)
%CLOSED%      %fprintf('NO z(len(z)) ~= z(1)\n');
%CLOSED%      return;
%CLOSED%   end;

%---------- remove repeated vertices ----------%
%
%  prevents dz == 0, which would cause us to miss winding
%  number of a corner.

%---------- this test allows us to use dz[vdup] = [] ----------%
 
   if length(z) < 3
      %fprintf('YES a point or line is a rectangle.\n');
      r = 1;
      return;
   end;

   dz = diff(z);
   vdup = find(dz == 0);
   z(vdup) = [];
   dz(vdup) = [];

%---------- check for single point (to avoid error)  ----------%

   lenz = length(z);
   if (lenz < 3) 
      r = 1; 
      %fprintf('YES a point or a line is a rectangle.\n');
      return; 
   end;

%---------- compute scale factor and save min/max for later use ----------%

   maxis = [ -min(x) max(x) -min(y) max(y)];
   scale = max(abs(maxis));
   fuzz  = 3 * eps * scale;
   fuzz2 = fuzz * scale;

%---------- Get all corner vertices ----------%
%
%  ddz will contain the dot product and cross product
%  of the dz vectors.
%  real(ddz) = r1 * r2 + i1 * i2 = dz1 dot   dz2
%  imag(ddz) = r1 * i2 - r2 * i1 = dz1 cross dz2
%
%  real == 0  => right angle
%  imag == 0  => colinear
%  sign(imag) => clockwise vs. counter clockwise
%  

   ddz = dz(1:(lenz-2)) .* conj(dz(2:(lenz-1)));

%---------- corn_idx will point to all corners ----------%

   corn_idx = find(abs(imag(ddz)) > fuzz2);
   ddz = ddz(corn_idx);

   %%fprintf('%g ',max(abs(real(ddz)))/fuzz2);

%---------- Now check that all are right angles ----------%

   if max(abs(real(ddz))) > fuzz2
      %fprintf('NO not all corners are right angles.\n');
      return;
   end;

   corn     = z(corn_idx+1);
   ncorn    = length(corn);

%---------- no corners => line or point ----------%

   if ncorn < 3
      if ncorn == 0
         r = 1;
         %fprintf('YES no corners => line or point\n');
         return;
      end;
      %fprintf('NO not enough corners %d.\n',ncorn);
      return;
   end;

%---------- find ordered unique corners ----------%
%
%  remove repeated corners and then search odd and even
%  corners separately to find 4 unique ordered corners.
%  Once repeats are removed all corners that are separated
%  by an even index will be opposite.
%  We are assured that ucorn(1:3) march around the rectangle.
%
%  This is messy but pretty fast.

   if (ncorn == 3) 
      ucorn = corn;
   else

%---------- remove repeated corners ----------%

      vdup = find(diff(corn) == 0);
      corn(vdup) = [];
      ncorn = length(corn);

      if ncorn < 4
         if ncorn < 3
            %fprintf('NO only %d unique corners.\n',ncorn);
            return;
         end;
         ucorn = corn;
      else

         ucorn(1:2) = corn(1:2);
         odd__corn = corn(3:2:ncorn);
         even_corn = corn(4:2:ncorn);
         odd__corn  = odd__corn(find(odd__corn ~= ucorn(1)));
         even_corn = even_corn(find(even_corn ~= ucorn(2)));
         if length(odd__corn) > 0
            ucorn(3) = odd__corn(1);
            if length(find(odd__corn ~= ucorn(3))) > 0
               %fprintf('NO too many odd corners.\n');
               return;
            end;
            if length(even_corn) > 0
               ucorn(4) = even_corn(1);
               if length(find(even_corn ~= ucorn(4))) > 0
                  %fprintf('NO too many even corners.\n');
                  return;
               end;
            end;
         else
            if length(even_corn) < 1
               %fprintf('NO only 2 unique corners.\n');
               return;
            end;
            if length(find(even_corn ~= even_corn(1))) > 0
               %fprintf('NO too many even corners.\n');
               return;
            end;
            ucorn = [even_corn(1),ucorn];
         end;
      end;
   end;

   nucorn = length(ucorn);


%---------- ucorn4 is my guess of 4th corner given ucorn(1:3) ----------%

   ucorn4 = ucorn(1) - ucorn(2) + ucorn(3);


   if (nucorn == 3) 

%---------- calculate winding number ----------%
%
%  the winding number count how many corners we
%  go past the sign depends on cw or ccw direction.
%  z1 and z2 will contain all points need to close 
%  the rectangle.
%  
   wind     = zeros(1,lenz);
   wind(corn_idx+1) = sign(imag(ddz));
   wind     = cumsum(wind);
   maxwind = max(wind);
   minwind = min(wind);
   dwind    = maxwind - minwind;
      
   if dwind < 3
      %fprintf('NO bad winding number %d.\n',dwind);
      return;
   end;

%---------- find closest point to missing corner ----------%

      z1 = z(find( wind == maxwind));
      z2 = z(find( wind == minwind));

      [dum,vdup1] = min( abs(z1  - ucorn4));
      [dum,vdup2] = min( abs(z2  - ucorn4));
      ucorn(4) = z1(vdup1(1));
      if ( z2(vdup2) ~= ucorn(4))
         %fprintf('No 4th corner not closed.\n');
         return;
      end;

      if (abs( ucorn(4) - ucorn4 ) > fuzz)
         %fprintf('NO due to missing 4th corner\n');
         return;
      end;

%---------- check for outlier points ----------%

      ucmax = [ -min(real(ucorn)) max(real(ucorn)) -min(imag(ucorn)) max(imag(ucorn)) ];
      if ( length( find ( maxis > ucmax)) > 0)
         %fprintf('NO due to outliers\n');
         return;
      end;

      r = 1;
      %fprintf('YES closed 3 corner rectangle.\n');
      return;

   end; % of 3 corner case

%---------- This is 4 cornered rectangle case ----------%

   %disp(abs( ucorn(4) - ucorn4)/fuzz);
   if (abs( ucorn(4) - ucorn4) > fuzz)
      %fprintf('NO 4 corners don''t form rectangle.\n');
      return;
   end;

%---------- check for outlier points ----------%

      ucmax = [ -min(real(ucorn)) max(real(ucorn)) -min(imag(ucorn)) max(imag(ucorn)) ];
      if ( length( find ( maxis > ucmax)) > 0)

      %fprintf('NO due to outliers\n');
      return;
   end;


%---------- calculate winding number ----------%
%
%  the winding number count how many corners we
%  go past the sign depends on cw or ccw direction.
%  z1 and z2 will contain all points need to close 
%  the rectangle.
%  
   wind     = zeros(1,lenz);
   wind(corn_idx+1) = sign(imag(ddz));
   wind     = cumsum(wind);
   maxwind = max(wind);
   minwind = min(wind);
   dwind    = maxwind - minwind;
      
   if dwind < 3
      %fprintf('NO bad winding number %d.\n',dwind);
      return;
   end;

%---------- Now check winding number ----------%

   if ( dwind > 4)
      %fprintf('YES 4-corner with dwind = %d\n',dwind);
      r  = 1;
      return;
   end;

%---------- Final case: closure of 4 corner rectangle ----------%

   z1 = z(find(wind == maxwind));
   z2 = z(find(wind == minwind));

%---------- get a vector direction along 'missing' side ----------%
%
%  We are assured that z1 and z2 each have at least one point
%  and that at least one of them has at least two points.
%  All points in z1 and z2 should lie on a straight line.
%
%  We take a vector in the line (if it is zero then there
%  are two overlapping points and we are done) and dot
%  all our points with it to get a signed distance along the line.

   vec = conj(z2(1) - z1(1) );

   if (vec == 0)
      r = 1;
      %fprintf('YES closed 4th side with same point.\n');
      return;
   end

   s1  = real( z1 .* vec);
   s2  = real( z2 .* vec);

   r = xor( min(s1) < max(s2),max(s1) < min(s2));

%P%   if (r)
%P%      %fprintf('YES closed 4 corner rectangle.\n');
%P%   else
%P%      %fprintf('NO one open side on  4 corner rectangle.\n');
%P%   end;

