function result = isrect( x, y)
%isrect( x, y) returns 1 if plot(x,y) is a rectangle, 0 otherwise
% -Assumed: multiplication is much more expensive than addition.
%   Otherwise it would be faster to map the container polygon with similarity
%   and then with affinity to the square whose corners have coordinates
%   1-tol and 3+tol, (tol = tolerance, e.g. 8*eps).
%   To qualify points we could simply use fix().
% -This version goes for fast performance on large number of points.
% -The requirement for the polygon to be closed is not important, for
%   xx=[x,fliplr(x)];  yy=[y,fliplr(y)]  yield the same container polygon
%   and it is closed.
% -The error checks are commented out for speed.
%
% Written for MATLAB 4.2c for MSWINDOWS,     Jan.20.1995  by  L. Hars

tic

%% if nargin ~= 2, error('Usage: result = isrect( x, y)'), end
%% if nargout > 1, error('Only one output is returned'), end

n  = length(x);

%% if n < 1  |  n ~= length(y)
%%    error('Coordinates must have >1, equal length')
%% end
%% if x(1) ~= x(n)  |  y(1) ~= y(n)
%%    error('Polygon must be closed')
%% end

if n < 4,  result = 1; return,  end          % 2-edge closed polygon

[f,ptol] = log2(pow2( eps, 4));              % 16*eps, CHANGE IF YOU WISH
tol = pow2( max(abs(x))+max(abs(y)), ptol);  % 32 * norm * eps tolerance

% ---   Find direction of the first edge   --- 
% We hope for not many repeated points at the beginning: no vectorisation
% We go for speed, not for stability. It is numerically more stable
% to find the longer edge of the container rectangle, but it needs looking
% at all the points.

k = 2;
while k <= n  &  abs( x(1) - x(k)) < tol  &  abs( y(1) - y(k)) < tol
   k = k + 1;
end
if k > n,  result = 1; return,  end          % 1 repeated point = rectangle

dx = x(k) - x(1);    dy = y(k) - y(1);

% ---  Rotate points to get a horizontal or vertical side  ---
% Can magnify. Scaling the rotation matrix saves half of the multiplications:

if abs(dx) > tol  &  abs(dy) > tol
   a = dy / dx;
   tmp = a * x;
   x = x + a*y;
   y = tmp - y;
   tol= pow2(max(abs(x))+max(abs(y)),ptol);  % NEW: 32*norm*eps tolerance
end

toc
tic

% ---  Find container edges  ---

minx = min(x) + tol; 
maxx = max(x) - tol;
miny = min(y) + tol;
maxy = max(y) - tol;

if maxx-minx < tol  |  maxy-miny < tol
   result = 1;  return              % 1 line rectangle OK
end

% ---  Test if all edges are on container sides  ---

l = ( x < minx);  r = ( x > maxx);  % vtcs on Left, Right sides
b = ( y < miny);  t = ( y > maxy);  % vtcs on Bottom, Top sides

n1 = 1 : n-1;     n2 = 2 : n;

if ~all( (l(n1) & l(n2)) | (r(n1) & r(n2))... % All vtcs must be on sides,
       | (t(n1) & t(n2)) | (b(n1) & b(n2)) )  % neighbors have unchanged side
   result = 0; return
end

% ---  Test if container sides are covered  ---

c  = ( l + t + r + b) > 1;          % Corners = vtcs on more sides
fc = find(c);                       % indx of corners

blu = miny;  blr = minx;            % Extrems alredy reached
tld = maxy;  tlr = minx;            % from  Bottom, Left, Top, Right
trd = maxy;  trl = maxx;            % going to Up, Down, Left, Right
bru = miny;  brl = maxx;

fck1  = fc(1);
for k = 2 : length(fc)
    fck  = fck1;                    % current corner
    fck1 = fc(k);                   % next corner
    if b(fck)
       if l(fck)
          if l(fck+1), blu = max( blu, max( y( fck : fck1)));
          else         blr = max( blr, max( x( fck : fck1)));  end
        else % r
          if r(fck+1), bru = max( bru, max( y( fck : fck1)));
          else         brl = min( brl, min( x( fck : fck1)));  end
        end
    else % t
       if l(fck)
          if l(fck+1), tld = min( tld, min( y( fck : fck1)));
          else         tlr = max( tlr, max( x( fck : fck1)));  end
        else % r
          if r(fck+1), trd = min( trd, min( y( fck : fck1)));
          else         trl = min( trl, min( x( fck : fck1)));  end
        end
    end
end;

if ~c(1)               % polygon starts/ends at non-corner vertex
    if b(fck1)         % last corner on the polygon is fck1
       if l(fck1)
          if l(fck1+1),blu = max( blu, max( y([fck1:n,2:fc(1)])));
          else         blr = max( blr, max( x([fck1:n,2:fc(1)])));  end
        else % r
          if r(fck1+1),bru = max( bru, max( y([fck1:n,2:fc(1)])));
          else         brl = min( brl, min( x([fck1:n,2:fc(1)])));  end
        end
    else % t
       if l(fck1)
          if l(fck1+1),tld = min( tld, min( y([fck1:n,2:fc(1)])));
          else         tlr = max( tlr, max( x([fck1:n,2:fc(1)])));  end
        else % r
          if r(fck1+1),trd = min( trd, min( y([fck1:n,2:fc(1)])));
          else         trl = min( trl, min( x([fck1:n,2:fc(1)])));  end
        end
    end
end

if  tld-blu > tol  |  trd-bru > tol  |  brl-blr > tol  |  trl-tlr > tol
    result = 0;  return
end                    % a side is not covered


result = 1;            % --- EVERYTHING IS OK ---

toc
