function r = isrect(x, y)
%ISRECT True if polygon encloses a rectangular shape.
%	ISRECT(X, Y) returns 1 if X and Y are the x- and y-coordinates
%	of the vertices of a polygon that encloses a rectangular
%	shape.	Otherwise it returns 0.  X and Y must be vectors that
%	have the same length.  The input polygon must be closed; in
%	other words, X(1)==X(length(X)) and Y(1)==Y(length(Y)).
%
%							Jan 22, 1995
%
%	        T. Hristov
%		Dept. of Mechanical Engineering
%		University of California at Irvine
%		Irvine, CA 92717-3975
%		Tel. (714)-824-7437
%		tihomir@cafws2.eng.uci.edu 		 

precision = 16 * eps;

dx=diff(x) ;
dy=diff(y) ;

% Remove the duplicated vertices
x = x([1 dx|dy]);
y = y([1 dx|dy]);

% A case of degenerate rectangle: a single point or a straight line
if (max(size(x)) <= 2)
	r = 1;
	return;
end

dx=diff(x);
dy=diff(y);
np = max(size(x));

Dnorms=sqrt(dx .* dx + dy .* dy);
cosines = (dx(2:np-1) .* dx(1:np-2) + dy(2:np-1) .* dy(1:np-2))  ...
		   ./ Dnorms(2:np-1) ./  Dnorms(1:np-2);

tmp = (dx(1) .* dx(np-1) +  dy(1) .* dy(np-1)) ./ Dnorms(np-1) ./  Dnorms(1) ;
cosines = [tmp cosines tmp];

% Let's fix the precision problems
c1=(abs(abs(cosines) - 1) <  precision) ;
c0= (abs(cosines) < precision) ;
cosines = c1 .* sign(cosines) + 0 * c0 + cosines .* (~c1 & ~c0);

if (abs(prod(cosines)) == 1)
	% All the cosines are +1 or -1
	% disp('A case of degenerate rectangle -- straight line')
	r = 1;
	return;
end

if (any( (abs(cosines) ~= 0) & (abs(cosines)  ~= 1)))
	% disp('Some of the angles are not right')
	r = 0;
	return;
end

% Translate the poligon so one of the vertices coincides with the origin
% it=min(find(cosines == 0));
x1=x-x(1);
y1=y-y(1);


% Rotate the polygon to make one of the sides to be colinear to the axes
imax=find(max(Dnorms) == Dnorms);
tmp = [dx(imax(1)) dy(imax(1)); -dy(imax(1)) dx(imax(1))] / Dnorms(imax(1)) * ...
       [x1 ; y1]  ;
x2 = tmp(1,:)  ;
y2 = tmp(2,:)  ;

x2 = x2 - min(x2);
y2 = y2 - min(y2);

x2 = x2 / max(x2);
y2 = y2 / max(y2);

% Let's fix the precision problems
   c1=(abs(abs(x2) - 1) <  precision) ;
   c0= (abs(x2) < precision) ;
   x2 = c1 + 0 * c0 + x2 .* (~c1 & ~c0);

   c1=(abs(abs(y2) - 1) <  precision) ;
   c0= (abs(y2) < precision) ;
   y2 = c1 + 0 * c0 + y2 .* (~c1 & ~c0);


% plot(x, y, x1, y1, x2,y2)

index = find(cosines == 0);
if ~all(((x2(index)==1) | (x2(index)==0)) & ((y2(index)==1) | (y2(index)==0)))
	% disp('Not a rectangle')
	r = 0;
	return;
end


% Now let's consider the case:
% X9 = [ 0   0 1 1 0 0	];  X9 = [X9 fliplr(X9)];
% Y9 = [ 0.8 1 1 0 0 0.2];  Y9 = [Y9 fliplr(Y9)];
% This polygon is closed X9(1)==X9(length(X9)) and Y9(1)==Y9(length(Y9)),
% but is not a rectangle. In this case we must check if all four sides
% of the rectangle are available.

for side=1:4  % For all four sides
    while(1) % because Matlab does not have "Continue" operator as C does

	if(side == 1) % x == 0, i.e the left side
		index = find(x2 == 0);
		z = y2(index);
	end

	if (side == 2) % y == 1, i.e the upper side
		index = find(y2 == 1);
		z = x2(index);
	end

	if (side == 3) % x == 1, i.e the right side
		index = find(x2 == 1);
		z = y2(index);
	end

	if (side == 4) % y == 0, i.e the lower side
		index = find(y2 == 0);
		z = x2(index);
	end


	if (max(size(index)) <= 1)
		% applies to the case
		% X = [0.1 1 1 .7];  Y=[ 0 0 1 1];
		% X = [X fliplr(X)];  Y = [Y fliplr(Y)];

		% disp('A single vertex or no vertex on that side')
		r = 0;
		return;
	end

	% Let's call a set of neighbouring verices on the current
	% side a segment. The indices of the vertices belonging
	% to the same segment differ by 1. If this difference is
	% bigger than 1 this indicates the start of a new segment.

	tmp = find(diff(index) > 1) ;
	BegSegment = [1 tmp+1];
	EndSegment = [tmp max(size(index))];

	% Redundant code for faster performance
	if  (max(size(BegSegment)) == 1) 
		% There is only one segment
		if ( (min(z) == 0) & (max(z) == 1))
			% disp('A single segment connecting the ends');
			break; % This side Ok, go to check the next side
			       % This break simulates the C "continue" operator
		else
			% disp('The only segment does not connect the ends');
			r = 0;
			return;
		end
	end

	MinSegment=zeros(size(BegSegment));
	MaxSegment=zeros(size(BegSegment));
	for nsegment=1:max(size(BegSegment))
		zsegment=z(BegSegment(nsegment):EndSegment(nsegment));
		MinSegment(nsegment)=min(zsegment);
		MaxSegment(nsegment)=max(zsegment);
	end

	% Redundant code for faster performance
	if any((MinSegment == 0) & (MaxSegment==1))
		% disp('There is a segment connectinig both ends')
		break; % while(1)
		       % This break simulates the C "continue" operator	
	end

	if (all(MinSegment ~= 0) | all(MaxSegment ~= 1))
		% disp('No segments starting from 0 or 1');
		r = 0;
		return;
	end

	if(max(MaxSegment(MinSegment == 0)) < min(MinSegment(MaxSegment == 1)))
		% disp('No overlapping segments');
		r = 0;
		return;
	end

        break; % unconditional break  
    end; % while(1)
end % for side=1:4

r = 1;  % All four sides are available 

end % isrect
