function [n,x,nbins]=histmulti(y,bins)
% For multidimensional histograms
%
% [n,x,nbins]=histmulti(y,bins)
%
% Every row in y represent a different data point and the columns
% are the different coordinates.
%
% bins: a rowvector => This is the number of bins in that coordinate.
% bins: a matrix => The limits for the different bins as produced in x.
%   Pad at end of the columns with NaN if different number of bins
%   in different coordinates (and unequal bins).
%
% The default is that bins=[10 ... 10];
%
% Result:
%  n - a sparse vector with the counts - use multilim for interpretation.
%  x - Corresponds to the bins-matrix (with NaN-padding if necessary).
%  nbins - The number of bins in the different coordinates.
%
%
% Written by Hans Olsson, hanso@dna.lth.se.
% Inspired by hist2 written by Kirill K. Pankratov, kirill@plume.mit.edu
% Which in turn was based on routines written by hanso@dna.lth.se

% Uses tobinh.

if (nargin<1)
  help histmulti
  return
end;
if (nargin<2)
  bins=10*ones(1,size(y,2));
end;
if (size(bins,2)~=size(y,2))
  error('Wrong number of bins in histmulti');
end;

if (size(bins,1)==1)
  nbins=bins;
  mincol=min(y);
  maxcol=max(y);
  binwidth=(maxcol-mincol)./bins;
  S=zeros(size(y,1),1);
  x=nan*zeros(max(bins),size(nbins,2));
  for col=1:size(bins,2)
    x(1:nbins(col),col)=(mincol(col)+(0:nbins(col)-1)*binwidth(col))';
    add=ceil((y(:,col)-mincol(col))/binwidth(col));
    add=add+(add==0);
    if (col==size(bins,2))
      S=S*bins(col)+add;
    else
      S=S*bins(col)+(add-1);
    end;
  end;
  % S is the bin that the items should be placed in.
else
  global binorder;
  x=bins;
  nbins=sum(~isnan(bins));
  S=zeros(size(y,1),1);
  I=1:size(y,1);
  for col=1:size(bins,2);
    binorder=nan*zeros(size(y,1),1);
    tobinh(y(:,col),I,x(:,col),1:nbins(col));
    if (col==size(bins,2))
      S=S*nbins(col)+binorder;
    else
      S=S*nbins(col)+(binorder-1);
    end;
  end;
  S=S(~isnan(S));
end;
S=sparse(S,1:length(S),1,prod(nbins),length(S));
n=sum(S');
