function F=steady(M)
% Steady(M) by C. Bailey, May 1996
%
% Steady() solves the set of differential equations defining a steady
% state enzyme system of N enzyme species.
%
% Input:  M = square matrix of differential coefficients
%             such as one generated by generate()
% Output: a symbolic column vector containing the steady state
%         concentrations of each enzyme species in the system
%         in terms of rate constants and reactant concentrations
%---------------------------------------------------------------------
% Assumptions:
%
% 1. Initial conditions are met, i.e. depletion of reactants is negligible
%
% 2. That the concentration of each enzyme species over the time scale
%    involved is constant.
%
% Mathematics:
%
% The system is described by N differential equations, where N is the number
% of enzyme species in the system. This can be represented by the equation:
% MxE = 0    where
% M is an NxN matrix of terms in rate constants and reactant concentrations
% E is a column vector of the concentrations of the enzyme species 1,2,...,N
% 
% Using Cramer's Rule, we can express the concentration of each species as:
% Ei=Di/D    where
% D is the determinant of the matrix M
% Di is the determinant of M with row and column i removed
%
% References:
%
% 1. Kistiakowsky, G. B. and Shaw, W. R. (1952) J. Biol. Chem, 75, 866-871
% 2. Hurst, R. O. (1967) Can. J. Biochem., 45, 2015-2039
% 3. Huang, C. Y. (1979) Met. Enz., 63, 54-84

% Initialise empty matrices to save time and redimensioning

N=symsize(M,1);
D=sym(zeros(N,1));
E=sym(zeros(N,1));
denom=sym('0');

% D = column vector containing the N Di

for i=1:N
 T=M;
 for j=1:N
  T=sym(T,j,i,'0');
  T=sym(T,i,j,'1');
 end
 D=sym(D,i,1,determ(T));		% Di = |T|
 denom=symadd(denom,determ(T));
end

F=simplify(symop(D,'/',denom));

% end of steady(M)

