function F=trans(N)
% Trans(N) by C. Bailey, May 1996
%
% trans() solves the set of differential equations which describe the
initial phase
% of an enzymatic reaction to give expressions for the concentration of
each enzyme
% species as a function of time, reactant concentrations and rate
constants. trans()
% can also be used to derive the equations governing molecular
interactions such as
% A + B <==> AB where one component is in great excess, such as is
monitored by SPR
% (surface plasmon resonance) using instruments such as a BIAcore.
%
% Input:  N = number of enzyme species in system
%         M = NxN matrix of terms input interactively
% M(i,j) = rate at which species 'j' is converted to species 'i'
% M(i,i) = rate at which species 'i' is converted to other species
%        = sum of k(j,i) where j=1,2,...,N
%
% Output: a symbolic column vector containing the expressions for each
enzyme species
%         in terms of time, reactant concentrations and rate constants
%-------------------------------------------------------------------------------------
%
% Assumptions:
%
% 1. Initial conditions are met, i.e. depletion of reactants is negligible
%    (without this assumption an analytical expression cannot be derived).
%
% 2. The initial conditions are that the concentration of the first species
%    is eo, and that of all the other species is 0
%
% Mathematics:
%
% The system is described by N differential equations, where N is the
number of enzyme
% species in the system. This can be represented by the equation:
% MxE = D    where
% M is an NxN matrix of terms in rate constants and reactant concentrations
% E is a column vector of the concentrations of the enzyme species 1,2,...,N
% D is a column vector of the derivatives of the concentration of the
enzyme species
%   with time.
%
% Applying a Laplace transform to this system creates a new equation:
% (M+zI)xE = C     where
% M is the same matrix
% z is the variable introduced by the transform
% I is the NxN identity matrix
% D is the same as before
% C is a column vector containing the initial concentrations of each species
%   = delta(1,i) x eo where delta(1,i) is 1 when i=1, 0 otherwise
% 
% Using Cramer's rule, the concentration of each enzyme as a function of time;
% Ei(z) = eo x Pi(z)/Q(z)
% where Q(z) is the determinant of M
%  and  Pi(z) is the determinant of M with row and column i removed
%
% Since Q(z) is a polynomial of degree N in z, with one root = 0, Heaviside's
% expansion formula can be used to express this equation as a series of
partial
% fractions, which on inversion yield the expression
% Ei(t)= eo x sum exp(alpha(k)*t)*Pi(alpha(k))/Q'(alpha(k))   where
% alpha(k) k=1,2,..,N are the N roots of Q(z)
%
% References:
%
% The method described above was a development of the work of several
authors, including:
% 1. Chou, K (1995) Biophys. Chem. 35, 1-24
% 2. Darvey, G (1968) J. Theoret. Biol. 19, 215-231
% 3. Hijazi, N and Laidler, K, J (1972) Can. J. Biochem., 51, 806-840
%
% Problems:
% 
% Repeated roots in Q(z) alter the form of Ei(t) and are not yet catered for.
% A root of multiplicity k will give terms in f(t^j)*exp(g(t)), j=1,2,..,k

% Initialise empty matrices to save time and redimensioning

M=sym(zeros(N,N));
D=sym(zeros(N,1));
Q=sym(zeros(1,1));
E=sym(zeros(N,1));

% Define matrix of diff. equations

for i=1:N
 for j=1:N
  t=input('Equation: ','s');
  if i==j
    t=symadd('z',t);
  else
    t=symop('-',sym(t));
  end
  M=sym(M,i,j,t);
 end
end
M

% D = column vector containing the N Pi(z)

for i=1:N
 T=M;
 for j=1:N
  T=sym(T,j,i,'0');
  T=sym(T,i,j,'1');
 end
 D=sym(D,i,1,determ(T));
end

% Q(z) = |M|

Q=determ(M);

% R = Q'(z)

R=diff(Q,'z');			% Q'(z)

% alpha = column vector containing roots of Q(z)

alpha=solve(Q,'z');

% nrt = no. of roots of alpha

nrt=symsize(alpha,1);

% if nrt < N then repeated roots and simple Heaviside expansion invalid

if nrt ~= N
  F='Repeated roots'
else

% Generate inverse transforms

for i=1:N
 for j=1:nrt
  rt=sym(alpha,j,1);			% select current root
  pak=subs(sym(D,i,1),rt,'z');		% pak = P(alpha(k))
  qak=subs(R,rt,'z');			% qak = Q'(alpha(k))
  eak=symop('exp(t*',rt,')');		% eak = exp(t*alpha(k))
  T=symdiv(pak,qak);
  T=symmul(T,eak);
  T=symadd(sym(E,i,1),T);		% T = current term in this eqtn
  E=sym(E,i,1,T);
 end
 M=sym('eo');
 T=symmul(sym(E,i,1),M);
 E=sym(E,i,1,T);
end

% Return column vector containing Ei(t)

F=E;
end

% end of trans(N)
