/****************************************************************/
/*	MatDiff is a C program which compares two .mat files.		*/
/*  It prints out lists of:										*/
/*     Variables in the first file but not in the second		*/
/*     Variables in the second file but not in the first		*/
/*	   Variables which appear in both files and are equal		*/
/* 	   Variables which appear in both files but are different	*/
/*																*/
/*  For the last category it will print out some information	*/
/*  about how the variables differ; if one is real while the	*/
/*  other is complex, if they are different sizes, etc.			*/
/* 																*/
/*  If the -n flag is specified on the command line it will		*/
/*  print the number of elements which differ in two matrices	*/
/*  with the same name and size.  								*/
/*																*/
/*  The -r flag allows the user to specify a ratio which 		*/
/*  determines whether two values are "different".  The 		*/
/*  default ratio is 1e-6.  So 1.0 and 1.0 + 2e-6 will be		*/
/*  considered to be the same, while 1.0 and 1.0 + 1e-7 		*/
/*  will be considered to be different.  						*/
/*																*/
/*	Version 1.0 												*/
/* 	Copyright (c) 1995 by Keith Rogers	kerog@draper.com		*/
/****************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include "mat.h"

#define	MAXVARSIZE	100
#define TRUE		1
#define FALSE		0


main(int argc, char *argv[])
{
	int 	i,j;
	char	temp[22];

	MATFile *firstfile, 			/* Input file descriptors */
			*secondfile;

	int		firstfileind = 0,		/* Indices into argv in which the */
			secondfileind = 0;		/* file names appear.             */

	int 	first_num_vars,			/* Number of variables in each file */
			second_num_vars;

	int		*first_list,			/* -1 if not in the other list,  */
			*second_list; 			/* otherwise index in other list */

	int		*common_diff_list, 		/* TRUE if matrix is same in both lists */
			same,					/* TRUE if two matrices are equal       */
			imag = FALSE,			/* TRUE if a matrix is complex          */
			numberflag = 0;				/* if TRUE, count number of different   */
									/* entries when two matrices differ     */

	int		num_first_only = 0,		/* Number of variables appearing only    */
			num_second_only = 0,	/* in the first file, only in the second */
			common_number = 0;		/* file, and in both files               */

	char  	**first_only_list, 		/* Names of variables appearing only in */
			**second_only_list,		/* the first file, only in the second   */
			**common_list; 			/* file, and in both files              */

	int		firstind = 0,			/* Indices into the above-declared lists */
			secondind = 0,
			commonind = 0;

	int		num_same = 0;			/* Number of common variables which */
									/* are equal. */

	int 	m1,m2,n1,n2;			/* Matrix dimensions */
	double 	*pr1,*pr2,*pi1,*pi2;	/* Matrix real and imaginary parts */
	Matrix	*mat1,*mat2;

	double	ratio = 1e-6;			/* Ratio of difference to value  */
									/* above which two variables are */
									/* considered to be different.   */

	char	**first_vars, 			/* Lists of variable names */
			**second_vars; 

	char 	**diffstr1,**diffstr2;	/* Strings to be printed if matrices */
									/* are different.					 */
	
	int		numdiffs;				/* Number of entries which differ in   */
									/* matrices of the same name appearing */
									/* in different files.                 */

	/**********************************/
	/* Process command line arguments */
	/**********************************/

	for(i=1;i<argc;i++)
	{
		if(strcmp(argv[i],"-n") == 0)
			numberflag = TRUE;
		else if(strcmp(argv[i],"-r") == 0)
			if(argc > i+1)
				ratio = atoi(argv[++i]);
			else
			{
				fprintf(stderr,"Usage: ios2mat [-n] [-r ratio] file1.mat file2.mat]\n");
				exit(1);
			}
		else if(strstr(argv[i],".mat"))
			if(firstfileind == 0)
				firstfileind = i;
			else if(secondfileind == 0)
				secondfileind = i;
			else
			{
				fprintf(stderr,"Only 2 files allowed for comparison!\n");
				exit(1);
			}
		else
		{
			fprintf(stderr,"Don't understand argument '%s'\n",argv[i]);
			exit(1);
		}
	}
	if(secondfileind == 0)
	{
		fprintf(stderr,"Usage: ios2mat [-n] [-r ratio] file1.mat file2.mat]\n");
		exit(1);
	}
	
	/******************/
	/* Open the files */
	/******************/

	firstfile = matOpen(argv[firstfileind],"r");
	if(firstfile == NULL)
	{
		fprintf(stderr,"Couldn't open file %s\n",argv[firstfileind]);
		exit(1);
	}
	secondfile = matOpen(argv[secondfileind],"r");
	if(secondfile == NULL)
	{
		fprintf(stderr,"Couldn't open file %s\n",argv[secondfileind]);
		exit(1);
	}

	/*********************/
	/* Get Variable List */
	/*********************/

	first_vars = matGetDir(firstfile,&first_num_vars);
	second_vars = matGetDir(secondfile,&second_num_vars);

	first_list = (int *)malloc(first_num_vars*sizeof(int));
	second_list = (int *)malloc(second_num_vars*sizeof(int));

	for(i=0;i<second_num_vars;i++)
		second_list[i] = -1;
	for(i=0;i<first_num_vars;i++)
	{
		first_list[i] = -1;
		for(j=0;j<second_num_vars;j++)
		{
			if(strcmp(first_vars[i],second_vars[j]) == 0)
			{
				first_list[i] = j;
				second_list[j] = i;
				common_number++;
				break;
			}
		}
	}

	/******************************/
	/* Allocate Storage for Lists */
	/******************************/

	num_first_only = first_num_vars-common_number;
	first_only_list = (char **)malloc(num_first_only*sizeof(char *));

	num_second_only = second_num_vars-common_number;
	second_only_list = (char **)malloc(num_second_only*sizeof(char *));

	common_list = (char **)malloc(common_number*sizeof(char *));

	firstind = secondind = commonind = 0;

	/****************************************/
	/* Create lists of common variables and */
	/* variables only appearing in the      */
	/* first file.                          */
	/****************************************/

	for(i=0;i<first_num_vars;i++)
		if(first_list[i] < 0)
			first_only_list[firstind++] = first_vars[i];
		else
			common_list[commonind++] = first_vars[i];

	/**********************************/
	/* Create list of variables only  */
	/* appearing in the second file.  */
	/**********************************/

	for(i=0;i<second_num_vars;i++)
		if(second_list[i] < 0)
			second_only_list[secondind++] = second_vars[i];

	common_diff_list = (int *)calloc(common_number,sizeof(int));
	diffstr1 = (char **)malloc(common_number*sizeof(char *));
	diffstr2 = (char **)malloc(common_number*sizeof(char *));
	for(i=0;i<common_number;i++)
	{
		diffstr1[i] = (char *)calloc(22,sizeof(char));
		diffstr2[i] = (char *)calloc(22,sizeof(char));
	}

	/**********************************/
	/* Compare variables appearing in */
	/* both files to see if they are  */
	/* the same.                      */
	/**********************************/

	for(i=0;i<common_number;i++)
	{
		imag = FALSE;
		same = TRUE;
		mat1 = matGetMatrix(firstfile,common_list[i]);
		matGetFull(firstfile,common_list[i],&m1,&n1,&pr1,&pi1);
		mat2 = matGetMatrix(secondfile,common_list[i]);
		matGetFull(secondfile,common_list[i],&m2,&n2,&pr2,&pi2);

		/*******************************/
		/* Check to see if either side */
		/* has an imaginary part.      */
		/*******************************/

		if(mxIsComplex(mat1))
		{
			if(mxIsComplex(mat2))
				imag = TRUE;
			else
			{
				same = FALSE;
				strcpy(diffstr1[i],"Complex");
				strcpy(diffstr2[i],"Real");
			}
		}
		else if(mxIsComplex(mat2))
		{
			same = FALSE;
			strcpy(diffstr2[i],"Complex");
			strcpy(diffstr1[i],"Real");
		}

		/*******************************/
		/* Check to see if either side */
		/* is a string.                */
		/*******************************/

		if(mxIsString(mat1))
		{
			if(!mxIsString(mat2))
			{
				same = FALSE;
				strcpy(diffstr1[i],"String");
				strcpy(diffstr2[i],"Number");
			}
		}
		else if(mxIsString(mat2))
		{
			same = FALSE;
			strcpy(diffstr2[i],"String");
			strcpy(diffstr1[i],"Number");
		}

		/*******************************/
		/* Check to see if either side */
		/* is a sparse matrix.         */
		/*******************************/
		
		if(mxIsSparse(mat1))
		{
			if(!mxIsSparse(mat2))
			{
				same = FALSE;
				strcpy(diffstr1[i],"Sparse");
				strcpy(diffstr2[i],"Full");
			}
		}
		else if(mxIsSparse(mat2))
		{
			same = FALSE;
			strcpy(diffstr2[i],"Sparse");
			strcpy(diffstr1[i],"Full");
		}	
	
		/******************************/
		/* Check to see if both sides */
		/* are the same size.         */
		/******************************/

		if(same)
		{
			if((m1 != m2) || (n1 != n2))
			{
				same = FALSE;
				sprintf(temp,"%dx%d",m1,n1);
				strcpy(diffstr1[i],temp);
				sprintf(temp,"%dx%d",m2,n2);
				strcpy(diffstr2[i],temp);
			}

			/*******************************/
			/* Both sides are matrices of  */
			/* the same size.  See if they */
			/* are the same.               */
			/*******************************/

			else

			{
				numdiffs = 0;
				for(j=0;j<m1*n1;j++)
				{
					if(pr1[j] != pr2[j])
						if(abs((pr1[j]-pr2[j])/pr1[j]) > ratio)
						{
							same = FALSE;
							if(numberflag)
								numdiffs++;
							else
								break;
						}
					else if(imag == TRUE)
					{
							if(pi1[j] != pi2[j])
								if(abs((pi1[j]-pi2[j])/pi1[j]) > ratio)
								{
									same = FALSE;
									if(numberflag)
										numdiffs++;
									else
										break;
								}
					}
				}
				if(!same)
					if(numberflag)
					{	
						sprintf(temp,"%d",numdiffs);
						strcpy(diffstr1[i],temp);
						if(numdiffs == 1)
							strcpy(diffstr2[i],"entry is different");
						else
							strcpy(diffstr2[i],"entries are different");
					}
					
			}
		}
		if(same)
		{
			common_diff_list[i] = 1;
			num_same++;
		}
		mxFree(pr1); mxFree(pi1);
		mxFree(pr2); mxFree(pi2);
	}

	/**********/
	/* OUTPUT */
	/**********/

	if(num_first_only > 0)
	{
		printf("Variables appearing only in %s:\n",argv[firstfileind]);
		for(i=0;i<num_first_only;i++)
			printf("%s\n",first_only_list[i]);
		printf("\n");
	}
	
	if(num_second_only > 0)
	{
	printf("Variables appearing only in %s:\n",argv[secondfileind]);
	for(i=0;i<num_second_only;i++)
		printf("%s\n",second_only_list[i]);
	printf("\n");
	}

	if(num_same > 0)
	{
		printf("Variables appearing in both files that are equal:\n");
		for(i=0;i<common_number;i++)
			if(common_diff_list[i])
				printf("%s\n",common_list[i]);
		printf("\n");
	}

	if(common_number-num_same > 0)
	{
		printf("Variables appearing in both files that are not equal:\n");
		printf("\n");
		printf("Variable Name        %-21.21s  %-21.21s\n",argv[firstfileind],argv[secondfileind]);
		for(i=0;i<common_number;i++)
			if(!common_diff_list[i])
				printf("%-15.15s      %-21.21s  %-21.21s\n",common_list[i],diffstr1[i],diffstr2[i]);
	}

	matClose(firstfile);
	matClose(secondfile);
	mxFree(first_vars);
	mxFree(second_vars);
}
