function varsize=savevar(fd, varname, variable, precision)
%
% function varsize=savevar(fd, varname, variable, precision)
% 
% Saves the 'variable' in a .mat format under 'varname'
% using 'precision'. 'Precision' is a string variable with 
% one of the following values: 
% 
%            	   'char   ' %  8 bits  unsigned int  
%                  'uchar  ' %  8 bits  unsigned int  
%                  'short  ' % 16 bits    signed int  
%                  'ushort ' % 16 bits  unsigned int  
%                  'int    ' % 32 bits    signed int  
%                  'long   ' % 32 bits    signed int  
%                  'float  ' % 32 bits             
%                  'float32' % 32 bits             
%                  'double ' % 64 bits             
%                  'float64' % 64 bits             
%                  'int8   ' %  8 bits  unsigned int 
%                  'int16  ' % 16 bits    signed int 
%                  'int32  ' % 32 bits    signed int 
%                  'uint8  ' %  8 bits  unsigned int 
%                  'uint16 ' % 16 bits  unsigned int 
%
% Returns the size of the variable in bytes. 
%
%   Example: 
%
%	x=65:75; 
%	fd = fopen('test.mat', 'wb');  
%	savevar(fd, 'x_char', setstr(x), 'char') ; 
%	savevar(fd, 'x_float', x, 'float') ; 
%	fclose(fd); 
%
%
% The function does not work for sparse matrices!
% The machine type codes for IBM_RS and CONVEX are not documented,
% so the fuction cannot work on those machines. 
% 
% 	Written by: Tihomir Hristov
% 		    Atmospheric Turbulence Lab
%                   Dept. of Mechanical Engineering
%                   University of California, Irvine
% 		    Irvine, CA 92697-3975
%                   Voice: (714)-824-7437
% 		    Email: tihomir@wave.eng.uci.edu
 

					   % VarType Code

     VarTypes = ['char   ' %  8 bits  unsigned  %  5
                 'uchar  ' %  8 bits  unsigned  %  5
                 'short  ' % 16 bits    signed  %  3
                 'ushort ' % 16 bits  unsigned  %  4
                 'int    ' % 32 bits    signed  %  2
                 'long   ' % 32 bits    signed  %  2
                 'float  ' % 32 bits            %  1
                 'float32' % 32 bits            %  1
                 'double ' % 64 bits            %  0
                 'float64' % 64 bits            %  0
                 'int8   ' %  8 bits   unsigned %  5
                 'int16  ' % 16 bits     signed %  3
                 'int32  ' % 32 bits     signed %  2
                 'uint8  ' %  8 bits   unsigned %  5
                 'uint16 ' % 16 bits   unsigned %  4
     ];

computers = [
%       PCWIN  - MS-Windows             DEC_RISC - DecStation
%       MAC2   - Macintosh              VAX_VMSD - VAX/VMS D_float
%       SUN4   - Sun SPARC              VAX_VMSG - VAX/VMS G_float
%       SOL2   - Solaris 2              ALPHA    - Dec Alpha
%       HP700  - HP 9000/700            HP300    - HP 9000/300
%       SGI    - Silicon Graphics       CRAY     - Cray
%       IBM_RS - IBM RS6000             CONVEX   - Convex

        'PCWIN   ' % 0
        'DEC_RISC' % 0
        'MAC2    ' % 1
        'VAX_VMSD' % 2
        'SUN4    ' % 1
        'VAX_VMSG' % 3
        'SOL2    ' % 1
        'ALPHA   ' % 0
        'HP700   ' % 1
        'HP300   ' % 1
        'SGI     ' % 1
        'CRAY    ' % 4
      % 'IBM_RS  ' % not documented
      % 'CONVEX  ' % not documented
];

P = [ 5 5 3 4 2 2 1 1 0 0 5 3 2 5 4]' ;
M = [ 0 0 1 2 1 3 1 0 1 1 1 4] ;

S = [ 1 1 2 2 4 4 4 4 8 8 1 2 4 1 2]' ; 

varname = varname(varname > 32);

[mrows ncols]=size(variable);

precision = lower(precision);
precision = precision(precision > 32);  % Eliminate spaces & controls
for i = 1:size(VarTypes,1)
        currentvt = VarTypes(i,:);
        currentvt = currentvt(currentvt > 32); % Eliminate spaces & controls
        if (max(size(currentvt)) == max(size(precision)))
                if(currentvt == precision)
                        vt_index = i;
                        break ;
                end
        end

end

comptype = computer;
for i = 1:size(computers,1)
        currentcomp = computers(i,:);
        currentcomp = currentcomp(currentcomp > 32); % Eliminate spaces & controls
        if (max(size(currentcomp)) == max(size(comptype)))
                if(currentcomp == comptype)
                        comp_index = i;
                        break ;
                end
        end
end


varsize = S(vt_index) * prod(size(variable)); 

MOPT = M(comp_index) * 1000 + 0 * 100 + P(vt_index) * 10 + isstr(variable);

namlen = max(size(varname)) + 1; % + 1 because of the string terminator
header = [ MOPT mrows ncols any(any(imag(variable))) namlen];

fwrite(fd, header, 'ulong');
fwrite(fd, [varname 0], 'uchar');
fwrite(fd, variable, precision);

  

