function [z0,w0] = scimapz0(prefix,wp,w,beta,z,c,qdat,aux)
%	Returns starting points for computing inverse maps.

%	Each wp(j) requires z0(j) whose image w0(j) is such that the
%	line segment from w0(j) to wp(j) lies in the target (interior
%	or exterior) region.  The algorithm here is to choose z0(j) as
%	a (weighted) average of successive pairs of adjacent
%	prevertices.  The resulting w0(j) is on a polygon side.  Each
%	choice is tested by looking for intersections of the segment
%	with (other) sides of the polygon.
%	
%	After randomly trying 10 weights with such prevertex pairs, the
%	routine gives up.  Slits are the most likely cause of trouble,
%	since the intersection method doesn't know which side of the
%	slit it's on.

%	By Toby Driscoll.  Last updated 7/7/94.

n = length(w);
shape = wp;
wp = wp(:);
z0 = wp;
w0 = wp;
from_disk = strcmp(prefix(1),'d');
from_hp = strcmp(prefix,'hp');
from_strip = strcmp(prefix,'st');
from_rect = strcmp(prefix,'r');
if from_strip
  kinf = max(find(isinf(z)));
  argw = cumsum([angle(w(3)-w(2));-pi*beta([3:n,1])]);
  argw = argw([n,1:n-1]);
else
  argw = cumsum([angle(w(2)-w(1));-pi*beta(2:n)]);
end
if from_disk
  argz = angle(z);
  argz(argz<=0) = argz(argz<=0) + 2*pi;
end

factor = 0.5;				% images of midpoints of preverts
done = zeros(1,length(wp));
m = length(wp);
iter = 0;

while m > 0				% while some not done
  for j = 1:n 				% cycle thru starting points
    % Choose a point on the jth side of the polygon.
    if from_disk
      if j<n
	basis = exp(i*(factor*argz(j) + (1-factor)*argz(j+1)));
      else 
	basis = exp(i*(factor*argz(n) + (1-factor)*(2*pi+argz(1))));
      end
    elseif from_hp
      if j < n-1			% between two finite points
	basis = z(j) + factor*(z(j+1)-z(j));
      elseif j==n-1			% between x(n-1) & Inf
	basis = max(10,z(n-1))/factor;
      else				% between -Inf and x(1)
	basis = min(-10,z(1))/factor;
      end
    elseif from_strip
      if j==1
	basis = min(-1,real(z(2)))/factor/4;
      elseif j==kinf-1
	basis = max(1,real(z(kinf-1)))/factor/4;
      elseif j==kinf
	basis = i+max(1,real(z(kinf+1)))/factor/4;
      elseif j==n
	basis = i+min(-1,real(z(n)))/factor/4;
      else 
	basis = z(j) + factor*(z(j+1)-z(j));
      end
    elseif from_rect
      basis = z(j) + factor*(z(rem(j,n)+1)-z(j));
    end
    z0(~done) = ones(m,1)*basis;
    if ~from_rect
      w0(~done) = ones(m,1)*feval([prefix,'map'],basis,w,beta,z,c,qdat);
    else 
      w0(~done) = ones(m,1)*feval([prefix,'map'],basis,w,beta,z,c,qdat,aux);
    end
    notdone = find(~done);
    done = ones(1,length(wp));
    % Test line segment for intersections with other sides.
    % We'll parameterize line segment and polygon side, compute parameters
    % at intersection, and check parameters at intersection.
    for k=[1:j-1,j+1:n]
      if isinf(w(k))
	A(:,1) = [cos(argw(k)+pi);sin(argw(k)+pi)];
	wk = w(rem(k,n)+1);
	s1max = Inf;
      else
	A(:,1) = [cos(argw(k));sin(argw(k))];
	wk = w(k);
	s1max = abs(w(rem(k,n)+1)-w(k));
      end
      for p = notdone
	A(:,2) = [real(w0(p)-wp(p));imag(w0(p)-wp(p))];
	% Get line segment and side parameters at intersection.
	s = A\[real(w0(p)-wk);imag(w0(p)-wk)];
	% Intersection occurs interior to side and segment? 
	if s(1)>=0 & s(1)<=s1max & s(2)>-10*eps & s(2)<1-10*eps
          % it's no good
	  done(p) = 0;
	end
      end
    end
    m = sum(~done);
    if ~m, break, end
  end
  if iter > 10
    error('Can''t seem to choose starting points.  Supply them yourself.')
  else
    iter = iter + 1;
  end
  factor = rand(1);			% abandon midpoints
end

shape(:) = z0;
z0 = shape;
shape(:) = w0;
w0 = shape;
