echo on;
%          INTRODUCTION TO THE GENERAL RECOGNITION THEORY TOOLBOX
%          ======================================================
%
%                             Version 1.3
%
%                       by Leola A. Alfonso-Reese
%                        University of California
%                        Santa Barbara, CA  93106
%
%                          Copyright (c) 1995,1996
%                     
%
% This is a package of MATLAB programs to generate and analyze data for
% psychology experiments based on the General Recognition Theory.
%
%
%                               Notes
%                               -----
%
% This toolbox is free and the author makes no warranty of any kind with
% regard to the programs or online help contained herein.  The author,
% Leola A. Alfonso-Reese, is not liable for any consequences of installing
% or running these files.
%
% The M-files and any modifications of the M-files contained in this
% toolbox may not be sold, published, or used for commercial gain.
%
% You may copy and modify the GRT Toolbox files as long as the original
% author is acknowledged AND you document any modifications you make
% as your own.  ALSO, IF THIS TOOLBOX IS USED IN ANY PUBLISHED RESEARCH,
% I WOULD APPRECIATE AN ACKNOWLEDGEMENT.
%
% Address any questions, comments, suggestions, and ESPECIALLY BUG
% REPORTS to:
%
%       Leola A. Alfonso-Reese
%       Psychology Department
%       University of California
%       Santa Barbara, CA  93106
%
%       reese@condor.psych.ucsb.edu
%
% I would enjoy email regarding your reaction to this toolbox.
%
% July 1996
%
%                              Thanks
%                              ------
%
% Thanks to David H. Brainard for helping me get started with MATLAB
% and for teaching me how to speed up my code.
% 
% Thanks to F. Gregory Ashby for many discussions of General Recognition
% Theory.
% 
% Thanks to Patricia Berretty and Sarah Queller for serving as my
% beta testers.
%
% Special thanks to Michael Steele Reese for serving as my UNIX beta
% tester and for originally suggesting that I make a GRT Toolbox!
%
%
%                        Installation Notes
%                        ------------------
%
% The toolbox is written to be used with the Macintosh version 4.2 of
% MATLAB.  Since all the files are simply text files, they may be
% installed on a UNIX machine and an IBM PC.  While things work
% normally on a UNIX machine, some problems will be encountered on an
% IBM PC.  This is because some routine names are long and DOS cannot
% distinguish between two routines with the same first eight characters.
% This problem can easily be fixed by changing names of the relevant
% routines.  Unfortunately, I am a Macintosh fan so I have no motivation
% to make a working PC version of GRT.  Finally, note that some data
% analysis functions depend on the Optimization Toolbox which can be
% purchased from the MathWorks, Inc.
%
% I suggest that all the GRT subroutine M-files be placed in a folder
% called GRT and that this folder be placed with other toolboxes in
% MATLAB's toolbox folder.  Also, all the demonstration M-files should be
% placed in the GRTdemos folder within the GRT folder.  Remember that
% the names of the GRT directory and the GRTdemos subdirectory must be
% added to the MATLAB path.
%
%
%                        After Installation
%                        ------------------
%
% For a brief description of all the functions in the GRT Toolbox, type
%     help GRT
%
% For a brief description of the demonstrations, type
%     help GRTdemos
%
% For a description of each GRT function, type 'help' followed by the
% function name, e.g.,
%     help dprime
%
% To see the contents of a function name or demonstration file, type
% 'type' followed by the function or demo name, e.g.,
%     type Comp2dStats
% 
%
%                 Background on General Recognition Theory
%                 ----------------------------------------
%
% General Recognition Theory (GRT; Ashby, 1992) is a multi-dimensional
% version of signal detection theory (Thurstone, 1927; Green & Swets,
% 1966). It was developed by psychologists (Ashby & Townsend, 1986;
% Ashby & Gott, 1988) to study the human categorization process.
%
% In GRT, a stimulus is represented by a point in multi-dimensional
% space. When a subject observes a multi-dimensional stimulus, the
% sensory and perceptual system adds noise to the signal.  This signal
% plus noise is called the perceptual effect.  The perceptual effects
% associated with a particular stimulus are represented by a multi-
% dimensional probability distribution.  A category is represented by
% a mixture of the associated stimulus distributions.  In a categorization
% task, the observer learns to separate the multi-dimensional space into
% regions, one region per category.  Then, when the observer needs to
% respond to a stimulus, he determines the region in which the percept
% falls and emits the category label associated with that region.  Thus,
% the boundaries separating these regions represent the decision rules
% the observer uses to classify stimuli.
%
% Since perceptual effects are not directly observable, it is difficult
% for an experimenter to examine the decision boundaries that subjects
% use to categorize stimuli.  The General Recognition Randomization
% Technique developed by Ashby and Gott (1988) provides a solution to
% this problem.  First, an experimenter randomly generates stimuli by
% adding external noise to each category prototype, where the external
% noise varies according to some distribution.  (The external noise must
% be large enough to make the perceptual noise negligible in comparison.)
% Then, the stimuli are presented to a subject one at a time at high
% contrast and long duration; this further reduces perceptual noise.  For
% each stimulus presentation, the subject's category response is recorded
% and if the task is a supervised one, then the subject receives feedback
% indicating whether his response was correct or not.  After all the
% responses are collected, the experimenter represents each exemplar as
% a point in space, determines the decision boundary that best predicts
% the subjects' responses, and uses this boundary to study the human
% categorization process.
%
%
%            Studying Human Categorization with the GRT Toolbox
%            --------------------------------------------------
%
% This toolbox provides routines to help an experimenter design 
% categorization (or identification) experiments, generate stimuli for
% these experiments, simulate a subject's responses, analyze categorization
% data, and graph results.  The typical user of this toolbox designs
% experiments for two-category tasks, where the categories are specified
% by multi-variate normal distributions.  This version of the toolbox
% provides tools for fitting the General Linear Classifier and the
% General Quadratic Classifier to a data set.
%
% The functions currently provided in this toolbox are
%   angvec2xymat.m      lindecisbnd.m       percorr.m           
%   bnd2slint.m         lindiscrim2dvals.m  plot2dlinbnd.m      
%   clip_vals.m         linprobcorr.m       plot2dquadbnd.m     
%   dprime.m            negloglike_2dGLC.m  plot2dresp.m        
%   dprimef.m           negloglike_2dGQC.m  plot2dstim.m        
%   fastnormalcdf.m     new2old_2dparams.m  quaddecisbnd.m      
%   fisherdiscrim2d.m   norm_old_2dparams.m randrows.m          
%   fit_2dGLC.m         normalcdf.m         sim2dlin.m          
%   fit_2dGQC.m         normalcdfinv.m      transample.m        
%   gensample.m         normalpdf.m         xymat2angvec.m      
%   getbndendpts.m      old2new_2dparams.m  

% The demonstrations currently provided in this toolbox are
%   Comp2dStats.m    Find2dGLC.m      Gen2dGRTstim.m   Sim2dSubj.m      
%   Design2dGRTexp.m Find2dGQC.m      Plot2dGRTdata.m  
echo off;
