% Lateral- Directional Air Vehicle Model
% Origin Knox Engineering, 164 Oldwoodwood Rd., Huntsville Al., 35811
% Use Basic Aerodynamic Data to
% Realize Lateral-Directional Air Vehicle Dynamics in State-Space, and
% Transfer Function Form
% Reference McRuer, Etkin, Roskam
% State Space Model dx / dt = a*x + b*u, y + I*x
% Vector x =
%     Beta                        - Radian
%     P                           - Radian / Sec
%     R                           - Radian / Sec
%     Phi                         - Radian
% Vector u =
%     Rudder                      - Radian
%     Spoiler                     - Radian
% Equlibrium 'Trim' Data
  keas = 75.                            % KNOTS
  h = 1000.,                            % Altitude - FEET
  w =266.,                              % Weight   - LBF
  alpha = 10.,                          % Angle-of-Attack - DEGREES
  phi = 0. ,                            % Roll Angle - DEGREES
  gam = 0. ,                            % Flt. Path Angle - DEGREES
% Geometric Data
  s = 10.   ;                           % Wing Area FT^2
  b = 8.43 ;                            % Span      FT
% Constants
  lapse = 6.87535e-6;                   % Atmos. Lapse Rate
  sigma = (1. - lapse*h)^4.2563;        % Rho / Rho Sea Level
  u1 = keas/sqrt(sigma)*1.688           % VTRUE - FT/SEC
  q=keas*keas/295.17                    % Dynamic Pressure - LBF/FT^2
  rad = 57.296;                         % Conversion Factor
  alphar = 10./rad;                     % Alpha Radians
  phir = 0./rad;                        % Phi Radians
  gamr = 0./rad;                        % Gamma Radians
  g = 32.17;                            % Gravitational Constant
  m = w/g;                              % Mass
%  Body Axis Inertias (Fuselage Ref. Line) -- Lbf-Ft^2-Sec
  xx = 10.89          ;
  xz = 2.84;
  zz = 25.6;
%  Convert Inertias to Stability Axes
%  Rotate  Axes Through an Angle Alpha (+) Above Fuselage Ref. Line
  csq = cos(alphar)^2;
  ssq = sin(alphar)^2;
  c2a = cos(2.*alphar);
  s2a = sin(2.*alphar);
 'Stability Axis Inertias'
  ixx = xx*csq + zz*ssq - xz*s2a
  ixz = 0.5*(xx-zz)*s2a + xz*c2a
  izz = zz*csq + xx*ssq + xz*s2a
% Enter Aerodynamic Data;  Non-Dimentional  ; Per Radian
  clp = -.436;
  clr = 0.262;
  clb = -0.223;
  cnp = -0.0234;
  cnr = -.323;
  cnb = .0958;
  cyp = 0.027;
  cyr = 0.121;
  cyb = -1.667;
  clds = -.02;
  cldr = .0315;
  cnds = .00138;
  cndr = -.14;
  cyds = -.003;
  cydr = 0.422;
%
% Compute Dimentional Derivatives
%
  qsdmv = q*s/m/u1;                % 1/SEC
  qsbdix = q*s*b/ixx;              % 1/SEC^2
  qsbdiz = q*s*b/izz;              % 1/SEC^2
  bd2v = b/2./u1;                  % SEC
  yv_R_SEC= qsdmv*cyb              % 1/m/u1 dY/d(beta)   1/SEC
  yp_R_RAD = qsdmv*bd2v*cyp        % 1/m/u1 dY/d(p)      1/RAD
  yr_R_RAD = qsdmv*bd2v*cyr        % 1/m/u1 dY/d(r)      1/RAD
  yphi_R_SEC = g/u1*cos(phir)      % 1/m/u1 dY/d(phi)    1/SEC
  ydr_R_SEC = qsdmv*cydr           % 1/m/u1 dY/d(rud)    1/SEC
  yds_R_SEC = qsdmv*cyds           % 1/m/u1 dY/d(spl)    1/SEC
  lbstr_R_SEC_2 = qsbdix*clb       % 1/ixx  dl/d(beta)   1/SEC^2
  lpstr_R_SEC = qsbdix*bd2v*clp    % 1/ixx  dl/d(p)      1/SEC
  lrstr_R_SEC = qsbdix*bd2v*clr    % 1/ixx  dl/d(r)      1/SEC
  ldsstr_R_SEC_2 = qsbdix*clds     % 1/ixx  dl/d(spl)    1/SEC^2
  ldrstr_R_SEC_2 = qsbdix*cldr     % 1/ixx  dl/d(rudd)   1/SEC^2
  nbstr_R_SEC_2 = qsbdiz*cnb       % 1/izz  dN/d(beta)   1/SEC^2
  npstr_R_SEC = qsbdiz*bd2v*cnp    % 1/izz  dN/d(p)      1/SEC
  nrstr_R_SEC = qsbdiz*bd2v*cnr    % 1/izz  dN/d(r)      1/SEC
  ndsstr_R_SEC_2  = qsbdiz*cnds    % 1/izz  dN/d(spl)    1/SEC^2
  ndrstr_R_SEC_2  = qsbdiz*cndr    % 1/izz  dN/d(rudd)   1/SEC^2
% Form State Space Realiziation; State Vector =
% Beta - radian; P- rad/sec; R- rad/sec; Phi-radian
% a1*xdot = a2*x + b1*u
% Premultiply by a1 to obtain state-space form - xdot = a*x + b*u
a1 = [ 1.    0.         0.        0.
       0.    1.         -ixz/ixx  0.
       0.    -ixz/izz   1.        0.
       0.    0.         0.        1. ];
a2 = [ yv_R_SEC       yp_R_RAD       yr_R_RAD-1.         yphi_R_SEC
       lbstr_R_SEC_2  lpstr_R_SEC    lrstr_R_SEC         0
       nbstr_R_SEC_2  npstr_R_SEC    nrstr_R_SEC         0.
       0.             1.             tan(gamr)           0. ];
% Form A Matrix
 a = a1\a2
% Form control matrix 'b1'; Controllers =
% Rudder- radians ; Spoiler - radians
 b1 = [ydr_R_SEC    yds_R_SEC
 ldrstr_R_SEC_2     ldsstr_R_SEC_2
 ndrstr_R_SEC_2     ndsstr_R_SEC_2
 0.                 0.              ];
% Form b Matrix
  b  = a1\b1
% Form c Matrix ; d Matrix
c = eye(4); d = zeros(4,2);
% Compute System Eigenvalues and Eigenvectors
 [v,d1] = eig(a)
% Compute Rudder Transfer Functions
 [num,den] = ss2tf(a,b,c,d,1)   ;
%
% Compute Transfer Functions
%
'Denominator Polynomial Coefficients in Descending Order',den
'Denominator Roots'
r0 = roots(den)
'beta / rudder numerator polynomial (rad/rad)'
p1 =             num(1,2:5)
' roots '
r1 = roots(p1)
' roll rate (p) / rudder numerator polynomial (rad/sec / rad)'
p2 =             num(2,2:5)
' roots '
r2 = roots(p2)
'  yaw rate (r) / rudder numerator polynomial (rad/sec / rad) '
p3 =             num(3,2:5)
' roots '
r3 = roots(p3)
' rollangle (phi) / rudder numerator polynomial (rad / rad)'
p4 =             num(4,3:5)
' roots '
r4 = roots(p4)
% Compute Spoiler Transfer Functions
[num2,den2] = ss2tf(a,b,c,d,2);
'beta / spoiler numerator polynomial (rad/rad)'
p5   =      num2(1,2:5)
' roots '
r5 = roots(p5)
' roll rate (p) / spoiler numerator polynomial (rad/sec / rad)'
p6   =      num2(2,2:5)
' roots '
r6 = roots(p6)
' yaw rate (r) / spoiler numerator polynomial (rad/sec / rad)'
p7 =        num2(3,2:5)
'  roots '
r7 = roots(p7)
' roll angle (phi) / spoiler numerator polynomial (rad/rad)'
p8  =       num2(4,3:5)
'  roots '
r8 = roots(p8)


