% Longitudinal Air Vehicle Model Formulation
% Origin: Knox Engineering, 164 Oldwood rd.,Huntsville Al. 35811
% Use Basic Aerodynamic Data to
% Realize Longitudinal Air Vehicle Dynamics in State-Space and
% Transfer Function Form
% Propeller & Conventional Jet Air Vehicle Model:
% Vectored Thrust Requires Different Formulation
% Reference McRuer, Roskam, Etkin
% State-Space Model dx / dt = a*x + b*u, y = I*x
% Vector x =
%    Velocity            - FT/SEC
%    Alpha               - RADIAN
%    Theta Dot           - RADIAN/SEC
%    Theta               - RADIAN
% Vector u =
%    Elevator Deflection - RADIAN
%    Thrust              - LBF
%
% Equlibrium 'Trim' Data
w = 266. ;       % Weight - Pounds Force
h = 1000. ;      % Altitude - Feet
keas = 75. ;     % Equivalent Airspeed - knots
gam  = 10. ;     % Flight Path Angle - Degrees
% Geometric Data
 sa = 10.;       % Wing Area - FT^2
 mac = 1.208;    % Mean Aerodynamic Chord - Feet
 iyy = 17.3;     % Inertia - LBF-SEC^2-FT
 z   = 0.0;       % Thrust to cg distance - Feet
% constants
 rad = 57.296;   % Conversion Factor
 g = 32.17;      % Gravitational Constant
 m = w/g;        % Mass
 lapse = 6.87535e-6;                     % Lapse Rate
 gamr = gam /rad;                        % Gamma - Radians
 sigma = (1. - lapse*h)^4.2563;          % sigma = (RHO/RHO-SEA LEVEL)
 qbar1 = keas*keas / 295.17;             % Dynamic Pressure-LBF/FT^2
 u1 = keas / sqrt(sigma) * 1.688;        % VTRUE - FT/SEC
% Aerodynamic data - 1 / rad - non-dimentional
 cl1 = 1.3413;        % Equlibrium Lift Coefficient
 clu = 0.;            % d(CL)/du * u1
 cla = 5.25;          % d(CL)/d alpha - 1/RAD
 clad = -8.16;        % d(CL)/d alpha dot * 2*u1/mac - 1/RAD
 cldlte = 0.314;      % d(CL)/d delta elevator - 1/RAD
 clq = 3.10;          % d(CL)/d pitch dot * 2*u1/mac - 1/RAD
 cm1 = 0.0036;        % Equlibrium Pitching Moment Coefficient
 cmu = 0.0    ;       % d(CM)/du * u1
 cma = -.716 ;        % d(CM)/d alpha - 1/RAD
 cmad = 1.57;         % d(CM)/d alpha dot * 2*u1/mac - 1/RAD
 cmq = -11.9;         % d(CM)/d pitch dot * 2*u1/mac - 1/RAD
 cmdlte = .564;       % d(CM)/d delta elevator - 1/RAD
 cd1 = 0.167   ;      % Equilibrium Drag Coefficient
 cdu = 0.       ;     % d(CD)/du * u1
 cda = .74216    ;    % d(CD)/d alpha - 1/RAD
 cddlte = 0.14   ;    % d(CD)/d delta elevator - 1/RAD
% Thrust data tc = thrust/qbar1/s
% Thrust data is separated into Aerodynamic,  e. g.
% CLT = Lift due to thrust / dynamic pressure / wing area,
% -- These terms are negligible for conventional jet aircraft
% but are significant for prop. and some STOL aircraft--  
% and  direct thrust  forces and moments , e. g. thvel
 thvel = 0.0      ;   % d(thrust)/du -  LBF/(FT/SEC)
 cltc = 0.20      ;   % d(CLT)/d(tc)
 cmtc = 0.00      ;   % d(CMT)/d(tc)
 cxtc = +1.00     ;   % d(CXT)/d(tc)
 cxt1 = 0.167     ;   % Equlibrium Aerodynamic Thrust Coefficient X-comp.
 cxtu = -.501     ;   % d(CXT)/du * u1
 clt1 = 0.2682    ;   % Equlibrium Aerodynamic Thrust Coefficient- Lift
 cltu = -.10      ;   % d(CLT)/du * u1
 cmt1 = 0.        ;   % Equlibrium Aerodynamic Thrust Coef. - Pitching Moment
 cmtu = 0.        ;   % d(CMT)/du * u1
% Compute Dimentional Derivatives
% XU,XTU,XW,ZW,MQ,ZU --(1/SEC)   ZWD--(1)
% XDLTE,ZDLTE --(FT/SEC^2); ZQ--(FT/SEC); MWD--(1/FT)
% MU,MTU,MW; -- (1/FT-SEC)
% qsu - LBF-SEC/FT; qsu/m - 1/SEC
 qsu = qbar1 * sa / u1;                           % LBF-SEC/FT
 zwd = -qsu * mac * clad / (2.*m*u1);             % (1)
 facw = 1. - zwd;                                 % (1)
 facu = u1 * facw;                                % FT/SEC
 xu_R_SEC = -qsu/m * ( cdu + 2. * cd1 ) ,         % 1/SEC
 xtu_R_SEC=  qsu/m * (cxtu + 2. * cxt1)+ thvel/m, % 1/SEC
 xw_R_SEC = -qsu/m * (cda  - cl1 ),               % 1/SEC
 xdlte_FT_R_SEC_2 = -qbar1/m * sa * cddlte,       % FT/SEC^2
 xdlts_FT_R_F_SEC_2 = +cxtc / m,                  % FT/LBF-SEC^2
 zu_R_SEC = -qsu/m * ( clu + 2. * cl1 ),          % 1/SEC
 ztu_R_SEC= -qsu/m * (cltu + 2. * clt1),          % 1/SEC
 zw_R_SEC = -qsu/m * ( cla + cd1 ),               % 1/SEC
 zq_FT_R_SEC = -qsu/m * mac * clq /2.,            % FT/SEC
 zdlte_FT_R_SEC_2 = -qbar1/m * sa * cldlte,       % FT/SEC^2
 zdlts_FT_R_F_SEC_2 = -cltc/m,                    % FT/LBF-SEC^2
 mu_R_FT_SEC = +qsu/iyy*mac*( cmu + 2.*cm1 ) ...
 +thvel*z/iyy,                                    % 1/FT-SEC
 mtu_R_FT_SEC= +qsu/iyy*mac*(cmtu + 2.*cmt1),     % 1/FT-SEC
 mw_R_FT_SEC = +qsu/iyy*mac*cma,                  % 1/FT-SEC
 mwd_R_FT= +qsu/iyy*mac^2* cmad /(2.*u1),         % 1/FT
 mq_R_SEC = +qsu/iyy*mac^2* cmq  /2.,             % 1/SEC
 mdlte_R_SEC_2 = qbar1/iyy*sa * mac * cmdlte,     % 1/SEC^2
 mdlts_R_F_SEC_2 = mac/iyy*(cmtc+z/mac),          % 1/LBF-SEC^2
format short e
% Form state matricies;
% Row 1
 a(1,1) = xu_R_SEC +xtu_R_SEC;
 a(1,2) = xw_R_SEC*u1;
 a(1,3)=0.;
 a(1,4)= -g*cos(gamr);
% Row 2
 a(2,1) = (zu_R_SEC+ztu_R_SEC) / facu;
 a(2,2)=zw_R_SEC/facw;
 a(2,3)=(u1+zq_FT_R_SEC)/facu;
          a(2,4) = -g * sin(gamr) / facu;
% Row 3
 a(3,1) = mu_R_FT_SEC + mtu_R_FT_SEC ...
 + mwd_R_FT*(zu_R_SEC+ztu_R_SEC) / facw;
 a(3,2) = mw_R_FT_SEC*u1 + mwd_R_FT*zw_R_SEC*u1 / facw;
 a(3,3) = mq_R_SEC + mwd_R_FT*(u1 + zq_FT_R_SEC) / facw;
 a(3,4) = -mwd_R_FT*g*sin(gamr)/facw;
% Row 4
 a(4,1)=0.; a(4,2)=0.; a(4,3)=1.; a(4,4)=0.;
% Form control (B) matrix; Col. 1- elevator; Col, 2 Thrust;
% Thrust - lbf ; Elevator- radian
 b(1,1) = xdlts_FT_R_F_SEC_2;
 b(2,1) = zdlts_FT_R_F_SEC_2 / facu;
 b(3,1) = mdlts_R_F_SEC_2 + mwd_R_FT*zdlts_FT_R_F_SEC_2/facw;
 b(4,1) = 0.;
% Column 2
 b(1,2) = xdlte_FT_R_SEC_2;
 b(2,2) = zdlte_FT_R_SEC_2 / facu;
 b(3,2) = mdlte_R_SEC_2 + mwd_R_FT*zdlte_FT_R_SEC_2/facw;
 b(4,2) = 0.;
%
% Form measurement and output matricies
 c = eye(4);
%
 d = [0 0; 0 0; 0 0; 0 0];
 [v,d1] = eig(a);
% Convert to transfer function form: (states/elevator)
 [num,den] = ss2tf(a,b,c,d,2);
'va = eigenvectors, ea =  eigenvalues of system'
[va,ea] = eig(a)
' '
' Compute    Transfer Functons'
' '
'Denominator Polynomial Coefficients in Descending Order',den
'denominator roots'
r0 = roots(den)
'velocity / elevator numerator polynomial (ft/sec/rad)'
p1 =       num(1,:)
'roots'
r1 = roots(p1)
'alpha / elevator numerator polynomial (rad/rad)'
p2 =       num(2,:)
'roots'
r2 = roots(p2)
'pitch rate / elevator numerator polynimial (rad/sec/rad)'
p3 =       num(3,:)
'roots'
r3 = roots(p3)
'theta (pitch) / elevator numerator polynomial (rad/rad)'

p4 =       num(4,:)
'roots'
r4 = roots(p4)
% Compute States / Thrust Transfer Functions
[num1,den] = ss2tf(a,b,c,d,1);
'velocity / thrust numerator polynomial (ft/sec / lbf)'
p5 =       num1(1,:)
'roots'
r5 = roots(p5)
'alpha / thrust numerator polynomial (rad / lbf)'
p6 =       num1(2,:)
'roots'
r6 = roots(p6)
'pitch rate / thrust numerator polynomial (rad/sec / lbf)'
p7 =       num1(3,:)
'roots'
r7 = roots(p7)
'theta (pitch) / thrust numerator polynomial (rad / lbf)'
p8 =       num1(4,:)
'roots'
r8 = roots(p8)

