function VarsOut=ames2(VarIn,Value,Gamma)

%FUNCTION:  AMES2
%
%SYNTAX:    function VarsOut=ames2(VarIn,Value,Gamma)
%
%ARGS:    - VarIn is a SCALAR having a value between 1 and 16.
%             The value of VarIN indicates the column of the table with which
%             the input is associated.  Multiple values are returned for
%             non-monotonic relationships.
%         - Value is a VECTOR containing values for the specified input variable
%         - Gamma is an optional vector or scalar of values for the specific 
%             heat of the flow medium. The Default Value = 7/5.
%
%
% Definition of Table Columns ...
%
%  Isentropic Relations                   Normal Shock Relations
%  ------------------------------------   --------------------------------------
%   1) M1    (Mach No)                    11) M2  (Mach No D.S. of Normal Shock)
%   2) P1/Pt (static/total pressure)      12) P2/P1   (static pressure ratio)
%   3) p1/pt (static/total density)       13) p2/p1   (static density ratio)
%   4) T1/Tt (static/total temperature)   14) T2/T1   (static temperature ratio)
%   5) abs(M1*M1-1)^(1/2)                 15) Pt2/Pt1 (total pressure ratio)
%   6) q1/Pt (dynamic/total pressure)     16) P1/Pt2  (static/total pressure)
%   7) A1/A* (actual/critical area)
%   8) V/a*  (actual/critical velocity)
%   9) Prandtl-Meyer Angle (Degrees)
%  10) Mach Angle (Degrees)
%

%REF: Ames Research Staff: Equations, Tables, and Charts for Compressible Flow,
%     NACA Report 1135, 1953.

% Compressible Flow Toolbox, Version 1.1, 23-FEB-96
% Written By: Kevin J. Melcher - 12/93
%             NASA Lewis Research Center
%             Cleveland, Ohio


%Condition the Input Arguments
 VarIn = abs(VarIn);
 Value = Value(:);
 [r,c]=size(Value); n=max(r,c);

 if nargin<3,
   Gamma = 7/5*ones(r,c);
 elseif max(size(Gamma))==1
   Gamma = Gamma*ones(r,c);
 else
   Gamma = Gamma(:);
 end%if

%Calculate Results
 %Calculate Some Common Constants
  Gm1= Gamma-1;
  Gp1= Gamma+1;
  C0 = 0.5*Gm1;		%(G-1)/2
  C1 = 0.5*Gp1;		%(G+1)/2
  D0 = -0.5./C0;	%-1/(G-1)
  D1 = Gamma.*D0;	%-G/(G-1)
  D2 = C1./Gm1;		%(G+1)/(G-1)/2

%Default Vector of Indices
% (required because some parameters return mulitples values)
 M1=nan*Value; k=[];

%Define Upper Bound on Search Space for Iterative Solutions of M1=F(Value)
 MnMax=1000;

%Define Name for Temporary File Used by Iterative Solutions
 Dir = lower(tempdir);  Name=lower(tempname); nd=max(size(Dir));
 FileName=sprintf('%s.m',Name);
 FuncName=Name;  FuncName(1:nd)=[];
 path(Dir,path);

%Convergence Criteria for Iterative Solutions
 opt=foptions; opt(2:3)=1e-8*ones(1,2);	%type 'help foptions'

%Calculate M1
 if VarIn==1, %from Mach No.
   %Check Bounds of Input
    if max(Value<0)>0,
      error('Parameter Out of Bounds:  Specified Mach No. < 0');
    end%if
   %Determine Mach Number(s)
    M1 = Value;
 elseif VarIn ==  2, %from Ps/Pt
   %Check Bounds of Input
    if max(Value<0)>0 | max(Value>1)>0,
      error('Parameter Out of Bounds:  Specified P/Pt < 0 or > 1');
    end%if
   %Determine Mach Number(s)
    M1 = ((Value.^(1.0./D1) - 1.0)./C0).^0.5;
 elseif VarIn ==  3, %from static density/total density
   %Check Bounds of Input
    if max(Value<0)>0 | max(Value>1)>0,
      error('Parameter Out of Bounds:  Specified Density Ratio < 0 or > 1');
    end%if
   %Determine Mach Number(s)
    M1 = ((Value.^(-Gm1) - 1.0)./C0).^0.5;
 elseif VarIn ==  4, %from Ts/Tt
   %Check Bounds of Input
    if max(Value<0)>0 | max(Value>1)>0,
      error('Parameter Out of Bounds:  Specified T/Tt < 0 or > 1');
    end%if
   %Determine Mach Number(s)
    M1 = ((Value.^(-1.0) - 1.0)./C0).^0.5;
 elseif VarIn ==  5, %from Beta
   %Check Bounds of Input
    if max(Value<0)>0,
      error('Parameter Out of Bounds:  Specified Beta < 0');
    end%if
   %Determine Mach Number(s)
    M1      = (1.0-Value'.*Value').^0.5;
    M1(2,:) = (1.0+Value'.*Value').^0.5;
    k(1,:)=1:n; k(2,:)=k(1:n);
 elseif VarIn ==  6, %from q/Pt
   %Check Bounds of Input
    Fmax=Gamma.*(Gamma.^(-Gamma./(Gamma-1)))+eps;	%maxima for each point
    if max(Value<0)>0,
      error('Parameter Out of Bounds:  Specified q/Pt < 0');
    elseif max(Value>Fmax),
      error('Parameter Out of Bounds:  Specified q/Pt > Max');
    end%if
   %Solve by minimizing the value of abs(M1(x)-x)
    %Generate M-file for solution
     fid=fopen(FileName,'w');
      fprintf(fid,'%s\n',['function Err=' FuncName '(Mn,Gam,qByPt);']);
      fprintf(fid,'%s\n','%NOTE: Mn, Gam, & qByPt MUST BE SCALARS');
      fprintf(fid,'%s\n','Gm1  =Gam-1;');
      fprintf(fid,'%s\n','MnSq =Mn*Mn;');
      fprintf(fid,'%s\n','FMn=0.5*Gam*MnSq*(1+0.5*Gm1*MnSq)^(-Gam/Gm1);');
      fprintf(fid,'%s\n','Err=abs(FMn-qByPt);');
     fclose(fid);
    %Use M-file to obtain solution at each point
     i=0;
     for j=1:n
      %Determine Low Mach Number
       i=i+1; k(i)=j; M1(i)=fmin(FuncName,0.00,sqrt(2),opt,Gamma(j),Value(j));
      %Determine High(er) Mach Number
       i=i+1; k(i)=j; M1(i)=fmin(FuncName,sqrt(2),MnMax,opt,Gamma(j),Value(j));
     end;%for
 elseif VarIn ==  7, %from A/A*
   %Check Bounds of Input
    if max(Value<1)>0,
      error('Parameter Out of Bounds:  Specified A/A* < 1');
    end%if
   %Solve by minimizing the value of abs(M1(x)-x)
    %Generate M-file for solution
     fid=fopen(FileName,'w');
      fprintf(fid,'%s\n',['function Err=' FuncName '(Mn,Gam,AbyAstar);']);
      fprintf(fid,'%s\n','%NOTE: Mn, Gam, & AbyAstar MUST BE SCALARS');
      fprintf(fid,'%s\n','Gm1=0.5*(Gam-1); Gp1=0.5*(Gam+1); Grat=0.5*Gp1/Gm1;');
      fprintf(fid,'%s\n','FMn=((Gp1^Grat)*Mn*((1+Gm1*Mn*Mn)^(-Grat)))^(-1);');
      fprintf(fid,'%s\n','Err=abs(FMn-AbyAstar);');
     fclose(fid);
    %Use M-file to obtain solution at each point
     i=0; 
     for j=1:n
      %Determine Low Mach Number
       i=i+1; k(i)=j; M1(i)=fmin(FuncName,0.00,1.00,opt,Gamma(j),Value(j));
      %Determine High(er) Mach Number
       i=i+1; k(i)=j; M1(i)=fmin(FuncName,1.00,MnMax,opt,Gamma(j),Value(j));
     end;%for
 elseif VarIn ==  8, %from V/a*
   %Check Bounds of Input
    Fmax=sqrt((Gamma+1)./(Gamma-1))+eps;	%maxima for each point
    if max(Value<0)>0,
      error('Parameter Out of Bounds:  Specified V/a* < 0');
    elseif max(Value>Fmax),
      error('Parameter Out of Bounds:  Specified V/a* > Max');
    end%if
   %Determine Mach Number(s)
    M1 = (Value.*Value ./ (C1 - C0.*Value.*Value)) .^ 0.5;
 elseif VarIn ==  9,  %from Prandtl-Meyer Angle (Degrees)
   %Check Bounds of Input
    Fmax=90*(sqrt((Gamma+1)./(Gamma-1))-1)+eps;	%maxima for each point
    if max(Value<0)>0,
      error('Parameter Out of Bounds:  Specified Prandtl-Meyer Angle < 0');
    elseif max(Value>Fmax),
      error('Parameter Out of Bounds:  Specified Prandtl-Meyer Angle > Max');
    end%if
   %Solve by minimizing the value of abs(M1(x)-x)
    %Generate M-file for solution
     fid=fopen(FileName,'w');
      fprintf(fid,'%s\n',['function Err=' FuncName '(Mn,Gam,PMdegrees);']);
      fprintf(fid,'%s\n','%NOTE: Mn, Gam, & PMdegrees MUST BE SCALARS');
      fprintf(fid,'%s\n','Grat=(Gam+1)/(Gam-1);');
      fprintf(fid,'%s\n','angle1  = sqrt((Mn*Mn-1)/Grat);');
      fprintf(fid,'%s\n','FMn=(sqrt(Grat)*atan(angle1)-acos(1/Mn))*180/pi;');
      fprintf(fid,'%s\n','Err=abs(FMn-PMdegrees);');
     fclose(fid);
    %Use M-file to obtain solution at each point
     i=0; 
     for j=1:n
      %The following "if" wrapper is needed for AMES2ERR.M
       if ~isnan(Value(j)) & Value(j)>=0,
         M1(j)=fmin(FuncName,1,MnMax,opt,Gamma(j),Value(j));
       end;
     end;%for
 elseif VarIn == 10, %from Mach Angle (degrees)
   %Check Bounds of Input
    if max(Value<0)>0 | max(Value>90)>0,
      error('Parameter Out of Bounds:  Specified Mach Angle < 0 or > 90');
    end%if
   %Determine Mach Number(s)
    M1 = 1.0./sin(Value*pi/180);
 elseif VarIn == 11, %from M2
   %Check Bounds of Input
    if max(Value>1)>0,
      error('Parameter Out of Bounds:  Specified M2 > 1');
    end%if
   %Determine Mach Number(s)
    M1 = ((Gm1.*Value.^2+2.0)./(2.0*Gamma.*Value.^2-Gm1)) .^ 0.5;
 elseif VarIn == 12, %from P2/P1
   %Check Bounds of Input
    if max(Value<1)>0,
      error('Parameter Out of Bounds:  Specified P2/P1 < 1');
    end%if
   %Determine Mach Number(s)
    M1 = ((Value.*Gp1+Gm1)./(2*Gamma)) .^ 0.5;
 elseif VarIn == 13, %from static density 2/static density 1
   %Check Bounds of Input
    if max(Value<1)>0,
      error('Parameter Out of Bounds:  Specified Density Ratio p2/p1 < 1');
    end%if
   %Determine Mach Number(s)
    M1 = (2.0*Value ./ (Gp1 - Gm1.*Value)) .^ 0.5;
 elseif VarIn == 14, %from T2/T1
   %Check Bounds of Input
    if max(Value<1)>0,
      error('Parameter Out of Bounds:  Specified T2/T1 < 1');
    end%if
   %Solve by minimizing the value of abs(M1(x)-x)
    %Generate M-file for solution
     fid=fopen(FileName,'w');
      fprintf(fid,'%s\n',['function Err=' FuncName '(Mn,Gam,T2byT1);']);
      fprintf(fid,'%s\n','%NOTE: Mn, Gam, & T2byT1 MUST BE SCALARS');
      fprintf(fid,'%s\n','M2=Mn*Mn;');
      fprintf(fid,'%s\n','FMn=(2*Gam*M2-Gam+1)*((Gam-1)*M2+2)/(M2*(Gam+1)^2);');
      fprintf(fid,'%s\n','Err=abs(FMn-T2byT1);');
     fclose(fid);
    %Use M-file to obtain solution at each point
     i=0;
     for j=1:n
      %Determine Mach Number(s)
       i=i+1;
       if ~isnan(Value(j)) & Value(j)>=0,
         M1(i)=fmin(FuncName,1,MnMax,opt,Gamma(j),Value(j));
       end%if
     end;%for
 elseif VarIn == 15, %from Pt2/Pt1
   %Check Bounds of Input
    if max(Value>1)>0,
      error('Parameter Out of Bounds:  Specified Pt2/Pt1 > 1');
    end%if
   %Solve by minimizing the value of abs(M1(x)-x)
    %Generate M-file for solution
     fid=fopen(FileName,'w');
      fprintf(fid,'%s\n',['function Err=' FuncName '(Mn,Gam,Pt2byPt1);']);
      fprintf(fid,'%s\n','%NOTE: Mn, Gam, &  Pt2byPt1 MUST BE SCALARS');
      fprintf(fid,'%s\n','Gp1=Gam+1; Gm1=Gam-1; M2=Mn*Mn;');
      fprintf(fid,'%s\n','FMn1=(Gp1*M2/(Gm1*M2+2))^(Gam/Gm1);');
      fprintf(fid,'%s\n','FMn2=(Gp1/(2*Gam*M2-Gm1))^(1/Gm1);');
      fprintf(fid,'%s\n','Err=abs(FMn1*FMn2-Pt2byPt1);');
     fclose(fid);
    %Use M-file to obtain solution at each point
     i=0;
     for j=1:n
      %Determine Mach Number
       i=i+1;
       if ~isnan(Value(j)) & Value(j)>=0,
         M1(i)=fmin(FuncName,1,MnMax,opt,Gamma(j),Value(j));
       end%if
     end;%for
 elseif VarIn == 16, %from P1/Pt2
   %Check Bounds of Input
    PstarByP0=C1.^D1;		%maxima for each point
   %Check Bounds of Input
    if max(Value>PstarByP0)>0,
      error('Parameter Out of Bounds:  Specified P1/Pt2 > P*/Pt');
    end%if
   %Solve by minimizing the value of abs(M1(x)-x)
    %Generate M-file for solution
     fid=fopen(FileName,'w');
      fprintf(fid,'%s\n',['function Err=' FuncName '(Mn,Gam,P1byPt2);']);
      fprintf(fid,'%s\n','%NOTE: Mn, Gam, &  P1byPt2 MUST BE SCALARS');
      fprintf(fid,'%s\n','Gp1=Gam+1; Gm1=Gam-1; M2=Mn*Mn;');
      fprintf(fid,'%s\n','FMn1=(0.5*Gp1*M2)^(-Gam/Gm1);');
      fprintf(fid,'%s\n','FMn2=(Gp1/(2*Gam*M2-Gm1))^(-1/Gm1);');
      fprintf(fid,'%s\n','Err=abs(FMn1*FMn2-P1byPt2);');
     fclose(fid);
    %Use M-file to obtain solution at each point
     i=0;
     for j=1:n
      %Determine Mach Number
       i=i+1;
       if ~isnan(Value(j)) & Value(j)>=0,
         M1(i)=fmin(FuncName,1,MnMax,opt,Gamma(j),Value(j));
       end%if
     end;%for
 end%if

%Delete Temporary M-file
 if exist(FileName)==2, delete(FileName); end

%Calculate Some Common Constants
 M1=M1(:); k=k(:);
 if ~isempty(k), Gamma=Gamma(k); end;
  Gamma=Gamma(:);
  Gm1= Gamma-1;
  Gp1= Gamma+1;
  C0 = 0.5*Gm1;		%(G-1)/2
  C1 = 0.5*Gp1;		%(G+1)/2
  D0 = -0.5./C0;	%-1/(G-1)
  D1 = Gamma.*D0;	%-G/(G-1)
  D2 = C1./Gm1;		%(G+1)/(G-1)/2
 end%if

 %Initialize Output Variable
  VarsOut = nan*ones([size(M1,1),16]);

 %Define Some more constants
  C2 = M1.*M1;		% M1^2
  C3 = C0.*C2+1.0;	%(1 + M1^2*(G-1)/2

  %1: Mach Number: M1
    VarsOut(:, 1) = M1;
  %2: Pressure Ratio:  p1/pt1
    VarsOut(:, 2) = C3.^D1;
  %3: Density Ratio: rho/rho-total
    VarsOut(:, 3) = C3.^D0;
  %4: Temperature Ratio: T1/Tt
    VarsOut(:, 4) = C3.^(-1.0);
  %5: Beta
    VarsOut(:, 5) = sqrt(abs(C2-1));
  %6: Dynamic Pressure Ratio: q1/pt1
    VarsOut(:, 6) = C2.*Gamma/2.0 .* VarsOut(:,2);
  %7: Area Ratio: A1/A*
   VarsOut(:, 7) = ( C1.^D2 .* M1 .* C3.^(-D2) ) .^ (-1.0);
    VarsOut(:, 7) = (0.5*Gp1).^(-0.5*Gp1./Gm1) ...
                    .*(1.0+0.5*Gm1.*M1.*M1).^(0.5*Gp1./Gm1) ...
                    ./M1;
  %8: Velocity Ratio: V1/a*1
   VarsOut(:, 8) = (C1.*C2.*VarsOut(:,4)) .^ (0.5);

 %Supersonic Flow/Normal Shock Calculations
  %9: Prandtl-Meyer Angle (Degrees)
    Grat=Gp1./Gm1;
    VarsOut(:, 9) = (sqrt(Grat).*atan(sqrt((C2-1)./Grat)) ...
                      -acos(1./M1))*180/pi;
  %10: Mach Angle (Degrees)
   VarsOut(:,10) = asin(1./M1)*180/pi;
  %11: Mach Number: M2
    Num = Gm1.*C2+2.0;
     Den = 2.0*Gamma.*C2-Gm1;
    VarsOut(:,11) = (Num./Den) .^ (0.5);
  %12: Static Pressure Ratio: p2/p1
   VarsOut(:,12) = (2.0*Gamma.*C2-Gm1)./Gp1;
  %13: Local Density Ratio: rho2/rho1
   VarsOut(:,13) = (Gp1.*C2)./(Gm1.*C2+2.0);
  %14: Static Temperature Ratio: T2/T1
    VarsOut(:,14) = (2.0*Gamma.*C2-Gm1).*(Gm1.*C2+2.0)./(Gp1.*Gp1.*C2);
  %15: Total Pressure Ratio:  pt2/pt1
    VarsOut(:,15) = VarsOut(:,12).^D0 .* VarsOut(:,13).^(-D1);
  %16: Pressure Ratio: p1/pt2
    VarsOut(:,16) = VarsOut(:,2) ./ VarsOut(:,15);


%Check/Correct Results
 %Return NAN if M1=nan
  ind=find(isnan(VarsOut(:,1)));
  if ~isempty(ind)
   VarsOut(ind,:)=nan*ones(size(VarsOut(ind,:)));
  end%if

 %Return NAN for Shock Calulations where M1 < 0.9999
 %(need a bit of leeway around M1=1.0)
  ind=find(M1<0.9999);
   if ~isempty(ind)
    VarsOut(ind,9:16)=nan*ones(size(VarsOut(ind,9:16)));
   end%if

 %Return real part of result for Shock Calulations where 0.9999<M1 & M1<1.00
 %(need a bit of leeway around M1=1.0)
  ind=find(0.9999<M1 & M1<1.0000);
   if ~isempty(ind)
    VarsOut(ind,9:16)=real(VarsOut(ind,9:16));
   end%if
