function ames2plt(MNmin,MNmax,Npts,Gamma);
%AMES2PLT    Normalize and plot output from function AMES2.
%	     AMES2PLT() uses function AMES2 to compute isentropic and normal
%	     shock flow properties at 250 points between Mach 0.01 and Mach 10.
%	     Each property is then scaled for the range 0.0 to 1.0 and plotted.
%	     Individual curves are labeled with a column number corresponding
%	     to the AMES2 output.
%
%	     AMES2PLT(MNmin,MNmax) plots results for a range of user specified
%	     Mach numbers where: MNmin is the minimum Mach No.; and MNmax is
%	     the maximum Mach No.
%
%	     AMES2PLT(MNmin,MNmax,Npts) in addition to allowing user specified
%	     Mach No., allows user to specify number of points per curve.
%
%	     AMES2PLT(MNmin,MNmax,Npts,Gamma) in addition to allowing user
%	     specified Mach No. and number of points per curve, allows user to
%	     specify ratio of specific heats used for calculation of flow
%	     properties.
%
%	     See also, AMES2 and AMES2ERR.

% Compressible Flow Toolbox, Version 1.1, 23-FEB-96
% Written By: Kevin J. Melcher - 12/94
%             NASA Lewis Research Center, Cleveland, Ohio
%             Cleveland State Univiersity, Cleveland, Ohio

%Check input args
 if nargin==2,
   error('INPUT ARG NOT DEFINED: ARG2 must be specified if ARG1 is specified.');
 end;
 if nargin<4,
   disp('  Note: Using Default. Gamma=1.4');
   Gamma=1.4;
 elseif Gamma<=0,
   disp('  Note: Using Default. Gamma=1.4');
   Gamma=1.4;
 end
 if nargin<3,
   Npts=250;
 end;
 if nargin<1,
   MNmin=0.01;
   MNmax=10;
 end;

%Get table values as a function of Mach Number
%Note: Mn=1 & Mn=sqrt(2) are Points of Interest
 M0(:)=[(MNmax-MNmin)*(0:Npts-3)/(Npts-3)+MNmin 1 sqrt(2)];
 M0=sort(M0);
 FM0=ames2(1,M0,Gamma);
 [n0,m0]=size(FM0);

%Delete ALL Figure Windows
 get(get(gcf,'parent'),'children');

%Setup Figures
 ss=get(0,'ScreenSize');
 figure(1); clf;
  fpos=get(1,'Position');
  dfpos=round([-0.025*ss(3) +0.025*ss(3) 0 0]);
  fpos=[ss(3)-fpos(3) 0 fpos(3:4)]+dfpos;
  set(1,'Position',fpos);
 figure(2); clf;
  set(2,'Position',fpos+dfpos);

%Generate Plot of Supersonic/Normal Shock Functions
  k=9:16; m=length(k);
  j=find(M0>=1); n=length(j);
  Fmin=min(FM0(j,k)); Fmax=max(FM0(j,k));
  Fnorm=(FM0(j,k)-ones(n,1)*Fmin)./(ones(n,1)*(Fmax-Fmin));
 plot(M0(j)*ones(1,m),Fnorm);
  xlabel('Mach No.');
  ylabel('Normalized Parameters');
  title('AMES2.M: Normal Shock Functions, Table Columns 9-16');
  dM0=(MNmax-1)/13;
  for i=1:m;
    l=max(find(dM0*(m-i)+1.5>M0(j)));
    text(M0(j(l)),Fnorm(l,i),sprintf('%d',i+k(1)-1), ...
         'HorizontalAlignment','center', ...
         'VerticalAlignment','middle');
  end

%Generate Plot of Isenstropic Functions
 figure(1); clf;
  k=2:8; m=length(k);
  Fmin=min(FM0(:,k)); Fmax=max(FM0(:,k));
  Fnorm=(FM0(:,k)-ones(n0,1)*Fmin)./(ones(n0,1)*(Fmax-Fmin));
 plot(M0*ones(1,m),Fnorm);
  xlabel('Mach No.');
  ylabel('Normalized Parameters');
  title('AMES2.M: Isentropic Functions, Table Columns 1-8');
  dM0=(MNmax-MNmin)/10;
  for i=1:size(Fnorm,2);
    j=max(find(dM0*i+MNmin>M0));
    text(M0(j),Fnorm(j,i),sprintf('%d',i+k(1)-1), ...
         'HorizontalAlignment','center', ...
         'VerticalAlignment','middle');
  end
