function VarsOut=fanno(VarNo,Value,Gamma) 
%FANNO	Compute Fanno flow functions.
%	VAROUT=FANNO(VARIN,VALUE) computes the Fanno functions specified in
%	Table A4 of "Compressible Fluid Flow" by Michel A. Saad. Here, VARIN
%	specifies the column number of a known property and VALUE (may be
%	a vector), the value of that property. Given this information, FANNO
%	solves for corresponding values of the other properties in the table
%	and returns them in VAROUT. Columns of VAROUT correspond to the columns
%	of Table A4. If the specified property is multi-valued (i.e., both
%	subsonic and supersonic solutions exist), both solutions are returned
%	with rows alternating between subsonic and supersonic solutions. For
%	these calculations, the ratio of specific heats of the fluid is assumed
%	to be 1.4, the nominal specific heat of air.
%
%	VAROUT=AMES2(VARIN,VALUE,GAMMA) allows the user to specify the value
%	of GAMMA, the specific heat of the fluid, for all elements of VALUE
%	(GAMMA is a scalar) or for each element of VALUE (GAMMA is a vector).
%
%	See also, FANNOERR and FANNOPLT.
%
%
% Definition of Columns Table A4, "Compressible Fluid Flow."
%
%  Fanno Functions
%  -----------------------------   -------------------------------------------
%  1) Mach No.                     5) V/V*   - Velocity Ratio
%  2) T/T*   - Static Temp Ratio      p*/p   - Inverse of Static Density Ratio
%  3) P/P*   - Static Pres Ratio   6) I/I*   - Implulse Function Ratio
%  4) P0/P0* - Total  Pres Ratio   7) 4fL*/D - Friction Term

%REF: "Compressible Fluid Flow," M. Saad, (c) 1993 by Prentice-Hall, pp 210-211.
%     "The Dynamic and Thermodynamics of Compressible Fluid Flow," A. Shapiro,
%      (c) 1953 by John Wiley & Sons, Inc., pp 167-168.

% Compressible Flow Toolbox, Version 1.1, 23-FEB-96
% Written By: Kevin J. Melcher - 12/94
%             NASA Lewis Research Center, Cleveland, Ohio
%             Cleveland State Univiersity, Cleveland, Ohio


%Condition the Input Arguments
 VarNo = abs(VarNo);
 Value = Value(:);
 [r,c]=size(Value); n=max(r,c);

 if ~exist('Gamma'),
   Gamma = 7/5*ones(r,c);
 elseif max(size(Gamma))==1
   Gamma = Gamma*ones(r,c);
 else
   Gamma = Gamma(:)
 end

%Calculate Results
 %Calculate Some Common Constants
  Gm1= Gamma-1;
  Gp1= Gamma+1;
  C0 = 0.5*Gm1;		%(G-1)/2
  C1 = 0.5*Gp1;		%(G+1)/2
  D0 = -0.5./C0;	%-1/(G-1)
  D1 = Gamma.*D0;	%-G/(G-1)
  D2 = C1./Gm1;		%(G+1)/(G-1)/2
  
%Default Vector of Indices
% (required because some parameters return mulitples values)
 M1=nan*ones(size(Value)); k=[]; 

%Define Upper Bound on Search Space for Non-linear Solutions of M1=F(Value)
 MnMax=100;	

%Define Name for Temporary File Used by Iterative Solutions
 Dir = lower(tempdir);  Name=lower(tempname); nd=max(size(Dir));
 FileName=sprintf('%s.m',Name);
 FuncName=Name;  FuncName(1:nd)=[];
 path(Dir,path);

%Convergence Criteria for Iterative Solutions
 opt=foptions; opt(2:3)=1e-8*ones(1,2);	%type 'help foptions'

%Calculate M1
 if VarNo==1, %from Mach No.
   M1 = Value;
 elseif VarNo ==  2, 					%from T/T*
   %Setup and solve using iterative technique
    %Generate M-file for solution
     fid=fopen(FileName,'w');
      fprintf(fid,'%s\n',['function Err =' FuncName '(Mn,Gam,TbyTstar);']);
      fprintf(fid,'%s\n',' %NOTE: Mn, Gam, & TbyTstar MUST BE SCALARS');
      fprintf(fid,'%s\n',' FMN = (Gam+1)/(2+(Gam-1)*Mn*Mn);');
      fprintf(fid,'%s\n',' Err = abs(FMN-TbyTstar);');
    fclose(fid);
    for j=1:length(Value(:));
      M1(j)=fmin(FuncName,0,MnMax,opt,Gamma(j),Value(j));
    end
 elseif VarNo ==  3, 					%from P/P*
   %Setup and solve using iterative technique
    %Generate M-file for solution
     fid=fopen(FileName,'w');
      fprintf(fid,'%s\n',['function Err =' FuncName '(Mn,Gam,PbyPstar);']);
      fprintf(fid,'%s\n',' %NOTE: Mn, Gam, & PbyPstar MUST BE SCALARS');
      fprintf(fid,'%s\n',' FMN = sqrt((Gam+1)/(2+(Gam-1)*Mn*Mn))/Mn;');
      fprintf(fid,'%s\n',' Err = abs(FMN-PbyPstar);');
    fclose(fid);
    for j=1:length(Value(:));
      M1(j)=fmin(FuncName,0,MnMax,opt,Gamma(j),Value(j));
    end
 elseif VarNo ==  4,					%from P0/P0*
   %Setup and solve using iterative technique
    %Generate M-file for solution
     fid=fopen(FileName,'w');
      fprintf(fid,'%s\n',['function Err =' FuncName '(Mn,Gam,P0byP0star);']);
      fprintf(fid,'%s\n',' %NOTE: Mn, Gam, & P0byP0star MUST BE SCALARS');
      fprintf(fid,'%s\n',' Gp1=Gam+1; Gm1=Gam-1; Gratio=0.5*Gp1/Gm1;');
      fprintf(fid,'%s\n',' FMN = (((2+Gm1*Mn*Mn)/Gp1)^Gratio)/Mn;');
      fprintf(fid,'%s\n',' Err = abs(FMN-P0byP0star);');
     fclose(fid);
    %M1=F(P0) is Double Valued
     i=0;
     for j=1:length(Value(:));
      %Subsonic Mach Number
       i=i+1; k(i)=j; M1(i)=fmin(FuncName,0,1,opt,Gamma(j),Value(j));
      %Supersonic Mach Number
       i=i+1; k(i)=j; M1(i)=fmin(FuncName,1,MnMax,opt,Gamma(j),Value(j));
     end
 elseif VarNo ==  5,					%from V/V*
   %Setup and solve using iterative technique
    %Generate M-file for solution
     fid=fopen(FileName,'w');
      fprintf(fid,'%s\n',['function Err =' FuncName '(Mn,Gam,VbyVstar);']);
      fprintf(fid,'%s\n',' %NOTE: Mn, Gam, & VbyVstar MUST BE SCALARS');
      fprintf(fid,'%s\n',' FMN = sqrt((Gam+1)/(2+(Gam-1)*Mn*Mn))*Mn;');
      fprintf(fid,'%s\n',' Err = abs(FMN-VbyVstar);');
     fclose(fid);
     for j=1:length(Value(:));
      M1(j)=fmin(FuncName,0,MnMax,opt,Gamma(j),Value(j));
     end
 elseif VarNo ==  6,					%from I/I*
   %Setup and solve using iterative technique
    %Generate M-file for solution
     fid=fopen(FileName,'w');
      fprintf(fid,'%s\n',['function Err =' FuncName '(Mn,Gam,IbyIstar);']);
      fprintf(fid,'%s\n',' %NOTE: Mn, Gam, & IbyIstar MUST BE SCALARS');
      fprintf(fid,'%s\n',' FMN = (1+Gam*Mn*Mn)/sqrt(2*(Gam+1)*(1+0.5*(Gam-1)*Mn*Mn))/Mn;');
      fprintf(fid,'%s\n',' Err = abs(FMN-IbyIstar);');
     fclose(fid);
    %M1=F(I) is Double Valued
     i=0;
     for j=1:length(Value(:));
      %Subsonic Mach Number
       i=i+1; k(i)=j; M1(i)=fmin(FuncName,0,1,opt,Gamma(j),Value(j));
      %Supersonic Mach Number
       i=i+1; k(i)=j; M1(i)=fmin(FuncName,1,MnMax,opt,Gamma(j),Value(j));
     end
 elseif VarNo ==  7,					%from 4fL*/Dh
   %Setup and solve using iterative technique
    %Generate M-file for solution
     fid=fopen(FileName,'w');
      fprintf(fid,'%s\n',['function Err =' FuncName '(Mn,Gam,f4LstarByDh);']);
      fprintf(fid,'%s\n',' %NOTE: Mn, Gam, & f4LstarByDh MUST BE SCALARS');
      fprintf(fid,'%s\n',' Gp1=0.5*(Gam+1); Gm1=0.5*(Gam-1); M2=Mn*Mn;');
      fprintf(fid,'%s\n',' FMN = (1-M2)/(Gam*M2)+(Gp1/Gam)*log(Gp1*M2/(1+Gm1*M2));');
      fprintf(fid,'%s\n',' Err = abs(FMN-f4LstarByDh);');
     fclose(fid);
    %M1=F(I) is Double Valued
     i=0;
     for j=1:length(Value(:));
      %Subsonic Mach Number
       i=i+1; k(i)=j; M1(i)=fmin(FuncName,0,1,opt,Gamma(j),Value(j));
      %Supersonic Mach Number
       i=i+1; k(i)=j; M1(i)=fmin(FuncName,1,MnMax,opt,Gamma(j),Value(j));
     end%if
 end%if

%Delete Temporary M-file
 if exist(FileName)==2, delete(FileName); end

%Calculate Some Common Constants
 M1=M1(:); k=k(:);
 if ~isempty(k), Gamma=Gamma(k); end;
  Gamma=Gamma(:);
  Gm1= Gamma-1;
  Gp1= Gamma+1;
  C0 = 0.5*Gm1;		%(G-1)/2
  C1 = 0.5*Gp1;		%(G+1)/2
  D0 = -0.5./C0;	%-1/(G-1)
  D1 = Gamma.*D0;	%-G/(G-1)
  D2 = C1./Gm1;		%(G+1)/(G-1)/2
 end;

 %Initialize Output Variable
  VarsOut = nan*ones([size(M1,1),7]);

 %Define Some more constants
  C2 = M1.*M1;		% M1^2
  C3 = C0.*C2+1.0;	%(1 + M1^2*(G-1)/2)

  %Mach Number: M1
    VarsOut(:,1) = M1;
  %Static Temperature Ratio
    VarsOut(:,2) = 0.5*Gp1./C3;
  %Static Pressure Ratio
    VarsOut(:,3) = (VarsOut(:,2).^0.5)./M1;
  %Total Pressure Ratio
    VarsOut(:,4) = (((2*C3./Gp1).^(Gp1./Gm1)).^0.5)./M1;
  %Velocity Ratio (also inverse static density ratio)
    VarsOut(:,5) = M1./(2*C3./Gp1).^0.5;
  %Impulse Function Ratio
    VarsOut(:,6) = M1.*(2*Gp1.*C3).^0.5;
    VarsOut(:,6) = (1+Gamma.*C2)./VarsOut(:,6);
  %4fL*/Dh
   FM1a=(1-C2)./(Gamma.*C2);
   FM1b=C1./Gamma;
   FM1c=0.5*Gp1.*C2./C3;
   FM1d=log(FM1c);
    i=find(M1==0);
    if ~isempty(i),
     FM1d(i)=inf*ones(length(i),1);
    end
   VarsOut(:,7) = FM1a+FM1b.*FM1d;

