function [ThetaW,ThetaS,DELmax,DELson,M1,delta]=oblqshck(M1,delta,gamma);
%OBLQSHCK	Compute weak & strong oblique shock angles.
%		[THETAW,THETAS]=OBLQSHCK(M1,DELTA,GAMMA) computes the weak     
%		shock angle, THETAW, and the strong shock angle, THETAS, that
%		result from compressing a Mach M1 flow by turning it DELTA
%		degrees. GAMMA is an optional input representing the specific
%		heat of the fluid. The specific heat of air, GAMMA=1.4, is used
%		if GAMMA is not specified by the user.
%
%		[THETAW,THETAS,DELTAMAX,DELTASON]=OBLQSHCK(M1,DELTA,GAMMA)
%		also returns the the maximum angle through which the flow can
%		turn without separating and the turning angle that results in
%		a sonic condition downstream of the shock.
%
%		OBLQSHCK() by itself generates a plot showing shock angle vs.
%		deflection angle for lines of constant Mach from 1 to 20. This
%		plot is a representation of Chart 2 in the NACA Report 1135.
%
%		Also see DELTAMAX, DELTASON, OBLQW12, OBLQW21

%REF: Ames Research Staff: Equations, Tables, and Charts for Compressible Flow,
%     NACA Report 1135, 1953.

% Compressible Flow Toolbox, Version 1.1, 23-FEB-96
% Written By: Kevin J. Melcher - 12/94
%             NASA Lewis Research Center, Cleveland, Ohio
%             Cleveland State Univiersity, Cleveland, Ohio


%function [ThetaW,ThetaS,DELmax,DELson]=oblqshck(M1,delta,gamma);
%
%From:
%  M1    ... Mach Number upstream of oblique
%  delta ... deflection angle (degrees)
%  gamma ... ratio of specific heats (def 1.4)
%
%Computes (in degrees):
%  ThetaW ... weak shock angle associated with M1 & delta
%  ThetaS ... strong shock angle associated with M1 & delta
%  DELmax ... maximum deflection angle associated with M1
%  DELson ... deflection angle associated with M1 which yields
%                sonic flow
%
%  NOTE: Also see DELTAMAX, DELTASON, OBLQW12 and OBLQW21
%

% Written By: Kevin J. Melcher
%             NASA Lewis Research Center
%             21000 Brookpark Road
%             Cleveland, OH 44039
%
% Original Version - 3/25/94 KJM

%Ref: "Equations, Tables, and Charts for Compressible Flow";
%     Ames Research Staff, NACA Report 1135, 1953, pp. 42-43.

%Ref: Hartley, T.T.; et. al.; "Exact and Approximate Solutions to
%     the Oblique Shock Equations for REal-Time Applications";
%     NASA CR-187173, August 1991.

%Test function
 if nargin==0,
   M1=[0.8 1:0.05:1.5 1.6:.1:2 2.2:.2:4 4.5 5 6 8 10 20 500];
   N=100; M=length(M1);
   [DELmax,THmax]=deltamax(M1);
   delta=[-1:N-2]'/(N-3)*DELmax(:)';
   M1=ones(N,1)*M1;
 end %if

%Size of M1
 [N,M]=size(M1);

%Default for gamma
 if nargin<3, gamma=1.4*ones(N,M); end

%Arg Size Check
 if N~=size(delta,1) | N~=size(gamma,1) ...
    | M~=size(delta,2) | M~=size(gamma,2),
   whos, error('Size of input args are not equal.')
 end %if

%Initialize Output Variables
 ThetaW=nan*ones(N,M);
 ThetaS=ThetaW;
 [DELmax,THmax]=deltamax(M1,gamma);
 i=find(1<=M1 & 0<=delta & delta<=DELmax);
 if isempty(i), return, end
 
%Solve Cubic Equation for Shock Angle Theta
 %General Computations
  delta=delta*pi/180;
  b=-((M1(i).^2+2)./M1(i).^2+gamma(i).*(sin(delta(i))).^2);
  c=(2.*M1(i).^2+1)./M1(i).^4 ...
      +(((gamma(i)+1).^2)./4+(gamma(i)-1)./M1(i).^2).*(sin(delta(i))).^2;
  d=-((cos(delta(i))).^2)./M1(i).^4;
  Q=(3.*c-b.^2)./9;
  R=(9.*b.*c-27.*d-2.*b.^3)./54;
  D=Q.^3+R.^2;
  del=zeros(size(i));
   j=find(R<0);
   if ~isempty(j), del(j)=pi*ones(size(j)); end
  phi=(atan(sqrt(-D)./R)+del)./3;
 %Weak shock Solution
  Xw=-sqrt(-Q).*(cos(phi)-sqrt(3).*sin(phi))-b/3;
  ThetaW(i)=atan(sqrt(Xw./(1-Xw)));
  % 1) "Handle" numerical errors in the complex plane by
  %    taking absolute value.
  % 2) Change radians to degrees
   ThetaW(i)=abs(ThetaW(i))*180/pi;
 %Strong Shock Solution
  if nargin==0 | nargout>=2,
   Xs=2.*sqrt(-Q).*cos(phi)-b/3;
   ThetaS(i)=atan(sqrt(Xs./(1-Xs)));
   % 1) "Handle" numerical errors in the complex plane by
   %    taking absolute value.
   % 2) Change radians to degrees
    ThetaS(i)=abs(ThetaS(i))*180/pi;
    j=find(90<ThetaS & ThetaS-90<1.0E-6);
    ThetaS(j)=90*ones(size(j));
  end
 %Sonic Deflection Angle
  if nargin==0 | nargout>=4,
   [DELson,THson]=deltason(M1,gamma);
  end


%Plot Results of Test Case
if nargin==0;
  clf; hold off;
  delta=delta*180/pi;
  plot(DELmax(1,:),THmax(1,:), 'g-', ...
       DELson(1,:),THson(1,:),'c-.'  );
  for k=1:M;
    iw=find(~isnan(ThetaW(:,k)));
    is=find(~isnan(ThetaS(:,k)));
    hold on;
%     plot([delta(is,k); DELmax(1,k); flipud(delta(iw,k))], ...
%          [ThetaS(is,k);THmax(1,k);  flipud(ThetaW(iw,k))], ...
%          'y');
     plot([delta(is,k); DELmax(1,k)],[ThetaS(is,k);THmax(1,k)],'y:', ...
	   flipud(delta(iw,k)), flipud(ThetaW(iw,k)),'y-');
    hold off;
  end
  title('OBLQSHCK: NACA Report 1135, Chart 2');
  a=axis; dx=a(2)-a(1); dy=a(4)-a(3); ax=gca;
  %Label X-axis
   xlabel('Deflection Angle (deg) ');
   ylabel('Oblique Shock Angle (deg) ');
  figure(gcf);
  ThetaW=[];
end
