function [Yhat,PI]=ifvalid(NetDef,nx,W1,W2,obsidx,Y,U)
%  IFVALID
%  -------
%          Validate a neural network based state space innovations form model
%          of a (possibly multivariable) dynamic system. I.e. a network model
%          generated by the function NNSSIF.
%
%          The following plots are produced: 
%          - Output(s) together with predicted output(s)
%          - Prediction error
%          - Auto correlation function of prediction error and cross
%            correlation between prediction error(s) and input(s)
%          - Histogram(s) showing the distribution of the prediction errors
%          - Extracted linear coefficients 
%
%           Call: 
%           [Yhat,PI] = ifvalid(NetDef,nx,W1,W2,obsidx,Y,U)               
%
%  Programmed by : Magnus Norgaard, IAU/EI/IMM
%  LastEditDate  : Sep 8, 1995  



% >>>>>>>>>>>>>>>>>>>>>>>>>>>>     INITIALIZATIONS     <<<<<<<<<<<<<<<<<<<<<<<<<<<< 
skip=1;
[ny,Ndat]= size(Y);                     % # of outputs and # of data
[nu,Ndat]= size(U);                     % # of inputs 
N        = Ndat - 1;                    % Size of complete training set
N2       = N-skip+1;                    % "Actual" size of training set
hidden   = length(NetDef(1,:));         % Number of hidden neurons
inputs   = nx+nu+ny;                    % Number of inputs to the network
outputs  = ny;                          % Network outputs equal to the number of states 
L_hidden = find(NetDef(1,:)=='L')';     % Location of linear hidden neurons
H_hidden = find(NetDef(1,:)=='H')';     % Location of tanh hidden neurons
L_output = find(NetDef(2,:)=='L')';     % Location of linear output neurons
H_output = find(NetDef(2,:)=='H')';     % Location of tanh output neurons
y1       = zeros(hidden,N);             % Hidden layer outputs
y1       = [y1;ones(1,N)];
y2       = zeros(outputs,N);            % Network output
Yhat     = zeros(ny,N);                 % Initialize prediction matrix
E        = zeros(ny,N);                 % Initialize prediction error matrix
E_new    = zeros(ny,N);                 % Initialize prediction error matrix
obsidx=obsidx(:)';                      % Find row indices
rowidx=obsidx;
for k=2:ny,
  rowidx(k)=obsidx(k)+rowidx(k-1);
end
nrowidx = 1:nx;                         % Not row indices
nrowidx(rowidx)=[];
Cidx=[1 rowidx(1:ny-1)+1];
C = zeros(ny,nx);
C(1:ny,Cidx)=eye(ny);
NNy      = N*ny;
skipidx  = ny*(skip-1)+1:NNy;



% >>>>>>>>>>>>>>>>>>>>  CONSTRUCT THE REGRESSION MATRIX PHI   <<<<<<<<<<<<<<<<<<<<<
PHI = zeros(inputs,N);
PHI(nx+1:nx+nu,:) = U(:,1:N);
PHI_aug = [PHI;ones(1,N)];                % Augment PHI with a row containing ones
Y       = Y(:,2:Ndat);                    % Extract the 'target' part of Y



% >>>>>>>>>>>>>>>>>>>>>>>>>>   COMPUTE NETWORK OUTPUT   <<<<<<<<<<<<<<<<<<<<<<<<<<<
for t=1:N,
  h1 = W1*PHI_aug(:,t);                 % Hidden neuron outputs
  y1(H_hidden,t) = pmntanh(h1(H_hidden));
  y1(L_hidden,t) = h1(L_hidden);    

  h2 = W2*y1(:,t);                      % Predicted states
  y2(H_output,t) = pmntanh(h2(H_output,:));
  y2(L_output,t) = h2(L_output,:);
  y2(nrowidx,t)  = y2(nrowidx,t) + PHI_aug(nrowidx+1,t);
  Yhat(:,t)      = C*y2(:,t);

  E(:,t) = Y(:,t) - Yhat(:,t);          % Prediction error
  Evec   = E(:);
  for d=1:min(1,N-t),
    PHI_aug(1:nx,t+1) = y2(:,t);
    PHI_aug(nx+nu+1:inputs,t+1) = E(:,t);
  end
end
SSE=Evec(skipidx)'*Evec(skipidx);       % Sum of squared errors (SSE)
PI       = SSE/(2*N2);                  % Performance index



% >>>>>>>>>>>>>>>>>>>>>>>>>>      PLOT THE RESULTS      <<<<<<<<<<<<<<<<<<<<<<<<<<<
si = figure-1;

% ---------- Output, Prediction and Prediction error ----------
for i=1:outputs
    figure(si+i)
    subplot(211)
    plot(Y(i,:),'y-'); hold on
    plot(Yhat(i,:),'m--');hold off
    xlabel('time (samples)')
    if outputs==1,
      title('Output (solid) and one-step ahead prediction (dashed)')
    else
      title([' Output (solid) and one-step ahead prediction (dashed) (output # ',num2str(i) ')'])
    end
    grid

    subplot(212)
    plot(E(i,:)');
    title('Prediction error (y-yhat)')
    xlabel('time (samples)')
    grid
    subplot(111)
    drawnow
end


% --------- Correlation functions ----------
for ii=1:outputs,
  figure(si+outputs+ii)
  eval(['subplot(' num2str(nu+1) '11)']);
  M=min(25,N-1);
  Eauto=xcorr(E(ii,:),'coeff');
  Eauto=Eauto(N:2*N-1);
  conf=1.96/sqrt(N);
  plot([0:M],Eauto(1:M+1),'y-'); hold on
  plot([0 M],[conf -conf;conf -conf],'m--');hold off
  xlabel('lag')
  title(['Auto correlation function of prediction error #' num2str(ii)])
  grid
  Ecov=cov(E(ii,:));

  for i=1:nu,
    eval(['subplot(' num2str(nu+1) '1' num2str(i+1) ')']);
    Ucov=cov(U(i:N));
    UEcross=xcov(E(ii,:),U(i,1:N),'unbiased')/sqrt(Ecov*Ucov)';
    plot([-M:M], UEcross(N-M:N+M),'y-'); hold on
    plot([-M M],[conf -conf;conf -conf],'m--');hold off
    xlabel('lag')
    title(['Cross correlation fct of control #' num2str(i)  ...
                               ' and prediction error #' num2str(ii)])
    ymax=min(5*conf,max([abs(UEcross)]));
    axis([-M M -ymax ymax]);
    grid
  end
  subplot(111)
  drawnow
end

% --------- Histograms over the prediction errors ---------
figure(si+2*outputs+1)
for ii=1:outputs,
  eval(['subplot(' num2str(outputs) '1' num2str(ii) ')']);
  hist(E(ii,:),20)
  title(['Histogram over prediction errors on output #' num2str(ii)])
end
subplot(111)

% ---------- Extract linear model from network ----------
    % ---------- Find derivative of output wrt. the past residuals ----------
    Amat=zeros(ny*nx,N);
    Kmat=zeros(nx*ny,N);
    Bmat=zeros(nx*nu,N);
    for t=1:N,
      dxdy1 = W2(:,1:hidden);
      for j = H_output',
        dxdy1(j,:) = W2(j,1:hidden)*(1-y2(j,t).*y2(j,t));
      end

      % Matrix with partial derivatives of the output from each hidden neurons with
      % respect to each input:
      dy1dx = W1(:,1:nx);
      dy1de = W1(:,nx+nu+1:inputs);
      dy1du = W1(:,nx+1:nx+nu);
      for j = H_hidden',
        dy1dx(j,:) = W1(j,1:nx)*(1-y1(j,t).*y1(j,t));
        dy1de(j,:) = W1(j,nx+nu+1:inputs)*(1-y1(j,t).*y1(j,t));
        dy1du(j,:) = W1(j,nx+1:nx+nu)*(1-y1(j,t).*y1(j,t));
      end

      % Matrix with partial derivative of each output with respect to each input
      Ahat = dxdy1 * dy1dx;
      Acoef = Ahat(rowidx,:);
      Bhat = dxdy1 * dy1du;
      Khat = dxdy1 * dy1de;
      Amat(:,t) = Acoef(:);
      Bmat(:,t) = Bhat(:);
      Kmat(:,t) = Khat(:);
    end

  figure(si+2*outputs+2)
  subplot(311)
  plot(Amat')
  title('Extracted A matrix coefficients')
  grid
  
  subplot(312)
  plot(Bmat')
  title('Extracted B matrix coefficients')
  grid
  
  subplot(313)
  plot(Kmat')
  title('Extracted K matrix coefficients')
  xlabel('time (samples)')
  grid
  subplot(111)
figure(si+1)

