function [W1,W2,PI_vector,iter]=marq2(NetDef,W1,W2,PHI,Y,trparms)
%  MARQ2
%  -----
%          Train a two layer neural network by using the Levenberg-Marquardt
%          method. Works exactly like the function "marq" except that only
%          fully connected networks can be trained.
%
%  Programmed by : Magnus Norgaard, IAU/EI/IMM
%  LastEditDate  : June 15, 1994


%----------------------------------------------------------------------------------
%--------------             NETWORK INITIALIZATIONS                   -------------
%----------------------------------------------------------------------------------
max_iter = trparms(1);
stop_crit= trparms(2);
lambda   = trparms(3);

[outputs,N] = size(Y);                  % # of outputs and # of data
[hidden,inputs] = size(W1);             % # of hidden units 
inputs=inputs-1;                        % # of inputs
L_hidden = find(NetDef(1,:)=='L')';     % Location of linear hidden units
H_hidden = find(NetDef(1,:)=='H')';     % Location of tanh hidden units
L_output = find(NetDef(2,:)=='L')';     % Location of linear output units
H_output = find(NetDef(2,:)=='H')';     % Location of tanh output units
y1       = zeros(hidden,N);             % Hidden layer outputs
y2       = zeros(outputs,N);            % Network output
index = outputs*(hidden+1) + 1 + [0:hidden-1]*(inputs+1); % A usefull vector!
index2 = (0:N-1)*outputs;               % Yet another usefull vector
dw       = 1;                           % Flag telling that the weights are new
PHI_aug  = [PHI;ones(1,N)];             % Augment PHI with a row containg ones
parameters1= hidden*(inputs+1);         % # of input-to-hidden weights
parameters2= outputs*(hidden+1);        % # of hidden-to-output weights
parameters = parameters1 + parameters2; % Total # of weights
PSI      = zeros(parameters,outputs*N); % Derivative of each output w.r.t. each weight
identity = eye(parameters);             % Identity matrix
ones_h   = ones(hidden+1,1);            % A vector of ones
ones_i   = ones(inputs+1,1);            % Another vector of ones
                                        % Create parametervector containing all weights
theta = [reshape(W2',parameters2,1) ; reshape(W1',parameters1,1)];
PI_vector = zeros(max_iter,1);          % A vector containing the accumulated SSE


%---------------------------------------------------------------------------------
%-------------                   TRAIN NETWORK                       -------------
%---------------------------------------------------------------------------------
clc;
c=fix(clock);
fprintf('Network training started at %2i.%2i.%2i\n\n',c(4),c(5),c(6));


% >>>>>>>>>>>>>>>>>>>>>  COMPUTE NETWORK OUTPUT  y2(theta)   <<<<<<<<<<<<<<<<<<<<<   
h1 = W1*PHI_aug;  
y1(H_hidden,:) = pmntanh(h1(H_hidden,:));
y1(L_hidden,:) = h1(L_hidden,:);    
y1_aug=[y1; ones(1,N)];

h2 = W2*y1_aug;
y2(H_output,:) = pmntanh(h2(H_output,:));
y2(L_output,:) = h2(L_output,:);

E        = Y - y2;                      % Training error
E_vector = E(:);                        % Reshape E into a long vector
SSE      = E_vector'*E_vector;          % Sum of squared errors (SSE)

for iteration=1:max_iter,
    
  if dw==1,
% >>>>>>>>>>>>>>>>>>>>>>>>>>>  COMPUTE THE PSI MATRIX  <<<<<<<<<<<<<<<<<<<<<<<<<<  
% (The derivative of each network output (y2) with respect to each weight)

    % ---------   Elements corresponding to the linear output units   -----------
    for i = L_output'
      index1 = (i-1) * (hidden + 1) + 1;

      % -- The part of PSI corresponding to hidden-to-output layer weights --
      PSI(index1:index1+hidden,index2+i) = y1_aug;
      % ---------------------------------------------------------------------
 
      % -- The part of PSI corresponding to input-to-hidden layer weights ---
      for j = L_hidden',
        PSI(index(j):index(j)+inputs,index2+i) = W2(i,j)*PHI_aug;
      end
      
      for j = H_hidden',
        tmp = W2(i,j)*(1-y1(j,:).*y1(j,:)); 
        PSI(index(j):index(j)+inputs,index2+i) = tmp(ones_i,:).*PHI_aug;
      end 
      % ---------------------------------------------------------------------    
    end

    % ----- Elements corresponding to the hyperbolic tangent output units   -----  
    for i = H_output',
      index1 = (i-1) * (hidden + 1) + 1;

      % -- The part of PSI corresponding to hidden-to-output layer weights --
      tmp = 1 - y2(i,:).*y2(i,:);
      PSI(index1:index1+hidden,index2+i) = y1_aug.*tmp(ones_h,:);
      % ---------------------------------------------------------------------
         
      % -- The part of PSI corresponding to input-to-hidden layer weights ---
      for j = L_hidden',
        tmp = W2(i,j)*(1-y2(i,:).*y2(i,:));
        PSI(index(j):index(j)+inputs,index2+i) = tmp(ones_i,:).* PHI_aug;
      end
      
      for j = H_hidden',
        tmp  = W2(i,j)*(1-y1(j,:).*y1(j,:));
        tmp2 = (1-y2(i,:).*y2(i,:));
        PSI(index(j):index(j)+inputs,index2+i) = tmp(ones_i,:)...
                                                  .*tmp2(ones_i,:).* PHI_aug;
      end
      % ---------------------------------------------------------------------
    end
    dw = 0;
  end
   
% >>>>>>>>>>>>>>>>>>>>>>>>>>>        COMPUTE h_k        <<<<<<<<<<<<<<<<<<<<<<<<<< 
    h = inv(PSI*PSI' + lambda*identity)*PSI*E_vector;
    theta_new = theta + h;
 
    % -- Put the parameters back into the weight matrices --
    W1_new = reshape(theta_new(parameters2+1:parameters),inputs+1,hidden)';
    W2_new = reshape(theta_new(1:parameters2),hidden+1,outputs)';
 
    
% >>>>>>>>>>>>>>>>>>>>  COMPUTE NETWORK OUTPUT  y2(theta+h)   <<<<<<<<<<<<<<<<<<<<   
    h1 = W1_new*PHI_aug;  
    y1(H_hidden,:) = pmntanh(h1(H_hidden,:));
    y1(L_hidden,:) = h1(L_hidden,:);
    y1_aug=[y1;ones(1,N)];
    
    h2 = W2_new*y1_aug;
    y2(H_output,:) = pmntanh(h2(H_output,:));
    y2(L_output,:) = h2(L_output,:);

    E_new        = Y - y2;                 % Training error
    E_new_vector = E_new(:);               % Reshape E into a long vector
    SSE_new  = E_new_vector'*E_new_vector; % Sum of squared errors (SSE)
    

% >>>>>>>>>>>>>>>>>>>>>>>>>>>      UPDATE  lambda     <<<<<<<<<<<<<<<<<<<<<<<<<<<<
  L = h'*PSI*E_vector + lambda*(h'*h);
  % Decrease lambda if SSE has fallen 'sufficiently'
  if (SSE - SSE_new) > (0.75*L),
     lambda = lambda/2;
  end
  % Increase lambda if SSE has grown 'sufficiently'
  if (SSE-SSE_new) <= (0.25*L),
    lambda = 2*lambda;
  end

 
%>>>>>>>>>>>>>>>>>>>>       UPDATES FOR NEXT ITERATION       <<<<<<<<<<<<<<<<<<<<<
  if SSE_new < SSE,                      % Update only if criterion has decreased
    W1 = W1_new;
    W2 = W2_new;
    theta = theta_new;
    E_vector = E_new_vector;
    SSE = SSE_new;
    dw = 1;
  end
 
  PI = SSE/(2*N);
  PI_vector(iteration) = PI;             % Collect PI in vector

  fprintf('iteration # %i   PI = %4.3e\r',iteration,PI); % Print on-line inform.
  if PI < stop_crit, break, end          % Check if stop condition is fullfilled
end
%---------------------------------------------------------------------------------
%-------------              END OF NETWORK TRAINING                  -------------
%---------------------------------------------------------------------------------
PI_vector = PI_vector(1:iteration-1);
c=fix(clock);
fprintf('\n\nNetwork training ended at %2i.%2i.%2i\n',c(4),c(5),c(6));
