function [dw,db] = learnqp(dw,db,w_slps,old_w_slps,b_slps,old_b_slps,tp)
% LEARNQP QuickPropagation learning rule.
%
%     [dW,dB] = learnqp(nrows, ncols, dw, db, w_slps, old_w_slps, b_slps, old_b_slps, tp)
%       nrows        - number of rows in weight matrix and bias matrix
%       ncols        - number of cols in weight matrix
%       dw           - old weight change matrix
%       db           - old bias change matrix
%       w_slps       - weight_slopes, weight error derivatives
%       old_w_slps   - weight error derivatives from last cycle
%       b_slps       - bias_slopes, bias error derivatives
%       old_b_slps   - bias error derivatives from last cycle
%       tp           - training parameters
%     Returns:
%       dW - a weight change matrix.
%       db - a bias   change matrix
%
%     Ray Byler, 24 Nov 1995, byler@cs.utk.edu.

if nargin ~= 7, error('Wrong number of arguments.'), end

% READ TRAINING VARIABLES
df            = tp(1);  % display frequency
me            = tp(2);  % max epochs to train
eg            = tp(3);  % error goal, SSE
lr            = tp(4);  % learning rate - NOT used by quickprop right now
decay         = tp(5);  % weight decay factor
split_epsilon = tp(6);  % 1 => divide epsilon by fan-in before use
epsilon       = tp(7);  % for gradient descent if last step was (almost) 0
max_factor    = tp(8);  % don't jump more than this times last step
mode_thresh   = tp(9);  % inside thresh, do grad descent; outside, jump
mc            = tp(10); % normal momentum term used in bpm
sig_prime_off = tp(11); % offset to sigmoid prime to kill flat spots
hyper_err     = tp(12); % 1 => atanh error function

[w_rows,w_cols] = size(dw);
[b_rows,b_cols] = size(db);

shrink_factor = max_factor ./  (1.0 + max_factor);

% determine smallest step increment
if split_epsilon == 1
  epsilon = epsilon ./ (w_cols + b_cols);
end

% LEARNING PHASE - calculate weight changes
% Unlike C, Matlab begins indexing with 1, not 0
% and Matlab handles biases separately from weights.

% We have to use "for" loops, not matrices, since each
% weight must be handled on an individual basis.

for j=1:w_rows
  for k=1:w_cols

    % for weights
    next_step = 0.0;
    % if outside threshold for gradient descent

    if dw(j,k) > mode_thresh | dw(j,k) < -mode_thresh
      % Last step was significantly positive

      if (dw(j,k) > mode_thresh & w_slps(j,k) > 0.0) | (dw(j,k) < -mode_thresh & w_slps(j,k) < 0.0)
        %  Add epsilon to next_step if slope positive
        next_step = next_step + ( epsilon .* w_slps(j,k));
      end
      % If slope > (or close to) prev slope, take max size step

      if (dw(j,k) > mode_thresh & w_slps(j,k) > shrink_factor .* old_w_slps(j,k)) | (dw(j,k) < -mode_thresh & w_slps(j,k) < shrink_factor .* old_w_slps(j,k))
        next_step = next_step + ( max_factor .* dw(j,k) );
      else
        % Use quadratic estimate
        next_step = next_step + ( (w_slps(j,k) ./ (old_w_slps(j,k) - w_slps(j,k))) .* dw(j,k) );
      end

    else
      % do normal gradient descent with momentum
      next_step = next_step + (epsilon .* w_slps(j,k)) + (mc .* dw(j,k));
    end
    dw(j,k) = next_step;
  end
end

% b_rows can be either 0, no bias, or 1
for j=1:b_rows
    % for biases - only one column
    next_step = 0.0;
    % if outside threshold for gradient descent
    if ( db(j,1) > mode_thresh | db(j,1) < -mode_thresh )
      % Last step was significantly positive

      if (db(j,1) > mode_thresh & b_slps(j,1) > 0.0) | (db(j,1) < -mode_thresh & b_slps(j,1) < 0.0)
        %  Add epsilon to next_step if slope positive
        next_step = next_step + ( epsilon .* b_slps(j,1));
      end
      % If slope > (or close to) prev slope, take max size step

      if (db(j,1) > mode_thresh & b_slps(j,1) > shrink_factor .* old_b_slps(j,1)) | (db(j,1) < -mode_thresh & b_slps(j,1) < shrink_factor .* old_b_slps(j,1))
        next_step = next_step + ( max_factor .* db(j,1) );
      else
        % Use quadratic estimate
        next_step = next_step + ( (b_slps(j,1) ./ (old_b_slps(j,1) - b_slps(j,1))) .* db(j,1) );
      end
    else
      % do normal gradient descent with momentum
      next_step = next_step + (epsilon .* b_slps(j,1)) + (mc .* db(j,1));
    end
    db(j,1) = next_step;
end


