function [w1,b1,te,tr]=tqp1(w1,b1,f1,p,t,tp)
% TQP1 Train 1-layer feed-forward network w/QuickPropagation.
%
%   [W1,B1,TE,TR] = TQP2(W1,B1,F1,P,T,TP)
%     Wi - SixR weight matrix of ith layer.
%     Bi - Six1 bias vector of ith layer.
%     F  - Transfer function (string) of ith layer.
%     P  - RxQ matrix of input vectors.
%     T  - S2xQ matrix of target vectors.
%     TP - Training parameters (optional).
%   Returns:
%     Wi - new weights.
%     Bi - new biases.
%     TE - the actual number of epochs trained.
%     TR - training record: [row of errors]
%
%   Training parameters are:
%     TP(1)  - df, display frequency, default = 1,
%              epochs between updating display
%     TP(2)  - me, max epochs, default = 1000,
%              maximum number of epochs to train
%     TP(3)  - eg, error goal, default = 0.2,
%              sum-squared error goal
%     TP(4)  - lr, learning rate, default = 0.01, NOT USED AT PRESENT.
%     TP(5)  - decay, default = -0.0001, weight decay factor
%     TP(6)  - split_epsilon, default = 0,
%              1 => divide epsilon by fan-in before use
%     TP(7)  - epsilon, default = 0.00012,
%              for gradient descent if last step was (almost) 0
%     TP(8)  - max_factor, default = 1.75,
%              don't jump more than this times last step
%     TP(9)  - mode_thresh, default = 0.0001,
%              inside threshold, do gradient descent; outside, jump
%     TP(10) - momentum, default = 0.9,
%              normal momentum term used in bpm
%     TP(11) - sig_prime_off, sigmoid_prime_offset, default 0.1
%              add to sigmoid to kill flat spots
%     TP(12) - hyper_err, error function type, default = 1.0
%              1 => use atanh error function
%
%   Missing parameters and NaN's are replaced with defaults.
%
%     QUICKPROP IS SENSITIVE TO CHANGES IN EPSILON, MAX_FACTOR,
%     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%     AND MODE_THRESH.
%     ~~~~~~~~~~~~~~~
%     If these values are too large the algorithm may thrash about
%     in an unstable manner.
%
%     Ray Byler, 24 Nov 1995, byler@cs.utk.edu.
%
%     This is Scott Fahlman's quickprop program  which  was  translated
%     from  Common  Lisp  into  C  by Terry Regier at the University of
%     California, Berkeley.   email:  regier@cogsci.berkeley.edu,  then
%     translated   into   Matlab  by  Ray  Byler,  24  Nov  95,  email:
%     byler@cs.utk.edu.
%
%     One weakness of all  Matlab  nnet  training  algorithms  is  that
%     connections  are  only  allowed between adjacent layers. The Lisp
%     and  C  versions   of   Quickprop   allow   connections   between
%     non-adjacent  layers. Moreover, the C version is compiled, hence,
%     faster.  However, the C and Lisp versions of quickprop only allow
%     one type of transfer function to  be  used  throughout  the  net,
%     normally hardcoded to ASIGMOID type, whereas Matlab allows you to
%     specify a different transfer function for each layer.
%
%     The  algorithm  and  some test results are described in Fahlman's
%     paper,  "Faster-Learning  Variations  on   Back-Propagation:   An
%     Empirical  Study"  in  Proceedings  of  1988 Connectionist Models
%     Summer School, published by Morgan Kaufmann.
%
%     Note:  the  parameter  called  "mu"  in  the  paper   is   called
%     "max_factor" here.

if nargin < 5, error('Not enough arguments.'), end

% TRAINING PARAMETERS
if nargin == 5, tp = []; end

% DEFAULT TRAINING VARIABLES
% my defaults
%                df me     eg  lr    decay    se   eps      max   mode    mom  so   hy
  tp = nndef(tp,[1  1000  0.2  0.01  -0.0001  0.0  0.00012  1.75  0.0001  0.9  0.1  1.0]);
% Fahlman's defaults
% tp = nndef(tp,[1  1000  0.2  0.01  -0.0001  1.0  0.55     1.75  0.0     0.9  0.1  1.0]);

% SET UP TRAINING VARIABLES
df            = tp(1);  % display frequency
me            = tp(2);  % max epochs to train
eg            = tp(3);  % error goal, SSE
lr            = tp(4);  % learning rate - NOT used by quickprop right now
decay         = tp(5);  % weight decay factor
split_epsilon = tp(6);  % 1 => divide epsilon by fan-in before use
epsilon       = tp(7);  % for gradient descent if last step was (almost) 0
max_factor    = tp(8);  % don't jump more than this times last step
mode_thresh   = tp(9);  % inside thresh, do grad descent; outside, jump
momentum      = tp(10); % normal momentum term used in bpm
sig_prime_off = tp(11); % offset to sigmoid prime to kill flat spots
hyper_err     = tp(12); % 1 => atanh error function

df1 = feval(f1,'delta');

% QUICKPROP UNIQUE SETUP
% use zero slopes & deltas to jumpstart quickprop
w1_slopes = 0 .* w1;
b1_slopes = 0 .* b1;

% this is the same way that bpx initializes dw's to zeros
dw1 = 0 .* w1;
db1 = 0 .* b1;

% PRESENTATION PHASE
a1 = feval(f1,w1*p,b1);
e1 = t - a1;
SSE = sumsqr(e1);

% PLOTTING FLAG
[r,q] = size(p);
[s2,q] = size(t);
plottype = max(r,s2) == 1;

% TRAINING RECORD
tr = zeros(1,me);
tr(1) = SSE;

% PLOTTING
clg;
message = sprintf('TRAINQP: %%g/%g epochs, SSE = %%g.\n',me);
fprintf(message,0,SSE);
if plottype
  h = plotfa(p,t,p,a1);
else
  h = ploterr(tr(1),eg);
end

% TRAINING
for i=1:me
  % CHECK PHASE
  if SSE < eg, i=i-1; break, end

  % save old slopes for each training epoch
  old_w1_slopes = w1_slopes;
  old_b1_slopes = b1_slopes;

  % create base for new slopes
  w1_slopes = decay*w1;
  b1_slopes = decay*b1;

  % calculate error vector
  e = cal_err(t,a1,hyper_err);

  % BACKPROPAGATION PHASE
  if strcmp(f1,'sigmoid') | strcmp(f1,'asigmoid')
    d1 = feval(df1,sig_prime_off, a1, e);
  else
    d1 = feval(df1,a1,e);
  end

  % LEARNING PHASE
  % calculate changes to new slopes
  w1_slopes = w1_slopes + d1 * p';
  b1_slopes = b1_slopes + d1 * ones(q,1);

  % calculate weight and bias changes
  [dw1,db1] = learnqp(dw1,db1,w1_slopes,old_w1_slopes,b1_slopes,old_b1_slopes,tp);

  % update weights and biases
  w1 = w1 + dw1; b1 = b1 + db1;

  % PRESENTATION PHASE
  a1 = feval(f1,w1*p,b1);
  e1 = t - a1;
  SSE = sumsqr(e1);

  % TRAINING RECORD
  tr(i+1) = SSE;

  % PLOTTING
  if rem(i,df) == 0
  fprintf(message,i,SSE)
    if plottype
      delete(h);
      h = plot(p,a1);
      drawnow;
    else
      h = ploterr(tr(1:(i+1)),eg,h);
    end
  end
end % end epoch loop

% TRAINING RECORD
tr = tr(1:(i+1));

% PLOTTING
if rem(i,df) ~= 0
  fprintf(message,i,SSE);
  if plottype
    delete(h);
    plot(p,a1);
    drawnow;
  else
    ploterr(tr,eg,h);
  end
end

% WARNINGS
if SSE > eg
  disp(' ')
  disp('TRAINQP: Network error did not reach the error goal.')
  disp('  Further training may be necessary, or try different')
  disp('  initial weights and biases and/or more hidden neurons,')
  disp('  OR YOU MAY NEED TO ADJUST QUICKPROP TRAINING PARAMETERS.')
  disp(' ')
end



