function [h,x] = setconst(J,Jgrad,xlb,xub,Jmin,x0,set,options);
% function [h,x] = setconst(J,Jgrad,xlb,xub,Jmin,x0,set,options);
%
% Solves constrained minimization problems where some of the variables are
% restricted to discrete values.  
%
% J 		name of routine for function value calculation (see CONSTR)
% Jgrad 	name of routine for gradient calculation (see CONSTR)
% xlb, xub 	lower and upper bounds for x
% Jmin 		objective value of best feasible solution found so far
% x0		initial value for x
% set		matrix of allowed discrete values.  First column
%		contains the number of discrete values for each component of x
%		(0 means continous).  The other elements of each row
%		is a sorted list of allowed discrete values.
%
%		Example: for x(1) in {0.1, 0.5, 0.8}, x(2) free and 
%			x(3) in {3.1, 4.0} then 
%			set=[3 0.1 0.5 0.8; 0 0 0 0; 2 3.1 4 0] 
%
% options	see CONSTR.
%
% h		Minimum function value
% x		Optimal value of x
%
% Implements a 'branch-and-bound' method on top of CONSTR and thus the
% Optimization Toolbox is required.
%
% The method is intended for problems that are basically continous, with
% some variables constrained to sets of standard values.

% Made 1992 by:
%	Ingar Solberg
%	Institutt for teknisk kybernetikk
%	Norges Tekniske Hgskole
%	N-7034 Trondheim
%	Norway
%
% Authors address from 1993:
% Ingar Solberg               Phone: +47 75179152
% Elkem Aluminium Mosjen     Fax:   +47 75179676
% P.O.Box 566                 E-mail: Ingar.Solberg@elkem.no
% N-8651 Mosjen              X400: G=Ingar;S=Solberg;A=normail;P=elkem;C=no
% Norway

% Applies a strategy where the problem is recursively divided into two
% subproblems by setting upper and lower bounds on the set-constrained
% variables.

% Solve problem without set constraints
[x,opt] = constr(J,x0,options,xlb,xub,Jgrad);

[m,n]=size(set);

% get function and constraint values

[h,gf]=eval([J,'(x)']);

% check if improvement
if h>=Jmin
  h = inf;	% No improvement, no further search along this branch.
  return
end

% check if the solution is feasible (except set constraints)
xtoolow  = find(x<xlb);
xtoohigh = find(x>xub);
if any(abs(gf(1:options(13)) > options(4))) | ...
   any(gf(options(13)+1:length(gf)) > options(4)) | ...
   any((x(xtoohigh)-xub(xtoohigh)) > options(4)) | ...
   any((xlb(xtoolow)-x(xtoolow)) > options(4))
     h=inf;	% Infeasible, no further search along this branch. 
     return;
end

% check if all set constraints are satisfied
k=0;
for i=1:m
  if set(i,1)>0
    if ~any(abs(x(i) - set(i,2:set(i,1)+1)) < options(4))
      k=i;	% Violation of this constraint.
      break;
    end
  end
end

if k>0		% some set constraint violated => recursive search needed
  above=min(find(set(k,2:set(k,1)+1)>x(k)));	
  below=max(find(set(k,2:set(k,1)+1)<x(k)));

% Solve first subproblem if it exists
  if ~isempty(below)
    xub1=xub;
    xub1(k)=set(k,below+1);	% upper bound 1. subproblem
    [h1,x1]=setconst(J,Jgrad,xlb,xub1,Jmin,x,set,options);
    if h1<Jmin
      Jmin=h1;	% Best so far
    end
  else
    h1=inf;
  end

% Solve second subproblem if it exists
  if ~isempty(above)
    xlb1=xlb;
    xlb1(k)=set(k,above+1);	% lower bound 2. subproblem
    [h2,x2]=setconst(J,Jgrad,xlb1,xub,Jmin,x,set,options);
  else
    h2=inf;
  end

  if h1<h2	% Return the best solution
    h=h1;x=x1;
  else
    h=h2;x=x2;
  end
end

