%CSIM   Simulation of an ion channel
%
%Syntax	[DAT L] = csim(Q, CURRENT, LEN, DT[, N[, START[, NOISE]]])
%
%Input	Q	Transition matrix
%	CURRENT	Vector with current levels for every state (incl. closed state)
%	LEN  	Length of simulation in sec
%	DT	Sampling interval in sec
%	N	Number of independt channels [1]
%	START	Starting state [1]
%	NOISE	Noise variance for every state [0]
%	
%Output	DAT	Data vector
%	L	Levelmatrix: [length state-1 current] (0 is closed state)
%		In the case of multiple channel records l gives only
%		levels for the last channel in simulation.
%
%Remark	The transition matrix Q is the transposed form of that matrix used
%	normally in the literature
%
%	Example		    20       200
%			C1 <---> C2 <---> O3 (10 pA)
%		            40       160
%
%				-20   40    0			
%			Q =	 20 -240  160
%				  0  200 -160	
%
%		Simulation: [d l] = csim(Q, [0 0 10], 8, 0.0001, 1);
%
%		Analysis: Probability to find the states 1, 2, 3 zu finden
%			 p(1)=sum(l(l(:,2)==1,1))/sum(l(:,1)) = 0.471 (theoret.)
%			 p(2)=sum(l(l(:,2)==2,1))/sum(l(:,1)) = 0.235
%			 p(3)=sum(l(l(:,2)==3,1))/sum(l(:,1)) = 0.294
%
%			 Mean open/closed time:
%			 ev=eventlst(d, [5], [0 10]);
%			 [mo mc] = meantime(ev)   =  [0.00625  0.0075] (theoret.)
%
%			 Distributions
%			 [x y] = intdis(ev, .5, 4, 1)  minimum open time
%			 [x y] = intdis(ev, .5, 4, 0)  minimum closed time
%
%	(c) A.K. 13.11.1995

function [dat, L]=csim(Q, current, len, dt, n, start, noise)

states=size(Q, 1);
if nargin<7, noise=zeros(1,states); end
if nargin<6, start=1; end
if nargin<5, n=1; end
tau=zeros(states,1);
len=len/dt;
noise=sqrt(noise);
dat=zeros(1, len);
L=[];

% Lebensdauer tau fuer jeden Zustand

for i=1:states,
    tau(i)=1/abs(min(Q(i,:)));
end %for i


% Simulation

rand('seed',sum(100*clock));
for i=1:n
    d=[]; d(1)=0;
    z=start;
    while length(d)<=len,
          time=-tau(z)*log(rand);
	  h=ones(1,time/dt)*current(z)+randn(1,time/dt)*noise(z);
	  d(length(d)+1:length(d)+length(h)) = h;
          L=[L; round(time/dt) z-1 current(z)];
          % Waehle neuen Zustand z
          r=Q(:,z); zs=find(r>0);
          p=r(zs)/sum(r(zs));
          z=randdis(zs, p);
   end %while
   dat=dat+d(1:len);
end %for i
