%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
%     program orbdemo                                                       *
%                                                                           *
%     don mackison                                                          *
%     university of colorado                                                *
%     2207 holyoke drive                                                    *
%     boulder, co 80303                                                     *
%     (303) 939-5880                                                        *
%     (303) 499-0883                                                        *
%                                                                           *
%     this program is used to generate the dynamics                         *
%     of one or more satellites                                             *
%                                                                           *
%     required subprograms include;                                         *
%                                                                           *
%     orbdemo.m : main driver program. compute intersatellite dynamics;     *
%             track 30 vectors in body fixed, orbital, earth fixed,         *
%             and geocentric inertial coordinate frames.                    *
%                                                                           *
%          orbit.m : perform all orbit calculations                         *
%               kepler.m  :  solve keplers equation for eccentric anamaly   *
%               gauss.m   :  solve gauss's equation for true anomaly        *
%               are.m     :  compute radial distance                        *
%               aorb.m    :  compute transformation from eci to lv frame    *
%               angmo.m   :  compute the orbital angular momentum           *
%               fdot.m    :  compute d/dt of true anomaly                   *
%               rdot.m    :  compute d/dt of radial distance                *
%                                                                           *
%          kalman.m : perform all kalman filter calculations                *
%               xprop.m   :  propagate the state estimate                   *
%               covprop.m :  propagate state error covariance matrix        *
%               kgain.m   :  compute the kalman gain matrix                 *
%               xud.m     :  compute the state estimate update              *
%               covud.m   :  compute state error covariance update          *
%                                                                           *
%          math routines:                                                   *
%               dot.m     :  inner product of two vectors                   *
%               cross.m   :  cross product of two vectors                   *
%                                                                           *
%          vectors:                                                         *
%               x1        :  earth fixed ground station unit vector         *
%                            (ecr greenwich)                                *
%               x1        :  unit vector to satellite in local vertical     *
%                         :  coords                                         *
%               x2        :  ground station unit vector (eci)               *
%               x3        :  eci unit vector to the satellite               *
%               x4        :  ground station vector (km) (eci)               *
%               x5        :  eci vector to the satellite (km)               *
%               x6        :  eci satellite to gs vector (km)                *
%               x7        :  eci satellite to gs unit vector = x6/x6norm    *
%               x7        :  eci gs to satellite unit vector = -x6/x6norm   *
%               x8        :  eci satellite to gs unit vector = x6/x6norm    *
%               x10       :  = x4 - x5. eci satellite to ground station     *
%                            vector (km).                                   *
%               x11       :  unit vector along orbit normal, 1st satellite  *
%                            lv coords. inertial angular velocity of local  *
%                            vertical frame at first satellite, in local    *
%                            vertical frame                                 *
%               x11p      :  unit vector along orbit normal for 2nd sat     *
%                            lv coords. inertial angular velocity of local  *
%                            vertical frame at second satellite, in local   *
%                            vertical frame                                 *
%               x12       :  inertial angular velocity of position vector, eci
%               x13       :  inertial angular velocity of vector from       *
%                            sat1 to sat 2                                  *
%               x14       :  velocity of 1st sat in orbit plane             *
%               x14p      :  velocity of 2d sat in orbit plane              *
%               x15       :  velocity of 1st sat in eci                     *
%               x15p      :  velocity of 2nd sat in eci                     *
%               x16       :  x15p - x15. velocity of 2nd sat relative to    *
%                            first, in eci                                  *
%               x17       :  x15p - x15 transformed to lv coords at 1st sat *
%               x18       :  unit vector from 1st sat to 2nd, in lv coords  *
%                            at 1st sat (lv1 coordinates)                   *
%               x19       :  -1*proj of x17 onto x18                        *
%               x20       :  x17 + x19, component of relative vel vector    *
%                            normal to vector from 1st sat to 2nd sat       *
%                            (lv1 coords)                                   *
%               x21       :  rho1/norm(rho1) = unit vector in lv frame at   *
%                            1st sat to 2nd sat                             *
%               x22       :  x20/isrange                                    *
%               x23       :  x11 tilde * rho12                              *
%               x24       :  rdot (inertial)                                *
%               x25                                                         *
%               x26                                                         *
%               x27                                                         *
%               x28                                                         *
%               x29                                                         *
%               x30                                                         *
%                                                                           *
%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

%     put test branch here
%      itest = 0
%      if itest == 1

%==========================================================================

%     clear all data
      clear

%     constants
      dpr =180/pi; rpd=1/dpr; gm=3.986009e5; we = 2*pi/86400.
      gha0 = 0.
      const = [rpd dpr gm we gha0]';

%     orbit parameters for ith satellite
%     x(i,1)     a     semi major axis (km)
%     x(i,2)     e     eccentricity
%     x(i,3)     i     inclination (deg)
%     x(i,4)     om    rt asc ascending node (deg)
%     x(i,5)     wp    arg perigee (deg)
%     x(i,6)     tp    time of perigee  (sec ut)

x = [7000 0.7  0*rpd 45*rpd 0*rpd 0
     7000 0   90*rpd 45*rpd 0*rpd 0]

           a1=x(1,1);e1=x(1,2);i1=x(1,3);om1=x(1,4);w1=x(1,5);tp1=x(1,6);
           a2=x(2,1);e2=x(2,2);i2=x(2,3);om2=x(2,4);w2=x(2,5);tp2=x(2,6);

%     compute orbit period (sec) for first satellite

      t0 = 2*pi/sqrt(gm/a1^3)

      icount = 0
      time = 0
      dt=100;

      while time < t0
           icount = icount + 1

%          develop a satellite trajectory (truth model)
           [f1,ea1,r1,h1,fd1,rd1,to1]=orbit(a1,e1,i1,om1,w1,tp1,time,const);
           traj1(icount,:)=[time,f1,ea1,r1,h1,fd1,rd1];
           trans1(3*icount-2:3*icount,1:3) = to1;

%          develop a target satellite trajectory (truth model)
           [f2,ea2,r2,h2,fd2,rd2,to2]=orbit(a2,e2,i2,om2,w2,tp2,time,const);
           traj2(icount,:)=[time,f2,ea2,r2,h2,fd2,rd2];
           trans2(3*icount-2:3*icount,1:3) = to2;
           time = time + dt
      end

%     at this point, the variable  'icount' is the number of
%     orbit points computed


fprintf('pause after computing orbit trajectories')
fprintf(' ')
fprintf('hit return to continue')

%     plot the output of the orbit model, first satellite

      xplot = traj1(:,1);
      yplot = traj1(:,2)*dpr;

      plot(xplot,yplot),title('orbdemo orbit model output'),...
                        xlabel('time (sec)'),...
                        ylabel('true anomaly 1 (deg)'),...
                        text(.7,.8,'orbdemo, plot1')
      grid
pause
      yplot = traj1(:,4);

      plot(xplot,yplot),title('orbdemo orbit model output'),...
                        xlabel('time (sec)'),...
                        ylabel('radius 1 (km)'),...
                        text(.7,.8,'orbdemo, plot2')
      grid
pause

      yplot = traj1(:,6)*dpr;

      plot(xplot,yplot),title('orbdemo orbit model output'),...
                        xlabel('time (sec)'),...
                        ylabel('f dot 1 (deg/sec)'),...
                        text(.7,.8,'orbdemo, plot3')
      grid
pause

%     plot the output of the orbit model, second satellite

      xplot = traj2(:,1);
      yplot = traj2(:,2)*dpr;

      plot(xplot,yplot),title('orbdemo orbit model output'),...
                        xlabel('time (sec)'),...
                        ylabel('true anomaly 2 (deg)'),...
                        text(.7,.8,'orbdemo, plot4')
      grid
pause
      yplot = traj2(:,4);

      plot(xplot,yplot),title('orbdemo orbit model output'),...
                        xlabel('time (sec)'),...
                        ylabel('radius 2 (km)'),...
                        text(.7,.8,'orbdemo, plot5')
      grid
pause

      yplot = traj2(:,6)*dpr;

      plot(xplot,yplot),title('orbdemo orbit model output'),...
                        xlabel('time (sec)'),...
                        ylabel('f dot 2 (deg/sec)'),...
                        text(.7,.8,'orbdemo, plot6')
      grid
pause

%     compute the orbit normal for the first orbit, n1
      fprintf('first orbit normal vector, n1:')
      n1 = to1(3,:)

%     compute the orbit normal for the second orbit, n2
      fprintf('second orbit normal vector, n2:')
      n2 = to2(3,:)

%     compute nodal vector for first orbit, n4
      fprintf('first orbit nodal vector, n4:')
      n4=[cos(om1) sin(om1) 0]'

%     compute nodal vector for second orbit, n5
      fprintf('second orbit nodal vector, n5:')
      n5 = [cos(om2) sin(om2) 0]'

%     compute cross product n3 = n1 x n2
      fprintf('n3 = n1 x n2 = orbit intersection vector:')
      n3 = cross(n1,n2)
%-----------------------------------------------------------------------------

%     at this point the following quantities have been computed for both
%     satellite orbits, over the time interval specified:

%     traj1(i,:) = [ t     f1     ea1     r1     h1     fd1     rd1 ]
%     traj2(i,:) = [ t     f2     ea2     r2     h2     fd2     rd2 ]

%     further, the orbit matrix for each time point exists
%     to1 is the eci to lv transformation for the first satellite
%     to2 is the eci to lv transformation for the second satellite

%     trans1(i,:) = [ to1 ]
%     trans2(i,:) = [ to2 ]


%                     E N D  O F  M O D E L
%============================================================================


