function [RayPts,ElemPts] = rayoptik(InputFile,CmdStr)
%RAYOPTIK  simple optical ray tracer
%           
%            [RayPts,ElemPts] = rayoptik(InputFile[,CommandString])
%
%          InputFile : M-file with information on optical elements and rays
%          CmdStr    : RayOptik commands (optional)
%
%          RayPts    : coordinates of rays
%          ElemPts   : grid representation of optical elements
%

% Version 0.4  February 1996
% Part of the RayOptik toolbox
% Copyright 1996 by Georg Beyerle.  All rights reserved.
% beyerle@tmf.jpl.nasa.gov

  MFile = [upper(mfilename) ': '];

  UnitVecV = [0;1;0];
  UnitVecU = [1;0;0];  

  CurrAxis   = [];

  if nargin < 2,
    CmdStr = [];
  end

  CmdStr = [' ' CmdStr];

%
%  what is where in matrix 'Elements'
%
  TypeIdx    = 1:10;     % element type

  DistIdx    = 11;       % distance from last element
  CenterIdx  = 12:14;    % position of center

  AngleIdx   = 15;       % tilt angel w.r.t. optical axis
  NormalIdx  = 16:18;    % orientation

  RadInIdx   = 19;       % inner radius
  RadOutIdx  = 20;       % outer radius
  ThickIdx   = 21;       % thickness at optical axis
  FTiltIdx   = 22;
  FTltNrmIdx = 23:25;
  BTiltIdx   = 26;  
  BTltNrmIdx = 27:29;
  
  MatrIdx    = 30:39;    % material
  FSrfTypIdx = 40:49;    % front surface
  FParIdx    = 50:51;    % surface parameters
  BSrfTypIdx = 52:61;    % back surface
  BParIdx    = 62:63;    % surface parameters

%
%  valid input data file?
%
  if exist(InputFile) ~= 2,
    error([upper(mfilename) ': RayOptik data file ' InputFile ' not found']);
  end


%
%  calculation loop
%
  while 1,

    RayPts  = [];
    ElemPts = [];

    SpotUV  = [];
    ScrVert = [];
    NofScrElem = 0;

    clear(InputFile);

    disp([ MFile 'read M-file ' InputFile ]);
    eval(['[Elements, Rays, Wvl, Units] = ' InputFile ';']);

    rochkinp(Elements, Rays, Wvl);

    Len = length(Rays(1,:));

%
%  plot rays in color corresponding to wavelength
%
    WvlCM = rowvlcmp(Wvl(2:Len));

%
%  first column gives optical axis ...
%
    OAPt  = Rays(1:3,1);
    OADir = Rays(4:6,1);
    OADir = OADir / sqrt(sum(OADir.^2));

%
%  ... and the remaining rays
%
    Pt  = Rays(1:3,2:Len);
    Dir = Rays(4:6,2:Len);
    Dir = Dir ./ ([1;1;1] * sqrt(sum(Dir.^2)));

    [RowPt,NofPt]   = size(Pt);
    [RowDir,NofDir] = size(Dir);
  
    NofRays = NofPt;
    NofElem = length(Elements(:,1));

    CurrPt  = [OAPt(:), Pt];
    CurrDir = [OADir(:), Dir];


%
%  loop over all optical elements
%
    for i = 1:NofElem,

      Normal    = [Elements(i,NormalIdx)'];
      Normal    = Normal ./ sqrt(sum(Normal.^2));
      Center    = [Elements(i,CenterIdx)'];
      Dist      = Elements(i,DistIdx);
      Angle     = Elements(i,AngleIdx) * pi /180;   % degree -> radian
      Radius(1) = Elements(i,RadInIdx);
      Radius(2) = Elements(i,RadOutIdx);
      Thickness = Elements(i,ThickIdx);
      ElemType  = Elements(i,TypeIdx);
      Material  = Elements(i,MatrIdx);

%
%  calculate center position and normal vector of front surface
%
      [Center, Normal] = roctrnrm(Center,Normal,CurrPt(:,1),CurrDir(:,1), ...
                                  Dist,Angle);


      disp([MFile setstr(ElemType) ' at position (' ...
          sprintf('%g,%g,%g ',Center) Units ')']);
      disp(['                     and normal vector (' ...
          sprintf('%.2g,%.2g,%.2g',Normal) ')']);

      if ElemType(1:2) == 'ap',
%
%  aperture:
%
        [CurrPt,CurrDir,Pts] = roapertr(CurrPt,CurrDir, ...
                                 Center,Normal,Thickness,Radius);

      elseif ElemType(1:2) == 'sc',
%
%  virtual screen:
%
        SavPt  = CurrPt(:,1);
        SavDir = CurrDir(:,1);

        [CurrPt,CurrDir,Center,Normal,Pts] = roscreen(CurrPt,CurrDir, ...
                                               Center,Normal);

%
%  in order to make insertion of additional virtual screens possible
%  without changing the 'Dist' parameter in matrix 'Elements' of all
%  following 'real' optical elements ...
%
        CurrPt(:,1)  = SavPt;
        CurrDir(:,1) = SavDir;

        if CmdStr(1) == 'm',
          Spot = rosptscr(Center,Normal,CurrPt,Len);
          SpotUV     = [SpotUV Spot(:,2:Len)];
          ScrVert    = [ScrVert Center];
          NofScrElem = NofScrElem + 1;
        end

      elseif findstr(Elements(i,TypeIdx),'mi') == 1,
%
%  mirror:
%

%
%  calculate surface parameters
%
        Par = rosetpar(Elements(i,FSrfTypIdx),Elements(i,FParIdx));

        [CurrFPt,CurrFDir,CurrPt,CurrDir,Pts] = romirror(CurrPt,CurrDir, ...
           Center,Normal,Thickness,Radius,Par);

%
%  save data from reflections (except last)
%
        RayPts  = [RayPts, CurrFPt];

      elseif ElemType(1:2) == 'le',
%
% lens
%

%
%  calculate surface parameters
%
        FPar = rosetpar(Elements(i,FSrfTypIdx),Elements(i,FParIdx));
        BPar = rosetpar(Elements(i,BSrfTypIdx),Elements(i,BParIdx));
        BPar(2) = BPar(2) * (-1);

        [CurrFPt,CurrFDir,CurrPt,CurrDir,Pts] = rolenses(CurrPt,CurrDir,Wvl,Center, ...
           Normal,Thickness,Radius,Material,FPar,BPar);

%
% store data from front surface of lense
%
        RayPts  = [RayPts, CurrFPt];

      elseif ElemType(1:2) == 'pr',
%
% prism
%
        FNormal = roplnnrm(Normal,Elements(i,FTiltIdx)*pi/180, ...
                           Elements(i,FTltNrmIdx));

        BNormal = roplnnrm(Normal,Elements(i,BTiltIdx)*pi/180, ...
                           Elements(i,BTltNrmIdx));

        [CurrFPt,CurrFDir,CurrPt,CurrDir,Pts] = roprisma(CurrPt,CurrDir,Wvl,Center, ...
           Normal,FNormal,BNormal,Thickness,Radius,Material);


%
% store data from front surface of prism
%
        RayPts  = [RayPts, CurrFPt];

      else

        error([upper(mfilename) ': unknown type >' setstr(ElemType) '<'])

      end

      RayPts  = [RayPts, CurrPt];
      ElemPts = [ElemPts, Pts];

    end

%
%  no plotting, if output is requested
%
    if nargout > 0,
      return;
    end

    if CmdStr(1) == 'm',
      CmdStr = rodspspt(SpotUV,ScrVert,NofRays,NofScrElem,Units,CmdStr);
    else
      [UnitVecU,UnitVecV,CmdStr,CurrAxis] = rodisply(NofRays,UnitVecU,UnitVecV, ...
                                       RayPts,ElemPts,WvlCM,Units,CmdStr,CurrAxis,InputFile);
    end

    if CmdStr(1) == 'q',
      if nargout < 1
        clear RayPts ElemPts;
      end
      return;
    end

  end

end


