function [UnitVecU,UnitVecV,CmdStr,CurrAxis] = rodisply(NofRays,UnitVecU,UnitVecV, ...
                                        RayPts,ElemPts,WvlCM,Units,CmdStr,CurrAxis,InputFile)
%RODISPLY plot rays and optical elements
%
%           [UnitVecU,UnitVecV,Key] = rodisply(NofRays,UnitVecU,UnitVecV, ...
%                                      RayPts,ElemPts,WvlCM,Units,CmdStr,CurrAxis,InputFile) 
%         NofRays   : number of rays
%         UnitVecU  : unit vector 
%         UnitVecV  : unit vector
%         RayPts    : rays
%         ElemPts   : optical elements
%         WvlCM     : color map for plotting rays
%         Units     : length units
%         CmdStr    : commad string
%         CurrAxis  : current axis
%         InputFile : RayOptik inputfile
%         

% Version 0.4  February 1996
% Part of the RayOptik toolbox
% Copyright 1996 by Georg Beyerle. All rights reserved.
% beyerle@tmf.jpl.nasa.gov

%
%  initialize parameters
%
  DeltaNorm = tan(5/180*pi);

  MoveFactor = 8;
  ZoomFactor = 8;

  GridOn    = 0;
  PrintHelp = 1;

%
%  plot elements black if background white, else plot white
%
  
  col = get(gcf,'color');
  if all(col == [1 1 1]),
    ColChar = 'k';
  else
    ColChar = 'w';
  end

  while 1,

    if PrintHelp,
      disp(' ');
      disp('commands');
      disp(' e,w : rotate east-/westwards');
      disp(' n,s : rotate north-/southwards');
      disp(' u,d : move up/down');
      disp(' l,r : move left/right');
      disp(' i,o : zoom in/out');
      disp(' p,t : expand/contract horizontally');
      disp(' c   : recalculate trace');
      disp(' g   : grid on/off');
      disp(' m   : spot diagrams (loop thru screens)');
      disp(' v   : call eval()');
      disp(' h   : help');
      disp(' q   : quit');
      PrintHelp = 0;
    end

    URay = UnitVecU' * RayPts;
    VRay = UnitVecV' * RayPts;

    Len = length(URay);
    URay = reshape(URay,NofRays+1,Len / (NofRays+1));
    VRay = reshape(VRay,NofRays+1,Len / (NofRays+1));

%
%  start of optical axis no longer needed
%
    URay(1,:) = [];  
    VRay(1,:) = [];  

    UElem = UnitVecU' * ElemPts(1:3,:);
    VElem = UnitVecV' * ElemPts(1:3,:);

%
%  plot rays and elements
%
    clg

    h = plot(URay',VRay','r-');

    for i=1:length(h)
      set(h(i),'color',WvlCM(i,:))
    end
    
    if GridOn, grid; end

    hold on
    plot(UElem',VElem',[ColChar '-'])
    hold off

    title('RayOptik');
    xlabel(Units);
    ylabel(Units);

    if length(CurrAxis) > 0,
      axis(CurrAxis);
    end

    CurrAxis = axis;

    text(CurrAxis(1),CurrAxis(3),[' File: ' InputFile],'fontsize',8, ...
    'VerticalAlignment','bottom','HorizontalAlignment','left');

    drawnow

%
%  ask for new command
%
    LastCmd = CmdStr(1);
    CmdStr(1) = [];
    if length(CmdStr) < 1,
      CmdStr = input('RO > ','s');
      if length(CmdStr) < 1,
        CmdStr = LastCmd;
      end
    end

%
%  execute command
%
    Key = CmdStr(1);

    Delta = cross(UnitVecU, UnitVecV) * DeltaNorm;

%
% rotate
%
    if Key == 'w'
      UnitVecU = UnitVecU + Delta;
    elseif Key == 'e'
      UnitVecU = UnitVecU - Delta;
    elseif Key == 'n'
      UnitVecV = UnitVecV + Delta;
    elseif Key == 's'
      UnitVecV = UnitVecV - Delta;

%
% contract/expand
%
    elseif Key == 't'
      d = (CurrAxis(2)-CurrAxis(1))/MoveFactor;
      CurrAxis(1) = CurrAxis(1) - d;
      CurrAxis(2) = CurrAxis(2) + d;
    elseif Key == 'p'
      d = (CurrAxis(2)-CurrAxis(1))/MoveFactor;
      CurrAxis(1) = CurrAxis(1) + d;
      CurrAxis(2) = CurrAxis(2) - d;

%
% move left/right/up/down
%
    elseif Key == 'r'
      d = (CurrAxis(2)-CurrAxis(1))/MoveFactor;
      CurrAxis(1:2) = CurrAxis(1:2) - d;
    elseif Key == 'l'
      d = (CurrAxis(2)-CurrAxis(1))/MoveFactor;
      CurrAxis(1:2) = CurrAxis(1:2) + d;
    elseif Key == 'd'
      d = (CurrAxis(4)-CurrAxis(3))/MoveFactor;
      CurrAxis(3:4) = CurrAxis(3:4) + d;
    elseif Key == 'u'
      d = (CurrAxis(4)-CurrAxis(3))/MoveFactor;
      CurrAxis(3:4) = CurrAxis(3:4) - d;

%
% zoom in/out
%
    elseif Key == 'i'
      dy = (CurrAxis(4)-CurrAxis(3))/ZoomFactor;
      dx = (CurrAxis(2)-CurrAxis(1))/ZoomFactor;
      CurrAxis(1) = CurrAxis(1) + dx;
      CurrAxis(2) = CurrAxis(2) - dx;
      CurrAxis(3) = CurrAxis(3) + dy;
      CurrAxis(4) = CurrAxis(4) - dy;
    elseif Key == 'o'
      dy = (CurrAxis(4)-CurrAxis(3))/ZoomFactor;
      dx = (CurrAxis(2)-CurrAxis(1))/ZoomFactor;
      CurrAxis(1) = CurrAxis(1) - dx;
      CurrAxis(2) = CurrAxis(2) + dx;
      CurrAxis(3) = CurrAxis(3) - dy;
      CurrAxis(4) = CurrAxis(4) + dy;
    elseif Key == 'g'
      if GridOn
        GridOn = 0;
      else
        GridOn = 1;
      end
    elseif Key == 'q' | Key == 'c' | Key == 'm',
      return;
    elseif Key == 'v',
      EvalStr = input('argument > ','s');
      eval(EvalStr,'disp(''eval error'');');
    else
      PrintHelp = 1;
    end

%
%  normalize
%
    UnitVecV = UnitVecV / sqrt(sum(UnitVecV.^2));
    UnitVecU = UnitVecU / sqrt(sum(UnitVecU.^2));

  end

end

