function ElemPoints = roelepts(Center,Normal,Radius,Thickness,FPar,BPar,FNorm,BNorm)
%ROELEPTS calculate grid points for graphical representation of optical
%         elements.
% 
%           ElemPoints = roelepts(Center,Normal,Radius,Thickness,FPar,BPar)
%
%         Center     : front center position
%         Normal     : normal vector at center
%         Thickness  : thickness of lens
%         Radius     : [inner radius, outer radius]
%         FPar       : surface parameter (front)
%         BPar       : surface parameter (back)
%         FNorm      : normal vector of front surface [optional]
%         BNorm      : normal vector of back surface [optional]
%

% Version 0.4  February 1996
% Part of the RayOptik toolbox
% Copyright 1996 by Georg Beyerle. All rights reserved.
% beyerle@tmf.jpl.nasa.gov

  if nargin == 8,
    TiltedSurface = 1;
  else
    TiltedSurface = 0;
  end
  
%
%  the higher StepA and StepB the better the graphical representation of
%  curved objects, however the longer takes the computation ...
%  
  StepA   = 32;
  StepB   = 8;

%
%  transformation matrix
%
  az = atan2(Normal(2),Normal(1));
  el = acos(Normal(3));

  T = [cos(az)*sin(el)  -cos(az)*cos(el)   sin(az)
       sin(az)*sin(el)  -sin(az)*cos(el)  -cos(az) 
       cos(el)           sin(el)           0     ];

%
%  transform normal vectors of front and back surface
%
  if TiltedSurface,
    TFNorm = T' * FNorm;
    TBNorm = T' * BNorm;
  end
  
%
%  circular border line (front and back), NaN are used to separate lines
%
  P      = [zeros(3,StepA+1) [NaN;NaN;NaN]];
  Psi    = [0:(2*pi/StepA):2*pi NaN];
  P(2,:) = cos(Psi);
  P(3,:) = sin(Psi);

%
%  four lines at outside, four at inside, separated by NaNs -> 24 cols
%
  Side   = zeros(3,24) * NaN;
  dR     = (Radius(2) - Radius(1)) / StepB;
  Rad    = [(Radius(1):dR:Radius(2)) NaN];
  RadLen = length(Rad)-1;

%
%  front side
%
  FDsp = roheight(Rad,FPar);

  FDspIn  = FDsp(1); 
  FDspOut = FDsp(RadLen);

%
%  back side
%
  BDsp = roheight(Rad,BPar);

  BDspIn  = BDsp(1); 
  BDspOut = BDsp(RadLen);


%
%  now construct matrices
%
  Z = [zeros(1,StepB+1) NaN];

  FCrs = [FDsp FDsp FDsp FDsp ;
          Rad    Z  -Rad    Z ;
          Z    Rad     Z -Rad ];

  BCrs = [BDsp BDsp BDsp BDsp ;
          Rad     Z -Rad    Z ;
          Z     Rad    Z -Rad ];

  if TiltedSurface,
    FCrs = (cross(cross(TFNorm,FCrs),TFNorm)) ./ ...
           sum(TFNorm .* [1;0;0])^2;
    BCrs = (cross(cross(TBNorm,BCrs),TBNorm)) ./ ...
           sum(TBNorm .* [1;0;0])^2;
  end

%
%  cross across surface
%
  Cross = [FCrs, ...
           BCrs-([Thickness;0;0]*ones(1,RadLen*4+4))];

%
%  side lines connecting front and back surface
%
  Side(:,1:3:10) = [FCrs(:,RadLen)     FCrs(:,2*RadLen+1) ...
                    FCrs(:,3*RadLen+2) FCrs(:,4*RadLen+3)];
  Side(:,2:3:11) = [BCrs(:,RadLen)     BCrs(:,2*RadLen+1) ...
                    BCrs(:,3*RadLen+2) BCrs(:,4*RadLen+3)] ...
                    - [Thickness;0;0] * [1,1,1,1];

  if Radius(1) > 0,
    Side(:,13:3:22) = [FCrs(:,1)          FCrs(:,RadLen+2) ...
                       FCrs(:,2*RadLen+3) FCrs(:,3*RadLen+4)];
    Side(:,14:3:23) = [BCrs(:,1)          BCrs(:,RadLen+2) ...
                       BCrs(:,2*RadLen+3) BCrs(:,3*RadLen+4)] ...
                       - [Thickness;0;0] * [1,1,1,1];
  end

%
%  rim
%
  POne = [ones(1,StepA+1) NaN];
  FRim = [P*Radius(2) + [FDspOut;0;0]*POne];
  BRim = [P*Radius(2) + [BDspOut;0;0]*POne];

  if TiltedSurface,
    FOfs = (cross(cross(TFNorm,FRim),TFNorm)) ./ ...
           sum(TFNorm .* [1;0;0])^2;
    BOfs = (cross(cross(TBNorm,BRim),TBNorm)) ./ ...
           sum(TBNorm .* [1;0;0])^2;
    Rim = [FOfs, BOfs - [Thickness;0;0]*POne];
  else
    Rim = [FRim, BRim - [Thickness;0;0]*POne];
  end

  if Radius(1) > 0,
    Rim = [Rim, ...
           P*Radius(1)+[FDspIn;0;0]*POne,         ...
           P*Radius(1)-[-BDspIn+Thickness;0;0]*POne];
  end

%
%  rotate object and move to proper position
%
  Side  = T * Side  + Center * ones(1,length(Side(1,:)));
  Rim   = T * Rim   + Center * ones(1,length(Rim(1,:)));
  Cross = T * Cross + Center * ones(1,length(Cross(1,:)));

  ElemPoints = [Rim,Side,Cross];

end
