function [OutPt1,OutDir1,OutPt2,OutDir2,ElemPoints] = rolenses(InPt,InDir,Wvl, ...
  FCenter,Normal,Thickness,Radius,Material,FPar,BPar)
%ROLENSES lens
%
%           [OutPt1,OutDir1,OutPt2,OutDir2,ElemPoints] = rolenses(InPt,InDir, ...
%               Wvl,FCenter,Normal,Thickness,Radius,Material,FPar,BPar)
%
%         InPt       : last intersection of incomimg rays
%         InDir      : direction of incoming rays
%         FCenter    : front center position of lens
%         Normal     : normal vector at center
%         Thickness  : thickness of lens
%         Radius     : [inner radius, outer radius]
%         Material   : meaterial of lens
%         FPar       : surface parameter (front)
%         BPar       : surface parameter (back)
%
%         OutPt1     : intersections front surface
%         OutDir1    : ray direction front surface
%         OutPt2     : intersections back surface
%         OutDir2    : ray direction back surface
%         ElemPoints : grid representation of lens
%

% Version 0.4  February 1996
% Part of the RayOptik toolbox
% Copyright 1996 by Georg Beyerle. All rights reserved.
% beyerle@tmf.jpl.nasa.gov

  OutPt1  = InPt;
  OutDir1 = InDir;
  OutPt2  = InPt;
  OutDir2 = InDir;

%
%  get refractive index
%
  RI = rorfridx(Wvl,Material);
  RI = RI(:)';

  RIAir   = 1;
  RIRatio = (RIAir ./ RI);     % assume refr. index of air = 1

  NofRays = length(InPt(1,:));

  One = ones(1,NofRays);

%
%  front side
%
  [FPt,FDir,FHit,FFct] = rosrfrfr(InPt,InDir,FCenter,Normal, ...
                                 Radius,FPar,RIRatio);
  
%
%  where does the ray hit surface?
%
  Idx = FHit(2,:);
  OutPt1(:,Idx)  = FPt(:,Idx);
  OutDir1(:,Idx) = FDir(:,Idx); 

  OutPt1(:,1)  = FPt(:,1);
  OutDir1(:,1) = FDir(:,1);

%
%  back side of lens
%
  BCenter = FCenter - Normal * Thickness;

  [BPt,BDir,BHit,BFct] = rosrfrfr(FPt,FDir,BCenter,Normal, ...
                                Radius,BPar,1 ./ RIRatio);

%
%  rays hitting front and back surface
%
  Idx = (FHit(2,:) & BHit(2,:));
  OutPt2(:,Idx)  = BPt(:,Idx);
  OutDir2(:,Idx) = BDir(:,Idx); 

%
%  check if rays hit cylindrical out side
%
  Idx = find(FHit(2,:) | FHit(3,:));
  IdxLen = length(Idx);
  if IdxLen > 0,

    From = -roheight(Radius(2),FPar);
    To   = Thickness - roheight(Radius(2),BPar);

    [Pt,Hit] = rocylind(FPt(:,Idx),FDir(:,Idx),FCenter,Normal, ...
                                Radius(2),2*FHit(2,Idx)-1,-From,To);

    OutPt2(:,Idx(Hit))  = Pt(:,Hit);
    OutDir2(:,Idx(Hit)) = Pt(:,Hit) * NaN;

  end

%%
%%  if lense has hole
%%
%  if (Radius(1) > 0),
%
%    Idx = find(FHit(1,:) | FHit(2,:));
%    IdxLen = length(Idx);
%
%    if IdxLen > 0,
%
%      From = -roheight(Radius(1),FPar);
%      To   = Thickness - roheight(Radius(1),BPar);
%
%      [Pt,Hit] = rocylind(FPt(:,Idx),FDir(:,Idx),FCenter,Normal, ...
%                          Radius(1),-2*FHit(2,Idx)+1,From,To);
%
%      OutPt2(:,Idx(Hit))  = Pt(:,Hit);
%      OutDir2(:,Idx(Hit)) = Pt(:,Hit) * NaN;
%
%    end
%  end

%
%  optical axis
%
  OutPt2(:,1)  = BPt(:,1);
  OutDir2(:,1) = BDir(:,1);

%
%  graphics
%

  ElemPoints = roelepts(FCenter,Normal,Radius,Thickness,FPar,BPar);

end
