function [OutPt1,OutDir1,OutPt2,OutDir2,ElemPoints] = romirror(InPt,InDir,FCenter, ...
                                       Normal,Thickness,Radius,Par)
%ROMIRROR mirror
%
%         InPt      : last intersection of incomimg rays
%         InDir     : direction of incoming rays
%         FCenter   : front center position 
%         Normal    : normal vector at center
%         Thickness : thickness of mirror
%         Radius    : [inner radius, outer radius]
%         Par       : surface parameter 
%
%         OutPt1     : intersections surface
%         OutDir1    : ray direction surface
%         OutPt2     : intersections surface (last)
%         OutDir2    : ray direction surface (last)
%         ElemPoints : grid representation of mirror
%          

% Version 0.4  February 1996
% Part of the RayOptik toolbox
% Copyright 1996 by Georg Beyerle. All rights reserved.
% beyerle@tmf.jpl.nasa.gov

  OutPt1  = InPt;
  OutDir1 = InDir;
  OutPt2  = InPt;
  OutDir2 = InDir;

  NofRays = length(InPt(1,:));

  One = ones(1,NofRays);

  BCenter = FCenter - Normal * Thickness;

%
%  front surface
%
  [FPt,FDir,FHit,FFct] = rosrfrfl(InPt,InDir,FCenter,Normal,Radius,Par);

  Idx = FHit(2,:);
  OutPt1(:,Idx)  = FPt(:,Idx);
  OutDir1(:,Idx) = FDir(:,Idx); 

  OutPt2(:,Idx)  = FPt(:,Idx);
  OutDir2(:,Idx) = FDir(:,Idx); 

%
%  more than one reflections possible in strongly curved mirrors
%
  Idx(1) = 0; 
  while max(Idx) & ~all(~Par) & Par(2) < 0,

    [NewFPt,NewFDir,NewFHit,NewFFct] = rosrfrl2(FPt,FDir,FCenter,Normal,Radius,Par);

    Idx = (NewFHit(2,:) & Idx);

    FPt(:,Idx)  = NewFPt(:,Idx);
    FDir(:,Idx) = NewFDir(:,Idx);

    OutPt1  = [OutPt1  FPt];
    OutDir1 = [OutDir1 FDir]; 

%
%  update
%
    OutPt2(:,Idx)  = FPt(:,Idx);
    OutDir2(:,Idx) = FDir(:,Idx); 

  end


%
%  check if mirror has hole
%
  if (Radius(1) > 0),

    Idx = find(FHit(1,:));
    IdxLen = length(Idx);
    if IdxLen > 0,

      From = -roheight(Radius(1),Par);
      To   = Thickness;

      [Pt,Hit] = rocylind(InPt(:,Idx),InDir(:,Idx),FCenter,Normal, ...
                          Radius(1),1,From,To);

      OutPt2(:,Idx(Hit))  = Pt(:,Hit);
      OutDir2(:,Idx(Hit)) = Pt(:,Hit) * NaN;

    end
  end
    
%
%  check if rays hit cylindrical out side
%
  Idx = find(FHit(3,:));
  IdxLen = length(Idx);
  if IdxLen > 0,

    From = -roheight(Radius(2),Par);
    To   = Thickness;

    [Pt,Hit] = rocylind(InPt(:,Idx),InDir(:,Idx),FCenter,Normal, ...
                                Radius(2),-1,From,To);

    OutPt2(:,Idx(Hit))  = Pt(:,Hit);
    OutDir2(:,Idx(Hit)) = Pt(:,Hit) * NaN;

  end


%
%  optical axis
%
  OutPt2(:,1)  = FCenter;
  OutDir2(:,1) = InDir(:,1) - 2 * Normal * (Normal' * InDir(:,1));

%
%  graphics
%  
  ElemPoints = roelepts(FCenter,Normal,Radius,Thickness,Par,[0,0,0]);

end

