function [OutPt,OutDir,Hit,Fct] = roplnrfr(InPt,InDir,Center,ObjNorm,PlnNorm, ...
                                           Radius,RIRatio)
%ROPLNRFR refracting plane tilted
%
%           [OutPt,OutDir,Hit] = roplnrfr(InPt,InDir,Center,ObjNorm,PlnNorm,Radius,RIRatio)
%
%         InPt      : starting point
%         InDir     : starting direction
%         Center    : center position 
%         ObjNorm   : normal vector of object
%         PlnNorm   : normal vector of surface
%         Radius    : radius
%         RIRatio   : ratio of refractive index 
%
%         OutPt     : intersections surface
%         OutDir    : ray direction surface
%

% Version 0.4  February 1996
% Part of the RayOptik toolbox
% Copyright 1996 by Georg Beyerle. All rights reserved.
% beyerle@tmf.jpl.nasa.gov

  NofRays = length(InPt(1,:));
  One = ones(1,NofRays);

  PMinVert = InPt - Center * One;
    
  PMinVertXNormal   = cross(PMinVert, PlnNorm * One);   
  DirXNormal        = cross(InDir, PlnNorm * One);     
  PMinVertDotNormal = PlnNorm' * PMinVert;
  DirDotNormal      = PlnNorm' * InDir;

  OutPt  = InPt;
  OutDir = InDir;

%
%  plane surface
%
  AbsDotDir = PlnNorm' * InDir;

  Crossing = (AbsDotDir ~= 0);

%
%  mark rays parallel to plane with NaN
%
  NonCrossIdx = find(~Crossing);
  AbsDotDir(NonCrossIdx) = NaN * NonCrossIdx;

  Fct = -((PlnNorm' * InPt - PlnNorm' * Center) ./ AbsDotDir);

  Idx = find(Fct < 0);
  Fct(Idx) = NaN * Idx;

  OutPt = InPt + ([1;1;1] * Fct) .* InDir;

  PtMinCtr = OutPt - Center * One;
  AbsPtMinCtr = sqrt(sum((OutPt - Center * One).^2));

  SqrtArg = (1 ./ RIRatio.^2) - (1  - AbsDotDir.^2);

  Idx = find(SqrtArg < 0);
  SqrtArg(Idx) = Idx * NaN;

  OutDir = PlnNorm * (-AbsDotDir - sqrt(SqrtArg)) + InDir;
  OutDir = OutDir ./ ([1;1;1] * sqrt(sum(OutDir.^2)));

  AbsPrjPtMinCtr = sqrt(sum(cross(ObjNorm*One,PtMinCtr).^2));

%
%  Hit index
%
  Hit = [0;0;0] * One;

  Hit(2,:) = (AbsPrjPtMinCtr <= Radius(2));
  Hit(3,:) = (~Hit(1,:) & ~Hit(2,:));


%
%  rays missing the prism
%
  Idx = Hit(3,:);
  OutDir(:,Idx) = InDir(:,Idx);

end

