function [OutPt1,OutDir1,OutPt2,OutDir2,ElemPoints] = roprisma(InPt,InDir, ...
            Wvl,FCenter,ObjNorm,FPlnNorm,BPlnNorm,Thickness,Radius,Material)
%ROPRISMA prism
%
%         InPt      : last intersection of incomimg rays
%         InDir     : direction of incoming rays
%         FCenter   : front center position 
%         ObjNorm   : normal vector of object
%         FPlnNorm  : normal vector of front surface
%         BPlnNorm  : normal vector of back surface
%         Thickness : thickness of prism
%         Radius    : radius
%
%         OutPt1     : intersections surface
%         OutDir1    : ray direction surface
%         OutPt2     : intersections surface (last)
%         OutDir2    : ray direction surface (last)
%         ElemPoints : grid representation of prism
%          

% Version 0.4  February 1996
% Part of the RayOptik toolbox
% Copyright 1996 by Georg Beyerle. All rights reserved.
% beyerle@tmf.jpl.nasa.gov

  OutPt1  = InPt;
  OutDir1 = InDir;
  OutPt2  = InPt;
  OutDir2 = InDir;

  NofRays = length(InPt(1,:));

  One = ones(1,NofRays);

%
%  get refractive index
%
  RI = rorfridx(Wvl,Material);
  RI = RI(:)';

  RIAir   = 1;
  RIRatio = (RIAir ./ RI);     % assume refr. index of air = 1


  BCenter = FCenter - ObjNorm * Thickness;

%
%  front surface
%
  [FPt,FDir,FHit,FFct] = roplnrfr(InPt,InDir,FCenter,ObjNorm,FPlnNorm, ...
                                  Radius,RIRatio);
%
%  back surface
%
  [BPt,BDir,BHit,BFct] = roplnrfr(FPt,FDir,BCenter,ObjNorm,BPlnNorm, ...
                                  Radius,1./RIRatio);

  Idx = FHit(2,:);
  OutPt1(:,Idx)  = FPt(:,Idx);
  OutDir1(:,Idx) = FDir(:,Idx); 
  OutPt2(:,Idx)  = FPt(:,Idx);
  OutDir2(:,Idx) = FDir(:,Idx); 

  Idx = BHit(2,:);
  OutPt2(:,Idx)  = BPt(:,Idx);
  OutDir2(:,Idx) = BDir(:,Idx); 

%
%  check if rays hit cylindrical out side
%
  Idx = find(FHit(2,:) | FHit(3,:));
  IdxLen = length(Idx);
  if IdxLen > 0,
    FRad = FPt-FCenter*One;
    BRad = BPt-BCenter*One;

    FHeightVec = cross(cross(ObjNorm,FRad),ObjNorm) - FRad;
    BHeightVec = cross(cross(ObjNorm,BRad),ObjNorm) - BRad;

    From = sum(FHeightVec(:,Idx) .* (ObjNorm*One(Idx)));
    To   = Thickness + sum(BHeightVec(:,Idx) .* (ObjNorm*One(Idx)));

    [Pt,Hit] = rocylind(FPt(:,Idx),FDir(:,Idx),FCenter,ObjNorm, ...
                                Radius(2),2*FHit(2,Idx)-1,From,To);

    OutPt2(:,Idx(Hit))  = Pt(:,Hit);
    OutDir2(:,Idx(Hit)) = Pt(:,Hit) * NaN;

  end

%
%  optical axis
%
  OutPt2(:,1)  = BPt(:,1);
  OutDir2(:,1) = BDir(:,1);

%
%  graphics
%  
  ElemPoints = roelepts(FCenter,ObjNorm,Radius,Thickness,[0,0,0],[0,0,0], ...
                        FPlnNorm,BPlnNorm);
end

