function [OutPt,OutDir,Hit,Fct] = rosrfrfr(InPt,InDir,Center,Normal, ...
                                           Radius,Par,RIRatio)
%ROSRFRFR refracting surface
%
%           [OutPt,OutDir,Hit] = rosrfrfr(InPt,InDir,Center,Normal,Radius,Par,RIRatio)
%
%         InPt      : starting point
%         InDir     : starting direction
%         Center    : center position 
%         Normal    : normal vector at center
%         Radius    : [inner radius, outer radius]
%         Par       : surface parameter 
%         RIRatio   : ratio of refractive index 
%
%         OutPt     : intersections surface
%         OutDir    : ray direction surface
%

% Version 0.4  February 1996
% Part of the RayOptik toolbox
% Copyright 1996 by Georg Beyerle. All rights reserved.
% beyerle@tmf.jpl.nasa.gov

% Changes:
% 96/02/17 - bug fix: rays are stopped if internal total reflection occurs

  NofRays = length(InPt(1,:));
  One = ones(1,NofRays);

  PMinVert = InPt - Center * One;
    
  PMinVertXNormal   = cross(PMinVert, Normal * One);   
  DirXNormal        = cross(InDir, Normal * One);     
  PMinVertDotNormal = Normal' * PMinVert;
  DirDotNormal      = Normal' * InDir;

%
%  surface parameters
%
  a = Par(1);
  b = Par(2);
  c = Par(3);

  OutPt  = InPt;
  OutDir = InDir;

%
%  Hit index
%
  Hit = [1;1;1] * One;

  if ~Par,
%
%  plane surface
%
    NormDotDir = Normal' * InDir;

    Crossing = (NormDotDir ~= 0);

%
%  mark rays parallel to plane with NaN
%
    NonCrossIdx = find(~Crossing);
    NormDotDir(NonCrossIdx) = NaN * NonCrossIdx;

    Fct = -((Normal' * InPt - Normal' * Center) ./ NormDotDir);

    Idx = find(Fct < 0);
    Fct(Idx) = NaN * Idx;

    OutPt = InPt + ([1;1;1] * Fct) .* InDir;

    SqrtArg = (1 ./ RIRatio.^2) - (1  - NormDotDir.^2);

    Idx = find(SqrtArg < 0);
    SqrtArg(Idx) = Idx * NaN;

    OutDir = Normal * (-NormDotDir - sqrt(SqrtArg)) + InDir;
    OutDir = OutDir ./ ([1;1;1] * sqrt(sum(OutDir.^2)));

    R = sqrt(sum((OutPt - Center * One).^2));

    Hit(1,:) = (R < Radius(1));
    Hit(2,:) = (R >= Radius(1) & R <= Radius(2));
    Hit(3,:) = (~Hit(1,:) & ~Hit(2,:));

  else
%
%  curved surface
%

%
%  intersecting point is   Fct * InDir + InPt, 
%  where  Fct^2 + Num1/Den*Fct + Num2/Den = 0
%
    Num1 = 2 * DirDotNormal .* (a*PMinVertDotNormal+b/2) + ...
           2 * sum(DirXNormal .* PMinVertXNormal);

    Num2 = a * PMinVertDotNormal.^2 + b * PMinVertDotNormal + c + ...
           sum(PMinVertXNormal.^2);

    Den = a * DirDotNormal.^2 + sum(DirXNormal.^2);

    Fct1 = One * NaN;
    Fct2 = One * NaN;

    Idx = find( Num1.^2/4 >= Num2.*Den & Den ~= 0);

    Fct1(Idx) = -Num1(Idx)./Den(Idx)/2 - ...
         sqrt((Num1(Idx)./Den(Idx)/2).^2 - Num2(Idx)./Den(Idx));
    Fct2(Idx) = -Num1(Idx)./Den(Idx)/2 + ...
         sqrt((Num1(Idx)./Den(Idx)/2).^2 - Num2(Idx)./Den(Idx));

    Idx = ( Den == 0 & Num1 ~= 0);
    Fct1(Idx) = -Num2(Idx)./Num1(Idx);
    Fct2(Idx) = -Num2(Idx)./Num1(Idx);

%
%  we only look forwards
%
    Idx = find( Fct1 < 0);
    Fct1(Idx) = NaN * Idx;
    Idx = find( Fct2 < 0);
    Fct2(Idx) = NaN * Idx;

    Pt1 = InPt + ([1;1;1] * Fct1) .* InDir;
    Pt2 = InPt + ([1;1;1] * Fct2) .* InDir;

    Pt1MinCtrDotNorm = sum((Pt1 - Center*One) .* (Normal * One));
    Pt2MinCtrDotNorm = sum((Pt2 - Center*One) .* (Normal * One));

    DspInner = roheight(Radius(1),Par);
    DspOuter = roheight(Radius(2),Par);

    Idx1 = (Pt1MinCtrDotNorm >= DspInner & Pt1MinCtrDotNorm <= DspOuter) | ...
           (Pt1MinCtrDotNorm <= DspInner & Pt1MinCtrDotNorm >= DspOuter);
    Idx2 = (Pt2MinCtrDotNorm >= DspInner & Pt2MinCtrDotNorm <= DspOuter) | ...
           (Pt2MinCtrDotNorm <= DspInner & Pt2MinCtrDotNorm >= DspOuter);

    OutPt(:,Idx1) = Pt1(:,Idx1);
    OutPt(:,Idx2) = Pt2(:,Idx2);

    Idx3 = (Pt1MinCtrDotNorm < DspInner & Pt1MinCtrDotNorm >= 0) | ...
           (Pt1MinCtrDotNorm > DspInner & Pt1MinCtrDotNorm <= 0);
    Idx4 = (Pt2MinCtrDotNorm < DspInner & Pt2MinCtrDotNorm >= 0) | ...
           (Pt2MinCtrDotNorm > DspInner & Pt2MinCtrDotNorm <= 0);

%
%
%
    Hit(1,:) = (Idx3 | Idx4);
    Hit(1,1) = 0;
    Hit(2,:) = (Idx1 | Idx2);
    Hit(2,1) = 1;
    Hit(3,:) = (~Hit(1,:) & ~Hit(2,:));


    Idx = (Idx1 & Idx2);
    if max(Idx),
      Fct = min(Fct1(Idx),Fct2(Idx));
      OutPt(:,Idx) = InPt(:,Idx) + ([1;1;1] * Fct) .* InDir(:,Idx);
    end

%
%  normal vector at OutPt
%
    OutPtMinVert      = OutPt - Center * One;
    OutPtMinVertXNorm = cross(OutPtMinVert,Normal*One);

    SqrtArg = b^2-4*a*(sum(OutPtMinVertXNorm.^2)+c);

    Idx = find(SqrtArg <= 0);
    SqrtArg(Idx) = NaN * Idx;

    NormAtOutPt = Normal * One + sign(b) * 2 * ( [1;1;1] ...
           * (1 ./ sqrt(SqrtArg))) .* cross(Normal,OutPtMinVertXNorm); 

%
%  normalize
%
    NormAtOutPt = NormAtOutPt ./ ([1;1;1]*sqrt(sum(NormAtOutPt.^2)));

%
% refraction
%
    NormAtOutPtDotInDir = sum(NormAtOutPt .* InDir);

    SqrtArg = (1 ./ RIRatio.^2) - (1  - NormAtOutPtDotInDir.^2);

    Idx = (SqrtArg >= 0 & Hit(2,:));
    OutDir(:,Idx) = ([1;1;1] * (-NormAtOutPtDotInDir(Idx) - sqrt(SqrtArg(Idx)))) .* ...
               NormAtOutPt(:,Idx) + InDir(:,Idx);

%
%  total reflection, stop ray
%
    Idx = (SqrtArg < 0 & Hit(2,:));
    OutDir(:,Idx) = NaN * OutDir(:,Idx);

%%
%%  total reflection
%%
%    Idx = ~Idx;
%    OutDir(:,Idx) = InDir(:,Idx) - 2 * ([1;1;1] ...
%           * sum(InDir(:,Idx) .* NormAtOutPt(:,Idx))) .* NormAtOutPt(:,Idx);

%
%  normalize
%
    OutDir = OutDir ./ ([1;1;1] * sqrt(sum(OutDir.^2)));
  
  end

end

