function [OutPt,OutDir,Hit,Fct] = rosrfrl2(InPt,InDir,Center,Normal, ...
                                           Radius,Par)
%ROSRFRL2 reflecting surface
%
%           [OutPt,OutDir,Hit] = rosrfrl2(InPt,InDir,Center,Normal,Radius,Par)
%
%         identical to rosrfrfl(), however it is assumed that InPt lies
%         already on the surface described by vector Par
%
%         InPt      : starting point
%         InDir     : starting direction
%         Center    : center position 
%         Normal    : normal vector at center
%         Radius    : [inner radius, outer radius]
%         Par       : surface parameter 
%
%         OutPt     : intersections surface
%         OutDir    : ray direction surface
%

% Version 0.4  February 1996
% Part of the RayOptik toolbox
% Copyright 1996 by Georg Beyerle. All rights reserved.
% beyerle@tmf.jpl.nasa.gov

  NofRays = length(InPt(1,:));
  One = ones(1,NofRays);

  PMinVert = InPt - Center * One;
    
  PMinVertXNormal   = cross(PMinVert, Normal * One);   
  DirXNormal        = cross(InDir, Normal * One);     
  PMinVertDotNormal = Normal' * PMinVert;
  DirDotNormal      = Normal' * InDir;

%
%  surface parameters
%
  a = Par(1);
  b = Par(2);
  c = Par(3);

  OutPt  = InPt;
  OutDir = InDir;

  Hit = [0;0;0] * One;

  if ~Par,
%
%  plane surface
%
    NormDotDir = Normal' * InDir;

    Crossing = (NormDotDir ~= 0);

%
%  mark rays parallel to plane with NaN
%
    NonCrossIdx = find(~Crossing);
    NormDotDir(NonCrossIdx) = NaN * NonCrossIdx;

    Fct = -((Normal' * InPt - Normal' * Center) ./ NormDotDir);

    Idx = find(Fct < 0);
    Fct(Idx) = NaN * Idx;

    OutPt = InPt + ([1;1;1] * Fct) .* InDir;
    OutDir = InDir - 2 * Normal * (Normal' * InDir);

    R = sqrt(sum((OutPt - Center * One).^2));

    Hit(1,:) = (R < Radius(1));
    Hit(1,1) = 0;
    Hit(2,:) = (R >= Radius(1) & R <= Radius(2));
    Hit(2,1) = 1;
    Hit(3,:) = (~Hit(1,:) & ~Hit(2,:));

  else
%
%  curved surface
%

%
%  intersecting point is   Fct * InDir + InPt, 
%  where  Fct^2 + Num1/Den*Fct + Num2/Den = 0
%
    Num1 = 2 * DirDotNormal .* (a*PMinVertDotNormal+b/2) + ...
           2 * sum(DirXNormal .* PMinVertXNormal);

    Num2 = a * PMinVertDotNormal.^2 + b * PMinVertDotNormal + c + ...
           sum(PMinVertXNormal.^2);

    Den = a * DirDotNormal.^2 + sum(DirXNormal.^2);

    Fct1 = One * NaN;
    Fct2 = One * NaN;

    Idx = find( Num1.^2/4 >= Num2.*Den & Den ~= 0);

    Fct1(Idx) = -Num1(Idx)./Den(Idx)/2 + ...
         sqrt((Num1(Idx)./Den(Idx)/2).^2 - Num2(Idx)./Den(Idx));
    Fct2(Idx) = -Num1(Idx)./Den(Idx)/2 - ...
         sqrt((Num1(Idx)./Den(Idx)/2).^2 - Num2(Idx)./Den(Idx));

    Idx = ( Den == 0 & Num1 ~= 0);

    Fct1(Idx) = -Num2(Idx)./Num1(Idx);
    Fct2(Idx) = -Num2(Idx)./Num1(Idx);

%
%  we only look forwards
%
%  value closest to zero is start point, we are not
%  interested in this one 
%
    Idx = find(min(abs([Fct1;Fct2])) == abs(Fct1));
    Fct1(Idx) = NaN * Idx;
    Idx = find(min(abs([Fct1;Fct2])) == abs(Fct2));
    Fct2(Idx) = NaN * Idx;
    Idx = find(Fct1 <= 0);
    Fct1(Idx) = NaN * Idx;
    Idx = find(Fct2 <= 0);
    Fct2(Idx) = NaN * Idx;

    Pt1 = InPt + ([1;1;1] * Fct1) .* InDir;
    Pt2 = InPt + ([1;1;1] * Fct2) .* InDir;

    Pt1MinCtrDotNorm = sum((Pt1 - Center*One) .* (Normal * One));
    Pt2MinCtrDotNorm = sum((Pt2 - Center*One) .* (Normal * One));

    DspInner = roheight(Radius(1),Par);
    DspOuter = roheight(Radius(2),Par);

    Idx1 = (Pt1MinCtrDotNorm >= DspInner & Pt1MinCtrDotNorm <= DspOuter) | ...
           (Pt1MinCtrDotNorm <= DspInner & Pt1MinCtrDotNorm >= DspOuter);
    Idx2 = (Pt2MinCtrDotNorm >= DspInner & Pt2MinCtrDotNorm <= DspOuter) | ...
           (Pt2MinCtrDotNorm <= DspInner & Pt2MinCtrDotNorm >= DspOuter);

    OutPt(:,Idx1) = Pt1(:,Idx1);
    OutPt(:,Idx2) = Pt2(:,Idx2);

    Idx = (Idx1 & Idx2);
    if max(Idx),
      Fct = min(Fct1(Idx),Fct2(Idx));
      OutPt(:,Idx) = InPt(:,Idx) + ([1;1;1] * Fct) .* InDir(:,Idx);
    end

    Idx3 = (Pt1MinCtrDotNorm < DspInner & Pt1MinCtrDotNorm >= 0) | ...
           (Pt1MinCtrDotNorm > DspInner & Pt1MinCtrDotNorm <= 0);
    Idx4 = (Pt2MinCtrDotNorm < DspInner & Pt2MinCtrDotNorm >= 0) | ...
           (Pt2MinCtrDotNorm > DspInner & Pt2MinCtrDotNorm <= 0);

%
%
%
    Hit(1,:) = (Idx3 | Idx4);
    Hit(1,1) = 0;
    Hit(2,:) = (Idx1 | Idx2);
    Hit(2,1) = 1;
    Hit(3,:) = (~Hit(1,:) & ~Hit(2,:));
    
%
%  normal vector at OutPt
%
    OutPtMinVert = OutPt - Center*One;
    OutPtMinVertXNorm = cross(OutPtMinVert,Normal*One);

    SqrtArg = b^2-4*a*(sum(OutPtMinVertXNorm.^2)+c);

    Idx = find(SqrtArg <= 0);
    SqrtArg(Idx) = NaN * Idx;

    if ~(a == 0 & b == 0),
      NormAtOutPt = Normal * One + sign(b) * 2 * ( [1;1;1] ...
           * (1 ./ sqrt(SqrtArg))) .* cross(Normal,OutPtMinVertXNorm); 
    else
      NormAtOutPt = cross(Normal,OutPtMinVertXNorm);
    end

%
%  normalize
%
    NormAtOutPt = NormAtOutPt ./ ([1;1;1]*sqrt(sum(NormAtOutPt.^2)));

%
% reflection
%
    OutDir = InDir - 2 * ([1;1;1] * sum(InDir .* NormAtOutPt)) ...
                    .* NormAtOutPt;

  end

end
