function y = corr(in1,in2,f)
%CORR    Compute correlations faster than DSP toolbox, and with bandpassing
%
% y = corr(x)
% Compute the autocorrelation of vector x.  Does NOT handle matrices 
% the way xcorr (in the Signal Processing toolbox) does.
%
% y = corr(x1,x2)
% Return the cross-correlation of vectors x1 and x2, which need not be 
% the same length.  This is similar to xcorr, but faster (the latter
% efficiently computes 4 correlations and throws 3 away).  Also, if 
% length(x1) is different from length(x2), xcorr returns a wrong-length 
% result.
%
% y = corr(x1,x2,f)
% As above, but also filter by the bandpass filter f.  f is a two-element
% vector, with values in [0,1], that specifies the passband; in f, the
% value 1 represents the Nyquist frequency.

if (nargin < 2), in2 = in1; end
x1 = in1(:);
x2 = in2(:);
n1 = length(x1);
n2 = length(x2);
nfft = 2 * 2^nextpow2(max(n1,n2));	% pad with 0's for fft & circular corr

y = conj(fft([x1;zeros(nfft-n1,1)])) .* fft([x2;zeros(nfft-n2,1)]);
if (nargin > 2)
  if (length(f) == 1), f = [0 f]; end	    % one number means lowpass filter
  n = nfft/2;
  i0 = round(f(1) * n);
  i1 = round(f(2) * n);
  y(   1 : i0) = zeros(i0,1);
  y(i1+1 : n)  = zeros(n-i1,1);
  y(nfft+1-i0 : nfft)    = zeros(i0,1);
  y(nfft+1-n  : nfft-i1) = zeros(n-i1, 1);
end
y = ifft(y);
y = [y(nfft+2-n1:nfft); y(1:n2)];

if (all(isreal(in1)) & all(isreal(in2)))
  y = real(y);
end
if (size(in1,1) == 1), y = y.'; end
