function [seq,nLeft] = readBinary(filename, offset, n, typ)
% READBINARY    Open a file, read data from it, close it, and return the data.
% 
% seq = readBinary(filename, offset, n, typ)
% From the file filename, skip offset bytes from the beginning, then
% read n objects of type typ and return as a column vector.  typ is 
% one of the types listed in fread, with these exceptions:
%
%    int and uint are always 32 bits
%    intN and uintN work only when N=8, 16, or 32 (is this true of fread?)
%    these abbrevations work too:
%        c    char        (8 bits)
%        s    short       (16 bits)
%        l    long        (32 bits)
%        f    float       (32 bits, or short float)
%        d    double      (64 bits, or long float)
%
% For the record, the accepted types given in fread are
%    char, schar, short, int, long, uchar, ushort, uint, ulong,
%    int8, int16, int32, float, double, float32, float64
%
% [seq,nLeft] = readBinary(filename, offset, n, typ)
% A second output arg returns the number of elements of the specified type
% that are left in the file after reading.  Thus
%          [dummy,nbytes] = readBinary(filename, 0, 0, 'c')
% tells how many bytes are in a file.  nLeft is always an integer.

typ = setstr(typ + (typ < 96) * 32);			% convert to lowercase

if     (strcmp(typ, 'c')), typ = 'char';
elseif (strcmp(typ, 's')), typ = 'short';
elseif (strcmp(typ, 'l')), typ = 'long';
elseif (strcmp(typ, 'f')), typ = 'float';
elseif (strcmp(typ, 'd')), typ = 'double';
end
if     (strcmp(typ, 'char')),   len = 1;
elseif (strcmp(typ, 'schar')),  len = 1;
elseif (strcmp(typ, 'short')),  len = 2;
elseif (strcmp(typ, 'int')),    len = 4;
elseif (strcmp(typ, 'long')),   len = 4;
elseif (strcmp(typ, 'float')),  len = 4;
elseif (strcmp(typ, 'double')), len = 8;
elseif (strcmp(typ, 'uchar')),  len = 1;
elseif (strcmp(typ, 'ushort')), len = 2;
elseif (strcmp(typ, 'uint')),   len = 4;
elseif (strcmp(typ, 'ulong')),  len = 4;
elseif (strcmp(typ, 'float32')),len = 4;
elseif (strcmp(typ, 'float64')),len = 8;
elseif (strcmp(typ, 'int8')),   len = 1;
elseif (strcmp(typ, 'int16')),  len = 2;
elseif (strcmp(typ, 'int32')),  len = 4;
elseif (strcmp(typ, 'uint8')),  len = 1;
elseif (strcmp(typ, 'uint16')), len = 2;
elseif (strcmp(typ, 'uint32')), len = 4;
else
  error(['Unknown data type code ', typ, '.']);
end

fd = fopen(filename, 'r');
if (fd < 0), error(['Can''t open the file for reading: ', filename]); end
fseek(fd, offset, 'bof');
[seq,nr] = fread(fd, n, typ);
if (n ~= Inf & nr ~= n)
  error(['Couldn''t read ',num2str(n),' items from file ', filename]);
end
if (nargout > 1)
  pos = ftell(fd);
  fseek(fd, 0, 'eof');
  nLeft = floor((ftell(fd) - pos) / len);
end
fclose(fd);
