function [data,name,sz] = readMatlabFile(filename, offset, n, varNum)
% READMATLABFILE    Read part or all of a variable stored in a .mat file.
%
% data = readMatlabFile(filename)
%    Read the given file and return the matrix for its first variable.
%    No check is made to ensure that the file is in fact a MATLAB file.
%
% data = readMatlabFile(filename, offset)
%    As above, but skip the first offset numbers in the variable.
%    If offset > 0, the row- and column-dimensions of the matrix are 
%    ignored; the matrix returned (data) is always a column vector.  
%    You can reshape it if necessary using the matrix dimensions (see below).
%
% data = readMatlabFile(filename, offset, n)
%    As above, but read only the first n numbers of the matrix.
%    If n == Inf, read the whole matrix.
%
% data = readMatlabFile(filename, offset, n, varNum)
%    As above, for reading MATLAB files with more than one variable
%    in them.  The first variable in the file is varNum=1.
%
% data = readMatlabFile(filename, offset, n, 'varName')
%    As above, but you can specify the name of the variable instead of
%    its position in the file.
%
% [data,name] = readMatlabFile( ... )
%    As above, but also returns the variable name as a string.
%
% [data,name,sz] = readMatlabFile( ... )
%    As above, but also return the dimensions of the MATLAB variable.
%    This is the size of the whole matrix, even if you read only
%    part of it.
%
% LIMITATION: readMatlabFile can handle only .mat files with ordinary 
%             MATLAB variables, i.e. real or complex 8-byte floats.
%
% See also readBinary.

if (nargin < 2), offset = 0; end
if (nargin < 3), n = Inf;    end
if (nargin < 4), varNum = 1; end

off = 0;
if (isstr(varNum))
  % find variable named varNum
  [dummy,len] = readBinary(filename, off, 0, 'c');
  cont = 1;
  while (cont)
    if (off >= len)
      error(['Couldn''t find variable ''', varNum, ''' in file ', filename]);
    end
    stuff = readBinary(filename, off, 5, 'L');
    name = readBinary(filename, off+20, stuff(5)-1, 'c');
    if (strcmp(name, varNum))
      cont = 0;
    else
      off = off + 20 + stuff(5) + stuff(2)*stuff(3)*8*(stuff(4)+1);
    end
  end
else
  % find varNum'th variable in file
  for i = 1:varNum-1
    stuff = readBinary(filename, off, 5, 'L');
    off = off + 20 + stuff(5) + stuff(2)*stuff(3)*8*(stuff(4)+1);
  end
end

stuff = readBinary(filename, off, 5, 'L');
off = off + 20;

if (digit(stuff(1),3) ~= 1), 
   error('ReadMatlabFile: Can only read MATLAB type 1 files (see LOAD).');
end
if (digit(stuff(1),2) ~= 0  &  stuff(2) > 1  &  stuff(3) > 1),
   disp('Warning: ReadMatlabFile can''t handle row-major arrays.');
end
if (digit(stuff(1),1) ~= 0),
   error('ReadMatlabFile can handle only double-precision numbers now.');
end

name = readBinary(filename, off, stuff(5)-1, 'c');
off = off + stuff(5);
sz = stuff(2) * stuff(3);

n = min(n, sz - offset);
data = readBinary(filename, off + offset*8, n, 'd');
off = off + sz*8;
if (stuff(4)),		% read imaginary part if necessary
   data = data + sqrt(-1)*readBinary(filename, off + offset*8, n, 'd');
   off = off + sz*8;
end
if (n >= sz), data = reshape(data, stuff(2), stuff(3)); end
if (digit(stuff(1),0) == 1), data = setstr(data); end

sz = [stuff(2) stuff(3)];
