function [sound,sRate,left,pOut] = soundIn(filename, offset, maxRead, pIn)
% sound = soundIn(filename)
%    Read a sound file, return the sound.  Works on MATLAB files,
%    Canary binary files, and all the file types that ReadSound
%    works on.
% 
% sound = soundIn(filename, offset)
%   As above, but skip the first offset samples in the file.
%   This works only for MATLAB, binary, and AIFF files.
%
% sound = soundIn(filename, offset, maxRead)
%    As above, but read at most maxRead samples.  If maxRead is Inf
%    or is not present (or is less than 0), read to the end of the file.
%
% sound = soundIn(filename, offset, maxRead, pIn)
%    'pIn' is a parameter used in various ways by different
%    file formats.
%	for Navy (.str) files, it's the beam number
%	for MATLAB files, it's the sound number (i.e. the column
%		number in the matrix)
%	for Sun/Next .snd and .au files, it's the channel number
%    Other formats don't use it yet.
%
% [sound,sRate] = soundIn( ... )
%    As above, but also return the sampling rate.  Doesn't work for
%    MATLAB files or Canary binary files.  Also doesn't work for
%    AIFF (.aif) files when you have a non-zero offset or 
%    non-negative maxRead.  Using this with maxRead==0 is a good
%    way to find out the sampling rate.
%
% [sound,sRate,left] = soundIn( ... )
%    As above, but also return the number of samples left in the file
%    after this read.
%
% [sound,sRate,left,pOut] = soundIn( ... )
%    As above, but also return an additional parameter that is format-
%    dependent.  
%	for Navy (.str) files, this is the number of beams
%	for MATLAB files, it's the number of sounds (i.e. number of
%		columns of the matrix)
%	for Sun/Next .snd and .au files, it's the number of channels
%    Other formats don't use it yet.

if (nargin < 2), offset = 0; end
if (nargin < 3), maxRead = Inf; end
if (maxRead < 0), maxRead = Inf; end

left = -1;                       % unknown
sRate = 0;                       % unknown
pOut  = 1;
try_read = 0;
ext = pathExt(filename);

if (strcmp(ext, 'm22') | strcmp(ext, 'mat')),
  if (strcmp(ext, 'm22')), sRate = 22255; end;		% may be overridden
  x = length(filename) - 4;
  if (filename(x-1:x) == '_5'), sRate = 22255/4;  end	% BARP filename hacks
  if (filename(x-1:x) == '_2'), sRate = 22255/11; end
  if (filename(x-1:x) == '_h'), sRate = 500;      end
  
  if (strcmp(ext, 'mat'))				% MATLAB file
    if (nargin < 4), pIn = 1; end
    varN = 1;                     % first locate the 'Csnd' or 'sound' variable
    while (1)
      [dummy, name, sz] = readMatlabFile(filename, varN, 0);
      if (strcmp(name, 'Csnd') | strcmp(name, 'sound')), break;
      else varN = varN + 1;  
      end
    end      
    maxRead = min(maxRead, sz(1) - offset);
    sound = readMatlabFile(filename, offset + sz(1)*(pIn-1), maxRead, varN);
    left = sz(1) - maxRead - offset;
    pOut = sz(2);
  else
    try_read = 1;					% SoundEdit file
  end

elseif (strcmp(ext, 'snd') | strcmp(ext, 'au'))
  if (nargin < 4), pIn = 1; end
  % magic offset nbytes format srate nchan
  vec = readBinary(filename, 0, 6, 'l');
  off1  = vec(2);
  fmt   = vec(4);
  sRate = vec(5);
  nchan = vec(6);
  
  magic = sum('.snd' .* (256 .^ (3:-1:0)));
  if (magic ~= vec(1))
    printf('Warning: file "%s" has a bad magic number.', filename);
  end
  
  if     (fmt == 1), sz = 1; typ = 'uchar'; mu = 1;	% mu-law
  elseif (fmt == 2), sz = 2; typ = 'char';  mu = 0;	%  8-bit linear
  elseif (fmt == 3), sz = 2; typ = 'short'; mu = 0;	% 16-bit linear
  elseif (fmt == 5), sz = 4; typ = 'long';  mu = 0;	% 32-bit linear
  elseif (fmt == 6), sz = 4; typ = 'float'; mu = 0;	% 32-bit float
  elseif (fmt == 7), sz = 8; typ = 'double';mu = 0;	% 64-bit float
  else
    error(sprintf('Sound data format %g that I can''t handle in file ', ...
	fmt, filename));
  end
  [dummy,nSamp] = readBinary(filename, off1, 0, typ);
  nSamp = floor(nSamp / nchan);
  maxRead = min(maxRead, nSamp - offset);
  sound   = readBinary(filename, off1 + offset*nchan*sz, maxRead, typ);
  left    = nSamp - maxRead - offset;
  if (nchan > 1), sound = sound(pIn : nchan : length(sound)); end
  if (mu), sound = mu2lin(sound); end
  pOut = nchan;

elseif (strcmp(ext, 'str')),			% Navy file
  if (nargin < 4), pIn = 1; end
  % nsamp nbeams day hour minute second     sRate(float)
  vec     = readBinary(filename,   0, 6, 'l');
  sRate   = readBinary(filename, 4*6, 1, 'f');
  nSamp   = vec(1);
  pOut    = vec(2);
  nBeamsMax = 40;
  offset  = 28 + nBeamsMax*4 + (pIn-1)*nSamp*4 + offset*4;
  maxRead = min(maxRead, nSamp - offset);
  sound   = readBinary(filename, offset, maxRead, 'f');
  left    = nSamp - maxRead - offset;		% samples left ON THIS BEAM
    

elseif (strcmp(ext, 'aif'))
  [nchan,nSamp,dummy,sRate] = aiffchunk(filename, 'COMM');
  maxRead = min(maxRead, nSamp - offset);
  sound   = aiffchunk(filename, 'SSND', offset, maxRead, 1);
  left    = nSamp - maxRead - offset;
  
elseif (strcmp(ext, 'adc')       | ...
        strcmp(ext, 'dac')       | ...
	strcmp(ext, 'wav')       | ...
	strcmp(ext, 'm11')       | ...
	strcmp(ext, 'm7')        | ...
        strcmp(ext, 'dy22')      | ...
        strcmp(ext, 'dy11')      | ...
        strcmp(ext, 'irc')       | ...
	strcmp(ext, 'macspeech'))
    try_read = 1;
  
elseif (strcmp(ext(1), 'b'))			% binary; should come last
  if (strcmp(ext, 'b1_3'))			% special case
    sRate = 128;
  else
    y = find(ext == '_');			% convert '_' to '.'
    ext(y) = setstr(ones(1,length(y)) * '.');
    if (length(str2num(ext(2:length(ext)))) > 0)
      sRate = 100 * str2num(ext(2:length(ext)));
    else
      error(['Unknown file format, thought to be binary, for file ',filename]);
      end
    end
  
  [dummy,nSamp] = readBinary(filename, 0, 0, 's');
  maxRead = min(maxRead, nSamp - offset);
  sound   = readBinary(filename, offset*2, maxRead, 's');
  left    = nSamp - maxRead - offset;

else
   error(['Can''t figure out file type of ', filename]);
end

if (try_read)
  if (offset ~= 0)
    disp(['Warning: soundIn: Can''t use non-zero offset fo',...
	    'r files of type ', filename]);
  end
  [sound,sRate] = readSound(filename);			% mex-file reading
  nSamp = 0;
  left = 0;
end
