function result = spectrogram(seq, frameSize, nOverlap, zeroPad)
%SPECTROGRAM Compute the short-time periodic windowed FFT of a signal.
%
% gram = SPECTROGRAM(sequence, frameSize, nOverlap [,zeroPad]):
% For each frame, window it with a Hanning window,
% compute the FFT, remove the negative frequencies,
% and take the log of the resulting elements.  Then hop ahead
% by (frameSize - nOverlap) elements to the next frame.
%
% If the zeroPad argument is supplied, that many zeros are appended
% to the frame before computing the FFT.  zeroPad+frameSize should be
% a power of two.
%
% The resulting array has (frameSize+zeroPad)/2 rows and
% a number of columns determined by the framesize and overlap values.
% Low frequencies come first, i.e. they have low row indices.

% Sample seq:     seq = readSound('B2311_1150.m7');
%                 spect = spectrogram(seq, 512, 0);
%                 mesh(spect);

if (nargin < 4), zeroPad = 0; end

[inrows, incols] = size(seq);
if inrows ~= 1 & incols ~= 1,
    error('Spectrogram requires a 1-dimensional sequence');
end

nfft = frameSize + zeroPad;
seqsize = max(inrows, incols);
outcols = floor(1 + (seqsize - frameSize) / (frameSize - nOverlap));
outrows = nfft / 2;
result = zeros(outrows, outcols);
window = hanning(frameSize);             % column vector

for i = 1:outcols
  start = 1 + (frameSize - nOverlap) * (i - 1);
  frame = reshape(seq(start:(start + frameSize - 1)), frameSize, 1);
  spectrum = fft(frame .* window, nfft);   % column vector
  %result(:,i) = abs(spectrum(1:outrows)) .^ 2;
  result(:,i) = abs(spectrum(1:outrows));
end
result = log(abs(result));		% move inside loop if memory problems
