function [z,p,k] = ellipapf(n, ws, rs)

%ELLIPAPF Elliptic analog lowpass filter prototype.
%	[Z,P,K] = ELLIPAPF(N,Ws,Rs) returns the zeros, poles, and gain
%	of an N-th order normalized prototype elliptic analog lowpass
%	filter with a stopband Rs decibels down, starting at Ws, 
%	with Ws being normalized to the cut-off frequency.

% 	Created from ELLIPAP.M

% Bernard Gosselin, gosselin@tcts.fpms.ac.be
% April 15, 1996.

if n == 1,     % special case; for n == 1, reduces to chebyshev type 1
	z = [];
	p = -sqrt(ws/(10^(rs/10)-1));	% only valid for chebyshev, with n==1
	k = -p;
	return
end

wp = 1;	% passband edge - normalized

m=(1/ws)^2;
if abs(1-m) < eps
	krat=0;
else
	capk=ellipke([m,1-m]);
	krat=capk(1)/(capk(2)*n); 	% krat -- need to find relevant k1 (Rp) so that 
					% n=(capk(1)*capk1(2))/(capk(2)*capk1(1))
end

rsrat=1/(10^(rs/10)-1);

% try to find epsilon, so that n=(capk(1)*capk1(2))/(capk(2)*capk1(1)),
% with elliptic parameter k1^2=epsilon^2 / (10^(rs/10)-1);
% As k1^2 is very small, finding epsilon instead of k1^2 improves the 
% accuracy of the solution
global ELLIP_KRAT
ELLIP_KRAT = krat;
global ELLIP_RSRAT
ELLIP_RSRAT=rsrat;
epsilon = fmins('epsratio',0.0);

k1sq=epsilon^2*rsrat;
capk1= ellipke([k1sq,1-k1sq]);
k1=sqrt(k1sq);
k1p=sqrt(1-k1sq);
rp=10*log10(1+epsilon^2);	%rp, dB of passband ripple

capk = ellipke(m);

% code starting here is the same as in ELLIPAP

m1 = 1 - m;
i = sqrt(-1);

% find zeros; they are purely imaginary and paired in complex conjugates
j = (1-rem(n,2)):2:n-1;
[ij,jj] = size(j);
% s is Jacobi elliptic function sn(u)
[s,c,d] = ellipj(j*capk/n,m*ones(ij,jj));
is = find(abs(s) > eps);
z = 1 ./(sqrt(m)*s(is));
z = i*z(:);	% make column vector
z = [z ; conj(z)];
% order the zeros for convenience later on
z = cplxpair(z);

% poles; one purely real if n is odd - the remainder are complex conjugate pairs
% put 1/epsilon, mp into global variables for calculating v0
global ELLIP_EPS ELLIP_MP;
ELLIP_EPS = 1 / epsilon;
ELLIP_MP = k1p^2;
% calculate v0, a 'fundamental' parameter for the poles related to inverse sc
% function . I.e. find r so sn(r)/cn(r) = 1/epsilon for the given parameter mp
r = fmins('vratio', ellipke(1-m));
v0 = capk*r/(n*capk1(1));
[sv,cv,dv] = ellipj(v0,1-m);
p = -(c.*d*sv*cv + i*s*dv)./(1-(d*sv).^2);
p = p(:);   % make column vector
% check to see if there is a real pole
if rem(n,2)
	ip = find(abs(imag(p)) < eps*norm(p));
	[pm,pn] = size(p);
	pp = 1:pm;
	pp(ip) = [];
	p = [p ; conj(p(pp))];
else
	p = [p; conj(p)];
end
p = cplxpair(p);	% order poles for later use

% gain
k = real(prod(-p)/prod(-z));
if (~rem(n,2))	% n is even order so patch gain
	k = k/sqrt((1 + epsilon^2));
end
