function [num, den, z, p] = ellipfix(n, Rs, Wn, Ws, ftype, anaflag)

%ELLIPFIX
%	Elliptic or Cauer digital and analog filter design.
%	[B,A] = ELLIPFIX(N,Rs,Wn,Ws) designs an N'th order lowpass digital
%	elliptic filter with a stopband Rs decibels down, and starting at Ws.
%	 ELLIPFIX returns the filter
%	coefficients in length N+1 vectors B and A.  The cut-off
%	frequency Wn must be 0.0 < Wn < 1.0, with 1.0 corresponding to
%	half the sample rate.  Use Rs = 20 as starting
%	point, if you are unsure about choosing it.
%
%	If Wn is a two-element vector, Wn = [W1 W2], ELLIPFIX returns an 
%	order 2N bandpass filter with passband  W1 < W < W2.
%	[B,A] = ELLIP(N,Rs,Wn,Ws,'high') designs a highpass filter.
%	[B,A] = ELLIP(N,Ws,Rs,Wn,Ws,'stop') is a bandstop filter if Wn = [W1 W2].
%
%	When used with three left-hand arguments, as in
%	[Z,P,K] = ELLIPFIX(...), the zeros and poles are returned in
%	length N column vectors Z and P, and the gain in scalar K. 
%
%	When used with four left-hand arguments, as in
%	[A,B,C,D] = ELLIPFIX(...), state-space matrices are returned. 
%
%	ELLIPFIX(N,Rs,Wn,Ws,'s'), ELLIPFIX(N,Rs,Wn,Ws,'high','s') and 
%	ELLIPFIX(N,Rs,Wn,Ws,'stop','s') design analog elliptic filters.  
%	In this case, Wn can be bigger than 1.0.
%
%	See also ELLIPORD, ELLIP, CHEBY2, BUTTER, BESSELF, FREQZ and FILTER.

%	Created from ELLIP.M

% Bernard Gosselin, gosselin@tcts.fpms.ac.be
% April 15, 1996.


btype = 1;
if (nargin == 5)
        if strcmp(ftype,'s')
		analog = 1;
	elseif strcmp(ftype,'z')
		analog = 0;
        else    % band-stop or high-pass
		btype = 3;  analog = 0;
	end
elseif (nargin == 6)
	btype = 3;
	if strcmp(anaflag,'s')
		analog = 1;
	elseif strcmp(anaflag,'z')
		analog = 0;
        else
		error('last input argument should be ''s'' or ''z''.');
	end
	btype = 3;  % band-stop or high-pass
else
	analog = 0;
end
if max(size(Wn)) == 2
	btype = btype + 1;
end

% step 1: get analog, pre-warped frequencies
if ~analog,
	fs = 2;
	u = 2*fs*tan(pi*Wn/fs);
	us= 2*fs*tan(pi*Ws/fs);
else
	u = Wn;
	us=Ws;
end

% step 2: convert to low-pass prototype estimate with passband edge of 1 
% and stopband edges determined by the following:
if btype == 1	% lowpass
	Wn = u;
	Ws=us/u;
elseif btype == 2	% bandpass
	Bw = u(2) - u(1);
	Wn = sqrt(u(1)*u(2));	% center frequency
	Ws=(us.^2-u(1)*u(2))./(us*Bw);
elseif btype == 3	% highpass
	Wn = u;
	Ws=u/us;
elseif btype == 4	% bandstop
	Bw = u(2) - u(1);
	Wn = sqrt(u(1)*u(2));	% center frequency
	Ws=us*Bw./(u(1)*u(2)-us.^2);
end

% find the more demanding spec:
Ws=min(abs(Ws));

% step 3: Get N-th order elliptic lowpass analog prototype
[z,p,k] = ellipapf(n, Ws, Rs);

% Transform to state-space
[a,b,c,d] = zp2ss(z,p,k);

% step 4: Transform to lowpass, bandpass, highpass, or bandstop of desired Wn
if btype == 1		% Lowpass
	[a,b,c,d] = lp2lp(a,b,c,d,Wn);

elseif btype == 2	% Bandpass
	[a,b,c,d] = lp2bp(a,b,c,d,Wn,Bw);

elseif btype == 3	% Highpass
	[a,b,c,d] = lp2hp(a,b,c,d,Wn);

elseif btype == 4	% Bandstop
	[a,b,c,d] = lp2bs(a,b,c,d,Wn,Bw);
end

% step 5: Use Bilinear transformation to find discrete equivalent:
if ~analog,
	[a,b,c,d] = bilinear(a,b,c,d,fs);
end

if nargout == 4
	num = a;
	den = b;
	z = c;
	p = d;
else	% nargout <= 3
% Transform to zero-pole-gain and polynomial forms:
	if nargout == 3
		[z,p,k] = ss2zp(a,b,c,d,1);
		num = z;
		den = p;
		z = k;
	else % nargout <= 2
		den = poly(a);
		num = poly(a-b*c)+(d-1)*den;
	end
end

