/******************************************************************************
  filter2d.c

Matlab MEX file to compute the 2-dimensional discrete filtering problem
in the manner of FILTER.  Output signal y is related to input signal x
by filter coefficients b and a:

        y(m,n) = sum_{i,j} (b(i,j) x(m-i,n-j)  -  a(i,j) y(m-i,n-j))

At the moment, a(0,0) is ignored.

The two-dimensional Z transform of y is related to that of x by:

        Y(z1,z2) = X(z1,z2) B(z1,z2) / (1 + A(z1,z2))

Michael Maurer, 27 September 1994. 
Copyright (c) 1995 by Michael Maurer.
******************************************************************************/

#include <math.h>
#include "mex.h"

#define MAX(x,y)     (((x) > (y)) ? (x) : (y))
#define MIN(x,y)     (((x) < (y)) ? (x) : (y))
#define IDX(i,j,m)   ((m)*(j)+(i))

static void
filter2d(double *b,
         unsigned int mb,
         unsigned int nb,
         double *a,
         unsigned int ma,
         unsigned int na,
         double *x,
         unsigned int mx,
         unsigned int nx,
         double *y)
{
   unsigned int i,j,ib,jb,ia,ja,kx,ky,ka,kb;

   for (i=0; i<mx; i++) {
      for (j=0; j<nx; j++) {
         ky = IDX(i,j,mx);
         y[ky] = 0;
         for (ib=0; ib<mb && ib<=i; ib++) {
            for (jb=0; jb<nb && jb<=j; jb++) {
               kb = IDX(ib,jb,mb);
               kx = IDX(i-ib,j-jb,mx);
               y[ky] += b[kb]*x[kx];
            }
         }
         for (ia=0; ia<ma && ia<=i; ia++) {
            for (ja=0; ja<na && ja<=j; ja++) {
               if (ia==0 && ja==0) continue;            /* ignore a[0,0] for now */
               ka = IDX(ia,ja,ma);
               kx = IDX(i-ia,j-ja,mx);
               y[ky] -= a[ka]*y[kx];
            }
         }
      }
   }
}

#define IN_B prhs[0]
#define IN_A prhs[1]
#define IN_X prhs[2]
#define OUT_Y plhs[0]

void
mexFunction(int nlhs,
            Matrix *plhs[],
            int nrhs,
            Matrix *prhs[])
{
   double *pb,*pa,*px,*py;
   unsigned int mb,nb,ma,na,mx,nx;

   if (nrhs != 3)
      mexErrMsgTxt("Three input arguments required.");
   
   pb = mxGetPr(IN_B);
   mb = mxGetM(IN_B);
   nb = mxGetN(IN_B);

   pa = mxGetPr(IN_A);
   ma = mxGetM(IN_A);
   na = mxGetN(IN_A);

   px = mxGetPr(IN_X);
   mx = mxGetM(IN_X);
   nx = mxGetN(IN_X);

   OUT_Y = mxCreateFull(mx,nx,REAL);
   py = mxGetPr(OUT_Y);

   filter2d(pb,mb,nb,
            pa,ma,na,
            px,mx,nx,
            py);
}
