function [ret0,ret1,n] = opColorMap(cmd, ch, bright, contr)
% opColorMap('setlimits', chan, brightness, contrast)
%    Set colormap limits when drawing a new image.
%
% [lolim,hilim,ngrays] = opColorMap('getlimits', chan)
%    Return the range of values which should be converted to pixel values
%    ranging from 1 to ngrays.
%
% ngrays = opColorMap('getlimits', chan)
%    With only one return value, return the number of gray values.
%
% repaint = opColorMap('install', chan, brightness, contrast)
%    If possible, install a colormap for the given brightness and 
%    contrast values and return 1.  If not possible, return 0 to indicate
%    that the pixels must be repainted.
%
% [b,c] = opColorMap('setbc', chan, spect)
%    Given a spectrogram, calculate initial brightness and contrast values.
%
% opColorMap('setgram', colorfunction)
%    Set the name of the colormap function that constructs the colormap to use.
%    The colorfunction should be a string calling a function with an arg of n,
%    as, for example, 'gray(n)'.
%
% opColorMap('setsel', rgb)
%    Set the selection color to the given 3-element RGB vector.

% Color handling:
% Brightness and contrast control two linear mappings:
% From spectrogram values into pixel values, and from pixel values
% into grey levels.  whiteval and blackval are the spectrogram
% values that are mapped to pixel values c1 and c0, respectively,
% which are mapped to white and black, respectively, in the color 
% map.  opLoLimit and opHiLimit are mapped to pixel values 1 to n.
%
% When pixel values are computed (see opBitsToPix), the color
% map is set up so that the range from black to white spans
% only a portion of the available (n=48) colors.  This is so that
% small changes to brightness or contrast can be effected by 
% changing opLoLimit and opHiLimit and installing a different 
% color map.

global opLoLimit opHiLimit opColor opColorDepth opT0 opT1 opF0 opF1 
global opIconColorMap opColorMapName opAxes opBrightness opContrast

n = 48;					% number of grays
span = 0.8;				% big=>fine, small=>coarse color map
minnie = log(1e0);			% min displayable value
maxie  = log(1e5);			% max displayable value

if (nargin >= 4)
  contr = max(1-contr, 0.001);		% slider sense is reverse of color map
end

if (strcmp(cmd, 'getlimits'))		% return limits
  if (nargout > 1)
    ret0 = opLoLimit(ch);
    ret1 = opHiLimit(ch);
  else
    ret0 = n;
  end

elseif (strcmp(cmd, 'setbc'))		% return brightness and contrast

  % Heuristic: 85% of pixels are white; 14% more scale linearly 
  % to half-grey; the rest scale linearly.
  spect = bright;
  pct = percentile(spect(:), [0.85  0.99]);
  white = pct(1);
  black = (pct(2) - pct(1)) * 2 + pct(1);
  ret0 =     ((black + white) - 2*minnie) / (maxie - minnie) / 2;  % brightness
  ret1 = 1 - ((black - white)           ) / (maxie - minnie) / 2;  % contrast

elseif (strcmp(cmd, 'setlimits'))		% set limits
  opLoLimit(ch) = (maxie - minnie) * (bright - contr) + minnie;
  opHiLimit(ch) = (maxie - minnie) * (bright + contr) + minnie;

elseif (strcmp(cmd, 'install'))		% try to install a color map
  
  % Construct color map.
  whiteval = (maxie - minnie) * (bright - contr * span) + minnie;
  blackval = (maxie - minnie) * (bright + contr * span) + minnie;
  
  range = opHiLimit(ch) - opLoLimit(ch);
  c0 = round((whiteval-opLoLimit(ch))/range * (n-1) + 1); % white before here
  c1 = round((blackval-opLoLimit(ch))/range * (n-1) + 1); % black after here
  
  if (c0 < 1 | c1 > n | c1-c0+1 < n/3)
    ret0 = 0;				% can't diddle cmap; must redraw pixels
    
  else
    
    eval(['basemap = ' opColorMapName ';']);

    % colormap: white in 1..c0; then gray shades in c0..c1; then black in c1..n
    map = [ones(c0-1,1) * basemap(1,:);
	   interp1((1:n)', basemap, linspace(1,n,c1-c0+1).');
	   ones(n-c1,1) * basemap(n,:)];

    % duplicate the map with yellow (or whatever) added for selections
    map = [map;
	   ones(n,1) * opColor * opColorDepth + (1-opColorDepth) * map;
	   opIconColorMap
	];

    colormap(map);
    ret0 = 1;
  end
  
elseif (strcmp(cmd, 'setgram'))
  opColorMapName = ch;
  for ch = 1 : length(opAxes)
    opColorMap('install', ch, opBrightness(ch), opContrast(ch));
  end

elseif (strcmp(cmd, 'setsel'))
  opColor = ch;
  for ch = 1 : length(opAxes)
    opColorMap('install', ch, opBrightness(ch), opContrast(ch));
  end

else
  error(['Osprey internal error: unknown command %s.', cmd]);

end
