function opSetFileInfo(ch, filename, srate)
% opSetFileInfo(ch, filename [,sRate])
%    For the given file, find out the sampling rate, the number of samples,
%    and the file type.  If there is a .spec file, set up the spectrogram
%    parameters for that spectrogram.
%
%    The filename may optionally have two lines, with the second line 
%    containing the sampling rate as a binary-file 'b' extension.
%    (See osprey.m, soundIn.m for an explanation.)
%
%    If the 'filename' is really a vector of samples, squirrel them away for
%    later use (see opSoundIn).  In this case the sampling rate must be 
%    specified as the second argument.
%
% See also opCurParams, opLoadSpect, opSaveSpect, opSoundIn.

global opSrate opNsamp opDataSize opHopSize opZeroPad opWinType opSpectParams 
global opBrightness opContrast opTMax opChannelCount opSignal

if (~isstr(filename))
  if (length(filename) < 64)
    error('You need to use either a file name or a reasonably long vector of samples.');
  end

  % User handed in a signal, rather than a file name.
  opSignal = filename;
  
  % Make sure it's columnar.
  if (nRows(opSignal) < nCols(opSignal))
    opSignal = opSignal.';
  end
  
  opFileName(ch, 'setsound', '');
  opSrate(ch) = srate;
  opNsamp(ch) = nRows(opSignal);
  opTMax(ch) = opNsamp(ch) / opSrate(ch);
  opChannelCount = nCols(opSignal);

else

  % File name.  Set up params.
  [dummy,opSrate(ch),opNsamp(ch),opChannelCount] = soundIn(filename,0,0);
  opChannelNum = 0;
  opTMax(ch) = opNsamp(ch) / opSrate(ch);
  typ = (opSrate(ch) > 0);
  if (typ)
    opFileName(ch, 'setsound', filename);
  else
    opFileName(ch, 'setsound', '');
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Now look for a spect file and set things up if it's there.
  
  filename = filename(1,:);
  spectname = [filename(1:length(filename)-length(pathExt(filename))), 'spec'];
  opFileName(ch, 'setspect', spectname);
  fid = fopen(spectname, 'r');
  if (fid >= 0),
    fclose(fid);
    info = readBinary(spectname, 0, 8, 'l');
    opSpectVer(ch)   = info(1);
    % don't set srate; it has to match for spect-file reading to work
    opDataSize(ch)   = info(3);
    opHopSize(ch)    = info(4) / info(3);
    opZeroPad(ch)    = info(5);
    opWinType(ch)    = info(6);
    opBrightness(ch) = info(7) / 1000;
    opContrast(ch)   = info(8) / 1000;
    opSpectParams(ch, 1:5) = info(2:6).';
    frameSize = (opDataSize(ch) * (1+opZeroPad(ch))) / 2;	% == fftSize/2
    [dummy,nnum] = readBinary(spectname, frameSize*4, 0, 'f');
    opNsamp(ch) = (nnum/frameSize - 1) * info(4) + opDataSize(ch);
    opTMax(ch) = opNsamp(ch) / opSrate(ch);
    typ = typ + 2;
  end
  
  if (typ == 0)
    error(['Can''t find sound or spect file ', filename]);
  end
end
