function [y0,y1,y2,y3] = aiffchunk(filename, typ, start, nframe, chans)
%AIFFCHUNK   Read a single 'chunk' of an AIFF file.
%
% [nchan,nframe,sampleSize,sRate] = aiffchunk(filename, 'COMM')
% From the given AIFF file, read the Common Chunk and return, respectively,
% the number of channels (e.g. 2 for stereo), the number of sample frames 
% (i.e. number of samples in each channel), the sampleSize in bits, and
% the sampling rate.
%
% sams = aiffchunk(filename, 'SSND', start, n, chans)
% From the AIFF file, read the Sound Data Chunk and return samples.
% 'start' is the sample number to start reading at (the first sample is 
% start=0), and n is the number of samples per channel to read.  If n=Inf,
% read the whole sound.  chans is a vector specifying which channels are
% desired, with channel 1 being the first channel.  All of the arguments
% after 'SSND' are optional; if chans is missing, all channels are 
% returned.
%    The samples are returned in a matrix, with each channel's samples
% filling one column of the matrix.  The matrix has as many columns
% as the length of chans.
%
% bytes = aiffchunk(filename, other_type)
% From the AIFF file, read a chunk whose type is given by other_type,
% and return it as a vector of bytes (values 0-255).  The entire chunk 
% is returned.

% Dave Mellinger  2/3/94   dkm1@cornell.edu

if (~strcmp(typ, 'COMM'))
  [nc,nf,ss] = aiffchunk(filename, 'COMM');	% work around MATLAB misfeature
end

fd = fopen(filename, 'r');
if (fd < 0), error(['File not present: ', filename]); end
[x,n] = fread(fd, 12, 'char');
if (n<12 | ~strcmp(x(1:4).', 'FORM') | ~strcmp(x(9:12).', 'AIFF'))
  fclose(fd);
  error(['File is not an AIFF file; can''t read it: ', filename]);
end

cont = 1;
while(cont)
  x = fread(fd, 4, 'char').';
  [len,n] = fread(fd, 1, 'long');
  if (n < 1)
    fclose(fd); error(['Unexpected end of file while reading ', filename]);
  end
  if (strcmp(x, typ))
    cont = 0;
  else
    fseek(fd, len, 0);			% skip len bytes
  end
end

if (strcmp(typ, 'COMM'))
  y0       = fread(fd, 1, 'short');	% nChan
  y1       = fread(fd, 1, 'ulong');	% nFrame
  y2       = fread(fd, 1, 'ushort');	% sampleSize
  exp      = fread(fd, 1, 'ushort');	% read a 10-byte float
  [mant,n] = fread(fd, 2, 'ulong');
  if (n < 1), 
    fclose(fd); error(['Unexpected end of file when reading ', filename]);
  end
  mant = mant(1) / 2^31 + mant(2) / 2^63;
  if (exp >= 32768), mant = -mant; exp = exp - 32768; end
  exp = exp - 16383;
  y3 = mant * 2^exp;			% sRate
  
elseif (strcmp(typ,'SSND'))
  if (nargin < 3), start  = 0;     end
  if (nargin < 4), nframe = nf;    end
  if (nargin < 5), chans  = 1:nc;  end
  if (nframe == Inf), nframe = nf; end
  off = fread(fd, 1, 'ulong');
  nbyte = ceil(ss/8);
  dtype = ['int', num2str(nbyte*8)];
  fseek(fd, 4+off + nbyte*start*nc, 0);	% skip blockSize ulong, unused bytes
  y0 = [];
  while (nframe > 0)
    q = min(nframe, round(1048576 / nc));	% read roughly 1M at once
    [s,n] = fread(fd, q*nc, dtype);
    if (n < q*nc)
      fclose(fd); error(['Unexpected end of file when reading ', filename]); 
    end
    s = reshape(s, nc, length(s)/nc);
    if (nbyte*8 ~= ss), s = s / 2^(nbyte*8 - ss); end	% fix left-aligned bits
    y0 = [y0; s(chans, :).'];
    nframe = nframe - q;
  end

else
  y0 = fread(fd, len, 'uchar');

end

fclose(fd);
