function [typ,outfilename,ext,chan] = soundType(filename)
% typ = soundType(filename)
%
%    Given a file name, figure out what type it is and return it as a text 
%    string.  Often the type is the same as the extension, but not always.
%    Handles multi-row filenames (where the second row specifies a filetype
%    extension or a channel number) correctly.
%
%    Known types:
%
%	Type	Extension(s)
%	----	------------
%	mat	.mat, .m11, .m22		% MATLAB file
%	snd	.snd, .au			% NeXT/Sun audio file
%	aif	.aif				% AIFF file
%	wav	.WAV, .wav			% WAVE file
%	str	.str				% Navy format
%	whole	.adc, .dac, .m11, .m7, .dy22, .dy11, .irc, .macspeech
%	binary	.b1, .b8, .b1_3, .b223, etc.	% 16-bit linear
%	none	everything else
%
%    The file type "whole" is for files that can be read only in their entirety
%    with readSound.  Unfortunately, readSound doesn't exist on Suns.
%
%    Handles two-row filenames correctly.  The optional second and third rows
%    can have things like 'type=aif' and 'chan=3' to specify, respectively,
%    the filetype for files that don't have the appropriate extension, and
%    the channel number to use for a multi-channel file.
%
% [typ,filename,ext] = soundType(filename)
%    As above, but also return the extension and filename as separate entities.
%    This is useful when multi-row filenames are a possibility.
%
% [typ,filename,ext,chan] = soundType(filename)
%    As above, but also return the channel number specified on a multi-row
%    filename.  If no channel is specified, chan is [].

outfilename = filename(1,:);		% may get changed below
ext = pathExt(outfilename);		% may get changed below
chan = [];				% may get changed below

% Process the multi-row filename items.
for i = 2:nRows(filename)
  line = deblank(filename(i,:));
  
  if (strncmp(line, 'type', 4))		% check 'type'
    ext = deblank(line(6:length(line)));
    if (ext(1) == '.') 
      ext = ext(2:length(ext)); 
    end
  elseif (strncmp(line, 'chan', 4))	% check 'chan'
    x = line(6:length(line));		% do not combine lines (MATLAB 4.2c bug)
    chan = sscanf(x, '%d');
  else
    error(['Unknown multi-row filename item: ', line])
  end
  
  % If this was the longer row, deblank the filename.
  if (nCols(line) == nCols(outfilename))
    outfilename = deblank(outfilename);
  end
end

% Is it a binary file extension?
if (length(ext) >= 2)
  if (strcmp(ext(1), 'b'))
    e = strsubst(ext, '_', '.');
    [a,count,err,nxt] = sscanf(e(2:length(e)), '%g');
    if (nxt == length(e))
      typ = 'binary';
      return
    end
  end
end

if (  strcmp(ext, 'mat') | ...			% MATLAB file
      strcmp(ext, 'm22') | ...
      strcmp(ext, 'm11'))
  typ = 'mat';
  
elseif (strcmp(ext, 'snd') | ...		% Sun/NeXT sound file
        strcmp(ext, 'au'))
  typ = 'snd';
  
elseif (strcmp(ext, 'str')),			% Navy file
  typ = 'str';
  
elseif (strcmp(ext, 'aif'))			% AIFF file
  typ = 'aif';
  
elseif (strcmp(lower(ext), 'wav'))		% WAV file
  typ = 'wav';
  
elseif (strcmp(ext, 'adc')       | ...		% must read whole file at once
        strcmp(ext, 'dac')       | ...
	strcmp(ext, 'wav')       | ...
	strcmp(ext, 'm11')       | ...
	strcmp(ext, 'm7')        | ...
        strcmp(ext, 'dy22')      | ...
        strcmp(ext, 'dy11')      | ...
        strcmp(ext, 'irc')       | ...
	strcmp(ext, 'macspeech'))
  typ = 'whole';

%elseif (isempty(ext))
  % Special case for Harold Mills -- extensionless files are 22050 Hz binary.
  %typ = 'harold';

else						% unknown or non-soundfile
  typ = 'none';

end
