function [sound,sRate,left,nchan] = soundIn(filename, offset, maxRead, chan)
%soundIn         Read part or all of a sound file.  Knows several file formats.
%
% sound = soundIn(filename)
%    Read a sound file, return the sound as a column vector.  The file format
%    is determined from the file extension.  Works on these types of files:
%
%	file type	extension
%	---------	---------
%	MATLAB files	.mat	(see below)
%	binary files	.b<n>   (see below)
%	AIFF files	.aif
%	Sun/NeXT audio	.au or .snd
%	WAV		.WAV or .wav (see below)
%       other		.adc, .dac, .m11, .m7, .dy22, .dy11, .irc, .macspeech
%
%    For MATLAB files, there must be variables named 'sound' and 'srate'.  
%
%    Binary files are specified with names like 'b8', where the 8 specifies 
%    the sampling rate in hundreds of Hertz (so .b8 would be 800 Hz).
%    Binary files are assumed to have mono 16-bit linear samples.
%
%    For WAV files, only 8-, 16-, and 32-bit linear format is handled; 
%    see wavchunk.m.  16-bit linear is also called "Microsoft PCM" format.
%
%    If the filename is a string array with more than one row, then 
%    the additional rows can be used to specify certain things about 
%    the file:
%
%	type=<extension>
%		Specify a file type.  This is useful when the file itself
%		doesn't have the right extension.  Examples:
%		    typ=b2	specify that it's a 200-Hz binary file
%		    typ=aif	specify that it's an AIFF file
%		    typ=WAV	specify that it's a WAV file
%		The extension is NOT appended to the file name; it's used
%		only for determining the file type.
%	chan=<n>
%		Specify a channel number.  This is useful only for multi-channel
%		files like AIFF and WAV files.  Channel numbers start from 0,
%		even for MATLAB files.  If the channel number is passed in both
%		the filename and the the 'chan' argument, the argument wins.
% 
% sound = soundIn(filename, offset)
%   As above, but skip the first offset samples in the file.
%   This works only for MATLAB, binary, AIFF, and .snd/.au files.
%
% sound = soundIn(filename, offset, maxRead)
%    As above, but read at most maxRead samples.  If maxRead is Inf
%    or is not present (or is less than 0), read to the end of the file.
%
% sound = soundIn(filename, offset, maxRead, chan)
%    Specify the channel number to read.  Channel numbering begins at 0.
%    This is meaningful only for multi-channel formats, like .aif.
%
% [sound,sRate,left,nchan] = soundIn( ... )
%    Additional return arguments:
%	sRate	sampling rate, in Hz; works well with maxRead=0
%	left	number of samples left in the file (all channels) after reading
%	nchan	number of channels in the file (for multi-channel formats)

% Dave Mellinger
% 3/96

if (nargin < 2), offset = 0; end
if (nargin < 3), maxRead = Inf; end
if (nargin < 4), chan = 0; end
if (maxRead < 0), maxRead = Inf; end

left = -1;                       % unknown
sRate = 0;                       % unknown
nchan  = 1;
try_read = 0;
[typ,filename,ext,ch] = soundFileType(filename);
if (length(ch) & nargin < 4)
  chan = ch;
end

if (strcmp(typ, 'mat'))
  if (strcmp(ext, 'm22')), sRate = 22254+54/99; end;	% may be overridden
  x = length(filename) - length(ext) - 1;
  if (filename(x-1:x) == '_5'), sRate = 22255/4;  end	% BARP filename hacks
  if (filename(x-1:x) == '_2'), sRate = 22255/11; end
  if (filename(x-1:x) == '_h'), sRate = 500;      end
  
  if (strcmp(ext, 'mat'))				% MATLAB file
    varN = 1;			% first locate the 'Csnd' or 'sound' variable
    nchan = -1;
    while (nchan < 0 | ~sRate)	% go until both sound and sRate are found
      [dummy, name, sz] = readMatlabFile(filename, 0, 0, varN);
      if (strcmp(name, 'Csnd') | strcmp(name, 'sound'))	% Csnd is Canary's name
	maxRead = min(maxRead, sz(1) - offset);
	sound = readMatlabFile(filename, offset + sz(1)*chan, maxRead,varN);
	left = sz(1) - maxRead - offset;
	nchan = sz(2);
      end
      if (strcmp(lower(name), 'srate') | strcmp(lower(name), 'samplingrate'))
	sRate = readMatlabFile(filename, 0, 1, varN);
      end
      varN = varN + 1;
    end      
  else
    try_read = 1;					% SoundEdit file
  end

elseif (strcmp(typ, 'snd'))			% NeXT/Sun files
  % magic offset nbytes format srate nchan
  vec = readBinary(filename, 0, 6, 'l');
  off1  = vec(2);
  fmt   = vec(4);
  sRate = vec(5);
  nchan = vec(6);
  
  magic = sum('.snd' .* (256 .^ (3:-1:0)));
  if (magic ~= vec(1))
    printf('Warning: file "%s" has a bad magic number.', filename);
  end
  
  if     (fmt == 1), sz = 1; typ = 'uchar'; mu = 1;	% mu-law
  elseif (fmt == 2), sz = 1; typ = 'char';  mu = 0;	%  8-bit linear
  elseif (fmt == 3), sz = 2; typ = 'short'; mu = 0;	% 16-bit linear
  elseif (fmt == 5), sz = 4; typ = 'long';  mu = 0;	% 32-bit linear
  elseif (fmt == 6), sz = 4; typ = 'float'; mu = 0;	% 32-bit float
  elseif (fmt == 7), sz = 8; typ = 'double';mu = 0;	% 64-bit float
  else
    error(sprintf('Sound data format %g that I can''t handle in file ', ...
	fmt, filename));
  end
  [dummy,nSamp] = readBinary(filename, off1, 0, typ);
  nSamp = floor(nSamp / nchan);
  maxRead = min(maxRead, nSamp - offset);
  sound   = readBinary(filename, off1 + offset*nchan*sz, maxRead*nchan, typ);
  left    = nSamp - maxRead - offset;
  if (nchan > 1), sound = sound(chan+1 : nchan : length(sound)); end
  if (mu), sound = mu2lin(sound); end

elseif (strcmp(typ, 'str')),			% Navy file
  % nsamp nbeams day hour minute second     sRate(float)
  vec     = readBinary(filename,   0, 6, 'l');
  sRate   = readBinary(filename, 4*6, 1, 'f');
  nSamp   = vec(1);
  nchan    = vec(2);
  nBeamsMax = 40;
  byteoffset  = 28 + nBeamsMax*4 + chan*nSamp*4 + offset*4;
  maxRead = min(maxRead, nSamp - offset);
  sound   = readBinary(filename, byteoffset, maxRead, 'f');
  left    = nSamp - maxRead - offset;		% samples left ON THIS BEAM

elseif (strcmp(typ, 'aif'))
  [nchan,nSamp,dummy,sRate] = aiffchunk(filename, 'COMM');
  maxRead = min(maxRead, nSamp - offset);
  sound   = aiffchunk(filename, 'SSND', offset, maxRead, chan+1);
  left    = nSamp - maxRead - offset;
  
elseif (strcmp(typ, 'wav'))
  [nchan,dummy,sRate] = wavchunk(filename, 'fmt');
  [sound,left] = wavchunk(filename, 'data', offset, maxRead, chan+1);
  
elseif (strcmp(typ, 'whole'))
    try_read = 1;
  
elseif (strcmp(typ, 'binary'))			% binary
  % Get sampling rate from extension name.
  sRate = 100 * str2num(strsubst(ext(2:length(ext)), '_', '.'));
  if (strcmp(ext, 'b1_3'))			% special case
    sRate = 128;
  end
  
  [dummy,nSamp] = readBinary(filename, 0, 0, 's');
  maxRead = min(maxRead, nSamp - offset);
  sound   = readBinary(filename, offset*2, maxRead, 's');
  left    = nSamp - maxRead - offset;

elseif (strcmp(typ, 'harold'))
  % Special case for Harold Mills -- extensionless files are 22050 Hz binary.
  sRate = 22050;
  [dummy,nSamp] = readBinary(filename, 0, 0, 's');
  maxRead = min(maxRead, nSamp - offset);
  sound   = readBinary(filename, offset*2, maxRead, 's');
  left    = nSamp - maxRead - offset;

else
   error(['Can''t figure out file type of ', filename]);
end

if (try_read)
  if (offset ~= 0)
    disp(['Warning: soundIn: Can''t use non-zero offset for files of type ',...
	    filename]);
  end
  if (sRate == 0), [sound,sRate] = readSound(filename);	    % mex-file reading
  else                     sound = readSound(filename);	    % mex-file reading
  end
  maxRead = min(maxRead, length(sound) - offset);
  sound = sound(offset+1 : maxRead)
  nSamp = 0;
  left = 0;
end
