function [y0,y1,y2,y3] = wavchunk(filename, typ, start, nframe, chans)
%WAVCHUNK   Read a single 'chunk' of a WAV file.
%
% [nchan,sampleSize,sRate,fmt] = wavchunk(filename, 'fmt')
%    From the given WAV file, read the WAVE chunk (the main header) and return,
%    respectively, the number of channels (e.g. 2 for stereo), the sampleSize
%    in bits/sample, the sampling rate, and the format (1 means 16-bit linear).
%
% sams = wavchunk(filename, 'data', start, n, chans)
%    From the WAV file, read the sound data chunk and return samples.
%    'start' is the sample number to start reading at (the first sample is 
%    start=0), and n is the number of samples per channel to read.  If n=Inf,
%    read the whole sound.  chans is a vector specifying which channels are
%    desired, with channel 1 being the first channel.  All of the arguments
%    after 'data' are optional; if chans is missing, all channels are 
%    returned.  If the sound is shorter than n samples, then a shortened
%    sams array is returned.
%       The samples are returned in a matrix, with each channel's samples
%    filling one column of the matrix.  The matrix has as many columns
%    as the length of chans.
%       NOTE: The full format of .WAV files is not yet handled.  Only the
%    first data chunk is looked at, and only linear samples (Microsoft
%    PCM format) is processed.  Samples must be no more than 16 bits long.
%
% [sams,left] = wavchunk(filename, 'data' [,start [,n [,chans]]])
%    As above, but also return the number of samples per channel left after
%    reading.  You can use wavchunk(file, 'data', 0, 0) to get the number
%    of samples in the file.
% 
% bytes = wavchunk(filename, other_type)
%    From the WAV file, read a chunk whose type is given by other_type,
%    and return it as a vector of bytes (values 0-255).  The entire chunk 
%    is returned.

% Dave Mellinger  10/30/95   dkm1@cornell.edu

while (length(typ) < 4), typ = [typ ' ']; end

if (~strcmp(typ, 'fmt '))
  [nc,ss,fs,fmt] = wavchunk(filename, 'fmt ');	% work around MATLAB misfeature
end

fd = fopen(filename, 'r', 'l');		% l means little-endian, for PC files
if (fd < 0), error(['File not present: ', filename]); end
[x,n] = fread(fd, 12, 'char');
if (n<12 | ~strcmp(x(1:4).', 'RIFF') | ~strcmp(x(9:12).', 'WAVE'))
  fclose(fd);
  error(['File is not a .WAV file; can''t read it: ', filename]);
end

cont = 1;
while(cont)
  x = fread(fd, 4, 'char').';
  [len,n] = fread(fd, 1, 'long');
  if (n < 1)
    fclose(fd); error(['Unexpected end of file while reading ', filename]);
  end
  if (strcmp(x, typ))
    cont = 0;
  else
    fseek(fd, len, 0);			% skip len bytes
  end
end

if (strcmp(typ, 'fmt '))
  % Read header info.
  y3       = fread(fd, 1, 'short');	% fmt
  y0       = fread(fd, 1, 'short');	% nChan
  y2       = fread(fd, 1, 'long');	% sRate
  fseek(fd, 6, 0);			% bytesPerSec (4 bytes), blockAlign (2)
  y1       = fread(fd, 1, 'ushort');	% sampleSize
  
elseif (strcmp(typ, 'data'))

  if (fmt ~= 1)				% 1 is PCM format
    error(sprintf('Unknown format in .WAV file: %d (I only know PCM format)',...
	fmt));
  end

  if (nargin < 3), start  = 0;     end
  if (nargin < 4), nframe = inf;   end
  if (nargin < 5), chans  = 1:nc;  end

  nbyte = ceil(ss/8);			% bytes/sample

  % Can't use int/uint for the data type because they're always big-endian.
  if     (ss <=  8), dtype = 'uchar';
  elseif (ss <= 16), dtype = 'short';
  elseif (ss <= 32 & ss > 24), dtype = 'long';
  else error('Can''t read samples that are 17 to 24 bits wide.');
  end

  fseek(fd, nbyte*start*nc, 0);		% skip blockSize ulong, unused bytes
  ntot = len / nc / nbyte - start;	% number left in chunk (samples/chan)
  nleft = min(nframe, ntot);		% number left to read  (ditto)
  y0 = [];				% The Answer
  s = 1;				% dummy value to get loop started
  while (nleft > 0 & length(s) > 0)
    q = min(nleft, round(1048576 / nc));  % read roughly 1MB
    s = fread(fd, q*nc, dtype);
    s = reshape(s, nc, length(s)/nc);
    if (ss <= 8), s = s - 128; end			% fix 1-byte uint samps
    if (nbyte*8 ~= ss), s = s / 2^(nbyte*8 - ss); end	% fix left-aligned bits
    y0 = [y0; s(chans, :).'];
    nleft = nleft - nCols(s);
    ntot = ntot - nCols(s);
  end
  y1 = ntot;

else
  y0 = fread(fd, len, 'uchar');

end

fclose(fd);
