function xlims = xlim(lo, hi)
%XLIM		set x-limits of the current plot
%
% xlim(lo,hi)
%   This is shorthand for set(gca, 'XLimits', [lo hi]),
%   except that it does the right thing if lo > hi.
%
% xlim([lo hi])
%   Same as above.
%
% xlim(vec)
%   This is shorthand for set(gca, 'Xlimits', [min(vec) max(vec)]).
%
% xlim('auto')
%   Set the x-axis limits so the endpoints are nice round numbers.
%   MATLAB does the work.
%
% xlim('manual')
%   Freezes the limits so MATLAB will not change them upon, say, printing.
%
% xlim('fit')
%   Set the x-axis limits so the first and last data points in the plot are
%   at the ends of the displayed range.
%
% xlim('mouse')
%   Gets two mouse clicks from the user on the figure and sets the
%   x- and y-limits to the specified rectangle.
%
% xlim('mousex')
%   Gets two mouse clicks from the user on the figure and sets
%   the x-limit, but not the y-limit, to the specified range.
%
%
% xl = xlim(...)
%   All of the above forms return the new x-limits.

if (nargin < 2)
  if (nargin > 0)
    if (strcmp(lo, 'auto') | strcmp(lo, 'manual'))
      set(gca, 'XlimMode', lo);
    elseif (strcmp(lo, 'fit'))		% fit axis limits to data
      m = Inf;
      M = -Inf;
      for ch = get(gca, 'Children').'
	tt = get(ch, 'Type');
	%if (any(strmatch(tt, 'line','image','patch','surface')))
	if (any(strmatch(tt, str2mat('line','image','patch','surface'))))
	  d = get(ch, 'XData');
	  d(isnan(d) | isinf(d)) = [];
	  m = min(m, min(min(d)));
	  M = max(M, max(max(d)));
	end
      end
      if (~isinf(m) & ~isinf(M))
	set(gca, 'XLim', [m M]);
      end
    elseif (strncmp(lo, 'mouse', 5))		% mouse, mousex
      disp('Please mouse-click twice on the figure.')
      [x,y] = ginput(2);
      set(gca, 'XLim', [min(x) max(x)]);
      if (strcmp(lo, 'mouse'))
	set(gca, 'YLim', [min(y) max(y)]);
      end
    else
      if (length(lo) == 2), xlim(lo(1), lo(2));
      else xlim(min(lo), max(lo));
      end
    end
  end
else
  xl = get(gca, 'XLim');
  if (strcmp(get(gca, 'XDir'), 'reverse'))
    xl = fliplr(xl);
  end
  if (isnan(lo)), lo = xl(1); end
  if (isnan(hi)), hi = xl(2); end
  if (lo < hi)
    set(gca, 'XLim', [lo hi], 'XDir', 'normal');
  else
    set(gca, 'XLim', [hi lo], 'XDir', 'reverse')
  end
end

if (nargout > 0)
  xlims = get(gca, 'XLim');
end
