function ylims = ylim(lo, hi)
%YLIM		set y-limits of the current plot
%
% ylim(lo,hi)
%   This is shorthand for set(gca, 'YLimits', [lo hi]),
%   except that it does the right thing if lo > hi.
%
% ylim([lo hi])
%   Same as above.
%
% ylim(vec)
%   This is shorthand for set(gca, 'Ylimits', [min(vec) max(vec)]).
%
% ylim('auto')
%   Set the y-axis limits so the endpoints are nice round numbers.
%   MATLAB does the work.
%
% ylim('manual')
%   Freezes the limits so MATLAB will not change them upon, say, printing.
%
% ylim('fit')
%   Set the y-axis limits so the first and last data points in the plot are
%   at the ends of the displayed range.
%
% ylim('mouse')
%   Gets two mouse clicks from the user on the figure and sets the
%   y- and y-limits to the specified rectangle.
%
% ylim('mousey')
%   Gets two mouse clicks from the user on the figure and sets
%   the y-limit, but not the x-limit, to the specified range.
%
%
% yl = ylim(...)
%   All of the above forms return the new y-limits.

if (nargin < 2)
  if (nargin > 0)
    if (strcmp(lo, 'auto') | strcmp(lo, 'manual'))
      set(gca, 'YlimMode', lo);
    elseif (strcmp(lo, 'fit'))		% fit axis limits to data
      m = Inf;
      M = -Inf;
      for ch = get(gca, 'Children').'
	tt = get(ch, 'Type');
	%if (any(strmatch(tt, 'line','image','patch','surface')))
	if (any(strmatch(tt, str2mat('line','image','patch','surface'))))
	  d = get(ch, 'YData');
	  d(isnan(d) | isinf(d)) = [];
	  m = min(m, min(min(d)));
	  M = max(M, max(max(d)));
	end
      end
      if (~isinf(m) & ~isinf(M))
	set(gca, 'YLim', [m M]);
      end
    elseif (strncmp(lo, 'mouse', 5))		% mouse, mousey
      disp('Please mouse-click twice on the figure.')
      [x,y] = ginput(2);
      set(gca, 'YLim', [min(y) max(y)]);
      if (strcmp(lo, 'mouse'))
	set(gca, 'YLim', [min(x) max(x)]);
      end
    else
      if (length(lo) == 2), ylim(lo(1), lo(2));
      else ylim(min(lo), max(lo));
      end
    end
  end
else
  yl = get(gca, 'YLim');
  if (strcmp(get(gca, 'YDir'), 'reverse'))
    yl = fliplr(yl);
  end
  if (isnan(lo)), lo = yl(1); end
  if (isnan(hi)), hi = yl(2); end
  if (lo < hi)
    set(gca, 'YLim', [lo hi], 'YDir', 'normal');
  else
    set(gca, 'YLim', [hi lo], 'YDir', 'reverse')
  end
end

if (nargout > 0)
  ylims = get(gca, 'YLim');
end
