function [s,inicond,sigma]=sirps(M,distrib,R,input1,input2,input3);
% A SOURCE FOR SIRP-LIKE SPEECH MODEL SIGNALS
%                                                      mw24-jul-91
%   [s,inicond,sigma] = sirps(M,'distrib',R);
%                     = sirps(M,'distrib',R,inicond);
%                     = sirps(M,'distrib',R,numer,denom);
%                     = sirps(M,'distrib',R,numer,denom,inicond);
%
%   s.............SIRP-like signal
%   sigma.........sample of the sigma-process
%   M.............length of 's' and 'sigma'
%   distrib.......type of distibution;
%                 N/n=normal,  L/l=Laplacian,  K/k=K0,
%                 G/g=Gamma ,  S/s=speech
%   numer/denom...numerator/denominator coefficients of the synthesis
%                 filter (otherwise 's' is uncorrelated)
%   R.............interpolation factor 'R=0,1,2,3...'
%   inicond.......initial conditions
%                 'inicond(1)=seed of the random number generator'
%                 otherwise 'seed=0' is used
%
% EXAMPLE: first call  [s,inicond]=sirps(100,'k',8,1,[1 -.8]);
%          second call [s,inicond]=sirps(100,'k',8,1,[1 -.8],inicond);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% date      : 24-jul-91
% author    : Martin Werner
% address   : Lehrstuhl fuer Nachrichtentechnik
%             Universitaet Erlangen - Nuernberg
%             Cauerstr. 7
%             W-8520 Erlangen, Germany
%             werner@nt.e-technik.uni-erlangen.de
% references: Brehm, H.; Stammler, W.: Description and Generation of
%              Spherically Invariant Speech-Model Signals.
%              Elsevier Science Publisher B.V. (North-Holland),
%              Signal Processing 12 (1987), pp. 119-141.
%             Werner, M.; A source for SIRP-like speech model signals.
%              Elsevier Science Publisher B.V. (North-Holland),
%              Speech communication (submitted)
%-----------------------------------------------------------------------
% The program 'sirps' produces samples of length M of SIRP-like signals.
% The algorithm is based on the model of Spherically Invariant Random
% Processes (SIRPs) for bandlimited telephone speech. The signal ampli-
% tudes are either normal, Laplacian, K0, Gamma or 'speech' distributed.
% Using the built-in 'filter' routine for the synthesis filter, a moving
% average (MA), autoregressive (AR) or ARMA model can be selected. To
% avoide distortions in the correlation function and to ensure ellip-
% soidal or circular contourlines of the bidimensional probability
% density function of the signal samples at desired distances the inter-
% polation factor 'R' has to be choosen carefully, e.g. 'R=8'. 'Sirps'
% can be repeatly invoked without block edge effects by saving the
% necessary internal data in the vector 'inicond' for the next call.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%------ check input arguments

if nargin<3 | nargin>6
  error('sirpsmw/24-jul-91 >>incorrect number of input arguments');
end
if distrib~='N' & distrib~='L' & distrib~='K' & distrib~='G' &...
   distrib~='n' & distrib~='l' & distrib~='k' & distrib~='g' &...
   distrib~='S' & distrib~='s'
   error('sirpsmw/24-jul-91 >> error on input argument dist');
end

if nargin==3
  SYNFILTER=0; SEED=0;
  if distrib~='N' & distrib~='n'
%    SETUP=100; zilp=zeros(1:4); zioi=zeros(1:3); MNEW=M; MREST=0;
    SETUP=100; zilp=zeros(1,4); zioi=zeros(1,3); MNEW=M; MREST=0;
end; end

if nargin==4
  SYNFILTER=0; SETUP=0; SEED=input1(1);
  if distrib~='N' & distrib~='n'
    if size(input1)==[1,1]
      SETUP=100; zilp=zeros(1:4); zioi=zeros(1:3); MNEW=M; MREST=0;
    else
      SETUP=0; zilp=input1(2:5); zioi=input1(6:8);
      MREST=input1(9); MNEW=M-MREST;
      if MREST~=0; srest=input1(10:9+MREST);
end; end; end; end

if nargin==5
  SYNFILTER=1; numer=input1; denom=input2; ld=length(denom); SEED=0;
  zig=zeros(1:ld-1); SETUP=100;
  if distrib~='N'&distrib~='n';
    zilp=zeros(1:4); zioi=zeros(1:3); MNEW=M; MREST=0;
end; end

if nargin==6
  SYNFILTER=1; numer=input1; denom=input2; ld=length(denom); SEED=input3(1);
  if size(input3)==[1,1]
    zig=zeros(1:ld-1); SETUP=100;
    if distrib~='N'&distrib~='n';
      zilp=zeros(1:4); zioi=zeros(1:3); MNEW=M; MREST=0;
    end
  else
    SETUP=0; zig=input3(2:ld);
    if distrib~='N'&distrib~='n'
      zilp=input3(ld+1:ld+4); zioi=input3(ld+5:ld+7); MREST=input3(ld+8);
      MNEW=M-MREST;
      if MREST~=0; srest=input3(ld+9:ld+8+MREST);
end; end; end; end

%------ Gaussian factor process

rand('normal'); rand('seed',SEED);
if SYNFILTER==0
%-- uncorrelated
  gauss=rand(1,M); ggauss=gauss;
elseif SYNFILTER==1
%-- synthesis filter
  gauss=rand(1,SETUP+M);
  [ggauss zig]=filter(numer,denom,gauss,zig);  SEED=rand('seed');
  inicond=[SEED zig]; ggauss=ggauss(SETUP+1:SETUP+M);
end
if distrib=='N'|distrib=='n'; s=ggauss; return; end

%------ sigma-distributed factor process

%-- stored samples
if MNEW==0
  s=[srest(1:M).*ggauss(1:M)];
  inicond=[inicond zilp zioi 0]; return
end
if MNEW<0
  s=[srest(1:M).*ggauss(1:M)];
  inicond=[inicond zilp zioi MREST-M srest(M+1:MREST)]; return
end

%-- new samples
if R==0; lc=ceil(MNEW/4); else; lc=ceil(MNEW/(4*R)); end
ugauss=rand(1,SETUP+lc); inicond(1)=rand('seed');
[ugauss,zilp]=filter([6.1874322e-004 -2.1174012e-003 3.0208311e-003 ...
-2.1174012e-003 6.1874322e-004],[1 -3.8920476  5.6818603 -3.6874004 ...
.89759034],ugauss,zilp);
if SETUP>=2; zioi=ugauss(SETUP-2:SETUP); end
if lc==1; inicond=[inicond zilp zioi(2:3) ugauss(SETUP+1)];
elseif lc==2; inicond=[inicond zilp zioi(3) ugauss(SETUP+1:SETUP+2)];
else; inicond=[inicond zilp ugauss(SETUP+lc-2:SETUP+lc)]; end

%-- mss optimal interpolation by a factor 4
loi=4*lc; ugaussoi=zeros(1:loi);
ugauss=[zioi ugauss(SETUP+1:SETUP+lc)]; l=lc-1;
ugaussoi(1:4:loi)=ugauss(2:2+l);
ugaussoi(2:4:loi)=-.0552127*ugauss(1:1+l)+.8209544*ugauss(2:2+l)+...
  .2737197*ugauss(3:3+l)-.0394672*ugauss(4:4+l);
ugaussoi(3:4:loi)=-.0631377*ugauss(1:1+l)+.5631337*ugauss(2:2+l)+...
  .5631337*ugauss(3:3+l)-.0631377*ugauss(4:4+l);
ugaussoi(4:4:loi)=-.0394672*ugauss(1:1+l)+.2737197*ugauss(2:2+l)+...
  .8209544*ugauss(3:3+l)-.0552127*ugauss(4:4+l);
ugaussoi(loi+1)=ugauss(3+l);

if R~=0
%-- linear interpolation
  llin=loi*R; ugauss=zeros(1:llin);
  ugauss(1:R:llin)=ugaussoi(1:loi); delta=1/R;
  for k=2:R
    ugauss(k:R:llin)=ugaussoi(1:loi)+...
    (ugaussoi(2:loi+1)-ugaussoi(1:loi))*delta*(k-1);
  end
else; llin=loi; ugauss=ugaussoi; end

%-- nonlinear mapping
if distrib=='L' | distrib=='l'
  delta=1.956631e-3; x1=.193706; x2=4.88962; x3=3.71568; x4=.683318;
  factor=1/sqrt(1.0277);
  charac=[0 3.95858e-2 5.60043e-2 6.86173e-2 7.92629e-2 8.86526e-2 ...
9.71514e-2 .104976 .112267 .119123 .125615 .131796 .137710 .143388 ...
.148858 .154141 .159257 .164222 .169047 .173746 .178328 .182802 .187175];
charac=[charac .191455 .195648 .199758 .203792 .207753 .211646 .215475 ...
.219242 .222951 .226605 .230206 .233757 .237260 .240717 .244131 .247502 ...
.250833 .254125 .257380 .260598 .263783 .266934 .270052 .273140 .276198];
charac=[charac .279227 .282227 .285201 .288148 .291069 .293966 .296838 ...
.299688 .302514 .305319 .308102 .310864 .313606 .316328 .319031 .321715 ...
.324380 .327028 .329658 .332271 .334868 .337448 .340013 .342562 .345096];
charac=[charac .347615 .350119 .352609 .355086 .357549 .359998 .362434 ...
.364858 .367269 .369668 .372055 .374429 .376793 .379145 .381486 .383815 ...
.386134 .388443 .390741 .393029 .395307 .397575 .399834 .402083 .404322];
charac=[charac .406553 .408774 .410987 .388443 .410987 .432654 .453572 ...
.473841 .493543 .512744 .531501 .549860 .567861 .585538 .602921 .620036 ...
.636904 .653547 .669983 .686226 .702292 .718192 .733939 .749544 .765015];
charac=[charac .780361 .795591 .810712 .825729 .840651 .855481 .870226 ...
.884891 .899479 .913996 .928445 .942830 .957154 .971420 .985632 .999792 ...
1.01390 1.02797 1.04199 1.05597 1.06991 1.08381 1.09767 1.11150 1.12530];
charac=[charac 1.13906 1.15280 1.16651 1.18019 1.19384 1.20748 1.22108 ...
1.23467 1.24823 1.26178 1.27530 1.28881 1.30230 1.31577 1.32923 1.34267 ...
1.35610 1.36951 1.38291 1.39630 1.40967 1.42304 1.43639 1.44973 1.46307 ];
charac=[charac 1.47639 1.48971 1.50301 1.51631 1.52960 1.54289 1.55616 ...
1.56943 1.58270 1.59596 1.60921 1.62246 1.63570 1.64894 1.66217 1.67540 ...
1.68862 1.70184 1.71506 1.72828 1.74149 1.75470 1.76790 1.78111 1.79431];
charac=[charac 1.80751 1.82071 1.83390 1.84709 1.86028 1.87347 1.88666 ...
1.89985 1.91304 1.92623 1.93941 1.95260 1.96578 1.97897 1.99215 2.00533 ...
2.01852 2.03170 2.04488 2.05807 2.07125 2.08443 2.09762 2.11080 2.12399];
charac=[charac 2.13717 2.15036 2.16354 2.17673 2.18992 2.20311 2.21630 ...
2.22949 2.24268 2.25587 2.26907 2.28226 2.29546 2.30866 2.32185 2.33505 ...
2.34825 2.36146 2.37466 2.38787 2.40107 2.41428 2.42749 2.44070 2.45391];
charac=[charac 2.46712 2.48034 2.49356 2.50677 2.51999 2.53322 2.54644 ...
2.55966 2.57289 2.58612 2.59935 2.61258 2.62581 2.63905 2.65228 2.66552 ...
2.67876 2.69200 2.70525 2.71849 2.73174 2.74499 2.75824 2.77149 2.78475];
charac=[charac 2.79800 2.81126 2.82452 2.83778 2.85104 2.86431 2.87758 ...
2.89085 2.90412 2.91739 2.93066 2.94394 2.95722 2.97050 2.98378 2.99706 ...
3.01035 3.02363 3.03692 3.05021 3.06351 3.07680 3.09010 3.10340 3.11670];
charac=[charac 3.13000 3.14330 3.15661 3.16991 3.18322 3.19653 3.20985 ...
3.22316 3.23648 3.24979 3.26311 3.27644 3.28976 3.30308 3.31641 3.32974 ...
3.34307 3.35640 3.36974 3.38307 3.39641 3.40975 3.42309 3.43643 3.44978];
charac=[charac 3.46312 3.47647 3.48982 3.50317 3.51652 3.52988 3.54323 ...
3.55659 3.56995 3.58331 3.59667 3.61004 3.62341 3.63677 3.65014 3.66351 ...
3.67689 3.69026 3.70364 3.71701];

elseif distrib=='G' | distrib=='g'
  delta=2.344682e-3; x1=.232123; x2=4.92149; x3=5.60535; x4=1.169455;
  factor=1/sqrt(1.0433);
  charac=[0 9.47462e-6 3.77852e-5 8.50115e-5 1.51118e-4 2.36097e-4 ...
3.39940e-4 4.62635e-4 6.04171e-4 7.64532e-4 9.43702e-4 1.14166e-3 ...
1.35840e-3 1.59388e-3 1.84810e-3 2.12101e-3 2.41261e-3 2.72286e-3];
charac=[charac 3.05172e-3 3.39918e-3 3.76520e-3 4.14974e-3 4.55276e-3 ...
4.97425e-3 5.41414e-3 5.87240e-3 6.34900e-3 6.84389e-3 7.35702e-3 ...
7.88834e-3 8.43782e-3 9.00540e-3 9.59103e-3 1.01947e-2 1.08162e-2 1.14557e-2];
charac=[charac 1.21130e-2 1.27881e-2 1.34809e-2 1.41914e-2 1.49195e-2 ...
1.56651e-2 1.64281e-2 1.72086e-2 1.80065e-2 1.88216e-2 1.96539e-2 ...
2.05034e-2 2.13700e-2 2.22535e-2 2.31540e-2 2.40713e-2 2.50055e-2 2.59563e-2];
charac=[charac 2.69238e-2 2.79078e-2 2.89083e-2 2.99252e-2 3.09584e-2 ...
3.20079e-2 3.30735e-2 3.41552e-2 3.52529e-2 3.63665e-2 3.74959e-2 ...
3.86411e-2 3.98019e-2 4.09783e-2 4.21702e-2 4.33775e-2 4.46001e-2 4.58379e-2];
charac=[charac 4.70909e-2 4.83589e-2 4.96419e-2 5.09397e-2 5.22523e-2 ...
5.35796e-2 5.49215e-2 5.62779e-2 5.76487e-2 5.90339e-2 6.04332e-2 ...
6.18467e-2 6.32743e-2 6.47158e-2 6.61711e-2 6.76402e-2 6.91230e-2  7.06194e-2];
charac=[charac 7.21292e-2 7.36524e-2 7.51890e-2 7.67387e-2 7.83015e-2 ...
7.98774e-2 8.14661e-2 8.30677e-2 8.46820e-2 8.63090e-2 8.79485e-2 ...
7.21292e-2 8.79485e-2 .105013 .123229 .142505 .162755 .183894 .205844];
charac=[charac  .228530 .251884 .275842 .300344 .325338 .350774 .376607 ...
.402796 .429304 .456098 .483147 .510424 .537905 .565566 .593387 .621352 ...
.649442 .677643 .705943 .734328 .762789 .791316 .819900 .848532 .877207];
charac=[charac .905917 .934657 .963421 .992206 1.02101 1.04982 1.07864 ...
1.10747 1.13630 1.16513 1.19396 1.22279 1.25161 1.28042 1.30923 1.33802 ...
1.36681 1.39559 1.42435 1.45310 1.48183 1.51055 1.53926 1.56795 1.59663];
charac=[charac 1.62529 1.65393 1.68256 1.71117 1.73977 1.76835 1.79691 ...
1.82545 1.85398 1.88250 1.91099 1.93947 1.96793 1.99638 2.02481 2.05322 ...
2.08162 2.11000 2.13836 2.16671 2.19505 2.22337 2.25167 2.27996 2.30823];
charac=[charac 2.33649 2.36474 2.39297 2.42119 2.44939 2.47758 2.50575 ...
2.53391 2.56206 2.59020 2.61832 2.64643 2.67453 2.70261 2.73069 2.75875 ...
2.78680 2.81483 2.84286 2.87087 2.89888 2.92687 2.95485 2.98282 3.01078];
charac=[charac 3.03873 3.06667 3.09460 3.12252 3.15043 3.17834 3.20623 ...
3.23411 3.26198 3.28985 3.31770 3.34555 3.37338 3.40121 3.42903 3.45685 ...
3.48465 3.51245 3.54023 3.56801 3.59579 3.62355 3.65131 3.67906 3.70680];
charac=[charac 3.73454 3.76226 3.78999 3.81770 3.84541 3.87311 3.90081 ...
3.92849 3.95618 3.98385 4.01152 4.03918 4.06684 4.09449 4.12214 4.14978 ...
4.17741 4.20504 4.23266 4.26028 4.28789 4.31550 4.34310 4.37069 4.39828];
charac=[charac 4.42587 4.45345 4.48103 4.50860 4.53616 4.56373 4.59128 ...
4.61883 4.64638 4.67393 4.70147 4.72900 4.75653 4.78406 4.81158 4.83909 ...
4.86661 4.89412 4.92162 4.94912 4.97662 5.00412 5.03161 5.05909 5.08657];
charac=[charac 5.11405 5.14153 5.16900 5.19647 5.22393 5.25139 5.27885 ...
5.30631 5.33376 5.36121 5.38865 5.41609 5.44353 5.47097 5.49840 5.52583 ...
5.55325 5.58068 5.60810];

elseif distrib=='S' | distrib=='s'
  delta=2.344682e-3; x1=.232123; x2=4.68702; x3=6.191372; x4=1.41085;
  factor=1/sqrt(1.0497);
  charac=[0 1.85918e-8 1.54212e-7 5.23887e-7 1.24465e-6 2.43352e-6 ...
4.20753e-6 6.68371e-6 9.97909e-6 1.42107e-5 1.94953e-5 2.59497e-5 ...
3.36906e-5 4.28345e-5 5.34981e-5 6.57976e-5 7.98494e-5 9.57696e-5];
charac=[charac 1.13674e-4 1.3367928e-4 1.5590044e-4 1.8045338e-4 ...
2.07454e-4 2.37016e-4 2.69257e-4 3.04290e-4 3.42231e-4 3.83194e-4 ...
4.27293e-4 4.74643e-4 5.25358e-4 5.79552e-4 6.37338e-4 6.98829e-4];
charac=[charac 7.64139e-4 8.33381e-4 9.06667e-4 9.84109e-4 1.06582e-3 ...
1.15191e-3 1.24250e-3 1.33768e-3 1.43758e-3 1.54231e-3 1.65197e-3 ...
1.76668e-3 1.88654e-3 2.01167e-3 2.14216e-3 2.27815e-3 2.41972e-3];
charac=[charac 2.56699e-3 2.72006e-3 2.87905e-3 3.04406e-3 3.21519e-3 ...
3.39256e-3 3.57626e-3 3.76640e-3 3.96310e-3 4.16644e-3 4.37654e-3 ...
4.59349e-3 4.81741e-3 5.04839e-3 5.28654e-3 5.53195e-3 5.78473e-3];
charac=[charac 6.04497e-3 6.31278e-3 6.58826e-3 6.87150e-3 7.16261e-3 ...
7.46167e-3 7.76880e-3 8.08407e-3 8.40760e-3 8.73947e-3 9.07977e-3 ...
9.42861e-3 9.78608e-3 1.01523e-2 1.05273e-2 1.09111e-2 1.13040e-2];
charac=[charac 1.17060e-2 1.21171e-2 1.25375e-2 1.29673e-2 1.34064e-2 ...
1.38551e-2 1.43134e-2 1.47814e-2 1.52592e-2 1.57468e-2 1.62444e-2 ...
1.67520e-2 1.72696e-2 1.77975e-2 1.83356e-2 1.88841e-2 1.38551e-2];
charac=[charac 1.88841e-2 2.49540e-2 3.21373e-2 4.04971e-2 5.00865e-2 ...
6.09475e-2 7.31112e-2 8.65972e-2 1.01414e-1 .117559 .135019 .153771 .173785 ...
.195020 .217432 .240969 .265577 .291198 .317773 .345241 .373542 .402616];
charac=[charac .432406 .462856 .493912 .525521 .557637 .590213 .623207 ...
.656578 .690290 .724309 .758602 .793141 .827899 .862850 .897974 .933249 ...
.968656 1.00418 1.03980 1.07551 1.11129 1.14714 1.18303 1.21896 1.25493];
charac=[charac 1.29093 1.32694 1.36296 1.39899 1.43502 1.47104 1.50706 ...
1.54306 1.57906 1.61503 1.65098 1.68691 1.72281 1.75869 1.79454 1.83036 ...
1.86615 1.90191 1.93763 1.97332 2.00897 2.04459 2.08018 2.11573 2.15124 ];
charac=[charac 2.18672 2.22216 2.25756 2.29293 2.32826 2.36356 2.39882 ...
2.43404 2.46923 2.50438 2.53950 2.57458 2.60963 2.64464 2.67962 2.71456 ...
2.74947 2.78435 2.81920 2.85401 2.88879 2.92354 2.95826 2.99294 3.02760];
charac=[charac 3.06223 3.09683 3.13139 3.16593 3.20044 3.23493 3.26938 ...
3.30381 3.33821 3.37258 3.40693 3.44125 3.47555 3.50982 3.54407 3.57829 ...
3.61249 3.64666 3.68081 3.71494 3.74905 3.78313 3.81719 3.85123 3.88525];
charac=[charac 3.91925 3.95322 3.98718 4.02111 4.05503 4.08893 4.12280 ...
4.15666 4.19050 4.22432 4.25812 4.29191 4.32567 4.35942 4.39315 4.42687 ...
4.46057 4.49425 4.52791 4.56156 4.59520 4.62882 4.66242 4.69601 4.72958];
charac=[charac 4.76314 4.79668 4.83021 4.86373 4.89723 4.93071 4.96419 ...
4.99765 5.03110 5.06453 5.09795 5.13136 5.16476 5.19814 5.23151 5.26487 ...
5.29822 5.33156 5.36489 5.39820 5.43150 5.46479 5.49807 5.53134 5.56460];
charac=[charac 5.59785 5.63109 5.66432 5.69754 5.73075 5.76394 5.79713 ...
5.83031 5.86348 5.89664 5.92979 5.96293 5.99607 6.02919 6.06231 6.09541 ...
6.12851 6.16160 6.19468];
end

if distrib=='K' | distrib=='k'
  factor=1/sqrt(1.0375); sigma=factor*ugauss;
else
  a=abs(ugauss);
  f=find(a<=x1);
  if length(f)
    b=a(f)/delta; index=floor(b); b=b-index; index=index+1;
    sigma(f)=factor*sign(ugauss(f)).*(charac(index)+...
      (charac(index+1)-charac(index)).*b); end
  f=find((a>x1) & (a<=x2));
  if length(f)
    b=0.1*a(f)/delta; index=floor(b); b=b-index; index=index+93;
    sigma(f)=factor*sign(ugauss(f)).*(charac(index)+...
      (charac(index+1)-charac(index)).*b); end
  f=find(a>x2);
  if length(f); sigma(f)=factor*sign(ugauss(f)).*(x3+x4*(a-x2)); end
end

%------ product process

if llin-MNEW==0;
  inicond=[inicond 0];
else;
  inicond=[inicond llin-MNEW sigma(MNEW+1:llin)];
end
sigma=[srest sigma(1:MNEW)]; s=[sigma(1:M).*ggauss]; SEED=rand('seed');
return
%
% End of sirps.m

