% STRETCH	Created in MatLab by	Brian Galebach
%					Systems Engineering Group, Inc.
%					9841 Broken Land Parkway, Suite 214
%					Columbia, MD 21044
%					SEGrp@aol.com
%					Programmed February 1995,
%					Modified May 1996
%
% This simulation models the stretch pulse compression technique with
% phase/frequency beamsteering at the subarray level.
% For instructions on how to use this simulation, see the file readme.txt.

% The simulation utilizes a graphical user interface in order
% to allow the user to quickly input the parameters.
% This is the main script which sets up the graphical user
% interface.  The script which is called when an interface item
% is changed or clicked is specified in the 'callback' property
% of that item.  The script which does the stretch pulse compression
% calculations is TBMCALC.M.


% Set the default range to 600 km.
defrange = 600;


% Make a figure the size of the screen and blue.
screen = get(0,'ScreenSize');

fig = figure('position',screen,'resize','off',...
 'numbertitle','off','name','Stretch Pulse Compression',...
	'color','blue','userdata','');

% Set the fontsize for all of the graphs to 12 point
goodsize = 12;

% Set the uicontrols scaling factor based on the size of the screen
uiscale = screen(3)/640;



% draw the graph which indicates the locations of targets
rangeplot = axes('box','on','units','pixels',...
	'position',[50,373,395,40]*uiscale,'xlim',(defrange+[-1 1]),'ylim',[-1 1],...
	'xgrid','on','ygrid','on','yticklabels',[],'fontsize',goodsize);
xlabel('Range (km)','fontsize',goodsize);
title('Target Locations','fontsize',goodsize);


% draw the graph which shows the spectrum of the targets
specplot = axes('box','on','units','pixels',...
	'position',[50,63,395,272]*uiscale,'xlim',(defrange+[-1 1]),...
	'xgrid','on','ygrid','on','fontsize',goodsize);
xlabel('Range (km)','fontsize',goodsize);
ylabel('Return Power (dB)','fontsize',goodsize);



% fields which control the centrer and width of ranges in the plots
rangecent = uicontrol(fig,'style','edit','string',int2str(defrange),'max',1,...
	'position',[460,385,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','tbmrange');
rangewidth = uicontrol(fig,'style','edit','string','1','max',1,...
	'position',[540,385,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','tbmrange');


% text for the minimum and maximum range fields
uicontrol(fig,'style','text','string','Center',...
	'position',[460,405,60,20]*uiscale,'horizontalalignment','center',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','Width',...
	'position',[540,405,60,20]*uiscale,'horizontalalignment','center',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','Displayed Ranges (km)',...
	'position',[460,425,140,20]*uiscale,'horizontalalignment','center',...
	'backgroundcolor','blue','foregroundcolor','green');


% text which displays the number of targets in the simulation
uicontrol(fig,'style','text','string','# of Targets =',...
	'position',[485,362,80,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');
numtarg = uicontrol(fig,'style','text','string','0',...
	'position',[570,362,50,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');


% buttons which control the targets
newtarg = uicontrol(fig,'style','push','string','New',...
	'position',[460,344,60,17]*uiscale,'horizontalalignment','center',...
	'callback','tbmnewt');
deltarg = uicontrol(fig,'style','push','string','Delete',...
	'position',[540,344,60,17]*uiscale,'horizontalalignment','center',...
	'callback','tbmdelt');
nexttarg = uicontrol(fig,'style','push','string','Next',...
	'position',[460,325,60,17]*uiscale,'horizontalalignment','center',...
	'callback','tbmnextt');
prevtarg = uicontrol(fig,'style','push','string','Previous',...
	'position',[540,325,60,17]*uiscale,'horizontalalignment','center',...
	'callback','tbmprevt');


% text which labels the current target parameters
uicontrol(fig,'style','text','string','Target ID',...
	'position',[460,303,75,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','Range',...
	'position',[460,281,75,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','RCS',...
	'position',[460,259,75,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','Velocity',...
	'position',[450,237,85,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','Angle Offset',...
	'position',[450,215,85,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','km',...
	'position',[605,281,40,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','dB',...
	'position',[605,259,40,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','m/s',...
	'position',[605,237,40,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','deg',...
	'position',[605,215,40,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');


% fields which control the current target parameters
targid = uicontrol(fig,'style','edit','string','0','max',1,...
	'position',[540,303,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','tbmdifft');
targrange = uicontrol(fig,'style','edit','string','0','max',1,...
	'position',[540,281,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','tbmparch');
targrcs = uicontrol(fig,'style','edit','string','0','max',1,...
	'position',[540,259,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','tbmparch');
targvel = uicontrol(fig,'style','edit','string','0','max',1,...
	'position',[540,237,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','tbmparch');
targang = uicontrol(fig,'style','edit','string','0','max',1,...
	'position',[540,215,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','tbmparch');


% set variables pertaining to the targets and their parameters
targparam = [];
numtargs = 0;
currtarg = 0;

%---------------------------------------------------------------
%---------------------------------------------------------------

% Check box which allows the user to choose Taylor weighting
taylorbox = uicontrol(fig,'style','checkbox','string','Taylor',...
	'position',[480,33,55,20]*uiscale,'backgroundcolor','blue',...
	'foregroundcolor','green',...
	'callback','tbmtay');



% text which labels the radar parameters
uicontrol(fig,'style','text','string','Ref. Range',...
	'position',[450,190,85,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','Min. Freq.',...
	'position',[450,168,85,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','Bandwidth',...
	'position',[460,146,75,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','Pulsewidth',...
	'position',[460,124,75,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','Sample Rate',...
	'position',[450,102,85,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','# of Samples',...
	'position',[450,80,85,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','Noise Floor',...
	'position',[450,58,85,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','km',...
	'position',[605,190,45,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','MHz',...
	'position',[605,168,45,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','MHz',...
	'position',[605,146,45,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','usec',...
	'position',[605,124,55,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','MHz',...
	'position',[605,102,45,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','dB',...
	'position',[605,58,45,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');


% fields which control the radar parameters
idealrange = uicontrol(fig,'style','edit','string',int2str(defrange),'max',1,...
	'position',[540,190,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','');
freq = uicontrol(fig,'style','edit','string','1000','max',1,...
	'position',[540,168,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','');
bandwidth = uicontrol(fig,'style','edit','string','500','max',1,...
	'position',[540,146,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','');
pulsewidth = uicontrol(fig,'style','edit','string','819.2','max',1,...
	'position',[540,124,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','tbmsampn');
samprate = uicontrol(fig,'style','edit','string','10','max',1,...
	'position',[540,102,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','tbmsampn');
numsamp = uicontrol(fig,'style','edit','string','8192','max',1,...
	'position',[540,80,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','tbmsampr');
noisefloor = uicontrol(fig,'style','edit','string','-40','max',1,...
	'position',[540,58,60,20]*uiscale,'backgroundcolor','magenta',...
	'callback','');


% button which does all the calculations and does the plot
calc = uicontrol(fig,'style','push','string','Calculate',...
	'position',[470,5,80,20]*uiscale,'horizontalalignment','center',...
	'callback','tbmcalc');


% Text which labels the array parameters
uicontrol(fig,'style','text','string','n',...
	'position',[5,5,10,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','Delx',...
	'position',[60,5,25,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','Azimuth',...
	'position',[165,5,50,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','cm',...
	'position',[135,5,20,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','deg',...
	'position',[265,5,20,20]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');
uicontrol(fig,'style','text','string','MMR',...
	'position',[305,5,30,20]*uiscale,'horizontalalignment','right',...
	'backgroundcolor','blue','foregroundcolor','green');


% fields which control the radar parameters
numel = uicontrol(fig,'style','edit','string','1','max',1,...
	'position',[20,5,25,20]*uiscale,'backgroundcolor','magenta',...
	'callback','');
delx = uicontrol(fig,'style','edit','string','10','max',1,...
	'position',[90,5,40,20]*uiscale,'backgroundcolor','magenta',...
	'callback','');
az = uicontrol(fig,'style','edit','string','0','max',1,...
	'position',[220,5,40,20]*uiscale,'backgroundcolor','magenta',...
	'callback','');
nummmr = uicontrol(fig,'style','edit','string','1','max',1,...
	'position',[340,5,25,20]*uiscale,'backgroundcolor','magenta',...
	'callback','nummatran = eval(get(nummmr,''string''));');


% button which prints the target return power plot
printbut = uicontrol(fig,'style','push','string','Print',...
	'position',[410,5,50,20]*uiscale,'horizontalalignment','center',...
	'callback','tbmprint');


% Check box which allows the user to choose to display the maximum
% amplitude.
%ampbox = uicontrol(fig,'style','checkbox','string','Max Amplitude',...
%	'position',[50,337,115,20]*uiscale,'backgroundcolor','blue',...
%	'foregroundcolor','green',...
%	'callback','calcamp = get(ampbox,''value'');');


% Text which displays the maximum amplitude in the spectrum graph
uicontrol(fig,'style','text','string','Max =',...
	'position',[315,336,45,18]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');
maxamp = uicontrol(fig,'style','text','string','',...
	'position',[358,336,90,18]*uiscale,'horizontalalignment','left',...
	'backgroundcolor','blue','foregroundcolor','green');


% Set initial parameters for the simulation
sll = 40;		% Default values
nbar = 10;		% for Taylor weighting
newtaylor = 1;		% Set a flag which indicates that Taylor weights
			% must be calculated when Taylor weighting is selected.
oldsamples = 0;		% Initialize a flag which is set when the number of samples
			% changes.
calculated = 0;		% Set a flag to indicate that no calculations have been performed.
nummatran = 1;		% Number of matched ranges in the MMR method.  1 indicates
			% a single matched range (normal operation).

