% TBMCALC
% This is the script which calculates the return power with stretch
% pulse compression based on the current parameters,
% and plots the data.  This version of tbmcalc adds in gaussian noise.


if numtargs > 0


% Get important parameters for the calculation
centrange = eval(get(idealrange,'string'));		% Reference Range (km)
startfreq = eval(get(freq,'string')) * 1000000;		% Starting Frequency (Hz)
bw = eval(get(bandwidth,'string')) * 1000000;		% Bandwidth (Hz)
pw = eval(get(pulsewidth,'string')) / 1000000;		% Pulsewidth (sec)
samplerate = eval(get(samprate,'string')) * 1000000;	% A/D converter sample rate (Hz)
numsamples = eval(get(numsamp,'string'));		% Number of A/D converter samples
numels = eval(get(numel,'string'));			% Number of subarrays in array
deltax = eval(get(delx,'string')) / 100;		% Distance between subarrays (m)
azimuth = eval(get(az,'string')) * pi/180;		% Azimuth of main beam (rad)
tayloron = get(taylorbox,'value');			% Flag which indicates Taylor weights
nf = 10^(eval(get(noisefloor,'string'))/20);		% Noise floor converted from dB


% Define and calculate other important parameters
clear i;
c = 299792458; 				% Speed of light (m/s)
freqsens = 2*bw/(c*pw);			% Frequency sensitivity factor used
					% to calculate target locations from
					% spectral frequencies (Hz/m)
delt = deltax*sin(azimuth)/c;		% Time delay between subarrays due to 
					% subarray spacing and steering angle (sec)
quadfactor = bw/(2*pw);			% A factor used in calculating phase of
					% stretch pulse waveform (Hz^2)
numspecsamp = 2^nextpow2(numsamples);	% The number of FFT data points used for
					% spectrum calculation.  (Next power of 2)


% If needed, calculate the taylor weights
if tayloron == 1
	if (newtaylor == 1) | (numsamples ~= oldsamples)
		taylorweights = taylor(sll,nbar,numsamples);
		newtaylor = 0;
	end
end
oldsamples = numsamples;


% Initialize the spectrum samples
spect = zeros(nummatran,numspecsamp);


% Initialize the iqdata
iqdata = zeros(numels,numsamples);


% Repeat for each subarray in the array
for elnum = 1:numels


% Initialize the I/Q data for the the correct subarray
iqdata(elnum,:) = zeros(1,numsamples);


% Calculate the time shift due to the location of the
% specific receiving subarray for the deramp function
elshift = delt*(elnum-(numels+1)/2);


% Repeat for every target
for count = 1:numtargs


% Get important target parameters
vel = targparam(count,3); 				% Target velocity (m/s)
targoffset = (targparam(count,1)/(1+vel/c) - centrange) * 1000; % Target offset from
							% matched range (m)
% (Target offset calculated by setting the target at the specified
% range when the beam is transmitted by the central subarray.)
targdelt = targoffset*2/c;
rcs = 10^(targparam(count,2)/20); 			% Radar cross section converted from dB
targangoff = targparam(count,4)*pi/180; 		% Target angle offset (rad)


% If the target is within the limits of this pulse then do calculations for this target
if abs(targoffset) < c*pw/2


% Calculate the Doppler factor
doppfactor = 1/(1+2*vel/c);

%----------------------------------------------------------------------
% If the target has an angle offset
%----------------------------------------------------------------------
if (targangoff ~= 0)

% Calculate the delt for the angle-offset target
offdelt = deltax*sin(azimuth+targangoff)/c; % sec

% Calculate the time shift due to the location of the receiving subarray
% for the angle-offset target
offelshift = offdelt*(elnum-(numels+1)/2); % sec


% Repeat for each transmitting subarray
for transel = 1:numels

% Calculate the time shift due to the location of the transmitting subarray
% for the regular angle
transshift = delt*(transel-(numels+1)/2); % sec

% Calculate the time shift due to the location of the transmitting subarray
% for the angle-offset target
offtransshift = offdelt*(transel-(numels+1)/2); % sec

% Calculate the time region within the pulsewidth for this target and this
% transmitting subarray
mintargtime = targdelt - offelshift - offtransshift*doppfactor; %sec
maxtargtime = mintargtime + pw * doppfactor; %sec
mintime = max([0,mintargtime]); %sec
maxtime = min([pw,maxtargtime]); %sec


% Calculate the minimum and maximum samples which will be affected
% by this target and their corresponding times
minsamp = max([1,ceil(mintime/pw * numsamples)]);
maxsamp = floor(maxtime/pw * numsamples);
times = pw*[minsamp:maxsamp]/numsamples; %sec
origtimes = times/doppfactor - targdelt - transshift + offtransshift + ...
	offelshift; %sec


% Calculate the phase difference between the return and the deramp
% functions, and negate to have correct spectrum plot
phasediff = quadfactor*(times+elshift).^2+startfreq*(times+elshift) - ...
	(quadfactor*origtimes.^2+startfreq*origtimes);  % revolutions



% Calculate the I/Q data at the sample points
iqdata(elnum,minsamp:maxsamp) = iqdata(elnum,minsamp:maxsamp) +...
	rcs/numels*(cos(2*pi*phasediff)+i*sin(2*pi*phasediff))*...
	cos(azimuth+targangoff);



end % for each transmitting subarray


%----------------------------------------------------------------------
else % the target has no angle offset
%----------------------------------------------------------------------


% Calculate the time region within the pulsewidth for this target
mintargtime = targdelt - (abs(delt)*(numels-1)/2)*doppfactor-elshift; %sec
maxtargtime = mintargtime + (pw+abs(delt)*(numels-1))*doppfactor; %sec
mintime = max([0,mintargtime]); %sec
maxtime = min([pw,maxtargtime]); %sec


% Calculate the minimum and maximum samples which will be affected
% by this target and their corresponding times
minsamp = max([1,ceil(mintime/pw * numsamples)]);
maxsamp = floor(maxtime/pw * numsamples);
times = pw*[minsamp:maxsamp]/numsamples; %sec
origtimes = times/doppfactor - targdelt  + elshift; %sec


% Calculate the phase difference between the return and the deramp
% functions, and negate to have correct spectrum plot
phasediff = quadfactor*(times+elshift).^2+startfreq*(times+elshift) - ...
	(quadfactor*origtimes.^2+startfreq*origtimes);  % revolutions


% Calculate the locations where beams originating from different
% subarrays start and stop
targontimes = mintargtime + abs(delt)*doppfactor*[0:numels-1];
targofftimes = maxtargtime - abs(delt)*doppfactor*[0:numels-1];


% Calculate the stair-step amplitude function
stairfun = ones(1,maxsamp-minsamp+1);

if targontimes(numels) > pw*minsamp/numsamples
for count = 1:numels-1
	if targontimes(count+1) > pw*minsamp/numsamples
	stairfun(max(minsamp,ceil(targontimes(count)/pw*numsamples))...
		-minsamp+1:...
		floor(targontimes(count+1)/pw*numsamples)-minsamp+1)...
		= ...
	stairfun(max(minsamp,ceil(targontimes(count)/pw*numsamples))...
		-minsamp+1:...
		floor(targontimes(count+1)/pw*numsamples)-minsamp+1)...
			* count/numels;
	end
end
end

if targofftimes(numels) < pw*maxsamp/numsamples
for count = 1:numels-1
	if targofftimes(count+1) < pw*maxsamp/numsamples
	stairfun(ceil(targofftimes(count+1)/pw*numsamples-minsamp+1):...
		min(maxsamp,floor(targofftimes(count)/pw*numsamples))...
		-minsamp+1) = ...
	stairfun(ceil(targofftimes(count+1)/pw*numsamples-minsamp+1):...
		min(maxsamp,floor(targofftimes(count)/pw*numsamples))...
		-minsamp+1) * count/numels;
	end
end
end


% Calculate the I/Q data at the sample points
iqdata(elnum,minsamp:maxsamp) = iqdata(elnum,minsamp:maxsamp) +...
	rcs*stairfun.*(cos(2*pi*phasediff)+i*sin(2*pi*phasediff))*...
	cos(azimuth+targangoff);

%----------------------------------------------------------------------
end % if the target has an angle offset
%----------------------------------------------------------------------

end % if target is within limits

end % for loop for all targets

end % for loop for all subarrays


%-------------------------------------------------------------------------------------
% Do Taylor weighting and FFT.  But first, apply multiple phase rotations to the data.
%-------------------------------------------------------------------------------------


% Calculate the matrix of phase rotations and modify the I/Q data;
phaserots = exp(-4*pi*i*quadfactor*(delt*([1:numels]'-(numels+1)/2))*...
	(numsamples/bw/nummatran*([1:nummatran]-(nummatran+1)/2)));
iqdata = conj(iqdata')*phaserots;


% Before Taylor weighting is applied, add thermal noise to the I/Q data
iqdata = iqdata + randn(numsamples,nummatran) * sqrt(numsamples) * numels * nf *...
	sqrt(2)/2 + ...
	randn(numsamples,nummatran) * sqrt(numsamples) * numels * nf * sqrt(2)/2 * i;


% Apply Taylor weighting if it is selected
if tayloron == 1
	iqdata = iqdata.*taylorweights(ones(1,nummatran),:)'; % (Tony's trick)
end


% Calculate the set of spectrums for each rotation
for count = 1:nummatran
	spect(count,:) = abs(fftshift(fft(iqdata(:,count),numspecsamp)))';
end


% Assemble the MMR composite spectrum.
compspect = zeros(1,numspecsamp);
for count = 1:nummatran
	compspect(floor((count-1)/nummatran*numspecsamp+1):...
		floor(count/nummatran*numspecsamp)) = ...
		spect(count,floor((count-1)/nummatran*numspecsamp+1):...
		floor(count/nummatran*numspecsamp));
end
spect = compspect;


% Scale the spectrum sample data
spect = spect/numsamples/numels;



% Display the maximum value of the data.
set(maxamp,'string',[sprintf('%.4f',20*log10(max(max(spect)))),' dB']);



% Convert the spectrum sample data to dB
spect = 20*log10(spect);



% Plot the spectrum graph
axes(specplot);
plot(centrange+samplerate/freqsens*([0:numspecsamp-1]/numspecsamp-...
	1/2)/1000,spect,'color','red');
tbmfixsp;


% Set the flag which indicates that data has been calculated.
calculated = 1;


end