% This function calculates the maximum of the spectrum data
% while correcting for the effects of straddle loss.
% The algorithm will use the golden search method to search for
% a maximum between the highest sample point and the higher of
% the two neighboring sample points.  This algorithm assumes that
% the function is "well behaved" between these two points.
% Because the spectrum does not contain the phase information, (the
% absolute value had been taken to get the spectrum values), the
% absolute value of the sinc function must be used to get a proper
% reconstruction.  Also, a proper value will be obtained only when
% the number of samples is an integer power of 2.


% Define the number of steps to search for a maximum
numsteps = 10;

% Calculate the recipricol of the golden ratio
golden = 2/(sqrt(5)+1);


% Find the current maximum
currmax = max(spectrum);


% Find the location of the current maximum
currmaxloc = find(spectrum == currmax);


% Find the location and value of the highest neighboring sample point,
% and define the limits and values of the starting points for the
% golden search algorithm
if currmaxloc == 1
	a1 = 1;
	a4 = 2;
elseif currmaxloc == numspecsamp
	a1 = numspecsamp-1;
	a4 = numspecsamp;
else
	if spectrum(currmaxloc+1) > spectrum(currmaxloc-1)
		a1 = currmaxloc;
		a4 = currmaxloc+1;
	else
		a1 = currmaxloc-1;
		a4 = currmaxloc;
	end
end
x1 = spectrum(a1);
x4 = spectrum(a4);


% If the golden search algorithm needs to be implemented, do it
if min([x1 x4]) ~= 0


% Initialize the intermediate points within the limits
a2 = 0;
a3 = 0;


% Repeat for the number of steps defined at the top of this function
for count = 1:numsteps


% Calculate the locations and values of the intermediate points
% within the limits
if a2 == 0
	a2 = a4 - (a4-a1)*golden;
	x2 = tbminter(spectrum,numspecsamp,a2);
end

if a3 == 0
	a3 = a1 + (a4-a1)*golden;
	x3 = tbminter(spectrum,numspecsamp,a3);
end

%[a1 a2 a3 a4]
%[x1 x2 x3 x4]
%pause;
% Decide which portion within the limits that the maximum must be
% and adjust the variables accordingly.
if x2 > x3
	a4 = a3;
	x4 = x3;
	a3 = a2;
	x3 = x2;
	a2 = 0;
else
	a1 = a2;
	x1 = x2;
	a2 = a3;
	x2 = x3;
	a3 = 0;
end


end % for loop

% Determine the maximum value and convert it to dB
currmax = max([x1 x2 x3 x4]);


end % if need to do search


% Convert the answer to dB
currmax = 20*log10(currmax);