% Filt_RLC.m  - This Matlab script sets up a graphical user interface (GUI) for plotting 
% frequency response of standard RLC ladder (low pass) filters.
% This script simplifies design and selection of low pass RLC filters by allowing a designer to
% quickly compare the responses of an RLC filter with ideal or non ideal component values.
% These filter components values for standard filters can be found in many filter handbook tables, 
% such as "Handbook of Filter Synthesis" by Anatol Zverev, Wiley.

% The GUI treats the prototype filter component values as edit able text.
% Therefore the values can be modified from the exact RLC values and the resulting 
% filter can then be plotted.  This also allows the designer to determine the deviations in the 
% filter response caused by RLC component value error. 
% The component values can all be directly enter for a filter that is not 
% included in the table of filters. 
% Methods for synthesis of arbitrary all pole, RLC low pass filters are described in
% the college text "Modern Filtering" by  Art Davis. 
% Many filter text books describe standard techniques that are useful to 
% design a filter as a low pass filters and then transform the designs to 
% other frequency ranges.

% The following parameters can be selected for plotting (vs frequency); 
% Amplitude, Phase delay, Group delay, Phase, and Input Impedance. 
% These are selected under the "Filter" pull down menu under the "plot type" sub menu. 

% The filter type, -3dB frequency, termination resistor, and order 
% are entered from the GUI interface.
% The filter type, order, and Rin are entered from a pull down menu. 
% The Filter 3dB frequency and plot range is entered in editable text windows.

% When the push button 'Load prototype' is activated.
% The presently selected filter component values are loaded from Zverev.mat as normalized prototype 
% filter components and the components are frequency and impedance scaled. 
% When the "plot filter" button in the GUI is activated the frequency response for the filter is
% plotted in another window. 
% This script calls PlotFilt.m to do the actual plotting.

% This script file requires that the following files be in the  Matlab search path.
% Zverev.mat, filt_rlc.m, and plotfilt.m.
% *note. When a particular value of Rin does not exist for a given filter then the components for 
% the next closest Rin are loaded.
% 
% written by David Winters
% email address  DAVID_WINTERS@MAXTOR.COM
%
% ************************ INITIALIZE variables ****************************
%
global  m_filt m_atten m_order m_ptype m_rlc m_dis
load zverev
Rout = 1;
Pi= 4*atan(1);
Fc = 1/(2*Pi);
ftypes = 1;
atten = 1;
order = 2;
ptype = 1;

strlc(1,:) = 'Rin =';
strlc(2,:) = 'C1  =';
strlc(3,:) = 'L2  =';
strlc(4,:) = 'C3  =';
strlc(5,:) = 'L4  =';
strlc(6,:) = 'C5  =';
strlc(7,:) = 'L6  =';
strlc(8,:) = 'C7  =';
strlc(9,:) = 'L8  =';
strlc(10,:) = 'C9  =';
strlc(11,:) = 'L10 =';
strlc(12,:) = 'C11 =';
strlc(13,:) = 'Rout=';


strfilt(1,:)= 'butrw';
strfilt(2,:)= 'chhun';
strfilt(3,:)= 'chten';
strfilt(4,:)= 'chhaf';
strfilt(5,:)= 'lphas';
strfilt(6,:)= 'lp05d';
strfilt(7,:)= 'lpp5d';
strfilt(8,:)= 'gauss';
strfilt(9,:)= 'ga6db';
strfilt(10,:)= 'g12db';
strfilt(11,:)= 'legen';

stratten(1,:) ='__0';
stratten(2,:) ='__1';
stratten(3,:) ='__2';
stratten(4,:) ='__3';
stratten(5,:) ='__4';
stratten(6,:) ='__5';
stratten(7,:) ='__6';
stratten(8,:) ='__7';
stratten(9,:) ='__8';
stratten(10,:) ='__9';
stratten(11,:) ='_10';

% ************************ INITIALIZE GUI menu ****************************

	h_ch=get(0,'Children');

	h_fig=figure('Visible','off');          % make GUI figure, start with hidden
	r_name=['Filtplot Figure No. ',num2str(h_fig)];   %build figure name with integer figure handle
	ctype=computer;                         %get computer type and scale text
	if strcmp(ctype(1:2),'PC'),
		fsize=20;
	else
		fsize=12;
	end
	h_tmp=text(.1,.1,'AB','Units','pixels','FontSize',fsize);       %display some test text
	txt_ext=get(h_tmp,'Extent');            %how many pixels in size is this text
	txt_ext=txt_ext(3:4).*[.5 1];           %normalize to 1char x 1 line

	delete(h_tmp);                          %and remove the text

	frame=txt_ext.*[35 16];                 %define frame size (30 chars x 16 lines)
	screen=get(0,'ScreenSize');             %get screen size in pixels
	set(h_fig,'NumberTitle','off','Name',r_name,'Resize','off','ColorMap',[], ...           %set figure pos and size and other props
		  'NextPlot','new','MenuBar','none');
	set(h_fig,'Position',[5 screen(4)-frame(2) frame(1) frame(2)]);       
	axis off;

%**************************set up second column with plot options******************

	xpos=frame(1)/3+10;                     %reset starting (x,y) location
	xwid=frame(1)/3-10;                     %define width of lines and height
	ypos=frame(2)-txt_ext(2);
	ywid=txt_ext(2);

	h_txt2=uicontrol(h_fig,'Style','text','String','Plot Control','Position',[xpos ypos 2*xwid ywid],'Units','pixels');

	ypos=ypos-txt_ext(2);                   %adjust y coordinate for next line
	H_HOLD_AXIS=uicontrol(h_fig,'Style','radio','String','hold axis','Position',[xpos ypos xwid ywid],'Units','pixels','Value',0);

	xpos=xpos+xwid;                         %adjust x coordinate for next line
	H_HOLD_PLOT=uicontrol(h_fig,'Style','radio','String','hold plot','Position',[xpos ypos xwid ywid],'Units','pixels','Value',0);

%skip 1 line and place heading for user inputs

	xpos=frame(1)/3+10;                     %reset starting (x,y) location
	ypos=ypos-1.25*txt_ext(2);

	h_txt3=uicontrol(h_fig,'Style','text','String','Plot Range','Position',[xpos ypos 2*xwid ywid],'Units','pixels');


	ypos=ypos-txt_ext(2);                   %adjust y coordinate for next line

	h_start_l=uicontrol(h_fig,'Style','text','String','Start (Hz)','Position',[xpos ypos xwid ywid],'HorizontalAlignment','Left','Units','pixels');
	xpos2=xpos+xwid-6*txt_ext(1);      %6 characters in edit field
	xwid2=6*txt_ext(1);
	H_S=uicontrol(h_fig,'Style','edit','String','0','Position',[xpos2 ypos xwid2 ywid],'Units','pixels');


	xpos=xpos+xwid;                         %adjust x coordinate for next line
	xpos2=xpos2+xwid;                       %adjust x coordinate for next line
	h_Stop_l=uicontrol(h_fig,'Style','text','String','Stop (Hz)','Position',[xpos ypos xwid ywid],'HorizontalAlignment','Left','Units','pixels');
	H_F=uicontrol(h_fig,'Style','edit','String',num2str(2*Fc),'Position',[xpos2 ypos xwid2 ywid],'Units','pixels');


	xpos=frame(1)/3+10;                     %reset starting (x,y) location
	xpos2=xpos+xwid-6*txt_ext(1);           %6 characters in edit field
	ypos=ypos-1.25*txt_ext(2);              %skip a 1.25 lines
	h_fig_l=uicontrol(h_fig,'Style','text','String','figure #','Position',[xpos ypos xwid ywid],'HorizontalAlignment','Left','Units','pixels');
	H_FIG_INP=uicontrol(h_fig,'Style','edit','String',num2str(h_fig+1),'Position',[xpos2 ypos xwid2 ywid],'Units','pixels');

	xpos=xpos+xwid;                         %adjust x coordinate for next line
	xpos2=xpos2+xwid;                       %adjust x coordinate for next line
	h_splt_l=uicontrol(h_fig,'Style','text','String','subplot #','Position',[xpos ypos xwid ywid],'HorizontalAlignment','Left','Units','pixels');
	H_SPLT_INP=uicontrol(h_fig,'Style','edit','String',num2str(111),'Position',[xpos2 ypos xwid2 ywid],'Units','pixels');

	xpos=frame(1)/3+10;                     %reset starting (x,y) location
	xpos2=xpos+xwid-6*txt_ext(1);           %6 characters in edit field
	ypos=ypos-1.25*txt_ext(2);              %skip a 1.25 lines


	h_Rout_l=uicontrol(h_fig,'Style','text','String','Rout (Ohms)','Position',[xpos ypos xwid ywid],'HorizontalAlignment','Left','Units','pixels');
	xpos2=xpos+xwid-6*txt_ext(1);      %6 characters in edit field
	xwid2=6*txt_ext(1);
	H_R=uicontrol(h_fig,'Style','edit','String','1','Position',[xpos2 ypos xwid2 ywid],'Units','pixels','callback',...
['Rout=str2num(get(H_R,''string''));']);


	xpos=xpos+xwid;                         %adjust x coordinate for next line
	xpos2=xpos2+xwid;                       %adjust x coordinate for next line

	h_Fc_l=uicontrol(h_fig,'Style','text','String','F 3dB (Hz)','Position',[xpos ypos xwid ywid],'HorizontalAlignment','Left','Units','pixels');
	H_C=uicontrol(h_fig,'Style','edit','String',num2str(Fc),'Position',[xpos2 ypos xwid2 ywid],'Units','pixels','callback',...
['Fc=str2num(get(H_C,''string''));']);

%skip 1.25 more line and add PLOT button with handle number of this figure defined in the callback
	ypos=ypos-1.25*txt_ext(2);
	xpos=frame(1)/3+10;                     %reset starting (x,y) location
	xwid=frame(1)/4;
	h_plot=uicontrol(h_fig,'Style','push','String','Plot Filter Response','Position',[xpos ypos xwid ywid],'Units','pixels', ...
'Callback',['for ci = 1:13,'...
,'rlc(ci)=str2num(get(m_rlc(ci),''string''));'...
,'end;'...
,'H_FILTPLOT=',num2str(h_fig),';plotfilt']);


% ************************ set up first column with RLC's ****************************

	ypos=ypos-1.25*txt_ext(2);
	xpos=frame(1)/3+10;                     %reset starting (x,y) location
	xwid=frame(1)/4;
	h_upd=uicontrol(h_fig,'Style','push','String','Load Prototype','Position',[xpos ypos xwid ywid],'Units','pixels' ...
,'Callback',[...
'profilt=',strfilt(ftypes,:),stratten(atten,:),'(order-1,:);'...
,'delete(m_dis(:));'...
,'delete(m_rlc(:));'...
,'xwid=frame(1)/3+2;'...
,'xpos=2;','xpos2=xpos+xwid/2;'...
,'ypos=frame(2)-2*txt_ext(2);'...
,'ywid=txt_ext(2);'...
,'for ci = 1:13,'...
,'rlc(ci)=profilt(ci)*(Rout^((-1)^(ci-1)))/(2*Pi*Fc);'...
,'rlc(1)=profilt(1)*Rout;'...
,'rlc(13)=Rout;'...
,'m_rlc(ci)=uicontrol(h_fig,''Style'',''edit'',''String'',num2str(rlc(ci)),''Position'',[xpos2 ypos xwid/2 ywid],''Units'',''pixels'');'...
,'ypos=ypos-txt_ext(2);'...
,'end;'...
,'ypos=ypos+4.5*txt_ext(2);'...
,'xpos=xpos+xwid+5;'...
,'m_dis(1)=uicontrol(h_fig,''Style'',''text'',''String'',[''Filter type='',get(m_filt(ftypes),''label'')],''Position'',[xpos ypos xwid ywid],''Units'',''pixels'');'...
,'ypos=ypos-txt_ext(2);'...
,'m_dis(2)=uicontrol(h_fig,''Style'',''text'',''String'',[''Rin / Rout='',get(m_atten(atten),''label'')],''Position'',[xpos ypos xwid ywid],''Units'',''pixels'');'...
,'ypos=ypos-txt_ext(2);'...
,'m_dis(3)=uicontrol(h_fig,''Style'',''text'',''String'',[''Filter Order='',get(m_order(order-1),''label'')],''Position'',[xpos ypos xwid ywid],''Units'',''pixels'');'...
,'ypos=ypos-txt_ext(2);'...
,'m_dis(4)=uicontrol(h_fig,''Style'',''text'',''String'',[''Plot type= '',get(m_ptype(ptype),''label'')],''Position'',[xpos ypos xwid ywid],''Units'',''pixels'');'...
,'global m_rlc m_dis ;'... 
]);


% ************************ set up filter menus **************************

h_filtset=uimenu(h_fig,'label','Filter');

	ftypes = uimenu(h_filtset,'label','Filter Type');
		m_filt(1) = uimenu(ftypes,'label','BUTERWORTH','Callback',['ftypes = 1;'...
,'set(m_filt(:),''checked'',''off'');','set(m_filt(1),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(9:13)=''butrw'';','set(h_upd,''callback'',oldcb);']);
		m_filt(2)=uimenu(ftypes,'label','CHEBYCHEV .01dB','Callback',['ftypes = 2;'...
,'set(m_filt(:),''checked'',''off'');','set(m_filt(2),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(9:13)=''chhun'';','set(h_upd,''callback'',oldcb);']);
		m_filt(3)=uimenu(ftypes,'label','CHEBYCHEV .1dB','Callback',['ftypes = 3;'...
,'set(m_filt(:),''checked'',''off'');','set(m_filt(3),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(9:13)=''chten'';','set(h_upd,''callback'',oldcb);']);
		m_filt(4)=uimenu(ftypes,'label','CHEBYCHEV .5dB','Callback',['ftypes = 4;'...
,'set(m_filt(:),''checked'',''off'');','set(m_filt(4),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(9:13)=''chhaf'';','set(h_upd,''callback'',oldcb);']);
		m_filt(5)=uimenu(ftypes,'label','LINEAR PHASE','Callback',['ftypes = 5;'...
,'set(m_filt(:),''checked'',''off'');','set(m_filt(5),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(9:13)=''lphas'';','set(h_upd,''callback'',oldcb);']);
		m_filt(6)=uimenu(ftypes,'label','LINEAR PHASE .05 DEG','Callback',['ftypes = 6;'...
,'set(m_filt(:),''checked'',''off'');','set(m_filt(6),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(9:13)=''lp05d'';','set(h_upd,''callback'',oldcb);']);
		m_filt(7)=uimenu(ftypes,'label','LINEAR PHASE .5 DEG','Callback',['ftypes = 7;'...
,'set(m_filt(:),''checked'',''off'');','set(m_filt(7),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(9:13)=''lpp5d'';','set(h_upd,''callback'',oldcb);']);
		m_filt(8)=uimenu(ftypes,'label','GAUSSIAN','Callback',['ftypes = 8;'...
,'set(m_filt(:),''checked'',''off'');','set(m_filt(8),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(9:13)=''gauss'';','set(h_upd,''callback'',oldcb);']);
		m_filt(9)=uimenu(ftypes,'label','GAUSSIAN TO 6dB','Callback',['ftypes = 9;'...
,'set(m_filt(:),''checked'',''off'');','set(m_filt(9),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(9:13)=''ga6db'';','set(h_upd,''callback'',oldcb);']);
		m_filt(10)=uimenu(ftypes,'label','GAUSSIAN TO 12dB','Callback',['ftypes = 10;'...
,'set(m_filt(:),''checked'',''off'');','set(m_filt(10),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(9:13)=''g12db'';','set(h_upd,''callback'',oldcb);']);
		m_filt(11)=uimenu(ftypes,'label','LEGENDER','Callback',['ftypes = 11;'...
,'set(m_filt(:),''checked'',''off'');','set(m_filt(11),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(9:13)=''legen'';','set(h_upd,''callback'',oldcb);']);

	atten = uimenu(h_filtset,'label','Rin/Rout');
		m_atten(1) = uimenu(atten,'label','0','Callback',['atten = 1;'...
,'set(m_atten(:),''checked'',''off'');','set(m_atten(1),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(14:16)=''__0'';','set(h_upd,''callback'',oldcb);']);
		m_atten(2) = uimenu(atten,'label','.1','Callback',['atten = 2;'...
,'set(m_atten(:),''checked'',''off'');','set(m_atten(2),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(14:16)=''__1'';','set(h_upd,''callback'',oldcb);']);
		m_atten(3) = uimenu(atten,'label','.2','Callback',['atten = 3;'...
,'set(m_atten(:),''checked'',''off'');','set(m_atten(3),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(14:16)=''__2'';','set(h_upd,''callback'',oldcb);']);
		m_atten(4) = uimenu(atten,'label','.3','Callback',['atten = 4;'...
,'set(m_atten(:),''checked'',''off'');','set(m_atten(4),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(14:16)=''__3'';','set(h_upd,''callback'',oldcb);']);
		m_atten(5) = uimenu(atten,'label','.4','Callback',['atten = 5;'...
,'set(m_atten(:),''checked'',''off'');','set(m_atten(5),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(14:16)=''__4'';','set(h_upd,''callback'',oldcb);']);
		m_atten(6) = uimenu(atten,'label','.5','Callback',['atten = 6;'...
,'set(m_atten(:),''checked'',''off'');','set(m_atten(6),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(14:16)=''__5'';','set(h_upd,''callback'',oldcb);']);
		m_atten(7) = uimenu(atten,'label','.6','Callback',['atten = 7;'...
,'set(m_atten(:),''checked'',''off'');','set(m_atten(7),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(14:16)=''__6'';','set(h_upd,''callback'',oldcb);']);
		m_atten(8) = uimenu(atten,'label','.7','Callback',['atten = 8;'...
,'set(m_atten(:),''checked'',''off'');','set(m_atten(8),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(14:16)=''__7'';','set(h_upd,''callback'',oldcb);']);
		m_atten(9) = uimenu(atten,'label','.8','Callback',['atten = 9;'...
,'set(m_atten(:),''checked'',''off'');','set(m_atten(9),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(14:16)=''__8'';','set(h_upd,''callback'',oldcb);']);
		m_atten(10) = uimenu(atten,'label','.9','Callback',['atten = 10;'...
,'set(m_atten(:),''checked'',''off'');','set(m_atten(10),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(14:16)=''__9'';','set(h_upd,''callback'',oldcb);']);
		m_atten(11) = uimenu(atten,'label','1.0','Callback',['atten = 11;'...
,'set(m_atten(:),''checked'',''off'');','set(m_atten(11),''checked'',''on'');'...
,'oldcb=get(h_upd,''callback'');','oldcb(14:16)=''_10'';','set(h_upd,''callback'',oldcb);']);

	order = uimenu(h_filtset,'label','Filter Order');
		m_order(1) = uimenu(order,'label','2','Callback',['order = 2;'...
,'set(m_order(:),''checked'',''off'');','set(m_order(1),''checked'',''on'');']);
		m_order(2) = uimenu(order,'label','3','Callback',['order = 3;'...
,'set(m_order(:),''checked'',''off'');','set(m_order(2),''checked'',''on'');']);
		m_order(3) = uimenu(order,'label','4','Callback',['order = 4;'...
,'set(m_order(:),''checked'',''off'');','set(m_order(3),''checked'',''on'');']);
		m_order(4) = uimenu(order,'label','5','Callback',['order = 5;'...
,'set(m_order(:),''checked'',''off'');','set(m_order(4),''checked'',''on'');']);
		m_order(5) = uimenu(order,'label','6','Callback',['order = 6;'...
,'set(m_order(:),''checked'',''off'');','set(m_order(5),''checked'',''on'');']);
		m_order(6) = uimenu(order,'label','7','Callback',['order = 7;'...
,'set(m_order(:),''checked'',''off'');','set(m_order(6),''checked'',''on'');']);
		m_order(7) = uimenu(order,'label','8','Callback',['order = 8;'...
,'set(m_order(:),''checked'',''off'');','set(m_order(7),''checked'',''on'');']);
		m_order(8) = uimenu(order,'label','9','Callback',['order = 9;'...
,'set(m_order(:),''checked'',''off'');','set(m_order(8),''checked'',''on'');']);
		m_order(9) = uimenu(order,'label','10','Callback',['order = 10;'...
,'set(m_order(:),''checked'',''off'');','set(m_order(9),''checked'',''on'');']);

	ptype = uimenu(h_filtset,'label','Plot Type');
		m_ptype(1) = uimenu(ptype,'label','AMPLITUDE','Callback',['ptype = 1;'...
,'set(m_ptype(:),''checked'',''off'');','set(m_ptype(1),''checked'',''on'');']);
		m_ptype(2) = uimenu(ptype,'label','PHASE DELAY','Callback',['ptype = 2;'...
,'set(m_ptype(:),''checked'',''off'');','set(m_ptype(2),''checked'',''on'');']);
		m_ptype(3) = uimenu(ptype,'label','GROUP DELAY','Callback',['ptype = 3;'...
,'set(m_ptype(:),''checked'',''off'');','set(m_ptype(3),''checked'',''on'');']);
		m_ptype(4) = uimenu(ptype,'label','PHASE','Callback',['ptype = 4;'...
,'set(m_ptype(:),''checked'',''off'');','set(m_ptype(4),''checked'',''on'');']);
		m_ptype(5) = uimenu(ptype,'label','INPUT Z','Callback',['ptype = 5;'...
,'set(m_ptype(:),''checked'',''off'');','set(m_ptype(5),''checked'',''on'');']);

% ************************ initialize menus ****************************

	ftypes = 1;
	atten = 1;
	order = 2;
	ptype = 1;

set(m_filt(:),'checked','off');
set(m_filt(ptype),'checked','on');

set(m_atten(:),'checked','off');
set(m_atten(atten),'checked','on');

set(m_order(:),'checked','off');
set(m_order(order-1),'checked','on');

set(m_ptype(:),'checked','off');
set(m_ptype(ptype),'checked','on');


profilt=butrw__0(order-1,:);
xwid=frame(1)/3+2;
xpos=2;xpos2=xpos+xwid/2;
ypos=frame(2)-2*txt_ext(2);
ywid=txt_ext(2);
for ci = 1:13
rlc(ci)=profilt(ci)*(Rout^((-1)^(ci-1)))/(2*Pi*Fc);
rlc(1)=profilt(1)*Rout;
rlc(13)=Rout;
uicontrol(h_fig,'Style','text','String',strlc(ci,:),'Position',[xpos ypos xwid/2 ywid],'Units','pixels');
m_rlc(ci)=uicontrol(h_fig,'Style','edit','String',num2str(rlc(ci)),'Position',[xpos2 ypos xwid/2 ywid],'Units','pixels');
ypos=ypos-txt_ext(2);
end;
ypos=ypos+4.5*txt_ext(2);
xpos=xpos+xwid+5;
m_dis(1)=uicontrol(h_fig,'Style','text','String',['Filter type=',get(m_filt(ftypes),'label')],'Position',[xpos ypos xwid ywid],'Units','pixels');
ypos=ypos-txt_ext(2);
m_dis(2)=uicontrol(h_fig,'Style','text','String',['Rin / Rout=',get(m_atten(atten),'label')],'Position',[xpos ypos xwid ywid],'Units','pixels');
ypos=ypos-txt_ext(2);
m_dis(3)=uicontrol(h_fig,'Style','text','String',['Filter Order=',get(m_order(order-1),'label')],'Position',[xpos ypos xwid ywid],'Units','pixels');
ypos=ypos-txt_ext(2);
m_dis(4)=uicontrol(h_fig,'Style','text','String',['Plot type= ',get(m_ptype(ptype),'label')],'Position',[xpos ypos xwid ywid],'Units','pixels');


H_FILTPLOT=num2str(h_fig);
%store all handles in UserData
	H=[H_HOLD_AXIS H_HOLD_PLOT H_S H_F H_FIG_INP H_SPLT_INP H_R H_C];
	set(h_fig,'UserData',H);
	set(h_fig,'Visible','on');
