%  PLOTFILT.m - This is the plot routine called by the GUI figure built by filt_rlc.m
%  This script reads the handles of the RLC window and plots the filter amplitude
%  phase delay, group delay, phase or input impedance.
%  The handles are stored in the entry window's UserData area. The handle of this area is passed
%  as H_FILTPLOT. 
%  In order to run the PLOTFILT script type "filt_rlc" from the matlab command window, then 
%  select the desired filter from the GUI, which is setup and activate the "load prototype" button
%  and then activate the "plot filter" button.
%
%  The filters produced are low pass LC ladder filters with alternating series inductors 
%  and shunt capacitors.  The filters are restricted to those terminated with a resistance 
%  Rout in parallel with a capacitor.  The filters are voltage input and voltage output 
%  except for the even order Rin = 0 filters, which must be implemented as current input,
%  current output filters (voltage output obtained by multiplying by Rout).
%
%          ---Rin-----L2-----L4-----L6-----L8-----L10------------Vout (= Iout*Rout)
%         |        |      |      |      |      |      |      |
%        Vin       C1     C3     C5     C7     C9     C11    Rout--Iout
%         |        |      |      |      |      |      |      |
%          --------------------------------------------------------Gnd
%
%
%
%
%          -----------L2-----L4-----L6-----L8-----L10------------Vout (= Iout*Rout)
%         |        |      |      |      |      |      |      |
%        Iin       C1     C3     C5     C7     C9     C11    Rout--Iout
%         |        |      |      |      |      |      |      |
%          --------------------------------------------------------Gnd
% 
%  Transfer functions are calculated by state variable method using a set of alternating KCL and KVL network 
%  equations. Therefore the state variables include both series currents and volatges, 

%  which greatly simplifies the calculation of the frequency responses, as can be seen below.
%  The resulting transfer function is the reciprocal of the continuant with alternating series impedance and
%  shunt admittance components along the matrix diagonal elements, and +-1 along the subdiagonals.
%  If base band signals are of interest it is recommended to use Phase delay rather than Group delay.
% 
%
%  The following variables must be defined before running this script 
%  (preferably through the GUI interface initiated by filt_rlc.m);
%   rlc(), Rout, Ptype, atten, order, H_S, H_F, and other GUI handles. 
%
% written by David Winters
% email David_Winters@Maxtor.com
%
% ***********Get graphics handles *************

	H_FILTPLOT_LIST=get(H_FILTPLOT,'UserData');                 %get handles for all GUI objects
	H_HOLD_AXIS=H_FILTPLOT_LIST(1);
	H_HOLD_PLOT=H_FILTPLOT_LIST(2);
	H_S=H_FILTPLOT_LIST(3);
	H_F=H_FILTPLOT_LIST(4);
	H_FIG_INP=H_FILTPLOT_LIST(5);
	H_SPLT_INP=H_FILTPLOT_LIST(6);
	H_R=H_FILTPLOT_LIST(7);
	H_C=H_FILTPLOT_LIST(8);

% *************Setup frequency vector, and filter continuant "filtdet" *************
numpoints = 300;                % number of points in the plot
Fs=str2num(get(H_S,'string')):str2num(get(H_F,'string'))/numpoints:str2num(get(H_F,'string'));
Fs(1)= Fs(2)/10;

% setup matrix elements in the continuant that are not frequency dependent

clear filtdet;
filtdet= eye(13);
if order/2 > fix(order/2)
norder = order;
else 
norder = order+1;
end;

for ci = 1:norder+1
filtdet(ci+1,ci)=-1;
filtdet(ci,ci+1)=1;
end;

filtdet(norder+2,norder+2)=rlc(13);
filtdet(1,1)=rlc(1);

% Calculate amplitude, phase, phase delay, group delay

if ptype<5
   for fi = 1:numpoints+1
	for ci = 2:norder+1
	   filtdet(ci,ci)=i*2*Pi*Fs(fi)*rlc(ci);
	end;
	if ((atten==1)&(norder==order))
	   Hres(fi) = 1/det(filtdet(2:13,2:13));
	else
	   Hres(fi) = rlc(13)/det(filtdet);
	end;

   end;
phase = unwrap(angle(Hres));
end;
%
%
	R_fig=str2num(get(H_FIG_INP,'String'));         %get desired figure number
	R_splt=str2num(get(H_SPLT_INP,'String'));       %get subplot number

	R_ch=get(0,'Children');                         %get root children; determine if figure exists
	if ~find(R_ch==R_fig),
		figure(R_fig);                          %if doesn't exist, create new figure
		subplot(R_splt);
		hold on;                                %auto ranging not affected, all traces will add to plot
	else
		figure(R_fig);                          %select desired figure
		subplot(R_splt);
		if get(H_HOLD_PLOT,'Value'),            %if holding entire plot,
			hold on;
		elseif get(H_HOLD_AXIS,'Value'),        %or if holding just axis parameters
			hold on;                        %this will hold anything manually scaled before cla
			cla;
		else                                    %not holding anything
			cla reset;
			hold on;
		end
	end

	clear R_fig R_splt R_ch;

%get start and finish frequency values
	R_s=str2num(get(H_S,'String'));
	R_f=str2num(get(H_F,'String'));
	if (R_s==0 & R_f==0),return,end

% set start frequency and plot filter responses

	R_Fs=Fs;                      
	R_maxFs=max(R_Fs);                              %check start and finish against limits
	if R_s>R_maxFs,R_s=R_maxFs;end
	if R_f>R_maxFs,R_f=R_maxFs;end


	if ptype ==1,
	  plot(Fs,abs(Hres),'y');
	  title('Amplitude Response');
	  xlabel('Frequency (Hz)');

	if ((atten==1)&(norder==order))
	  ylabel('Iout / Iin');
	else
	  ylabel('Vout / Vin');
	end;

	end

	if ptype ==2,
	  plot(Fs,-(phase./Fs)/(2*Pi),'r');
	  title('Phase Delay');
	  ylabel('Phase Delay (sec)');
	  xlabel('Frequency (Hz)');
	end

	if ptype ==3,
	  plot(Fs(2:numpoints+1),-(diff(phase)./diff(Fs))/(2*Pi),'g');
	  title('Group Delay');
	  ylabel('Group Delay (sec)');
	  xlabel('Frequency (Hz)');
	end

	if ptype ==4,
	  plot(Fs,phase,'b');
	  title('Phase (radians)');
	  ylabel('Phase');
	  xlabel('Frequency (Hz)');
	end


% Calculate and plot Input Impedance. Different equation are required to avoid divide by zero such as 1/(1/inf).

	if ptype==5,

if  ((norder>order)&(atten>1))
    for fi = 1:numpoints+1
	for ci = 2:norder+1
	   filtdet(ci,ci)=i*2*Pi*Fs(fi)*rlc(ci);
	end;
     Hres(fi) = det(filtdet)/(det(filtdet(4:13,4:13)));
   end;

elseif ((norder==order)&(atten>1))
   for fi = 1:numpoints+1
	for ci = 2:norder+1
	   filtdet(ci,ci)=i*2*Pi*Fs(fi)*rlc(ci);
	end;
     Hres(fi) = det(filtdet)/(det(filtdet(2:13,2:13)));
   end;

elseif ((norder==order)&(atten==1))
   for fi = 1:numpoints+1
	for ci = 2:(norder+1)
	   filtdet(ci,ci)=i*2*Pi*Fs(fi)*rlc(ci);
	end;
     Hres(fi) = det(filtdet(3:13,3:13))/(det(filtdet(2:13,2:13)));
   end;

elseif ((norder>order)&(atten==1))
   for fi = 1:numpoints+1
	for ci = 2:norder+1
	   filtdet(ci,ci)=i*2*Pi*Fs(fi)*rlc(ci);
	end;
     Hres(fi) = (det(filtdet(3:13,3:13)))/det(filtdet(4:13,4:13));
   end;
else;

end;

	  plot(Fs,abs(Hres),'c');
	  title('Filter Input |Z|');
	  ylabel('|Z| (Ohms)');
	  xlabel('Frequency (Hz)');
	end


	hold off;

	clear R_s R_f R_maxFs R_Fs H_FILTPLOT H_FILTPLOT_LIST 
	clear H_HOLD_AXIS H_HOLD_PLOT H_S H_F H_FIG_INP H_SPLT_INP
