function OutHandles=anprpdlg(Action     , Handle, WorkingFig, ...
                             TextStrings, Values, CallBacks , ...
                             Tags       , FigTag, FigTitle  , ...
                             FigHelp                          ...
                             )
%ANPRPDLG Properties dialog function.
%	OUT=ANPRPDLG(ACTION,HANDLE,WORKINGFIG,TEXT,VALS,CALLBACK, ...
%	                   TAGS,FIGTAG,FIGTITLE,FIGHELP)
%	creates the properties modification dialog for a given object.
%	
%	FIG contains the handle of the dialog figure.
%
%	ACTION may be one of 4 strings:
%	'#Initialize'  Initialize the figure.
%	        '#OK'  OK button callback.
%	    '#Cancel'  Cancel button callback.
%	      '#Help'  Help button callback.
%
%	HANDLE is the handle of the object to be modified.
%
%	WORKINGFIG is the handle for the figure the object is displayed on.
%
%	TEXT is a matrix of text strings for the titles of the edit text
%	control areas.
%
%	VALS is a matrix of text strings for the current values in the
%	edit text control areas.
%
%	CALLBACK is a matrix of text strings that define the callback
%	functions for each of the edit text control areas.
%
%	TAGS ia a matrix of text strings that define the tags for the 
%	edit text control areas.
%
%	FIGTAG is the tag for the dialog figure that is created.  
%
%	FIGTITLE is the title for the figure.
%
%	FIGHELP is the help text for the figure.
%
%	See also ANDOT, ANRECT, ANTEXT.

%	Loren Dean  March, 1995.
%	Copyright (c) 1995 by The MathWorks, Inc.

if nargin==1,
  PrpDlgFig=get(0,'CurrentFigure');
  Data=get(PrpDlgFig,'UserData');
  WorkingFig=Data(1);
end

Data=get(WorkingFig,'UserData');

if nargin<=7,FigTag='PrpDlgFig';end

if nargin<=8,  
  SimHandle=Data(1);
  FigTitle=['Object Properties: ' get_param(SimHandle,'Name')];
end

if nargin<=9,
  FigHelp=[ ...
   'The entries in each of these blocks get evaluated.  If an entry is'
   'bad, a default value will be used as determined by the calling    '
   'function.  Vectors and matrices should be entered with square     '
   'brackets enclosing them.  Strings do not need quotes enclosing    '
   'them.  Signals can be referenced as u(n) where n is the signal    '
   'number to be used.                                                '
   'An example entry is:                                              '
   '  u(1)+50*cos(u(2))                                               '
          ];
end

if strcmp(Action,'#Initialize'),
  %%% Determine if a Properties dialog for this figure is already open
  %%% and close it if it is
  Figs=findobj(0,'Type','figure','Tag',FigTag);
  for lp=1:length(Figs),
    FigHandle=get(Figs(lp),'UserData');
    if any(FigHandle==WorkingFig),close(Figs(lp));end
  end % for lp

  % Set the Figure Position
  NumEdits=length(TextStrings(:,1));
  BtnHgt=20;
  
  FigPosition=get(0,'DefaultFigurePosition'); 
  FigPosition(1:2)=FigPosition(1:2)-100;
  FigPosition(3:4)=[400 80+(NumEdits+1)*2*BtnHgt];

  %%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%% Create the Properties Dialog Figure

  PrpDlgFig=figure(                                     ...
                  'Name'         , FigTitle           , ...
                  'NumberTitle'  , 'off'              , ...
                  'BackingStore' , 'on'               , ...
                  'Position'     , FigPosition        , ...
                  'Color'        , [0.9 0.9 0.9]      , ...
                  'MenuBar'      , 'none'             , ...
                  'UserData'     , [WorkingFig Handle], ...
                  'KeyPressFcn'  , 'anedit #KeyPress' , ...
                  'Visible'      , 'off'              , ...
                  'Tag'          , FigTag               ...
                  );
  
  pOKh=100; pOKv=20; pOKhgt=BtnHgt; pOKwdth=50;

  HFrmBtn=uicontrol(PrpDlgFig                                   , ...
                   'Style'    , 'frame'                         , ...
                   'Position' , ...
                    [pOKh-5 pOKv-5 (pOKwdth+20)*3+10 pOKhgt+10]   ...
                   );

  
  HBtnOK=uicontrol(PrpDlgFig                                   , ...
                  'String'   , 'OK'                            , ...
                  'Style'    , 'pushbutton'                    , ...
                  'Units'    , 'pixels'                        , ...
                  'Tag'      , 'OK'                            , ...
                  'Position' , ...
                     [pOKh+2*(pOKwdth+30) pOKv pOKwdth pOKhgt] , ...
                  'UserData' , deblank(CallBacks(NumEdits+1,:)), ...
                  'CallBack' , 'anprpdlg #OK'                    ...
                  );


  if ~isempty(Handle),              
    CurrentData=get(Handle,'UserData');
  else,
    CurrentData=[];
  end    

  HBtnCancel=uicontrol(PrpDlgFig                                 , ...
                      'String'   , 'Cancel'                      , ...
                      'Style'    , 'pushbutton'                  , ...
                      'Units'    , 'pixels'                      , ...
                      'Position' , ...
                            [pOKh+pOKwdth+30 pOKv pOKwdth pOKhgt], ...
                      'Tag'      , 'Cancel'                      , ...
                      'UserData' , [CurrentData;CurrentData]     , ...
                      'CallBack' , 'anprpdlg #Cancel'              ...
                      );

  HBtnHelp=uicontrol(PrpDlgFig                            , ...
                    'String'   ,'Help'                    , ...
                    'Style'    ,'pushbutton'              , ...
                    'Units'    ,'pixels'                  , ...
                    'Position' ,[pOKh pOKv pOKwdth pOKhgt], ...
                    'Tag'      ,'Help'                    , ...
                    'UserData' ,FigHelp                   , ...
                    'CallBack' ,'anprpdlg #Help'            ...
                    );

  pCh=20;pCv=pOKv+35;pChgt=pOKhgt;pCwdth=FigPosition(3)-2*pCh;

  if isempty(Handle),
    TextString1='Input Dialog';
    TextString2=' ';    
  else,
    TextString1='Reference block input variable names as "u".';
    TextString2='Example:sin(u(1)*pi/180)+10';
end    
  
  HTxt1Help=uicontrol(PrpDlgFig                                        , ...
                     'String'   ,TextString1                           , ...
                     'Style'    ,'text'                                , ...
                     'Units'    ,'pixels'                              , ...
                     'Position' ,[pCh pCv+(NumEdits+1)*40 pCwdth pChgt], ...
                     'Tag'      , 'HelpText1'                            ...
                     );
  HTxt2Help=uicontrol(PrpDlgFig                               , ...
                     'String'   ,TextString2                  , ...
                     'Style'    ,'text'                       , ...
                     'Units'    ,'pixels'                     , ...
                     'Position' , ...
                     [pCh pCv+(NumEdits+1)*40-20 pCwdth pChgt], ...
                     'Tag'      ,'HelpText2'                    ...
                     );

  HCoordFrm=uicontrol(PrpDlgFig                                    , ...
                     'Style'    , 'frame'                          , ...
                     'Position' , ...
                     [pCh-5 pCv-5 pCwdth+10 NumEdits*2*pChgt+15] ...
                     );

  for lp=1:NumEdits,
    pXTh=pCh;pXTv=pCv+(NumEdits-lp)*40;pXTwdth=pCwdth-5;pXThgt=pChgt;
    TxtHandle(lp)=uicontrol(PrpDlgFig                                 , ...
                       'String'           , deblank(TextStrings(lp,:)), ...
                       'Style'            , 'text'                    , ... 
                       'Position'           , ...
                       [pXTh pXTv+20 pXTwdth pXThgt]                  , ... 
                       'BackGroundColor'  ,[0.7 0.7 0.7]              , ...
                       'HorizontalAlignment','left'                     ...
                       );
    OutHandles(lp)=uicontrol(PrpDlgFig                                , ...
                            'Style'          ,'edit'                  , ...
                            'Position'       , ...
                            [pXTh pXTv pXTwdth pXThgt]                , ...
                            'BackGroundColor',[1 1 1]                 , ...
                            'HorizontalAlignment','left'              , ...
                            'Tag'            ,deblank(Tags(lp,:))     , ...
                            'CallBack'       ,deblank(CallBacks(lp,:)), ...
                            'String'         ,deblank(Values(lp,:))     ...
                            );


  end % for lp
  if isempty(Handle),set(HBtnCancel,'Visible','off');end
  set(PrpDlgFig,'Visible','on');
  set([HFrmBtn HBtnOK HBtnCancel HBtnHelp HCoordFrm TxtHandle OutHandles], ...
      'Units','Normalized' ...
      );
  set(0,'CurrentFigure',PrpDlgFig) 
%%%%%%%%%%
%%% OK %%%
%%%%%%%%%%
elseif strcmp(Action,'#OK'),

  OKHandle=findobj(PrpDlgFig,'Type','uicontrol','Tag','OK');
  eval(get(OKHandle,'UserData'));
  Data=get(PrpDlgFig,'UserData');
  if length(Data)>1,anedit('#Deselect',WorkingFig);end
  close(PrpDlgFig)

%%%%%%%%%%%%%%
%%% Cancel %%%
%%%%%%%%%%%%%%
elseif strcmp(Action,'#Cancel'),
  HCancel=findobj(PrpDlgFig,'Type','uicontrol','Tag','Cancel');
  CurrentData=get(HCancel,'UserData');
  DataSize=size(CurrentData,1);
  DataHalf=DataSize/2;
  
  Data=get(PrpDlgFig,'UserData');
  if length(Data)>1,  
    WorkingFig=Data(1);
    CurrentPt=Data(2);
  
  
    set(CurrentPt,'UserData',CurrentData(DataHalf+1:DataSize,:));
    ansimltn('#Reset',WorkingFig)
    anedit('#Deselect',WorkingFig)
  end    
  close(PrpDlgFig)

%%%%%%%%%%%%
%%% Help %%%
%%%%%%%%%%%%
elseif strcmp(Action,'#Help'),
  HelpHandle=findobj(PrpDlgFig,'Type','uicontrol','Tag','Help');
  FigHelp=get(HelpHandle,'UserData');
  helpdlg(FigHelp,'Animation Dialog Help');
    
end % if strcmp    







