function ansimltn(Action,SimWorkingFig)
%ANSIMLTN Animation simulation menu function.
%	ANSIMLTN(ACTION,SIMWORKINGFIG) is the callback 
%	function for the Simulation menu on the animation figure window.
%
%	ACTION may be one of 9 strings:
%	           '#Run'  Run the simulation.
%	          '#Halt'  End the simulation.
%	    '#Start/Stop'  Start and stop the simulation.
%	    '#MakeStatic'  Deak with static objects before and after simulation
%	       '#Restart'  Restart the simulation.
%	       '#Disable'  Disable features during simulation.
%	        '#Enable'  Enable features after simulation.
%	  '#WarnBtnPress'  Set ButtonDown Functions to Warn on ButtonPress
%	     '#WarnPress'  Reset Warning Dialog
%	         '#Reset'  Redraw the objects.
%	         '#SetIC'  Set Input Initial Conditions
%	      '#UpdateIC'  Update the Input Initial Condition Vector
%
%	SIMWORKINGFIG is the handle of the figure the animation takes place on.
%
%

%	See also ANSIMINI.

%	Loren Dean   March, 1995.
%	Copyright (c) 1995 by The MathWorks, Inc.

if ~strcmp(Action,'#UpdateIC'),
  if nargin==2,
    WorkingFig=SimWorkingFig;
  else,
    WorkingFig=findobj(0,'type','figure','Tag','WorkingFig');
  end
  Data=get(WorkingFig,'UserData');
  if ~isempty(Data),
    WorkingAxes=Data(2);
    SimHandle=Data(1);
    SimRoot=bdroot(SimHandle);
  else,
    Action='';
  end
end

%%%%%%%%%%%
%%% Run %%%
%%%%%%%%%%%
if strcmp(Action,'#Run'),
  set_param(SimRoot,'Simulation Running','start');
    
%%%%%%%%%%%%
%%% Halt %%%
%%%%%%%%%%%%
elseif strcmp(Action,'#Halt'),
  set_param(SimRoot,'Simulation Running','stop')

%%%%%%%%%%%%%%%%%%%
%%% Make Static %%%
%%%%%%%%%%%%%%%%%%%
elseif strcmp(Action,'#MakeStatic'),

  StartHandle=findobj(WorkingFig,'Type','uicontrol','Tag','StartButton');
%  ResetHandle=findobj(WorkingFig,'Type','uicontrol','Tag','ResetButton');
  StaticData=get(StartHandle,'UserData');
  FigData=get(WorkingFig,'UserData');
  WorkingAxes=FigData(2);
  Data=get(WorkingAxes,'UserData');
%  ColorData=get(ResetHandle,'UserData');  

  if isempty(StaticData), % simulation starting

    Loc=[];    
    for lp=1:size(Data,1),
      StaticFlag=isempty(findstr(setstr(get(Data(lp,1),'UserData')),'u('));
      if StaticFlag, % it's static
        Loc=[Loc lp];
      end        
    end % for lp
    
    
    StaticData=[];    
    for lp=1:length(Loc),
      CurrentObj=Data(Loc(lp),1);      
      StaticData=[StaticData;Data(Loc(lp),:)];      
    
    end
    if ~isempty(Loc),Data(Loc,:)=[];end
    set(WorkingAxes,'UserData',Data);
    if isempty(StaticData),StaticData=0;end
    set(StartHandle,'UserData',StaticData);
    
  else, % Simulation Stopping
    
    if prod(size(StaticData))~=1,
      for lp=1:size(StaticData,1),
        Data=[Data;StaticData(lp,:)];      
      end        
    end
    set(StartHandle,'UserData',[]);
    set(WorkingAxes,'UserData',Data);    
    
  end
  
%%%%%%%%%%%%%%%%%%
%%% Start/Stop %%%
%%%%%%%%%%%%%%%%%%
elseif strcmp(Action,'#Start/Stop'),
  BtnStartHandle=findobj(WorkingFig, ...
                        'Type','uicontrol'  , ...
                        'Tag' ,'StartButton'  ...
                        );
                        
  StartHandle=findobj(WorkingFig, ...
                     'Type'     ,'uimenu'            , ...
                     'Tag'      ,'MenuSimulationStart' ...
                     );
  RestartHandle=findobj(WorkingFig, ...
                       'Type'     ,'uimenu'              , ...
                       'Tag'      ,'MenuSimulationRestart' ...
                       );

  HideHandle = findobj(WorkingFig, ...
                      'Type'     ,'uimenu'              , ...
                      'Tag'      ,'MenuOptionHideStatus'  ...
                      );
  StatusHandles=get(HideHandle,'UserData');
  
  HStatus=StatusHandles(5);
 
  AxisData=get(WorkingAxes,'UserData');
  
  StartString=get(StartHandle,'Label');

  if strcmp(StartString,'&Start')|strcmp(StartString,'Start'),
    if strcmp(StartString,'Start'),
      StartLabel='Stop';
    else,
      StartLabel='&Stop';
    end
    set(StartHandle, ...
       'Label'     ,StartLabel      , ...
       'CallBack'  ,'ansimltn #Halt'  ...
       );
    set(BtnStartHandle, ...
       'String'        ,StartLabel     , ...
       'CallBack'     ,'ansimltn #Halt'  ...
       );
       
    set(RestartHandle,'Enable','on');
    
    StatMsg='Simulation Running.';
    
  elseif strcmp(StartString,'&Stop')|strcmp(StartString,'Stop'),
    if strcmp(StartString,'Stop'),
      StartLabel='Start';
    else,
      StartLabel='&Start';
    end
    set(StartHandle, ...
       'Label'     ,StartLabel    , ...
       'CallBack'  ,'ansimltn #Run' ...
       );
    set(BtnStartHandle, ...
       'String'        ,StartLabel   , ...
       'CallBack'     ,'ansimltn #Run' ...
       );
    set(RestartHandle,'Enable','off');

    if isempty(AxisData),
      StatMsg='Simulation Stopped. Use Object toolbar to add objects.';
    else,
      StatMsg='Simulation Stopped.  To Modify/Add objects, press Reset.';
    end
  end % if strcmp

  set(HStatus,'String',StatMsg);
   
%%%%%%%%%%%%%%%
%%% Restart %%%
%%%%%%%%%%%%%%%
elseif strcmp(Action,'#Restart'),

  set_param(SimRoot,'Simulation Running','restart');

%%%%%%%%%%%%%%%%%%%
%%% Show Trails %%%
%%%%%%%%%%%%%%%%%%%
elseif strcmp(Action,'#ShowTrails'),  
    
  SimFigData=get(WorkingFig,'UserData');
  WorkingAxes=SimFigData(2);
  SimData=get(WorkingAxes,'UserData');

  if ~isempty(SimData),
    TrailsHandle=findobj(WorkingFig,'type','uicontrol','Tag','ShowTrails');
    if get(TrailsHandle,'Value'),
      EraseMode='none';
    else,
      EraseMode='xor';
    end
      
    set(SimData(:,1)  , ...
       'EraseMode'    ,EraseMode, ...
       'ButtonDownFcn',''         ...
       );
  end % if ~isempty

  
%%%%%%%%%%%%%%%%%%%%
%%% Clear Trails %%%
%%%%%%%%%%%%%%%%%%%%
elseif strcmp(Action,'#ClearTrails'),  
  set(WorkingFig,'Color',get(WorkingFig,'Color'));
   
%%%%%%%%%%%%%%%
%%% Disable %%%
%%%%%%%%%%%%%%%
elseif strcmp(Action,'#Disable'),
  if WorkingFig~=[],
    HMenuEdit=findobj(WorkingFig,'Type','uimenu','Tag','MenuEdit');
    set(HMenuEdit,'Enable','off');

    SimFigData=get(WorkingFig,'UserData');
    WorkingAxes=SimFigData(2);
    SimData=get(WorkingAxes,'UserData');

    set(WorkingFig       , ...
       'ButtonDownFcn','', ...
       'KeyPressFcn'  ,''  ...
       );
    set(WorkingAxes          , ...
       'ButtonDownFcn', ''   , ...
       'DrawMode'     ,'fast'  ...
       );

    if ~isempty(SimData),
      TrailsHandle=findobj(WorkingFig,'type','uicontrol','Tag','ShowTrails');
      if get(TrailsHandle,'Value'),
        EraseMode='none';
      else,
        EraseMode='xor';
      end
      
      set(SimData(:,1)  , ...
         'EraseMode'    ,EraseMode, ...
         'ButtonDownFcn',''         ...
         );
      anedit('#Deselect',WorkingFig)
    end % if ~isempty
    
    CloseHandle=findobj(WorkingFig,'Type','uimenu','Tag','MenuFileClose');
    set(CloseHandle,'UserData',1);
    
    
    HideBtnHandle=findobj(WorkingFig, ...
                         'Type'     ,'uimenu'               , ...
                         'Tag'      ,'MenuOptionHideButtons'  ...
                         );
 
    ButtonHandles=get(HideBtnHandle,'UserData');
    set(ButtonHandles,'ButtonDownFcn','');
    SetICMenuHandle=findobj(WorkingFig, ...
                           'Type'     ,'uimenu'             , ...
                           'Tag'      ,'MenuSimulationSetIC'  ...
                           );
    set(SetICMenuHandle,'Enable','off');

    HideHandle = findobj(WorkingFig, ...
                        'Type'     ,'uimenu'              , ...
                        'Tag'      ,'MenuOptionHideStatus'  ...
                        );
    StatusHandles=get(HideHandle,'UserData');                      

    set(StatusHandles(2),'Visible','off');
    if strcmp(get(HideHandle,'Checked'),'off'),
      set(StatusHandles(6:7),'Visible','on');
    end
     
    ResetMenuHandle=findobj(WorkingFig, ...
                           'Type'     ,'uimenu'             , ...
                           'Tag'      ,'MenuSimulationReset'  ...
                           );
    set(ResetMenuHandle,'Enable','off');

    AutoHandle=Data(3);
    XLimit=get(WorkingAxes,'XLim');
    YLimit=get(WorkingAxes,'YLim');
    AData=ones(100,1)*[XLimit YLimit];
    set(AutoHandle,'UserData',AData);
    
  end % if WorkingFig

%%%%%%%%%%%%%%
%%% Enable %%%
%%%%%%%%%%%%%%
elseif strcmp(Action,'#Enable'),
  if WorkingFig~=[],
    set(WorkingAxes,'DrawMode','normal');
    HMenuEdit=findobj(WorkingFig,'Type','uimenu','Tag','MenuEdit');
    set(HMenuEdit,'Enable','on');

    SimFigData=get(WorkingFig,'UserData');
    WorkingAxes=SimFigData(2);
    SimData=get(WorkingAxes,'UserData');

    set(WorkingFig                             , ...
       'ButtonDownFcn', 'anfig #FigButtonPress', ...
       'KeyPressFcn'  , 'anfig #KeyPress'        ...
       );

    set(WorkingAxes,'ButtonDownFcn','anfig #AxisButtonPress');

    if ~isempty(SimData),
      set(SimData(:,1),'ButtonDownFcn','anedit #ButtonDown');
      anedit('#Deselect',WorkingFig);
    end % if ~isempty
   
    HideBtnHandle=findobj(WorkingFig, ...
                         'Type'     ,'uimenu'               , ...
                         'Tag'      ,'MenuOptionHideButtons'  ...
                         );

    ButtonHandles=get(HideBtnHandle,'UserData');
    set(ButtonHandles,'ButtonDownFcn','btnpress');
    btnpress(WorkingFig,'ObjectButtons',1)

    SetICMenuHandle=findobj(WorkingFig, ...
                           'Type'     ,'uimenu'             , ...
                           'Tag'      ,'MenuSimulationSetIC'  ...
                           );
    set(SetICMenuHandle,'Enable','on');

  end % if WorkingFig
%%%%%%%%%%%%%%%%%%%%
%%% WarnBtnPress %%%
%%%%%%%%%%%%%%%%%%%%
elseif strcmp(Action,'#WarnBtnPress'),
  if WorkingFig~=[],
    SimFigData=get(WorkingFig,'UserData');
    WorkingAxes=SimFigData(2);
    SimData=get(WorkingAxes,'UserData');

    set(WorkingFig       , ...
       'ButtonDownFcn','ansimltn #WarnPress', ...
       'KeyPressFcn'  ,'ansimltn #WarnPress'  ...
       );
    set(WorkingAxes,'ButtonDownFcn','ansimltn #WarnPress');

    if ~isempty(SimData),
      set(SimData(:,1), ...
         'ButtonDownFcn','ansimltn #WarnPress', ...
         'Interruptible','yes'                  ...
         );
    end % if ~isempty
    
    HideBtnHandle=findobj(WorkingFig, ...
                         'Type'     ,'uimenu'               , ...
                         'Tag'      ,'MenuOptionHideButtons'  ...
                         );

    ButtonHandles=get(HideBtnHandle,'UserData');
    set(ButtonHandles,'ButtonDownFcn','ansimltn #WarnPress');
    
  end % if WorkingFig
  
%%%%%%%%%%%%%%%%%
%%% WarnPress %%%
%%%%%%%%%%%%%%%%%
elseif strcmp(Action,'#WarnPress'),

    %% The following statement should work but it doesn't because of
    %% a bug, so we have to do a work around    
    if strcmp(get(WorkingFig,'SelectionType'),'open'),
      OpenFlag=1;
    else,
      OpenFlag=0;
    end    
    ResetHandle=findobj(WorkingFig, ...
                      'Type'      ,'uimenu'             , ...
                      'Tag'       ,'MenuSimulationReset'  ...
                      );
    ResetVal=get(ResetHandle,'UserData');
    if ~OpenFlag & ~ResetVal,    
      set(ResetHandle,'UserData',1);
      Answer=questdlg(['The object(s) must be reset to their initial '
                       'positions before any modifications to the    '
                       'animation figure window can take place.      '
                       '                                             '
                       'Do you wish to reset the object(s) now?      '], ...
                       'Yes','No');
                     
      set(ResetHandle,'UserData',0);                   
      if strcmp(Answer,'Yes'),ansimltn('#Reset',WorkingFig);end
    end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Reset Object Postions %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif strcmp(Action,'#Reset'),
  
  ansimltn('#Enable',WorkingFig);
  AllPtsData=get(WorkingAxes,'UserData');
  if ~isempty(AllPtsData),
    SetICHandle=findobj(WorkingFig, ...
                       'Type'     ,'uimenu'             , ...
                       'Tag'      ,'MenuSimulationSetIC'  ...
                       );
    
    u=zeros(1,100);
    uTemp=eval(get(SetICHandle,'UserData'),'zeros(1,100)');
    u(1:length(uTemp))=uTemp;
    
    ansetobj(WorkingFig);
    anreset(WorkingFig,u);
  end %if ~isempty
  ResetHandle=findobj(WorkingFig,'Type','uicontrol','Tag','ResetButton');
  set(ResetHandle,'Visible','off');
  ResetMenuHandle=findobj(WorkingFig, ...
                         'Type'     ,'uimenu'             , ...
                         'Tag'      ,'MenuSimulationReset'  ...
                         );
  set(ResetMenuHandle,'Enable','off');
  
  HStatus=findobj(WorkingFig,'Type','uicontrol','Tag','StatusBar');
  if isempty(AllPtsData),
    set(HStatus, ...
       'String',['Simulation Stopped. ' ...
       'Use Object toolbar to add animation objects.'] ...
       );
  else,
    set(HStatus,'String','Simulation Stopped.');
  end
  anedit('#Deselect',WorkingFig)      

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Set Input Initial Conditions %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif strcmp(Action,'#SetIC'),

  Handle=findobj(WorkingFig,'Type','uimenu','Tag','MenuSimulationSetIC');
  CurrentData=get(Handle,'UserData');

  if CurrentData(1)~='[',CurrentData=['[ ' CurrentData];end
  if CurrentData(length(CurrentData))~=']',CurrentData=[CurrentData ' ]'];end
  
  TextStrings='Initial Condition Vector:';
  Values=CurrentData;
  Callbacks=setstr(ones(length(Values(:,1))+1,1)*'ansimltn #UpdateIC');
  Tags='ICVector';
  FigTag='SetICFig';
  FigTitle=['Initial Input Vector: ' get_param(SimHandle,'Name')];
  
  FigHelp=['This vector determines the positioning of the objects'
           'when Reset is selected.  The vector sets the values  '
           'of the signals for objects whose position depends on '
           'a signal.  The vector allows for up to 100 input     '
           'signals and the default value for all signals is 0.  '
          ];

  Handles = anprpdlg('#Initialize', Handle, WorkingFig, ...
                     TextStrings  , Values, Callbacks , ...
                     Tags         , FigTag, FigTitle  , ...
                     FigHelp                            ...
                     );

%%%%%%%%%%%%%%%%%%%%%%%%
%%% Update IC Vector %%%
%%%%%%%%%%%%%%%%%%%%%%%%
elseif strcmp(Action,'#UpdateIC'),
  SetICFig=get(0,'CurrentFigure');
  SetData=get(SetICFig,'UserData');
  WorkingFig=SetData(1);
  ICVectHandle=findobj(SetICFig,'Type','uicontrol','Tag','ICVector');
  ICVectString=get(ICVectHandle,'String');
  ICVectString=deblank(fliplr(ICVectString));
  ICVectString=deblank(fliplr(ICVectString));
  if ICVectString(1)~='[',
    ICVectString=['[ ' ICVectString];
  end  
  if ICVectString(length(ICVectString))~=']',
    ICVectString=[ICVectString ' ]'];
  end
  
  SetICHandle=findobj(WorkingFig,'Type','uimenu','Tag','MenuSimulationSetIC');
  set(SetICHandle,'UserData',ICVectString);
  ansimltn('#Reset',WorkingFig);
  
end % if Action
