 echo on
% This file runs assorted tests on the mex version of EPISODE
% Episode integrates Simulink S-functions.
% The file EPISYSTEM2 serves as a nice test case.  It has 3 states,
% 2 outputs, and 2 inputs.  The 1st output equals the first state
% and is a exponential filter with time constant 1.  This is handy
% for testing piece-wise linear vs piece-wise constant inputs
% EPISODE behaves much like RK23, RK45, GEAR, etc.

% The general calling convention for EPISODE is
% [t,x, y]=episode('sysname',tfinal,x0,options,utable, P1, ..., Pn, 'jacname') 
% where all lhs arguments and all rhs arguments AFTER tfinal are optional  
pause
% Here is quick first test

plant='episystem2';  % You can use ANY S-function plant model
tfinal=2;  % or use a vector [tstart tend] if tstart is not 0.0
clg
episode(plant,tfinal)
% With no LHS arguments, episode computes and plots the output
pause
clc, home,
% RHS 3 redefines the intial conditions:
x0 =[1.2 1.2 1.2]';	% this initial condition differs from S-function default 

% RHS 4 is a vector of options modifying EPISODE's behavior
% first 6 options are analogous to those of RK23 etc.
opt1 = 1e-5;  % relative error bound (default is 1e-5)
opt2 = 0.00001; % min step size - has no effect on EPISODE at this time
                % but MUST be less than max step size. 
opt3 = 0.04; % max step size  -- initial trial step size and 
             % output size spacing. default is (tfinal-tstart)/50
             % We'll play with this in a bit to see its effect.
opt4 = 10; % MF method flag -- see original EPISODE documentation 
           % (default = 10)
opt5 = 0; % display EPISODE "warnings" if > 0 (default is -1) 
          % (fatal errors always displayed)
opt6 = 1;  %  set to 2 for plotting (default is 1)
pause

% The following are additional options used by episode
opt7 = 3; % IERROR to episode - see EPISODE documentation (default = 3)
opt8 = 1; % INDEX value - if 1 results use interpolation (default mode)
         % if 2, step size adjusted to hit output times exactly
opt9 = 1; % if 2 uses piece-wise constant inputs. 
          % if not = 2, uses piece-wise linear inputs (default mode)
	         % NOTE: this option has no effect unless RHS 5 is a table

options = [opt1 opt2 opt3 opt4 opt5 opt6 opt7 opt8 opt9];
% note: if any options = 0  gives the default for said option
pause

clc,home,
% here we try out some of the options (We'll do others later)
 [t,x,y]=episode(plant,tfinal); % uses all defaults
 [tic,xic,yic]=episode(plant,tfinal,x0); % same but with new ic's

% a simple plot of the outputs verifies that new ic's were used
pause
clg, plot(t,y,tic,yic,'--'), pause

% now change relative error
opt1=1e-1; % this is a rather large value, so results my not be accurate
[tb,xb,yb]=episode(plant,tfinal,[],opt1); % same, but large relative error
%(note that if fewer than 9 options are given(as above)
% the remaining unspecified options resort back to defaults)

% plot these on the small error curve using '*'
clg, plot(t,y,tb,yb,'*'), pause
% indeed, the error is quite noticible
pause

clc,home,
% now we try a new MF and new index with a full options vector
options(4)=23; % new MF uses implicit method w/ full numerical Jacobian
options(8)=2; % Index=2 --> step exactly to each t in output vector
[tn,xn,yn]=episode(plant,tfinal,x0,options);

clg, plot(tic,yic,tn,yn,'*'), pause
% note that FOR THIS PROBLEM, MF and INDEX have no noticible effect

pause, clc, home,
% now we toy with max step size
options(4)=0; % set MF back to default
options(8)=0; % set INDEX back to default
options(3)=5; % this is such a large step that EPISODE's output 
             % contains only tstart and tfinal values
[t1,x1,y1]=episode(plant,tfinal,x0,options);

options(3) = .2; % this gives us output every 0.2 seconds
[t2,x2,y2]=episode(plant,tfinal,x0,options);

options(3) = 0.001; % this asks for 2000 steps, but we have a limit
                   % of 100, so max step size becomes (tfinal-tstart)/100
[tb,xb,yb] = episode(plant,tfinal,x0,options);

% the * markers in the following plots denote the individual data points
clg, subplot(221), plot(tic,yic,t1,y1,'*'),
subplot(222), plot(tic,yic,t2,y2,'*'),
subplot(223), plot(tic,yic,tb,yb,'*'), pause
 
 
clc, home
% RHS 5 can be a string containing the name of an Mfile that calculates
% the current input based solely on the current time eg 'my_f_of_t'
% Alternatively, RHS 5 can be a table of times and input values organized as
% follows:
%            [t0 u1(t0) u2(t0) ... um(t0)
%             t1 u1(t1) u2(t1) ... um(t1)
%               
%             tf u1(tf) u2(tf) ... um(tf)]
%
%      where t0 < t1 < ... < tf
% NOTE!!!: the time vector in this matrix over-rides the RHS(2) values
%           and generates output ONLY for each ti in the matrix
%           ie in the above example, tstart=t0 and tend = tf

pause, clc, home,
% lets test using inputs.  First we use an Mfile
[ti,xi,yi]=episode(plant,tfinal,x0,[],'my_u_of_t');
% Observe that the input case (dashed lines) alters the output
% from the standard zero input (solid line) output 
clg, plot(tic,yic,ti,yi,'--'), pause

% now we'll try an input table 
utable = [0.0 0.0 0.0; 0.1 0.0 0.0; 0.3 0.0 0.0; 0.5 0.0 0.0;...
          1.0 0.0 2.0; 1.5 0.0 0.0; 2.0 0.0 0.0 ];
% By default, the inputs act like piece-wise ramps. For example 
% input2 ramps from 0 to 2 between t=.5 and t=1 and
% then ramps from 2 to 0 between t=1 and t=1.5

[trk,xrk, yrk]=rk23(plant,tfinal,x0,[],utable); % compare rk23 results
[tr,xr, yr]=episode(plant,tfinal,x0,[],utable);
clg, plot(trk,xrk,tr,xr,'*'), pause
% as seen, the results are essentially the same using episode or RK23
    
% now use input 1 and compare step vs ramp
x0n=[0 0 0]';  % this is a steady state for this system
utable=[0.0 0.0 0.0; 1.0 2.0 0.0; 2.0 2.0 0.0; 3.0 2.0 0.0;...
          4.0 2.0 0.0; 5.0 2.0 0.0; 6.0 2.0 0.0; 10.0 0.0 0.0; 
          11.0 0.0 0.0; 13.0 0.0 0.0; 15.0 0.0 0.0];

[tr,xr,yr]=episode(plant,15,x0n,[],utable);  % ramp type input

options(9) = 2; % utable interpreted as piece-wise constants
options(8)=2;
% NOTE the input table describes a square pulse in the middle of
% the time interval (between t=1 and t=10). Without the pulse, the 
% systems sits at a steady state.   In such cases, unless INDEX=2, 
% Episode may step beyond the input pulse, giving you a null response.

[tc,xc,yc]=episode(plant,15,x0n,options,utable);

% Note in the following graph that the dashed line (the pulse input 
% result for state 1) looks (as it should) like a square pulse passing
% through an exponential filter.  The solid line results from an
% input that has a steep ramp up to and much shallower ramp down from
% the peak value of 2.0  
clg, plot(tr,xr(:,1), tc,xc(:,1),'--'), pause

clc, home
% finally, we try using the optional parameters
P1 = 0.5;
[tic2,xic2,yic2]=episode(plant,tfinal,x0,[],[],P1);
% In this case, P1 should (and does) produce xic=xic2, with yic2 = yic/2
clg, subplot(211), plot(tic,xic,tic2,xic2,'*'),
subplot(212), plot(tic,yic,tic2,yic2,'*')
pause,

P1 = 0.0;
[tic2,xic2,yic2]=episode(plant,tfinal,x0,[],[],P1);
% With this P1 value, yic2 = 0, and the states behave independantly
% as simple exponential decays (input was zero)
clg, subplot(211), plot(tic2, xic2,'--'),
subplot(212), plot(tic2,yic2,'--')
pause

% Hope you find the MEX version of EPISODE a useful addition
% to the MATLAB/S integration options.
pause

echo off,
clear,
clg, clc, home  




 
