%
% GRIBSINI  Initialises GRIBS data
%
%              *** METRIC ***
%
%                                 J King  12-Dec-95

% GRIBS can be freely distributed as long as the usual reference/
% acknowledgement is made to my PhD thesis:

%     "Robust Mulitvariable Control Law Design for Missile Autopilots"
%     J King
%     Cranfield University, RMCS
%     Shrivenham
%     SWINDON
%     Wilts SN6 8LA
%     U.K.


%
%========================================================================
%		Initial Operating Point
%========================================================================
%
% Specify:  (for straight and level flight)
%
%     (1) Mach number
%     (2) Height (m)
%     (3) Theta (deg)
%

	Mach_0	= 0.7;
	height_0 = 4000;	% height (m)
%
	phi_0	= 0;		% degrees
	theta_0	= 6;		% degrees
	psi_0	= 0;		% degrees
%
	sigma_0 = theta_0;	% Straight and level flight
	lamda_0 = 0.0;

%
%========================================================================
%		Degrees of Freedom
%========================================================================
%
% Set to 1 for full freedom, 0 for fixing state values to initial values
% (useful for looking at just longitudinals etc)
%
	U_dof = 1;
	V_dof = 1;
	W_dof = 1;
	P_dof = 1;
	Q_dof = 1;
	R_dof = 1;
%
%========================================================================
%		Constants
%========================================================================
%
	d2r = pi / 180;			% degrees to radians
	r2d = 180 / pi;			% radians to degrees
	g_0 = 9.80665;			% Gravitational acceleration (METRIC)
%
%========================================================================
%		Simple Atmospheric Model (Sea-level values)
%========================================================================
% Derived from Horton's MSc Thesis, Bath University, UK
%
	rho_sl = 1.23;				% air density at sea level (kg/m^3)
	s_sl   = 340.0;				% speed of sound at sea level (m/s)
%
	alt_0  = height_0 / 1000;		% Altitude (km)
	rho_0  = rho_sl - 0.094*alt_0;		% simple model of air density at altitude
	if (rho_0 <= 0.23), rho_0 = 0.23;, end	% Limit rho so not -ve
	s_0    = s_sl - 4*alt_0;		% simple model for speed of sound at alt.
%
	Vt_0   = Mach_0 * s_0;			% Total velocity
%	Mach_0 = Vt_0/s_0;			% mach number
	qbar_0 = 0.5*rho_0*Vt_0*Vt_0;		% dynamic pressure
%
%========================================================================
%		Quaternion Setup
%========================================================================
%
% Convert initial euler angles to quaterions
%
	cphi2 = cos(phi_0*d2r/2);
	sphi2 = sin(phi_0*d2r/2);
	cthe2 = cos(theta_0*d2r/2);
	sthe2 = sin(theta_0*d2r/2);
	cpsi2 = cos(psi_0*d2r/2);
	spsi2 = sin(psi_0*d2r/2);
%
%
	e0 = ( cphi2 * cthe2 * cpsi2 ) + ( sphi2 * sthe2 * spsi2 );
	e1 = ( sphi2 * cthe2 * cpsi2 ) - ( cphi2 * sthe2 * spsi2 );
	e2 = ( cphi2 * sthe2 * cpsi2 ) + ( sphi2 * cthe2 * spsi2 );
	e3 = ( cphi2 * cthe2 * spsi2 ) - ( sphi2 * sthe2 * cpsi2 );
%
% Enforce normalization
%
	quat_scale = 1 / sqrt( e0^2 + e1^2 + e2^2 + e3^2 );
%
	kquat = 10.0;
%
%========================================================================
%		Initial values on states
%========================================================================
%
	U_0 = Vt_0*cos(theta_0*d2r);
	V_0 = 0.0;
	W_0 = Vt_0*sin(theta_0*d2r);
	P_0 = 0.0;
	Q_0 = 0.0;
	R_0 = 0.0;
%
	e0_0 = e0 * quat_scale;
	e1_0 = e1 * quat_scale;
	e2_0 = e2 * quat_scale;
	e3_0 = e3 * quat_scale;
%
	North_0 = 0.0;
	East_0  = 0.0;
	Down_0  = -height_0;

%
%===============================================================================
%
%botbotbotbotbotbotbotbotbotbotbotbotbotbotbotbotbotbotbotbotbotbotbotbotbotbotbotbotbot

