function copyblok(hblkDestination, hblkSource, bAdd )
%COPYBLOK	  copy block to system.
%     Copies a block, and all of its components into a model or submodel
%
% INPUTS:
%     hblkDestination: Block handle to the destination. Must be a 
%         subsystem block
%     hBlkSource: Block handle to source block.
%     bAdd: If set, then the blocks, including hblkSource are added.
%          otherwise, the children of hblkSource replace the children of
%          hblkDestination...
% OUTPUTS:
%	none
% GLOBALS:
%	none
%
%TestData:  hblkDestination = vhAllBlocks(i)
%TestData:  hblkSource=vhVersionBlock; bAdd = 1;
%TestData:  hblkDestination = get_param(bdroot,'Handle');nargin = 2;
%Testdata:  hblkSource = get_param([bdroot,'/Version'],'Handle'); bAdd = 1;
%Testdata:  hblkDestination = vhChildren(i); hblkSource = vhLibBlock; bAdd = 0;
%Testdata:  hblkDestination = get_param('Untitled/Random Masked Block', 'handle');
%Testdata:  hblkSource = get_param('bdlib/Random Masked Block','handle');
%
%	Author: Mike Wolf 7/11/95
%	Copyright (c) 1995 by The MathWorks Inc.
%	Revision: Bob Silverman 8/21/95
%	   stylistic changes
%	   function name changes
 
if nargin < 2
   error "copyblok requires at least two arguments."
end
if nargin == 2
   bAdd = 1;
end

%	Get the type info for the destination block

sType = get_param( hblkDestination, 'BlockType');
if ~strcmp(sType,'Subsystem') &  ~strcmp(sType,'System') 
   error "Destination block must be a subsystem block"
end

%	get fully qualified name of destination block
 
sNameParent = get_param( hblkDestination, 'Parent');
if length( sNameParent) ~= 0 
   sNameParent = [sNameParent, '/' ];
end

sNameDestination = [ sNameParent, get_param(hblkDestination, 'Name')];

% now see if we are adding a block (and children), or replacing children
 
if bAdd			 % we are adding a block: make its name...
   sBaseName = [sNameDestination, '/', get_param( hblkSource, 'Name')];
 
%	Make sure this is not a duplicate....
 
   sNewName = sBaseName ;
   wSynNo = 0;
   while (1)
      if isempty( get_param(sNewName, 'Handle' ) )
         break;			 %does not exist, so ok to use.
      else
         wSynNo = wSynNo + 1;	 %Try the next sequence number
         sNewName = [ sBaseName, int2str( wSynNo ) ];
      end
   end
 
   sBlockType = get_param( hblkSource, 'BlockType' );
   if strcmp( sBlockType, 'Subsystem' )
      new_system( sNewName );
   else
      add_block( sBlockType, sNewName );
   end

%	if it is masked, copy the mask
 
   if ~isempty( get_param( hblkSource, 'Mask Type' ) )
      set_param( sNewName, 'Mask Type', get_param( hblkSource, 'Mask Type' ));
      set_param( sNewName, 'Mask Dialogue', get_param( hblkSource, 'Mask Dialogue' ));
      set_param( sNewName, 'Mask Translate', get_param( hblkSource, 'Mask Translate' ));
      set_param( sNewName, 'Mask Help', get_param( hblkSource, 'Mask Help' ));
      set_param( sNewName, 'Mask Entries', get_param( hblkSource, 'Mask Entries' ));
      set_param( sNewName, 'Mask Display', get_param( hblkSource, 'Mask Display' ));
      set_param( sNewName, 'Drop Shadow', get_param( hblkSource, 'Drop Shadow' ));

%	Get the dimensions of the enclosing subsystem's display window....
 
      vLocation = get_param( hblkDestination, 'location');
      dwWinHeight = vLocation( 4) - vLocation(2);
      dwWinWidth = vLocation(3) - vLocation(1);

%	Get the dimensions of the copied subsystem....
 
      vPosition = get_param( hblkSource, 'position' );
      dwBlockHeight = vPosition(4) - vPosition(2);
      dwBlockWidth = vPosition(3) - vPosition(1);

      x1 = dwWinWidth - dwBlockWidth;
      y1 = dwWinHeight - dwBlockHeight;
      offset = .45*dwBlockHeight;
      vPosition1 = [ x1, y1-offset, dwWinWidth, dwWinHeight-offset];
      set_param( sNewName, 'position', vPosition1 );
   end 	% of ~isempty
 
   if strcmp( sBlockType, 'SubSystem' )
 
%	now add all the children to this new block....
 
      vhChildren = getchild( hblkSource );
      for i = 1:length(vhChildren)
         copyblok( get_param(sNewName, 'Handle' ), vhChildren(i), 1 );
      end
   end
else	% if bAdd
 
%	if we are replacing, not adding, then we must create a new set
%	of inports to replace the old ones; then delete all the old children
%	then add the contents....
 
   vhDestChildren = getchild( hblkDestination );
   nInports = length(getblock( vhDestChildren, 'BlockType', 'Inport'));
   nOutports = length(getblock( vhDestChildren, 'BlockType', 'Outport'));

   wPortNo = nInports + 1;
   if nInports > 0 
      for i = 1:nInports
         while 1
            sDestBlock = [sNameDestination,'/Inport',int2str(wPortNo) ];
            if isempty(get_param(sDestBlock, 'Handle' ))
            break;
         end
         wPortNo = wPortNo + 1;
         end	 %while
      add_block( 'built-in/Inport', sDestBlock );
      vInPorts(i) = wPortNo;
      wPortNo = wPortNo  + 1;
      end	 %for 
   end   %if Inports
 
   wPortNo = nOutports;
 
   if nOutports > 0
      for i = 1:nOutports
         while 1
            sDestBlock = [sNameDestination,'/Outport',int2str(wPortNo) ];
            if isempty(get_param(sDestBlock, 'Handle' ))
            break;
         end
         wPortNo = wPortNo + 1;
         end    %while
      add_block( 'built-in/Outport', sDestBlock );
      vOutPorts(i) = wPortNo;
      end   %for
   end %if

%	Now delete the original children
 
   for i = 1:length(vhDestChildren)
      delete_block( vhDestChildren(i) );
   end

   sDestParent =  get_param(hblkDestination,'Parent');
   sDestName = get_param(hblkDestination,'name');
   sFullNameParent = sprintf('%s/%s', sDestParent, sDestName);
 
%	And delete all the lines
 
   mxLines = get_param(hblkDestination, 'Lines');
   [r,c] = size(mxLines);
   for i = 1:r
      pt = mxLines(i,6:7);
      delete_line( sFullNameParent, pt );
   end
%   disp('done deleting children');
 
%	add the children of the source
 
   vhSourceChildren = getchild( hblkSource );
   sSource = get_param( vhSourceChildren(1), 'Parent');
 
%%%%   NOTE POSSIBLE NAME COLLISION IF NEW NAME MATCHES SYNTHESIZED NAME
 
   for i = 1:length( vhSourceChildren)
      sName = get_param(vhSourceChildren(i), 'Name');
      sNewName =sprintf('%s/%s/%s',sDestParent,sDestName, sName);
      sSourceName = sprintf('%s/%s', sSource, get_param(vhSourceChildren(i),'Name') );
      add_block( sSourceName, sNewName );
   end

%	finally, delete the inports and outports that were added.
 
   for i = 1:length(vInPorts)
      sDestBlock = [sNameDestination,'/Inport',int2str(vInPorts(i)) ];
%      disp(sprintf('deleting block %s', sDestBlock));
      delete_block( sDestBlock);
   end
   for i = 1:length(vOutPorts)
      sDestBlock = [sNameDestination,'/Outport',int2str(vOutPorts(i)) ]; 
%      disp(sprintf('deleting block %s', sDestBlock));
      delete_block( sDestBlock);
   end
 
%%	hblkSource = get_param('bdlib/Random Masked Block', 'handle');
 
   mxLines = get_param( hblkSource, 'Lines');
   [r,c] = size(mxLines);
   for i = 1:r
      vLine = mxLines(i,:);
      vLine = vLine( 6:length(vLine));
      vLine = vLine(find(~isnan(vLine)));
      mxLine = reshape( vLine, 2, length(vLine)/2);
      mxLine = mxLine';
      add_line( sNameDestination, mxLine );
   end
end	% if bAdd
