/*
 * STVDTF  Time Varying Discrete Transfer Function block
 *
 *          This block implements a discrete time transfer function
 *          whose transfer function polynomials are passed in via
 *          the input vector.  This is useful for discrete time
 *          adapative control applications.
 *
 *  Rick Spada, 3-17-94
 *  Copyright (c) 1994 by The MathWorks, Inc.
 *  All Rights Reserved
 */

/*
 * The following #define is used to specify the name of this S-Function.
 */

#define S_FUNCTION_NAME stvdtf

#include <stdio.h>    /* needed for declaration of sprintf */

#ifdef MATLAB_MEX_FILE
#include "mex.h"      /* needed for declaration of mexErrMsgTxt */
#endif

/*
 * Need to include simstruc.h for the definition of the SimStruct and
 * its associated macro definitions.
 */

#include "simstruc.h"

/*
 * Defines for easy access to the numerator and denominator polynomials
 * that are passed in.
 */

#define NUMERATOR         ssGetArg(S, 0)
#define DENOMINATOR       ssGetArg(S, 1)
#define PARAM_SAMPLE_TIME ssGetArg(S, 2)
#define SAMPLE_TIME       ssGetArg(S, 3)

#define NUM_ARGS          4

#define GetNumEl(mat)     (mxGetM(mat)*mxGetN(mat))

/*
 * mdlInitializeSizes - initialize the sizes array
 *
 * The sizes array is used by SIMULINK to determine the S-function block's
 * characteristics (number of inputs, outputs, states, etc.).
 */

static void mdlInitializeSizes(SimStruct *S)
{
    int numDiscStates;
    int numInputs;
    
    /*
     * verify that the proper number of arguments were passed in
     */
    
    if (ssGetNumArgs(S) != NUM_ARGS) {
#ifdef MATLAB_MEX_FILE
	char err_msg[256];
	sprintf(err_msg, "Wrong number of input arguments passed to S-function MEX-file.\n\
%d input arguments were passed in when expecting %d input arguments.\n", ssGetNumArgs(S) + 4, NUM_ARGS + 4);
	mexErrMsgTxt(err_msg);
#endif
	return;
    }

    /*
     * the numerator must be of a lower order than the denominator
     */
    
    if (GetNumEl(NUMERATOR) > GetNumEl(DENOMINATOR)) {
#ifdef MATLAB_MEX_FILE
	mexErrMsgTxt("The denominator must be of higher order than the numerator.");
#endif
	return;
    }

    /*
     * the parameter sample rate must be slower than the block sample time
     */

    if (mxGetPr(PARAM_SAMPLE_TIME)[0] < mxGetPr(SAMPLE_TIME)[0]) {
#ifdef MATLAB_MEX_FILE
	mexErrMsgTxt("The parameter sample rate must be slower than the block sample rate");
#endif
	return;
    }

    numDiscStates = GetNumEl(DENOMINATOR) - 1;
    numInputs = 1 + 2 * (numDiscStates + 1);
    
    ssSetNumContStates(    S, 0);                   /* number of continuous states */
    ssSetNumDiscStates(    S, numDiscStates);       /* number of discrete states */
    ssSetNumInputs(        S, numInputs);           /* number of inputs */
    ssSetNumOutputs(       S, 1);                   /* number of outputs */
    ssSetDirectFeedThrough(S, 0);                   /* direct feedthrough flag */
    ssSetNumSampleTimes(   S, 2);                   /* number of sample times */
    ssSetNumInputArgs(     S, NUM_ARGS);            /* number of input arguments */
    ssSetNumRWork(         S, 2*(numDiscStates+1)); /* number of real work vector elements */
    ssSetNumIWork(         S, 0);                   /* number of integer work vector elements */
    ssSetNumPWork(         S, 0);                   /* number of pointer work vector elements */
}

/*
 * mdlInitializeSampleTimes - initialize the sample times array
 *
 * This function is used to specify the sample time(s) for your S-function.
 * If your S-function is continuous, you must specify a sample time of 0.0.
 * Sample times must be registered in ascending order.  If your S-function
 * is to acquire the sample time of the block that is driving it, you must
 * specify the sample time to be INHERITED_SAMPLE_TIME.
 */

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTimeEvent(S, 0, mxGetPr(SAMPLE_TIME)[0]);
    ssSetSampleTimeEvent(S, 1, mxGetPr(PARAM_SAMPLE_TIME)[0]);
    
    ssSetOffsetTimeEvent(S, 0, 0.0);
    ssSetOffsetTimeEvent(S, 1, 0.0);
}

/*
 * mdlInitializeConditions - initialize the states
 *
 * In this function, you should initialize the continuous and discrete
 * states for your S-function block.  The initial states are placed
 * in the x0 variable.  You can also perform any other initialization
 * activities that your S-function may require.
 */

static void mdlInitializeConditions(double *x0, SimStruct *S)
{
    int i;
    int numDiscStates = ssGetNumDiscStates(S);
    double *num = ssGetRWork(S);
    double *den = num + ssGetNumDiscStates(S) + 1;
    double *numParam, *denParam;
    double den0;

    /*
     * the discrete states are all initialized to zero
     */

    for (i = 0; i < numDiscStates; i++) {
	x0[i]  = 0.0;
	num[i] = 0.0;
	den[i] = 0.0;
    }

    num[numDiscStates] = 0.0;
    den[numDiscStates] = 0.0;

    /*
     * squirrel away the initial numerator and denominator
     */

    denParam = mxGetPr(DENOMINATOR);
    den0 = denParam[0];
    for (i = 0; i < GetNumEl(DENOMINATOR); i++)
	*den++ = *denParam++ / den0;

    numParam = mxGetPr(NUMERATOR);
    num += ssGetNumDiscStates(S) + 1 - GetNumEl(NUMERATOR);
    for (i = 0; i < GetNumEl(NUMERATOR); i++)
	*num++ = *numParam++ / den0;

    /*
     * normalize if this transfer function has direct feedthrough
     */

    num = ssGetRWork(S);
    den = num + numDiscStates + 1;
    for (i = 1; i < numDiscStates + 1; i++)
	num[i] -= den[i]*num[0];
}

/*
 * mdlOutputs - compute the outputs
 *
 * In this function, you compute the outputs of your S-function
 * block.  The outputs are placed in the y variable.
 */

static void mdlOutputs(double *y, double *x, double *u, SimStruct *S, int tid)
{
    if (ssIsSampleHitEvent(S, 0, tid)) {
	int i;
	int numDiscStates = ssGetNumDiscStates(S);
	double *num = ssGetRWork(S);
	
	/*
	 * The discrete system is evaluated using a controllable state space
	 * representation of the transfer function.  This implies that the
	 * output of the system is equal to:
	 *
	 *     y(k) = Cx(k) + Du(k)
	 *          = [ b1 b2 ... bn]x(k) + b0u(k)
	 *
	 * where b0, b1, b2, ... are the coefficients of the numerator polynomial:
	 *
	 *    B(q) = b0 q^n + b1 q^n-1 + b2 q^n-2 + ... + bn-1 q + bn
	 */

	*y = *num++ * *u;
	for (i = 0; i < numDiscStates; i++)
	    *y += *num++ * *x++;
    }
}

/*
 * mdlUpdate - perform action at major integration time step
 *
 * This function is called once for every major integration time step.
 * Discrete states are typically updated here, but this function is useful
 * for performing any tasks that should only take place once per integration
 * step.
 */

static void mdlUpdate(double *x, double *u, SimStruct *S, int tid)
{
    int i;
    int numDiscStates = ssGetNumDiscStates(S);
    double *num = ssGetRWork(S);
    double *den = num + numDiscStates + 1;
    
    /*
     * sample hit event 0, update the discrete states
     */
    
    if (ssIsSampleHitEvent(S, 0, tid)) {
	double firstTerm, x0;
	
	/*
	 * The discrete system is evaluated using a controllable state space
	 * representation of the transfer function.  This implies that the
	 * next discrete states are computed using:
	 *
	 *     x(k+1) = Ax(k) + Bu(k)
	 *            = [-a1 -a2 ... -an] [x1(k)] + [u(k)]
	 *              [  1  0  ...   0] [x2(k)] + [0]
	 *              [  0  1  ...   0] [x3(k)] + [0]
	 *              [  .  .  ...   .]   .     +  .
	 *              [  .  .  ...   .]   .     +  .
	 *              [  .  .  ...   .]   .     +  .
	 *              [  0  0  ... 1 0] [xn(k)] + [0]
	 *
	 * where a1, a2, ... are the coefficients of the numerator polynomial:
	 *
	 *    A(q) = q^n + a1 q^n-1 + a2 q^n-2 + ... + an-1 q + an
	 */
	
	firstTerm = -den[1] * x[0];
	x0 = 0.0;
	for (i = 1; i < numDiscStates; i++) {
	    double tmp;
	    
	    tmp = x[i];
	    x[i] = x[i-1];
	    x0 -= den[i+1] * tmp;
	}
	
	x[0] = x0 + firstTerm + u[0];
    }
    
    /*
     * sample hit event 1, update the transfer function parameter inputs
     */
    
    if (ssIsSampleHitEvent(S, 1, tid)) {
	double den0;
	int numCoeffs;
	int allZero;
	
	/*
	 * the first numerator input is after the system input
	 */
	
	u++;

	/*
	 *---------------------------------------------------------------
	 * if all inputs are zero, ignore it could be unconnected inputs
	 *---------------------------------------------------------------
	 */
	
	/*
	 * Get the first denominator coefficient.  It will be used
	 * for normalizing the numerator and denominator coefficients.
	 */
	
	numCoeffs = numDiscStates + 1;
	den0 = u[numCoeffs];
	if (den0 == 0.0)
	    den0 = mxGetPr(DENOMINATOR)[0];
	
	/*
	 * grab the numerator
	 */

	allZero = 1;
	for (i = 0; (i < numCoeffs) && allZero; i++)
	    allZero &= u[i] == 0.0;

	if (allZero) {
	    double *numParam;

	    /*
	     * move the input to the denominator input
	     */
	    
	    u += numCoeffs;

	    /*

	    /*
	     * get the denominator from the input parameter
	     */
	    
	    numParam = mxGetPr(NUMERATOR);
	    num += numCoeffs - GetNumEl(NUMERATOR);
	    for (i = 0; i < GetNumEl(NUMERATOR); i++)
		*num++ = *numParam++ / den0;
	} else {
	    for (i = 0; i < numCoeffs; i++)
		*num++ = *u++ / den0;
	}
	
	/*
	 * grab the denominator
	 */
	
	allZero = 1;
	for (i = 0; (i < numCoeffs) && allZero; i++)
	    allZero &= u[i] == 0.0;

	if (allZero) {
	    double *denParam;
	    
	    denParam = mxGetPr(DENOMINATOR);
	    den0 = denParam[0];
	    for (i = 0; i < GetNumEl(DENOMINATOR); i++)
		*den++ = *denParam++ / den0;

	} else {
	    for (i = 0; i < numCoeffs; i++)
		*den++ = *u++ / den0;
	}
	
	/*
	 * normalize if this transfer function has direct feedthrough
	 */
	
	num = ssGetRWork(S);
	den = num + numCoeffs;
	for (i = 1; i < numCoeffs; i++)
	    num[i] -= den[i]*num[0];
    }
}

/*
 * mdlDerivatives - compute the derivatives
 *
 * In this function, you compute the S-function block's derivatives.
 * The derivatives are placed in the dx variable.
 */

static void mdlDerivatives(double *dx, double *x, double *u, SimStruct *S, int tid)
{
}

/*
 * mdlTerminate - called when the simulation is terminated.
 *
 * In this function, you should perform any actions that are necessary
 * at the termination of a simulation.  For example, if memory was allocated
 * in mdlInitializeConditions, this is the place to free it.
 */

static void mdlTerminate(SimStruct *S)
{
}

#ifdef	MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
