/*
 * RTCLOCK.C
 *
 *   S-Function for polled synchronization with the PC real-time clock
 *   or the UNIX system clock.
 *
 *
 *  Copyright (c) 1994 by The MathWorks, Inc.
 *  All Rights Reserved
 */


/*
	at each control time
		target_ticks = simulation_time + startup_ticks
*/


#undef S_FUNCTION_NAME
#define S_FUNCTION_NAME	rtclock

#if defined (__WATCOMC__) || (__HIGHC__)
#include <bios.h>      /* For _bios_timeofday                       */
#else                  /* !__WATCOMC__ || __HIGHC__                 */
#include <time.h>      /* For time_t                                */
#include <sys/timeb.h> /* For ftime                                 */
#endif                 /* __WATCOMC__ || __HIGHC__                  */

#include <math.h>      /* For fmod and modf                         */
#include "simstruc.h"  /* Where simulation structure, S, is defined */

#ifdef MATLAB_MEX_FILE
#include "mex.h"
#endif

/* Definitions for PC real-time clock */
#define PC_CLOCK_FREQ		(18.2)    /* Hertz   */
#define TICKS_IN_24_HOURS	(1573040)

/* Definitions for UNIX system clock */
#define UNIX_SYS_CLK_FREQ	(1000.0)  /* Hertz  */

#define SECS_IN_24_HOURS	(86400.0)
#define MAX_TIME			(1.0e6)

/* Input Arguments */
#define SAMPLE_TIME_ARG		ssGetArg(S,0) 
#define TOLERANCE_ARG		ssGetArg(S,1)
#define NUMBER_OF_ARGS		(2)

#define NSAMPLE_TIMES		(1)

/* IWORK Storage Allocation */
#define START_TIME_STAMP	(0)
#define WARNING_FLAG		(1)
#define NUMBER_OF_IWORKS	(2)

/* RWORK Storage Allocation */
#define TOLERANCE			(1)
#define NUMBER_OF_RWORKS	(2)


static void mdlInitializeSizes(S)
	SimStruct *S;
{
	if (ssGetNumArgs(S) != NUMBER_OF_ARGS) {
#ifdef MATLAB_MEX_FILE
		mexErrMsgTxt("Wrong number of input arguments passed.\nTwo arguments are expected\n");
#endif
	} else {
		/* Set-up size information */
		ssSetNumContStates(S, 0);
		ssSetNumDiscStates(S, 0);
		ssSetNumInputs(S, 0);
		ssSetNumOutputs(S, 0);
		ssSetDirectFeedThrough(S, 0);
		ssSetNumInputArgs(S, NUMBER_OF_ARGS);
		ssSetNumSampleTimes(S, NSAMPLE_TIMES);
		ssSetNumIWork(S, NUMBER_OF_IWORKS);
		ssSetNumRWork(S, NUMBER_OF_RWORKS);
	}
}
 
/* Function to initialize sample times */
static void mdlInitializeSampleTimes(S)
	SimStruct *S;
{
	double sample_time;

	sample_time = mxGetPr(SAMPLE_TIME_ARG)[0];
	ssSetSampleTimeEvent(S, 0, sample_time);
	ssSetOffsetTimeEvent(S, 0, 0);
}
 
static void mdlInitializeConditions(x0, S)
	double *x0;
	SimStruct *S;
{
#if defined (__WATCOMC__) || defined (__HIGHC__)
	long ticks;

	_bios_timeofday(_TIME_GETCLOCK, &ticks);
	ssSetIWorkValue(S, START_TIME_STAMP, ticks);
#else  /* !__WATCOMC__ || __HIGHC__ */
	struct timeb tod;
	double start_time;
	double seconds;
	double ones;
	double the_rest;

	ftime(&tod);
	/* Store the starting time as the number of seconds
	 * plus milliseconds.  The maximum time stored is 
	 * one million seconds (11.5 days).
	 */
	seconds = (double)(tod.time / MAX_TIME);
	ones = modf(seconds, &the_rest);
	start_time = (ones * MAX_TIME) + (double)(tod.millitm / UNIX_SYS_CLK_FREQ);
	ssSetRWorkValue(S, START_TIME_STAMP, start_time);
#endif  /* __WATCOMC__ || __HIGHC__ */

	ssSetRWorkValue(S, TOLERANCE, mxGetPr(TOLERANCE_ARG)[0]);
	ssSetIWorkValue(S, WARNING_FLAG, 0);
}

/* Function to compute outputs */
static void mdlOutputs(y, x, u, S, tid)
	double *y;
	double *x;
	double *u;
	SimStruct *S;
	int tid;
{
}

/* Function to compute model update */
#if defined (__WATCOMC__) || defined (__HIGHC__)
static void mdlUpdate(x, u, S, tid)
	double *x;
	double *u;
	SimStruct *S;
	int tid;
{
	long current_ticks;
	long start_ticks;
	long target_ticks;
	double time_elapsed;
	double sample_time;
	double tolerance;

	/**** obtain current PC clock ticks since midnight ****/
#ifdef __WATCOMC__
	if (_bios_timeofday(_TIME_GETCLOCK, &current_ticks))
#elif __HIGHC__
	if (!_bios_timeofday(_TIME_GETCLOCK, &current_ticks))
#endif
	{ /* passed midnight */
		ssSetIWorkValue(S, START_TIME_STAMP, 0);
	}

	/**** update sample time on the fly ****/
	sample_time = mxGetPr(SAMPLE_TIME_ARG)[0];
	ssSetSampleTimeEvent(S, 0, sample_time);

	/* Get the tolerance and convert it to clock ticks */
	tolerance = ssGetRWorkValue(S, TOLERANCE);
	tolerance *= PC_CLOCK_FREQ;

	start_ticks = ssGetIWorkValue(S, START_TIME_STAMP);
	time_elapsed = fmod(ssGetT(S), SECS_IN_24_HOURS);
	target_ticks = (long)(time_elapsed * PC_CLOCK_FREQ) + start_ticks;

	/**** synchronize with real-time ****/
	while (current_ticks < target_ticks)
#ifdef __WATCOMC__
		if (_bios_timeofday(_TIME_GETCLOCK, &current_ticks))
#elif __HIGHC__
		if (!_bios_timeofday(_TIME_GETCLOCK, &current_ticks))
#endif
		{
			ssSetIWorkValue(S, START_TIME_STAMP, 0);
			target_ticks %= TICKS_IN_24_HOURS;
		}

		if ((current_ticks > target_ticks + (long)tolerance) &&
			!ssGetIWorkValue(S, WARNING_FLAG))
		{
			mexPrintf("Warning: Simulation may be slower than real time\n");
			ssSetIWorkValue(S, WARNING_FLAG, 1);
		}
			
}
#else  /* !__WATCOMC__ || __HIGHC__ */
static void mdlUpdate(x, u, S, tid)
	double *x;
	double *u;
	SimStruct *S;
	int tid;
{
	double current_time;
	double start_ticks;
	double target_ticks;
	double time_elapsed;
	double sample_time;
	double time_ratio;
	struct timeb tod;
	double time_now;
	double ones;
	double the_rest;
	double tolerance;

	/**** update sample time on the fly ****/
	sample_time = mxGetPr(SAMPLE_TIME_ARG)[0];
	ssSetSampleTimeEvent(S, 0, sample_time);

	/* Get the tolerance */
	tolerance = ssGetRWorkValue(S, TOLERANCE);

	/* Find out how much simulation time has passed
	 * and then add it to the start time to determine
	 * how long to wait.
	 */
	start_ticks = ssGetRWorkValue(S, START_TIME_STAMP);
	time_elapsed = fmod(ssGetT(S), SECS_IN_24_HOURS);
	target_ticks = time_elapsed + start_ticks;

	/**** synchronize with the system clock ****/
	while (1)
	{
		/* Get the time now and compare it to the 
		 * target time.  If we have passed the target
		 * time, break out of the loop.
		 */
		ftime(&tod);
		time_now = (double)(tod.time / MAX_TIME);
		ones = modf(time_now, &the_rest);
		current_time = (ones * MAX_TIME) + (double)(tod.millitm / UNIX_SYS_CLK_FREQ);
		if (current_time >= target_ticks)
		{
			if ((current_time > target_ticks + tolerance) &&
				!ssGetIWorkValue(S, WARNING_FLAG))
			{
				mexPrintf("Warning: Simulation may be slower than real time\n");
				ssSetIWorkValue(S, WARNING_FLAG, 1);
			}
			break;
		}
	}
}
#endif  /* __WATCOMC__ || __HIGHC__ */

/* Function to compute derivatives */
static void mdlDerivatives(dx, x, u, S, tid)
	double *dx;
	double *x;
	double *u;
	SimStruct *S;
	int tid;
{
}

/* Function to perform housekeeping at execution termination */
static void mdlTerminate(S)
	SimStruct *S;
{
}

#ifdef MATLAB_MEX_FILE	/* Is this file being compiled as a MEX-file? */
#include "simulink.c"	/* Mex glue */
#else
#include "cg_sfun.h"	/* Code generation glue */
#endif

