/*
 *
 *  SD/FAST Interface MEX template
 *  Copyright (c) 1994 by The Mathworks, Inc.
 *  All Rights Reserved
 *
 *  by: Jay Torgerson
 *
 */

#include "mex.h"      
#include "simstruc.h" /* Where simulation structure, S, is defined */


#define sdmax(A,B)    ((A)>(B)?(A):(B))  /* special define for max operator */


/* 
 *
 * Define parameters specific to the SD/FAST interface 
 *
 */
#define NINPUTS    (int)mxGetScalar(ssGetArg(S,0))  /* number of inputs     */
#define NOUTPUTS   (int)mxGetScalar(ssGetArg(S,1))  /* number of outputs    */
#define Q0         mxGetPr(ssGetArg(S,2))           /* init position states */
#define U0         mxGetPr(ssGetArg(S,3))           /* init velocity states */
#define Q0MAT      ssGetArg(S,2)                    /* ptr to vec q0        */
#define U0MAT      ssGetArg(S,3)                    /* ptr to vec u0        */
#define NQ         (int)sdmax((mxGetM(Q0MAT)), (mxGetN(Q0MAT))) /* length of Q0 vector  */
#define NU         (int)sdmax((mxGetM(U0MAT)), (mxGetN(U0MAT))) /* length of U0 vector  */
#define LQ0MAT     ssGetArg(S,4)                    /* ptr to locked vec q0 */
#define LU0MAT     ssGetArg(S,5)                    /* ptr to locked vec u0 */
#define LOCKEDQ0   mxGetPr(ssGetArg(S,4))           /* locked q0 vecotr     */
#define LOCKEDU0   mxGetPr(ssGetArg(S,5))           /* locked u0 vecotr     */
#define TOLQ0      mxGetScalar(ssGetArg(S,6))       /* tolerance for q0     */
#define TOLU0      mxGetScalar(ssGetArg(S,7))       /* tolerance for u0     */


static void mdlInitializeSizes(S)
    SimStruct *S;
{
	/* 
	 * Setup size information  
	 */
	ssSetNumInputArgs(S,8);
	ssSetNumContStates(S,NQ+NU);       
	ssSetNumDiscStates(S,0);
	ssSetNumInputs(S,NINPUTS);
	ssSetNumOutputs(S,NOUTPUTS);
	ssSetDirectFeedThrough(S,0);  /* No direct feed-through */
}

static void mdlInitializeConditions(q0, S)
    double *q0;
    SimStruct *S;
{

    double t0 = ssGetT(S);     /* get initail time                           */
    double *u0 = &q0[NQ];      /* define u0 state vector                     */
    double *lockq0, *locku0;   /* define ptrs to locked vectors              */
    int numcalled;             /* number of times sdstate was called         */
    int err;                   /* error value from sdassemble and sdinitvel  */
    int i;                     /* dummy counter                              */

   /*
    * Check to make sure input SD/FAST parameters are ok
    */
    sdcheckparams(S,&lockq0,&locku0);

   /* 
    * Intialize q0 and u0 vectors
    */
    for (i=0;i<NQ;i++)
        q0[i] = Q0[i];
   
    for (i=0;i<NU;i++)
        u0[i] = U0[i];

   /* 
    * Initialize the SD/FAST system by calling 
    * sdinit() and sdstate(t0,q0,u0) 
    * 
    * t0 = initial time 
    * q0 = initial position state vector
    * u0 = initial velocity state vector
    */

    sdinit();
    sdassemble(t0,q0,lockq0,TOLQ0,100000,&numcalled,&err); 
    sdinitvel (t0,q0,locku0,TOLU0,100000,&numcalled,&err); 
}


static void mdlDerivatives(dq, q, inputs, S,tid)
    double *dq, *q, *inputs;
    SimStruct *S;
	int tid;
{
    double t  = ssGetT(S);          /* get the simulation time             */
    double *u  = &q[NQ];            /* define velocity state vector        */
    double *du = &dq[NQ];           /* define veclocity deriv state vector */
    
    sdstate(t,q,u);                 /* update state vectors         */
    sdinputs(t,q,u,dq,du,inputs);   /* call user defined sdinputs() */
    sdderiv(dq,du);                 /* calculate state derivatives  */
}


static void mdlOutputs(outputs, q, inputs, S,tid)
    double *outputs, *q;
	double *inputs;
    SimStruct *S;
	int tid;
{
    double t  = ssGetT(S);           /* get the simulation time       */
    double *u  = &q[NQ];             /* set velocity state vector     */
    sdoutputs(t,q,u,inputs,outputs); /* call user defined sdoutputs() */
}


sdcheckparams(S,lockq0,locku0) 
    SimStruct *S;
    double **lockq0,**locku0;
{
     int lq0,lu0;
     int numq,numu;
     int info[50];
     int i;
     

   /*
    * make sure that q0, u0, locked q0 and locked u0 are not empty and 
    *      if the locked vectors are empty fill them with zeros and 
    *      make sure that the locked vectors are zero or one only
    *      also, if the size of q is not the same as q0 or the size of
    *      u is not the same size as u0, then stop simulation and give
    *      appropriate error message
    */
     
   /* 
    * get info on system and compute what the state vectors sizes are 
    */
    sdinfo(info);
    numq = info[2] + info[7];
    numu = info[2];

   /* 
    * check to see if q0 and u0 are the same sizes as q and u respectfully  
    */
    if  ( NQ != numq )
        mexErrMsgTxt("q0 is not the same size as q");

    if  ( NU != numu )
        mexErrMsgTxt("u0 is not the same size as u");

   /* 
    * check the lock vectors and make sure that they are not empty 
    * if they are empty make them zeros                          
    */ 

   if   ( (mxGetM(LQ0MAT)*mxGetN(LQ0MAT)) == 0) {
        *lockq0 = mxCalloc(NQ,sizeof(double));
        lq0 = NQ;
    }
    else {
        *lockq0 = mxGetPr(LQ0MAT);
        lq0 = sdmax((mxGetM(LQ0MAT)), (mxGetN(LQ0MAT))); /* length of locked q0 */
    }

   if   ( (mxGetM(LU0MAT)*mxGetN(LU0MAT)) == 0) {
        *locku0 = mxCalloc(NU,sizeof(double));
        lu0 = NU;
	}
    else {
        *locku0 = mxGetPr(LU0MAT);
        lu0 = sdmax((mxGetM(LU0MAT)), (mxGetN(LU0MAT))); /* length of locked u0 */    
    }

   /*
    * make sure that the locked vectors are the right size 
    */
    if  (lq0 != NQ) 
        mexErrMsgTxt("lockedq0 and q0 vectors must be the same size");
    if  (lu0 != NU) 
        mexErrMsgTxt("lockedu0 and u0 vectors must be the same size");

   /* 
    * make sure that locked vectors are zeros or ones only  
    */
    for (i=0;i<NQ;i++) 
        if  ( ((*lockq0)[i] != 1) && ((*lockq0)[i] != 0)  && ((*lockq0)[i] != 1) )
            mexErrMsgTxt("elements of lock vectors must be 0 or 1");

    for (i=0;i<NU;i++) 
        if  ( ((*locku0)[i] != 1) && ((*locku0)[i] != 0)  && ((*locku0)[i] != 1) )
            mexErrMsgTxt("elements of lock vectors must be 0 or 1");
}



/* STUBS FOR UNUSED ACCESS FUNCTIONS */
static void mdlTerminate (S)
SimStruct *S;
{
/* do nothing */
}

static void mdlInitializeSampleTimes (S)
SimStruct *S;
{
/* do nothing */
}

static void mdlUpdate(x, u, S, tid)
    double *x, *u;
    SimStruct * S;
    int tid;
{
/* do nothing */
}


/* stub function for SD/FAST Interface */
sduforce(t,a,b) 
    double t;
    double a[];
    double b[];
{
/* do nothing */
}    

#ifdef MATLAB_MEX_FILE          /* Has standalone simulation been defined ? */
#include "simulink.c"           /* Mex glue */
#else
#include "cg_glue.h"            /* Real-time glue */
#endif

