#include <float.h>               /* define largest & smallest double precision #s */

#include "simstruc.h"            /* required for S-function */

#define S_FUNCTION_NAME table3   /* required for S-function */

/* pointers to the matrix structures */

#define X_VEC_PTR ssGetArg( S,0 )                       
#define Y_VEC_PTR ssGetArg( S,1 )
#define Z_VEC_PTR ssGetArg( S,2 ) 
#define TAB_PTR   ssGetArg( S,3 ) 

/* pointers to the real data */

#define X_VEC mxGetPr( ssGetArg( S,0 ) )
#define Y_VEC mxGetPr( ssGetArg( S,1 ) )
#define Z_VEC mxGetPr( ssGetArg( S,2 ) )
#define TAB   mxGetPr( ssGetArg( S,3 ) )

/* Give the inputs some names */

#define X_REF_IN u[0]
#define Y_REF_IN u[1]
#define Z_REF_IN u[2]

/* Macros to define the lengths of the vectors in the sense of MATLAB's LENGTH command */

#define LENGTH_X    MAX( mxGetM(X_VEC_PTR), mxGetN(X_VEC_PTR) )
#define LENGTH_Y    MAX( mxGetM(Y_VEC_PTR), mxGetN(Y_VEC_PTR) )
#define LENGTH_Z    MAX( mxGetM(Z_VEC_PTR), mxGetN(Z_VEC_PTR) )
#define LENGTH_TAB  MAX( mxGetM(TAB_PTR),   mxGetN(TAB_PTR) )

/* create 2nd level of macros for setting/getting the work values based on relevant names */

#define OLD_REF(IND)             ssGetRWorkValue( S, IND )
#define OLD_INDEX(IND)           ssGetIWorkValue( S, IND )
								 
#define SET_OLD_REF(IND,VAL)     ssSetRWorkValue(S,IND,VAL)
#define SET_OLD_INDEX(IND,VAL)   ssSetIWorkValue(S,IND,VAL)


/* misc defs */

#define TRUE  1
#define FALSE 0

#define INF DBL_MAX      /* infinity for double precision on this system */

#define MAX(A, B)  ((A) > (B) ? (A) : (B))   /* compute the max of 2 numbers */



/* supporting functions */

/*
 *1-D, linear interpolation:                                                      
 * Inputs:   Xvec:   vector of X values                                           
 *           Yvec:   vector of Y values                                           
 *
 *           Xin :   the desired X value (reference signal)                       
 *           Offset: offset for the X and Y vectors (0 for no offset)             
 *                                                                                
 * Return:   interpolated Y value                                                   
 */

double Table1( double Xvec[], double Yvec[], double Xin, int OffSet )
{
 
  double xLow = Xvec[ OffSet ];
  double xHi  = Xvec[ OffSet + 1 ];
  double yLow = Yvec[ OffSet ];
  double yHi  = Yvec[ OffSet + 1 ];

 
  if ( Xin == xHi )                
    return( yHi );                             
  else  
    return( yLow + ( ( ( yHi - yLow ) * ( Xin - xLow ) ) / ( xHi - xLow ) ) );
}



/*
 *Returns the lowindex for interpolation                                                      
 * Inputs:   Xvec     :   ptr to independent values
 *           L        :   length of X                                           
 *           Xin      :   the desired X value (reference signal)                       
 *           SearchDir:   search direction
 *           SimStruct:   the simstruct
 *           Offset   : offset for the X and Y vectors (0 for no offset)
 *           WorkIndex: the index into the work vectors to retrieve appropriate old indices     
 *                                                                                
 * Return:   the index                                                   
 */
int GetIndex( double Xvec[], int L, double Xin, int SearchDir, SimStruct *S, int WorkIndex )
{
 int i, LowIndex;
 int OldIndex = OLD_INDEX(WorkIndex);  /* 
					* retrieve the index from the previous search 
					* start search here, for speed
					*/
 
 /* determine "bracketing" numbers */
 /*mexPrintf("\nNEW: OldIndex = %d, Searchdir = %d", OldIndex, SearchDir); */
 if (SearchDir > 0) {

   for (i = OldIndex; i < L; i++) {
   
     if ( Xvec[i] >= Xin ) {
        LowIndex = i - 1;
        break;
     }
   }
 
 }
 else {
 
   for (i = OldIndex; i >= 0; i--) {
     /*mexPrintf("i = %d, Xvec[i] = %f, Xin = %f", i, Xvec[i], Xin);*/
     if ( Xvec[i] < Xin ) {
        LowIndex = i;
        break;
     }
   }
 } 
 
 /* store the index from this time step in a work vector */
 if (i == L) {
   /* stepped over top boundary */
   SET_OLD_INDEX(WorkIndex,L-1);
 } else if (i < 0) {
   /* stepped over lower boundary */
   SET_OLD_INDEX(WorkIndex,0);
 } else {
   /* no problem, just use i */
   SET_OLD_INDEX(WorkIndex,i);
 }  


 /* check for out of range cases to setup for extrapolation */

 if ( i == L  )                     LowIndex = L-2;
 if ( i == -1 )                     LowIndex = 0; 
 if ( (i == 0) && (SearchDir > 0) ) LowIndex = 0;

 /*mexPrintf("\nLowIndex = %d", LowIndex); */
 return(LowIndex);

} /* end GetLowIndex */



/* This function returns the matrix element corresponeding to the row and col       
 *  indices (C-Style:start with 0) that are entered.  The inputs are the dimensions 
 *  of the table (M & N), the desired row and column (row, col), and the pointer to 
 *  the real data of the matrix                                                     
 */

double GetMatrixElement( int M, int N, int row, int col, double mat[] )
{
 
 if ( (row >= M) || (col >= N) || (row < 0) || (col < 0)  )
    mexErrMsgTxt("exceeded matrix dimensions");
  
 
 return( mat[ ( col * M ) + row ] );

} /* end GetMatrixElement */



/*
 * This function implements a 2-D table.  Pass in the pointers to the x and y vectors,    
 * the index of the lower bracketing numbers for x and y, the desired x and y values,     
 * a pointer to the real data of the table and the pointer to the table matrix structure. 
 * The interpolated value is returned.                                                    
 */                                                                                                                                                                                
 

double Table2( double Xvec[], double Yvec[], int XLowIndex, int YLowIndex,
			   double Xin, double Yin, double tab[], Matrix *TabPtr )
{
 int i, j;
 double ytemp[2], TempTab[2];
		
 
 /* First build a table by using TABLE1 to get values at the bracketing y numbers *
  * based on the desired x value (Xin).  The result is TempTab.                   */
 
 for ( i=0; i<2; i++ ) {
	for ( j=0; j<2; j++ ) {
	ytemp[j] = GetMatrixElement( mxGetM( TabPtr ), mxGetN( TabPtr ),
									 YLowIndex + i, XLowIndex + j, tab );
	}

	TempTab[i] = Table1( Xvec + XLowIndex, ytemp, Xin, 0 );
 }
	
 /* use Table 1 to interpolate TempTab based on the desired Y value (Yin) */

 return(Table1( Yvec + YLowIndex, TempTab, Yin, 0 ) );

}  /* end Table2 */




/* This function returns TRUE if the matrix is a vector and FALSE otherwise. *
 * The input argument is a pointer to a matrix                               */

int IsVector( Matrix *mat )
{

  int m,n;

  m = mxGetM(mat); n = mxGetN(mat);

  if ( (m != 1) && (n != 1) )             /* one of the dimensions must be 1 */
    return(FALSE);      
  else
    return(TRUE);  
}



/* This function returns TRUE if the vector is monotinic and FALSE otherwise. *
 * Pass in a pointer to the real data, and the length of the vector           */

int IsMonotonic( double v[], int l )
{ 
  int i;

  for ( i=2; i<l; i++) {
     if ( v[i-1] >= v[i] )
	return(FALSE);
  }
  return(TRUE);
}






static void mdlInitializeSizes(SimStruct *S)
{
    ssSetNumContStates     ( S, 0 );      /* number of continuous states */
    ssSetNumDiscStates     ( S, 0 );      /* number of discrete states */
    ssSetNumInputs         ( S, 3 );      /* number of inputs */
    ssSetNumOutputs        ( S, 1 );      /* number of outputs */
    ssSetDirectFeedThrough ( S, 1 );      /* direct feedthrough flag */
    ssSetNumSampleTimes    ( S, 1 );      /* number of sample times */
    ssSetNumInputArgs      ( S, 4 );      /* number of input arguments */
    ssSetNumRWork          ( S, 3 );      /* number of real work vector elements */
    ssSetNumIWork          ( S, 3 );      /* number of integer work vector elements */
    ssSetNumPWork          ( S, 0 );      /* number of pointer work vector elements */
						    
}


static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTimeEvent( S, 0, INHERITED_SAMPLE_TIME );
    ssSetOffsetTimeEvent( S, 0, 0.0 );
}

static void mdlInitializeConditions(double *x0, SimStruct *S)
{
  
 #ifdef  MATLAB_MEX_FILE                     /* error checking only if mex file */ 
  /* Are X Y and Z vectors? */

  if ( !( IsVector( X_VEC_PTR ) ) || !( IsVector( Y_VEC_PTR ) ) || !( IsVector( Z_VEC_PTR ) ) )
     mexErrMsgTxt("X Y and Z must be vectors");


  /* ARE X Y and Z monotonic? */

  if ( !( IsMonotonic( X_VEC, MAX( mxGetM( X_VEC_PTR ), mxGetN( X_VEC_PTR ) ) ) ) ||
	   !( IsMonotonic( Y_VEC, MAX( mxGetM( Y_VEC_PTR ), mxGetN( Y_VEC_PTR ) ) ) ) ||
	   !( IsMonotonic( Z_VEC, MAX( mxGetM( Z_VEC_PTR ), mxGetN( Z_VEC_PTR ) ) ) )     )
	{
	 mexErrMsgTxt("X Y and Z must be monotonic");
	}

	
  /* Make sure that the table is not a scalar */
  
  if ( (mxGetM(TAB_PTR) == 1) && (mxGetN(TAB_PTR) == 1) )
    mexErrMsgTxt("The table cannot be a scalar");

  
  /* rows of table should equal length y */

  if (mxGetM(TAB_PTR) != LENGTH_Y )
     mexErrMsgTxt("rows of table should equal length of Y");

  /* columns of table should equal length(x)*length(z) */

  if ( mxGetN(TAB_PTR) != (LENGTH_X*LENGTH_Z) )
     mexErrMsgTxt("columns of table should equal length(x)*length(z)");

 #endif  /* end of error checking */

  /* intialize OLD_INDICES to 0 */

  SET_OLD_INDEX( 0,0 );
  SET_OLD_INDEX( 1,0 );
  SET_OLD_INDEX( 2,0 );

  /* initialize OLD_REF values to be < the input at time 0 to ensure correct
   *   search direction.
   */

  SET_OLD_REF(0, -INF );
  SET_OLD_REF(1, -INF );
  SET_OLD_REF(2, -INF );

}  /* end mdlInitializeConditions */


static void mdlOutputs(double *y, double *x, double *u, SimStruct *S, int tid)
{
  
  int LowIndices[3];      /* store lower of bracketing #'s for x, y and z vects */
  int VectorLengths[3];         
					   
  int i;                                 
  int SearchDir;

  int    nTabEntry;

  double ytemp[2];
  double *LevelPtr;

  double *vectors[3];           /* ptrs to x, y and z vectors */


  /* build array of pointers to X, Y & Z vectors */
  vectors[0] = X_VEC;
  vectors[1] = Y_VEC;
  vectors[2] = Z_VEC;
  
  
  /* determine vector lengths */

  VectorLengths[0] = mxGetM( X_VEC_PTR ) * mxGetN( X_VEC_PTR );
  VectorLengths[1] = mxGetM( Y_VEC_PTR ) * mxGetN( Y_VEC_PTR );
  VectorLengths[2] = mxGetM( Z_VEC_PTR ) * mxGetN( Z_VEC_PTR );

   
  
  /* number of elements in each 2-D table */

  nTabEntry = VectorLengths[0] * VectorLengths[1];

  
  /* The lower of the bracketing indices for the X, Y and Z vectors */
  
  for (i=0; i<3; i++ ){
    
    /* determine search direction by comparing old & new ref values */

    SearchDir = -1;
    
    if ( u[i] >= OLD_REF(i) ) SearchDir = 1;
 
    LowIndices[i] = GetIndex( vectors[i], VectorLengths[i], u[i], SearchDir, S, i );
   

    SET_OLD_REF( i, u[i] );
	 
  }



  /* LevelPtr points to the 1st element of lower of the 2 bracketing 2-D tables */

  LevelPtr = TAB + ( nTabEntry * LowIndices[2] );
  
  /* 
   * Build a 1-D table problem by doing a 2-D table look-up on bracketing 2-D planes  
   * The result is the ytemp array                                                    
   */
  
  ytemp[0] = Table2(X_VEC, Y_VEC, LowIndices[0], LowIndices[1], X_REF_IN, Y_REF_IN,
						  LevelPtr, TAB_PTR);

  ytemp[1] = Table2(X_VEC, Y_VEC, LowIndices[0], LowIndices[1], X_REF_IN, Y_REF_IN,
				    LevelPtr + nTabEntry, TAB_PTR);

  
  /* Do a 1-D table look-up of ytemp based on the z values */
  
  y[0] = Table1( Z_VEC + LowIndices[2], ytemp, Z_REF_IN, 0 );

}  /* end mdlOutputs */


static void mdlUpdate(double *x, double *u, SimStruct *S, int tid)
{
    return;
}


static void mdlDerivatives(double *dx, double *x, double *u, SimStruct *S, int tid)
{
    return;
}


static void mdlTerminate(SimStruct *S)
{
    return;
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
