/* 
 * STVMGAIN   Time-varying matrix gain S-function MEX-file. This block 
 *  implements a time-varying matrix gain. The block accepts two inputs of   
 *  variable width. The width of the system input must match the number 
 *  of columns of the initial gain matrix. The second input, the gain 
 *  is a vector which is resized to a matrix the same size as the  
 *  initial gain K. Syntax: [sys,x0,str,tid] = stvmgain(t,x,u,flag,K,Ts) 
 *  Gary Levenson 5/25/94 
 */ 

/* 
 * The following #define is used to specify the name of this S-Function. 
 */ 

#define S_FUNCTION_NAME stvmgain 

#include <stdio.h>    /* needed for declaration of sprintf */ 

#ifdef MATLAB_MEX_FILE 
#include "mex.h"      /* needed for declaration of mexErrMsgTxt */ 
#endif 

/* 
 * need to include simstruc.h for the definition of the SimStruct and 
 * its associated macro definitions. 
 */ 

#include "simstruc.h" 

/* 
 * Defines for easy access to the matrix that are passed in 
 */ 

#define K                       ssGetArg(S,0) 
#define PARAM_SAMPLE_TIME       ssGetArg(S,1) 

#define NUM_ARGS                2 
#define GetSize(mat)            (mxGetM(mat) * mxGetN(mat))   /* size of gain matrix */


#define NOUTPUTS                mxGetM(K)   /* columns of K */
#define NINPUTS                 mxGetN(K)   /* rows of K */

/* * mdlInitializeSizes - initialize the sizes array 
 * 
 * The sizes array is used by SIMULINK to determine the S-function block's 
 * characteristics (number of inputs, outputs, states, etc.). 
 */ 

static void mdlInitializeSizes(SimStruct *S) 
{ 
  int matrix_size; 
  int numInputs; 
  
  /* 
   * Set-up size information, only do this if there is 1 input arg  
   */ 
  
  matrix_size = GetSize(K);               /* dimensions of input matrix */
  
  /*  
   * # of columns of u must match number of rows in K, 
   * so input width is columns of u (NOUTPUTS) + matrix_size  
   */ 
  
  numInputs = matrix_size + NINPUTS;      
  
  if (ssGetNumArgs(S) == NUM_ARGS) { 
    ssSetNumContStates(    S, 0);          /* number of continuous states */ 
    ssSetNumDiscStates(    S, 1);          /*  
                                            *  number of discrete states 
                                            *  needed to force update function
                                            *  since this function has no states
                                            *  and direct feedthrough
                                            */
    ssSetNumInputs(        S, numInputs);  /* number of inputs */ 
    ssSetNumOutputs(       S, NOUTPUTS);   /* number of outputs */ 
    ssSetDirectFeedThrough(S, 1);          /* direct feedthrough flag */ 
    ssSetNumSampleTimes(   S, 1);          /* number of sample times */ 
    ssSetNumInputArgs(     S, NUM_ARGS);   /* number of input arguments */ 
    ssSetNumRWork(         S, matrix_size);/* number of real work vector elements */ 
    ssSetNumIWork(         S, 0);          /* number of integer work vector elements */ 
    ssSetNumPWork(         S, 0);          /* number of pointer work vector elements */ 
  } else { 
#ifdef MATLAB_MEX_FILE 
    char err_msg[256]; 
    sprintf(err_msg,
	    "Wrong number of input arguments passed to S-function MEX-file.\n"
	    "%d input arguments were passed in when expecting %d input arguments.\n", ssGetNumArgs(S) + 4, NUM_ARGS + 4); 
    mexErrMsgTxt(err_msg); 
#endif 
    
  }     
} 

/* 
 * mdlInitializeSampleTimes - initialize the sample times array 
 * 
 * This function is used to specify the sample time(s) for your S-function. 
 * If your S-function is continuous, you must specify a sample time of 0.0. 
 * Sample times must be registered in ascending order. 
 */ 

static void mdlInitializeSampleTimes(SimStruct *S) 
{ 
  ssSetSampleTimeEvent(S, 0, mxGetPr(PARAM_SAMPLE_TIME)[0]); 
  ssSetOffsetTimeEvent(S, 0, 0.0); 
} 

/* 
 * mdlInitializeConditions - initialize the states 
 * 
 * In this function, you should initialize the continuous and discrete 
 * states for your S-function block.  The initial states are placed 
 * in the x0 variable.  You can also perform any other initialization 
 * activities that your S-function may require. 
 */ 

static void mdlInitializeConditions(double *x0, SimStruct *S) 
     
{ 
  double *gainValues = ssGetRWork(S);
  
  /*  store initial gain matrix values */
  
  gainValues = mxGetPr(K); 
} 

/* 
 * mdlOutputs - compute the outputs 
 * 
 * In this function, you compute the outputs of your S-function 
 * block.  The outputs are placed in the y variable. 
 */ 

static void mdlOutputs(double *y, double *x, double *u, SimStruct *S, int tid) 
{ 
  int i,j;  
  double accum; 
  double *kpr; 
  
  /* 
   * Matrix Multiply: y = K * u 
   */ 
  
  kpr = ssGetRWork(S);     
  
  for (i = 0; i < NOUTPUTS; i++) { 
    accum = 0.0; 
    
    /* K * u */ 
    for (j = 0; j < NINPUTS; j++) 
      accum += kpr[i + NOUTPUTS*j] * u[j]; 
    
    y[i] = accum; 
  } 
} 

/* 
 * mdlUpdate - perform action at major integration time step 
 * 
 * This function is called once for every major integration time step. 
 * Discrete states are typically updated here, but this function is useful 
 * for performing any tasks that should only take place once per integration 
 * step. 
 */ 

static void mdlUpdate(double *x, double *u, SimStruct *S, int tid) 
{ 
  int matrix_size = GetSize(K); 
  int i; 
  double *gainValues = ssGetRWork(S);         /* pointer to new gain values */ 
  
  /* sample hit event 1, update the gain matrix */
  
  /*
   * ssIsSampleHitEvent(S, 0, tid) for continuous case
   * ssGetSampleTimeEvent(S, 0) == 0.0  for discrete block with discrete driving block 
   * ssGetSampleTimeEvent(S, 0) == -1.  should stay as a workaround to the bug we just gecked
   */
  
  if (ssIsSampleHitEvent(S, 0, tid) ||                /* if continuous block, update is okay */
      (ssGetSampleTimeEvent(S, 0) == 0.0) ||          /* if discrete, but driven by discrete, also okay */
      (ssGetSampleTimeEvent(S, 0) == -1.0)) {         /* if discrete, but driven by continuous, this is
							 the workaround to the bug with updating        */
    
    int allZero; 
    
    
    /* the first gain value is after the system inputs */ 
    
    u += NINPUTS; 
    
    
    /* 
     *  check for zeros on the input, it could be unconnected inputs 
     *  get the next gain matrix 
     */
    
    allZero = 1; 
    for (i=0; (i < matrix_size) && allZero; i++) 
      allZero &= u[i] == 0.0;       
    
    /*
     * if the inputs are not all zero, update the gain matrix
     */
    
    if (!allZero) {                 
      for (i = 0; i < matrix_size; i++) 
	*gainValues++ = *u++; 
    } 
    
  }
}                      

/* 
 * mdlDerivatives - compute the derivatives 
 * 
 * In this function, you compute the S-function block's derivatives. 
 * The derivatives are placed in the dx variable. 
 */ 

static void mdlDerivatives(double *dx, double *x, double *u, SimStruct *S, int tid) 
{ 
  
} 

/* 
 * mdlTerminate - called when the simulation is terminated. 
 * 
 * In this function, you should perform any actions that are necessary 
 * at the termination of a simulation.  For example, if memory was allocated 
 * in mdlInitializeConditions, this is the place to free it. 
 */ 

static void mdlTerminate(SimStruct *S) 
{ 
} 

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */ 
#include "simulink.c"      /* MEX-file interface mechanism */ 
#else 
#include "cg_sfun.h"       /* Code generation registration function */ 
#endif          
