function typarams(sys,outfile,overwrite);
% TYPARAMS: Read Simulink file, extract nontrivial dialog box entries.
%   The information in the dialog boxes of simulink blocks is usually
%   not visible when printing the block diagrams. The output of TYPARAMS
%   complements the block diagrams to form a complete paper documentation
%   of a model.
%
%   TYPARAMS('SYS','OUTFILE','OVERWRITE') opens SYS.m and writes the 
%   block names, parameter names and values for all parameters NOT in
%   the TYPARAMS_IGNORELIST (defined inside the TYPARAMS function) 
%   to OUTFILE.
%
%   If OUTFILE is omitted, the output goes to the command window.
%
%   If OUTFILE already exists, and 
%   OVERWRITE = 'ask' (default)  the user is asked, 
%   OVERWRITE = 'always'         it is overwritten without asking
%   OVERWRITE = 'never'          execution continues without writing.
%   (the last two are useful when calling TYPARAMS from a script)
%
%   Ought to work on unix, Mac and PC, not tested only on PC:
%   Change the 'setup text variables' part if necessary.
%
%   Requires the file typaramX.m
%
%   Set the parameter  use_tabs_flag  inside typaramX to
%   1 - If the parameters and values in the output should be tab-separated
%   0 - If blanks should be filled in to align the output text

%
% Name: typarams.m
% Date: 26.09.94
%
%  Markus A. Mueller, Sept. 26, 1994
%  Copyright (c) 1994 by Markus A. Mueller
%  ABB Corporate Research Center, CH-5405 Baden/Switzerland
%
%       Permission is granted to modify and re-distribute this
%       code in any manner as long as this notice is preserved.
%       Not for use in commercial products.
%       All standard disclaimers apply.
%
% Version history:
% Date      Change
% 06.11.95  Adapted to PC and Macintosh (CR/LF, dirsep, itemsep, file exist)
%           No test facilities available for PC - good luck

% define these globals to simplify call to typaramX
% instead of putting them in the call as arguments
global TYPARAMS_OUTFID
global TYPARAMS_IGNORELIST
global TYPARAMS_NL

% setup text variables according to computer system

if isunix
  nl = '\n';
  dirsep = '/';
else
   c = computer;
   if strcmp(c(1:2),'PC') 
     nl = '\n\r';
     dirsep = '\';
   elseif strcmp(c(1:3),'MAC') 
     nl = '\n';
     dirsep = ':';
   else
	 disp('Your operating system is neither unix, Mac, nor PC');
	 disp('change the code inside typarams.m to add correct');
	 disp('separator characters for directories and lines');
	 disp('---> Aborted typarams');
	 return
   end
end

% open input file

[infid,msg] = fopen([sys '.m']);
if infid == -1
  disp(['error opening file ''' sys '.m'' for input'])
  disp(msg)
  return
end

% 3rd argument default 
if nargin < 3
  overwrite = 'ask';
end

% output file specification and existence check

if nargin < 2
  outfid = 1;                       % output to cmd window
  outfile = 'command window';
elseif strcmp(overwrite,'always') 	% it's neither 'ask' or 'never'
  outfid=fopen(outfile,'wt');       % always open given file
else 
  % if findstr(outfile,ls) 		    % outfile exists in local dir
  if exist([pwd,dirsep,outfile])==2 	% outfile exists in local dir
    if strcmp(overwrite,'ask')          % and asking is specified
	  instr = input(['File ' outfile ' exists, overwrite? [no] '],'s');
	  if instr == []
	    instr = 'n';
	  end
	  if any(instr(1) == 'jyJY')      % ok to overwrite
	    outfid=fopen(outfile,'wt'); 	% open given file, overwriting
	  else                            % get out, due to 'ask'
	    disp(['Aborted typarams for ' sys ' with ' outfile ...
		      ': already exists']); 	
	    fclose(infid);
	    return
	  end
    else                          % file exists, do not ask, due to 'never' 
	  disp(['Aborted typarams for ' sys ' with ' outfile ...
	        ': already exists']); 	
	  fclose(infid);
	  return
    end
  else                            % file does not exist yet
    outfid=fopen(outfile,'wt');   % open new file
  end
end

% copy the outfid to the global variable with a longer name,
% to be used inside typaramX
TYPARAMS_OUTFID = outfid;
TYPARAMS_NL     = nl;

% define block parameters to be ignored

TYPARAMS_IGNORELIST = ...
             ['position orientation Window hide name Location '... % general
			  'Drop Shadow Mask Display '...    % more general
              'inputs Port mat-name '...     % add & port blocks
			  'Vgain Hgain Vmax Hmax '...    % scope blocks
			  'Mask Help '];     % masks

% write header info to output file

underline = ones(size(sys))*'=';
time = clock;
fprintf(outfid,['Parameters of simulink system    %s' nl],sys);
fprintf(outfid,['                                 %s' nl nl],underline);
fprintf(outfid,['Name of this file: %s' nl],   outfile);
fprintf(outfid,['Creation date    : %s, %sh' nl nl],date,int2str(time(4:5)));
fprintf(outfid,['Ignoring Parameters:' nl '%s' nl nl],   TYPARAMS_IGNORELIST);

% loop through input file linewise

count = 0;
prevline = [];

while 1
  line = fgetl(infid);
  llen = size(line,2);
  if ~isstr(line), break, end     % end of file
  if llen>=3
    if line(llen-2:llen) == '...'
      %  save line for concatenation with next if it ends with '...'
      prevline = [prevline,line(1:llen-3)];
    else							    
	  % process line
      line = [prevline,line];
	  spos = findstr('set_param',line);
      if spos
	    % it's a call to set_param, replace the string 'set_param'
		% by 'typaramX' and eval it
	    line(spos:spos+7) = 'typaramX';
		line(spos+8)=[];
	    %disp(line)                          % for debugging only
		eval(line,...
		'fprintf(outfid,[nl ''**** Error evaluating'' nl ''  %s'' nl],line);')
	  end
      prevline = [];
    end
  end
  count=count + 1;		
end

fprintf(outfid,[ nl '--- End of file: %s ---' nl],   outfile);

fclose(infid);
if nargin >= 2
  fclose(outfid);
end
%sound(sin(1:10000))


