% demo.m - demo script for multivariate probability functions
%
% Author: Claude D. Bilbo
%

echo on
% Mean vector and covariance matrix for a lognormal distribution.

mln = [3 5 4]'
cln = [4 7 5;7 16 11;5 11 9]

pause % strike any key

% The log of that lognormal distribution has a mean and
% covariance given by:

[mn,cn] = lgn2norm(mln,cln)

% Or use NORM2LGN to convert mn and cn to mln and cln.

pause % strike any key

% LGN2NORM is used by RANDLN to generate normal deviates and
% then take the exponential of the result.  After converting
% the statistics of the lognormal distribution, you can use 
% any multivariate gaussian random number algorithm desired.

randn('seed',0), log(randln([3,3],mln,cln))
randn('seed',0), randn2([3,3],mn,cn)

pause % strike any key

% Generate some lognormal random deviates
mln'
cln
rln = randln([3,20000],mln,cln);
mean(rln')
cov(rln')

pause % strike any key
clear rln

% Plotting a bivariate lognormal distribution using DLN...
xlow = [0 0]';
xhigh = [5 8]';
n_pts = 50;
x = zeros(2,n_pts);
x(1,:) = xlow(1):(xhigh(1)-xlow(1))/(n_pts-1):xhigh(1);
x(2,:) = xlow(2):(xhigh(2)-xlow(2))/(n_pts-1):xhigh(2);

y = zeros(n_pts);
for i = 1:n_pts
  y(i,:)=dln([x(1,i)*ones(1,n_pts);x(2,:)],mln(1:2),cln(1:2,1:2));
end

figure,surf(x(2,:),x(1,:),y)
set(gca,'view',[100 50])
title('a lognormal surface')

pause % strike any key

% Plotting the univariate lognormal distributions

xhigh = 2*xhigh;
x(1,:) = xlow(1):(xhigh(1)-xlow(1))/(n_pts-1):xhigh(1);
x(2,:) = xlow(2):(xhigh(2)-xlow(2))/(n_pts-1):xhigh(2);

figure
plot(x(1,:),dln(x(1,:),mln(1),cln(1,1)),x(2,:),dln(x(2,:),mln(2),cln(2,2)))
title('Two univariate lognormal distributions')

pause % strike any key

% Plotting another bivariate lognormal distribution...
mln = [0.6767;46.9];
cln = [ (0.406)^2 0 ;0 (16.884)^2  ];
xlow = [0 0]';
xhigh = mln+3*sqrt(diag(cln));

x = zeros(2,n_pts);
x(1,:) = xlow(1):(xhigh(1)-xlow(1))/(n_pts-1):xhigh(1);
x(2,:) = xlow(2):(xhigh(2)-xlow(2))/(n_pts-1):xhigh(2);

y = zeros(n_pts);
for i = 1:n_pts
  y(i,:)=dln([x(1,i)*ones(1,n_pts);x(2,:)],mln,cln);
end

figure,surf(x(2,:),x(1,:),y)
set(gca,'view',[100 50])
title('another lognormal surface')

pause % strike any key

load data1  % load gaussian sum estimate

y2 = zeros(n_pts);

% This part is loaded from a file, not calculated.
% The calculation takes a while.  Set skip=0 to do
% the actual number crunching.

skip=1;

if skip
  load data2
else
  for i = 1:n_pts
    y2(i,:)=dgaussum([x(1,i)*ones(1,n_pts);x(2,:)],gsum_weights, ...
                     gsum_means,gsum_covs);
  end
end

% Plotting a gaussian sum approximation to the previous distribution.
% Each term in the sum was constrained to have the same covariance.

figure,surf(x(2,:),x(1,:),y2)
set(gca,'view',[100 50])
title('100 point gaussian sum approximation to previous surface')

pause % strike any key

% Plotting the error of the gaussian sum approximation
% to the lognormal distribution.

figure,surf(x(2,:),x(1,:),(y2-y))
set(gca,'view',[100 50])
title('error surface for approximation')

pause % strike any key

% Plotting the percent difference between the gaussian 
% sum approximation and the lognormal distribution.

figure,surf(x(2,:),x(1,:),abs(y-y2)./(y2+y)*50)
set(gca,'view',[100 50])
title('and percent difference')

pause % strike any key

% Plotting the cdf's of the marginal distributions, 1st dimension

figure
plot(x(1,:),pln(x(1,:),mln(1),cln(1,1)),x(1,:), ...
     pgaussum(x(1,:),gsum_weights, ...
     gsum_means(1:2:prod(size(gsum_means))), ...
     gsum_covs(1:2:size(gsum_covs,1),1)))
title('cdf''s of marginal distributions, 1st dimension')
legend('lognormal dist.', 'gauss. sum approx.',0)

pause % strike any key

mn = [3 5 ]'
cn = [4 7 ;7 16]

% Plotting a bivariate gaussian distribution using DGAUSS...
xlow = mn-3*sqrt(diag(cn));
xhigh = mn+3*sqrt(diag(cn));
n_pts = 50;
x2 = zeros(2,n_pts);
x2(1,:) = xlow(1):(xhigh(1)-xlow(1))/(n_pts-1):xhigh(1);
x2(2,:) = xlow(2):(xhigh(2)-xlow(2))/(n_pts-1):xhigh(2);

y3 = zeros(n_pts);
for i = 1:n_pts
  y3(i,:)=dgauss([x2(1,i)*ones(1,n_pts);x2(2,:)],mn,cn);
end

figure,surf(x2(2,:),x2(1,:),y3)
set(gca,'view',[100 50])
title('a gaussian surface')

pause % strike any key

% Plotting the cdf's of the marginal distributions, 1st dimension

figure
plot(x2(1,:),pgauss(x2(1,:),mn(1),cn(1,1)),x2(2,:),pgauss(x2(2,:),mn(2),cn(2,2)))
title('cdf''s of marginal distributions, gaussian')
legend('1st dimension', '2nd dimension',0)

