function d=dgauss(x,m,c)
%DGAUSS	Multivariate gaussian probability density function (pdf).
%	DGAUSS(X,MEAN,COV) returns the jointly normal pdf
%	at the vector X, for the gaussian density with mean vector
%	MEAN and covariance matrix COV. X may also be a matrix of 
%	column vectors, with the pdfs returned as a row vector.
%
%	See also DGAUSSUM, DLN

% File:	dgauss.m
%
% Copy:	Copyright(c) 1994, 1995 Claude D. Bilbo
%	Copy freely, but document any changes.
%
% Email:cbilbo@mindspring.com
%	bilboc02@eng.uab.edu
%
% Snail:Department of Electrical and Computer Engineering
%	University of Alabama at Birmingham
%	1150 10th Avenue South
%	UAB Station, Birmingham, AL 35294
%
% Desc:	Function to return the density at a point of a 
%	jointly gaussian distribution.
%
% Use:	d = dgauss(x,m,c)
%	  where,
%	x = random vector(s) at which to find density
%       m = mean vector of the distribution
%	c = covariance matrix of the distribution
%
%    or, d = dgauss(x,c)
%         where,
%	x = random vector(s) at which to find density
%	c = covariance matrix of the distribution
%         and,
%       the mean of the distribution is assumed to be zero
%
%    or, d = dgauss(x)
%         where,
%	x = random vector(s) at which to find density
%         and,
%       the mean of the distribution is assumed to be zero
%         and,
%       the covariance matrix of the distribution is assumed to
%       be the identity matrix
%
% Refs:	Athanasios Papoulis,"Probability, Random Variables, and Stochastic
%	Processes," Third Edition, McGraw-Hill, 1991, p. 197.
%
%       T.W. Anderson,"An Introduction to Multivariate Statistical
%       Analysis," pg. 17, John Wiley & Sons, New York, 1958.
%
% Author: Claude D. Bilbo
%

% If called with two arguments, or one
if nargin == 2
  c=m;
  m=zeros(size(x(:,1)));
elseif nargin == 1
  m=zeros(size(x(:,1)));
  c=eye(size(x,1));
end

d = zeros(1,size(x,2));

% Error Checking
if size(m,2)~=1        % Make sure m is a column vector
  if size(m,1)==1      
    m=m';
  else
    error('mean must be a vector');
  end
end

if size(c,1) ~= size(c,2)
  error('Covariance matrix is not square')
elseif size(x,1) ~= length(c)
  error('Size of random vector and covariance matrix do not match')
end

dbz = find( diag(c) == 0 );     % Indices of zero variances
dbnz = find( diag(c) ~= 0 );    % Indices of non-zero variances

c = c(dbnz,dbnz);               % Trim off non-zero variance terms
detc = det(c);

if detc == 0
  error('Covariance matrix is singular')
elseif detc < 0
  error('Determinant of covariance matrix is negative')
elseif ~finite(detc)
  error('Determinant of covariance matrix not finite')
end

invc = -0.5*inv(c);                    
k = 1/sqrt((2*pi)^length(dbnz)*detc);
x = x-m*ones(1,size(x,2));     

% Calculate densities
for i=1:size(x,2)
  if any( x(dbz,i) ~= zeros(size(dbz)) )
    d(i) = 0;
  elseif ~isempty(dbnz)
    d(i) = k*exp(x(dbnz,i)'*invc*x(dbnz,i));
  else
    d(i) = Inf;
  end
end

return
