function d=dln(x,m,c)
%DLN	Multivariate log-normal probability density function (pdf).
%	DLN(X,MEAN,COV) returns the jointly log-normal pdf
%	at the vector X, for the log-normal density with 
%	mean vector MEAN and covariance matrix COV.  X may 
%	also be a matrix of column vectors, with the densities 
%	returned as a row vector.
%
%	See also DGAUSS, DGAUSSUM

% File:	dln.m
%
% Copy:	Copyright(c) 1994,1995 Claude D. Bilbo
%	Copy freely, but document any changes.
%
% Email:cbilbo@mindspring.com
%	bilboc02@eng.uab.edu
%
% Snail:Department of Electrical and Computer Engineering
%	University of Alabama at Birmingham
%	1150 10th Avenue South
%	UAB Station, Birmingham, AL 35294
%
% Desc:	Function to return the density at a point of a 
%	multivariate lognormal distribution.
%
% Use:	d = dln(x,m,c)
%	  where,
%	x = random vector(s) at which to find density
%	m = mean vector of multivariate lognormal distribution
%	c = covariance matrix of multivariate lognormal distribution
%
% Refs:	Athanasios Papoulis,"Probability, Random Variables, and Stochastic
%	Processes," Third Edition, McGraw-Hill, 1991, p. 97,183,197.
%
%       J. Aitchison, and J.A.C. Brown,"The Lognormal Distribution, with 
%       special reference to its uses in economics," Cambridge Press, 
%       London, 1957, p. 11-12.
%
% Author: Claude D. Bilbo
%

d = zeros(1,size(x,2));

% Error Checking

if size(m,2)~=1        % Make sure m is a column vector
  if size(m,1)==1      
    m=m';
  else
    error('mean must be a vector');
  end
end

if size(c,1) ~= size(c,2)
  error('Covariance matrix is not square')
elseif size(x,1) ~= length(c)
  error('Size of random vector and covariance matrix do not match')
end

[m,c]=lgn2norm(m,c);            % Transform mean and variance from
                                % lognormal to normal

dbz = find( diag(c) == 0 );     % Indices of zero variances
dbnz = find( diag(c) ~= 0 );    % Indices of non-zero variances

c = c(dbnz,dbnz);               % Trim off non-zero variance terms
detc = det(c);

if detc == 0
  error('Covariance matrix is singular')
elseif detc < 0
  error('Determinant of covariance matrix is negative')
elseif ~finite(detc)
  error('Determinant of covariance matrix not finite')
end

invc = -0.5*inv(c);
k = 1/sqrt((2*pi)^length(dbnz)*detc);

% Calculate densities
for i=1:size(x,2)
  if all(x(:,i) > 0)
    a = log(x(:,i)) - m;
    if any( a(dbz) ~= zeros(size(dbz)) )
      d(i) = 0;
    elseif ~isempty(dbnz)
      d(i) = k*exp(a(dbnz)'*invc*a(dbnz))/prod(x(dbnz,i));
    else
      d(i) = Inf;
    end
  else
    d(i) = 0;
  end
end

return
