function [Mn,Cn]=lgn2norm(Mln,Cln)
%LGN2NORM Convert mean vector and covariance matrix of a 
%	multivariate lognormal distribution to the mean
%	vector and covariance matrix of the corresponding 
%	multivariate normal distribution.
%	
%	[MN,CN]=LGN2NORM(MLN,CLN) yields the mean MN and
%	covariance matrix CN of the gaussian distribution
%	that corresponds to the lognormal distribution with
%	mean vector MLN and covariance matrix CLN
%
%	See also NORM2LGN

% File:	lgn2norm.m
%
% Copy:	Copyright(c) 1995, Claude D. Bilbo
%	Copy freely, but document any changes.
%
% Email:cbilbo@mindspring.com
%	bilboc02@eng.uab.edu
%
% Snail:Department of Electrical and Computer Engineering
%	University of Alabama at Birmingham
%	1150 10th Avenue South
%	UAB Station, Birmingham, AL 35294
%
% Desc:	Convert mean vector and covariance matrix of a 
%	multivariate lognormal distribution to the 
%       mean vector and covariance matrix of the 
%       corresponding multivariate normal distribution.
%
% Use:	[Mn,Cn]=lgn2norm(Mln,Cln)
%	  where,
%	Mn  = vector of means of multivariate normal distribution
%	Cn  = covariance matrix of the multivariate normal distribution
%	Mln = vector of means of multivariate lognormal distribution
%	Cln = covariance matrix of the multivariate lognormal distribution
%
% Refs:	Athanasios Papoulis,"Probability, Random Variables, and Stochastic
%	Processes," Third Edition, McGraw-Hill, 1991, pp. 152,190,197.
%
%       J. Aitchison, and J.A.C. Brown,"The Lognormal Distribution, with 
%       special reference to its uses in economics," Cambridge Press, 
%       London, 1957, p. 12.
% 
% Author: Claude D. Bilbo
%

% Error Checking

if size(Mln,2)~=1        % Make sure Mln is a column vector
  if size(Mln,1)==1      
    Mln=Mln';
  else
    error('First argument must be a vector');
  end
end

if size(Cln,1) ~= size(Cln,2)
  error('Covariance matrix is not square')
end

if length(Mln) ~= length(Cln)
  error('Size of mean vector and covariance matrix do not match')
end

% Calculate normal covariance matrix and mean vector

Cn = log( Cln ./ (Mln*Mln') + 1 );

Mn = log( Mln .* exp( -0.5 * diag(Cn) ) );

return
