function [Mln,Cln]=norm2lgn(Mn,Cn)
%NORM2LGN Convert mean vector and covariance matrix of a 
%	multivariate normal distribution to the mean vector
%	and covariance matrix of the corresponding 
%	multivariate lognormal distribution.
%	
%	[MLN,CLN]=NORM2LGN(MN,CN) yields the mean MLN and
%	covariance matrix CLN of the lognormal distribution
%	that corresponds to the gaussian distribution with
%	mean vector MN and covariance matrix CN
%
%	See also LGN2NORM

% File:	norm2lgn.m
%
% Copy:	Copyright(c) 1995, Claude D. Bilbo
%	Copy freely, but document any changes.
%
% Email:cbilbo@mindspring.com
%	bilboc02@eng.uab.edu
%
% Snail:Department of Electrical and Computer Engineering
%	University of Alabama at Birmingham
%	1150 10th Avenue South
%	UAB Station, Birmingham, AL 35294
%
% Desc:	Convert mean vector and covariance matrix of a 
%	multivariate normal distribution to the 
%	mean vector and covariance matrix of the 
%	corresponding multivariate lognormal distribution.
%
% Use:	[Mln,Cln]=lgn2norm(Mn,Cn)
%	  where,
%	Mn  = vector of means of multivariate normal distribution
%	Cn  = covariance matrix of the multivariate normal distribution
%	Mln = vector of means of multivariate lognormal distribution
%	Cln = covariance matrix of the multivariate lognormal distribution
%
% Refs:	Athanasios Papoulis,"Probability, Random Variables, and Stochastic
%	Processes," Third Edition, McGraw-Hill, 1991, pp. 152,190,197.
%
%       J. Aitchison, and J.A.C. Brown,"The Lognormal Distribution, with 
%       special reference to its uses in economics," Cambridge Press, 
%       London, 1957, p. 12.
% 
% Author: Claude D. Bilbo
%

% Error Checking

if size(Mn,2)~=1        % Make sure Mn is a column vector
  if size(Mn,1)==1      
    Mn=Mn';
  else
    error('First argument must be a vector');
  end
end

if size(Cn,1) ~= size(Cn,2)
  error('Covariance matrix is not square')
end

if length(Mn) ~= length(Cn)
  error('Size of mean vector and covariance matrix do not match')
end

% Calculate lognormal covariance matrix and mean vector

Mln = exp( Mn + 0.5*diag(Cn) );

Cln = (exp(Cn)-1) .* (Mln*Mln');

return

