function p = pgauss(x,m,v)
%PGAUSS	Distribution function of a univariate gaussian
%	probability density.
%	
%	P = PGAUSS(X,M,V) yields the probability that X
%	lies in the range (-Inf,X], for a gaussian density
%       with mean M, and variance V.
%
%	See also PLN, PGAUSSUM 

% File:	pgauss.m
%
% Copy:	Copyright(c) 1995 Claude D. Bilbo
%	Copy freely, but document any changes.
%
% Email:cbilbo@mindspring.com
%	bilboc02@eng.uab.edu
%
% Snail:Department of Electrical and Computer Engineering
%	University of Alabama at Birmingham
%	1150 10th Avenue South
%	UAB Station, Birmingham, AL 35294
%
% Desc:	Function to return the distribution function of
%       a univariate gaussian probability density.
%	
% Use:	p = pgauss(x,m,v)
%	  where,
%	x = random variable at which to find distribution fcn.
%       m = mean of the gaussian distribution
%	v = variance of the distribution
%
%
% Refs:	Athanasios Papoulis,"Probability, Random Variables, and Stochastic
%	Processes," Second Edition, pp. 47-48, 73-74, McGraw-Hill,
%       New York, 1984.
%
%       Murray R. Spiegel,"Schaum's Outline Series: Mathematical
%       Handbook of Formulas and Tables," pp. 183, 257, McGraw-Hill, 
%       New York, 1968.
%       
% Author: Claude D. Bilbo
%

x_m = x-m;

if any(any(v==0))                 % Check for delta fcns
  if prod(size(v))==1             % scalar variance case
    p = zeros(size(x_m));
    ind = find(x_m>=0);
    p(ind) = ones(size(ind));
  else                            % variance nonscalar case
    indnz = find(v~=0);
    p = zeros(size(v));
    if prod(size(x_m))==1         % with scalar x and m
      if ~isempty(indnz)
        p(indnz) = (erf(x_m./sqrt(2*v(indnz)))+1)*0.5;
      end
      indz = find(v==0);
      if x_m>=0
        p(indz) = ones(size(indz));
      end
    else                          % with x or m nonscalar
      if ~isempty(indnz)
        p(indnz) = (erf(x_m(indnz)./sqrt(2*v(indnz)))+1)*0.5;
      end
      indz = find((v==0)&(x_m>=0));
      p(indz) = ones(size(indz));
    end
  end
else                              % If no delta fcns
  p=(erf(x_m./sqrt(2*v)) + 1)*0.5;
end
