function p = pln(x,m,v)

%PLN	Distribution function of a univariate lognormal
%	probability density.
%	
%	P = PLN(X,M,V) yields the probability that X
%	lies in the range (-Inf,X], for a lognormal density
%       with mean M, and variance V.
%
%	See also PGAUSS, PGAUSSUM 

% File:	pln.m
%
% Copy:	Copyright(c) 1995 Claude D. Bilbo
%	Copy freely, but document any changes.
%
% Email:cbilbo@mindspring.com
%	bilboc02@eng.uab.edu
%
% Snail:Department of Electrical and Computer Engineering
%	University of Alabama at Birmingham
%	1150 10th Avenue South
%	UAB Station, Birmingham, AL 35294
%
% Desc: Function to return the distribution function of
%       a univariate lognormal probability density.
%	
% Use:	p = pln(x,m,v)
%	  where,
%	x = random variable at which to find distribution fcn.
%       m = mean of the gaussian distribution
%	v = variance of the distribution
%
%
% Refs:	T.T. Soong, "Probabilistic Modeling and Analysis in Science
%       and Engineering," pg. 205, John Wiley & Sons, New York.
%
%       Athanasios Papoulis,"Probability, Random Variables, and Stochastic
%       Processes," Second Edition, pp. 47-48, 73-74, McGraw-Hill,
%       New York, 1984.
%
%       Murray R. Spiegel,"Schaum's Outline Series: Mathematical
%       Handbook of Formulas and Tables," pp. 183, 257, McGraw-Hill, 
%       New York, 1968.
%       
% Author: Claude D. Bilbo
%

if (prod(size(m))>1) | (prod(size(v))>1)
  error('mean and variance must be scalar')
end

if (m <= 0)
  error('mean must be greater than zero')
elseif v<0
  error('variance must be >= 0')
end

if v==0
  p = x >= m;
  return
else
  [m,v] = lgn2norm(m,v);
  p=zeros(size(x));
  ind = find(x > 0);   % result is zero if x<=0
  p(ind)=(erf((log(x(ind))-m)./sqrt(2*v)) + 1)*0.5;
  return
end
