function y = randln(dim,mln,cln)
%RANDLN	Multivariate lognormal random number generator.
%	RANDLN(DIM,MEAN,COV) returns a matrix with DIM(1)
%	rows and DIM(2) columns.  Each column of the
%	returned matrix is a lognormal random vector with
%	mean vector MEAN and covariance matrix COV.
%
%	See also RANDN2

% File:	randln.m
%
% Copy:	Copyright(c) 1995, Claude D. Bilbo
%	Copy freely, but document any changes.
%
% Email:cbilbo@mindspring.com
%	bilboc02@eng.uab.edu
%
% Snail:Department of Electrical and Computer Engineering
%	University of Alabama at Birmingham
%	1150 10th Avenue South
%	UAB Station, Birmingham, AL 35294
%
% Desc:	Function to generate multivariate random numbers 
%       with a joint lognormal distribution.
%
% Use:	y = randln(dim,mln,cln)
%	  where,
%	y = multivariate random vector(s), size(y) = dim
%	dim(1)   = length of random vector
%       dim(2)   = number of random vectors to produce
%	mln = mean vector of multivariate lognormal distribution
%	cln = covariance matrix of multivariate lognormal distribution
%
% Refs:	T.W. Anderson,"An Introduction to Multivariate Statistical
%       Analysis," pg. 19, John Wiley & Sons, New York, 1958.
%
%       J. Aitchison, and J.A.C. Brown,"The Lognormal Distribution, with 
%       special reference to its uses in economics," Cambridge Press, 
%       London, 1957, p. 11-12.
%
%       C. Ray Wylie, Louis C. Barrett, "Advanced Engineering
%       Mathematics," pp. 733-735, McGraw-Hill, New York, 1982.
%
% Author: Claude D. Bilbo
%

% CAVEAT: This algorithm works by making a linear transformation
%         on independent N(0,1) variates.  The assumption is that
%         RANDN deviates are uncorrelated with each other.

% convert from lognormal to normal statistics

[mn,cn]=lgn2norm(mln,cln);

% compute eigenvalues and eigenvectors

[v,d] = eig(cn,eye(size(cn)));

% generate N(0,1) independent normal deviates

y=randn(dim(1),dim(2));

% transform to N(0,d)

y = y .* (sqrt(diag(d))*ones(1,dim(2)));

% transform to exp(N(mn,cn))

y = exp((inv(v'))*(y + ((v'*mn)*ones(1,dim(2)))));

% The line directly above was once computed as below.
% The steps may make it more obvious how the alogrithm works.
% Obviously, two more lines could have been removed, also.
%
%c=inv(v');
%mu=v'*mn;
%y = y + (mu*ones(1,dim(2)));
%y=c*y;
%y = exp(y);
