% KS_TOOLS Toolbox
% Version 1.1 5/6/95
% Copyright 1995 by R H Smallwood and University of Sheffield
%
% This toolbox contains m-files for the Kolmorogov-Smirnov
% test in one and two dimensions, together with utility routines
% for testing the implementations.
%
% The one and two dimensional K-S tests are described in:
%
% 1)	J A Peacock (1983) Two dimensional goodness-of-fit
%	testing in astronomy. Monthly Notices of the Royal 
%	Astronomical Society v202 615-627
%
% 2)	G Fasano and A Franceschini (1987) A multi-dimensional
%	version of the Kolmorogov-Smirnov Test. Monthly Notices
%	of the Royal Astronomical Society v225 155-170
%
% 3)	W H Press, S A Teukolsky, W T Vetterling and B P Flannery
%	(1992) Numerical Recipes in FORTRAN: The Art of Scientific
%	Computing. Cambridge University Press
%
% 4)	W T Vetterling, S A Teukolsky, W H Press and B P Flannery
%	(1992) Numerical Recipes Example Book: FORTRAN. Cambridge
%	University Press.
%
% 5)	R H Smallwood (1995) Two dimensional Kolmorogov-Smirnov test
%	for binned data. (to be published)
%
% The main purpose of this toolbox is to provide a two-dimensional
% test for binned data, as described in reference 5. This test is
% my extension of the test described in references 1 and 2. The
% probabilities are the result of Monte-Carlo testing of the routine.
% This is described fully in reference 5, and the necessary programs
% for repeating this are included in the toolbox.
%
% The 'standard' routines, in versions which owe a good deal to
% Press et al, have been included so that comparisons can be made.
% The programs have been written as MATLAB programs and
% are not simply translated from the Fortran programs in Press et al.
%
% The calulation of probabilities in PEARSON requires BETAINC from
% the Statistics Toolbox. This is not essential to the calculation of
% the Kolmorogov-Smirnov probability, and the line could therefore
% be removed if the Statistics Toolbox is not available.
%
% No attempt has been made to optimise the execution times of the
% routines by writing MEX files - in particular, QUADCT, QUADCTB,
% and XORCISE could be made much faster.
%
% I would welcome comments and/or corrections:
%
%	Dr R H Smallwood
%	Department of Medical Physics and Clinical Engineering
%	University of Sheffield
%	Royal Hallamshire Hospital
%	Sheffield
%	S10 2JF
%	UK
%
%	Tel:	+44-(0)114-271-3146
%	Fax:	+44-(0)114-271-3403
%	e-mail:	r.smallwood@sheffield.ac.uk
%
%
% KOLMOROGOV-SMIRNOV TESTS IN ONE AND TWO DIMENSIONS
%
%
% KSTWO.M
%	This is a basic one-dimensional test for the two-sample
%	case. It is included because the one-dimensional case
%	is well-known, and playing with this can give a feel 
%	for the use of K-S tests. 
%
%		function [d,prob] = KSTWO(data1,data2) computes the 
% 		Kolmogorov-Smirnov statistic d and the significance
% 		level prob for the null hypothesis that the data sets 
% 		data1 and data2 are drawn from the same distribution.
% 		Small values of prob show that the cumulative
%		distribution function of data1 is significantly
%		different from that of data2. Adapted from Press,
%		Teukolsky, Vetterling and Flannery, Numerical
%		Recipes in Fortran p619.
%		Uses PROBKS
%
%
% KS2D2S.M
%	This is the two-dimensional test described by Fasano and
%	Franceschini as a modification of Peacock's test. This is
%	again a two-sample test, and the data is in the form of
%	two sets of coordinate pairs.
% 		function [d,prob,rr] = KS2D2S(data1,data2) is a
%		two-dimensional Kolmogorov-Smirnov test on two
%		samples data1 and data2. data1(2,n1) and data2(2,n2)
%		contain the x,y coordinates for two distributions.
%		The K-S statistic d and the significance level prob
%		for the null hypothesis that the data sets data1 and
%		data2 are drawn from the same distribution are 
%		computed. Small values of prob show that the two
%		samples are significantly different. The test is
%		slightly distribution-dependent, so prob is only
%		an estimate. Adapted from Press, Teukolsky,
%		Vetterling and Flannery, Numerical Recipes in 
%		Fortran p643. Generate test data using XKS2D2S.M
%		Uses PROBKS and QUADCT
%
%
% KS2D2SB.M
%	This is a two-dimensional, two sample test for data that
%	has been placed in bins ie it is designed to test the
%	significance of differences between two MATLAB arrays.
%	The significance levels are quite different from those
%	for the previous test (KS2D2S.M), and have been found
%	by Monte-Carlo testing. As discussed in reference 5, the
%	significance values are valid for arrays with 64, 256 and
%	1024 elements, but values for intermediate numbers of 
%	elements can be found by interpolation.  	
%		function [prob,Z] = KS2S2B(data1,data2) is a
%		two-dimensional Kolmogorov-Smirnov test on two
%		samples data1 and data2 where the data is in bins
%		(ie pixelated data). data1(m1,n1) and data2(m2,n2)
%		contain the two distributions. The K-S statistic d
%		and the significance level prob for the null
%		hypothesis that the data sets data1 and data2
%		are drawn from the same distribution are computed.
%		Small values of prob show that the two samples are
%		significantly different. The probability is
%		calculated from curves fitted to the results
%		of Monte-Carlo tests, and is valid for 8x8, 16x16
%		and 32x32 arrays in the range 0.001<=p<=0.20.
%		Use XKS2D2S to generate test data for KS2D2SB.
%		Uses QUADCTB.
%
%
% ROUTINES CALLED BY THE KOLMOROGOV-SMIRNOV TESTS
%
%
% QUADCT.M
%	This routine is the core of KS2D2S, and finds the fraction
%	of the data contained in each of the four quadrants defined
%	by an input coordinate pair.
% 		function [nf] = QUADCT(A,x,y) - given an origin (x,y)
%		in the array of points A, return the normalised
%		fraction of the points in each quadrant round the
%		origin. A(2,n) contains the coordinates of n counts.
%		The output array nf(4) contains the fraction in
%		quadrants taken counterclockwise from top right.
%		Adapted from Press, Teukolsky, Vetterling and
%		Flannery, Numerical Recipes in Fortran p642.
%
%
% QUADCTB.M
%	This routine is the equivalent of QUADCT for data in arrays.
%	Unlike QUADCT, this is called once only, and returns the
%	maximum normalised difference between the distributions.
%		function [D] = QUADCTB(A,B) is a modified version of
%		QUADCT for use with arrays A(m1,n1) and B(m2,n2)
%		containing sA and sB counts rather than from A(2,n)
%		containing the coordinates of n counts. Given an
%		origin (x,y) in the array, return the normalised
%		maximum value of the difference between the 
%		distributions for all x,y.
%
%
% PROBKS.M
%	Calculates the Kolmorogov-Smirnov probability for KSTWO
%	and KS2D2S.
%		function [prob] = PROBKS(Q) calculates for the
%		Kolmogorov-Smirnov statistic D the significance level
%		prob. Adapted from Press, Teukolsky, Vetterling
%		and Flannery, Numerical Recipes in Fortran p620.
% 		Checked using in XPROBKS in Numerical Recipes Example
%		Book p185
%
%
% PEARSON.M
%	Calculates the correlation coefficient for use in KS2D2S.
%		function (r,prob,z) = PEARSON(x,y,mode) computes
%		the correlation coefficient r, Student's t probabilty
%		prob, and Fisher's z for two vectors x and y. Adapted
%		from Press, Teukolsky, Vetterling and Flannery,
%		Numerical Recipes in Fortran p632. Checked with
%		example on p187 of Numerical Recipes Example Book
%
%
% KSSIGNIF.M
%	Calculates the significance level for binned data for any square
%	matrix of size 8x8 to 32x32, using interpolation between the
%	values calculated by the Monte-Carlo testing.
% 		function [prob] = KSSIGNIF(Z,matrix_size,verbose) 
%		calculates the significance level for the Kolmorogov-Smirnov 
%		statistic Z for two binned data sets.
%		Z is the Kolmorogov-Smirnov statistic calculated by KS2D2SB.M
% 		MATRIX_SIZE=[rows,columns] where ROWS=COLUMNS and
%		8<=ROWS<=32.
%		VERBOSE=[0|1] sets out-of-range announcement off|on.
%
%
% ROUTINES FOR GENERATING TEST DATA
%
%
% INITSEED.M
%	Writes NSEED.MAT, which is used by XORCISE.
%		MATLAB uses the same seed for the random number
%		generator on each initialisation of MATLAB ie
%		the sequences of random numbers are identical
%		after each initialisation! To use all the ~2E9
%		random numbers, the seed is stored at the end of
%		each call to XORCISE, and is loaded at the beginning
%		of the next call. The seed is stored in NSEED.MAT.
%
%	INITSEED.M is a test routine, which is not used in XORCISE.M,
%	because the reading and writing of the seed is included
%	within XORCISE.M.
%
%	!!!!	This should also be used in  NKS2D2S.M if the	!!!!
%	!!!!	routine is to be used for Monte-Carlo testing.	!!!!
%
%
% NKS2D2S.M
%	Generates random coordinate pairs to test KS2D2S.
%		function [data1,data2] = nks2d2s(n1,n2,shrink)
%		generates 2-d random samples data1 and data2 to test
%		KS2D2S. Shrink alters the variance of the data.
%		See Numerical Recipes Example Book p193
%
%
% BINS.M
%	Sorts the data generated by NKS2D2S in bins for testing
%	KS2D2SB.
%		function [B1,B2] = BINS(A1,A2,m,n,minx,maxx,...
%		miny,maxy,draw) places the x,y pairs in vectors
%		A1(2,k) and A2(2,k) in arrays of mxn equally spaced
%		bins, and returns the contents of the bins as
%		vectors B1 and B2. The range of the bins is given
%		by minx etc.
%
%
% XKS2D2S.M
%	Same as NKS2D2S with the option of using BINS to place the
%	data into square arrays
%
%
% XORCISE.M
%	Performs Monte-Carlo testing on KS2D2SB.
%		function [Z,prcnt] = XORCISE(n1,n2,xsize,ysize,...
%		repeats,store) generates data for Monte-Carlo testing
%		of the 2 sample 2 dimensional Kolmorogov-Smirnov test
%		KS2D2SB. 2-d random samples data1(2,n1) and
%		data2(2,n2) are generated by XKS2D2S using shrink=1
%		to give identical distributions. The K-S statistic
%		Z=d*sqrt(N) for the null hypothesis that the data
%		sets data1 and data2 are drawn from the same
%		distribution are computed by KS2D2SB. Large values
%		of Z show that the two samples are significantly
%		different. Z is the K-S statistic normalised for
%		sample size. BINS is used to place the x,y pairs
%		in arrays of xsize x ysize equally spaced bins.
%		Z and the percentiles of d are then returned in
%		Z and prcnt.
%		Uses XKS2D2S, BINS, KS2D2SB.
%		The output arguments, and strings giving the input
%		arguments, are saved to a temporary disk file.
%
%%
%
% KS_TOOLS\CONTENTS.M 5/6/95 Rod Smallwood