function [prob] = kssignif(Z,matrix_size,verbose)

% KSSIGNIF(Z,matrix_size,verbose) calculates the significance
% level for the Kolmorogov-Smirnov statistic Z for two binned
% data sets.
% Z is the Kolmorogov-Smirnov statistic calculated by KS2D2SB.M
% MATRIX_SIZE=[rows,columns] where ROWS=COLUMNS and 8<=ROWS<=32.
% VERBOSE=[0|1] sets out-of-range announcement off|on.
%
% Version 1.0 RHS 13/1/95

if nargin<3
	verbose=1;	% default is to give messages
end

% catch out-of range input values of Z
if Z<1.1
	prob=0.2;
	if verbose~=0
		fprintf('Out of range, probability>0.20\n')
	end
	break
elseif Z>2.1
	prob=0.001;
	if verbose~=0
		fprintf('Out of range, probability<0.001\n')
	end
	break
end

% check that matrices are square and within limits
rows=matrix_size(1);columns=matrix_size(2);
if rows~=columns
	error('array must be square')
elseif rows<8 | rows>32
	error('size limits 8x8 to 32x32')
end

% fitted coefficents for 8x8, 16x16, 25x25 and 32x32 arrays
% from Monte Carlo testing. The probability can be calculated
% from ln(p)=aZ^2+bZ+c for 0.001<=p<=0.20.

coeffs=[-2.772  2.014 -0.333
        -2.154  0.476  1.277
        -3.132  4.075 -1.496
        -3.470  4.810 -1.732];

Zi=1.1:0.05:2.1;	% values of Z for which p is estimated

% estimate p values for 8x8, 16x16, 25x25 and 32x32 arrays
p8est=polyval(coeffs(1,:),Zi);
p16est=polyval(coeffs(2,:),Zi);
p25est=polyval(coeffs(3,:),Zi);
p32est=polyval(coeffs(4,:),Zi);

% form a table of p v Z.
% First column is size of array.
% First row is values of Z.
data=[ [0 8 16 25 32]' [Zi
                        exp(p8est)
                        exp(p16est)
                        exp(p25est)
                        exp(p32est)]];

% use table look-up to find probability for given Z and size
prob=table2(data,rows,Z);

% catch p values outside limits
if prob>0.2
	prob=0.2;
	if verbose~=0
		fprintf('Out of range, probability>0.20\n')
	end
elseif prob<0.001
	prob=0.001;
	if verbose~=0
		fprintf('Out of range, probability<0.001\n')
	end
end

