function [Z,prcnt] = xorcise(n1,n2,xsize,ysize,repeats,filename,draw)

% function [Z,prcnt] = XORCISE(n1,n2,xsize,ysize,repeats,filename,draw)
% generates data for Monte-Carlo testing of the 2 sample
% 2 dimensional Kolmorogov-Smirnov test KS2D2SB. 
% 2-d random samples data1(2,n1) and data2(2,n2) are generated
% using shrink=1 to give identical distributions. The K-S
% statistic d for the null hypothesis that the
% data sets data1 and data2 are drawn from the same
% distribution are computed by KS2D2SB. Large values of d
% show that the two samples are significantly different. The KS
% statistic normalised for sample size is Z.
% BINS is used to place the x,y pairs in arrays of
% xsize x ysize equally spaced bins. KS2D2S2B
% is then used to test the binned data. Z and the percentiles
% of d are then returned in Z and prcnt.
%
% Uses XKS2D2S, BINS, KS2D2SB.
% see Numerical Recipes Example Book p193
%
%		MATLAB uses the same seed for the random number
%		generator on each initialisation of MATLAB ie
%		the sequences of random numbers are identical
%		after each initialisation! To use all the ~2E9
%		random numbers, the seed is stored at the end of
%		each call to XORCISE, and is loaded at the beginning
%		of the next call. The seed is stored in NSEED.MAT.
%
% Version 1.0 RHS 4/5/94
%		Modified version of XORCISE.M to calculate
%		sizexsize probabilities only for Monte Carlo
%		testing.
% Version 1.1 RHS 4/6/94
%		Modified to allow non-square arrays
% Version 1.2 RHS 14/6/94
%		seed for rand saved from previous and loaded
% Version 1.3 RHS 29/6/94
%		Saving to 'filename' added

if(nargin<7)
	draw=0;		% default is do not draw figures
end

if(nargin<6)
	filename='tempstor';	% default filename
end

% Now load seed file for random number generator, if it exists,
% otherwise find current value and save in nseed.mat.

if (exist('nseed.mat')==2)
	load nseed.mat
else
	nseed=rand('seed')	% get current value of seed
	save \matlab\toolbox\ks_tools\nseed.mat nseed
end

fprintf('Starting value of seed is %i\n',nseed)

rand('seed',nseed)

tic			% start timer

fprintf('Starting trial number 1...\n')

for k=1:repeats
	if isint(k/100)
		fprintf('Number of trials completed: %i\n',k)
				% screen output to show state
	end
	[data1,data2]=xks2d2s(n1,n2,1.0);	% generate x,y pairs
	[B1,B2]=bins(data1,data2,xsize,ysize);	% xsize x ysize bins
	[prob(k),Z(k)]=ks2d2sb(B1,B2,0);	% K-S test on binned data
end

t=fix(toc);			% elapsed time

	z=sort(Z);			% sort in ascending order
	prcnt=prctile(z,[0 50 80 90 95 98 99 99.9 100]);
					% calculate percentiles

if draw~=0			% draw figures
	words1=num2str(n1);words2=num2str(n2);...
	words3=num2str(xsize);words4=num2str(ysize);...
	words5=num2str(repeats);
	string=[words1 '+' words2 ', ' words3 'x' words4 ', ' words5 ' repeats']; 
	figure(1);a4paper;plot(z);grid on;...
		title(string);ylabel('Z=D*sqrt(N)');
	x=0:100;y=prctile(Z,(100-x));
	figure(2);a4paper;plot(y,x/100);grid on;...
		title(string);ylabel('P(>Z)');xlabel('Z');
end

	words1=num2str(n1);words2=num2str(n2);...
	words3=num2str(xsize);words4=num2str(ysize);...
	words5=num2str(repeats);words6=num2str(t);
	string1=[words1 '+' words2 ', ' words3 'x' words4 ...
	', ' words5 ' repeats']
	string2=['Elapsed time ' words6 'secs']

dynamicname=['\matlab\data\' filename];
eval(['save ' dynamicname ' Z prcnt string1 string2'])
		% save in file

% save seed for random number generator

nseed=rand('seed');
save \matlab\toolbox\ks_tools\nseed.mat nseed