echo on; clc; format short; pause on; more on;

% FILENAME      audidemo.m
% CALLS         qr2ud.m, lu0.m
% PURPOSE       Demonstrate the concepts and features of the Augmented UD
%               Identification approach.
% INPUTS        None
% OUTPUTS       None
% CHANGELOG     $Modified: Tue Mar 22 14:52:19 1994 by niu $


pause; clc;   % Press any key to continue 

%                  Augmented UD Identification (AUDI)
%                  ==================================
%
%   The augmented UD identification (AUDI) algorithm is a fundamental
%   reformulation and efficient implementation of the widely used least-squares
%   estimator. The AUDI method is superior to the original least-squares method
%   in almost all aspects and is recommended to be used as a complete
%   replacement/upgrade of the least-squares method for all applications such as
%   statistics, system identification, signal processing.
%   
%   The AUDI method is an order recursive decomposition method. It
%   simultaneously produces the parameter estimates and corresponding loss
%   functions for all models from order 1 to a user specified maximum. The
%   factorization structure results in improved numerical performance including
%   non-sensitivity to overparameterization. The multiple models provided by
%   AUDI can be used for many applications such as order determination, model
%   reduction, estimation of signal-to-noise ratio, monitoring parameter
%   identifiability, tracking of parameter changes.
%   
%   The AUDI algorithm can be flexibly implemented by using many of the well
%   established factorization methods such as LU factorization, QR
%   decomposition, Cholesky decomposition, Bierman's UD factorization, all of
%   which are well known for their superior numerical performance. Both batch
%   and time recursive implementations are possible and available. The structure
%   of the information flow in the AUDI algorithm is also well-suited for
%   parallel processing or VLSI implementation.
%   
%   For system identification, AUDI stands for a complete set of algorithms that
%   are analogous to the conventional least-squares family methods, which
%   include the basic least-squares form, the extended least-squares form, the
%   instrumental variable form and the corresponding extensions in multivariable
%   systems.
%   
%   The AUDI algorithm essentially combines many nice properties and useful
%   features into a single and compact yet very flexible algorithm and provides
%   a natural choice for control-oriented identification, as well as many other
%   applications. Compared with conventional least-squares, AUDI is simple in
%   concept, reliable in numerical performance, versatile in application and
%   easy in extension.
%   
%   The AUDI approach is expected to be interested by people in several
%   different fields.
%   
%   For more details about AUDI, see the references listed at the end of this
%   program.


    pause; clc;   % Press any key to continue 

%  This demonstration shows how the AUDI approach is used in the area of
%  system identification and parameter estimation. This demostration is
%  self-explanatory and the user only need to run the MATLAB file:
%  audidemo.m


%  Let's start by generating some simulation data.

%  Assuming the real system to be identified can be represented by the
%  following difference equation model (Astrom model)
%     y(t) - 1.5*y(t-1) + 0.7*y(t-2) = u(t-1) + 0.5*u(t-2) + v(t)
%  where y(t) and u(t) are the model output and input respectively, and v(t)
%  is zero-mean white noise. 

%  The transfer function form is defined by B/A, which is

   B = [0  1.0 0.5];
   A = [1 -1.5 0.7];


   pause;             % Press any key to continue.


%  Assuming that the data length and standard deviation of the system noise
%  are 

   Len = 500;                  % data length.
   std = 0.5;                  % standard deviation of noise. 
%  ^^^
%  (You can change the noise level _std_ for comparison.)


%  Using a random binary sequence (RBS) as system input, the system
%  input/output data sequence can then be generated by 

   th0 = poly2th(A, B);
   e = randn(Len,1);
   u = sign(randn(Len,1));
   y = idsim([u, std*e], th0);

   pause;         % Press any key to continue.

%  Let's look at the first 200 points of the input/out data.

   z = [y u];
   idplot(z, 1:200); 
   pause;   clc;    % Press any key to continue.

%  The above mehtod of generating input/output data is from Ljung's system
%  identification toolbox. Actually, any method can be used to generate the
%  system data. Pre-stored data can also be used and loaded at this point.

   pause; clc;  % Press any key to start AUDI 


%  Now we can start using the AUDI approach to identify this system.
%
%  Assuming that the real system order is unknown and the maximum possible
%  order is 
   n = 4;    % (You can use a higher order for comparison.)

%  Then the dimension of the augmented information matrix  is
   m = 2*n + 1;

%  and the augmented information matrix (AIM) S is constructed by 
   S = zeros(m,m);
   h = zeros(m,1);
   for t=n+1:Len;
       h(1:2:m) = -y(t-n:t)';
       h(2:2:m) =  u(t-n:t-1)';
       S = S + h*h';
   end

%  Here h is the augmented data vector with a special structure:
%    h=[-y(t-4),u(t-4),-y(t-3),u(t-3),-y(t-2),u(t-2),-y(t-1),u(t-1),-y(t)]';
% NB:    ^^^^^^^^^^^^^  ^^^^^^^^^^^^^  ^^^^^^^^^^^^^  ^^^^^^^^^^^^^  ====
%    1. The input and output are in pairs. 
%    2. The current process output -y(t) is included in the vector.
%    3. The dimension is thus m=2*n+1=2*4+1=9.


   pause; clc;             % Press any key to continue.

%  S is the augmented information Matrix (AIM)

   disp( S(1:min(7,m),1:min(7,m)) );


%  This augmented infromation matrix S contains, implicitly, all the
%  information on the model parameters and loss functions for all models
%  from order 1 to the user-specified maximum order n. 

   pause;             % Press any key to continue.

%  All these information can be extracted by applying the LU decomposition
%  to the AIM. As a result, all the information on model parameters and loss
%  functions are  explicitly obtained in two matrices, UU and DD.
   
   [L, E] = lu0(S);    UU = inv(L');    DD = diag(diag(E));

%  The resulting matrix UU is called the PARAMETER MATRIX, it contains all
%  the parameter estimates for all the models from order 1 to 4. 
%  The matrix DD is called the LOSS FUNCTION MATRIX, and it contains all the
%  corresponding loss functions for all models in UU. 

%  Note here lu0 is a function performing a LU factorization with no
%  permutation since this is not available from the MATLAB built-in lu
%  function call.

   pause; clc;             % Press any key to continue.

%  Let's look at the parameter matrix first. 

   disp(UU(1:min(7,m),1:min(7,m)));

%  The UU matrix provides the parameter estimates for all models from 
%  order 1 to the maximum possible order n=4 
%     the parameter of the 1st order model is found in column 2*1+1=3.
%     the parameter of the 2nd order model is found in column 2*2+1=5.
%     the parameter of the 3rd order model is found in column 2*3+1=7.
%     the parameter of the 4th order model is found in column 2*4+1=9.

   pause;  % Press any key to continue 

%  The 2*2+1 = 5th column, which is the 2nd order model parameters, together
%  with the true system parameters, are  

   disp([UU(1:5,5), [0.7,0.5,-1.5,1.0,1.0]']);

%  It is seen that this is quite a good estimate.

   pause; clc;             % Press any key to continue.


%  Parameter Estimates of other orders are listed as follows.

%  First Order Model Parameters:
   nn=1*2+1; disp(UU(nn:-1:1,nn)');
%  Second Order Model Parameters
   if n>1; nn=2*2+1; disp(UU(nn:-1:1,nn)'); end
%  Third order Model parameters.
   if n>2; nn=3*2+1; disp(UU(nn:-1:1,nn)'); end
%  Forth order Model parameters.
   if n>3; nn=4*2+1; disp(UU(nn:-1:1,nn)'); end


pause; clc

%  Now let's examine the loss function matrix DD 
   disp(DD(1:min(7,m),1:min(7,m)));

% DD contains all the loss functions of all models from 1 to n=4.
%  the loss function of the 1st order model is the 2*1+1=3rd diagonal element.
%  the loss function of the 2nd order model is the 2*2+1=5th diagonal element.
%  the loss function of the 3rd order model is the 2*3+1=7th diagonal element.
%  the loss function of the 4th order model is the 2*4+1=9th diagonal element.

   pause;             % Press any key to continue.


% The loss functions can be plotted against the model orders

  loss=diag(DD);   plot(0:n, loss(1:2:m)); 
  title('Loss functions vs. Model Orders');
  xlabel('Model Orders'); ylabel('Loss Functions'); 
  pause; % Press any key to continue.

%  Clearly it is seen that when the model order increases from 0 to 1 and
%  from 1 to 2, the loss functions have very significant decreases. However,
%  when the model order increases from 2 to 3 and from 3 to 4, the decreases
%  in loss functions cease to be significant. This suggests that the
%  estimated model order should be 2. Some quantitive method such as the
%  F-test and AIC can be used to determine the model order.

%  Let's see the figure again.
   pause;             %  Press any key to continue.

   figure(gcf); grid;
   pause; clc;             %  Press any key to continue.

%  In summary, the procedures to implement the AUDI method are:
%    1) Follow the special structure of the augmented data vector.
%    2) Formulate the augmented information matrix.
%    3) Decompose the AIM into LU factored form.
%  then all the parameter estimates and corresponding loss functions for
%  all models from order 1 to the user-specified maximum order n are
%  contained in matrix UU=inv(L') and DD=D respectively.
% 
%  Simple, right?  That is one of main advantages of AUDI: simplicity!

   clear S L E UU DD loss;
   pause; clc;             %  Press any key to continue.



%  Other decomposition methods, such as LDL', Cholesky, QR and UD
%  decompositions, can all be used to produce the UU and DD matrices, which
%  are the parameter and loss function matrices and have standard
%  interpretation.
%  This is another main advantage of AUDI: flexibility.

   pause;   %  Press any key to continue 

%  For example, the QR decomposition method can be used, and is also
%  preferred because it has very good numerical properties and does not
%  require formulating the augmented information matrix (data product
%  matrix). This is shown as follows

   H = zeros(Len,m);
   h = zeros(m,1);
   for t = n+1:Len
       h(1:2:m) = -y(t-n:t)';
       h(2:2:m) =  u(t-n:t-1)';
       H(t,:) = h';
   end

%   Again note the special structure of the augmented data vector
%      h=[-y(t-4),u(t-4),-y(t-3),u(t-3),-y(t-2),u(t-2),-y(t-1),u(t-1),-y(t)]';
%   NB:     ^^^^^^^^^^^^^  ^^^^^^^^^^^^^  ^^^^^^^^^^^^^  ^^^^^^^^^^^^^  ====
%   which is the same as before.
%   H is the augmented data matrix and is just a stack of shifted
%   input/output data. No calculation is involved to formulate the data
%   matrix. 

    pause;  %  Press any key to continue 
       
%   Applying the QR decomposition to H

   R = triu(qr(H));

%   then the parameter matrix UU and loss function matrix DD can be formulated
%   easily from the triangular matrix R

   [U D] = qr2ud(R);    % Simple information extraction.
   UU    = inv(U);      % NB: U is unit upper triangular.
   DD    = D*D;

   pause; clc;  %  Press any key to continue 


%   where UU and DD are found to be the same as that obtained by LU
%   decomposition.
    disp(UU(1:min(7,m),1:min(7,m)));
    disp(DD(1:min(7,m),1:min(7,m)));

    pause; clc;  %  Press any key to continue 

%  As a conclusion, the implementation of the AUDI concept can be very
%  flexible: QR, UD, LU, LDL', Cholesky decomposiitons can all be used, both
%  in a batch or recursive manner. The final results are the two matrices:
%  parameter matrix UU and loss function matrix DD, which contain useful
%  information for many applications.
%  For a detailed discussion on how to use other factorization techniques for
%  AUDI implementation, see references below. 
%  
%  For a more practical implementation of the AUDI approach, try the
%  following two MATLAB funcitons included in this demo package:
%    1) baudi.m --- batch implementation of the AUDI method.
%    2) raudi.m --- recursive implementation


%  You can even test the reliability of the AUDI method by setting the
%  standard deviation of system noise (std) to zero and overparameterize the
%  system using a maximum possible model order of, say, 10 or 20, and see
%  the results! This is a very ill-conditioned case and least-squares method
%  fails to give a solution. Try AUDI!
%
%  You can also specify the correct system order n=2 as the maximum possible
%  model order, which is perfectly legal. In this case, AUDI is not so
%  overwhelmingly better than conventional least-squares, but it is never
%  worse. In another word, if you choose AUDI instead of least-squares, you
%  lose nothing. The worst case is that AUDI has the same numerical
%  performance and produce the same amount of information as least-squares.
%  Or with a mathematical formula
%            AUDI >= LS     for all ....



   pause;  clc;  %  Press any key to contiue 


%  Finally, to show another advantage of the AUDI method: versatility,
%  consider the problem of solving linear simultaneous equations 
%               A x = b
%  with, for example

   A = randn(10,3);
   b = randn(10,1);

   pause;  %  Press any key to continue 

% The AUDI approach can be used to obtain the solution by

   R = triu(qr([A -b]));
   [U D] = qr2ud(R);
   UU    = inv(U);
   DD    = D*D;
   UU, DD


%  where the last column of UU gives the least-squares solution to the
%  equations and the last element of DD gives the least-squares fitting
%  error for the solution.

   pause; clc;  %  Press any key to continue 


%  The MATLAB built-in function `\' produces
%  

     x = A\b;
     echo off; format long e;
     disp(' ');
     disp('         x=A\b                   AUDI');
     disp([x, UU(1:3,4)])
     format short; echo on
  
     pause; %  Press any key to continue 

%  The least-squares fitting error, which is given by
    err = (b-A*x)'*(b-A*x);
    err

%  equals to the last element of DD,

    disp([err, DD(4,4)]);

     
%  Although 'A\b' seems to be much simpler that AUDI, internally MATLAB
%  is implementing a similar but more complicated calculation than AUDI.


   pause;  clc;  %  Press any key to continue 

%  Well, got an idea about AUDI? 
%  If you feel interested and wanna know more about AUDI, you can look up
%  the following key references, or drop me a mail at
%  niu@comet.eche.ualberta.ca. 
%
%  Thank you for your interest.
%
%  Key References: 
%   1.  Niu, S., Augmented UD identification for process control,
%       PhD thesis, 1994, University of Alberta, Edmonton, Canada.
%   2.  Niu, S., Fisher, D.~G. and Xiao, D.  (1992), An augmented {UD}
%       identification algorithm, International Journal of Control, Vol. 56,
%       No.1, pp193 -- 211.
%   3.  Niu, S., Fisher, D.~G. and Xiao, D.  (1993), A factored form of the
%       Instrumental Variable identification algorithm, International
%       Journal of Adaptive Control and Signal Processing, Vol.7, No.4,
%       pp261--273. 
%   4.  Niu, S., Xiao, D. and Fisher, D.~G. (1990), A recursive algorithm
%       for simultaneous identification of model order and parameters, IEEE
%       Transactions on Acoustics, Speech, and Signal Processing, Vol.38,
%       No.5, pp884-886.
%   5.  Niu, S., Fisher, D.~G. (1993), Simultaneous structure identification
%       and parameter estimation of multivariable systems, International
%       Journal of Control, in press.
%   6.  Niu, S., Fisher, D.~G.  (1993), Simultaneous estimation of process
%       parameters, noise variance and signal-to-noise ratio, IEEE
%       Transactions on Signal Processing, to appear.
%
    pause; clc;  %  Press any key to return 
