function [U,D,residual]=audiupdate(U,D,h,lambda)

% FUNCTION	[U,D,v_hat]=audiupdate(U,D,h,lambda)
% PURPOSE	Recursive AUDI update for SISO system (subroutine)
% INPUTS	U : parameter matrix
%		D : loss function matrix
%		h : augmented data vector
%		lambda: forgeting factor
% OUTPUTS	U : parameter matrix
%		D : loss function matrix
%		residual: residual for use by ELS
% AUTHOR        Steve Niu (niu@isy.liu.se or niu@comet.eche.ualberta.ca)
% CHANGELOG	$\Modified: Sun Mar  6 17:13:07 1994 by niu $
% REFERENCE	Niu, S., Augmented UD identification for process control,
%		PhD thesis, 1994, University of Alberta, Edmonton, Canada.


% Check arguments
	if (nargin<3 | nargin>4)
		error('wrong number of input arguments.'); end
	if (nargin==3), lambda=1.0; end

% Dimensions
	[m,nh] = size(h);
	if (nh==2), IV=1; end
	if (nh==1), IV=0; end

% Initializations
	beta = zeros(m+1,1); 
	nu   = zeros(m,m);   
	mu   = zeros(m,1);

% U, D update
	beta(1)=lambda;
	f = U'*h;
	if (IV) 
		g=D.*f(:,2); 
	else
		g=D.*f(:,1);
	end

	for j=1:m
		beta(j+1) = beta(j) + f(j,1)*g(j);
		D(j)      = D(j)*beta(j)/beta(j+1)/lambda;
		mu(j)     = -f(j,1)/beta(j);
		nu(j,j)   = g(j);
		if (D(j)<eps), D(j)=eps; end
		if (D(j)>1.0/eps), D(j)=1.0/eps; end
	end
	for j=2:m
		nu(1:j-1,j) = nu(1:j-1,j-1) + U(1:j-1,j)*nu(j,j);
		U(1:j-1,j)  = U(1:j-1,j) + nu(1:j-1,j-1)*mu(j);
	end

% the residual
	residual = -f(m,1)/beta(m-1);

% back to the calling function
	return
	
