function [U,D]=baudi(z,nn,method)

% FUNCTION      [U,D]=baudi(z,nn,method)
% PURPOSE       Batch AUDI for SISO system
% INPUTS        z=[y,u]: input/output data sequence
%               nn=[na,nb,nk]: model structure
%               method: identification method
%                       'LU'    - LU Factorization
%                       'QR'    - QR Decomposition (default!!!)
%                       'CHOL'  - Cholsky decomposition
% OUTPUTS       U : parameter matrix
%               D : loss function matrix
% CALLS         lu0.m, qr2ud.m
% AUTHOR        Steve Niu (niu@isy.liu.se or niu@comet.eche.ualberta.ca)
% CHANGELOG     $Modified: Tue Mar 22 15:55:01 1994 by niu $
% REFERENCE     Niu, S., Augmented UD identification for process control,
%               PhD thesis, 1994, University of Alberta, Edmonton, Canada.


% Check arguments
        if (nargin<2 | nargin>3)
                error('wrong number of input arguments.'); end
        if (nargin<3), method='QR'; end

% Identification method.
        LU=0; QR=0; CHOL=0;
        method = upper(method);
        if (strcmp(method,'LU')), LU=1; end
        if (strcmp(method,'QR')), QR=1; end
        if (strcmp(method,'CHOL')), CHOL=1; end
        if (LU+QR+CHOL ~= 1)
                error('wrong identification method specified.'); end

% Dimensions
        [N,nz] = size(z);
        n      = max(nn(1:2));
        m      = n*2+1;
        nk     = nn(3);

% Initialization
        h = zeros(m,1);
        if (LU|CHOL), S=zeros(m,m); end
        if (QR),      H=zeros(H,m); end

% Formulate the augmented data matrix (for QR) or the augmented information
% matrix (for LU and Cholesky)
        for t=nk+2:N
                h(1:m-2) =  h(3:m);
                h(m-1)   =  z(t-nk-1,2);
                h(m)     = -z(t,1);
                if (t>nk+n+2)
                        if (LU|CHOL); S = S + h*h'; end
                        if (QR);      H(t,:) = h' ; end
                end
        end
        

% Convert the decomposition results to the standard parameter and loss
% function matrices.
        if (LU)
                [L,E] = lu0(S);
                U     = inv(L');
                D     = diag(E);
        else
                if (CHOL); R  = chol(S); end
                if (QR);   R  = triu(qr(H)); end
                [U D] = qr2ud(R);
                U  = inv(U);
                D  = D*D;
        end

                
