function        [U,D] = qr2ud(R)
% FILENAME      [U,D] = r2ud(R)
% PURPOSE       decompose the  R matrix from QR decomposition into 
%               UD form.
% INPUTS        R: from QR decomposition.
% OUTPUTS       U: unit-upper-triangular matrix.
%               D: diagonal matrix.
% AUTHOR        Steve Niu (niu@isy.liu.se or niu@comet.eche.ualberta.ca)
% CHANGELOG     $Modified: Tue Mar 22 15:53:38 1994 by niu $
% COMMENTS      The parameter and loss function matrices UU and DD can usually
%               be obtained by
%                  UU = inv(R)*diag(R)
%                  DD = diag(R)*diag(R)
%               However, when the data matrix H is ill-conditioned, some of
%               the diagonal elements of R may be zeros and thus R is not
%               invertible. This function converts the R matrix into a product
%               of an unit-upper-triagular matrix U and a diagonal matrix D,
%               which are always available from R. Then the parameter and loss
%               function matrices are obtained by
%                  UU = inv(U);
%                  DD = D*D
%               Therefore numerical problems are avoided. This is a
%               generalization of the QR decomposition form of AUDI where R
%               can be any size and can be singular or non-singular.


%  Dimensions
    
    [mr,ml]=size(R);
    m = ml;

    U=eye(m,m); D=zeros(m,m);


%  The follow instructions extract the diagonal elements of R to form an
%  diagonal matrix D and divide each row of R by its diagonal element to form
%  the unit upper-trianugular matrix U. If the diagonal element is zero, the
%  this row is skipped and assumed to be zeros.

    Dmax = max(abs(diag(R)));
    tol  = eps*Dmax*1.0e3;

    for i=1:min(mr,ml)
        D(i,i)=R(i,i);
        if ((D(i,i)>tol)|(D(i,i)<-tol))
                for j=i+1:m
                    U(i,j)=R(i,j)/D(i,i);
                end
        end
    end

    

% back 
   return
