function [U,D,UN,DN]=raudi(z,nn,lambda,method)

% FUNCTION	[U,D,UN,DN]=raudi(z,nn,lambda,method)
% PURPOSE	Recursive AUDI for SISO system
% INPUTS	z=[y,u]: input/output data sequence
%		nn=[na,nb,nk]: model structure
%		lambda: forgeting factor
%		method: identification method
%			'LS'  - least-squares
%			'ELS' - extended least-squares (default!!!).
%			'IV'  - instrumental variable method
% OUTPUTS	U : parameter matrix
%		D : loss function matrix
%		UN: parameter matrix for noise model (ELS only);
%		DN: loss function matrix for noise model (ELS only);
% CALLS         audiupdate.m, els2ud.m
% AUTHOR        Steve Niu (niu@isy.liu.se or niu@comet.eche.ualberta.ca)
% CHANGELOG	$Modified: Tue Mar 22 15:54:10 1994 by niu $
% REFERENCE	Niu, S., Augmented UD identification for process control,
%		PhD thesis, 1994, University of Alberta, Edmonton, Canada.


	echo off; clc

% Check arguments
	if (nargin<2 | nargin>4)
		error('wrong number of input arguments.'); end
	if (nargin<4), method='ELS'; end
	if (nargin<3), lambda=1.0;   end
	if (lambda<0.2|lambda>1.0)
		error('wrong forgetting factor.'); end

% Identification method
	method=upper(method);
	LS=0; ELS=0; IV=0;
	if (strcmp(method,'LS'));  LS=1;  base=2; end
	if (strcmp(method,'ELS')); ELS=1; base=3; end
	if (strcmp(method,'IV'));  IV=1;  base=2; end
	if (LS+ELS+IV ~= 1)
		error('wrong identification method specified'); end
	if (ELS~=1 & nargout>2)
		error('wrong number of output argument.'); end

% Dimensions;
	[N,nz] = size(z);
	n      = max(nn(1:2));
	m      = n*base+1;
	nk     = nn(3);

% Initializations
	U = eye(m,m);
	D = ones(m,1)*1.0e3;
	if (LS),  h=zeros(m,1); end
	if (ELS), h=zeros(m,1); residual=0; end
	if (IV),  h=zeros(m,2); end

% U, D Estimation
	for t=nk+2:N
		h(1:m-base,1) =  h(base+1:m,1);	
		h(m,1)        = -z(t,1);
		h(m-1,1)      =  z(t-nk-1,2);
		if (ELS), h(m-2,1)=residual; end
		if (IV)
			if (t<nk+2+20), h(:,2)=h(:,1); end
			if (t>nk+21)
				h(1:m-base,2) = h(base+1:m,2);
				h(m-1,2)      = z(t-nk-1,2);
				h(m,2)        = -U(1:m-1,m)'*h(1:m-1,2);
			end
		end

		if (t>nk+n+2)
			[U,D,residual]=audiupdate(U,D,h,lambda);
		end
	end


% if ELS, convert result to standard U,D form
	if (ELS)
		UE=U; DE=D;
		[U,D,UN,DN] = els2ud(UE,DE);
	end

% convert D to loss function matrix 
	D=1.0./D;

% if no output, display the results
	if (nargout==0)
		format compact
		disp('Parameter matrix:')
		disp(U); 
		disp('Press any key to continue...'); 
		pause; disp('  ');
		disp('Loss Function Matrix:');
		disp(diag(D)); 
		disp('Press any key to continue...'); 
		pause; disp('  ');
		format loose
	end

% Back to calling function.
	return;



