function audit( mfile, arg1, arg2 )
%AUDIT		Prepare an audit of an m-file
%
%Usage:		AUDIT( MFILE [,'VERBOSE'], [,'SILENT'], [,'NODISPLAY'] )
%
%	This function calls a Perl script which parses and analyzes the
%	tokens in the specified m-file.  A summary of the results is output
%	in the command window.
%
%	Unless invoked with the 'NODISPLAY' flag, a full-colour listing of
%	the m-file is displayed in a figure window with an accompanying GUI.
%
%	If the function is invoked with the 'VERBOSE' flag, then the full
%	paths of the m-files (or, search path entities) are printed.
%
%	If the function is invoked with the 'SILENT' flag, then the
%	usage summary is suppressed.
%
%	Clicking on the text of any function (any yellow text, by default)
%	with the left mouse button will cause the selected function's HELP
%	text to be displayed on the command window.  Shift-clicking (or
%	control-clicking, middle- or right-button-clicking) will re-invoke
%	AUDIT with the 'DISPLAY' and 'SILENT' options.
%
%Example:
%		>> audit zoom
%
%Notes:	You must have Perl installed on your system for this to work!
%
%	An m-file is said to RECURSE if it invokes itself.
%
%	The 'display' mode entities have the following "factory" settings:
%
%		Entity		Colour		Weight		Angle
%		=========	=======		======		======
%		arguments	cyan		normal		normal
%		globals		blue		bold		normal
%		variables	green		normal		normal
%		m-files		yellow		bold		normal
%		built-ins	yellow		normal		normal
%		comments	magenta		normal		italic
%		strings		white		bold		italic
%		default		white		normal		normal
%		unknowns	red		bold		italic
%	
%	where	"arguments" are both the inputs and the outputs defined in
%			in a function definition,
%		"m-files" refers to any m-file, mex-file or simulink function
%			on the search path,
%	
%	Because MATLAB's namespace can be modified by scripts and the EVAL
%	command, not all variables can be recognized as such.  Since the
%	unidentifiable names probably will have an EXIST code of zero, they
%	can be initially flagged as 'unknown' functions.
%
%	The display mode highlights 'unknown' names, but the textual summary
%	list will assume that 'unknown' names are variables.
%
%	The Perl script creates a temporary m-file which is then exectuted
%	(and then deleted) by MATLAB.  The temporary m-file is placed in
%	the current directory, which means you need write permission in
%	that directory.  You can, of course, specify full path names...
%	
%	The display mode does not correctly handle implicit string arguments.
%	That is "HOLD ON" and "HOLD( 'ON' )" are identical invocations, but
%	the former's argument shows up in highlighted as 'default' text,
%	whereas the latter's is correctly highlighted as a 'string'.
%
%	The parsing is believed to correctly support multiple line (i.e.,
%	using the '...' continuation) as well as comments and strings.
%	Please inform me of any incorrect parsing.
%
%	The m-file being analyzed is assumed to be a syntactically correct.
%
%Distribution:
%
%	This file is provided to you by Ahlea Systems Corp., makers of the
%	Pattern Recognition Toolbox for MATLAB.  Please visit our web site at
%		http://vvv.com/~caughey/ahlea
%
%	Comments, bug reports, etc. can be made to Dave Caughey at
%		caughey@ahlea.com

%       (c) Copyright 1996 by Ahlea Systems Corp.  All rights reserved.
%
%	This file may not be included in any commecially distributed without
%	the express written permission of the author.


%
% Step zero... do argument checking
%
display = ' -display ';
verbose = 0;

if nargin == 2 | nargin == 3,
    for i = 1:nargin-1,
	eval( sprintf( 'arg = lower(arg%d);', i ) );
	if strcmp( arg, 'nodisplay' ),
	    display = ' ';
	elseif strcmp( arg, 'verbose' ),
	    verbose = 1;
	elseif strcmp( arg, 'silent' ),
	    verbose = -1;
	else,
	    help audit;
	    return;
	end;
    end;
elseif nargin ~= 1,
    help audit;
    return;
end;

%
% Step one... try and locate the source file.
%	If the name has extraneous spaces, strip them off
%	If the name ends with a '.m', then strip it off
%
mfile = mfile(mfile~=' ');
stripped = '';
if strcmp( mfile((-1:0)+length(mfile)), '.m' ),
    stripped = '.m';
    mfile = mfile(1:length(mfile)-2);
end;
mfilepath = which(mfile);
if isstr( mfilepath ) == 0,
    mfilepath = ls( [mfile stripped] );
    if isempty( mfilepath ),
	return;
    end;
end;

%
% Step two... try and locate the perl script
%	look in 1) current directory, followed by 2) $HOME/matlab
%
[s,plpath] = unix('if -e audit.pl echo audit.pl');
if isempty( plpath ),
    [s,plpath] = unix(['if -e ' getenv('HOME') '/matlab/audit.pl echo ' getenv('HOME') '/matlab/audit.pl']);
    if isempty( plpath ),
	fprintf( 'audit: support file ''audit.pl'' not found\n' );
	return;
    end
end;

%
% Step three... run the perl script and the execute the generated script.
%	Delete the script after execution.
%

[s,v] = unix([plpath(1:length(plpath)-1) display mfilepath]);
clear audit_display;
eval( 'audit_display( verbose );' );
delete audit_display.m;

