function bytes = auwrite(y,filename,fs,format)
%   AUWRITE Write mu-law encoded audio file.
%       AUWRITE(Y,'filename',fs,format,rate)
%       Copyright (c) 1984-94 by The MathWorks, Inc.
% 
%      Modified: Matt Grund  1996 matt@sardine.whoi.edu
%
%      now takes a format number which is:
%
%		1: 8 bit mu-law
%		2: 8 bit linear
%		3: 16 bit linear
%		5: 32 bit linear
%
%      and a sample rate, which can be any int, but is typically:
%           
%               8012, 22050, 44100, or 48000
%
%      default values are 16 bit linear, 44100 Hz for "auwrite(data,'filename');"
%      default filename is 'tmp.snd' for "auwrite(data);"
%
%      number of output channels is the number of columns in data.
%

magic=779316836;          % ".snd"

if ( nargin < 1 )
	error('auwrite: no data ( too few parameters )');
end

if ( nargin == 1 )
	filename='tmp.snd';
end

if ( nargin < 3 )
	fs=44100;
end

if (nargin < 4 )
	format=3;
end

if (format==1)
	formstring='8 bit mu-law';
	precstring='uchar';
	y=lin2mu(y);
	bytespersample=1;
end;

if (format==2)
	formstring='8 bit linear';
	bytespersample=1;
	precstring='int8';
end;

if (format==3)
	formstring='16 bit linear';
	bytespersample=2;
	precstring='int16';
end;

if (format==5)
	formstring='32 bit linear';
	bytespersample=4;
	precstring='int32';
end;

if ((format~=5)&(format~=1)&(format~=2)&(format~=3))
	formstring='UNRECOGNIZED';
	bytespersample=1;
end;

rate=fs;
channels=min(size(y));
bytes=length(y)*bytespersample*channels;
name=filename;

% print header report
fprintf(1,'\nauwrite:\n');
fprintf(1,'        file:         "%s"\n',name);
fprintf(1,'        format:        %s\n',formstring);
fprintf(1,'        channels:      %d\n',channels);
fprintf(1,'        size:          %d bytes\n',bytes);
fprintf(1,'        sample rate:   %d Hz\n',rate);
duration=bytes/bytespersample/channels/rate;
fprintf(1,'        duration:      %06.2f seconds\n\n',duration);
fp = fopen(filename,'wb');
if fp == -1
	error(['Cannot open ' filename ' with write privilege.'])
end

%write the header
fwrite(fp,magic,'int32');	% magic number
fwrite(fp,28,'int32');		% location
fwrite(fp,bytes,'int32');	% size
fwrite(fp,format,'int32');	% format
fwrite(fp,rate,'int32');	% sample rate
fwrite(fp,channels,'int32');	% number of channels
fwrite(fp,0,'int32');		% comment

%write the data
y = y';
y=reshape(y,size(y,1)*size(y,2),1);
fwrite(fp,y,precstring);
fclose(fp);





