function latinize(filein, fileout, overwriteOK)
% Modify the input file (a MATLAB-generated PostScript file) to make it print in% the Latin-1 encoding for the benefit of non-U.S. users.  Write the result to
% the output file.  Both input and output files must be on the MATLABPATH.
% If overwriteOK is nonzero, then don't worry about overwriting the output file.% Otherwise quit if the output file already exists.  (Default is don't 
% overwrite.)
% Some fonts are not mapped to Latin-1, specifically Symbol and Zapf Dingbats,
% since they have special encodings.  Other such fonts can be added to this
% M-file if desired.
%
% Here's an explanation of the purpose of this M-file:
%
% In MATLAB 4.1 and earlier, characters outside of the ASCII range (for example,% o-with-umlaut) that appear in figure windows don't get printed correctly in
% PostScript.  The latinize.m M-file provides a workaround for this problem,
% which is fixed in MATLAB 4.2.  Suppose that you've generated a PostScript file% myfigure.ps.  Call latinize as follows: latinize('myfigure.ps', 'fixed.ps') to% generate a file fixed.ps that will print correctly.  
%
% Note: latinize.m works as is with MATLAB 4.1.  In order to use latinize.m with% MATLAB 4.0, you need to obtain another M-file, findstr.m (available from 
% ftp.mathworks.com:/pub/contrib/misc).  latinize.m does not work with any 
% versions of MATLAB prior to 4.0.
 
if(strcmp(filein, fileout) == 1)
  error(sprintf('Input file = output file = %s.  These two files cannot be identical.', filein));
end
if nargin < 3 
  overwriteOK = 0;
end
if overwriteOK == 0
  if exist(fileout)
    error(sprintf('The output file ''%s'' already exists', fileout));
  end
end
fidi = fopen(filein);
if(fidi == -1) 
  error(sprintf('Unable to open input file ''%s''', filein));
end
fido = fopen(fileout, 'w');
if(fido == -1) 
  error(sprintf('Unable to open output file ''%s''', fileout));
end
 
% find the beginning of the MathWorks dictionary
mwdictstring = '/MathWorks';
mwlength = length(mwdictstring);
while 1
  line = fgetl(fidi);
  if isempty(line) 
    fprintf(fido, '\n');
  else
    fprintf(fido, '%s\n', line);
  end
  if line == -1
    error('Couldn''t find MathWorks PostScript dictionary');
  elseif length(line) >= mwlength
    if strcmp(line(1:mwlength), mwdictstring) == 1
      break;  % found the MathWorks dictionary, stop looking
    end
  end
end
 
% Add the "latinize" definition.  Takes one arg, the font name, and changes
% that font's encoding to Latin-11.  Leaves the font name arg on the stack.
for str =...
['/latinize { %def                                      '
 '    dup ISOLatin1Encoding exch dup                    '
 '    findfont dup length dict begin                    '
 '      { 1 index /FID ne {def}{pop pop} ifelse } forall'
 '      exch /Encoding exch def                         '
 '    currentdict end definefont pop                    '
 '} bind def                                            ']'
  fprintf(fido, '%s\n', str);
end
 
% in every occurrence of our FMS macro, change the font to use the Latin-1
% encoding (except for the special fonts Symbol and Zapf Dingbats)
FMSlength = length('FMS');
symbolLength = length('/Symbol');
zapfDingbatsLength = length('/ZapfDingbats');
while 1
  line = fgetl(fidi);
  len = length(line);
  if isempty(line) 
    fprintf(fido, '\n');
  elseif line == -1
    break;  % end of file
  % find lines that end in 'FMS' (we invoke the FMS macro on all fonts in the 
  % course of setting them up) and reencode the fonts if they are not special
  elseif strcmp(line(max(len-FMSlength+1, 1):len), 'FMS') == 1 &...
         strcmp(line(1:min(symbolLength,len)), '/Symbol') == 0 &...
         strcmp(line(1:min(zapfDingbatsLength,len)), '/ZapfDingbats') == 0
    blanks = findstr(line, ' ');
    if length(blanks) == 0
      error('Unable to find blank char in FMS line');
    end
    endOfFontName = blanks(1);
    % insert "latinize" between the font name and the call to FMS
    % in order to reencode the font in Latin-1
    fprintf(fido, '%slatinize%s\n', line(1:endOfFontName), line(endOfFontName:len));
  else
    fprintf(fido, '%s\n', line);
  end
end
 
fclose(fidi);
fclose(fido);
