#include <math.h>
#include <stddef.h>
#include "cmex.h"
#include <conio.h>

/* I/O port addresses */
static int BASE= 0x220;


static int volt2int( double volt )
{
	return(  (int)(volt/10.0*16384.0) + 8192 );
}



static void writeda( unsigned int data, int channel )
{
	if( channel == 0 )  {
		outp(BASE+4,data);
		outp(BASE+5,data>>8);
	}
	else  {
		outp(BASE+6,data);
		outp(BASE+7,data>>8);
	}
}




/* writeda( y, [chan] )   writes the voltage specified in y to
   channel 0 if no channel is specified, or the channel given by chan.
   If chan == 2, both channels are written to. */

static writeda_fcn(int nlhs, Matrix *plhs[], int nrhs,Matrix *prhs[])
{
	int channel=0, n, m;
	
	/* Check arguments! */
	if( (nrhs<1) || (nrhs>2) )
		mex_error( "writeda: wrong number of input parameters" );
	if( nrhs == 2 )  {
		channel = (int) prhs[1]->pr[0];
		if( (channel<0) || (channel>2) )
			mex_error( "writeda: illegal channel number" );
	}
	if( channel == 2 )  {
		/* If two data samples given, put first on CH0 and second
		   on CH1, otherwise the single sample on both channels */
		n = prhs[0]->n; m = prhs[0]->m;
		if( (n==2) || (m==2) )  {
			writeda( volt2int(prhs[0]->pr[0]), 0 );
			writeda( volt2int(prhs[0]->pr[1]), 1 );
		}
		else  {
			writeda( volt2int(prhs[0]->pr[0]), 0 );
			writeda( volt2int(prhs[0]->pr[0]), 1 );
		}
	}
	else    writeda( volt2int(prhs[0]->pr[0]), channel );
	/* Return 0 */
	plhs[0]->pr[0] = 0.0;
	return(0);
}

/* Set_pacer() sets the sampling period for A/D acquisitions. It
   is a bit complicated, due to the troublesome nature of the timer
   circuitry.  The sample period is determined by a 2 MHz clock that
   is divided by two cascade-coupled 16-bit counters.  The formula for
   the sample period then is:

	T = 0.5E-6*C1*C2

   where C1 and C2 are the values in counters 1 and 2.

   Assume that C1 is always set to 2.  Then T=C2 microseconds, yielding a
   maximum sample period of about 64 ms.  To get a longer period, C1 may
   be increased by a factor of 10. If C1 is 2000, the sample period may be
   varied between 1 ms and 64 s.
   The algorithm implemented below sets the minimum 'power of 10' required
   for the specified sampling period. */

static void set_pacer(double period)
{
	unsigned int power=2, intperiod;

	period *= 1E6;  /* Unit: microseconds */

	while( period/0xFFFF > 1.0 )  {
		power *= 10;
		period /= 10.0;
	}

	intperiod = (unsigned) period;
	outp(BASE+3,0x74);
	outp(BASE+1,power);
	outp(BASE+1,power>>8);
	outp(BASE+3,0xB4);
	outp(BASE+2,intperiod);
	outp(BASE+2,intperiod>>8);
}


/* PCL_readad() reads analog data from one channel into the array
   pointed to by data. If only one data point is requested, the
   pacer clock is not used. */

static void readad( char *data, int count, int channel, double period )
{
	int temp, temp2;

	if( count == 0 ) return;
	outp(BASE+10,channel);          /* Select input channel */
	if( count == 1 )  {
		outp(BASE+11,1);        /* Start immediate conversion */
		/* Wait for data conversion to finish */
		while( (temp=inp(BASE+5)) & 0x40 );
		*data++ = (char) inp(BASE+4); /* LSB */
		*data = (char) temp;   /* MSB */
	}
	else  {
		/* set pacer */
		set_pacer(period);
		/* enable pacer */
		outp(BASE+11,2);
		/* Read data */
		while( count-- )  {
			while( (temp=inp(BASE+5)) & 0x40 );
			*data++ = (char) inp(BASE+4);
			*data++ = (char) temp;
		}
		/* disable pacer */
		outp(BASE+11,0);
	}
}

static void int2volt(double *volt,int *data,int count)
{
	while( count-- )
		*volt++ = (*data++ - 8192) *(10.0/16384.0);
}





/* readad reads one sample from channel 0
   readad(chan) reads one sample from channel chan
   readad(count,dt) reads count samples from channel 0 with period dt
   readad(count,dt,chan) reads count samples from channel chan
   with period dt.  */

static void readad_fcn(int nlhs, Matrix *plhs[], int nrhs,Matrix *prhs[])
{
	int *data, chan, count;
	double dt;

	if( nrhs == 0 )  {
		data = (int *) mex_calloc(1,sizeof(int));
		plhs[0] = create_matrix(1,1,REAL);
		if( (data==NULL) || (plhs[0]==NULL) )  {
			mex_printf("Could not allocate READAD memory");
			mex_error("READAD error");
		}
		readad((char *)data,1,0,0.0);
		int2volt(plhs[0]->pr, data, 1 );
	}
	else if( nrhs == 1 )  {
		/* readad(chan) */
		chan = (int) prhs[0]->pr[0];
		if(  (chan<0) || (chan>15)  )  {
			mex_printf("Illegal channel %1.0f\n",(double) chan);
			mex_error("READAD error");
		}
		data = (int *) mex_calloc(1,sizeof(int));
		plhs[0] = create_matrix(1,1,REAL);
		if( (data==NULL) || (plhs[0]==NULL) )  {
			mex_printf("Could not allocate READAD memory");
			mex_error("READAD error");
		}
		readad((char *)data,1,chan,0.0);
		int2volt(plhs[0]->pr, data, 1 );
	}
	else if( nrhs == 2 )  {
		/* readad(count,dt) */
		count = (int) prhs[0]->pr[0];
		dt = prhs[1]->pr[0];
		data = (int *) mex_calloc(count,sizeof(int));
		plhs[0] = create_matrix(count,1,REAL);
		if( (data==NULL) || (plhs[0]==NULL) )  {
			mex_printf("Could not allocate READAD memory");
			mex_error("READAD error");
		}
		readad((char *)data,count,0,dt);
		int2volt(plhs[0]->pr, data, count );
	}
	else if( nrhs == 3 )  {
		/* readad(count,dt,chan) */
		count = (int) prhs[0]->pr[0];
		dt = prhs[1]->pr[0];
		chan = (int) prhs[2]->pr[0];
		data = (int *) mex_calloc(count,sizeof(int));
		plhs[0] = create_matrix(count,1,REAL);
		if( (data==NULL) || (plhs[0]==NULL) )  {
			mex_printf("Could not allocate READAD memory");
			mex_error("READAD error");
		}
		readad((char *)data,count,chan,dt);
		int2volt(plhs[0]->pr, data, count );
	}
	else mex_error("Illegal number of READAD parameters for PCL714 board");
}


static void init_fcn()
{
	/* Do nothing*/
}

static void digout( unsigned int data )
{
	outp(BASE+13,data);
	outp(BASE+14,data>>8);
}

/* digout(y) writes the 16-bit value y to the digital output lines. */
static void digout_fcn(int nlhs, Matrix *plhs[], int nrhs,Matrix *prhs[])
{
	digout( (unsigned) prhs[0]->pr[0] );
}

int digin(void)
{
	int low;

	low = inp(BASE+6);
	return( (inp(BASE+7)<<8) | low );
}



/* y=digin reads the 16-bit value from the digital intput lines to y. */
static void digin_fcn(int nlhs, Matrix *plhs[], int nrhs,Matrix *prhs[])
{
	plhs[0] = create_matrix(1,1,REAL);
	if( plhs == NULL )   mex_error("Cannot allocate DIGIN memory");
	plhs[0]->pr[0] = (double) digin();
}

static void set_io_port_fcn(int nlhs, Matrix *plhs[], int nrhs,Matrix *prhs[])
{
	BASE = (int) prhs[0]->pr[0];
}

user_fcn(int nlhs, Matrix *plhs[], int nrhs, Matrix *prhs[])
{
	int mode;
	static int (*fcn[])() = { writeda_fcn,
				  readad_fcn,
				  init_fcn,
				  digout_fcn,
				  digin_fcn,
				  set_io_port_fcn,
				    NULL };

	mode = (int) prhs[0]->pr[0];
	if( (mode<0) || (mode>7) )  mex_error("illegal mode number");
	plhs[0] = create_matrix(1,1,REAL);
	fcn[mode](nlhs,plhs,nrhs-1,prhs+1);
	return(0);
}
