
/////////////////////////////////////////////////////////////////////////
//
//	File	:	dcomplex.h -- double precision complex class
//
//	Author	:	Bart van Tongeren
//
//	Date	:	Tue Jan 18 17:46:59 MET 1994
//
/////////////////////////////////////////////////////////////////////////

//
// Define double precision complex class that is const correct and
// has a complete set of overloaded +,-,*,/,+=,-=,*=,/= operators.
// Defines conversions from and to the complex class, which enables
// the use of the friendly functions in the complex math library.
//

#ifndef DCOMPLEX_H
#define DCOMPLEX_H

#include <complex.h>

# ifndef sun
typedef complex dcomplex;
# else

class dcomplex
{
	double re, im;
public:
	dcomplex() {
	}
	dcomplex(double r, double i = 0) : re(r), im(i) {
	}
	dcomplex(const complex& c) : re(real(c)), im(imag(c)) {
	}
	operator complex() const {
		return complex(re, im);
	}

	dcomplex  operator - () const;
	dcomplex  operator + () const;

friend	dcomplex operator + (const dcomplex& lhs, const dcomplex& rhs);
friend	dcomplex operator - (const dcomplex& lhs, const dcomplex& rhs);
friend	dcomplex operator * (const dcomplex& lhs, const dcomplex& rhs);
friend	dcomplex operator / (const dcomplex& lhs, const dcomplex& rhs);

	dcomplex& operator += (const dcomplex& rhs);
	dcomplex& operator -= (const dcomplex& rhs);
	dcomplex& operator *= (const dcomplex& rhs);
	dcomplex& operator /= (const dcomplex& rhs);

	dcomplex& operator += (double rhs);
	dcomplex& operator -= (double rhs);
	dcomplex& operator *= (double rhs);
	dcomplex& operator /= (double rhs);

	// work-around cfront 2.1 bug:
	// ambiguous use of overloaded operator
	dcomplex& operator += (const complex& rhs);
	dcomplex& operator -= (const complex& rhs);
	dcomplex& operator *= (const complex& rhs);
	dcomplex& operator /= (const complex& rhs);
};

inline istream& operator >> (istream& is, dcomplex& dc)
{
	complex c;	// cannot use temporary for reference dc !!

	is >> c;	// call operator >> (istream&, complex)
	dc = c;

	return is;
}

	/*---*\
	|  +  |
	\*---*/

inline dcomplex dcomplex::operator + () const {
	return *this;
}
inline dcomplex& dcomplex::operator += (double rhs) {
	re += rhs;
	return *this;
}
inline dcomplex& dcomplex::operator += (const dcomplex& rhs) {
	re += rhs.re;
	im += rhs.im;
	return *this;
}
inline dcomplex& dcomplex::operator += (const complex& rhs) {
	return *this += (const dcomplex&)rhs;
}
inline dcomplex operator + (const dcomplex& lhs, double rhs) {
	dcomplex ret = lhs;
	return ret += rhs;
}
inline dcomplex operator + (double lhs, const dcomplex& rhs) {
	dcomplex ret = rhs;
	return ret += lhs;
}
inline dcomplex operator + (const dcomplex& lhs, const dcomplex& rhs) {
	return dcomplex(lhs.re + rhs.re, lhs.im + rhs.im);
}
inline dcomplex operator + (const dcomplex& lhs, const complex& rhs) {
	return lhs + (const dcomplex&)rhs;
}
inline dcomplex operator + (const complex& lhs, const dcomplex& rhs) {
	return (const dcomplex&)lhs + rhs;
}
inline dcomplex operator + (double lhs, const complex& rhs) {
	return lhs + (const dcomplex&)rhs;
}
inline dcomplex operator + (const complex& lhs, double rhs) {
	return (const dcomplex&)lhs + rhs;
}

	/*---*\
	|  -  |
	\*---*/

inline dcomplex dcomplex::operator - () const {
	return dcomplex(-re, -im);
}
inline dcomplex& dcomplex::operator -= (double rhs) {
	re -= rhs;
	return *this;
}
inline dcomplex& dcomplex::operator -= (const dcomplex& rhs) {
	re -= rhs.re;
	im -= rhs.im;
	return *this;
}
inline dcomplex& dcomplex::operator -= (const complex& rhs) {
	return *this -= (const dcomplex&)rhs;
}
inline dcomplex operator - (const dcomplex& lhs, double rhs) {
	dcomplex ret = lhs;
	return ret -= rhs;
}
inline dcomplex operator - (double lhs, const dcomplex& rhs) {
	dcomplex ret = -rhs;
	return ret += lhs;
}
inline dcomplex operator - (const dcomplex& lhs, const dcomplex& rhs) {
	return dcomplex(lhs.re - rhs.re, lhs.im - rhs.im);
}
inline dcomplex operator - (const dcomplex& lhs, const complex& rhs) {
	return lhs - (const dcomplex&)rhs;
}
inline dcomplex operator - (const complex& lhs, const dcomplex& rhs) {
	return (const dcomplex&)lhs - rhs;
}
inline dcomplex operator - (double lhs, const complex& rhs) {
	return lhs - (const dcomplex&)rhs;
}
inline dcomplex operator - (const complex& lhs, double rhs) {
	return (const dcomplex&)lhs - rhs;
}

	/*---*\
	|  *  |
	\*---*/

inline dcomplex& dcomplex::operator *= (double rhs) {
	re *= rhs;
	im *= rhs;
	return *this;
}
inline dcomplex& dcomplex::operator *= (const dcomplex& rhs) {
	return *this = dcomplex(re*rhs.re - im*rhs.im,
				re*rhs.im + im*rhs.re);
}
inline dcomplex& dcomplex::operator *= (const complex& rhs) {
	return *this *= (const dcomplex&)rhs;
}
inline dcomplex operator * (const dcomplex& lhs, double rhs) {
	dcomplex ret = lhs;
	return ret *= rhs;
}
inline dcomplex operator * (double lhs, const dcomplex& rhs) {
	dcomplex ret = rhs;
	return ret *= lhs;
}
inline dcomplex operator * (const dcomplex& lhs, const dcomplex& rhs) {
	dcomplex ret = lhs;
	return ret *= rhs;
}
inline dcomplex operator * (const dcomplex& lhs, const complex& rhs) {
	return lhs * (const dcomplex&)rhs;
}
inline dcomplex operator * (const complex& lhs, const dcomplex& rhs) {
	return (const dcomplex&)lhs * rhs;
}
inline dcomplex operator * (double lhs, const complex& rhs) {
	return lhs * (const dcomplex&)rhs;
}
inline dcomplex operator * (const complex& lhs, double rhs) {
	return (const dcomplex&)lhs * rhs;
}

	/*---*\
	|  /  |
	\*---*/

inline dcomplex& dcomplex::operator /= (double rhs) {
	re /= rhs;
	im /= rhs;
	return *this;
}
inline dcomplex& dcomplex::operator /= (const dcomplex& rhs) {
	(complex&)(*this) /= (const complex&)rhs;
	return *this;
}
inline dcomplex& dcomplex::operator /= (const complex& rhs) {
	return *this /= (const dcomplex&)rhs;
}
inline dcomplex operator / (const dcomplex& lhs, double rhs) {
	dcomplex ret = lhs;
	return ret /= rhs;
}
inline dcomplex operator / (double lhs, const dcomplex& rhs) {
	dcomplex ret = lhs;
	return ret /= rhs;
}
inline dcomplex operator / (const dcomplex& lhs, const dcomplex& rhs) {
	return (const complex&)lhs / (const complex&)rhs;
}
inline dcomplex operator / (const dcomplex& lhs, const complex& rhs) {
	return lhs / (const dcomplex&)rhs;
}
inline dcomplex operator / (const complex& lhs, const dcomplex& rhs) {
	return (const dcomplex&)lhs / rhs;
}
inline dcomplex operator / (double lhs, const complex& rhs) {
	return lhs / (const dcomplex&)rhs;
}
inline dcomplex operator / (const complex& lhs, double rhs) {
	return (const dcomplex&)lhs / rhs;
}

# endif
#endif
