
////////////////////////////////////////////////////////////////////////
//
//		C++ - Matlab eXternal Engine programming interface
//
//      File    :       mx_engine.h
//
//	Author	:	Copyright (c) 1993,1994 by Bart P.O. van Tongeren
//			Delft University of Technology,
//			Applied Physics Department - Spin Imaging Group,
//			P.O. box 5046, 2600 GA Delft, The Netherlands
//
//	Email	:	tong@si.tn.tudelft.nl
//
//	Date	:	Sat Jun 18 16:03:13 MET DST 1994
//
////////////////////////////////////////////////////////////////////////

#ifndef MX_ENGINE_H
#define MX_ENGINE_H

/*
	setenv DEBUG or DEBUG_MATLAB in environment for debug info on stderr

	setenv MX_ENGINE_STARTCMD in environment to choose your MATLAB engine:
	- unset or empty string	: start "matlab" on local host
	- "hostname"		: start "matlab" on remote host using command:
		"rsh hostname 'setenv DISPLAY hostname:0; matlab'"
	- any other string: executed literally, e.g.:
		"rsh exaflop 'setenv DISPLAY quickx:0; matlab'"
*/

#include <MXE/mx_matrix.h>
#include <strstream.h>
#include <complex.h>
#include <stdlib.h>

typedef struct engine Engine;

//----------------------------------------------------------------------
// class MX_engine - class defining MX_engine Programming Interface
//----------------------------------------------------------------------
class MX_engine
{
public:
	//----------------------------------------------------------------------
	// constructors, destructor
	//----------------------------------------------------------------------
		MX_engine();			// does not start engine yet
		MX_engine(const char* startcmd);// calls open(startcmd)
	virtual	~MX_engine();			// calls close() if open

	//----------------------------------------------------------------------
	// run the MATLAB engine in forked child process
	//----------------------------------------------------------------------
	Engine*	open(const char* startcmd = getenv("MX_ENGINE_STARTCMD"));
	void	close();

public:
	//----------------------------------------------------------------------
	// get/put matrices from/to the MATLAB Engine's workspace
	//----------------------------------------------------------------------
	double		get_double	(const char* name);
	MX_double_1D	get_double_1D	(const char* name);
	MX_double_2D	get_double_2D	(const char* name);

	complex		get_complex	(const char* name);
	MX_complex_1D	get_complex_1D	(const char* name);
	MX_complex_2D	get_complex_2D	(const char* name);

	MX_engine&	put_double	(double x, const char* name);
	MX_engine&	put_double_1D	(const MX_double_1D& M);
	MX_engine&	put_double_2D	(const MX_double_2D& M);

	MX_engine&	put_complex	(complex x, const char* name);
	MX_engine&	put_complex_1D	(const MX_complex_1D& M);
	MX_engine&	put_complex_2D	(const MX_complex_2D& M);

	//----------------------------------------------------------------------
	// send statements or expressions to the MATLAB Engine's stdin
	//----------------------------------------------------------------------
	void		eval		(const char* statement);

	//----------------------------------------------------------------------
	// build statements or expressions using operator <<
	// followed by << eval to send them to MATLAB's Engine's stdin
	//----------------------------------------------------------------------
	MX_engine&	operator <<	(const char* string);
	MX_engine&	operator <<	(double value);
	MX_engine&	operator <<	(complex value);
friend	MX_engine&	eval		(MX_engine& mx_engine);

	//----------------------------------------------------------------------
	// manipulate the MATLAB Engine by the manipulator function
	//----------------------------------------------------------------------
	MX_engine&	operator <<	(MX_engine& (*manipulator) (MX_engine&))
					{
						return manipulator(*this);
					}

	//----------------------------------------------------------------------
	// print the output of MATLAB on the clog stream (i.e. stderr)
	//----------------------------------------------------------------------
	void		log_output(size_t bufsize = 25*80);

private:
	//----------------------------------------------------------------------
	// data members related to MATLAB's external interface library
	//----------------------------------------------------------------------
	Engine*		engine;
	int		is_open;
	ostrstream*	statementstream;
	char*		outputbuffer;
};

//----------------------------------------------------------------------
// manipulator functions
// eval:	send statementstream build by operator << to MATLAB's stdin
// endl:	appends a newline character to the statementstream
// flush:	flush event queue and complete drawing the screen
// example:	MX_engine matlab; matlab.open();
// 		matlab	<< "t = 0:pi/50:10*pi;" << eval;
//		matlab	<< "plot3(sin(t), cos(t), t)" << endl
//			<< "xlabel('" << 50*10 << " time steps')" << flush;
//----------------------------------------------------------------------
	MX_engine&	eval(MX_engine& m);
inline	MX_engine&	endl(MX_engine& m)  { return m << "\n"; }
inline	MX_engine&	flush(MX_engine& m) { return m << "\ndrawnow" << eval; }

#endif
