
////////////////////////////////////////////////////////////////////////
//
//		C++ - Matlab eXternal Engine programming interface
//
//      File    :       mx_matrix.h
//
//			stronly typed interface to MATLAB Matrix structures
//
//	Author	:	Copyright (c) 1993,1994 by Bart P.O. van Tongeren
//			Delft University of Technology,
//			Applied Physics Department - Spin Imaging Group,
//			2628 CJ Delft, The Netherlands
//
//	Email	:	tong@si.tn.tudelft.nl
//
//	Date	:	Sat Jun 18 16:03:12 MET DST 1994
//
////////////////////////////////////////////////////////////////////////

#ifndef MX_MATRIX_H
#define MX_MATRIX_H

#ifndef __STDC__
# define __STDC__
  extern "C" {
# include <MATLAB/matrix.h>
  }
# undef __STDC__
#else
  extern "C" {
# include <MATLAB/matrix.h>
  }
#endif
#include <stddef.h>

/*
	classes defined in this file:

	MX_matrix	MX_string
			MX_full		MX_complex_1D
					MX_complex_2D
					MX_double_1D
					MX_double_2D

	The classes MX_matrix and MX_full are just base classes
	and cannot be instantiated.

	The pointer the Matrix is supposed to be allocated by mxCalloc,
	as it will be freed by mxFreeMatrix in the MX_matrix destructor.

	The MX_engine works with the MX_ matrices. These matrices are
	merely a typesafe interface to the Matrix struct and do not
	have features such as operator overloading, smart copying etc.
	You can make conversions between the MX_ matrices and
	your favourite matrix class library (here named with prefix MM_).
	Then it is possible to derive a class MM_engine from MX_engine,
	that works with your own MM_ matrices.
	The mm_ files demonstrate how this can be done. This mechanism
	is to avoid any dependencies of this C++ - MATLAB interface
	on a particular matrix library.
*/


//----------------------------------------------------------------------
// MX_matrix -- base class, only instances of derived classes are posssible
//----------------------------------------------------------------------
class MX_matrix
{
public:	//------------------------------------------------------
	// interface to mx-functions operating on the matrix
	//------------------------------------------------------
	const	char*	getName() const		  { return mxGetName(mx); }
		void	setName(const char* name) { mxSetName(mx,(char*)name); }

		int	getM() const { return mxGetM(mx); }
		int	getN() const { return mxGetN(mx); }

		int	isComplex() const	{ return mxIsComplex(mx); }
		int	isDouble() const	{ return mxIsDouble(mx); }
		int	isNumeric() const	{ return mxIsNumeric(mx); }
		int	isString() const	{ return mxIsString(mx); }
		int	isFull() const		{ return mxIsFull(mx); }
		int	isSparse() const	{ return mxIsSparse(mx); }

	//------------------------------------------------------
	// access the internal Matrix structure and data arrays
	// take care using these functions
	//------------------------------------------------------
		Matrix*	getMatrix()		{ return mx; }
	const	Matrix*	getMatrix() const	{ return mx; }

	//------------------------------------------------------
	// destructor assumes mx is callocated by mxCalloc
	// constructor is only accessible by derived classes
	//------------------------------------------------------
	virtual		~MX_matrix() { freeMatrix(); }
	void		freeMatrix() { mxFreeMatrix(mx); }

protected:		MX_matrix(Matrix* mx = 0, const char* name = 0);

	//------------------------------------------------------
	// mx is the pointer to the MATLAB Matrix data structure
	//------------------------------------------------------
	void		setMatrix(Matrix* mx_)	{ mx = mx_; }
private:
	Matrix*		mx;
};

//----------------------------------------------------------------------
// MX_string -- string matrix
//----------------------------------------------------------------------
class MX_string : public MX_matrix
{
public:
	MX_string(const MX_string& that);
	MX_string(Matrix* mx, const char* name = 0);
	MX_string(const char* str, const char* name = 0);

	MX_string&	operator = (const MX_string& that);

	//------------------------------------------------------
	// copy string in a new char[]; user should delete[] it
	//------------------------------------------------------
	char*		getString() const;
private:
	MX_string*	dizzy() const { return (MX_string*) this; }
};

//----------------------------------------------------------------------
// MX_full -- base class, only instances of derived classes are posssible
//----------------------------------------------------------------------
class MX_full : public MX_matrix
{
protected:
	MX_full(const MX_full& that);
	MX_full(Matrix* mx, const char* name = 0);
	MX_full(int m, int n, int ComplexFlag, const char* name = 0);

	MX_full&	operator = (const MX_full& that);
public:
	double*		getPr()			{ return mxGetPr(getMatrix()); }
	double*		getPi()			{ return mxGetPi(getMatrix()); }

	const double*	getPr() const		{ return dizzy()->getPr(); }
	const double*	getPi() const		{ return dizzy()->getPi(); }
private:
	MX_full*	dizzy() const { return (MX_full*) this; }
};

//----------------------------------------------------------------------
// MX_double_1D -- 1 dimensional full matrix of double elements
//----------------------------------------------------------------------
class MX_double_1D : public MX_full
{
public:
	MX_double_1D(Matrix* mx, const char* name = 0);
	MX_double_1D(size_t m, const char* name = 0);
	MX_double_1D(size_t m, const double pr[],
			       const char* name = 0);

	double	getScalar() const { return mxGetScalar(dizzy()->getMatrix()); }
private:
	MX_double_1D*	dizzy() const { return (MX_double_1D*) this; }
};

//----------------------------------------------------------------------
// MX_double_2D -- 2 dimensional full matrix of double elements
//----------------------------------------------------------------------
class MX_double_2D : public MX_full
{
public:
	MX_double_2D(Matrix* mx, const char* name = 0);
	MX_double_2D(size_t m, size_t n, const char* name = 0);
	MX_double_2D(size_t m, size_t n, const double pr[],
					 const char* name = 0);

	double	getScalar() const { return mxGetScalar(dizzy()->getMatrix()); }
private:
	MX_double_2D*	dizzy() const { return (MX_double_2D*) this; }
};

//----------------------------------------------------------------------
// MX_complex_1D -- 1 dimensional full matrix of complex elements
//----------------------------------------------------------------------
class MX_complex_1D : public MX_full
{
public:
	MX_complex_1D(Matrix* mx, const char* name = 0);
	MX_complex_1D(size_t m, const char* name = 0);
	MX_complex_1D(size_t m, const double pr[],
				const double pi[],
				const char* name = 0);
private:
	MX_complex_1D*	dizzy() const { return (MX_complex_1D*) this; }
};

//----------------------------------------------------------------------
// MX_complex_2D -- 2 dimensional full matrix of complex elements
//----------------------------------------------------------------------
class MX_complex_2D : public MX_full
{
public:
	MX_complex_2D(Matrix* mx, const char* name = 0);
	MX_complex_2D(size_t m, size_t n, const char* name = 0);
	MX_complex_2D(size_t m, size_t n, const double pr[],
					  const double pi[],
					  const char* name = 0);
private:
	MX_complex_2D*	dizzy() const { return (MX_complex_2D*) this; }
};

#endif
