
////////////////////////////////////////////////////////////////////////
//
//		C++ - Matlab eXternal Engine programming interface
//
//      File    :       mm_engine.cc
//
//	MM_engine is a specialized MX_engine tailored to MM matrices
//
//	Author	:	Copyright (c) 1993,1994 by Bart P.O. van Tongeren
//			Delft University of Technology,
//			Applied Physics Department - Spin Imaging Group,
//			P.O.box 5046, 2600 GA Delft, The Netherlands
//
//	Email	:	tong@si.tn.tudelft.nl
//
//	Date	:	Sat Jun 18 16:03:09 MET DST 1994
//
////////////////////////////////////////////////////////////////////////

#include <MXE/mm_engine.h>
#include <MXE/mm_matrix.h>
#include <new.h>


//----------------------------------------------------------------------
// initialize the predefined global MM_engine 'matlab'
//----------------------------------------------------------------------
MM_engine	matlab = matlab;	   // global MM_engine 'matlab'
int		MM_engine_init::count = 0; // static MM_engine_init data member

MM_engine_init::MM_engine_init()
{
	if (count++ == 0)	// start engine of predefined MM_engine object
	{
		new (&matlab) MM_engine;
		matlab.open();
	}
}

MM_engine_init::~MM_engine_init()
{
	if (--count == 0)
		;		// matlab's destructor will call close()
}

//----------------------------------------------------------------------
// FUNCTION:    get_double
// PURPOSE:     transfer a MATLAB scalar to a C++ double
// ARGUMENTS:   name -- name of MATLAB variable
// RETURN:      C++ double
//----------------------------------------------------------------------
double
MM_engine::get_double(const char* name)
{
	return MX_engine::get_double(name);
}

//----------------------------------------------------------------------
// FUNCTION:    get_double_1D
// PURPOSE:     transfer a MATLAB matrix variable to a C++ MM_double_1D
// ARGUMENTS:   name -- name of MATLAB matrix variable
// RETURN:      C++ MM_double_1D matrix
//----------------------------------------------------------------------
MM_double_1D
MM_engine::get_double_1D(const char* name)
{
	MX_double_1D mx_double_1d = MX_engine::get_double_1D(name);

	return MM_double_1D_MX(mx_double_1d);
}

//----------------------------------------------------------------------
// FUNCTION:    get_double_2D
// PURPOSE:     transfer a MATLAB matrix variable to a C++ MM_double_2D
// ARGUMENTS:   name -- name of MATLAB matrix variable
// RETURN:      C++ MM_double_2D matrix
//----------------------------------------------------------------------
MM_double_2D
MM_engine::get_double_2D(const char* name)
{
	MX_double_2D mx_double_2d = MX_engine::get_double_2D(name);

	return MM_double_2D_MX(mx_double_2d);
}

//----------------------------------------------------------------------
// FUNCTION:    get_dcomplex
// PURPOSE:     transfer a MATLAB scalar to a C++ dcomplex
// ARGUMENTS:   name -- name of MATLAB variable
// RETURN:      C++ dcomplex
//----------------------------------------------------------------------
dcomplex
MM_engine::get_dcomplex(const char* name)
{
	complex r = MX_engine::get_complex(name);
	return r;
}

//----------------------------------------------------------------------
// FUNCTION:    get_dcomplex_1D
// PURPOSE:     transfer a MATLAB matrix variable to a C++ MM_dcomplex_1D
// ARGUMENTS:   name -- name of MATLAB matrix variable
// RETURN:      C++ MM_dcomplex_1D matrix
//----------------------------------------------------------------------
MM_dcomplex_1D
MM_engine::get_dcomplex_1D(const char* name)
{
	MX_complex_1D mx_complex_1d = MX_engine::get_complex_1D(name);

	return MM_dcomplex_1D_MX(mx_complex_1d);
}

//----------------------------------------------------------------------
// FUNCTION:    get_dcomplex_2D
// PURPOSE:     transfer a MATLAB matrix variable to a C++ MM_dcomplex_2D
// ARGUMENTS:   name -- name of MATLAB matrix variable
// RETURN:      C++ MM_dcomplex_2D matrix
//----------------------------------------------------------------------
MM_dcomplex_2D
MM_engine::get_dcomplex_2D(const char* name)
{
	MX_complex_2D mx_complex_2d = MX_engine::get_complex_2D(name);

	return MM_dcomplex_2D_MX(mx_complex_2d);
}

//----------------------------------------------------------------------
// FUNCTION:    put_double
// PURPOSE:     transfer C++ double to a MATLAB scalar
// ARGUMENTS:   name -- name of MATLAB variable
// ARGUMENTS:	x -- C++ double object
//		name -- name of MATLAB variable
//----------------------------------------------------------------------
MM_engine&
MM_engine::put_double(const double x, const char* name)
{
	return (MM_engine&)MX_engine::put_double(x, name);
}

//----------------------------------------------------------------------
// FUNCTION:    put_double_1D
// PURPOSE:     transfer a C++ MM_double_1D to a MATLAB column vector
// ARGUMENTS:   M -- C++ MM_double_1D object
//		name -- name of MATLAB matrix variable
//----------------------------------------------------------------------
MM_engine&
MM_engine::put_double_1D(const MM_double_1D& M, const char* name)
{
	MM_double_1D_MX		mm_double_1d_mx = M;

	MX_engine::put_double_1D(mm_double_1d_mx.make_mx(name));
	return *this;
}

//----------------------------------------------------------------------
// FUNCTION:    put_double_2D
// PURPOSE:     transfer a C++ MM_double_2D to a MATLAB matrix
// ARGUMENTS:   M -- C++ MM_double_2D object
//		name -- name of MATLAB matrix variable
//----------------------------------------------------------------------
MM_engine&
MM_engine::put_double_2D(const MM_double_2D& M, const char* name)
{
	MM_double_2D_MX		mm_double_2d_mx = M;

	MX_engine::put_double_2D(mm_double_2d_mx.make_mx(name));
	return *this;
}

//----------------------------------------------------------------------
// FUNCTION:    put_dcomplex
// PURPOSE:     transfer C++ dcomplex to a MATLAB scalar
// ARGUMENTS:   name -- name of MATLAB variable
// ARGUMENTS:	x -- C++ dcomplex object
//		name -- name of MATLAB variable
//----------------------------------------------------------------------
MM_engine&
MM_engine::put_dcomplex(const dcomplex x, const char* name)
{
	return (MM_engine&)MX_engine::put_complex(x, name);
}

//----------------------------------------------------------------------
// FUNCTION:    put_dcomplex_1D
// PURPOSE:     transfer a C++ MM_dcomplex_1D to a MATLAB column vector
// ARGUMENTS:   M -- C++ MM_dcomplex_1D object
//		name -- name of MATLAB matrix variable
//----------------------------------------------------------------------
MM_engine&
MM_engine::put_dcomplex_1D(const MM_dcomplex_1D& M, const char* name)
{
	MM_dcomplex_1D_MX	mm_dcomplex_1d_mx = M;

	MX_engine::put_complex_1D(mm_dcomplex_1d_mx.make_mx(name));
	return *this;
}

//----------------------------------------------------------------------
// FUNCTION:    put_dcomplex_2D
// PURPOSE:     transfer a C++ MM_dcomplex_2D to a MATLAB matrix
// ARGUMENTS:   M -- C++ MM_dcomplex_2D object
//		name -- name of MATLAB matrix variable
//----------------------------------------------------------------------
MM_engine&
MM_engine::put_dcomplex_2D(const MM_dcomplex_2D& M, const char* name)
{
	MM_dcomplex_2D_MX	mm_dcomplex_2d_mx = M;

	MX_engine::put_complex_2D(mm_dcomplex_2d_mx.make_mx(name));
	return *this;
}
