
////////////////////////////////////////////////////////////////////////
//
//		C++ - Matlab eXternal Engine programming interface
//
//      File    :       mx_engine.cc
//
//	Author	:	Copyright (c) 1993,1994 by Bart P.O. van Tongeren
//			Delft University of Technology,
//			Applied Physics Department - Spin Imaging Group,
//			P.O. box 5046, 2600 GA Delft, The Netherlands
//
//	Email	:	tong@si.tn.tudelft.nl
//
//	Date	:	Sat Jun 18 16:03:07 MET DST 1994
//
////////////////////////////////////////////////////////////////////////

#include <MXE/mx_engine.h>
#include <strstream.h>
#include <iomanip.h>
#include <string.h>
#include <stdlib.h>
#include <sysent.h>
#include <signal.h>
#include <errno.h>
#include <ctype.h>
#include <float.h>

#ifndef __STDC__
# define __STDC__
  extern "C" {
# include <MATLAB/engine.h>
  }
# undef __STDC__
#else
  extern "C" {
# include <MATLAB/engine.h>
  }
#endif

inline int debug() {
	return ((getenv("DEBUG") != 0) || (getenv("DEBUG_MATLAB") != 0));
}


//------------------------------------------------------
// start and stop the MATLAB engine
//------------------------------------------------------
Engine*
MX_engine::open(const char* startcmd)
{
	if (debug())
		cerr << "MX_engine::open(\"" << startcmd << "\");\n";

	if (is_open)
	{
		cerr	<< "MX_engine::open(\"" << startcmd << "\"): "
			<< "MATLAB engine is already opened\n";
		exit(1);
	}

	engine = engOpen((char*)startcmd);

	if (engine == 0)
	{
		cerr	<< "MX_engine::open(\"" << startcmd << "\"): "
			<< "could not start MATLAB engine\n";
		exit(1);
	}

	log_output();	// enable printing of MATLAB output on stderr

	eval("%% starting engine ...");	// will exit if unsuccessful

	is_open = 1;	// is opened

	return engine;
}

void
MX_engine::close()
{
	if (debug())
		cerr << "MX_engine::close();\n";

	if (!is_open)
	{
		cerr	<< "MX_engine::close(): "
			<< "MATLAB engine is not opened\n";
		exit(1);
	}

	int exitcode = engClose(engine);

	if (exitcode != 0)	// error occured
	{
		cerr	<< "MX_engine::close(): "
			<< "could not close MATLAB engine\n";
		exit(1);
	}

	is_open = 0;	// MATLAB stopped
}


//------------------------------------------------------
// MX_engine constructors and destructor
//------------------------------------------------------
MX_engine::MX_engine()
:	is_open(0), statementstream(0), outputbuffer(0)
{
	if (debug())
		cerr << "MX_engine::MX_engine()\n";

	// do not call open() yet
}

MX_engine::MX_engine(const char* startcmd)
:	is_open(0), statementstream(0), outputbuffer(0)
{
	if (debug())
		cerr << "MX_engine::MX_engine(char*)\n";

	open(startcmd);
}

MX_engine::~MX_engine()
{
	if (debug())
		cerr << "MX_engine::~MX_engine()\n";

	if (is_open)
		close();
	delete statementstream;
	delete outputbuffer;
}

//----------------------------------------------------------------------
// print the output of MATLAB on the clog stream (i.e. stderr)
//----------------------------------------------------------------------
void
MX_engine::log_output(size_t bufsize)
{
	delete outputbuffer;

	outputbuffer = bufsize ? new char[bufsize] : 0;

	engOutputBuffer(engine, outputbuffer, bufsize);
}

//----------------------------------------------------------------------
// FUNCTION:	eval member function
// PURPOSE:	send a statement or expression to stdin of MATLAB process;
//		send MATLAB's output via outputbuffer to clog
//----------------------------------------------------------------------
void
MX_engine::eval(const char* statement)
{
	if (debug())
		cerr << "MX_engine::eval(\"" << statement << "\");\n";

	// check for errors in execution
	if (engEvalString(engine, (char*)statement) != 0)
	{
		cerr << "MX_engine::eval(\"" << statement << "\") failed\n";
		exit(1);
	}

	if (outputbuffer)
		clog << outputbuffer << flush;
}

//----------------------------------------------------------------------
// FUNCTION:	operator <<
// PURPOSE:	append (part of) a statement or expression to statementstream
//----------------------------------------------------------------------
MX_engine&
MX_engine::operator << (const char* string)
{
	if (debug())
		cerr << "MX_engine::operator << (\"" << string << "\");\n";

	if (statementstream == 0)
		statementstream = new ostrstream;

	*statementstream << string;

	return *this;
}

MX_engine&
MX_engine::operator << (double value)
{
	if (debug())
		cerr << "MX_engine::operator << (" << value << ");\n";

	if (statementstream == 0)
		statementstream = new ostrstream;

	*statementstream << value;

	return *this;
}

MX_engine&
MX_engine::operator << (complex value)
{
	if (debug())
		cerr << "MX_engine::operator << (" << value << ");\n";

	if (statementstream == 0)
		statementstream = new ostrstream;

	*statementstream << "(" << real(value) << "+"
				<< imag(value) << "*sqrt(-1))";

	return *this;
}

//----------------------------------------------------------------------
// FUNCTION:	eval manipulator
// PURPOSE:	send statementstream build by operator <<  to stdin of
//		MATLAB process;
//		send MATLAB's output via outputbuffer to clog
//----------------------------------------------------------------------
MX_engine&	eval(MX_engine& m)
{
	if (m.statementstream != 0)
	{
		*m.statementstream << ends;
		m.eval(m.statementstream->str());	// freeze embedded str
		delete [] m.statementstream->str();	// delete embedded str
		delete m.statementstream;		// delete ostrstream
		m.statementstream = 0;
	}
	return m;
}

//----------------------------------------------------------------------
// FUNCTION:    get_complex
// PURPOSE:     transfer a MATLAB 1*1 matrix variable to a C++ complex
// ARGUMENTS:   name -- name of MATLAB matrix variable
// RETURN:      C++ complex
//----------------------------------------------------------------------
complex
MX_engine::get_complex(const char* name)
{
	if (debug())
		cerr << "MX_engine::get_complex(\"" << name << "\");\n";

	Matrix	*matrix = engGetMatrix(engine, (char*)name);

	if (matrix == 0)
	{
		cerr	<< "MX_engine::get_complex(\"" << name << "\"): "
			<< "unsuccessful engGetMatrix(...) call\n";
		exit(1);
	}

	MX_complex_1D	mx = matrix;

	if (mx.getM() * mx.getN() > 1 || !mx.isNumeric())
	{
		cerr	<< "MX_engine::get_complex(\"" << name << "\"): "
			<< "not a number\n";
		exit(1);
	}

	complex x = complex(*mx.getPr(), mx.isComplex() ? *mx.getPi() : 0);

	return x;
}

//----------------------------------------------------------------------
// FUNCTION:    get_double
// PURPOSE:     transfer a MATLAB 1*1 matrix variable to a C++ double
// ARGUMENTS:   name -- name of MATLAB matrix variable
// RETURN:      C++ double
//----------------------------------------------------------------------
double
MX_engine::get_double(const char* name)
{
	if (debug())
		cerr << "MX_engine::get_double(\"" << name << "\");\n";

	Matrix	*matrix = engGetMatrix(engine, (char*)name);

	if (matrix == 0)
	{
		cerr	<< "MX_engine::get_double(\"" << name << "\"): "
			<< "unsuccessful engGetMatrix(...) call\n";
		exit(1);
	}

	MX_double_1D	mx = matrix;

	if (mx.getM() * mx.getN() > 1)
	{
		cerr	<< "MX_engine::get_double(\"" << name << "\"): "
			<< "not a number\n";
		exit(1);
	}

	if (!(mx.isDouble()))
	{
		cerr	<< "MX_engine::get_double(\"" << name << "\"): "
			<< "not a double\n";
		exit(1);
	}

	double x = *mx.getPr();

	return x;
}

//----------------------------------------------------------------------
// FUNCTION:    get_complex_1D
// PURPOSE:     transfer a MATLAB matrix variable to a C++ MX_complex_1D
// ARGUMENTS:   name -- name of MATLAB matrix variable
// RETURN:      C++ MX_complex_1D matrix
//----------------------------------------------------------------------
MX_complex_1D
MX_engine::get_complex_1D(const char* name)
{
	if (debug())
		cerr << "MX_engine::get_complex_1D(\"" << name << "\");\n";

	Matrix	*matrix = engGetMatrix(engine, (char*)name);

	if (matrix == 0)
	{
		cerr	<< "MX_engine::get_complex_1D(\"" << name << "\"): "
			<< "unsuccessful engGetMatrix(...) call\n";
		exit(1);
	}

	return MX_complex_1D(matrix);
}

//----------------------------------------------------------------------
// FUNCTION:    get_double_1D
// PURPOSE:     transfer a MATLAB matrix variable to a C++ MX_double_1D
// ARGUMENTS:   name -- name of MATLAB matrix variable
// RETURN:      C++ MX_double_1D matrix
//----------------------------------------------------------------------
MX_double_1D
MX_engine::get_double_1D(const char* name)
{
	if (debug())
		cerr << "MX_engine::get_double_1D(\"" << name << "\");\n";

	Matrix	*matrix = engGetMatrix(engine, (char*)name);

	if (matrix == 0)
	{
		cerr	<< "MX_engine::get_double_1D(\"" << name << "\"): "
			<< "unsuccessful engGetMatrix(...) call\n";
		exit(1);
	}

	return MX_double_1D(matrix);
}

//----------------------------------------------------------------------
// FUNCTION:    get_complex_2D
// PURPOSE:     transfer a MATLAB matrix variable to a C++ MX_complex_2D
// ARGUMENTS:   name -- name of MATLAB matrix variable
// RETURN:      C++ MX_complex_2D matrix
//----------------------------------------------------------------------
MX_complex_2D
MX_engine::get_complex_2D(const char* name)
{
	if (debug())
		cerr << "MX_engine::get_complex_2D(\"" << name << "\");\n";

	Matrix	*matrix = engGetMatrix(engine, (char*)name);

	if (matrix == 0)
	{
		cerr	<< "MX_engine::get_complex_2D(\"" << name << "\"): "
			<< "unsuccessful engGetMatrix(...) call\n";
		exit(1);
	}

	return MX_complex_2D(matrix);
}

//----------------------------------------------------------------------
// FUNCTION:    get_double_2D
// PURPOSE:     transfer a MATLAB matrix variable to a C++ MX_double_2D
// ARGUMENTS:   name -- name of MATLAB matrix variable
// RETURN:      C++ MX_double_2D matrix
//----------------------------------------------------------------------
MX_double_2D
MX_engine::get_double_2D(const char* name)
{
	if (debug())
		cerr << "MX_engine::get_double_2D(\"" << name << "\");\n";

	Matrix	*matrix = engGetMatrix(engine, (char*)name);

	if (matrix == 0)
	{
		cerr	<< "MX_engine::get_double_2D(\"" << name << "\"): "
			<< "unsuccessful engGetMatrix(...) call\n";
		exit(1);
	}

	return MX_double_2D(matrix);
}

//----------------------------------------------------------------------
// FUNCTION:    put_complex
// PURPOSE:     transfer a C++ complex to a MATLAB matrix variable
// ARGUMENTS:   x -- C++ complex object
//              name -- name of MATLAB variable
//----------------------------------------------------------------------
MX_engine&
MX_engine::put_complex(complex x, const char* name)
{
    if (debug())
    	cerr << "MX_engine::put_complex(\"" << name << "\");\n";

    ostrstream s;

    s	<< setprecision(DBL_DIG)
    	<< name << " = (" << real(x) << ")+(" << imag(x) << ")*sqrt(-1);"
	<< ends;

    eval(s.str());

    delete [] s.str();

    return *this;
}

//----------------------------------------------------------------------
// FUNCTION:    put_double
// PURPOSE:     transfer a C++ double to a MATLAB matrix variable
// ARGUMENTS:   x -- C++ double object
//              name -- name of MATLAB variable
//----------------------------------------------------------------------
MX_engine&
MX_engine::put_double(double x, const char* name)
{
    if (debug())
    	cerr << "MX_engine::put_double(\"" << name << "\");\n";

    ostrstream s;

    s	<< setprecision(DBL_DIG)
	<< name << " = " << x << ";" << ends;

    eval(s.str());

    delete [] s.str();

    return *this;
}

//----------------------------------------------------------------------
// FUNCTION:    put_complex_1D
// PURPOSE:     transfer a C++ MX_complex_1D to a MATLAB column vector
// ARGUMENTS:   M -- C++ MX_complex_1D object
//----------------------------------------------------------------------
MX_engine&
MX_engine::put_complex_1D(const MX_complex_1D& M)
{
	if (debug())
		cerr	<< "MX_engine::put_complex_1D(["
			<< M.getM() << " x "
			<< M.getN() << "], \""
			<< M.getName() << "\");\n";

	Matrix	*matrix = (Matrix*)M.getMatrix();

	if (engPutMatrix(engine, matrix) != 0)
	{
		cerr	<< "MX_engine::put_complex_1D(\"" << M.getName() << "\"): "
			<< "unsuccessful engPutMatrix(...) call\n";
		exit(1);
	}

	return *this;
}

//----------------------------------------------------------------------
// FUNCTION:    put_double_1D
// PURPOSE:     transfer a C++ MX_double_1D to a MATLAB column vector
// ARGUMENTS:   M -- C++ MX_double_1D object
//----------------------------------------------------------------------
MX_engine&
MX_engine::put_double_1D(const MX_double_1D& M)
{
	if (debug())
		cerr	<< "MX_engine::put_double_1D(["
			<< M.getM() << " x "
			<< M.getN() << "], \""
			<< M.getName() << "\");\n";

	Matrix	*matrix = (Matrix*)M.getMatrix();

	if (engPutMatrix(engine, matrix) != 0)
	{
		cerr	<< "MX_engine::put_double_1D(\"" << M.getName() << "\"): "
			<< "unsuccessful engPutMatrix(...) call\n";
		exit(1);
	}

	return *this;
}

//----------------------------------------------------------------------
// FUNCTION:    put_complex_2D
// PURPOSE:     transfer a C++ MX_complex_2D to a MATLAB matrix
// ARGUMENTS:   M -- C++ MX_complex_2D object
//----------------------------------------------------------------------
MX_engine&
MX_engine::put_complex_2D(const MX_complex_2D& M)
{
	if (debug())
		cerr	<< "MX_engine::put_complex_2D(["
			<< M.getM() << " x "
			<< M.getN() << "], \""
			<< M.getName() << "\");\n";

	Matrix	*matrix = (Matrix*)M.getMatrix();

	if (engPutMatrix(engine, matrix) != 0)
	{
		cerr	<< "MX_engine::put_complex_2D(\"" << M.getName() << "\"): "
			<< "unsuccessful engPutMatrix(...) call\n";
		exit(1);
	}

	return *this;
}

//----------------------------------------------------------------------
// FUNCTION:    put_doublex_2D
// PURPOSE:     transfer a C++ MX_double_2D to a MATLAB matrix
// ARGUMENTS:   M -- C++ MX_double_2D object
//----------------------------------------------------------------------
MX_engine&
MX_engine::put_double_2D(const MX_double_2D& M)
{
	if (debug())
		cerr	<< "MX_engine::put_double_2D(["
			<< M.getM() << " x "
			<< M.getN() << "], \""
			<< M.getName() << "\");\n";

	Matrix	*matrix = (Matrix*)M.getMatrix();

	if (engPutMatrix(engine, matrix) != 0)
	{
		cerr	<< "MX_engine::put_double_2D(\"" << M.getName() << "\"): "
			<< "unsuccessful engPutMatrix(...) call\n";
		exit(1);
	}

	return *this;
}
