
////////////////////////////////////////////////////////////////////////
//
//		C++ - Matlab eXternal Engine programming interface
//
//      File    :       mx_example.cc
//
//	Author	:	Copyright (c) 1993,1994 by Bart P.O. van Tongeren
//			Delft University of Technology,
//			Applied Physics Department - Spin Imaging Group,
//			P.O. box 5046, 2600 GA Delft, The Netherlands
//
//	Email	:	tong@si.tn.tudelft.nl
//
//	Date	:	Sat Jun 18 16:03:10 MET DST 1994
//
//	Note	:	This example (but for MATLAB 3.5) is described in
//
//	B.P.O. van Tongeren, e.a.,
//	"Embedding of PRO-MATLAB into NMR Signal Processing Applications",
//	Journal of Magnetic Resonance, 100/3, dec. 1992, p. 619-624.
//
////////////////////////////////////////////////////////////////////////

#include <MXE/mx_engine.h>

MX_engine	matlab;		// the MATLAB engine

int main(int argc, char* argv[])
{
	if (argc == 1)		// start the engine
		matlab.open();
	else	matlab.open(argv[1]);

	// record what happens
	matlab	<< "diary " << argv[0] << ".diary" << eval;

	const int M = 32;	// size of 2D model function X
	const int N = 32;	//	i.e. the number of time domain samples
	const int K = 3;	// number of poles in k direction
	const int L = 2;	// number of poles in l direction
	const int KL = K*L;

	// Pk(K),	// complex poles, where
	// Pl(L);	//	realpart=damping, imagpart=frequency
	// C(K,L);	// complex amplitude matrix

	// insert complex values into the pole matrices
	static double	Pk_real[K] = { -0.04, -0.02, -0.06 };
	static double	Pk_imag[K] = { -2,     1,     2 };
	static double	Pl_real[L] = { -0.02, -0.06 };
	static double	Pl_imag[L] = { -2,     1 };

	// let all peaks have the same amplitude and no phase shift
	// C = 1;
	static double	C_real[KL] = { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 },
			C_imag[KL] = { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };

	MX_complex_1D	Pk(K, Pk_real, Pk_imag, "Pk"),
			Pl(L, Pl_real, Pl_imag, "Pl");

	MX_complex_2D	C(K, L, C_real, C_imag, "C");

	// transport the matrices to the matlab process
	matlab	.put_complex_1D(Pk)
		.put_complex_1D(Pl)
		.put_complex_2D(C)
		.put_double(M, "M")
		.put_double(N, "N");

	// compute time domain reconstruction in matlab; make a surface plot
	matlab	<< "Zk = exp(Pk).';"
		<< "Zl = exp(Pl);"

		<< "U = ones(size(Zk));"

		<< "for k = 2:M;"
		<< "	U(k,:) = U(k-1,:) .* Zk;"
		<< "end"
		<< eval

		<< "V = ones(size(Zl));"
		<< "for l = 2:N;"
		<< "	V(:,l) = V(:,l-1) .* Zl;"
		<< "end"
		<< eval

		<< "X = U * C * V;"

		<< "surfl(abs(fftshift(fft2(X))));"
		<< "view(-30, 10);"
		<< "title('modulus fft');"
		<< flush
		;

	// return the matrix X from matlab to C++ for further processing
	MX_complex_2D	X = matlab.get_complex_2D("X");

	// for later inspection
	matlab	<< "save " << argv[0] << eval;

	cout	<< "Hit return to quit.." << flush;
	cin.get();

	return 0;
}
