
////////////////////////////////////////////////////////////////////////
//
//		C++ - Matlab eXternal Engine programming interface
//
//      File    :       mx_matrix.cc
//
//			stronly typed interface to MATLAB Matrix structures
//
//	Author	:	Copyright (c) 1993,1994 by Bart P.O. van Tongeren
//			Delft University of Technology,
//			Applied Physics Department - Spin Imaging Group,
//			P.O.box 5046, 2600 GA Delft, The Netherlands
//
//	Email	:	tong@si.tn.tudelft.nl
//
//	Date	:	Sat Jun 18 16:03:06 MET DST 1994
//
////////////////////////////////////////////////////////////////////////

#include <MXE/mx_matrix.h>
#include <string.h>
#include <assert.h>

//----------------------------------------------------------------------
// MX_matrix -- base class, only instances of derived classes are posssible
//----------------------------------------------------------------------
MX_matrix::MX_matrix(Matrix* mx, const char* name) {
	setMatrix(mx);
	if (name) setName(name);
}

//----------------------------------------------------------------------
// MX_string -- string matrix
//----------------------------------------------------------------------
MX_string::MX_string(const MX_string& that) {
	char*	str = that.getString();

	setMatrix(mxCreateString(str));
	delete [] str;
	setName(that.getName());
}

MX_string::MX_string(Matrix* mx, const char* name)
:	MX_matrix(mx, name) {
	assert(isString());
}

MX_string::MX_string(const char* str, const char* name) {
	setMatrix(mxCreateString((char*)str));
	setName(name);
}

MX_string&
MX_string::operator = (const MX_string& that) {
	if (this == &that)
		return *this;

	char	*str  = that.getString(),
		*name = new char[strlen(getName())+1];

	strcpy(name, getName());
	freeMatrix();
	setMatrix(mxCreateString(str));
	setName(name);
	delete [] name;
	delete [] str;
	return *this;
}

char*
MX_string::getString() const {
	int	strlen = getN() * getM() + 1;
	char	*str   = new char[strlen];

	mxGetString(dizzy()->getMatrix(), str, strlen);
	return str;
}

//----------------------------------------------------------------------
// MX_full -- base class, only instances of derived classes are posssible
//----------------------------------------------------------------------
MX_full::MX_full(const MX_full& that) {
	setMatrix(mxCreateFull(that.getM(), that.getN(), that.isComplex()));
	memcpy(getPr(), that.getPr(), getM()*getN()*sizeof(double));
	if (that.isComplex())
		memcpy(getPi(), that.getPi(), getM()*getN()*sizeof(double));
	setName(that.getName());
}

MX_full::MX_full(Matrix* mx, const char* name)
:	MX_matrix(mx, name) {
	assert(isFull());
}

MX_full::MX_full(int m, int n, int ComplexFlag, const char* name) {
	setMatrix(mxCreateFull(m, n, ComplexFlag));
	setName(name);
}

MX_full&
MX_full::operator = (const MX_full& that) {
	if (this == &that)
		return *this;

	char	*name = new char[strlen(getName())+1];

	strcpy(name, getName());
	freeMatrix();
	setMatrix(mxCreateFull(that.getM(), that.getN(), that.isComplex()));
	memcpy(getPr(), that.getPr(), getM()*getN()*sizeof(double));
	if (that.isComplex())
		memcpy(getPi(), that.getPi(), getM()*getN()*sizeof(double));
	setName(name);
	delete [] name;
	return *this;
}

//----------------------------------------------------------------------
// MX_double_1D -- 1 dimensional full matrix of double elements
//----------------------------------------------------------------------
MX_double_1D::MX_double_1D(Matrix* mx, const char* name)
:	MX_full(mx, name) {
	assert( isDouble() && (getM() == 1 || getN() == 1));
}

MX_double_1D::MX_double_1D(size_t m, const char* name)
:	MX_full(m, 1, 0, name) { }

MX_double_1D::MX_double_1D(size_t m, const double pr[], const char* name)
:	MX_full(m, 1, 0, name) {
	memcpy(getPr(), pr, m*sizeof(double));
}

//----------------------------------------------------------------------
// MX_double_2D -- 2 dimensional full matrix of double elements
//----------------------------------------------------------------------
MX_double_2D::MX_double_2D(Matrix* mx, const char* name)
:	MX_full(mx, name) {
	assert(isDouble());
}

MX_double_2D::MX_double_2D(size_t m, size_t n, const char* name)
:	MX_full(m, n, 0, name) { }

MX_double_2D::MX_double_2D(size_t m, size_t n,	const double pr[],
						const char* name)
:	MX_full(m, n, 0, name) {
	memcpy(getPr(), pr, m*n*sizeof(double));
}

//----------------------------------------------------------------------
// MX_complex_1D -- 1 dimensional full matrix of complex elements
//----------------------------------------------------------------------
MX_complex_1D::MX_complex_1D(Matrix* mx, const char* name)
:	MX_full(mx, name) {
	assert(isNumeric() && (getM() == 1 || getN() == 1));
}

MX_complex_1D::MX_complex_1D(size_t m, const char* name)
:	MX_full(m, 1, 1, name) { }

MX_complex_1D::MX_complex_1D(size_t m,	const double pr[],
					const double pi[],
					const char* name)
:	MX_full(m, 1, 1, name) {
	memcpy(getPr(), pr, m*sizeof(double));
	memcpy(getPi(), pi, m*sizeof(double));
}

//----------------------------------------------------------------------
// MX_complex_2D -- 2 dimensional full matrix of complex elements
//----------------------------------------------------------------------
MX_complex_2D::MX_complex_2D(Matrix* mx, const char* name)
:	MX_full(mx, name) {
	assert(isNumeric());
}

MX_complex_2D::MX_complex_2D(size_t m, size_t n, const char* name)
:	MX_full(m, n, 1, name) { }

MX_complex_2D::MX_complex_2D(size_t m, size_t n,	const double pr[],
							const double pi[],
							const char* name)
:	MX_full(m, n, 1, name) {
	memcpy(getPr(), pr, m*n*sizeof(double));
	memcpy(getPi(), pi, m*n*sizeof(double));
}
