function mxx2mat(sbfile,slfile,opt)
%MXX2MAT translates ASCII MATRIXx Data files into MATLAB data file.
%
%       MXX2MAT('MXXFILE') translates the MATRIXx data file MXXFILE into 
%	MATLAB data file with the file name same as MXXFILE and extension mat.
%
%       MXX2MAT('MXXFILE','MATFILE') translates the SystemBuild file MXXFILE
%       into a MATLAB data file MATFILE.MAT. The extension of MATFILE will
%       have extension .mat no matter what extension has been specified.
%
%       MXX2MAT('MXXFILE','MATFILE',OPT) translates the SystemBuild file
%       MXXFILE into a MATLAB data file MATFILE.MAT with some options
%	specified in OPT. When OPT='up', all variable names saved in
%	upper case (defualt). When OPT='low', all variable names saved
%	in lower case. When OPT='up/low',all variable names are lower case
%	with the first character in upper case.
%
%	Run LOAD SLFILE to load data if there is any data saved in the 
%	SystemBuild file and/or in the conversion.
%
%	This function calls a function appsave. If appsave is not in your 
%	path, you may use opensb to add the file into your path. Or, copy
%	the file into your working directory.

%	Wes Wang 6/22/93
%	Copyright (c) 1990-94 by The MathWorks, Inc.
%	$Revision: 1.2 $  $Date: 1994/06/28 17:10:15 $

% This function is really lengthy; however, the data flow and functionality
% in each section are clear. The reason for not dividing this function
% into smaller functions is to avoid passing too many variables
% between functions and to avoid leaving a file open between functions.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Routine check up and make up for input variables                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check if at least one parameter exists. If not, give error message.
err = 0; y=[];

if ~exist('appsave')
    disp('You may use OPENSB to translate a SystemBuild file to SIMULINK');
    eval('opensb;');
    if ~exist('appsave')
	error('You have to make file APPSAVE available in your path');
    end;    
end;

% if there is no parameter input, error
if nargin <1,
  error('Usage: DATAMXX SystemBuildFile');
end;

% if file to be saved is not specified
save_flag = 1;
if nargin <2 
  slfile = sbfile;
  save_flag = 0;
end;

% upper or lower case processing.
case_save = 0;
if nargin > 2
  if findstr(opt,'low')
    case_save = 1;
    if findstr(opt,'up')
      case_save = 2;
    end;
  end;
end

% Check if slfile is empty. If it is empty, convert it from sbfile
if isempty(slfile)
  slfile = sbfile;
  save_flag = 0;
end;

% when top block is not specified
if nargin < 3
   topblk = [];
end

% make sure slfile is a row vector
[i,j]=size(slfile);
if i>j, 
  slfile = slfile'; 
end;

% take out the path indicated by '/' from SIMULINK file
ind = find(slfile == '/');
if ~isempty(ind)
  slfile = slfile(ind(length(ind))+1:length(slfile));
end;

% if it is a VMS computer, remove the header in a different way.

if findstr(lower(computer),'vms')
  ind = find(slfile == ']');
  if ~isempty(ind)
    slfile = slfile(ind(length(ind))+1:length(slfile));
  end;
end;

% add extension for slfile as *.m
ind = find(slfile == '.');

if isempty(ind)
 slstring = slfile;
else
 slstring = slfile(1:min(ind)-1);
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read MATRIXx DATA file                                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% open SB file; and search for the necessary information.
f_sb = fopen(sbfile,'r');
if f_sb < 0
  error(['Cannot Open File ''' sbfile '''. Please Check Your Filename.'])
  return;
end;
x = fread(f_sb, 20, 'char');
tmp = find(x == 77);
if isempty(tmp)
  fclose(f_sb)
  error('Your file is not a MATRIXx DATA File')
  return;
end;
flag_bi = 0;
if tmp(1) > 1
  x = fread(f_sb, tmp(1)-1, 'char');
  flag_bi = 1;
end;

%number of variables to be read in
numz = fscanf(f_sb,'%d',1);
xn  = 0;

x = fread(f_sb, 21, 'char');

fprintf('Your MATRIXx data file was saved on')
fprintf(setstr(x(4:21)'));
fprintf('\n')

if flag_bi == 1
  fclose(f_sb)
  fprintf('We are sorry for cannot transfer your binary file yet.\n');
  return;
  numz = 0;
end;

% prepare files to be saved
save sb2sltmp slstring numz xn

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read and Save Data Variables						%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if numz > 0
  fprintf('Reading and Saving variables.')
  x = fread(f_sb, numz*20+ceil(numz/4), 'char');
  ind = find(x == 10);
  for i = length(ind):-1:1
    x(ind(i)) = [];
  end;
  for i=1:numz
    var_n = [var_n; setstr(x((i-1)*20+1:(i-1)*20+11)')];
    var_r = [var_r; str2num(setstr(x((i-1)*20+10:(i-1)*20+15))')];
    var_c = [var_c; str2num(setstr(x((i-1)*20+16:(i-1)*20+20))')];
  end;
  zzz = 'sb2sltmp';
  for i=1:numz
    fprintf('.');
    x = fread(f_sb, 40, 'char');
    x = fscanf(f_sb, '%e', var_r(i)*var_c(i));
    y = [];
    for j=1:var_c(i)
      tmp = (j-1)*var_r(i)+1:j*var_r(i);
%      tmp = j:var_c(i):var_c(i)*var_r(i);
      if length(x) >= max(tmp)
        y=[y, x(tmp)];
      end;
    end;
    x = deblank(var_n(i,:));
    ind = find(x==' ');
    if ~isempty(ind)
        x = x(1:ind(1)-1);
    end;
    if case_save
      x = lower(x);
      if case_save == 2
        x(1) = upper(x(1));
      end;
    end;
    appsave(zzz, x, y);
  end;
  disp([ num2str(numz) ' variables saved in the file.'])
end;

fclose(f_sb);

if xn>0 | numz>0
  clear
  load sb2sltmp
%  xs = [xs(1:5) slstring xs(14:length(xs)) zzz(13:length(zzz))];
%  eval(xs);
  %clear the space
  fprintf('\nData file ''')
  fprintf(slstring);
  fprintf('.mat'' has been created.\n')
  fprintf('''load ')
  fprintf(slstring);
  fprintf(''' before use the variables.\n')
  eval(['save ' slstring])
end;

clear
if findstr(lower(computer),'vms')
  eval('!del sb2sltmp.mat;*');
  if exist('apptmp.mat')
    eval('!del apptmp.mat;*');
  end;
elseif findstr(lower(computer),'pc')
  eval('!del sb2sltmp.mat');
  if exist('apptmp.mat')
    eval('!del apptmp.mat');
  end;
else
  eval('!rm sb2sltmp.mat');
end;
