function [delta,theta,Mx]=deltason(Mx,gamma)
%DELTASON	For isentropic flow, compute the deflection that results in
%		a sonic downstream Mach No. Also compute the corresponding
%		oblique shock angle.
%
%		[DELTA,THETA]=DELTASON(MX) computes, for each Mach number
%		specified in MX, the flow deflection (DELTA) that results in
%		a sonic downstream Mach No. Also, computes the corresponding
%		oblique shock angle (THETA). Both DELTA and THETA have units
%		of degrees.
%
%		DELTASON() by itself, computes the sonic deflection angle for
%		a Mach range from 1.0 to 15.0 and plots the results.
%
%		Also see, DELTAMAX.

%Ref: Shapiro, Ascher H,; "The Dynmaics and Thermodynamics of
%     Compressible Fluid Flow", John Wiley & Sons, 1953, p. 587.
%

% Compressible Flow Toolbox, Version 1.1, 23-FEB-96
% Written By: Kevin J. Melcher - 12/94
%             NASA Lewis Research Center, Cleveland, Ohio
%             Cleveland State Univiersity, Cleveland, Ohio


%No input args => test case
 if nargin==0,
   Mx=[0.8 1:0.1:10 11:0.5:15 5.0E2]
 end %if

%Size of Mx
 [N,M]=size(Mx);

%Default for gamma
 if nargin<2,
   gamma=1.4*ones(N,M);
 else
  %Arg Size Check
   if N~=size(gamma,1) | M~=size(gamma,2)
     error('Size of input args are not equal.')
   end
 end

%Variation on a theme
 g      = gamma;
 gm1    = g-1;
 gp1    = g+1;
 MxSQRD = Mx.*Mx;

%Calculate theta
 a = ones(N,M)./(g.*MxSQRD);
 b = (gp1.*MxSQRD-(3-g))/4;
 c = (9+g)/16 - (3-g).*MxSQRD/8 + gp1.*MxSQRD.*MxSQRD/16;
 SinSQRD=a.*(b+sqrt(gp1.*c));
 theta=asin(sqrt(SinSQRD));

%Calculate Delta
 d = (MxSQRD.*SinSQRD-1).*cot(theta);
 e = 0.5*gp1.*MxSQRD - MxSQRD.*SinSQRD + 1;
 delta=atan(d./e);

%Mx < 1.0 has no solution
% (i.e., existence of shock => Mx > 1.0
 i=find(Mx<1);
 if ~isempty(i);
   delta(i)=nan*ones(size(i));
   theta(i)=delta(i);
 end

%Convert results to degrees
 theta=theta*180/pi;
 delta=delta*180/pi;

%Plot Results for Test Case
 if nargin==0
   n=length(Mx);
   i=1:n-1;
   clf
   subplot(2,1,1);
    plot(Mx(i),delta(i)); grid;
    a=axis; dx=a(2)-a(1); dy=a(4)-a(3);
    hold on; plot(a(1:2),[delta(n) delta(n)],'c--'); hold off;
    title('Deflection Angle for Sonic Flow vs. Mach Number');
    xlabel('Mx'); ylabel('Deflection Angle (deg)');
    %Label Asymptote
     x=a(1)+0.02*dx; y=delta(n);
     text(x,y,['Asymptote @ ' num2str(delta(n)) ' Degrees'], ...
              'FontSize',get(gcf,'DefaultTextFontSize')*0.7);
   subplot(2,1,2);
    plot(Mx(i),theta(i)); grid;
    a=axis; dx=a(2)-a(1); dy=a(4)-a(3);
    xlabel('Mx'); ylabel('Shock Angle (deg)');
   figure(gcf);
  end %if
