function [beta,mu,res,covarbeta,covdiff,devlist]=glmfit(y,x)
%GLMFIT Fits a generalised linear model (glm)
% USE: [beta,fits,res,covarbeta,covdiff,devlist]=glmfit(y,x)
%   where  y  is the response variable;
%             (For a binomial response, y  has two columns:
%              the first contains  y, the second has the sample sizes)
%          x  is the covariate matrix 
%             (the number of columns of  x  is the number of variables.  If 
%              a constant term is needed,  x  should contain a vector of ones.)
%          beta  contains the parameters estimates (first column) 
%             and standard errors (second column);
%          fits  are the fitted values;
%          covarbeta  covariance matrix of the parameter estimates;
%          res  are the standardised residuals:
%             (y-fits)*sqrt(prior wt / (scale parameter*variance function))
%          covdiff  is the var/covariance matrix of standard error of parameter
%             differences.
%          devlist  is a vector of residual deviance for iterations of the fit.
%
% Both vectors  y,  x,  should be the same length: the number of observations.
%
% Only  y  is needed.  If  x  is not supplied, only the constant term is fitted.
%
%ALSO SEE:  glmlab (fitting glm's (uses glmfit)), the  paramtrs.m  file

%Copyright 1996 Peter Dunn
%19/11/1996

global LINKFUNCTION ERRDISFUNCTION DEVIANCE DEGREESOFF NAMEPW NAMEOS OFFSET;
global FORMAT SCALEPAR RESTYPE FLAG YVAR XVARS FITVALS WEIGHTS M RESIDS;
global NAMEXV NAMEYV NAMELIST NAMESIZE INC_CONST DETAILSFILE;

line=makeline('','-'); lline=makeline; eline=makeline(1);wline=makeline(2);
   %Error checks and fiddling to get around other problems
if nargin==0,
   if ~isempty(YVAR)|~isempty(XVARS),
      if ~isempty(YVAR), 
         y=YVAR; 
      else
         bell;makeline(1);
         disp(' No response variable has been assigned, so there is');
         disp(' nothing to fit.  Please define a response (y) variable.');
         makeline;return;
      end;
      if ~isempty(XVARS), x=XVARS;end;
   else
      help glmfit;return;
   end;
end;
   %Setting up of current settings
if isempty(INC_CONST), INC_CONST=1; end;
if isempty(FORMAT), format=[0 1]; FORMAT=format; else, format=FORMAT; end;
if isempty(RESTYPE), typer='Pearson';RESTYPE=typer;else typer=RESTYPE;end;
if isempty(ERRDISFUNCTION), ERRDISFUNCTION='normal';end;
if isempty(LINKFUNCTION), 
   if strcmp(ERRDISFUNCTION,'poisson'), LINKFUNCTION='log';
   elseif strcmp(ERRDISFUNCTION,'binoml'), LINKFUNCTION='logit';
   elseif strcmp(ERRDISFUNCTION,'gamma'), LINKFUNCTION='recip';
   else LINKFUNCTION='id'; %normal and own
   end;
end;

if isempty(DEVIANCE),
   makeline('WELCOME :->');
   disp('   Welcome to  glmlab -- generalised linear modelling with matlab.');
   disp('   Programmed by: Peter Dunn, USQ.  Last revision: November 1996.');
end;
[yrows,ycols]=size(y);
if yrows<ycols, y=y';end;%each row is an observation
ylen=length(y);
   %Do some more fiddling and error checks:
altered=0;
if ycols>2,
   bell;makeline(1);
   disp( ' The response variable should be a single column vector (or, in the');
   disp( ' case of a binomial variable, have two columns).  Your response');
   disp([' variable, however, has ',num2str(ycols),' columns.']);
   disp( ' I don''t know what to do with such an input!');
   makeline;return;
elseif ycols==2, %set up for binoml
   m=y(:,2);y=y(:,1);
   if ~strcmp(ERRDISFUNCTION,'binoml'),
      ERRDISFUNCTION='binoml';LINKFUNCTION='logit';disp(wline);
      disp(' Distribution and link function changed to BINOMIAL and LOGIT.');
      FID=fopen(DETAILSFILE,'a');
      fprintf(FID,' --- binomial --- logit\n');fclose(FID);
      disp(lline);
   end;
   if ~exist('LINKFUNCTION'), LINKFUNCTION='logit';end;
   errdis=ERRDISFUNCTION;link=LINKFUNCTION;
   altered=1;
else
   if strcmp(ERRDISFUNCTION,'binomial'),
      ERRDISFUNCTION='normal';LINKFUNCTION='id';disp(wline);
      disp(' Distribution and link function changed to NORMAL and IDENTITY.');
      FID=fopen(DETAILSFILE,'a');
      fprintf(FID,' --- normal --- id\n');fclose(FID);
      disp(lline);
   end;
   errdis=ERRDISFUNCTION;link=LINKFUNCTION;
   m=zeros(yrows,1); %dummy  m  vector 
end;
m=m(:);
if ~altered,
   if strcmp(ERRDISFUNCTION,'binoml')&(ycols==1),
      if all(y<=1)&all(y>=0), %then assume probabilities
         m=ones(size(y));
      else
         ERRDISFUNCTION='normal';LINKFUNCTION='id';
         disp(wline);
         disp(' The response variable is not set up for binomial responses,');
         disp(' so the distribution and link have been set to NORMAL and ID.');
         disp(' For binomial errors, the response variable should be:');
         disp('     [the counts   the denominators (sample size)]');
         bell;makeline;
         FID=fopen(DETAILSFILE,'a');
         fprintf(FID,' --- normal --- id\n');fclose(FID);
      end;
   end;
end;
errdis=ERRDISFUNCTION;link=LINKFUNCTION;
if nargin==1, x=ones(yrows,1); end;
if (nargin==0)&INC_CONST,
   x=[ones(yrows,1),XVARS];
end;
if exist('NAMELIST')==1,
   namelist=NAMELIST;
else
   if ~exist(namelist)
      namelist=blanks(size(x,1))';
   end;
end;
if INC_CONST,
   if isempty(NAMELIST),
      namexv='[Const]'; namelist='Constant';
   else
      namexv=['[Const,',NAMEXV(2:length(NAMEXV))];
      namelist=str2mat('Constant',NAMELIST);
   end;
else
   namexv=NAMEXV;
end;
if isempty(NAMEXV),
   namexv='[Const]';namelist='Constant';
   INC_CONST=1;set(findobj('tag','HIC'),'value',1);
   x=ones(yrows,1);
end;
   %end that bit of fiddling
   %some more error checks:
if isempty(WEIGHTS), WEIGHTS=ones(yrows,1);end;
if isempty(OFFSET), OFFSET=zeros(yrows,1);end;
weights=WEIGHTS;offset=OFFSET;
if strcmp(ERRDISFUNCTION,'poisson'),
   if any(y.*weights<0),
      disp(eline);
      disp([' For the poisson distribution, responses must all']);
      disp( ' be non-negative.  Some responses are negative.');
      makeline;bell;return;
   end;
end;
if strcmp(ERRDISFUNCTION,'gamma'),
   if any((~(y>0)).*weights),
      disp(eline);
      disp(' For the gamma distribution, responses must all');
      disp(' positive.  Some responses are non-positive.');
      makeline;bell;return;
   end;
end;
if strcmp(LINKFUNCTION,'sqroot')&any(y.*weights<0),
   disp(eline);
   disp(' For the square root link function, all responses must be');
   disp(' non-negative.  Some negative responses are present.');
   makeline;bell;return;
end;
if strcmp(LINKFUNCTION,'log')&any(y.*weights<0),
   disp(eline);
   disp(' The log link requires that the response variable is always');
   disp(' non-negative.  Some non-positive responses are present.');
   makeline;bell;return;
end;
clear flag p l altered;
if strcmp(errdis,'binoml'),
   if any( m<y ), m=[]; %somewhere there are more obs than samples
      disp(' ');disp(eline);
      disp( ' There is an error with the vector of sample sizes!');
      disp([' The number of observations can''t be more than the' ...
          ' sample size!']); disp( ' Please check the input.');
      makeline;bell;return;
  end;
end;
   %end those error checks

zerowts=sum(weights==0);effpts=ylen; if zerowts>0, effpts=ylen-zerowts;end;
if isempty(SCALEPAR),
   if strcmp(errdis,'normal')|strcmp(errdis,'gamma'),
      SCALEPAR='md';
   else
      SCALEPAR=1;
  end;
end;
if format(1), %display fitting information
   disp(lline);
   if isstr(LINKFUNCTION), 
      l=upper(LINKFUNCTION); 
   else 
      l=['TO POWER OF ',num2str(LINKFUNCTION)];
   end;
   disp([' INFORMATION: Distribution/Link: ',upper(errdis),'/',l]);
   if zerowts>0,
      add='s';if effpts==1,add='';end;
      disp([blanks(14),'Fitting based on:  ',...
            num2str(effpts),' observation',add]);
   end;
   goodwt=sum((weights/max(weights))==1);%goodwt==length(weight) if all ones
   goodof=sum(offset==0);%goodof==length(offset) if all zeros
   if goodwt~=ylen,
      disp([blanks(14),'Prior Weights:     ',NAMEPW]);
   end;
   if goodof~=ylen,
      disp([blanks(14),'Offset Variable:   ',NAMEOS]);
   end;
   if isempty(SCALEPAR),
      if strcmp(errdis,'normal')|strcmp(errdis,'gamma'),
         disp([blanks(14),'Scale parameter estimated from mean deviance.']);
      else
         disp([blanks(14),'Scale parameter set to 1.']);
      end;
   else
      if isstr(SCALEPAR),
         disp([blanks(14),'Scale parameter estimated from mean deviance.']);
      else
         disp([blanks(14),'Scale parameter set to ',num2str(SCALEPAR),'.']);
      end;
   end;
end;
 
   %do the number crunching 
[beta,mu,xtwx,devlist,l]=irls(y,x,m);
   %display the results of the fitting
if format(1), thisline=line;end;
if format(2)&(~format(1)), 
   thisline=' -----------------------------------------';
end;
disp(thisline);
FLAG=0;
distinfo=['d',errdis];
wts=sum(weights==0);
curdev=devlist(length(devlist));%deviance for current model
curdf=effpts-length(beta);%df for current model
if ( strcmp(errdis,'normal')|strcmp(errdis,'gamma') ) & (curdf==0),
   dispers=Inf;%Otherwise gives warning: Divide by zero
else
   dispers=feval(distinfo,'disppar',curdev,curdf,m,weights);
end;
covarbeta=pinv(xtwx)*dispers;
%Display results
if format(2)|nargout>0, %display the parameter estimates
   if format(2),
      disp('   Estimate        S.E.      Variable');
      disp(' -----------------------------------------');
   end;
   jj=0;bb=zeros(size(x,2),2); 
   for ii=1:size(x,2),
      vn=namelist(ii,:);
      if any(l==ii), %unaliased variables
         jj=jj+1; se=sqrt(covarbeta(jj,jj));
         if isinf(se)&format(2),
            fprintf(' %10.6f     %s    %s\n',beta(jj),'???????',vn);
         elseif format(2),
            fprintf(' %10.6f %12.6f   %s\n',beta(jj),se,vn);
         end;
         bb(ii,2)=se;bb(ii,1)=beta(jj);
      else  %aliased variables
         if format(2),
            fprintf(' %10.6f     %s   %s\n',0.0,' aliased',vn);
         end;
      end;
   end;
end;
beta=bb;
disp(' -----------------------------------------');
%Residuals calculations
if ~isempty(DEVIANCE),  %deviance exists so we print changes
   deldev=curdev-DEVIANCE; deldf=curdf-DEGREESOFF; %defined as in GLIM
   if exist('SCALEPAR')&(~isstr(SCALEPAR)),
      sdev=curdev/SCALEPAR;sdeldev=deldev/SCALEPAR;
      fprintf('Scaled Deviance: %13.6f    (change: %+13.6f)\n',sdev,sdeldev);
   else
      fprintf('Deviance: %20.6f    (change: %+13.6f)\n',curdev,deldev);
   end;
   FID=fopen(DETAILSFILE,'a');
   fprintf('Residual df: %17.0f    (change: %+13.0f)\n',curdf,deldf);
   fprintf(FID,'%12.6f %13.6f %3.0f %5.0f     %s\n',...
           curdev,deldev,curdf,deldf,[NAMEYV,';',namexv]);
else %deviance doesn't exist, so this is the first fit
   if exist('SCALEPAR'),
     if isstr(SCALEPAR), sdev=curdev; else sdev=curdev/SCALEPAR; end;
     fprintf('Scaled deviance: %13.6f           Link: %s\n',curdev,upper(link));
   else
     fprintf('Deviance: %19.6f            Link: %s\n',curdev,upper(link));
   end;
   fprintf('Residual df: %17.0f   Distribution: %s\n',curdf,upper(errdis));
   if exist(DETAILSFILE), 
      disp('(Deleting DETAILS.m file.)');delete(DETAILSFILE); 
   end;
   FID=fopen(DETAILSFILE,'a');
   fprintf(FID,'(Created at %s on %s.)\n',mytime,date);
   fprintf(FID,'   Deviance        Change   df  Change   Variables\n');
   fprintf(FID,'%12.6f      %12.0f           %s\n',...
             curdev,curdf,[NAMEYV,';',namexv]);
end;
if (~isempty(NAMEPW))|(~isempty(NAMEOS)),
   fprintf(FID,'   The above fit includes the following:\n');
end;
if (~isempty(NAMEPW))&(~isempty(NAMEOS)),
   fprintf(FID,'      Prior weights: %s; Offset: %s\n',NAMEPW,NAMEOS);
else
   if ~isempty(NAMEPW), fprintf(FID,'      Prior weights: %s\n',NAMEPW);end;
   if ~isempty(NAMEOS), fprintf(FID,'      Offset:        %s\n',NAMEOS);end;
end;
fclose(FID);

if isinf(dispers)|isnan(dispers),
   fprintf('Dispersion parameter cannot be found: O degrees of freedom.\n');
else
   fprintf('Scale parameter (dispersion parameter): %16.6f\n', dispers);
end;

DEVIANCE=curdev; DEGREESOFF=curdf;
k=1;
if ~isempty(SCALEPAR), if ~isstr(SCALEPAR), k=SCALEPAR; end; end;
reps=1;
   %calculate the residuals
if strcmp(typer,'quantile')&...
         (strcmp(ERRDISFUNCTION,'poisson')|strcmp(ERRDISFUNCTION,'binoml')),
   reps=0;disp(' '); disp(wline);
   disp('  Quantile residuals contain random elements for binomial and');
   disp('  poisson distributions.  It is best to repeat to find a few');
   disp('  sets of residuals to accurately gauge their properties.');
   while round(reps)<1,
      reps=input([' * How many replications do you want',...
                  ' (default is one): ']);
   end; 
   if length(reps)==0,reps=1;end;tag='s';if rep==1, tag='';end;
   disp([' Finding ',reps,' set',tag,' of quantile residuals.']);disp(line);
end;    
res=zeros(length(y),reps);
for ii=1:reps,
   if strcmp(typer,'deviance'), %deviance residuals
      rescol=zeros(yrows,1);
      for jj=1:ylen,
         yy=y(jj);mmu=mu(jj);
         rescol(jj)=sign(yy-mmu)*...
          sqrt(feval(distinfo,'scdev',yy,mmu,m(jj),weights(jj)));
      end;
   elseif strcmp(typer,'Pearson'), %pearson residuals
      rescol=zeros(yrows,1);
      for jj=1:ylen,
         yy=y(jj);mmu=mu(jj);
         rescol(jj)=(yy-mmu)* sqrt(weights(jj)/...
                    (k*feval(distinfo,'varfn',m(jj),mmu,m(jj),weights(jj))) );
      end;
   else %quantile residuals
      dist=ERRDISFUNCTION;
      if strcmp(dist,'normal'),
         rescol=(y-mu)./sqrt(k) .*sqrt(weights);
      elseif strcmp(dist,'gamma'),
         rescol=invnorm(cdfgam(y,mu,k)).*sqrt(weights);
      else %discrete distns
         ok=1;
         if strcmp(dist,'binoml'),
            phat=mu./m;p0=cdfbino(y-1,m,phat);p1=cdfbino(y,m,phat);
            p0=p0-p0.*(y==0);
         elseif strcmp(dist,'poisson'),
            p0=cdfpoiss(y-1,mu);p1=cdfpoiss(y,mu);
         else %unknown (user)
            ok=0;
         end;
         if ok,
            loadseed; r=p0+(p1-p0).*rand(length(p0),1);
            rescol=invnorm(r);saveseed;
         elseif ok==0,
            rescol=[];
         end;
      end;
   end;
   res(:,ii)=rescol;
end;
if ok==0,
   disp(wline);disp(' Quantile residuals could not be calculated for the ');
   disp([' ',upper(dist),' distribution.  Sorry.']);disp(line);FLAG=2;
end;
   %calculate other output variables
if nargout>4,
   covdiff=zeros(size(covarbeta));
   for ii=1:length(covarbeta),
      for jj=ii+1:length(covarbeta),
         cvd=sqrt( covarbeta(ii,ii)+covarbeta(jj,jj)-2*covarbeta(ii,jj) );
         covdiff(ii,jj)=cvd;covdiff(jj,ii)=cvd;
      end;
   end;
end;
   %fix up some things for use elsewhere
FITVALS=mu; RESIDS=res;M=m;
   %make residual plots available
if ~isempty(findobj('String','RESIDUAL PLOTS...')),
   set(findobj('String','RESIDUAL PLOTS...'),'Visible','on');
end;

clear FID ii jj;

return;
