function isentbl(Filename,Mn,Gamma);
%ISENTBL	Generate ASCII Table of isentropic flow functions.
%		ISENTBL() generates an ASCII table of the isentropic flow
%		functions found in columns 2 thru 8 of Tables I & II in NACA
%		Report 1135 (Ames Tables). The function values are computed
%		for Mach numbers from 0 to 100 using the ratio of specific heat
%		for air (i.e., Gamma=1.4). Text is written to file ISENTBL.TXT.
%		ISENTBL(FILENAME,MN,GAMMA) computes the flow functions and
%		writes the ASCII data to the file specified in FILENAME.
%		Functions are evaluated at Mach numbers	specified in MN using
%		the ratio of specific heats GAMMA.
%
%		This function is primary useful for generating tables that can
%		be used when MATLAB is unavailable.
%
%		Also see, AMES2.

%		REF: Ames Research Staff: Equations, Tables, and Charts for
%		     Compressible Flow, NACA Report 1135, 1953.

% Compressible Flow Toolbox, Version 1.1, 23-FEB-96
% Written By: Kevin J. Melcher - 12/94
%             NASA Lewis Research Center, Cleveland, Ohio
%             Cleveland State Univiersity, Cleveland, Ohio


%Check Input Args
 if nargin<1, Filename='isentbl.txt'; end
 if nargin<2, Mn=[0:0.01:1 1.1:0.1:10 11:100]; end
 if nargin<3, Gamma=1.4; end

%Get Table Values
 FMn=ames2(1,Mn,Gamma);

%Define Header
 Header = [
 '     M or M1        P/Pt          p/pt          T/Tt    ' ...
 '      Beta          q/Pt          A/A*          V/a*'];
 %....!....!....!....2....!....3....!....4....!....5....!....6
 TopLabel =sprintf('Isentropic Flow Table for Gamma=%f',Gamma);
 NBlanksBeg=fix((length(Header)-length(TopLabel))/2);
 NBlanksEnd=length(Header)-length(TopLabel)-NBlanksBeg;
 Header=[setstr(32*ones(1,NBlanksBeg)) TopLabel setstr(32*ones(1,NBlanksEnd));
	 Header];
 
%Open the File
 fid=fopen(Filename,'w');
 
%Write Data
 for i=1:length(Mn)
   if rem(i-1,25)==0
    fprintf(fid,'\n%s\n',Header(1,:)');
    fprintf(fid,'%s\n',Header(2,:)');
   end
   fprintf(fid,'%14.5e',FMn(i,1:8));
   fprintf(fid,'\n');
 end

%Close the File
 fclose(fid);
