function [My,theta,P0ybyP0x,Mx,delta]=oblqw12(Mx,delta,gamma)
%OBLQW12	Compute downstream weak shock functions from upstream info.
%		[MDS,THETA,PTRATIO]=OBLQW12(MUS,DELTA,GAMMA) computes the
%		downstream Mach number, MDS, oblique shock angle, THETA, and
%		total pressure ratio, PTRATIO from upstream Mach number, MUS,
%		compressive turning angle, DELTA, and the ratio of specific
%		heats for the fluid, GAMMA. If GAMMA is not specified, it is 
%		assumed to be 1.4, the ratio of specific heats for air.
%
%		OBLQW12() by itself generates plots showing
%			1) shock angle vs. deflection angle
%			   for lines of constant Mach from 1 to 20.
%			2) Mach Parameter vs. Deflection Angle
%			3) total pressure ratio vs. shcok angle
%
% 	NOTE:	Values for output args are returned as NaN for the
%       	following cases where a valid solution does not exist:
%			delta < 0
%			delta > 48.5847 (max deflection @ Mx=infinity)
%
%		Also see DELTAMAX, DELTASON, OBLQSHCK, OBLQW21

%REF: Ames Research Staff: Equations, Tables, and Charts for Compressible Flow,
%     NACA Report 1135, 1953.

% Compressible Flow Toolbox, Version 1.1, 23-FEB-96
% Written By: Kevin J. Melcher - 12/94
%             NASA Lewis Research Center, Cleveland, Ohio
%             Cleveland State Univiersity, Cleveland, Ohio


%Test function
 if nargin==0,
   Mx=[0.8 1:0.05:1.5 1.6:.1:2 2.2:.2:4 4.5 5 6 8 10 20 500];
   N=100; M=length(Mx);
   [DELmax,THmax]=deltamax(Mx);
   delta=[-1:N-2]'/(N-3)*DELmax(:)';
   Mx=ones(N,1)*Mx;
 end %if

%Size of Mx
 [N,M]=size(Mx);

%Default for gamma
 if nargin<3, gamma=1.4*ones(N,M); end

%Arg Size Check
 if N~=size(delta,1) | N~=size(gamma,1) ...
    | M~=size(delta,2) | M~=size(gamma,2),
   whos, error('Size of input args are not equal.')
 end %if

%Init output args
 My=nan*ones(N,M);
 P0ybyP0x=My;

%Solve for Max Deflection Angle (DELmax)
% & shock angle (theta)
 [DELmax,THmax]=deltamax(Mx,gamma);
 theta=oblqshck(Mx,delta,gamma);

%Calculate My
 i = find(Mx>=1 & 0<delta & delta<=DELmax);
 if ~isempty(i),
   delta = delta*pi/180;
   theta = theta*pi/180;
   g     = gamma(i);
   gm1   = g-1;
   gp1   = g+1;
   MxSinSq  = (Mx(i).*sin(theta(i))).^2;

   My(i)    = ((2+gm1.*MxSinSq)./(2*g.*MxSinSq-gm1)).^(1/2) ...
                  ./sin(theta(i)-delta(i));
   P0ybyP0x(i) = (gp1./(2*g.*MxSinSq-gm1)).^(ones(size(gm1))./gm1) ...
               .* ((gp1.*MxSinSq)./(2+gm1.*MxSinSq)).^(g./gm1);
   theta = theta*180/pi;
   delta = delta*180/pi;
  end %if


%Special Case: delta=0
 i=find(delta==0);
 if ~isempty(i),
  My(i)=Mx(i); P0ybyP0x(i)=ones(size(P0ybyP0x(i)));
 end %if

%Special Cases
 i=find(Mx<1 | delta<0 | delta>DELmax);
 if ~isempty(i),
  My(i)=nan*ones(size(i)); theta(i)=My(i); P0ybyP0x(i)=My(i);
 end %if

if nargin>0, return, end

%Delete ALL Figure Windows
 get(0,'children');

%Setup Figures
 ss=get(0,'ScreenSize');
 figure(1); clf;
  fpos=get(1,'Position');
  dfpos=round([-0.025*ss(3) +0.025*ss(3) 0 0]);
  fpos=[ss(3)-fpos(3) 0 fpos(3:4)]+dfpos;

%Plot Results of Test Case
  [DELson,THson]=deltason(Mx,gamma);
  figure(1); clf; set(gcf,'Position',fpos);
 %Plot theta=F(Mx,delta)
  plot(DELmax(1,:),THmax(1,:), 'g--', ...
       DELson(1,:),THson(1,:),'c--'  );
  for k=1:M;
    i=find(~isnan(theta(:,k)));
    hold on;
     plot([delta(i,k); DELmax(1,k)], ...
          [theta(i,k); THmax(1,k)],  'y');
    hold off;
  end
  title('OBLQW12: Weak Shock Wave Angle vs. Deflection Angle');
  a=axis; dx=a(2)-a(1); dy=a(4)-a(3); ax=gca;
  %Label X-axis
   xlabel('Deflection Angle (deg)');
   ylabel('Oblique Shock Angle (deg)');
    
 %Plot My=F(Mx,delta)
  figure(2); clf; set(gcf,'Position',fpos+dfpos);
   i=find(~isnan(theta(:,k)));
   plot(delta(N-1,:),1-1./My(N-1,:),'g');
  for k=1:M;
    i=find(~isnan(theta(:,k)));
    hold on;
   plot(delta(i,k),1-1./My(i,k),'y');
    hold off;
  end
  title('OBLQW12: Mach Number Parameter vs. Deflection Angle');
  a=axis; dx=a(2)-a(1); dy=a(4)-a(3); ax=gca;
   xlabel('Deflection Angle (deg)');
   ylabel('1-1/Mach(downstream)');
    
 %Plot PTr=F(Mx,delta)
  figure(3); clf; set(gcf,'Position',fpos+dfpos*2);
   i=find(~isnan(theta(:,k)));
   plot(theta(N-1,:),P0ybyP0x(N-1,:),'g');
  for k=1:M;
    i=find(~isnan(theta(:,k)));
    hold on;
   plot(theta(i,k),P0ybyP0x(i,k),'y');
    hold off;
  end
  title('OBLQW12: Total Pressure Ratio vs. Shock Angle');
  a=axis; dx=a(2)-a(1); dy=a(4)-a(3); ax=gca;
   xlabel('Oblique Shock Angle (deg)');
   ylabel('P0(downstream)/P0(upstream)');
  figure(2); figure(1);

 %No Output if Plotting!
  My=[];

