function [MX,theta,P0ybyP0x,My,delta]=oblqw21(My,delta,gamma)
%OBLQW21	Compute upstream weak shock functions from downstream info.
%		[MUS,THETA,PTRATIO]=OBLQW21(MDS,DELTA,GAMMA) computes the
%		upstream Mach number, MUS, oblique shock angle, THETA, and
%		total pressure ratio, PTRATIO from downstream Mach number,
%		MUS, compressive turning angle, DELTA, and the ratio of
%		specific heats for the fluid, GAMMA. If GAMMA is not specified,
%		it is assumed to be 1.4, the ratio of specific heats for air.
%
%		OBLQW21() by itself performs a consistency check on the 
%		calculations. Three input matrices are generated. One with
%		columns of constant Mach and rows of variying Mach number, MX1.
%		One with rows of constant DELTA and columns with DELTA varying
%		from 0 to DELTAMAX=F(MX1). These parameters are used as input 
%		to function OBLQW12 which computes the Mach downstream of an
%		oblique shock, MY. The MY and DELTA are then used as input to
%		this function. An ERR is obtained by subtracting MX1 from the
%		upstream Mach number, MX2, computed by OBLQW21.
%
% 	NOTE:	Values for output args are returned as NaN for the
%       	following cases where a valid solution does not exist:
%			delta < 0
%			delta > deltamax (max deflection @ Mx=infinity)
%
%		Also see DELTAMAX, DELTASON, OBLQSHCK, OBLQW12

%REF: Ames Research Staff: Equations, Tables, and Charts for Compressible Flow,
%     NACA Report 1135, 1953.

% Compressible Flow Toolbox, Version 1.1, 23-FEB-96
% Written By: Kevin J. Melcher - 12/94
%             NASA Lewis Research Center, Cleveland, Ohio
%             Cleveland State Univiersity, Cleveland, Ohio


%Test function
 if nargin==0,
   Mx=[1:5 10 20];
   N=10; M=length(Mx);
   [DELmax,THmax]=deltamax(Mx);
   delta=[0:N-1]'/(N-2)*DELmax(:)';
   Mx=ones(N,1)*Mx;

   My = oblqw12(Mx,delta);
 end %if

%Size of My
 [N,M]=size(My);

%Default for gamma
 if nargin<3, gamma=1.4*ones(N,M); end

%Arg Size Check
 if N~=size(delta,1) | N~=size(gamma,1) ...
    | M~=size(delta,2) | M~=size(gamma,2),
   whos, error('Size of input args are not equal.')
 end %if

%Vectorize Matrices
% My=My(:); delta=delta(:); gamma=gamma(:);

%Init output args
 MX=nan*My;
 theta=MX;
 P0ybyP0x=MX;

%Define Name for Temporary File Used by Iterative Solutions
 Dir = lower(tempdir);
 Name=lower(tempname); nd=max(size(Dir));
 FileName=sprintf('%s.m',Name);
 FuncName=Name;  FuncName(1:nd)=[];
 path(Dir(1:length(Dir)-1),path);

%Create Temporary M-file for Solving Non-Linear Equations
 fid=fopen(FileName,'w');
  fprintf(fid,'%s\n',['function Err=' FuncName '(Mx,My,Delta,Gam);']);
  fprintf(fid,'%s\n', ' %NOTE: Mx, My, Delta, & Gam MUST BE SCALARS');
  fprintf(fid,'%s\n', '   MyEst=oblqw12(Mx,Delta,Gam);');
  fprintf(fid,'%s\n', ' Err=abs(MyEst-My);');
 fclose(fid);

%Solve for Mx
 InBounds = (0<=delta & delta<=deltamax(1e12) & isnan(My)==0);
 nInB=length(InBounds(:));
 opt=foptions; foptions(2:3)=[1.0E-6 1.0E-6];
 clc; 
 for j=1:nInB
  disp(sprintf('Solving Nonlinear Equation %d of %d for Upstream Mach Number...', ...
		j,nInB));
  if InBounds(j),
    MX(j)=fmin(FuncName,My(j),100,foptions,My(j),delta(j),gamma(j));
  end%if
  home
 end

%Delete Temporary File
 delete([Name '.m']);

%Calculate oblique shock angle and total pressure ratio
 if nargout>1,
   InB=find(InBounds==1);
   [idum,theta(InB),P0ybyP0x(InB)]=oblqw12(MX(InB),delta(InB),gamma(InB));
 end;

%Plot Results for Test case
 if nargin>0, return, end;
   Err=MX-Mx
   plot(delta,Err);
   title('OBLQW21: Consistency Check Showing Lines of Constant Upstream Mach');
   xlabel('Flow Deflection Angle (deg)')
   ylabel('Err=Mx(computed)-Mx(specified)')
   for j=1:M
     i=max(find(InBounds(:,j)==1));
     text(delta(i,j),Err(i,j),sprintf('Mx=%g',Mx(i,j)));
   end%for
   figure(gcf);

 %No output if Plotting!
  Mx=[];
