function controls = opMakeControls(ch)
% controls = opMakeControls(ch)
%    Return the vector of controls for the current window.  Assumes opFig
%    is an Osprey window (see opInitFigure).  If the controls already exist,
%    returns immediately; otherwise creates the controls and returns them.
%    The controls are NOT positioned correctly; see opPositionControls.
%
% The order of elements in the controls vector is
%
%   1   background pixel-based axes (opWindowAxes)
%   2   Horz. Zoom   button
%   3   Horz. Crunch button
%   4   Vert. Zoom   button
%   5   Vert. Crunch button
%   6   contrast slider
%   7   contrast text
%   8   Set button
%   9   brightness slider
%  10   brightness text
%  11   Play button
%  12   Playback Rate button
%  13   Data Size   pulldown
%  14   Zero Pad    pulldown
%  15   Hop Size    pulldown
%  16   Window Type pulldown
%  17   h scroll bar
%  18   v scroll bar
%  19   main image axes object

global opControls opFig opWindowAxes opContrastSlider opContrastText
global opBrightnessSlider opBrightnessText opPlayBut opPlayRateMenu
global opDataSizePopup opZeroPadPopup opHopSizePopup opWinTypePopup
global opHScrollBar opVScrollBar opAxes
global opVCrunchIcon opVZoomIcon opHCrunchIcon opHZoomIcon


if (opExists(ch) == 2)
  controls = opControls;
  return;
end

% Create new controls.

set(opFig, 'Units', 'pixels');
controls = zeros(1,20);

if (~exist('opHCrunchIcon'))
  opIcons;
end

controls(1)  = axes('Visible', 'off');
opWindowAxes = controls(1);

% Zoom/crunch buttons.
set(gcf, 'NextPlot', 'add');

controls(2)  = axes('Units', 'pixels');
image(opHZoomIcon,   'ButtonDownFcn', 'opZoomCrunch(''h'',''z'')');
set(controls(2), 'Visible', 'off');

controls(3)  = axes('Units', 'pixels');
image(opHCrunchIcon, 'ButtonDownFcn', 'opZoomCrunch(''h'',''c'')');
set(controls(3), 'Visible', 'off');

controls(4)  = axes('Units', 'pixels');
image(opVZoomIcon,   'ButtonDownFcn', 'opZoomCrunch(''v'',''z'')')
set(controls(4), 'Visible', 'off');

controls(5)  = axes('Units', 'pixels');
image(opVCrunchIcon, 'ButtonDownFcn', 'opZoomCrunch(''v'',''c'')')
set(controls(5), 'Visible', 'off');

set(gcf, 'NextPlot', 'new');
axes(opWindowAxes);

% Contrast and brightness sliders and associated text
controls(6)  = uicontrol('Style', 'slider', ...
	       'Callback', 'opSlider(''c'')', ...
	       'Value', 0.5);
opContrastSlider = controls(6);
uiSlider(opContrastSlider);
controls(7)  = text('Units', 'pixels', 'String', 'Contrast');
opContrastText(ch) = controls(7);

controls(8)  = uicontrol('Style','Pushb', ...
               'Callback', ['opRefresh(',num2str(ch),',0,1)'], ...
               'String', 'Set');

controls(9)  = uicontrol('Style', 'slider', ...
	       'Callback','opSlider(''b'')', ...
	       'Value', 0.5);
opBrightnessSlider = controls(9);
uiSlider(opBrightnessSlider);
controls(10) = text('Units','pixels', 'String','Brightness');
opBrightnessText(ch) = controls(10);


% Play and Playback Rate buttons
controls(11) = uicontrol('Style','PushB', ...
               'Callback', 'opPlay(''button'')', ...
	       'HorizontalAlign', 'center');
opPlayBut(ch) = controls(11);
opPlay('disptext', ch);	                     	% set to default value
controls(12) = uicontrol('Style', 'popup', ...
               'HorizontalAlign', 'right', ...
               'Callback', 'opPlay(''setrate'')');
opPlayRateMenu = controls(12);

% Pulldowns -- data size, zero pad, hop size, win type
controls(13) = uicontrol('Style', 'Popup', ...
               'String', opDataSizeF('string'), ...
	       'Callback', 'opDataSizeF');
opDataSizePopup = controls(13);
opDataSizeF(ch);		                % display current value
controls(14) = uicontrol('Style', 'Popup', ...
               'String', opZeroPadF('string'), ...
	       'Callback', 'opZeroPadF');
opZeroPadPopup = controls(14);
opZeroPadF(ch);			                % display current value
controls(15) = uicontrol('Style', 'Popup', ...
               'String', opHopSizeF('string'), ...
	       'Callback', 'opHopSizeF');
opHopSizePopup = controls(15);
opHopSizeF(ch);			                % display current value
controls(16) = uicontrol('Style', 'Popup', ...
               'String', opWinTypeF('string'), ...
               'Callback', 'opWinTypeF');
opWinTypePopup = controls(16);
opWinTypeF('disp', ch);		                % display current value

% Main scroll bars
controls(17) = uicontrol('Style', 'slider', ...
               'Callback', 'opSlider(''h'')');
opHScrollBar = controls(17);
uiSlider(opHScrollBar);
controls(18) =  uicontrol('Style', 'slider', ...
                'Callback', 'opSlider(''v'')');
opVScrollBar = controls(18);		
uiSlider(opVScrollBar);

% Main image axes object
if (opExists(ch) == 2), delete(opAxes(ch)); end;  % not sure if this is needed
controls(19) = axes('Units', 'pixels', 'Box', 'off', 'YDir', 'normal', ...
    'TickDir', 'out');
opAxes(ch) = controls(19);

opControls = controls;
