function opPrint(cmd,np)
% opPrint('wysiwyg')
% Print the image the same size as it appears on the screen.
%
% opPrint('eps')
% Like wysiwyg, but saves a Postscript file instead of printing.
%
% opPrint('fullpage')
% Print the image so it fills up a page.
%
% opPrint('manypages' [,np])
% Several pages (np of them) of the gram are printed.  The printed images 
% are the same scaling as the screen, as well as the same height, but are 
% wide enough to fill up the page.  If np is not supplied the user is asked.
%
% Page margins are given by opPrintMargin, which is set in opNewChan.
%
% See also print, printopt.

global opPrintMargin opContrastText opBrightnessText opc opFig opAxes
global opT0 opT1 opF0 opF1 opTMax opSrate opDataSize opZeroPad opHopSize
global opWinType uiInput1 uiInputButton opMeasureTexts opMeasureNums
global opEpsDir opEpsMargin

if (nargin < 1), cmd = 'wysiwyg'; end

mar = iff(strcmp(cmd, 'eps'), opEpsMargin, opPrintMargin);

fig = opFig; ax = opAxes(opc);  
maxT = opTMax(opc);
textDrop = 0.5;			% inches from image bottom to text middle

% Save everything...
ornt = get(fig, 'PaperOrientation');
orient(iff(strcmp(cmd, 'eps'), 'portrait', 'landscape'));
aunit = get(ax,  'Units');
apos  = get(ax,  'Position');
punit = get(fig, 'PaperUnits');
ppos  = get(fig, 'PaperPosition');
set(opBrightnessText(opc), 'Visible', 'off');
set(opContrastText(opc),   'Visible', 'off');
for i = [opMeasureTexts(opc,:), opMeasureNums(opc,:)];
  set(i, 'Visible' ,'off'); 
end
sav = [opT0(opc) opT1(opc) opF0(opc) opF1(opc)];

% Find out r, the number of pixels per inch, and set all the units right.
set(ax,  'Units',      'inches'); r = sub(get(ax,'Position'),3);
set(ax,  'Units',      'pixels'); r = sub(get(ax,'Position'),3) / r;
set(fig, 'Units',      'pixels');
set(fig, 'PaperUnits', 'inches');

printsize = get(fig, 'PaperSize');				% inches
if (strcmp(cmd, 'fullpage'))
  destsize = printsize - mar(1:2) - mar(3:4);			% inches
  np = 1;							% # of pages
elseif (strcmp(cmd, 'wysiwyg') | strcmp(cmd, 'eps'))
  destsize = sub(get(ax, 'Position'),3:4)/r;			% inches
  np = 1;
elseif (strcmp(cmd, 'manypages'))
  if (nargin < 2)
    uiInput('Print several pages','OK|Cancel','opPrint(''manypages'',-1)', ...
      [0 0], ['How many pages to print?', 10, ...
      '(enter Inf to print to the end of the sound)'], '');
    return
  elseif (np == -1)
    np = str2num(uiInput1);
    if (uiInputButton ~= 1 | length(np) ~= 1), return; end
    if (np < 1 | np ~= floor(np)), return; end
  end
  p = get(ax, 'Position');			% pixels
  destsize = [(printsize(1) - mar(1) - mar(3)), p(4)/r];	% inches
  sPerInch = (opT1(opc) - opT0(opc)) / (p(3)/r);
else 
  error(['Internal error: Unknown command argument for opPrint: ', cmd]);
end

str = sprintf('%s    %g/%gx/%g/%s', opFileName(opc), opDataSize(opc), ...
    opZeroPad(opc), opHopSize(opc), opWinTypeF('name', opc));
scale = sub(get(fig,'Position'),3:4) ./ printsize;	% screenpix / inches
set(fig, 'PaperPosition', [0 0 printsize]);
while (np > 0)
  if (strcmp(cmd, 'manypages'))
    opT1(opc) = opT0(opc) + destsize(1)*sPerInch;
    if (opT1(opc) > maxT),		% this happens once at end of sound
      destsize(1) = destsize(1) / ((opT1(opc)-opT0(opc)) / (maxT-opT0(opc)));
      opT1(opc) = maxT;
    end
    set(ax, 'Units', aunit, 'Position', apos);  opRefresh(opc); drawnow;
  end
  set(ax, 'Units', 'pixels', 'Position', [mar(1:2).*scale, destsize.*scale]);
  yl = get(ax,'YLim');
  tx = text('String', str, 'FontName', 'Times ', 'FontSize', 10, 'Position',...
      [sub(get(ax,'XLim'),1) (yl(1) - textDrop*diff(yl)/destsize(2))], ...
      'Visible', 'on');
  set(ax,'Units','normalized');		% as of MATLAB 4.1, must do this
  drawnow('discard');
  opPointer('watch');
  
  if (strcmp(cmd, 'eps'))
    if (~exist('opEpsDir'))
      opEpsDir = pathDir(opFileName(opc));
    end
    [f,p] = uiputfile(opEpsDir, 'Save Encapsulated PostScript file as...');
    if (isstr(f))
      opEpsDir = p;
      eval(['print -deps ', p, pathSep, f]);
    end
  else
    print
  end

  delete(tx);
  opT0(opc) = opT1(opc);	% for next page
  if (abs(opT0(opc) - maxT) < 1e-10), break; end
  np = np - 1;
end
opPointer('crosshair');

% Restore everything...
opT0(opc)=sav(1); opT1(opc)=sav(2); opF0(opc)=sav(3); opF1(opc)=sav(4);
set(ax,  'Units', aunit);
set(ax,  'Position', apos);
if (strcmp(cmd, 'manypages'));  opRefresh(opc);  end
set(fig, 'PaperUnits', punit);
set(fig, 'PaperPosition', ppos);
set(fig, 'PaperOrientation', ornt);
set(opBrightnessText(opc), 'Visible', 'on');
set(opContrastText(opc),   'Visible', 'on');
for i = [opMeasureTexts(opc,:), opMeasureNums(opc,:)];
  set(i, 'Visible' ,'on'); 
end
