function [m1,m2,v1,v2,u] = EM(P)
% [mean1,mean2,v1,v2,u] = EM(P)
%
% The Wolfe Estimate-Maximize Algorithm for parameter estimation
% for a mixture of two normal distributions as it occurs with the
% interval estimates of a signal with two periods.
%
% Inputs:
%   P		the mixture distribution of two gaussian distributions
% Outputs:
%   mean1	mean of distribution 0
%   mean2	mean of distribution 1
%   v1		variance of distribution 0
%   v2		variance of distribution 1
%   u		membership function (same length as P, values in [0,1])
%
%
% Silke Witt, Spring '95
%
% Changes by Dave Mellinger, 7/95:
% 
% Cosmetic changes.
%
% The original code set variables M1, M2, Sigma1, Sigma2 (Sigma was renamed to 
% V) at the end of the function, but never returned them (the return variable
% names are lowercase).  Changed this (and hopefully fixed it).
%
% v1 and v2 were originally named sigma1 and sigma2, but they seem to hold
% variances rather than stdevs so I renamed them.  Also added sqrt() to the
% calls to gauss() because gauss wants stdevs, not variances.

c = 2;				% number of mixtures (NOT USED)
P = P(:)';			% make into a row vector
len = length(P);

% Get initial values for u -- pre-partitioning with the fuzzy c-means algorithm.
% The m and v values here are thrown away.
%
[m1,m2,v1,v2,u] = fuzzyCMeans(P);

% Iterate until membership function u doesn't change from one cycle to the next.
%
epsilon = 1e-8;
dif = epsilon + 1;		% use dummy value > epsilon to start first cycle
while (dif > epsilon),
  u_old = u;
  
  % Update parameters (maximization).
  %
  alpha1 = sum(u(1,:)) / len;
  alpha2 = sum(u(2,:)) / len;
  m1 = sum(u(1,:) .* P) / (len * alpha1);
  m2 = sum(u(2,:) .* P) / (len * alpha2);
  v1 = sum(u(1,:) .* (P - m1).^2) / (len * alpha1);
  v2 = sum(u(2,:) .* (P - m2).^2) / (len * alpha2);
  
  % Compute new partition (estimation).
  %
  if (v1 ~= 0), A = alpha1 * gauss(P, m1, sqrt(v1));
  else          A = (P == m1) * alpha1;
  end
  if (v2 ~= 0), B = alpha2 * gauss(P, m2, sqrt(v2));
  else          B = (P == m2) * alpha2;
  end
  
  u = zeros(2,len);
  ix = (A ~= 0);
  u(1,ix) = 1 ./ (1 + B(ix) ./ A(ix));
  ix = (B ~= 0);
  u(2,ix) = 1 ./ (1 + A(ix) ./ B(ix));
  
  % Get difference between estimates.
  %
  dif = sum(abs(u_old - u) * ones(len,1)) / (2 * len);
end;
