function [x,y,bnds]=fhisto(yplot,start,width,xplot)
%FHISTO Plots a frequency histogram from the raw data in column vector, ydata.
%USE:  [x,y,bounds]=fhisto(ydata,start,width,xplot)
%   where  x  is the x-axis midpoints;
%          y  is the y-axis heights;
%          bounds  is the matrix of class boundaries: [lower upper];
%          ydata  is the data for which the histogram is needed;
%          width  is the class width;
%          start  is the starting class lower boundary;
%          xplot  determines how the x-axis is labelled:
%            xplot = 0  means to use class mid-points;
%            xplot = 1  means to use class boundaries.
%            BY DEFAULT, xplot = 1  (plot with boundaries).
%
%  Only the ydata is necessary.
%
% SEE ALSO: rhisto

%Peter Dunn
%Last edit: 4/3/1996

global TYPE;

yplot=yplot(:);
if any(isinf(yplot)), 
   ind=isinf(yplot); yplot(ind)=[];count=sum(ind);
   tag='s have ';if count==1,tag=' has ';end;
   disp([num2str(sum(ind)),' infinite value',tag,'been omitted from the plot.'])
end;
if any(isnan(yplot)), 
   ind=isnan(yplot); yplot(ind)=[];count=sum(ind);
   tag='s have been ';if count==1,tag=' has been ' ;end;
   disp([num2str(sum(ind)),' non-numeric value',tag,'omitted from the plot.'])
end;
if nargin>1,
   if max(yplot)<start,
      errm='The starting point supplied omits all the data!';
      if strcmp(TYPE,'rel'), x=errm;y=0;bounds=0;return; else error(errm); end;
   end;
   if min(yplot)<start,
      ind=find(yplot<start); yplot(ind)=[];num=num2str(length(ind));
      tag='s.';if strcmp(num,'1'), tag='.';end;
      disp(['The starting point supplied will omit ',num,' data point',tag]);
   end;
end;
if nargin<3, 
   numc=floor(log(length(yplot))/log(2)) +1;%See Iman and Conover (1983) p55
   width=(max(yplot)-min(yplot))/numc;width=width+(width==0);
   p10=ceil(log10(width))-1;%approx size of width is 10^p10
   width=ceil(width/10^p10)*10^p10;
end;
if nargin==1, 
   start=min(yplot)-width/2;
   p10=ceil(log10(width))-1;%approx size of width is 10^p10
   start=floor(start/10^p10)*10^p10; %round start same as width
end;
if nargin~=4; xplot = 1; end;

%Error checks
if ~(width>0),
   errm='The class width must be positive.';
   if strcmp(TYPE,'rel'), x=errm;y=0;bounds=0;return;else error(errm);end;
end;
if isstr(xplot),
   errm='Input arguments must be numeric';
   if strcmp(TYPE,'rel'), x=errm;y=0;bounds=0;return;else error(errm);end;
end;
%End error checks

figure;
numclasses=floor((max(yplot)-start)/width)+1;
y=zeros(numclasses+1,1); %counts
bounds = zeros( numclasses+2,1 );
bounds=[0:numclasses+1]*width +start; %class boundaries
%Calculate numbers in each class
for i=1:length(bounds)-1;
   y(i+1)=length( find( (yplot>=bounds(i))&(yplot<bounds(i+1)) ) );
end;
if strcmp(TYPE,'rel'), y=y./sum(y)*100;end;

%Plot:
for i=1:length(bounds)-1,
   hold on;
   line([bounds(i),bounds(i)],[0,y(i+1)]);
   line([bounds(i),bounds(i+1)],[y(i+1),y(i+1)]);
   line([bounds(i+1),bounds(i+1)],[y(i+1),0]);
end;

axis([min(bounds)-(width/2),max(bounds)+(width/2),0,max(y)+1]);
if xplot==0|nargout>0,
   for i=1:numclasses+1,
      set(gca,'xtick',[]);
      x(i+1)=bounds(i)+(width/2); %The midpoints
   end;
end;

if xplot==0,
   x(1)=x(2)-width;
   x(numclasses+2)=x(numclasses+1)+width;
   set(gca,'xtick',x);
else
   set(gca,'xtick',bounds);
end;

nlabs=get(gca,'XTickLabels'); set(gca,'XTickLabels',[]);
if numclasses>7, %Prob too cramped on x-axis: use every second label
   labs=nlabs; in=[2:2:size(labs,1)+1];
   for j=1:length(in),nlabs(in(j),:)=blanks(size(labs,2));end;
end;
set(gca,'XTickLabels',nlabs);

%Some plot fixing-up
labs=get(gca,'XTickLabels'); low=str2num(labs(1,:));
if (start==0)&(low==0), start=1;low=1;end;
fac10=round(log10(start/low));
tag='';
if fac10~=0, tag=[' (x10^',num2str(fac10),')'];end;
xlabel(['x-Axis',tag]);
x=x';bounds=bounds';
tag1='';tag2='';
if strcmp(TYPE,'rel'),tag1='Relative ';tag2=' (in %)';end;
ylabel([tag1,'Frequency',tag2]);
title([tag1,'Frequency Histogram']);
set(gca,'Box','on'); %a personal preference
hold off;

%tidy outputs
x=x(2:length(x)-1);y=y(2:length(y)-1);
bnds=zeros(size(bounds,1)-2,2);
bnds(:,1)=bounds(1:length(bounds)-2);
bnds(:,2)=bounds(2:length(bounds)-1);
