/******************************************/
/*                                        */
/* Copyright (c) 1996 by Adam L. Schwartz */
/*          All right reserved.           */
/*                                        */
/*****************************************************************************/
/* Modified: Nov. 5, 1996                                                    */
/*****************************************************************************/
/* This file was modified.c so that it can be linked with                    */
/* either CFSQP or NPSOL (the latter if NPSOL is a define'd symbol).         */
/* When using NPSOL, the file npsol.opt is created.                          */
/* Also, with NPSOL, if there is an IEEE floating point error detected       */
/* during a linesearch, the linesearch will automatically backtrack.         */
/* RIOTS will define a global variable, opt_program, in Matlab's workspace   */
/* which contains the name of the SQP program being used.                    */
/* Using NPSOL, RIOTS can handle somewhere between 500 and 1000 decision     */
/* variables.                                                                */
/* Using CFSQP, RIOTS can handle somewhere between 200 and 600  decision     */
/* variables.                                                                */
/*****************************************************************************/
/* This program makes calls to simulate(), a MEX program that performs the   */
/* numerical integration of the system dynamics.                             */
/*                                                                           */
/* If the symbol NPSOL is defined at compile time, this code will be         */
/* compiled for for linking with NPSOL instead of CFSQP                      */
/*****************************************************************************/
/* This program solves the following problem:                                */
/*                                                                           */
/* min from eta max from {j member bold q sub o} f sup j ( eta )             */
/*                                                                           */
/*   subject to:  x dot = h(t,x,u) , t member [a,b]                          */
/*                  l sup j (t,x(t),u(t)) <= 0 ; j member bold q sub t       */
/*                  g sup j (b,x(a),x(b)) <= 0 ; j member bold q sub i       */
/*                  g sup j (b,x(a),x(b))  = 0 ; j member bold q sub e       */
/*                                                                           */
/* where f sub j ( eta ) =bydef g sup j ( xi , x(tf) )                       */
/*                              + int sub a sup b l(t,x,u) dt                */
/*                                                                           */
/* Also, simple bounds on the controls and any free initial conditions       */
/* can be specified.                                                         */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/* [u,x,f,g,lambda] = riots([x0,{fixed,{x0min,x0max}}],u0,t,Umin,Umax,       */
/*                           params,[miter,{var,{fd,{feasbl}}}],ialg,        */
/*                           {[eps,epsneq,objrep,bigbnd]},{scaling},         */
/*                           disp,{lambda,istate});                          */
/*                                                                           */
/* INPUTS:                                                                   */
/* =======                                                                   */
/* x0     --- initial conditions on state, x0 is n by 1.                     */
/* fixed  --- a n by 1 vector which indicates whether the corresponding      */
/*            initial state in x0 is fixed (1) or free (0).  Those x0 that   */
/*	      are free become decision variables.  If fixed                  */
/*	      is not specified, all initial conditions are assumed fixed.    */
/* x0min  --- Can only be specified if fixed is specified.  For any x0(i)    */
/*            with corresponding fixed(i) = 0, x0min(i) provides a lower     */
/*	      bound.  Your can specify x0min(i) = -huge(i) to have no lower  */
/*	      bound.  If x0min is specified, do not also specify             */
/*	      endpoint constraints for x0.                                   */
/*	      For fixed(i)=1, you can let x0min(i) = x0(i).                  */
/* x0max  --- Can only be specified if fixed is specified.  For any x0 with  */
/*            corresponding fixed = 0, x0max provides an upper bound. You    */
/*	      can specify x0max = huge to have no upper bound.               */
/*	      If x0max is specified, do not also specify                     */
/*	      endpoint constraints for x0.                                   */
/*	      For fixed(i)=1, you can let x0max(i) = x0(i).                  */
/* u0     --- m by N+1 intial control guess.                                 */
/* t      --- Discretization mesh, t is N+1 by 1. (t = [a:(b-a)/N:b]')       */
/* Umin   --- m by N+1 or m by 1 lower bounds on u.                          */
/*            If Umin = [], then the lower bounds are set to -bigbnd.        */
/* Umax   --- m by N+1 or m by 1  upper bounds on u.                         */
/*            If Umax = [], then the lower bounds are set to bigbnd.         */
/* params --- Vector of constant system parameters (specify [] if no params.)*/
/* miter  --- Maximum number of iterations.                                  */
/* var    --- Add a term proportional to var times the total variation of    */
/*            the first derivative of the control to the objective function. */
/*            The variation is computed in original coordinates.             */
/*            A good choice for var is about 1/10000 of the optimal cost.    */
/*            This regularizes problems with singular controls and problems  */
/*            with relaxed optimal controls.                                 */
/*	      Feature is turned of for splines of order > 2 since these      */
/*            controls are smooth.                                           */
/*            Must have at least 4 control breakpoints to use this option.   */
/* fd     --- (optional) 0: analytic gradients, 1: finite-differences.       */
/* feasbl --- (optional for use with CFSQP)                                  */
/*                       0: allow constraint violation during line search.   */
/*                       1: enforce strict feasibility of inequality         */
/*                          constraints at all times (during phase 2).       */
/*            Note: Feasibility is always mainted w.r.t. bounds.             */
/* ialg   --- Integration algorithm to be used (see comments in simulate.c). */
/*            For the variable stepsize algorithms (5,6) ialg can be passed  */
/*	          as an array [ialg,intrnl_knots,rtol,atol].                 */
/* ----------                                                                */
/* For CFSQP:                                                                */
/* eps    --- tolerance for size of search direction                         */
/*            (must be bigger than machine precision).                       */
/* epsneq --- tolerance for nonlinear equality constraints.                  */
/* objrep --- Stopping criteria for percentage change in objective.          */
/*            Not used if nonlinear equality constraints are present.        */
/* bigbnd --- big number (can use variable 'realmax' defined in Matlab) or   */
/*            the biggest number you expect any decision variable to reach.  */
/* ----------                                                                */
/* For NPSOL:                                                                */
/* eps    --- Termination tolerance (this value will be squared since NPSOL  */
/*            use the square-root of eps).                                   */
/* epsneq --- tolerance for nonlinear equality constraints.                  */
/* objrep --- Precision of function values.                                  */
/* bigbnd --- big number (can use variable 'realmax' defined in Matlab) or   */
/*            the biggest number you expect any decision variable to reach.  */
/* ----------                                                                */
/* DEFAULT values for [eps,epsneq,objrep,bibnd] = [1e-6,1e-4,0,1e6].         */
/* ----------                                                                */
/* scaling--- (default) 00:  no function scaling.                            */
/*                      01:  no function scaling, constraint scaling 1       */
/*                      10:  function scaling 1,  no constraint scaling      */
/*                      11:  function scaling 1,  constraint scaling 1       */
/*                      12:  function scaling 1,  constraint scaling 2       */
/*                      21:  function scaling 2,  constraint scaling 1       */
/*                      22:  function scaling 2,  constraint scaling 2       */
/*             Don't use constraint scaling 2 if u = 0 is not feasible.      */
/*             Note: On warm starts, the previous scaling will be used.      */
/* disp   --- (default 1) Controls amount of printing.                       */
/*            0 : Minimal output.                                            */
/*            1 : Full output.                                               */
/* ----------                                                                */
/* For NPSOL:                                                                */
/* lambda --- When performing a warm start with NPSOL this is the extension  */
/*            to the new dimension of the output lambda from the previous    */
/*            call to RIOTS.                                                 */
/*            []: (default) standard cold start.                             */
/*            -1: Cause Hessian to be stored unpermuted in                   */
/*                perperation for the next call with lambda = 1.             */
/*             1: Cause a warm start using previous Hessian and              */
/*                multipliers.  Only works if number of decision variables   */
/*                is unchanged from previous call and lamda = -1 was used    */
/*                on previous call.                                          */
/*            [lambda,istate]: Do warm start.  Hessian computed using        */
/*                finite-difference approximation.  You can obtain lambda    */
/*                from the previously outputted lambda using distribute.m    */
/* ----------                                                                */
/*                                                                           */
/* OUTPUTS:                                                                  */
/* ========                                                                  */
/* u --- optimal control.                                                    */
/* x --- optimal trajectory.                                                 */
/* f --- optimal costs.                                                      */
/* ----------                                                                */
/* g --- constraint values:                                                  */
/* For CFSQP, the order of constraints is as follows:                        */
/*                                     nonlinear endpoint inequality         */
/*                                     nonlinear trajectory inequality       */
/*                                     linear endpoint inequality            */
/*                                     linear trajectory inequality          */
/*                                     nonlinear endpoint equality           */
/*                                     linear endpoint equality              */
/* For NPSOL the order of constraints is as follows:                         */
/*                                     linear endpoint inequality            */
/*                                     linear trajectory inequality          */
/*                                     linear endpoint equality              */
/*                                     nonlinear endpoint inequality         */
/*                                     nonlinear trajectory inequality       */
/*                                     nonlinear endpoint equality           */
/* ----------                                                                */
/* lambda --- First column is the Lagrange multiplier estimates:             */
/*            ninputs*length_u + n_free_x0 simple bounds,                    */
/*            constraint multipliers (in same order as output g above)       */
/*            objective function multipliers [CFSQP only].                   */
/* (multipliers for bounds are given for transformed coordinates when        */
/*  using CFSQP).                                                            */
/* For NPSOL, lambda = [lambda,istate] where istate indicates constraint     */
/* activity as explained in the NPSOL manual.                                */
/*                                                                           */
/*****************************************************************************/
/* REQUIRED FUNCTIONS: The following functions, to be provided by the        */
/* user, must be available for dynamic linking at runtime (for a description */
/* of these functions, see ~adams/xmath/optimize/systems/template.c), except */
/* for those that are identically zero.                                      */
/*                                                                           */
/* void activate(message)                                                    */
/* void init(neq,params)                                                     */
/* void h(neq,t,x,u,xdot)                                                    */
/* double l(neq,t,x,u)                                                       */
/* double Dh(neq,t,x,u,h_x,h_u)                                              */
/* double Dl(neq,t,x,u,l_x,l_u)                                              */
/* double cost(neq,t,x0,xf)                                                  */
/* double Dcost(neq,t,x0,xf,J_x0,J_xf)                                       */
/*                                                                           */
/* Constraints should be organized in the following order:                   */
/*    nonlinear inequality                                                   */
/*    linear    equality                                                     */
/*    nonlinear inequality                                                   */
/*    linear    equality                                                     */
/*                                                                           */
/* The array params is passed to init() before the first iteration.          */
/* Note that trajectory constraints are returned by l() and                  */
/* endpoint constraints are returned by cost_fnc().                          */
/*****************************************************************************/
/* 
   WARNING: State constraints are linear only if both the constraints,
   phi(t,x,u) or E(b,xi,xf), are linear and, if they involve x, the
   system dynamics must also be linear.  Linear constraints are
   a u t o m a t i c a l l y treated as nonlinear if the system is 
   nonlinear (except simple bounds on the control and initial conditions).  
*/
/* 
   If DIRECT is define'd at compile time, calls to simulate are
      made directly instead of through Matlab (simulate.c
      must be modified so that it is called simulate() instead of
      mexFunction().  This is done by compiling simulate.c with
      -DDIRECT).
*/
/* 
  In the code, the vector of decision parameters, z, is a one-dimensional
  array of length nparam, composed in the following order:

  z=u(1,1),u(1,2),..,u(1,m),u(2,1),..,u(2,m),..,u(length_u,1),..,u(length_u,m),
    x0(free_x0[1]),..,x0(free_x0[n_free_x0])

  so u(i,j) is stored in z[(i-1)*m + (j-1)]
  and x0(i) is stored in z[ncontrols+i] (if there are any free initial states).

  Also, there are many global variables.  Here are some of the most important:
     points    --- Number of integration breakpoints
     N         --- Number of time intervals (points - 1)
     Order     --- Order of control parameterization
     ninputs   --- Dimension of u(t)
     nstates   --- Dimension of x(t)
     length_u  --- Number of control parameters for each input (N + Order - 1)
     ncontrols --- ninputs*length_u

  BACTRACKING
  -----------
  When linked with NPSOL, a floating point error during a linesearch causes
  NPSOL to backtrack.  Floating point errors during gradient computations
  causes riots to terminate.
*/
/*****************************************************************************/

#include "mex.h"
#include <math.h>
#ifdef NPSOL
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#define TRUE 1
#define FALSE 0
#else
#include "cfsqpusr.h"
#endif

/*****************************************************************************/
/*           Some changeable #define statements written here.                */
/*****************************************************************************/
/* Set TRANSFORM to 0 to get no transformation.                        */
/* Set TRANSFORM to 1 to get transformation of decision variables.     */
/* Set TRANSFORM to 2 to get transformation of 1/sqrt(N) only.         */
/* TRANSFORM 2 only works for piecewise constant and piecewise linear. */
/* Its good to use TRANSFORM because: 
     (1) Fewer iterations.
     (2) Termination criterion is independent of discretization level.
*/
#define TRANSFORM 1

/* Set DISCRETE_HESSSIAN to 0 to use lift_Hessian.m */
/* Set DISCRETE_HESSSIAN to 1 to use comp_hess.m    */
/*   for calculating Hessian for warm start.        */
#define DISCRETE_HESSIAN 1


/* Set L1 to 0 for using l2 norm                          */
/* Set L1 to 1 for using approximate l1                   */
/*   for computing total variation of control derivative. */ 
/* RHO is a parameter used when L1 is define'd as 1.      */
#define L1 0
#define RHO 0.1

/*****************************************************************************/

#define MIN_INPUTS 8
#ifndef NPSOL
#define MAX_INPUTS 11
#else
#define MAX_INPUTS 12
#endif
#define MIN_OUTPUTS 0
#define MAX_OUTPUTS 5

#define MAX(a,b) ((a) > (b) ? (a) : (b))
#define MIN(a,b) ((a) < (b) ? (a) : (b))

/*********************I M P O R T A N T ***********************/
/* This is an important define.  If number of items of system */
/* information changes, you must also change this.            */
/**************************************************************/
#define N_INFO 13
static void transform(),inv_transform(),*cd;
static char sim_command[256];		    /* Make sure this is big enough! */
static int IEEE_error,Bounds_p;
static int sys_info[N_INFO]={0,0,0,0,0,1,0,0,0,0,0,0,0};
static int points,N,ncontrols,*free_x0,n_free_x0,Order,Spline_p;
static int nstates,ninputs,nsys_params,nfunctions,nconstraints,LTI;
static int nlt_ineq,lt_ineq,nle_eq,le_eq,nle_ineq,le_ineq;
/* ineq --- inequality constraint, eq --- equality constraint */
/* nlt --- nonlinear trajectory,   lt --- linear trajectory   */
/* nle --- nonlinear endpoint  ,   le --- linear endpoint     */
static int length_u,driver,scale_p,transform_computed_p,Finite_Difference_p;
static double *time,*x0;
static double *Q,*utemp;
static double *function_scaling,Min_Variation;
static double sqp_J1;
Matrix *Mtime,*ialg,*Mx0,*Mform,*Mcontrol,*Maction,*Mf_num;
Matrix *x_soln;

#ifdef NPSOL
extern npsol_(), npoptn_();
/* Declare variables that are defined in cfsqpusr.h */
static int x_is_new,memory_initialized=FALSE;
static int *istate;
static double *tmp_grad,objrep,objeps,gLgeps;
static double *Clambda,*R,*z_old;

/* Free memory that was allocated by calloc() rather than mxCalloc(). */
static void 
clear_memory()
{
  if (memory_initialized) {
    free(++function_scaling); free(istate); free(Clambda); free(R);
    memory_initialized=FALSE;
  }
}
#endif

/* Clean up Matlab variables. */
static void 
clear_Matlab()
{
  mexEvalString("clear sqp_scaling");
  if (Spline_p) {
    mexEvalString("clear sqp_rtD sqp_size_D sqp_U");
    mexEvalString("clear global sqp_sqrt_M_inv");
  }
}

static void 
error(message)
     char *message;
{
  mexSetTrapFlag(0); /* So Matlab doesn't complain about too many outputs. */
  clear_Matlab();
  mexPrintf("\nriots: ");
  mexErrMsgTxt(message);
}

#ifdef __STDC__
void 
#else
int
#endif
mexFunction(nlhs, lhs, nrhs, rhs)
     int nlhs, nrhs;
     Matrix *lhs[], *rhs[];
{
  static void obj(), constr();
  static void gradob(), gradcn(),resimulate();
#ifdef NPSOL
  static void compute_linear_constraints();
  static int objfun(), confun();
#endif
  static void display_info(),print_version();
  static void scale(),unscale();
  int startup();
  static int old_nparam=-1,old_lambda_size,old_points,old_scale_size;
  int i,j,disp;
  int nparam,nf,nineq,neq,mode,iprint,miter,neqn,nineqn;
  int ncsrl,ncsrn,nfsr,inform;
  int f_size,g_size,lambda_size,*mesh_pts;
  int f_calls,g_calls,c_calls;
#ifdef NPSOL
  FILE *fp;
  int nclin,ncnln,nrowa,nrowj,*iw,leniw,lenw,iter,ioptns;
  int warm_start,unwind_Hessian_p,failure;
  static int Hessian_stored_p=FALSE,previous_scale_p;
  double *A,*CJAC,*objgrad,*w,val_lin_constr;
  double *R_ptr,*dstate,*dfree_x0;
  Matrix *temp_mat_ptr;
#endif
  double bigbnd,eps,epsneq,udelta;
  double *z,*bl,*bu;
  Matrix *init_states,*u,*Umax,*Umin,*params;
  Matrix *u_soln,*f,*g,*lambda;

  /*****************************************************************/
  /* Check validity of inputs and store them in appropriate order. */
  /*****************************************************************/

  if (nrhs < MIN_INPUTS || nrhs > MAX_INPUTS ||
      nlhs < MIN_OUTPUTS || nlhs > MAX_OUTPUTS) {
#ifdef NPSOL
    mexPrintf("Usage for NPSOL:\n");
    mexPrintf("[u,x,f,g,lambda] = ");
    mexPrintf("riots([x0,{fixed,{x0min,x0max}}],u0,t,Umin,Umax,params,");
    mexPrintf("[miter,{var,{fd}}],\n");
    mexPrintf("                         ");
    mexPrintf("ialg,{[eps,epsneq,objrep,bigbnd]},{scaling},{disp}");
    mexPrintf(",{lambda})\n");
#else
    mexPrintf("Usage for CFSQP:\n");
    mexPrintf("[u,x,f,g,lambda] = ");
    mexPrintf("riots([x0,{fixed,{x0min,x0max}}],u0,t,Umin,Umax,params,");
    mexPrintf("[miter,{var,{fd,{feasbl}}}],\n");
    mexPrintf("                         ");
    mexPrintf("ialg,{[eps,epsneq,objrep,bigbnd]},{scaling},{disp})\n");
#endif
    for ( i = 0 ; i < nlhs ; i++ )
      lhs[i] = mxCreateFull(0,0,REAL);
    return;
  }

#ifdef NPSOL
  if (mexAtExit(clear_memory))
    mexErrMsgTxt("Couldn't register exit function.");
#endif
  mexSetTrapFlag(1);

  if ( nrhs >= 11 && mxGetPr(rhs[10]) != NULL ) {
    disp = (int)mxGetScalar(rhs[10]);
  }
  else
    disp = 1;
  if (disp) print_version();

  if ( mxGetM(rhs[2]) != 1 )
    error("Time must be a row vector.");

  points = mxGetN(rhs[2]);		           /* Length of time vector. */
  N = points - 1;                                  /* Discretiziation level. */
  time = mxGetPr(rhs[2]);

  ialg = rhs[7];
  driver = (int) mxGetScalar(ialg);
  if ( driver < 0 || driver > 6 ) 
    error("ialg must be 0, 1, ..., or 6.");

  params = rhs[5];

  /* Call to startup() loads system information. */
  Mform = mxCreateFull(1,1,REAL); 
  u = rhs[1];
  length_u = startup(time,ialg,params,u,sys_info);

  nstates     = sys_info[0];  ninputs    = sys_info[1]; 
  nsys_params = sys_info[2];  nfunctions = sys_info[5];
  nlt_ineq    = sys_info[6];  lt_ineq    = sys_info[7];
  nle_ineq    = sys_info[8];  le_ineq    = sys_info[9];
  nle_eq      = sys_info[10]; le_eq      = sys_info[11];
  LTI         = sys_info[12];

  if ( mxGetM(u) != ninputs || mxGetN(u) != length_u )
    error("Control wrong dimension.");
  
#ifdef NPSOL
  if ( nfunctions != 1 )
    error("Can only have 1 objective function when using NPSOL");
#endif

  if ( mxGetPr(params) == NULL && nsys_params != 0 )
    error("Wrong number of parameters.");
  else 
    if ( mxGetM(params)*mxGetN(params) != nsys_params )
      error("Wrong number of parameters.");
  
  if ( disp ) {
    /* Call simulate(0) to print out system information. */
    *mxGetPr(Mform) = 0.0; 
#ifdef DIRECT
    simulate(0,lhs,1,&Mform);
#else
    mexCallMATLAB(0,lhs,1,&Mform,"simulate");
#endif
  }
  /* For nonlinear systems, treat all constraints as nonlinear. */
  if ( LTI == 0 ) {
    if ( (lt_ineq > 0 || le_ineq > 0 || le_eq > 0) && disp )
      mexPrintf("Treating linear constraints as nonlinear.\n");
    nlt_ineq += lt_ineq; lt_ineq = 0;
    nle_ineq += le_ineq; le_ineq = 0;
    nle_eq   += le_eq;   le_eq   = 0;
  }

  init_states = rhs[0];
  if ( mxGetM(init_states) != nstates || mxGetN(init_states) == 3 
      || mxGetN(init_states) > 4 ) {
    error("Initial state vector must have n rows and 1, 2 or 4 columns.");
  }
  /* Store initial conditions into x0. */
  x0 = (double*)mxCalloc(nstates,sizeof(double));
  free_x0 = (int*)mxCalloc(nstates,sizeof(double));
  n_free_x0 = 0;
  if ( mxGetN(init_states) == 1 ) {
    for ( i = 0 ; i < nstates ; i++ ) 
      /* All initial states are fixed. */
      x0[i] = mxGetPr(init_states)[i];
  }
  else {
    j = 0;
    for ( i = 0 ; i < nstates ; i++ ) {
      x0[i] = mxGetPr(init_states)[i];
      if ( mxGetPr(init_states)[nstates+i] == 0 ) {
	n_free_x0++;
	free_x0[j++] = i;	    /* Stored, in order, the number of each  */
      }				    /* state with a fixed initial condition. */
    }
  }

  
  /* The number of decision variables is the number of */
  /* controls plus the number of free initial states.  */
  ncontrols = ninputs*length_u;
  nparam = ncontrols + n_free_x0;


  /* Check upper and lower bounds if given.           */
  /* If only one column given, expand Umin (Umax) out */
  /* to be the same number of columns as u0.          */
  if ( mxGetPr(rhs[3]) != NULL ) {
    Umin = rhs[3];
    if ( mxGetM(Umin) != ninputs )
      error("Lower bounds must have same number of rows as control.");
    if ( mxGetN(Umin) == 1 ) {
      Umin = mxCreateFull(ninputs,length_u,REAL);
      for ( i = 0 ; i < length_u ; i++ )
	for ( j = 0 ; j < ninputs ; j++ )
	  mxGetPr(Umin)[i*ninputs+j] = mxGetPr(rhs[3])[j];
    }
    else
      if ( mxGetN(Umin) != length_u )
	error("Lower bounds are the wrong length.");
  }
  if ( mxGetPr(rhs[4]) != NULL ) {
    Umax = rhs[4];
    if ( mxGetM(Umax) != ninputs )
      error("Upper bounds must have same number of rows as control.");
    if ( mxGetN(Umax) == 1 ) {
      Umax = mxCreateFull(ninputs,length_u,REAL);
      for ( i = 0 ; i < length_u ; i++ )
	for ( j = 0 ; j < ninputs ; j++ )
	mxGetPr(Umax)[i*ninputs+j] = mxGetPr(rhs[4])[j];
    }
    else
      if ( mxGetN(Umax) != length_u )
	error("Upper bounds are the wrong length.");
  }


  udelta = 0.0;
  if ( nrhs >= 9 && mxGetPr(rhs[8]) != NULL ) {
    if ( mxGetM(rhs[8])*mxGetN(rhs[8]) != 4 )
      error("tol must have length 4.");
    eps    = mxGetPr(rhs[8])[0];
    epsneq = mxGetPr(rhs[8])[1];
    objrep = mxGetPr(rhs[8])[2];
    bigbnd = mxGetPr(rhs[8])[3];
  }
  else {
    eps    = 1.e-6;
    epsneq = 1.e-4;
    objrep = 0.0;
    bigbnd = 1.e6;
  }

  miter  = (int) mxGetScalar(rhs[6]);

  nf     = nfunctions;
  nfsr   = 0;

  /* Number of equality constraints.   */
  neqn = nle_eq;      
  neq  = neqn + le_eq;

  /* Number of inequality constraints. */
#ifndef NPSOL
  nineqn = nlt_ineq + nle_ineq;  
  ncsrn  = nlt_ineq;  
  nineq  = nineqn + lt_ineq + le_ineq;
  ncsrl  = lt_ineq;		 
  if (nfsr+ncsrl+ncsrn > 0) {
    mesh_pts = (int *) mxCalloc(nfsr+ncsrl+ncsrn,sizeof(int));
    for ( i = 0 ; i < nfsr+ncsrl+ncsrn ; i++ )
      mesh_pts[i] = points;
  }
#else
  ncsrn = ncsrl = ncsrn = 0;
  nineqn = nlt_ineq*points + nle_ineq;
  nineq = nineqn + lt_ineq*points + le_ineq;
  ncnln = nineqn + neqn;
  nrowj = MAX(ncnln,1);
  nclin = nineq + neq - ncnln;
  nrowa = MAX(nclin,1);
#endif


  /* Should expirement with this. */
  if ( neqn+nineqn == 0 )
    mode = 100;  /* Armijo line-search mode.  */
  else
    mode = 110;  /* Non-monotone line-search. */
  Min_Variation = 0.0;
  if (mxGetN(rhs[6]) > 1 || mxGetM(rhs[6]) > 1) {
    Min_Variation = mxGetPr(rhs[6])[1];
    if ( Min_Variation <= 0.0 ) 
      Min_Variation = 0.0;
    else {
      if ( Order > 2 ) {
	mexPrintf("Not adding variation of control derivative to objective");
	mexPrintf("because controls are smooth.\n");
	Min_Variation = 0.0;
      }
      else {
	if ( N < 4-Order )
	  error("Not enough controls to use control derivative variation.");
	Min_Variation /= ninputs*(N+Order-3)*N*N;
      }
    }
  }
  Finite_Difference_p = FALSE;
  if (mxGetN(rhs[6]) > 2 || mxGetM(rhs[6]) > 2) {
    if ((int)mxGetPr(rhs[6])[2]) {
      Finite_Difference_p = TRUE;
      /* If using lsoda(), call it using ialg = 6 so that it doesn't try  */
      /* to use user supplied Jacobians when integrating stiff equations. */
      if ( driver == 5 ) driver = 6;
    }
  }
#ifndef NPSOL
  if (mxGetN(rhs[6]) > 3 || mxGetM(rhs[6]) > 3)
    if ((int)mxGetPr(rhs[6])[3]) mode += 100;  /* Ensure strict feasibility. */
#endif
  
  if ( nrhs >= 10 && mxGetPr(rhs[9]) != NULL ) {
    if ( mxGetN(rhs[9]) > 1 || mxGetM(rhs[9]) > 1 )
      error("Scale input must be a scalar");
    scale_p = mxGetScalar(rhs[9]);
    i = scale_p/10; j = scale_p - i*10;
    if ( i < 0 || i > 2 || j < 0 || j > 2 )
      error("Valid scaling choices are: 00,01,10,11,12,21,22.");
  }
  else
    scale_p = FALSE;
  
#ifdef NPSOL
  
  warm_start = unwind_Hessian_p = FALSE;
  if ( nrhs >= 12 && mxGetPr(rhs[11]) != NULL ) {
    if (mxGetN(rhs[11])*mxGetM(rhs[11]) == 1 ) {
      warm_start = mxGetScalar(rhs[11]);
      if ( warm_start )
	if ( warm_start != -1)
	  if ( !Hessian_stored_p )
	    error("\nCan't do a warm start: haven't done a proper cold start yet.\n");
	  else
	    unwind_Hessian_p = TRUE;
      else
	unwind_Hessian_p = TRUE;
    }
    else
      warm_start = 1;
    if ( warm_start ) {
#if DISCRETE_HESSIAN
      if ( !memory_initialized && warm_start == 1 ) {
	warm_start = FALSE;
	mexPrintf("\nCan't do a warm start: haven't done a proper cold start yet.\n");
      }
#else
      unwind_Hessian_p = TRUE;
      if ( !(Hessian_stored_p && memory_initialized) && warm_start == 1 ) {
	warm_start = FALSE;
	mexPrintf("\nCan't do a warm start: haven't done a proper cold start yet.\n");
	}
#endif
    }
  }

#endif

  /***********************************************************************/
  /*                           Memory Allocation                         */
  /***********************************************************************/
  
  f_size = nf-nfsr;
  for ( i = 0 ; i < nfsr ; i++ )
    f_size += mesh_pts[i];
  f_size=MAX(f_size,1);
  
  g_size=nineq+neq-(ncsrl+ncsrn);
  for ( i = 0 ; i < ncsrn ; i++ )
    g_size += mesh_pts[i+nfsr];
  for ( i = 0 ; i < ncsrl ; i++ )
    g_size += mesh_pts[i+nfsr+ncsrn];
  nconstraints = g_size;
  g_size=MAX(g_size,1);

  f      = mxCreateFull(f_size,1,REAL);
  g      = mxCreateFull(g_size,1,REAL);
#ifndef NPSOL
  lambda_size = nparam + f_size + g_size;
  lambda = mxCreateFull(lambda_size,1,REAL);
#else
  lambda_size = nparam + nclin + ncnln;
  lambda = mxCreateFull(lambda_size,2,REAL);
#endif

  
  z     = (double *)mxCalloc(nparam,sizeof(double));
  Q     = (double *)mxCalloc(length_u,sizeof(double));
  utemp = (double *)mxCalloc(ncontrols,sizeof(double));

  /* Store control and free intial conditions into decision vector. */
  
  /* Check to see if there are any control bounds. */
  if ( mxGetPr(rhs[3]) != NULL || mxGetPr(rhs[4]) != NULL )
    Bounds_p = TRUE;
  else
    Bounds_p = FALSE;

#if TRANSFORM
  transform_computed_p = FALSE;
  transform(mxGetPr(u),z);
#else
  for ( i = 0 ; i < ncontrols ; i++ )
    z[i] = mxGetPr(u)[i];
#endif
  for ( i = 0 ; i < n_free_x0 ; i++ )
    z[i+ncontrols] = x0[free_x0[i]];

#ifdef NPSOL
  z_old = (double *)mxCalloc(nparam,sizeof(double));
  A = (double *)mxCalloc(nrowa,nparam*sizeof(double));
  CJAC = (double *)mxCalloc(nrowj,nparam*sizeof(double));

  /***********************************************************************/
  /*              Lift Hessian and Multipliers for warm start.           */
  /***********************************************************************/
  if ( warm_start == 1 ) {
    temp_mat_ptr = rhs[11];
    /* Only compute Hessian if lambda is supplied with multipliers. */
    if (nparam != old_nparam || 
	mxGetM(temp_mat_ptr)*mxGetN(temp_mat_ptr) > 1) {
      /* Get extended istate and Clambda */
      if ( mxGetM(temp_mat_ptr) != lambda_size || mxGetN(temp_mat_ptr) != 2 )
	error("Extended vector of Lagrange multipliers wrong size.");
      /* Resize Clambda[] and istate[]. */
      free(istate); free(Clambda);
      Clambda = (double *)calloc(lambda_size,sizeof(double));
      istate = (int *)calloc(lambda_size,sizeof(int));
      for ( i = 0 ; i < lambda_size ; i++ ) {
	Clambda[i] = mxGetPr(temp_mat_ptr)[i];
	istate[i]  = (int) mxGetPr(temp_mat_ptr)[lambda_size+i];
      }
      mexPutFull("sqp_new_lambda",1,lambda_size,Clambda,NULL);

      /* Lift Hessian into higher dimensional space. */
#if DISCRETE_HESSIAN

      if (Finite_Difference_p)
	error("Won't compute Hessian by finite diff. without analytic gradients.");
      mexPutFull("sqp_x0",nstates,1,x0,NULL);
      mexPutFull("sqp_u",ninputs,length_u,mxGetPr(u),NULL);
      mexPutFull("sqp_time",1,points,time,NULL);
      mexPutFull("sqp_ialg",mxGetM(ialg),mxGetN(ialg),mxGetPr(ialg),NULL);
      mexPutFull("sqp_min_var",1,1,&Min_Variation,NULL);
      dfree_x0 = (double*) mxCalloc(n_free_x0,sizeof(double));
      for ( i = 0 ; i < n_free_x0 ; i++ )
	dfree_x0[i] = (double) free_x0[i];
      mexPutFull("sqp_free_x0",n_free_x0,1,dfree_x0,NULL);
      mxFree(dfree_x0);
      mexPutFull("sqp_scaling",1,nfunctions+nineq+neq,function_scaling+1,NULL);
      mexPutFull("sqp_Q",1,length_u,Q,NULL);
#if TRANSFORM
      if ( Spline_p ) {
	sprintf(sim_command,
		"sqp_new_R=comp_hess(sqp_x0,sqp_u,sqp_time,sqp_ialg,sqp_new_lambda,sqp_free_x0,sqp_scaling,%d,%d,1,sqp_min_var);",Order,points);
      }
      else {
	sprintf(sim_command,
		"sqp_new_R=comp_hess(sqp_x0,sqp_u,sqp_time,sqp_ialg,sqp_new_lambda,sqp_free_x0,sqp_scaling,%d,%d,1,sqp_min_var,sqp_Q);",Order,points);
      }
#else
      sprintf(sim_command,
	     "sqp_new_R=comp_hess(sqp_x0,sqp_u,sqp_time,sqp_ialg,sqp_new_lambda,sqp_free_x0,sqp_scaling,0,%d,1,sqp_min_var);",points);
#endif
      failure = mexEvalString(sim_command);
      mexEvalString("clear sqp_u sqp_x0 sqp_time sqp_ialg sqp_free_x0");
      mexEvalString("clear sqp_new_lambda sqp_min_var sqp_Q");

#else

      mexEvalString("clear sqp_new_lambda");
      mexPutFull("sqp_R",old_nparam,old_nparam,R,NULL);
#if TRANSFORM
      sprintf(sim_command,
	      "sqp_new_R = lift_Hessian(sqp_R,1,%d,1,%d,%d);",
	      n_free_x0,Order,ninputs);
#else
      sprintf(sim_command,
	      "sqp_new_R = lift_Hessian(sqp_R,1,%d,2,%d,%d);",
	      n_free_x0,Order,ninputs);
#endif
      failure = mexEvalString(sim_command);
      mexEvalString("clear sqp_R");

#endif
      /* Resize R. */
      free(R);
      R = (double *)calloc(nparam,nparam*sizeof(double));
      if ( failure ) {
	mexEvalString("clear sqp_new_R");
#if 0
	mexPrintf("Could not get Hessian...using the identity matrix.\n");
	/* Store identity into R. */
	for ( i = 0 ; i < nparam ; i++ )
	  R[i*nparam + i] = 1.0;
#endif
	mexPrintf("Could not get Hessian...performing cold start.\n");
	warm_start = FALSE;
      }
      else {
	temp_mat_ptr = mexGetMatrixPtr("sqp_new_R");
	if (temp_mat_ptr == NULL || 
	    mxGetN(temp_mat_ptr) != nparam || mxGetM(temp_mat_ptr) != nparam)
	  error("Lifted Hessian is not the right size.");
	R_ptr = mxGetPr(temp_mat_ptr);
	/* Store upper triangular Cholesky factor of new Hessian.      */
	/* Remember: both Matlab and Fortran store matrices tranposed. */
	for ( i = 0 ; i < nparam ; i++ )
	  for ( j = 0 ; j <= i ; j++ )
	    R[i*nparam+j] = R_ptr[i*nparam+j];
	mexEvalString("clear sqp_new_R");
      }
    }
    else
      if (!Hessian_stored_p)
	error("Can't use previous Hessian without proper cold start.");
  }
  else {
    if (memory_initialized) {
      free(++function_scaling); free(istate); free(Clambda); free(R);
      memory_initialized = FALSE;
    }
  }
  if ( !memory_initialized ) {
    R = (double *)calloc(nparam,nparam*sizeof(double));
    Clambda = (double *)calloc(lambda_size,sizeof(double));
    istate = (int *)calloc(lambda_size,sizeof(int));
    function_scaling = (double*)calloc(nfunctions+nineq+neq,sizeof(double));
    function_scaling--;
    warm_start = FALSE;
    memory_initialized = TRUE;
  }
  old_nparam = nparam;
  old_lambda_size = lambda_size;
  old_points = points;
  old_scale_size = f_size + g_size;

  /******************************************************************/

  objgrad = (double *)mxCalloc(nparam,sizeof(double));
  tmp_grad = (double *)mxCalloc(nparam,sizeof(double));
  leniw = 3*nparam + nclin + 2*ncnln;
  lenw  = 20*nparam;
  if ( nclin > 0 || ncnln > 0 ) lenw += 2*nparam*nparam + 11*nclin;
  if ( ncnln > 0 ) lenw += nparam*nclin + (2*nparam+21)*ncnln;
  iw = (int *)mxCalloc(leniw,sizeof(int));
  w  = (double *)mxCalloc(lenw,sizeof(double)); 
  bu = (double *)mxCalloc(nparam+nclin+ncnln,sizeof(double));
  bl = (double *)mxCalloc(nparam+nclin+ncnln,sizeof(double));
#else
  bu = (double *)mxCalloc(nparam,sizeof(double));
  bl = (double *)mxCalloc(nparam,sizeof(double));

  function_scaling = (double*)mxCalloc(nfunctions+nineq+neq,sizeof(double));
  function_scaling--;
#endif

  /******************************************************************/

  /* Store lower and upper control bounds.                */
  /* If no bounds given, set lower bounds to -bigbnd and  */
  /* upper bound to bigbnd.                               */
  if ( mxGetPr(rhs[3]) == NULL )
    for ( i = 0 ; i < ncontrols ; i++ )
      bl[i] = -bigbnd;
  else
#if TRANSFORM
#if TRANSFORM == 2
    if (Spline_p)
      error("Can't use TRANSFORM = 2 with higher-order splines.");
#endif
    transform(mxGetPr(Umin),bl);
#else
  for ( i = 0 ; i < ncontrols ; i++ ) 
    bl[i] = mxGetPr(Umin)[i];
#endif
  if ( mxGetPr(rhs[4]) == NULL )
    for ( i = 0 ; i < ncontrols ; i++ )
      bu[i] = bigbnd;
  else
#if TRANSFORM
    transform(mxGetPr(Umax),bu);
#else
  for ( i = 0 ; i < ncontrols ; i++ ) 
    bu[i] = mxGetPr(Umax)[i];
#endif

  /* If there are any free initial conditions, store upper and */
  /* lower bounds for them, if specified.                      */
  for ( i = 0 ; i < n_free_x0 ; i++ ) 
    if ( mxGetN(init_states) == 4 ) {
      bl[i+ncontrols] = mxGetPr(init_states)[2*nstates+free_x0[i]];
      bu[i+ncontrols] = mxGetPr(init_states)[3*nstates+free_x0[i]];
    }
    else {
      bl[i+ncontrols] = -bigbnd;
      bu[i+ncontrols] = bigbnd;
    }

  /*************************************************************/
  /*     Make sure upper and lower bounds are satisfied.       */
  /*************************************************************/
  for ( i = 0 ; i < nparam ; i++ ) {
    if ( z[i] < bl[i] ) z[i] = bl[i];
    if ( z[i] > bu[i] ) z[i] = bu[i];
  }
  
#ifdef NPSOL
  /* Set the last nclin+ncnln values of bu[] to zero and set    */
  /* bl[] to zero for equality constraints, -bigbnd for         */
  /* inequality constraints.   The order of the constraints is: */
  /*     linear inequality,     linear equality,                */
  /*     nonlinear inequality,  nonlinear equality.             */
  /* The bounds for linear constraints are computed in the call */
  /* to compute_linear_constraints() below.                     */
  /* Eventually I may want to allow two-sided inequalities,     */
  /* but for now I'll keep it compatible with calls to CFSQP.   */
  for ( i = 0 ; i < ncnln ; i++ ) {
    bu[i+nparam+nclin] = 0.0;
    if ( i < nle_ineq + nlt_ineq*points )
      bl[i+nparam+nclin] = -bigbnd;	/* nonlinear inequality constraints. */
    else
      bl[i+nparam+nclin] = 0.0;		  /* nonlinear equality constraints. */
  }
#endif

  Mx0      = mxCreateFull(nstates,1,REAL);
  Mcontrol = mxCreateFull(ninputs,length_u,REAL);
  Mf_num   = mxCreateFull(1,1,REAL); 
  Mtime    = mxCreateFull(1,points,REAL);
  Maction  = mxCreateFull(1,1,REAL);
  x_soln   = mxCreateFull(nstates,points,REAL);
  for ( i = 0 ; i < points ; i++ )
    mxGetPr(Mtime)[i] = time[i];
  for ( i = 0 ; i < nstates ; i++ )
    mxGetPr(Mx0)[i] = x0[i];

  f_calls = g_calls = c_calls = 0 ;
#ifdef NPSOL
  if ( !warm_start ) {
    for ( i = 1 ; i <= nfunctions+nineq+neq ; i++ )
      function_scaling[i] = 1.0;
    scale(nparam,scale_p,z,bl,bu,&f_calls,&g_calls,&c_calls);
    previous_scale_p = scale_p;
  }
  else
    scale_p = previous_scale_p;
#else
  for ( i = 1 ; i <= nfunctions+nineq+neq ; i++ )
    function_scaling[i] = 1.0;
  scale(nparam,scale_p,z,bl,bu,&f_calls,&g_calls,&c_calls);
#endif
  
  x_is_new = TRUE;
  IEEE_error = FALSE;
#ifndef NPSOL
  /***********************************************************************/
  /*           Call  CFSQP  to solve the discretize problem.             */
  /***********************************************************************/

  if (disp) mexPrintf("\nCalling CFSQP.\n");
  mexEvalString("global opt_program");
  mexEvalString("opt_program = 'CFSQP';");

  if (disp)
    iprint = 12;
  else
    iprint = 1;
  if ( !Finite_Difference_p )
    cfsqp(nparam,nf,nfsr,nineqn,nineq,neqn,neq,ncsrl,ncsrn,mesh_pts,
	  mode,iprint,miter,&inform,bigbnd,eps,epsneq,udelta,bl,bu,
	  z,mxGetPr(f),mxGetPr(g),mxGetPr(lambda),obj,constr,gradob,gradcn,cd);
  else
    cfsqp(nparam,nf,nfsr,nineqn,nineq,neqn,neq,ncsrl,ncsrn,mesh_pts,
	  mode,iprint,miter,&inform,bigbnd,eps,epsneq,udelta,bl,bu,
	  z,mxGetPr(f),mxGetPr(g),mxGetPr(lambda),obj,constr,grobfd,grcnfd,cd);

#else
  /***********************************************************************/
  /*           Call  NPSOL  to solve the discretize problem.             */
  /***********************************************************************/

  /* Set NPSOL defaults.  Can't use npoptn_() because */
  /* FORTRAN doesn't know about '\0'.                 */

  fp = fopen("npsol.opt","w");
  fprintf(fp,"Begin\n");
  fprintf(fp,"Nolist\n");
  fprintf(fp,"Derivative Level = 0\n");
  fprintf(fp,"Infinite Bound Size = %g\n",bigbnd);
  fprintf(fp,"Iteration Limit = %d\n",miter);
  if (disp)
    fprintf(fp,"Major Print level = 5\n");
  else
    fprintf(fp,"Major Print level = 0\n");
  fprintf(fp,"Optimality Tolerance = %g\n",eps*eps);
  fprintf(fp,"Central Difference Interval = 1.0e-5\n");
  fprintf(fp,"Difference Interval = 1.0e-6\n");
  if (epsneq > 0.0) fprintf(fp,"Feasibility Tolerance = %g\n",epsneq);
  if (objrep > 0.0) 
    fprintf(fp,"Function Precision = %g\n",objrep);
  else
    fprintf(fp,"Function Precision = %g\n",eps*eps);
#define CHECK_GRAD 0
#if CHECK_GRAD
  /* If doing a gradient verification, do not let "Function Precision = 0". */
  if ( objrep == 0.0 ) fprintf(fp,"Function Precision = 1.0e-6\n");
  fprintf(fp,"Verify Level = 1\n");
#else
  fprintf(fp,"Verify Level = -1\n");
#endif
  /* The next option is to allow warm starts for subsequent calls to RIOTS. */
  if (unwind_Hessian_p)
    fprintf(fp,"Hessian = Yes\n"); 
  if (warm_start) 
    fprintf(fp,"Warm Start\n");
  else
    fprintf(fp,"Cold Start\n");
  fprintf(fp,"End\n");
  fclose(fp);
  ioptns = 9;			       /* I don't think this number matters. */
  npfile_(&ioptns,&inform);
  if (inform) error("Couldn't read npsol.opt option file.");

  resimulate(z);
  for ( i = 0 ; i < nparam ; i++ ) z_old[i] = z[i];

  compute_linear_constraints(nparam,nclin,nrowa,bigbnd,z,
			     bl+nparam,bu+nparam,A);

  iter = 0;
  if (disp) mexPrintf("\nCalling NPSOL.\n");
  mexEvalString("global opt_program");
  mexEvalString("opt_program = 'NPSOL';");

  Hessian_stored_p = FALSE;
  npsol_(&nparam,&nclin,&ncnln,&nrowa,&nrowj,&nparam,A,bl,bu,confun,objfun,
	 &inform,&iter,istate,mxGetPr(g),CJAC,Clambda,mxGetPr(f),
	 objgrad,R,z,iw,&leniw,w,&lenw);

  if ( unwind_Hessian_p && !( inform == 2 || inform == 3 || inform > 6 ) )
    Hessian_stored_p = TRUE;

  mexPrintf("\nCompleted %d  riots  iterations.  ",iter);
  /* Compute values for the linear constraints.         */
  /* g_i(z) = g_i(0) + <A_i,z> ; g_i(0) = -bu[nparam+i] */
  for ( i = 0 ; i < nclin ; i++ ) {
    val_lin_constr = -bu[nparam+i];
    for ( j = 0 ; j < nparam ; j++ )
      val_lin_constr += A[j*nrowa+i]*z[j];
    mxGetPr(g)[ncnln+i] = val_lin_constr;
  }

#if TRANSFORM
  inv_transform(Clambda,mxGetPr(lambda));
  for ( i = ncontrols ; i < lambda_size ; i++ )
    mxGetPr(lambda)[i] = Clambda[i];    
  for ( i = 0 ; i < lambda_size ; i++ )
    mxGetPr(lambda)[lambda_size+i] = (double) istate[i];
#else
  for ( i = 0 ; i < lambda_size ; i++ ) {
    mxGetPr(lambda)[i] = Clambda[i];
    mxGetPr(lambda)[lambda_size+i] = (double) istate[i];
  }
#endif

  /***********************************************************************/
#endif
    
  mxFreeMatrix(Mform); mxFreeMatrix(Mx0); mxFreeMatrix(Mcontrol);
  mxFreeMatrix(Maction);
    
  unscale(nparam,scale_p,z,mxGetPr(f),mxGetPr(g));

  u_soln = mxCreateFull(ninputs,length_u,REAL);
#if TRANSFORM
  inv_transform(z,mxGetPr(u_soln));
#else
  /* Note that ncontrols = length_u*ninputs. */
  for ( i = 0 ; i < ncontrols ; i++ )
      mxGetPr(u_soln)[i] = z[i];
#endif

  switch (nlhs) {
  case 5:
    lhs[4] = lambda;
  case 4:
    lhs[3] = g;
  case 3: 
    lhs[2] = f;
  case 2: 
    lhs[1] = x_soln;
  case 1:
    lhs[0] = u_soln;
  }

  switch (nlhs) {
  case 0:
    mxFreeMatrix(u_soln);
  case 1:
    mxFreeMatrix(x_soln);
  case 2: 
    mxFreeMatrix(f);
  case 3:
    mxFreeMatrix(g);
  case 4:
    mxFreeMatrix(lambda);
  }

  display_info(inform);
  if (scale_p && disp)
    mexPrintf("Extra function, gradient and constraint calls for scaling = %d, %d, %d\n",f_calls,g_calls,c_calls);

  return;
}

/**************************************************************************/
/* This function gets called by obj(), gradob(), constr(), or gradcn() if */
/* x_is_new to TRUE.  It resimulates the system dynamics with the new z   */
/**************************************************************************/
static void 
resimulate(z)
     double *z;
{
  int i,failure;
  Matrix *lhs[2],*rhs[6];

#if TRANSFORM
  inv_transform(z,utemp);
  for ( i = 0 ; i < ncontrols ; i++ )
    mxGetPr(Mcontrol)[i] = utemp[i];
#else
  for ( i = 0 ; i < ncontrols ; i++ )
    mxGetPr(Mcontrol)[i] = z[i];
#endif
  for ( i = 0 ; i < n_free_x0 ; i++ )
    mxGetPr(Mx0)[free_x0[i]] = z[ncontrols+i];

  *mxGetPr(Maction) = 2.0; *mxGetPr(Mform) = 1.0;

  rhs[0] = Mform; rhs[1] = Mx0; rhs[2] = Mcontrol;
  rhs[3] = Mtime; rhs[4] = ialg; rhs[5] = Maction; 

  /***********************************************/
  /* [sqp_J1,x]=simulate(1,x0,u0,t,ialg,action); */
  /***********************************************/
#ifdef DIRECT
  simulate(2,lhs,6,rhs);
#else
  failure = mexCallMATLAB(2,lhs,6,rhs,"simulate");
  if (failure)
    error("Couldn't simulate system.");
#endif

  sqp_J1 = mxGetScalar(lhs[0]);
  /* Check to see if simulate returned  NaN */
  /* indicating a floating point error.     */
  if ( mexIsNaN(sqp_J1) ) {
    IEEE_error = TRUE;
    return;
  }

  for ( i = 0 ; i < nstates*points ; i++ )
    mxGetPr(x_soln)[i] = mxGetPr(lhs[1])[i];

  mxFreeMatrix(lhs[0]); mxFreeMatrix(lhs[1]);
  x_is_new = FALSE;
  return;
}

/***************************************************************/
/* This function returns the value of the j-th objective fnc.  */
/***************************************************************/
static void 
obj(nparam,j,z,fj,cd)
     void *cd;
     int nparam,j;
     double *z,*fj;
{
  int failure,i,k;
  double temp,cost,idelta1,idelta2;
  Matrix *rhs[3],*lhs[1];
  
  if (x_is_new) resimulate(z);
#ifdef NPSOL
  if (IEEE_error) return;
#else
  if (IEEE_error) {
    error("Floating point error during objective computation.");
  }
#endif

  if ( j == 1 ) {
    /* No sense in recomputing sqp_J1. */
    cost = sqp_J1;
  }
  else {
    *mxGetPr(Mform) = 2.0; rhs[0] = Mform;
    *mxGetPr(Mf_num) = (double)j; rhs[1] = Mf_num;
    *mxGetPr(Maction) = 1.0; rhs[2] = Maction;

    /***************************/
    /* cost = simulate(2,j,1); */
    /***************************/
#ifdef DIRECT
    simulate(1,lhs,3,rhs); 
#else
    failure = mexCallMATLAB(1,lhs,3,rhs,"simulate");
    if (failure)
      error("Couldn't evaluate objective function.");
#endif
    cost = mxGetScalar(lhs[0]); 
    if ( mexIsNaN(cost) )
      error("Floating point error during objective function computation.");
    mxFreeMatrix(lhs[0]);
  }
  *fj = function_scaling[j]*cost;
  
  /****************************************************/
  /* Add in term for variation of control derivative. */
  /****************************************************/
  if ( Min_Variation > 0.0 ) {
#if TRANSFORM
    /* Note that utemp[] will already be loaded by resimulate() */
    /* with u[] in untransformed coordinates.                   */
#define Z utemp
#else
#define Z z
#endif  

    for ( i = 0 ; i < length_u-2 ; i++ ) {
      idelta1 = 1.0/(time[i+1] - time[i]);
      idelta2 = 1.0/(time[i+2] - time[i+1]);
      for ( k = 0 ; k < ninputs ; k++ ) {
	temp=fabs((Z[(i+2)*ninputs+k] - Z[(i+1)*ninputs+k])*idelta2 -
		  (Z[(i+1)*ninputs+k] - Z[i*ninputs+k])*idelta1);
#if L1
	if ( temp < RHO ) temp = (temp*temp+RHO*RHO)/(2*RHO);
	*fj += Min_Variation*temp;
#else
	*fj += Min_Variation*temp*temp/2;
#endif
      }
    }
  }
}

/***************************************************************/
/* This function computes the gradients of the objectives fncs.*/
/***************************************************************/
static void 
gradob(nparam,j,z,gradfj,dummy,cd)
     int nparam,j;
     double *z,*gradfj;
     void (* dummy)(),*cd;
{
  int i,k,failure;
  double temp1,temp2,idelta1,idelta2;
  double *grad,*grad_x0;
  Matrix *Mgrad,*Mgrad_x0,*rhs[3],*lhs[2];

  if (x_is_new) resimulate(z);
  if (IEEE_error) {
    error("Couldn't simulate system during obj. gradient computation.");
  }

  *mxGetPr(Mform) = 2.0; rhs[0] = Mform;
  *mxGetPr(Mf_num) = (double)j; rhs[1] = Mf_num;
  *mxGetPr(Maction) = 2.0; rhs[2] = Maction;

  /*************************************/
  /* [grad_u,grad_x0]=simulate(2,j,2); */
  /*************************************/
#ifdef DIRECT
  simulate(2,lhs,3,rhs);
#else
  failure = mexCallMATLAB(2,lhs,3,rhs,"simulate");
  if (failure)
    error("Couldn't evaluate objective gradient.");
#endif

  Mgrad    = lhs[0]; grad    = mxGetPr(Mgrad);
  Mgrad_x0 = lhs[1]; grad_x0 = mxGetPr(Mgrad_x0);

  if ( mexIsNaN(mxGetScalar(Mgrad)) )
    error("Floating point error during objective gradient computation.");

  if (scale_p) {
    for ( i = 0 ; i < ncontrols ; i++ )
      grad[i] *= function_scaling[j];
    for ( i = 0 ; i < n_free_x0 ; i++ )
      grad_x0[free_x0[i]] *= function_scaling[j];
  }

  /*******************************************************/
  /* Add in gradient of variation of control derivative. */
  /*******************************************************/
  if ( Min_Variation > 0.0 ) {
#if TRANSFORM
    /* Note that utemp[] will already be loaded by resimulate() */
    /* with u[] in untransformed coordinates.                   */
#define Z utemp
#else
#define Z z
#endif  
    for ( i = 0 ; i < length_u-2 ; i++ ) {
      idelta1 = 1.0/(time[i+1] - time[i]);
      idelta2 = 1.0/(time[i+2] - time[i+1]);
      for ( k = 0 ; k < ninputs ; k++ ) {
	temp1=((Z[(i+2)*ninputs+k] - Z[(i+1)*ninputs+k])*idelta2 -
	       (Z[(i+1)*ninputs+k] - Z[i*ninputs+k])*idelta1);
#if L1
	if ( temp1 < -RHO ) 
	  temp2 = -Min_Variation;
	else 
	  if ( temp1 > RHO ) 
	    temp2 = Min_Variation;
	  else 
	    temp2 = Min_Variation*temp1/RHO;
#else
	temp1 *= Min_Variation;
#endif
	grad[i*ninputs+k] += temp1*idelta1;
	grad[(i+1)*ninputs+k] -= temp1*(idelta1+idelta2);
	grad[(i+2)*ninputs+k] += temp1*idelta2;
      }
    }
  }

#if TRANSFORM
  /* The gradients from both the original objective function as     */
  /* we as the gradient for the total variation of the control      */
  /* derivative have to be untransformed into original coordinates. */
  inv_transform(grad,gradfj);
#else
  for ( i = 0 ; i < ncontrols ; i++ )
    gradfj[i] = grad[i];
#endif
  for ( i = 0 ; i < n_free_x0 ; i++ )
    gradfj[i+ncontrols] = grad_x0[free_x0[i]];
  mxFreeMatrix(Mgrad);
  mxFreeMatrix(Mgrad_x0);
}

/***************************************************************/
/* This function computes the value of constraints, including  */
/* equality constraints from the collocation conditions.       */
/***************************************************************/
static void 
constr(nparam,j,z,gj,cd)
     void *cd;
     int nparam,j;
     double *z,*gj;
{
  int failure,constraint,l;
  Matrix *lhs[1],*rhs[3];

  if (x_is_new) resimulate(z);
#ifdef NPSOL
  if (IEEE_error) return;
#else
  if (IEEE_error) {
    *gj = 1.0;
    return;
  }
#endif

  /*******************************************/
  /* The order of constraints is as follows: */
  /*   nonlinear endpoint inequality         */
  /*   nonlinear trajectory inequality       */
  /*   linear endpoint inequality            */
  /*   linear trajectory inequality          */
  /*   nonlinear endpoint equality           */
  /*   linear endpoint equality              ****************************/
  /* In the integration routines, the order of function evaluations is: */
  /* objectives, trajectory constraints and then endpoint constraints.  */
  /* and within each category nonlinear constraints proceed linear one. */
  /*                                                                    */
  /* Note: in the calculation below, (j-l-1)/points is integer division.*/
  /**********************************************************************/
  if ( j <= nle_ineq ) {
    /* This is nonlinear endpoint inequality. */
    constraint = nfunctions + j + (nlt_ineq + lt_ineq)*points;
    l = nfunctions + j;
  }
  else 
    if ( j <= nle_ineq + nlt_ineq*points ) {
      /* This is nonlinear trajectory inequality. */
      constraint = nfunctions + j - nle_ineq;
      l = nle_ineq;
      l = nfunctions + l + (j-l-1)/points + 1;
    }
    else
      if ( j <= nle_ineq + nlt_ineq*points + le_ineq ) {
	/* This is linear endpoint inequality. */
	constraint = nfunctions + j + lt_ineq*points;
	l = nfunctions + nle_ineq + nlt_ineq + j;
      }
      else
	if ( j <= nle_ineq + (nlt_ineq+lt_ineq)*points + le_ineq ) {
	  /* This is linear trajectory inequality. */
	  constraint = nfunctions + j - nle_ineq - le_ineq;
	  l = nle_ineq + nlt_ineq + le_ineq;
	  l = nfunctions + l + (j-l-1)/points + 1;
	}
	else {
	  /* This is nonlinear and linear endpoint equality. */
	  constraint = nfunctions + j;
	  l = nfunctions + nle_ineq + nlt_ineq + le_ineq + lt_ineq;
	}
  
  *mxGetPr(Mform)  = 2.0; 
  *mxGetPr(Mf_num) = (double)constraint;
  *mxGetPr(Maction) = 1.0;
  rhs[0] = Mform; rhs[1] = Mf_num; rhs[2] = Maction; 
  
  /******************************************/
  /* constr_val = simulate(2,constraint,1); */
  /******************************************/
#ifdef DIRECT
  simulate(1,lhs,3,rhs);
#else
  failure = mexCallMATLAB(1,lhs,3,rhs,"simulate");
#endif
  if (failure)
    error("Couldn't evaluate constraint.");

#ifdef NPSOL
  if ( mexIsNaN(mxGetScalar(lhs[0])) ) {
    IEEE_error = TRUE;
    return;
  }
#else
  if ( mexIsNaN(mxGetScalar(lhs[0])) )
    error("Floating point error during constraint computation.");
#endif

  *gj = mxGetScalar(lhs[0])*function_scaling[l];
  mxFreeMatrix(lhs[0]);
}

/***************************************************************/
/* Compute the gradients of the constraint functions.          */
/***************************************************************/
static void 
gradcn(nparam,j,z,gradgj,dummy,cd)
     int nparam,j;
     double *z,*gradgj;
     void (* dummy)(),*cd;
{
  int i,failure,constraint,l,m,n;
  Matrix *grad,*grad_x0,*lhs[2],*rhs[3];

  if (x_is_new) resimulate(z);
  if (IEEE_error) {
    error("Couldn't simulate system during cnstr. grad. computation.");
  }

  /* First compute derivative E_x(t,xi,xf)*dxf/du. */
  if ( j <= nle_ineq ) {
    constraint = nfunctions + j + (nlt_ineq + lt_ineq)*points;
    l = nfunctions + j;
  }
  else 
    if ( j <= nle_ineq + nlt_ineq*points ) {
      constraint = nfunctions + j - nle_ineq;
      l = nle_ineq;
      l = nfunctions + l + (j-l-1)/points + 1;
    }
    else
      if ( j <= nle_ineq + nlt_ineq*points + le_ineq ) {
	constraint = nfunctions + j + lt_ineq*points;
	l = nfunctions + nle_ineq + nlt_ineq + j;
      }
      else
	if ( j <= nle_ineq + (nlt_ineq+lt_ineq)*points + le_ineq ) {
	  constraint = nfunctions + j - nle_ineq - le_ineq;
	  l = nle_ineq + nlt_ineq + le_ineq;
	  l = nfunctions + l + (j-l-1)/points + 1;
	}
	else {
	  constraint = nfunctions + j;
	  l = nfunctions + nle_ineq + nlt_ineq + le_ineq + lt_ineq;
	}

  *mxGetPr(Mform) = 2.0; *mxGetPr(Mf_num) = (double)constraint;
  *mxGetPr(Maction) = 2.0;
  rhs[0] = Mform; rhs[1] = Mf_num; rhs[2] = Maction; 

  /************************************************/
  /* [grad_u,grad_x0] = simulate(2,constraint,2); */
  /************************************************/
#ifdef DIRECT
  simulate(2,lhs,3,rhs);
#else
  failure = mexCallMATLAB(2,lhs,3,rhs,"simulate");
  if (failure) 
    error("Couldn't evaluate constraint gradient.");
#endif
  grad = lhs[0];
  grad_x0 = lhs[1];

  if ( mexIsNaN(mxGetScalar(grad)) )
    error("Floating point error during constraint gradient computation.");

#if TRANSFORM
  inv_transform(mxGetPr(grad),gradgj);
#else
  /* Note: ncontrols = length_u*ninputs. */
  for ( i = 0 ; i < ncontrols ; i++ )
    gradgj[i] = mxGetPr(grad)[i];
#endif
  for ( i = 0 ; i < n_free_x0 ; i++ )
    gradgj[i+ncontrols] = mxGetPr(grad_x0)[free_x0[i]];
  mxFreeMatrix(grad);
  mxFreeMatrix(grad_x0);

  if (scale_p)
    for ( i = 0 ; i < nparam ; i++ )
      gradgj[i] *= function_scaling[l];
}

/***************************************************************/
/* This function is called only before the SQP is executed.    */
/* It determines the scaling appropriate to the selected       */
/* integration alogrithm.                                      */
/***************************************************************/
static void 
transform(u1,u2)
     double *u1,*u2;
{
  int i,j,failure,m,n;
  double old_delta,delta,*tempr,*tempi;

  /* Compute the transformation, M. But store Q = M^(-1/2)    */
  /* since inv_transform() called much more than transform(). */
  if (!transform_computed_p) {
    Spline_p = FALSE;
#if TRANSFORM == 2
    switch (Order) {
    case 1:
      /* Piecewise constant. */
      for ( i = 0 ; i < length_u ; i++ ) {
	delta = time[i+1] - time[i];
	Q[i] = 1.0/sqrt(delta);
      }
      break;
    case 2:
      /* Piecewise, continuous linear. */
      for ( i = 0 ; i < length_u ; i++ ) {
	if (i < N) delta = time[i+1] - time[i];
	if ( i == 0 || i == N )
	  Q[i] = 1.0/sqrt(delta/2);
	else
	  Q[i] = 1.0/sqrt((old_delta + delta)/2);
	old_delta = delta;
      }
      break;
    }
#else  
    switch (Order) {
    case 1:
      /* Piecewise constant. */
      for ( i = 0 ; i < length_u ; i++ ) {
	delta = time[i+1] - time[i];
	Q[i] = 1.0/sqrt(delta);
      }
      break;
    case 2:
      /* Piecewise, continuous linear.                        */
      /* If there are control bounds, use diagonal transform. */
      /* Otherwise, fall through to next case statement which */
      /* sets up the real spline transform matrix.            */
      if ( Bounds_p || driver == 2 ) {
	for ( i = 0 ; i < length_u ; i++ ) {
	  if (i < N) delta = time[i+1] - time[i];
	  if ( i == 0 || i == N )
	    Q[i] = 1.0/sqrt(delta/2);
	  else
	    Q[i] = 1.0/sqrt((old_delta + delta)/2);
	  old_delta = delta;
	}
	break;
      }
    case 3:
    case 4:
      /* B-splines of order 3,4 or 2 if Bounds_p = FALSE. */
      Spline_p = TRUE;
      mexPrintf("\nComputing spline variable transformation...\n");
      /* First create scaling matrix Malpha. */
      mexPutFull("sqp_time",1,points,time,NULL);
      sprintf(sim_command,"sqp_Malpha = transform(sqp_time,%d);",Order);
      failure = mexEvalString(sim_command);
      if (failure)
	error("Failed to compute the spline scaling matrix.");
      /* Compute Schur decomposition of Malpha.  We'll save sqp_U for */
      /* later use in inv_transform().                                */
      failure = mexEvalString("[sqp_U,sqp_D] = schur(full(sqp_Malpha));");
      if (failure)
	error("Couldn't compute Schur decomposition of scaling matrix.");
      /* Compute sqrt(T^(-1)) for use later in inv_transform(). */
      /* Make it a global variable so comp_hess() can acces it. */
      mexEvalString("sqp_rtD = sqrt(diag(sqp_D));");
      mexEvalString("sqp_size_D = length(sqp_rtD);global sqp_sqrt_M_inv");
      mexEvalString("sqp_sqrt_M_inv = sqp_U*spdiags(ones(sqp_size_D,1)./sqp_rtD,0,sqp_size_D,sqp_size_D)*sqp_U';");
      mexEvalString("clear sqp_D sqp_Malpha sqp_time");
      mexPrintf("done.\n");
      break;
    default:
      /* This is in case I'm using RK33 or RK44. */
      for ( i = 0 ; i < length_u ; i++ ) {
	if ( i < N ) delta = time[i+1] - time[i];
	if ( i % 3 == 1 )
	  Q[i] = 1.0/sqrt(delta*2/3);
	else
	  Q[i] = 1.0/sqrt(delta/6);
      }
      break;
    }
#endif

    transform_computed_p = TRUE;
  }
 
  if (!Spline_p)
    /* Transform the control. */
    for ( i = 0 ; i < length_u ; i++ )
      for ( j = 0 ; j < ninputs ; j++ ) 
	u2[i*ninputs+j] = u1[i*ninputs+j]/Q[i];
  else {
    mexPutFull("sqp_u1",ninputs,length_u,u1,NULL);
    failure = mexEvalString("sqp_u2 = sqp_u1*sqp_U*spdiags(sqp_rtD,0,sqp_size_D,sqp_size_D)*sqp_U';");
    if (failure)
      error("Error in performing spline coordinate transformation.");
    mexGetFull("sqp_u2",&m,&n,&tempr,&tempi);
    for ( i = 0 ; i < length_u ; i++ )
      for ( j = 0 ; j < ninputs ; j++ )
	u2[i*ninputs+j] = tempr[i*ninputs+j];
    mxFree(tempr);
    mexEvalString("clear sqp_u1 sqp_u2");
  }
}

/***************************************************************/
/* Perform the pre-computed scaling inverse-transformation.    */
/***************************************************************/
static void 
inv_transform(u1,u2)
     double *u1,*u2;
{
  int i,j,m,n,failure;
  double *tempr,*tempi;

  if (!Spline_p)
    for ( i = 0 ; i < length_u ; i++ )
      for ( j = 0 ; j < ninputs ; j++ )
	u2[i*ninputs+j] = u1[i*ninputs+j]*Q[i];
  else {
    mexPutFull("sqp_u1",ninputs,length_u,u1,NULL);
    failure = mexEvalString("sqp_u2 = sqp_u1*sqp_sqrt_M_inv;");
    if (failure)
      error("Error in performing inverse spline coordinate transformation.");
    mexGetFull("sqp_u2",&m,&n,&tempr,&tempi);
    for ( i = 0 ; i < length_u ; i++ )
      for ( j = 0 ; j < ninputs ; j++ )
	u2[i*ninputs+j] = tempr[i*ninputs+j];
    mxFree(tempr);
    mexEvalString("clear sqp_u1 sqp_u2");
  }
}

static void 
display_info(inform)
     int inform;
{
  char *message;

#ifndef NPSOL
  switch (inform) {
  case 0:
    message = "Normal Termination.\n";
    break;
  case 1:
    message = "No feasibile point found for linear constraints.\n";
    break;
  case 2:
    message = "No feasibile point found for nonlinear constraints.\n";
    break;
  case 3:
    message = "Reached the maximum number of iterations.\n";
    break;
  case 4:
    message="Line search failed to find a new iterate (stepsize too small).\n";
    break;
  case 5:
    message = "QP solver failed to construct descent direction.\n";
    break;
  case 6:
    message = "QP solver failed to construct a feasible direction.\n";
    break;
  case 7:
    message = "Input data are not consistent.\n";
    break;
  case 8:
    message = "New iterate essentially identical to previous iterate.\n";
    break;
  default:
    mexPrintf("Hum, I don't know about this message: %d.\n",inform);
    message = "";
  }
#else
  switch (inform) {
  case 0:
    message = "Normal Termination.\n";
    break;
  case 1:
    message = "Kuhn-Tucker conditions satisfied but sequence did not converge.\n";
    break;
  case 2:
    message = "No feasibile point found for linear constraints.\n";
    break;
  case 3:
    message = "No feasibile point found for nonlinear constraints.\n";
    break;
  case 4:
    message = "Completed maximum number of iterations.\n";
    break;
  case 6:
    message = "Line search failed.\n";
    break;
  case 7:
    message = "Large error in the objective or constraint gradients found.\n";
    break;
  case 9:
    message = "Invalid input parameter to NPSOL.\n";
    break;
  default:
    mexPrintf("Hum, I don't know about this message: %d.\n",inform);
    message = "";
  }
#endif

  mexPrintf(message);
}

/***************************************************************/
/* This function gets the system information and sets the      */
/* length of the constrol vector.                              */
/***************************************************************/
int startup(t,ialg,params,u,sys_info)
     int sys_info[N_INFO];
     double *t;
     Matrix *ialg,*params,*u;
{
  int length_u,i,failure;
  Matrix *info,*lhs[1],*rhs[2];

  *mxGetPr(Mform) = 0.0; rhs[0] = Mform;
  rhs[1] = params;
  /**************************************/
  /* sqp_info = simulate(0,sqp_params); */
  /**************************************/
#ifdef DIRECT
  simulate(1,lhs,2,rhs);
#else
  failure = mexCallMATLAB(1,lhs,2,rhs,"simulate");
  if (failure)
    error("Couldn't pass system parameters/get system information.");
#endif

  info = lhs[0];
  for ( i = 0 ; i < N_INFO ; i++ )
    sys_info[i] = (int) mxGetPr(info)[i];
  mxFreeMatrix(info);
  
  switch ((int) mxGetScalar(ialg)) {
  case 0:
  case 1:
    length_u = N;
    Order = 1;
    break;
  case 2:
  case 3:
    length_u = points;
    Order = 2;
    break;
  case 4:
    /* For RK4, the control can be a linear, quadratic, or cubic spline. */
    /* If not, we may be linking RK44 which doesn't use splines.         */
    Order = ((int) mxGetN(u)) - N + 1;
    if (Order > 1 && Order < 5)
      length_u = N + Order - 1;
    else {
      length_u = 3*N;
      Order = 0;
    }
    break;
  case 5:
  case 6:
    /* For lsoda Order can be 1, 2, 3 or 4. */
    Order = ((int) mxGetN(u)) - N + 1;
    length_u = N + Order - 1;
    break;
  default:
    length_u = N;
    Order = 1;
  }
  
  return length_u;
}

/*************************************************************/
/*   Compute initial scaling for objective and constraints.  */
/*   There is one scaling for each function and constriant.  */
/*************************************************************/
#ifdef NPSOL
#define FACTOR 20.0
#else
#define FACTOR 10.0
#endif
static void 
scale(nparam,scale_p,z,bl,bu,f_calls,g_calls,c_calls)
     int nparam,scale_p,*f_calls,*g_calls,*c_calls;
     double *z,*bl,*bu;
{
  void (* dummy)();
  static void project();
  static double gestimate();
  int i,j,k,l;
  double f0,f1,pert_size,max_z,max_g,norm_g;
  double *z1,*dz,*grad;

  if (scale_p) {
    mexPrintf("Computing initial scalings...\n");
    
    /*****************************************************/
    /*          First scale objective functions.         */
    /*****************************************************/
    if ( scale_p/10 == 1 ) {
      x_is_new = TRUE;
      for ( j = 1 ; j <= nfunctions ; j++ ) {
	obj(nparam,j,z,&f0,cd); (*f_calls)++;
	function_scaling[j] = FACTOR/(1.0+fabs(f0));
	mexPrintf("Objective %d scaling = %e\n",j,function_scaling[j]);
      }
    }
    if ( scale_p/10 == 2) {
      z1 = (double *)mxCalloc(nparam,sizeof(double));
      dz = (double *)mxCalloc(nparam,sizeof(double));
      grad = (double *)mxCalloc(nparam,sizeof(double));
      max_z = 0.0;
      for ( i = 0 ; i < nparam ; i++ )
	max_z = MAX(max_z,fabs(z[i]));
      
      for ( j = 1 ; j <= nfunctions ; j++ ) {

	x_is_new = TRUE;
	obj(nparam,j,z,&f0,cd); (*f_calls)++;
	gradob(nparam,j,z,grad,dummy,cd); (*g_calls)++;
	for ( i = 0 ; i < nparam ; i++ )
	  dz[i] -= grad[i];
	max_g = 0.0;
	for ( i = 0 ; i < nparam ; i++ )
	  max_g = MAX(max_g,fabs(dz[i]));
	pert_size = 1e-6*(1.0 + max_z)/(1.0 + max_g);
	for ( i = 0 ; i < nparam ; i++ )
	  dz[i] *= pert_size;
	
	project(z,dz,bl,bu,z1);		     /* z1 = [z+dz] clipped. */
	x_is_new = TRUE; 
	obj(nparam,j,z1,&f1,cd); (*f_calls)++;
	function_scaling[j] = 
	  FACTOR/gestimate(nparam,dz,grad,bl,bu,f0,f1);
	
	mexPrintf("Objective %d scaling = %e\n",j,function_scaling[j]);
      }
      mxFree(grad); mxFree(dz); mxFree(z1);
    }

    /*****************************************************/
    /*              Now scale constraints.               */
    /*****************************************************/    
    /* Scale endpoint constraint in the following order: */
    /*   nonlinear inequality                            */
    /*   linear inequality                               */
    /*   nonlinear and linear equality                   */
    /*****************************************************/    
    k = 0; l = nfunctions;
    if ( scale_p - (scale_p/10)*10 == 1 ) {
      x_is_new = TRUE;
      for ( j = 1 ; j <= nle_ineq + le_ineq + nle_eq+le_eq ; j++ ) {
	if ( j == nle_ineq+1 ) {
	  k = nle_ineq+nlt_ineq*points;
	  l = nfunctions+nle_ineq+nlt_ineq;
	}
	if ( j == nle_ineq+le_ineq+1 ) {
	  k = le_ineq+le_ineq+(nlt_ineq+nle_ineq)*points;
	  l = nfunctions+le_ineq+le_ineq+nlt_ineq+nle_ineq;
	}
	constr(nparam,k+j,z,&f0,cd); (*c_calls)++;
	max_z = MAX(fabs(f0),1.0);
	function_scaling[l+j] = FACTOR/max_z;
	mexPrintf("Endpoint constraint %d scaling = %e\n",
		  j,function_scaling[l+j]);
      }      
    }
    if ( scale_p - (scale_p/10)*10 == 2 ) {
      x_is_new = TRUE;
      z1 = (double *)mxCalloc(nparam,sizeof(double));		/* z1[] = 0. */
      grad = (double *)mxCalloc(nparam,sizeof(double));
      /* Use the given initial conditions. */
      for ( i = ncontrols ; i < nparam ; i++ )
	z1[i] = z[i];
      k = 0; l = nfunctions;
      for ( j = 1 ; j <= nle_ineq + le_ineq + nle_eq+le_eq ; j++ ) {
	if ( j == nle_ineq+1 ) {
	  k = nle_ineq+nlt_ineq*points;
	  l = nfunctions+nle_ineq+nlt_ineq;
	}
	if ( j == nle_ineq+le_ineq+1 ) {
	  k = le_ineq+le_ineq+(nlt_ineq+nle_ineq)*points;
	  l = nfunctions+le_ineq+le_ineq+nlt_ineq+nle_ineq;
	}
	constr(nparam,k+j,z,&f0,cd); (*c_calls)++;
	if ( fabs(f0) > 1.0e-3 ) {
	  function_scaling[l+j] = FACTOR/fabs(f0);
	  mexPrintf("Endpoint constraint %d scaling = %e\n",
		    j,function_scaling[l+j]);
	}
	else {
	  norm_g = 0.0;
	  gradcn(nparam,k+j,z,grad,dummy,cd); (*g_calls)++;
	  for ( l = 0 ; l < nparam ; l++ )
	    norm_g += grad[l]*grad[l];
	  norm_g = sqrt(norm_g);
	  if ( norm_g > 1.0e-3 ) {
	    function_scaling[l+j] = FACTOR/norm_g;
	    mexPrintf("Endpoint constraint %d scaling = %e\n",
		      j,function_scaling[l+j]);
	  }
	  else
	    mexPrintf("Didn't scale endpoint constraint %d.\n",j);
	}
      }
      mxFree(grad); mxFree(z1);
    }

    /*******************************************************************/
    /* Scale nonlinear trajectory inequalities, nonlinear then linear. */
    /*******************************************************************/
    if ( scale_p - (scale_p/10)*10 ) {
      for ( j = 0 ; j < nlt_ineq + lt_ineq ; j++ ) {
	if ( j < nlt_ineq ) {
	  k = nle_ineq+j*points;
	  l = nfunctions+nle_ineq+j+1;
	}
	else {
	  k = nle_ineq+le_ineq+nlt_ineq*points+j*points;
	  l = nfunctions+nle_ineq+le_ineq+nlt_ineq+j+1;
	}
	max_z = 1.0;
	for ( i = 1 ; i <= points ; i++ ) {
	  constr(nparam,k+i,z,&f0,cd); (*c_calls)++;
	  max_z = MAX(fabs(f0),max_z);
	}
	function_scaling[l] = FACTOR/max_z;
	mexPrintf("Trajectory constraint %d scaling = %e\n",
		  j,function_scaling[l]);
      }      
    }
  }
}

static double
gestimate(nparam,dz,gradfj,bl,bu,f0,f1)
     int nparam;
     double *dz,*gradfj,*bl,*bu,f0,f1;
{
  int i;
  double inner_prod1,inner_prod2;

  inner_prod1 = inner_prod2 = 0.0;
  for ( i = 0 ; i < nparam ; i++ ) {
    inner_prod1 += dz[i]*gradfj[i];
    inner_prod2 += dz[i]*dz[i];
  }

  inner_prod1 = 2*fabs((f1 - f0 - inner_prod1)/inner_prod2);
  if ( inner_prod1 < 1.0e-4 ) {
    inner_prod1 = 0.0;
    for ( i = 0 ; i < nparam ; i++ )
      inner_prod1 += gradfj[i]*gradfj[i];
    inner_prod1 = sqrt(inner_prod1);
    if ( inner_prod1 < 1.0e-3 ) 
      inner_prod1 = 1.0 + fabs(f0);
  }

  return inner_prod1;
}

/* Replaces the contents of z with [z+dz] clipped at bounds. */
static void
project(z,dz,bl,bu,z1)
     double *z,*dz,*bl,*bu,*z1;
{
 int i;
 
 /* Clip controls at their bounds. */
 for ( i = 0 ; i < length_u*ninputs ; i++ )
   z1[i] = MAX(MIN(z[i]+dz[i],bu[i]),bl[i]);

 /* Clip free initial conditions at their bounds. */
 for ( i = ncontrols ; i < ncontrols+n_free_x0 ; i++ ) 
   z1[i] = MAX(MIN(z[i]+dz[i],bu[i]),bl[i]);
}

static void
unscale(nparam,scale_p,z,f,g)
     int nparam,scale_p;
     double *z,*f,*g;
{
  int i,j,k,l;

  if (scale_p) {

    /* Unscale objective functions. */
    for ( j = 1 ; j <= nfunctions ; j++ )
      f[j-1] /= function_scaling[j];

    /* Unscale endpoint inequalities. */
    k = 0; l = nfunctions;
    for ( j = 1 ; j <= nle_ineq + le_ineq + nle_eq+le_eq; j++ ) {
      if ( j == nle_ineq+1 ) {
	k = nle_ineq+nlt_ineq*points;
	l = nfunctions+nle_ineq+nlt_ineq;
      }
      if ( j == nle_ineq+le_ineq+1 ) {
	k = le_ineq+le_ineq+(nlt_ineq+nle_ineq)*points;
	l = nfunctions+le_ineq+le_ineq+nlt_ineq+nle_ineq;
      }
      g[k+j-1] /= function_scaling[l+j];
    }

    /* Unscale trajectory inequalities. */
    for ( j = 0 ; j < nlt_ineq + lt_ineq ; j++ ) {
      if ( j < nlt_ineq ) {
	k = nle_ineq+j*points;
	l = nfunctions+nle_ineq+j+1;
      }
      else {
	k = nle_ineq+le_ineq+nlt_ineq*points+j*points;
	l = nfunctions+nle_ineq+le_ineq+nlt_ineq+j+1;
      }
      for ( i = 1 ; i <= points ; i++ )
	g[k+i-1] /= function_scaling[l];
    }      
  }
}

#ifdef NPSOL

/*************************************************************/
/*     Compute objective function and gradient.              */
/*************************************************************/
void dummy()
{
}

static int 
objfun(mode,n,x,objf,objgrd,nstate)
     int *mode, *n, *nstate;
     double *x,*objf,*objgrd;
{
  int i;

  for ( i = 0 ; i < *n ; i++ )
    if ( x[i] != z_old[i] ) {
      x_is_new = TRUE;
      break;
    }
  if (x_is_new)
    for ( i = 0 ; i < *n ; i++ )
      z_old[i] = x[i];

  if ( *mode == 0 || *mode == 2 ) {
    obj(*n,1,x,objf,cd);
    if ( IEEE_error ) {
      IEEE_error = FALSE;
      *mode = -1;
      mexPrintf("obfun: Backtracking the linesearch...\n");
      return 0;
    }
  }

  if ( !Finite_Difference_p )
    if ( *mode == 1 || *mode == 2) {
      gradob(*n,1,x,objgrd,dummy,cd);
      if ( IEEE_error )
	error("Floating error while computing objective gradient.");
    }
  return 0;
}

/*************************************************************/
/*     Compute nonlinear constraint values and gradients.    */
/*      NPSOL computed linear constraints directrly from     */
/*          the Jacobian of the linear constraints           */
/*************************************************************/
static int 
confun(mode,ncnln,n,nrowj,needc,x,c,CJAC,nstate)
     int *mode,*ncnln,*n,*nrowj,*needc,*nstate;
     double *x,*c,*CJAC;
{
  register int i,j,k,rows;
  
  for ( i = 0 ; i < *n ; i++ )
    if ( x[i] != z_old[i] ) {
      x_is_new = TRUE;
      break;
    }
  if (x_is_new)
    for ( i = 0 ; i < *n ; i++ )
      z_old[i] = x[i];

  if ( *mode == 0 || *mode == 2 ) {
    /* Compute nonlinear constraint values. */
    for ( i = 0 ; i < *ncnln ; i++ )
      if ( needc[i] > 0 ) {
	if ( i < nle_ineq+nlt_ineq*points )
	  /* This is a nonlinear inequality constraint. */
	  k = 0;
	else
	  /* This is a nonlinear equality constraint. */
	  k = le_ineq + lt_ineq*points;
	
	constr(*n,k+i+1,x,&c[i],cd);
	if ( IEEE_error ) {
	  IEEE_error = FALSE;
	  *mode = -1;
	  mexPrintf("confun: Backtracking the linesearch...\n");
	  return 0;
	}
      }
  }
  
  if ( !Finite_Difference_p ) {
    if ( *mode == 1 || *mode == 2 ) {
      /* Compute Jacobian of nonlinear constraints. */
      rows = *nrowj;
      for ( i = 0 ; i < *ncnln ; i++ )
	if ( needc[i] > 0 ) {
	  if ( i < nle_ineq+nlt_ineq*points )
	    /* This is a nonlinear inequality constraint. */
	    k = 0;
	  else
	    /* This is a nonlinear equality constraint. */
	    k = le_ineq + lt_ineq*points;
	  
	  gradcn(*n,k+i+1,x,tmp_grad,dummy,cd);
	  if ( IEEE_error )
	   error("Floating error while computing constraint gradient.");
	  for ( j = 0 ; j < *n ; j++ )
	    CJAC[j*rows+i] = tmp_grad[j];
	}
    }
  }
  return 0;
}

/*************************************************************/
/*     Compute Jacobian matrix for linear constraints.       */
/*     and the affine offset to upper and lower bounds.      */
/*************************************************************/

static void 
compute_linear_constraints(nparam,nclin,nrowa,bigbnd,z,bl_A,bu_A,A)
     int nparam,nclin,nrowa;
     double bigbnd,*z,*bl_A,*bu_A,*A;
{
  void (* dummy)();
  int i,j,k;
  double gz,g0,dgz,pert;

  for ( i = 0 ; i < nclin ; i++ ) {
    if ( i < le_ineq + lt_ineq*points ) 
      /* First do the linear inequality constraints. */
      k = nle_ineq+nlt_ineq*points;
    else
      /* Next do the linear equality constraints. */
      k = nle_ineq+nlt_ineq*points+nle_eq;

    constr(nparam,k+i+1,z,&gz,cd);
    if ( !Finite_Difference_p )
      gradcn(nparam,k+i+1,z,tmp_grad,dummy,cd);
    else {
      /* We'll have to estimate the gradients by finite-differences. */
      pert = 0.0001;
      for ( j = 0 ; j < nparam ; j++ ) {
	x_is_new = TRUE;
	z[j] += pert;
	constr(nparam,k+i+1,z,&dgz,cd);
	tmp_grad[j] = (dgz - gz)/pert;
	z[j] -= pert;
      }
      x_is_new = TRUE;
    }
    for ( j = 0 ; j < nparam ; j++ )
      A[j*nrowa+i] = tmp_grad[j];

    /***********************************************/
    /* Compute offset g(0) = g(z) - < grad g , z > */
    /* so that linear equality contraints become:  */
    /*       -g(0)  <=  < grad g, z >  <=  -g(0)   */
    /* and linear inequality constraints become:   */
    /*    -infinity <=  < grad g, z >  <=  -g(0).  */
    /***********************************************/
    g0 = gz;
    for ( j = 0 ; j < nparam ; j++ ) 
      g0 -= tmp_grad[j]*z[j];

    bu_A[i] = -g0;
    if ( i < le_ineq + lt_ineq*points )
      bl_A[i] = -bigbnd;		   /* Linear inequality constraints. */
    else
      bl_A[i] = -g0;			   /* Linear equality constraints.   */
  }
}

#endif

static void
print_version()
{
  printf("\n");
  printf("#=============================================================#\n");
  printf("|                                                             |\n");
  printf("|                        R  I  O  T  S                        |\n");
  printf("|     (Recursive Integration Optimal Trajectory Software)     |\n");
  printf("|                        Version  1.0a                        |\n");
  printf("|                                                             |\n");
  printf("|          A.L. Schwartz                  E. Polak            |\n");
  printf("|                                                             |\n");
  printf("#=============================================================#\n");
}
