function [m1,m2,v1,v2,u] = fuzzyCMeans(P)
% [m1,m2,v1,v1,u] = fuzzyCMeans(P)
%
% The Fuzzy c-means Algorithm for clustering data into two clusters
% as it occurs with the interval estimates of a signal with two periods.
%
% Input:
%   P		the distribution to be partitioned
% Outputs:
%   mean1	mean of distribution 0
%   mean2	mean of distribution 1
%   v1		variance of distribution 0
%   v2		variance of distribution 1
%   u		fuzzy membership function (array of same length as P, 2 rows,
%			with values in [0 1])
%
% Silke Witt, Spring '95
%
% Changes by Dave Mellinger, 7/95:
%
% The code originally rounded the return values m1, m2, v1, and v2
% to integers.  I couldn't see any reason for this, so it's gone.
%
% Added code to check the case when P(j) equaled m1 or m2.  It used to 
% generate a divide-by-0 error.


c = 2;			% number of mixtures (NOT USED; 2 is assumed)
P = P(:)';		% make into a row vector
len = length(P);
m = 2;			% exponent for main calculation

% Get initial values for u (0's and 1's).
%
u = (P < mean(P));
u(2,:) = ~u(1,:);

% Iterate until membership function u doesn't change from one cycle to the next.
%
epsilon = 1e-8;
dif = epsilon + 1;		% use dummy value > epsilon to start first cycle
while (dif > epsilon)
  u_old = u;
  
  % Update parameters.
  m1 = sum((u(1,:) .^ m) .* P) / sum(u(1,:) .^ m);
  m2 = sum((u(2,:) .^ m) .* P) / sum(u(2,:) .^ m);
  
  v1 = sum(u(1,:) .* (P - m1).^2) / sum(u(1,:));	% formerly called sigma1
  v2 = sum(u(2,:) .* (P - m2).^2) / sum(u(2,:));	% formerly called sigma2
  
  % Estimate new partition.
  A = ((P - m1).^2 / v1) .^ (2/(m-1));
  B = ((P - m2).^2 / v1) .^ (2/(m-1));
  C = ((P - m1).^2 / v2) .^ (2/(m-1));
  D = ((P - m2).^2 / v2) .^ (2/(m-1));

  % Calculate membership function u.  Have to be careful to do the division
  % only on non-zero denominators.
  u = zeros(2,len);
  ix = (B ~= 0);		% indices of values to compute
  u(1,ix) = 1 ./ (A(ix) ./ B(ix) + 1);
  ix = (C ~= 0);		% indices of values to compute
  u(2,ix) = 1 ./ (D(ix) ./ C(ix) + 1);

  % get difference between last estimate and this one
  dif = sum(abs(u_old - u) * ones(len,1)) / (2*len);
end;
